/**
 *  @file Play Disney Parks Trivia API.
 *  @copyright 2017 Play Disney Parks. All rights reserved.
 *  @author Casaretto, Juan <juan.x.casaretto.-nd@disney.com>
 */

var TriviaAPI = {

  /**
   * Fetchs questions from the deck filtered by the given locationId.
   *
   * @param {string} locationId - String representing the location's identifier.
   *
   * @returns {Promise.<ResolvedCallback<TriviaQuestion[]>|RejectedCallback<Error>>} A promise for asynchronously read
   * a list of filtered questions.
   *
   * @example
   * var locationId = "FantasylandDLR";
   *
   * TriviaAPI.fetchRoundQuestions(locationId)
   * .then(function(result) {
   *      console.log('Trivia Questions: ' + result.questions);
   * })
   * .catch(function(error) {
   *      console.log("Error " + error)
   *  });
   *
   */
   fetchRoundQuestions: function(locationId) {
     return new Promise(function(resolve, reject) {
       var promised = _PlayAPI._createPromise(resolve, reject);
       var params = {
         'locationId': locationId
       };

       _PlayAPI._checkCommand(_TriviaAPI.requestType.TRIVIA_FETCH_QUESTIONS, params, promised);
     });
   },

  /**
   * Stores the answered question on the database.
   *
   * @param {string} questionId - String representing the question's identifier.
   * @param {bool} isCorrect - Boolean to determine if the question was answered correctly.
   *
   * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously store an
   * answered question.
   *
   * @example
   * var questionId = "111-222-333";
   *
   * TriviaAPI.storeAnsweredQuestion(questionId, true)
   * .then(function(result) {
   *      console.log('Trivia Question Stored');
   * })
   * .catch(function(error) {
   *      console.log('Error Storing Trivia Question');
   *  });
   *
   */
   storeAnsweredQuestion: function(questionId, isCorrect) {
     return new Promise(function(resolve, reject) {
       var promised = _PlayAPI._createPromise(resolve, reject);
       var params = {
         'questionId': questionId,
         'isCorrect': isCorrect
       };

       _PlayAPI._checkCommand(_TriviaAPI.requestType.TRIVIA_STORE_QUESTION, params, promised);
     });
   },

   storeRoundCompleted: function(locationId) {
     return new Promise(function(resolve, reject) {
       var promised = _PlayAPI._createPromise(resolve, reject);
       var params = {
         'locationId': locationId
       }
       _PlayAPI._checkCommand(_TriviaAPI.requestType.TRIVIA_STORE_ROUND_COMPLETED, params, promised);
     });
   },

   /**
    * Fetches assets for trivia game customization
    *
    * @param {string} locationId - String representing the trivia game current location identifier
    *
    * @returns {Promise.<ResolvedCallback<TriviaImage[]>|RejectedCallback<Error>>} A promise for asynchronously read
    * a list of assets (images and colors (hex strings)).
    *
    * @example
    * var locationId = "FantasylandDLR";
    *
    * TriviaAPI.fetchAssets(locationId)
    * .then(function(result) {
    *      console.log('Trivia Images: ' + result.images);
    *      console.log('Trivia background color: ' + result.backgroundColor);
    * })
    * .catch(function(error) {
    *      console.log("Error " + error)
    *  });
    *
    */
    fetchAssets: function(locationId) {
      return new Promise(function(resolve, reject) {
        var promised = _PlayAPI._createPromise(resolve, reject);
        var params = {
          'locationId': locationId
        };

        _PlayAPI._checkCommand(_TriviaAPI.requestType.TRIVIA_FETCH_ASSETS, params, promised);
      });
    }
 };

var _TriviaAPI = {
    requestType: {
        TRIVIA_FETCH_QUESTIONS: 'TRIVIA_FETCH_QUESTIONS',
        TRIVIA_STORE_QUESTION: 'TRIVIA_STORE_QUESTION',
        TRIVIA_FETCH_ASSETS: 'TRIVIA_FETCH_ASSETS',
        TRIVIA_STORE_ROUND_COMPLETED: 'TRIVIA_STORE_ROUND_COMPLETED'
    }
};

/**
 *
 * Key/value pair values that defines a trivia question.
 *
 */
var TriviaQuestion = function(identifier, question, funFact, correctAnswer, otherAnswers, points) {
    this.identifier = identifier;
    this.question = question;
    this.funFact = funFact;
    this.correctAnswer = correctAnswer;
    this.otherAnswers = otherAnswers;
    this.points = points;
};

/**
 * Image data for trivia game customization
 */
 var TriviaImage = function(identifier, data) {
   this.identifier = identifier;
   this.data = data;
 }
