/**
 *  @file Play Disney Parks API for game developers.
 *  @copyright 2016 Play Disney Parks. All rights reserved.
 *  @author McGinnis, Brian <Brian.McGinnis@disney.com>
 *  @author Baird, Jayson <Jay.Baird@disney.com>
 *  @author Casaretto, Juan <Juan.X.Casaretto.-ND@disney.com>
 *  @author Tripode, Matias <matias.x.tripode.-ND@disney.com>
 */

/**
 * @callback ResolvedCallback
 * @param {T} result - Resolved value of type <T> corresponding the request.
 */
/**
 * @callback RejectedCallback
 * @param {Error} reason - Rejected reason
 *
 */

var self;

// Dictionary of promises
var promises = {};

// Current promise request id
var requestCounter = 0;

/**
 * Public JS API
 * @class PlayAPI
 * class representing Play Disney Parks API for game developers.
 * Game developers will normally import playapi.js file into their HTML Web page before loading their game’s javascript code.
 * This API provides functionality to subscribe/unsubscribe for events (using event listener)
 * and also functionality to read/write game stats/states, player profile, and state among others.
 * For more information you might want to check [references]{@link https://docs.google.com/document/d/1S5K5fjGWOxC0wBVqKqj666yTNFUNkXeAi3FgNltCi9E/edit#}
 */
var PlayAPI = {
    /**
     * Enum that defines event type constans that get published by the Play Disney API.
     * All events published by this API have an entry in this enum
     *
     * @readonly
     * @enum {string}
     * @todo Decide who renders the message, the platform or the game.
     * @todo Location concept/requirement is still under discussion. This might affect DEVICE_ON_LOCATION_CHANGED in the future.
     */

    gameEvents: {
        /**
         * The platform is ejecting the game. The platform requests games to
         * eject mainly as the player nears a location where the game should not
         * be played anymore. The developer should handle this event by closing
         * down the game gracefully and displaying a message to the user that the game
         * is being shutdown. All queue games are required to handle this event.
         */
        GAME_EJECT: 'GAME_EJECT',

        /**
         * The game is being initialized by the platform. The game should put extra
         * initialization code that occurs post-platform initilization in this event's handler.
         */
        GAME_INIT_EVENT: 'GAME_INIT',

        /**
         * When the platform finds an error (e.g: network connectivity issues) it sends this
         * event to the game. The handler for this event will receive an object of type {Error}
         * which has a description and an enumerator {ErrorType} both to distinguish
         * the reason that caused such error.
         */
        GAME_ON_ERROR: 'GAME_ON_ERROR',

        /**
         * The platform is sending this event to the game when the native back button is pressed.
         */
        GAME_BACK: 'GAME_BACK',

        /**
         * The platform is sending location updates to the game.
         * Location object {Location} has geo location information (latitude, longitude) as well
         * as park specific one (attraction, land and park names). Each park specific
         * information will be nil in case the platform can not determine it or guest is not
         * located in any of them (e.g: user walking in main street Magic Kingdom, it will
         * return attraction = null, land = null, park name = Magic Kingdom). As Play Disney Parks is
         * targeted to be used on property only no location information will be send if
         * the user is off property.
         */
        DEVICE_ON_LOCATION_CHANGED: 'DEVICE_ON_LOCATION_CHANGED',

        /**
         * The platform is sending acceleration updates to the game.
         * Acceleration object {Acceleration} containing 3-axis acceleration data.
         */
        DEVICE_ON_ACCELERATION_CHANGED: 'DEVICE_ON_ACCELERATION_CHANGED',

        /**
         * The platform is sending attitude updates to the game.
         * Motion object {Motion} containing Attitude(pitch, roll and yaw) and userAcceleration (x, y and z) data.
         * In case the Reference Attitude was set, then another {Motion} object will passed as `deltaAttitude`.
         */
        DEVICE_ON_MOTION_CHANGED: 'DEVICE_ON_MOTION_CHANGED',

        /**
         * The platform is sending the device network status to the game.
         * Network status object {NetworkStatus} containing a string.
         */
        DEVICE_ON_NETWORK_CHANGED: 'DEVICE_ON_NETWORK_CHANGED',

        /**
         * The platform is sending the device orientation to the game.
         * Orientation type object {OrientationType} containing a string.
         */
        DEVICE_ON_ORIENTATION_CHANGED: 'DEVICE_ON_ORIENTATION_CHANGED',

        /**
         * The platform is sending microphone reading to the game.
         * Microphone object {PlayMicrophoneReading} containing Peak Power and Average
         * Reading information.
         */
        DEVICE_ON_MICROPHONE_READING_CHANGED: 'DEVICE_ON_MICROPHONE_READING_CHANGED',

        /**
         * The platform is sending the device the average color captured by the camera.
         * Returns a {GameColor} object with a RGB color.
         */
        CAPTURE_CAMERA_AVERAGE_COLOR: 'CAPTURE_CAMERA_AVERAGE_COLOR',

        /**
         * The platform signals that the current speech on going has ended.
         * Returns a {string} called `text` indicationg the text that was just spoken.
         */
        SPEECH_ENDED: 'SPEECH_ENDED'
    },

    /**
     * Game init function will return information about the logged in player and current game.
     * Player information, current game state and game definition (aka metadata)
     *
     *
     *
     * @returns {Promise.<ResolvedCallback<GameInitResult>|RejectedCallback<Error>>} A promise for asynchronously save stats.
     *
     * @example
     *
     * PlayAPI.gameInit()
     *      .then(function(result) {
     *         this.playerInfo = result.playerInfo;
     *         this.gameState = result.gameState;
     *         this.gameTitle = result.gameDefinition.title;
     *       })
     *      .catch(function(error) {
     *         console.log("Error " + error)
     *       });
     */
    gameInit: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_INIT)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Save stats for the current game instance and logged in user.
     * Game developers should save their stats at least once at the
     * end of the game. Game stats get published and used to derive achievements, leaderboards,
     * game metrics and other uses downstream.
     *
     *
     * @param {object} stats - The game stats to be saved.  A key/value pair dictionary of simple
     * stat names and numeric values as defined for your game in the Game type Definition metadata. Example
     * value: { “hits” : 20, “shots_fired” : 33 }
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously save stats.
     *
     * @example
     *
     * var stats = {
     * 'level_reached': 1,
     * 'score': game.global.score,
     * };
     *
     * PlayAPI.gameStatsSave(stats)
     *                      .then(function(result) {
     *                          console.log('Successfully saved state');
     *                       })
     *                      .catch(function(error) {
     *                          console.log("Error " + error)
     *                       });
     */
    gameStatsSave: function(stats) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'stats': stats
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GAME_STATS_SAVE, params, promised);
        });

        return myPromise;
    },

    /**
     * Gets current user's location. Game developers can get current location by either subscribing to event
     * 'PLAYER_ON_LOCATION_CHANGED' or by executing this function. This function will return a Location object
     * only if guest is on property, otherwise it will return null. For more details on Location data check
     * Location class definition.
     *
     * @returns {Promise.<ResolvedCallback<Location>|RejectedCallback<Error>>} A promise for asynchronously read current location.
     * for more details.
     *
     * @example
     *
     * PlayAPI.locationRead()
     * .then(function(result) {
     *      console.log(result.location)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     * @todo  Location concept/requirement is still under discussion. This might affect
     * PLAYER_ON_LOCATION_CHANGED and readCurrentLocation in the future.
     */
    locationRead: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.PLAYER_CURRENT_LOCATION)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Close the game and return to the platform.
     *
     * @param none
     *
     * @returns {Promise.<ResolvedCallback<Bool>|RejectedCallback<Error>>} A promise returning that game close request was received.
     *
     * @example
     *
     * PlayAPI.closeGame()
     *  .then(function(result) {
     *     // This will be called before the game is truly closed / killed.
     *     // Use this opportunity to clean up any resources or save any state.
     *     console.log("Game close request received");
     *  })
     *  .catch(function(error) {
     *     console.log("Unable to close game");
     *  });
     */
    closeGame: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_CLOSE)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Presents a view on top of the game controller to confirm if the user is willing to quit the game.
     *
     * @param none
     *
     * @returns {Promise.<ResolvedCallback<Bool>|RejectedCallback<Error>>} A promise returning a boolean that indicates
     * if the user chose to close the game or not.
     *
     * @example
     *
     * PlayAPI.presentExitView()
     *  .then(function(result) {
     *     // This will be called when the user selects an option.
     *     // Use this opportunity to clean up any resources or save any state in case the user chooses to close the game.
     *     console.log(result.isGameClosing);
     *  })
     *  .catch(function(error) {
     *     console.log("Unable to present exit view");
     *  });
     */
    presentExitView: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_EXIT_VIEW)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Presents a view on top of the game controller to confirm if the user is willing to quit the game. Also includes
     * hooks to enter cancel and confirmation actions.
     *
     * @param none
     *
     * @returns {Promise.<ResolvedCallback<Bool>|RejectedCallback<Error>>} A promise returning a boolean that indicates
     * if the user chose to close the game or not.
     *
     * @example
     *
     * var confirmAction = "Exit_Confirm"
     * var confirmData = {
     *   'link.category': "Trivia",
     *   'round.number': this.props.quizStore.numberOfConsecutivePlays+1
     *     }
     * var cancelAction = "Exit_Cancel"
     * var cancelData = {
     *   'link.category': "Trivia"
     *     }
     * PlayAPI.presentExitView(confirmAction, confirmData, cancelAction, cancelData)
     *  .then(function(result) {
     *     // This will be called when the user selects an option.
     *     // Use this opportunity to clean up any resources or save any state in case the user chooses to close the game.
     *     console.log(result.isGameClosing);
     *  })
     *  .catch(function(error) {
     *     console.log("Unable to present exit view");
     *  });
     */
    presentExitViewWithActions: function(confirmAction, confirmData, cancelAction, cancelData) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'confirmAction': confirmAction,
                'confirmData': confirmData,
                'cancelAction': cancelAction,
                'cancelData': cancelData
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GAME_EXIT_VIEW, params, promised);
        });

        return myPromise;
    },

    /**
     * Get waitTime (in minutes) for the location passed as parameter.
     * With this value game developers can know for how long a guest will be waiting in the queue
     * until actually takes the ride.
     * If user is not located in an attraction it will return an 'undefined' value.
     * @param {Location} location
     *
     * @returns {Promise.<ResolvedCallback<WaitTime>|RejectedCallback<Error>>} A promise for asynchronously read wait times.
     *
     * @example
     * var location = new Location();
     * location.locationID = '903';
     *
     * PlayAPI.locationWaitTimeRead(location)
     *  .then(function(result) {
     *      console.log('Retrieve Wait Times Success');
     *      var waitTime = result.waitTime.
     *      if (waitTime != null) {
     *          this.waitTimesText.setText('Wait time: '+ waitTime);
     *      }
     *  })
     * .catch(function(error) {
     *      console.log('Retrieve Wait Times Error');
     *  });
     * @todo Location concept still needs definition/requirements. For this function with location id is
     * sufficient to get wait time
     */
    locationWaitTimeRead: function(location) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._checkCommand(_PlayAPI.requestType.WAITTIME_CURRENT_ATTRACTION, location, promised);
        });

        return myPromise;
    },

    /**
     * Read game stats for the current game instance and logged in user.
     * Game stats are recorded by games during game play.
     * Game developers might want to read stats for specific game instance and player to know
     * about her/his performance playing it. For e.g: For games like breakout, the speed of the ball
     * would depend on player performance. Faster balls for players with score > 50 and
     * elapsed time < 40 seconds.
     *
     *
     * @returns {Promise.<ResolvedCallback<GameStats>|RejectedCallback<Error>>} A promise for asynchronously read stats.
     *
     *
     * @example
     *
     *
     * PlayAPI.gameStatsRead()
     * .then(function(result) {
     *      console.log(result);
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     * });
     */
    gameStatsRead: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_STATS_READ)(promised.reqId);
        });
        
        return myPromise;
    },


    /**
     * Read game defintions for the current game instance and logged in user.
     * Game defintions are defined in the DEU metadata.
     *
     * @returns {Promise.<ResolvedCallback<GameDefintion>|RejectedCallback<Error>>} A promise for asynchronously read stats.
     *
     *
     * @example
     *
     *
     * gameDefinitionRead()
     * .then(function(defintion) {
     *      console.log(defintion);
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     * });
     */
    gameDefinitionRead: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_DEFINITION_READ)(promised.reqId);
        });

        return myPromise;
    },
    /**
     * Read game state for current game instance and logged in user.
     * Game state values are recorded by games during game play.
     * One main use case is using game state for storing data that enables a game to be serialized to a data
     * store and restarted from the same point in the game at a later time.
     * For example, as the player hits the end of a queue, the platform will send an event to the game
     * to “eject” all game play. The game can store its current game state so the game can resume later.
     *
     *
     * @returns {Promise.<ResolvedCallback<GameState>|RejectedCallback<Error>>} A promise for asynchronously save stats.
     *
     * @example
     *
     * PlayAPI.gameStateRead()
     * .then(function(result) {
     *      console.log(result.state)
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     */
    gameStateRead: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.GAME_STATE_READ)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Save game state for current game instance and logged in user
     * Game state values are recorded by games during game play.
     * One main use case is using game state for storing data that enables a game to be serialized to a data store
     * and restarted from the same point in the game at a later time.
     * For example, as the player hits the end of a queue, the platform will send an event to the game
     * to “eject” all game play. The game can store it’s current game state so the game can resume later.
     *
     *
     * @param {object} status - String representing the status to be saved.
     * @param {object} state - Key/value pair representing the state to be saved.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously save state.
     *
     * @example
     * var status = GameStatus.GAME_STATUS_IN_PROGRESS;
     * var state = { 'level': 1, 'points': 40 };
     *
     * PlayAPI.gameStateSave(status, state)
     * .then(function(result) {
     *      console.log('Game state successfully saved: ' + result);
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    gameStateSave: function(status, state) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'status': status,
                'state': state
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GAME_STATE_SAVE, params, promised);
        });

        return myPromise;
    },

    /**
     * Save game's Last Played date.  This must be called in case the game should be marked as played when it's
     * completed.
     *
     * @param {string} locationId - The locationId to mark as played.
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously save last played.
     * @example
     * var locationId = 'someLocation';
     *
     * PlayAPI.lastPlayedSave(locationId)
     * .then(function(result) {
     *      console.log('Last Played successfully saved for ' + locationId);
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     */
    lastPlayedSave: function(locationId) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'locationId': locationId,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GAME_LAST_PLAYED_SAVE, params, promised);
        });

        return myPromise;
    },

    /**
     * Save game event logfor current game instance and logged in user
     * Game event logs are for game developers to log data for later analysis.
     * This is defined and used by the game developer so we don’t specify much structure for it here.
     * Game events will likely get a TTL (time to live) setting and cleaned out from the
     * database periodically later.
     *
     * @param {object} log - The key/value pair values to be saved.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously save Log.
     *
     * @example
     * var log = {
     *  'wait_time': 10, //time in minutes
     *  'attraction_name': 'Space Mountain',  //hit target 4 times
     *  'playing_time': 360, //time in seconds
     * };
     *
     *
     * PlayAPI.eventLogSave(log)
     * .then(function(result) {
     *      console.log('Game log successfully saved: ' + result);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    eventLogSave: function(log) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'log': log,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.EVENT_LOG_SAVE, params, promised);
        });

        return myPromise;
    },

    /**
     * Read event channel for current game instance and logged in user.
     * The event channel is a tool for sending events (e.g: push notifications) to the play game so they can be responded to.
     *
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for asynchronously subscribe to events.
     *
     * @example
     *
     * PlayAPI.eventChannelSubscribe()
     *  .then(function(result) {
     *      console.log(result);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    eventChannelSubscribe: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.PLAYER_EVENTCHANNEL_READ)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Activates the camera on the host device.
     * The camera output will be displayed underneath the game content at the bottom-most layer.
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for asynchronously activating the device camera.
     *
     * @example
     *
     * var cameraType = CameraType.BACK_CAMERA;
     *
     * PlayAPI.startCamera(cameraType)
     *  .then(function(result) {
     *      console.log(result);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    startCamera: function(cameraType) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'camera_type': cameraType,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.START_CAMERA, params, promised);
        });

        return myPromise;
    },

    /**
     * Checks the camera access status.
     * The status will be returned to the promise as a CameraAccessStatus
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for asynchronously retrieving the access status for the camera.
     *
     * @example
     *
     * PlayAPI.cameraAccessStatus()
     *  .then(function(result) {
     *      console.log("Camera access status= \(result["status"])");
     *
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    cameraAccessStatus: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.CAMERA_ACCESS_STATUS)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Captures an image from the camera.
     * The camera output will be returned to the promise as Base64 encoded image data.
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for asynchronously retrieving an image from the device camera, with the corresponding orientation of the device.
     *
     * @example
     *
     * PlayAPI.captureImage()
     *  .then(function(result) {
     *      console.log(result);
     *      var imageData = result.imageData;
     *      var imageOrientation = result.imageOrientation
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    captureImage: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.CAPTURE_CAMERA)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Deactivates the camera on the host device.
     * The camera output will be terminated and no longer captured by the device.
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for asynchronously deactivating the device camera.
     *
     * @example
     *
     * PlayAPI.endCamera()
     *  .then(function(result) {
     *      console.log(result);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    endCamera: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.END_CAMERA)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Saves a base64 encoded image data to the device's camera roll.
     *
     * @returns {Promise.<ResolvedCallback<MessageChannel>|RejectedCallback<Error>>} A promise for returning the state of saving to the device's photo library.
     *
     * @example
     *
     * PlayAPI.saveToPhotos(imageData<base64encoded>)
     *  .then(function(result) {
     *      console.log(result);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    saveToPhotos: function(imageData) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'imageData': imageData,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.SAVE_TO_PHOTOS, params, promised);
        });

        return myPromise;
    },

    /**
     * Activates the camera on the host device and starts capturing the average color of the camera image.
     *
     * @param {CameraType} The Camera type (back/front)
     * @param {GameRegion} The region of the screen in which the camera will capture the color. If no region is provided,
     * the camera will capture the whole screen.
     *
     * @returns A {GameColor} object with a RGB color.
     *
     * @example
     *
     * var cameraType = CameraType.BACK_CAMERA;
     * var region = new GameRegion(0, 0, 100, 100);
     *
     * PlayAPI.subscribeToAverageColorStreamEvent(cameraType, region, this.onColorCapture);
     *
     */
    subscribeToAverageColorStreamEvent: function(cameraType, region, handler) {
        var params = {
            'camera_type': cameraType,
            'region': region
        };

        myPromise = _PlayAPI._subscribeToEvent(PlayAPI.gameEvents.CAPTURE_CAMERA_AVERAGE_COLOR, params, handler);

        return myPromise;
    },

    /**
     * Read achievement definitions.
     *
     * @returns {Promise.<ResolvedCallback<Achievement[]>|RejectedCallback<Error>>} A promise for asynchronously read
     * achievement definitions.
     *
     * @example
     *
     *
     * PlayAPI.readAchievementDefinitions()
     * .then(function(result) {
     *      for (var i = 0; i < result.achievements.length; i++) {
     *          var achievement = result.achievements[i]
     *          console.log(achievement.identifier + '\n');
     *      }
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     */
    readAchievementDefinitions: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.READ_ACHIEVEMENT_DEFINITIONS)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Read the player's achievements. Achievements are earned by guests as they play the game.
     * Game developers typically update achievements during game play, or as the game closes down.
     * We will have “external” achievements which are awarded as non-game events occur
     * (e.g. ride an attraction) or a guest plays multiple games.
     * We also need to support “promo” achievements which enable cast members to
     * reward disgruntled guests or reward special event attendees.
     *
     * @returns {Promise.<ResolvedCallback<Achievement[]>|RejectedCallback<Error>>} A promise for asynchronously read player's achievement.
     *
     * @example
     *
     *
     * PlayAPI.readPlayerAchievements(playerId)
     * .then(function(result) {
     *      for (var i = 0; i < result.achievements.length; i++) {
     *          var achievement = result.achievements[i]
     *          console.log(achievement.identifier + '\n');
     *          console.log(achievement.isAwarded + '\n');
     *      }
     * })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     */
    readPlayerAchievements: function(playerId) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'playerId': playerId,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.READ_PLAYER_ACHIEVEMENT, params, promised);
        });

        return myPromise;
    },

    /**
     * Grant achievement for one or more players. Achievements are earned by guests as they play the game.
     * Game developers typically update achievements during game play, or as the game closes down.
     * Players can be granted achievements due to their performance playing games. The key part for
     * developers is to get identifier of the achievement to be granted, that can be accomplished by
     * calling readAchievements function.
     *
     * @param {String} achievementId - The id of the achievement to be granted.
     * @param {String[]} playersIds - Array containing the list of players this achievement will be granted to.
     * @param {Bool} showNotification - Determines if a notification will be shown to the user after granting an achievement.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously grant an achievement for the logged in player.
     *
     * @example
     *
     *  //Somewhere when your game inits
     *  create: function() {
     *      //Retrieve definition for game's achievements
     *
     *      PlayAPI.readAchievements()
     *          .then(function(result) {
     *          game.global.gameAchievements = result.achievements;
     *      })
     *      .catch(function(error){
     *      });
     *  },
     *
     *  //Try to grant achievement on game over
     *  gameOver: function() {
     *      this.grantAchievement();
     *  },
     *
     *  grantAchievement: function() {
     *      // Check whether the player can get an achievement
     *      // This logic can be more complex.
     *      // E.g: canGetAnAchievement = (time_elapse < time_to_get_achievement &&
     *      //                          score >= minimum_score_for_achievement)
     *      var canGetAnAchievement = (game.global.score >= 15);
     *      if (!canGetAnAchievement) {
     *          return;
     *      }
     *
     *      //This holds all possible achievements that a player
     *      //can get by playing this game
     *      var achievements = game.global.gameAchievements;
     *
     *      //For testing just take the first one
     *      var achievementId = achievements[0].identifier;
     *      var playersIds = ['id1', 'id2'];
     *      var showNotification = true;
     *
     *      PlayAPI.grantAchievementToPlayers(achievementId, playersIds, showNotification)
     *      .then(function(result) {})
     *      .catch(function(error) {});
     * },
     */
    grantAchievementToPlayers: function(achievementId, playersIds, showNotification) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'achievementId': achievementId,
                'playersIds': playersIds,
                'showNotification': showNotification
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GRANT_PLAYER_ACHIEVEMENT, params, promised);
        });

        return myPromise;
    },

    /**
     * Shows a notification to the user about the recently granted achievement.
     *
     * @param {String} achievementId - The id of the achievement to be shown on a notification.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously perform
     * a notification.
     *
     * @example
     *
     * PlayAPI.showNotificationForAchievement(achievementId)
     * .then(function(result) {})
     * .catch(function(error) {});
     */
    showNotificationForAchievement: function(achievementId) {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'achievementId': achievementId
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.SHOW_ACHIEVEMENT_NOTIFICATION, params, promised);
        });

        return myPromise;
    },

    /**
     * Read logged in player profile information. This function requires no parameters as it will return profile
     * information for already logged in player.
     *
     * @returns {Promise.<ResolvedCallback<PlayerInfo>|RejectedCallback<Error>>} A promise for asynchronously read player's profile information.
     *
     * @example
     *
     * PlayAPI.playerInfoRead()
     * .then(function(result) {
     *      console.log(result.playerInfo);
     * })
     * .catch(function(error) { console.log("Error " + error) });
     * }
     */
    playerInfoRead: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.PLAYER_PROFILE_READ)(promised.reqId);
        });

        return myPromise;
    },

    handlePromiseReponse: function(response) {
        _PlayAPI._handlePromiseReponse(response);
    },

    /**
     * Log action in analytics. Requirements of what should be passed as action and category is
     * defined by analytics team for each game and screens.
     *
     * @param {string} action - The action to be logged.
     * @param {string} category - The category for the action.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously log an action into analytics backend.
     *
     * @example
     *
     * var action = 'retro_score_increased';
     * var category = 'player_actions';
     *
     * PlayAPI.analyticsLogAction(action, category)
     * .then(function(success) {
     *      console.log(success);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    analyticsActionLog: function(action, category) {
      return this.analyticsActionObjectLog(action, {'link.category': category})
    },

    /**
     * Log action in analytics. Requirements of what should be passed as action and data is
     * defined by analytics team for each game and screens.
     *
     * @param {string} action - The action to be logged.
     * @param {object} data - The data for the action.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously log an action into analytics backend.
     *
     * @example
     *
     * var action = 'retro_score_increased';
     * var data = {
     *  'action': action,
     *  'link.category': 'Game'
     *  };
     *
     * PlayAPI.analyticsLogAction(action, data)
     * .then(function(success) {
     *      console.log(success);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    analyticsActionObjectLog: function(action, data) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'action': action,
                'data': data,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.ANALYTICS_LOG_ACTION, params, promised);
        });

        return myPromise;
    },

    /**
     * Log state in analytics. Requirements of what should be passed as state and data is
     * defined by analytics team for each game and screens.
     *
     * @param {string} state - The state to be logged.
     * @param {object} data - The data (dictionary) to be logged.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously log a state into analytics backend.
     *
     * @example
     *
     * var state = 'retro_main_screen';
     *
     * PlayAPI.analyticsStateLog(state, data)
     * .then(function(success) {
     *      console.log(success)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    analyticsStateLog: function(state, data) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'state': state,
                'data': data,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.ANALYTICS_LOG_STATE, params, promised);
        });

        return myPromise;
    },

    /**
     * Log a start action in analytics. Requirements of what should be passed as action and data is
     * defined by analytics team for each game and screens.
     *
     * @param {string} action - The action to be logged.
     * @param {object} data - The data for the action.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously log an action into analytics backend.
     *
     * @example
     *
     * var action = 'retro_score_increased';
     * var data = {
     *  'action': action,
     *  'link.category': 'Game'
     *  };
     *
     * PlayAPI.analyticsTimedActionStart(action, data)
     * .then(function(success) {
     *      console.log(success);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    analyticsTimedActionStart: function(state, data) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'state': state,
                'data': data
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.ANALYTICS_LOG_TIMED_START, params, promised);
        });

        return myPromise;
    },

    /**
     * Ends a timed action in analytics. Requirements of what should be passed as action and data is
     * defined by analytics team for each game and screens.
     *
     * @param {string} action - The action to be logged.
     * @param {object} data - The data for the action.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for asynchronously log an action into analytics backend.
     *
     * @example
     *
     * var action = 'retro_score_increased';
     *
     * PlayAPI.analyticsTimedActionEnd(action, data)
     * .then(function(success) {
     *      console.log(success);
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    analyticsTimedActionEnd: function(state) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'state': state
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.ANALYTICS_LOG_TIMED_END, params, promised);
        });

        return myPromise;
    },

    /**
     * Opens the Player Management module in a native window. This function requires no parameters as it will return an
     * array of players the user selects. The user could also select one or none.
     *
     * @returns {Promise.<ResolvedCallback<Player[]>|RejectedCallback<Error>>} A promise for asynchronously get the players the user selected.
     *
     * @example
     *
     * PlayAPI.openPlayerManagement()
     * .then(function(result) {
     *      console.log(result.players);
     * })
     * .catch(function(error) { console.log("Error " + error) });
     * }
     */
    openPlayerManagement: function() {
        self = this;
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.PLAYER_MANAGEMENT_MODULE)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Opens the Sharing Center module in a native window so the user can share the selected achievement. Returns a boolean
     * with information about the share completion.
     *
     * @param {String} achievementId - The identifier of the achievement to be shared.
     *
     * @returns {Promise.<ResolvedCallback<Boolean>|RejectedCallback<Error>>} A promise to asynchronously share the
     * selected achievement.
     *
     * @example
     *
     * PlayAPI.shareAchievement(achievementId)
     * .then(function(result) {
     *      console.log(result.completed);
     * })
     * .catch(function(error) { console.log("Error " + error) });
     * }
     */
    shareAchievement: function(achievementId) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'achievementId': achievementId,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.SHARE_ACHIEVEMENT, params, promised);
        });

        return myPromise;
    },

    /**
     * Opens the Sharing Center module in a native window so the user can share the selected base64 encoded image.
     * Returns a boolean with information about the share completion.
     *
     * @param {String} imageData - The identifier of the image to be shared.
     *
     * @returns {Promise.<ResolvedCallback<Boolean>|RejectedCallback<Error>>} A promise to asynchronously share the
     * selected image.
     *
     * @example
     *
     * PlayAPI.shareImage(imageData<base64encoded>)
     * .then(function(result) {
     *      console.log(result.completed);
     * })
     * .catch(function(error) { console.log("Error " + error) });
     * }
     */
    shareImage: function(imageData) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'imageData': imageData,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.SHARE_IMAGE, params, promised);
        });

        return myPromise;
    },

    updateGameId: function(gameId) {
        self = this;
        myPromise = new Promise(function (resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'game_id': gameId,
            };
            _PlayAPI._checkCommand(_PlayAPI.requestType.GAME_ID_UPDATE, params, promised);
        });

        return myPromise;

    },

    /**
     * Sends a vibrate notification to the host device.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for
     * asynchronously vibrating the device.
     *
     * @example
     *
     * PlayAPI.vibrateDevice()
     *  .then(function(success) {
     *      console.log(success)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    vibrateDevice: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.VIBRATE_DEVICE)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Sets the reference attitude for future delta operation.  Then, on subsequent Motion event callbacks,
     * a new `deltaAttitude` value will be provided containing the Ref Attitude vs Current Attitude.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for
     * asynchronously setting the reference attitude.
     *
     * @example
     *
     * PlayAPI.setReferenceAttitude()
     *  .then(function(success) {
     *      console.log(success)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    setReferenceAttitude: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.SET_REFERENCE_ATTITUDE)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Resets the reference attitude.  Then, on subsequent Motion event callbacks, `deltaAttitude`
     * will NOT be provided.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for
     * asynchronously resetting the reference attitude.
     *
     * @example
     *
     * PlayAPI.resetReferenceAttitude()
     *  .then(function(success) {
     *      console.log(success)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    resetReferenceAttitude: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.RESET_REFERENCE_ATTITUDE)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Returns the available voices to be used to call `speak` function.
     *
     * @returns {Promise.<ResolvedCallback<Voice[]>|RejectedCallback<Error>>} A promise for
     * asynchronously getting the list of available voices.
     *
     * @example
     *
     * PlayAPI.listAvailableVoices()
     *  .then(function(result) {
     *      console.log(result.voices)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    listAvailableVoices: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.LIST_AVAILABLE_VOICES)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Request for a text to be spoken to the user.
     *
     * @param {String} text - Text to be read to the user.
     * @param {String} voiceId - Optional.  Id of the voice to be used to speak the text to the user.
     *                           This id must be one found on the result from `listAvailableVoices` call.  If this value
     *                           is `undefined`, the native system will use the default voice.
     *
     * @returns {Promise.<ResolvedCallback<Boolean>|RejectedCallback<Error>>} A promise to asynchronously speak the text
     * to the user.
     *
     * @example
     *
     * PlayAPI.speak(text, voiceId)
     * .then(function(success) {
     *      console.log(success);
     * })
     * .catch(function(error) { console.log("Error " + error) });
     * }
     */
    speak: function(text, voiceId) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'text': text
            };
            if (voiceId != undefined) {
                params.voiceId = voiceId
            }
            _PlayAPI._checkCommand(_PlayAPI.requestType.SPEAK_TEXT, params, promised);
        });

        return myPromise;
    },

    /**
     * Cancels the ongoing speech and any pending speak request that was not processed yet.
     *
     * @returns {Promise.<ResolvedCallback<SuccessResult>|RejectedCallback<Error>>} A promise for
     * asynchronously cancelling ongoing speech.
     *
     * @example
     *
     * PlayAPI.cancelSpeech()
     *  .then(function(success) {
     *      console.log(success)
     *  })
     * .catch(function(error) {
     *      console.log("Error " + error)
     *  });
     *
     */
    cancelSpeech: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(_PlayAPI.requestType.CANCEL_SPEECH)(promised.reqId);
        });

        return myPromise;
    },

    /**
     * Callback function send from native API.
     * Based on the requestType it will dispatch an event (dispatchEvent).
     */
    nativeCallback: function(res) {
        var response = _PlayAPI._parseIfNecessary(res);
        var currentEvent = '';

        if (response.status == 'SUCCESS') {
            switch (response.requestType) {
                case 'GAME_EJECT' :
                    currentEvent = this.gameEvents.GAME_EJECT;
                    break;
                case 'GAME_INIT':
                    currentEvent = this.gameEvents.GAME_INIT_EVENT;
                    break;
                case 'GAME_BACK':
                    currentEvent = this.gameEvents.GAME_BACK;
                    break;
                case 'DEVICE_ON_LOCATION_CHANGED':
                    currentEvent = this.gameEvents.PLAYER_ON_LOCATION_CHANGED;
                    break;
                case 'DEVICE_ON_ACCELERATION_CHANGED':
                    currentEvent = this.gameEvents.DEVICE_ON_ACCELERATION_CHANGED;
                    break;
                case 'DEVICE_ON_NETWORK_CHANGED':
                    currentEvent = this.gameEvents.DEVICE_ON_NETWORK_CHANGED;
                    break;
                case 'DEVICE_ON_ORIENTATION_CHANGED':
                    currentEvent = this.gameEvents.DEVICE_ON_ORIENTATION_CHANGED;
                    break;
                case 'DEVICE_ON_MOTION_CHANGED':
                    currentEvent = this.gameEvents.DEVICE_ON_MOTION_CHANGED;
                    break;
                case 'DEVICE_ON_MICROPHONE_READING_CHANGED':
                    currentEvent = this.gameEvents.DEVICE_ON_MICROPHONE_READING_CHANGED;
                    break;
                case 'CAPTURE_CAMERA_AVERAGE_COLOR':
                    currentEvent = this.gameEvents.CAPTURE_CAMERA_AVERAGE_COLOR;
                    break;
                case 'SPEECH_ENDED':
                    currentEvent = this.gameEvents.SPEECH_ENDED;
                    break;
                case 'SHOW_CONTROL_EFFECT_IN_RANGE':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE;
                    break;
                case 'SHOW_CONTROL_EFFECT_READY':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_READY;
                    break;
                case 'SHOW_CONTROL_EFFECT_READY_TO_TRIGGER':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_READY_TO_TRIGGER;
                    break;
                case 'SHOW_CONTROL_EFFECT_PENDING':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_PENDING;
                    break;
                case 'SHOW_CONTROL_EFFECT_TIMEOUT':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_TIMEOUT;
                    break;
                case 'SHOW_CONTROL_TRIGGER_SUCCESS':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS;
                    break;
                case 'SHOW_CONTROL_TRIGGER_FAILURE':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE;
                    break;
                case 'SHOW_CONTROL_DATA_RECEIVED':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_DATA_RECEIVED;
                    break;
                case 'SHOW_CONTROL_SHOW_RUNNING':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING;
                    break;
                case 'SHOW_CONTROL_SHOW_RESETTING':
                    currentEvent = this.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING;
                    break;
                case 'BEACON_GAME_ADVANCE_IN_RANGE':
                    currentEvent = this.DBeacon.events.BEACON_GAME_ADVANCE_IN_RANGE;
                    break;
                case 'AR_SESSION_DID_RECOGNIZE_IMAGE':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE;
                    break;
                case 'AR_SESSION_DID_UPDATE_IMAGE':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_UPDATE_IMAGE;
                    break;
                case 'AR_SESSION_DID_UPDATE_WORLD':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_UPDATE_WORLD;
                    break;
                case 'AR_SESSION_DID_REMOVE_IMAGE':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_REMOVE_IMAGE;
                    break;
                case 'AR_SESSION_DID_ERROR':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_ERROR;
                    break;
                case 'AR_SESSION_DID_PAUSE':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_PAUSE;
                    break;
                case 'AR_SESSION_DID_RESUME':
                    currentEvent = this.AugmentedReality.events.AR_SESSION_DID_RESUME;
                    break;
                default:
                    //ignore unkown events
                    console.warn("Unknown event: " + response.requestType);
                    break;
            }
            if (currentEvent != '') {
                var event = new CustomEvent(currentEvent, { 'detail': response.payload });
                window.dispatchEvent(event);
            }

        } else {
            var event = new CustomEvent(this.gameEvents.GAME_ON_ERROR, { 'detail': response });
            window.dispatchEvent(event);
        }
    },

    /**
     * Subscribe to an event.
     * @param {enum} event - The event type to listen for.
     * @param {function} handler - Function that will be called when the event is dispatched.
     * @see GameEvents definition.
     *
     * @example
     * //1. Subscribe to events you want to listen for.
     *  bindEvents: function () {
     *      PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_EJECT, this.onGameEject);
     *      PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_ON_ERROR, this.onGameError);
     *      PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_INIT_EVENT, this.onGameInit);
     *      PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_BACK, this.onGameBack);
     * },
     *
     * //2. Execute custom logic when callback for GAME_INIT_EVENT is received
     * onGameInit: function(result) {
     *   // Perform custom initialization logic for your game
     *   // Read game state and determine whether this game instance can be resumed (player already played it
     *   // but couldn't finish it yet) or it is a new one. Based on that information game developer
     *   // can display two options "tap for new game" and "tap to continue playing"
     * },
     *
     * //3. Execute custom logic based on type of error
     * onGameError: function(error) {
     *   switch (error.errorType) {
     *      case ErrorType.ERROR_NETWORK_FAILED:
     *          //Display network connection banner with information for the user
     *          //Save game state
     *          break;
     *      case ErrorType.ERROR_SERVER_FAILED:
     *          //Save game state
     *          break;
     *      default:
     *   }
     * }
     *
     * //4. Execute custom logic when callback for GAME_EJECT is received
     * onGameEject: function(payload) {
     *      //Save game stats
     *      PlayAPI.gameStatsSave(stats)
     *                      .then(function(result) { console.log('Successfully saved stats');})
     *                      .catch(function(error) { console.log("Error " + error) });
     *
     *      //Save game state
     *      PlayAPI.gameStateSave(gameState)
     *                            .then(function(result) {
     *                                  console.log('Game state successfully saved: ');
     *                             })
     *                            .catch(function(error) { console.log("Error " + error) });
     * }
     *
     * //5. Execute custom logic when callback for GAME_BACK is received
     * onGameBack: function(result) {
     *   // Perform custom logic when native back button is triggered.
     *   console.log("PlayAPI.gameEvents.GAME_BACK successfully called");
     * }
     */
    subscribeToEvent: function(event, handler) {
        myPromise = _PlayAPI._subscribeToEvent(event, {}, handler);
        return myPromise;
    },

    /**
     * Unsubscribe to an event.
     * @param {string} event - Name of the event.
     * @param {function} handler - Function that will be removed.
     * @see GameEvents definition.
     * @see subscribeToEvent defintion.
     *
     * @example
     * //1. Unregister event handler.
     * PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.GAME_EJECT_EVENT, this.onGameEject);
     */
    unsubscribeToEvent: function(event, handler) {
        myPromise = _PlayAPI._unsubscribeToEvent(event, {}, handler);
        return myPromise;
    },
};

/**
 * Public Classes.
 * Games
 */


/**
 * Class GameInitResult
 * Represents game definition data.
 *
 * @property {PlayerInformation} playerInfo - Represents player profile information.
 * @property {GameState} gameState - Represents game state information.
 * @property {GameDefinition} gameDefinition - Represents game definition information.
 *
 */
GameInitResult = function() {
};

GameInitResult.prototype.playerInfo = null;
GameInitResult.prototype.gameState = null;
GameInitResult.prototype.gameDefinition = null;

/**
 * Class GameDefinition
 * Represents game definition data.
 * @property {string} title - Title of the game.
 * @property {Dictionary} stateSchema - Schema for the key/value pairs that represent game state.
 * @property {Dictionary} statsSchema - Schema for the key/value pairs that represent game stats.
 *
 */
GameDefinition = function() {
};

GameDefinition.prototype.title = '';
GameDefinition.prototype.stateSchema = null;
GameDefinition.prototype.statsSchema = null;


/**
 * Class SuccessResult
 * Represents a successfull response. The returned object will be an empty one.
 * as no usefull information is needed to represent a successfull execution.
 *
 */

var SuccessResult = function() {
};

/**
 * Class WaitTime
 * Represents how long (in minutes) a guest might have to wait in an attraction/entertainment queue.
 *
 * @property {int} waitTime - Wait time in minutes.
 *
 */

var WaitTime = function() {
};

WaitTime.prototype.waitTime = 0;


/**
 * Class PlayerInformation
 * Represents a player profile.
 *
 * @property {string} name - Logged in player's name
 * @property {string} email - Logged in player's email.
 * @property {string} avatarURL - Logged in player's avatar url.
 *
 */

var PlayerInformation = function() {
};

PlayerInformation.prototype.name = '';
PlayerInformation.prototype.email = '';
PlayerInformation.prototype.avatarURL = '';

/**
 * Enum that defines level of expertise for a player.
 *
 * @readonly
 * @enum {string}
 */
var PlayerExpertise = {
    /**
     Represents a beginner player.
     */
    PLAYER_EXPERTISE_BEGINNER: 'PLAYER_EXPERTISE_BEGINNER',
    /**
     Represents an intermediate player.
     */
    PLAYER_EXPERTISE_INTERMEDIATE: 'PLAYER_EXPERTISE_INTERMEDIATE',
    /**
     Represents an advance player.
     */
    PLAYER_EXPERTISE_ADVANCE: 'PLAYER_EXPERTISE_ADVANCE',
    /**
     Represents an expert player.
     */
    PLAYER_EXPERTISE_EXPERT: 'PLAYER_EXPERTISE_EXPERT',
};

/**
 * Class MessageChannel
 * Represents a subscribe to event channels response. The returned object will be
 *
 * @property {string} id - Identifier for the message.
 * @property {MessageChannelType} type - Type of message. Example: game_promo_announcement, achievement_notification
 * @property {string} description - String representing the description of the message.
 * @property {string} expiration - Timestamp for when event expires.
 * @property {string} version - Flag for indicating guest was shown the message.
 *
 */
var MessageChannel = function() {
};

MessageChannel.prototype.id = '';
MessageChannel.prototype.type = '';
MessageChannel.prototype.description = '';
MessageChannel.prototype.expiration = '';
MessageChannel.prototype.unread = true;

/**
 * Enum that defines message channel type that get published by the Play Disney API.
 *
 * @readonly
 * @enum {string}
 */
var MessageChannelType =
{
    /**
     Native platform sends a game promo message.
     */
    MESSAGE_TYPE_GAME_PROMO: 'MESSAGE_TYPE_GAME_PROMO',
    /**
     Native platform sends an achievement notfication.
     */
    MESSAGE_TYPE_ACHIEVEMENT_NOTIFICATION: 'MESSAGE_TYPE_ACHIEVEMENT_NOTIFICATION',
    /**
     Native platform sends a general purpose message to the channel.
     */
    MESSAGE_TYPE_GENERAL: 'MESSAGE_TYPE_GENERAL',
};

/**
 * Class Location
 * @property {float} latitude - Geo location latitude
 * @property {float} longitude - Geo location longitude
 * @property {string} attractionName - Disney attraction name. User facing value (e.g: Space Mountain).
 * @property {string} landName - Disney land name. User facing value (e.g: Adventure Land).
 * @property {string} parkName - Disney park name. User facing value (e.g: Magic Kingdom).
 * @property {string} locationID - Location id. Non user facing value. This can be used to fetch wait times
 * and another properties specific for an attraction.
 */
var Location = function() {
};

Location.prototype.latitude = 0.0;
Location.prototype.longitude = 0.0;
Location.prototype.attractionName = '';
Location.prototype.locationID = '';
Location.prototype.landName = '';
Location.prototype.parkName = '';

/**
 * Class PlayMicrophoneReading
 * @property {float} peakPower - Peak power in decibels for the recorded sound
 * @property {float} averagePeakPower - Average power in decibels for the recorded sound
 */
var PlayMicrophoneReading = function() {
};

PlayMicrophoneReading.prototype.peakPower = 0.0;
PlayMicrophoneReading.prototype.averagePeakPower = 0.0;

/**
 * Enum that defines error type constans that get published by the Play Disney API.
 *
 * @readonly
 * @enum {string}
 */
var ErrorType =
{
    /**
     Native platform detected a network issue due to weak signal.
     */
    ERROR_NETWORK_WEAK: 'ERROR_NETWORK_WEAK',
    /**
     Native platform detected a network issue due to no signal.
     */
    ERROR_NETWORK_FAILED: 'ERROR_NETWORK_FAILED',
    /**
     JS API found a null property in a dictionary or dictionary is completely null.
     */
    ERROR_PARAMETER_NULL: 'ERROR_PARAMETER_NULL',
    /**
     Native platform detected a failure in a service call.
     */
    ERROR_SERVER_FAILED: 'ERROR_SERVER_FAILED',
};

/**
 * Class Error
 * @property {ErrorType} errorType - Type of error.
 * @property {string} description - String with information that describes what caused the error.
 */
var Error = function(errorType, description) {
    this.errorType = errorType;
    this.description = description;
};

Error.prototype.message = function() {
    return 'Error type: ' + this.errorType + ', description: ' + this.description;
};

/**
 * Achievements are earned by guests as they play the game.
 * Game developers typically update achievements during game play,
 * or as the game closes down. We will have “external” achievements
 * which are awarded as non-game events occur (e.g. ride an attraction)
 * or a guest plays multiple games.
 *
 * @property {string} identifier - Achievement identification
 * @property {string} title - Achievement title
 * @property {string} preEarnDescription - Description of achievement before it’s earned
 * @property {string} postEarnDescription - Description of achievement after it’s earned
 * @property {string} imageURL - URL to image for achievement
 * @property {int} pointsAwarded - Number of game points when achievement earned (zero is okay value too)
 * @property {Bool} isAwarded - Value indicating the state of the achievement
 */
var Achievement = function() {
};

Achievement.prototype.identifier = '';
Achievement.prototype.title = '';
Achievement.prototype.preEarnDescription = '';
Achievement.prototype.postEarnDescription = '';
Achievement.prototype.imageURL = '';
Achievement.prototype.pointsAwarded = 0;
Achievement.prototype.isAwarded = false;

/**
 *
 * Key/value pair values that define game stats. Game state values are recorded by games during game play.
 * Game developer has to check game definition (aka DEU metadata) in order to know which key/value pairs
 * are part of the game state definition.
 *
 */
var GameState = function(){
};

GameState.prototype.created = '';
GameState.prototype.updated = '';
GameState.prototype.status = '';
GameState.prototype.state = {};

/**
 * Key/value pair values that define game stats. Game stats are recorded by games during game play.
 * Game developer has to check game definition (aka DEU metadata) in order to know which key/value pairs
 * are part of game stats definition.
 */
var GameStats = function(){
};

/**
 * Enum that defines in which status is a game. This type constans that get published by the Play Disney API.
 *
 * @readonly
 * @enum {string}
 */
var GameStatus = {
    /**
     Game has not started yet
     */
    GAME_STATUS_NOT_STARTED: 'not-started',
    /**
     Game already started.
     */
    GAME_STATUS_IN_PROGRESS: 'in-progress',
    /**
     Player finshed the game successfully.
     */
    GAME_STATUS_COMPLETED: 'completed',
    /**
     Game was paused, e.g: it received an GAME_EJEC event due to player is at the of the queue
     */
    GAME_STATUS_PAUSED: 'paused'
};

/**
 * Enum that defines the network status of the device.
 *
 * @readonly
 * @enum {string}
 */
var NetworkStatus = {
    /**
     Device has no connection.
     */
    NETWORK_STATUS_NO_CONNECTION: 'NETWORK_STATUS_NO_CONNECTION',

    /**
     Device is connected to WWAN or WiFi.
     */
    NETWORK_STATUS_CONNECTED: 'NETWORK_STATUS_CONNECTED'
};

/**
 * Enum that defines the orientation type of the device.
 *
 * @readonly
 * @enum {string}
 */
var OrientationType = {
    /**
     Standard portrait orientation.
     */
    DEVICE_ORIENTATION_PORTRAIT: 'DEVICE_ORIENTATION_PORTRAIT',

    /**
     Upside down portrait orientation.
     */
    DEVICE_ORIENTATION_PORTRAIT_UPSIDE_DOWN: 'DEVICE_ORIENTATION_PORTRAIT_UPSIDE_DOWN',

    /**
     Landscape orientation from the left side.
     */
    DEVICE_ORIENTATION_LANDSCAPE_LEFT: 'DEVICE_ORIENTATION_LANDSCAPE_LEFT',

    /**
     Landscape orientation from the right side.
     */
    DEVICE_ORIENTATION_LANDSCAPE_RIGHT: 'DEVICE_ORIENTATION_LANDSCAPE_RIGHT',

    /**
     Device oriented flat, face up.
     */
    DEVICE_ORIENTATION_FACE_UP: 'DEVICE_ORIENTATION_FACE_UP',

    /**
     Device oriented flat, face down.
     */
    DEVICE_ORIENTATION_FACE_DOWN: 'DEVICE_ORIENTATION_FACE_DOWN',

    /**
     Unknown device orientation.
     */
    DEVICE_ORIENTATION_UNKNOWN: 'DEVICE_ORIENTATION_UNKNOWN',
};

/**
 * Enum that defines the camera position, front or back.
 *
 * @readonly
 * @enum {string}
 */
var CameraType = {
    UNSPECIFIED: 0,
    BACK_CAMERA: 1,
    FRONT_CAMERA: 2,
};

/**
 * Enum that defines the camera access status.
 *
 * @readonly
 * @enum {integer}
 */
var CameraAccessStatus = {
    NOT_DETERMINED: 0,
    RESTRICTED:  1,
    DENIED: 2,
    AUTHORIZED: 3
};

/**
 * Enum that defines the player's type.
 *
 * @readonly
 * @enum {string}
 */
var PlayerType = {
    /**
     The logged in user.
     */
    TYPE_PRIMARY: 'TYPE_PRIMARY',

    /**
     The anonymous players.
     */
    TYPE_SECONDARY: 'TYPE_SECONDARY'
};

/**
 *
 * Key/value pair values that defines a region on the game screen.
 *
 */
var GameRegion = function(x, y, width, height) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
};

/**
 *
 * Key/value pair values that defines an RGB color.
 *
 */
var GameColor = function(r, g, b) {
    this.r = r;
    this.g = g;
    this.b = b;
};

/**
 *
 * Key/value pair values that defines a player from the Roster.
 *
 */
var Player = function(identifier, name, type) {
    this.identifier = identifier
    this.name = name
    this.type = type
};

/**
 *
 * Key/value pair values that defines the device Attitude.
 *
 */
var Attitude = function(pitch, roll, yaw) {
    this.pitch = pitch;
    this.roll = roll;
    this.yaw = yaw;
};

/**
 *
 * Key/value pair values that defines the device acceleration.
 *
 */
var Acceleration = function(x, y, z) {
    this.x = x;
    this.y = y;
    this.z = z;
};

/**
 *
 * Key/value pair values that defines the device Motion.
 * Contains the device Attitude and user Acceleration
 *
 */
var Motion = function(attitude, userAcceleration) {
    this.attitude = attitude;
    this.userAcceleration = userAcceleration;
};

/**
 * Key/value pairs that define an available voice to use when calling
 * `speak` command.
 */
var Voice = function(identifier, name, lang, quality) {
    this.identifier = identifier;
    this.name = name;
    this.lang = lang;
    this.quality = quality;
};

/**
 * Enum that defines the event type.
 *
 * @readonly
 * @enum {string}
 */
var EventType = {
    EVENT_TYPE_SUBSCRIBE: 'EVENT_TYPE_SUBSCRIBE',
    EVENT_TYPE_UNSUBSCRIBE: 'EVENT_TYPE_UNSUBSCRIBE'
};

/**
 *
 * Key/value pair values that defines the object sent by the GameAdvance packet.
 * Contains the game id, waypoint id and the current rssi
 *
 */
var GameAdvance = function(gameId, waypointId, rssi) {
    this.gameId = gameId;
    this.waypointId = waypointId;
    this.rssi = rssi;
};

var _PlayAPI = {

    requestType: {
        GAME_INIT: 'GAME_INIT',
        GAME_STATS_SAVE: 'GAME_STATS_SAVE',
        GAME_CLOSE: 'GAME_CLOSE',
        GAME_EXIT_VIEW: 'GAME_EXIT_VIEW',
        PLAYER_CURRENT_LOCATION: 'PLAYER_CURRENT_LOCATION',
        WAITTIME_CURRENT_ATTRACTION: 'WAITTIME_CURRENT_ATTRACTION',
        GAME_STATS_READ: 'GAME_STATS_READ',
        GAME_DEFINITION_READ: 'GAME_DEFINITION_READ',
        GAME_STATE_READ: 'GAME_STATE_READ',
        GAME_STATE_SAVE: 'GAME_STATE_SAVE',
        GAME_LAST_PLAYED_SAVE : 'GAME_LAST_PLAYED_SAVE',
        EVENT_LOG_SAVE: 'EVENT_LOG_SAVE',
        PLAYER_EVENTCHANNEL_READ: 'PLAYER_EVENTCHANNEL_READ',
        START_CAMERA: 'START_CAMERA',
        CAPTURE_CAMERA: 'CAPTURE_CAMERA',
        END_CAMERA: 'END_CAMERA',
        SAVE_TO_PHOTOS: 'SAVE_TO_PHOTOS',
        CAMERA_ACCESS_STATUS: 'CAMERA_ACCESS_STATUS',
        READ_ACHIEVEMENT_DEFINITIONS: 'READ_ACHIEVEMENT_DEFINITIONS',
        READ_PLAYER_ACHIEVEMENT: 'READ_PLAYER_ACHIEVEMENT',
        GRANT_PLAYER_ACHIEVEMENT: 'GRANT_PLAYER_ACHIEVEMENT',
        SHOW_ACHIEVEMENT_NOTIFICATION: 'SHOW_ACHIEVEMENT_NOTIFICATION',
        PLAYER_PROFILE_READ: 'PLAYER_PROFILE_READ',
        ANALYTICS_LOG_ACTION: 'ANALYTICS_LOG_ACTION',
        ANALYTICS_LOG_STATE: 'ANALYTICS_LOG_STATE',
        ANALYTICS_LOG_TIMED_START: 'ANALYTICS_LOG_TIMED_START',
        ANALYTICS_LOG_TIMED_END: 'ANALYTICS_LOG_TIMED_END',
        PLAYER_MANAGEMENT_MODULE: 'PLAYER_MANAGEMENT_MODULE',
        SHARE_ACHIEVEMENT: 'SHARE_ACHIEVEMENT',
        SHARE_IMAGE: 'SHARE_IMAGE',
        GAME_ID_UPDATE: 'GAME_ID_UPDATE',
        VIBRATE_DEVICE: 'VIBRATE_DEVICE',
        SET_REFERENCE_ATTITUDE : 'SET_REFERENCE_ATTITUDE',
        RESET_REFERENCE_ATTITUDE: 'RESET_REFERENCE_ATTITUDE',
        LIST_AVAILABLE_VOICES: 'LIST_AVAILABLE_VOICES',
        SPEAK_TEXT: 'SPEAK_TEXT',
        CANCEL_SPEECH: 'CANCEL_SPEECH'
    },

    _checkCommand: function(command, params, promised) {
        if (_PlayAPI._checkParameters(params)) {
            _PlayAPI._createCommandWithParams(command, params)(promised.reqId);
        } else {
            var error = new Error(ErrorType.ERROR_PARAMETER_NULL,'Error dictionary as parameter can not have null values');
            promised.rejectFunc(error);
        }
    },
    /**
     * Private method.
     * It will return true only if all properties in the dictionary are not null
     */
    _checkParameters: function(dictionary) {

        if (dictionary == null) { return false; }

        for ( var property in dictionary ) {
            if (dictionary[property] == null) {
                return false;
            }
        }

        return true;
    },
    /**
     * Private method.
     * Create a wrapper for promise and store it in dictionary for later use.
     */
    _createPromise: function(resolve, reject) {
        var promised =  {
        reqId: requestCounter++,
        resolveFunc: resolve,
        rejectFunc: reject
        };

        promises[promised.reqId] = promised;
        return promised;
    },

    /**
     * Private method.
     * Removes a promise from promises dictionary.
     */
    _removePromise: function(reqId) {
        var promisesCount = Object.keys(promises).length;
        if (promisesCount > 0 &&
            reqId >= 0 &&
            reqId < promisesCount ) {
            delete promises[reqId];
        }
    },

    /**
     * Private method.
     * Send command to swift via JS interop.
     */
    _sendCommand: function(cmdObject) {
        console.log("API::Sending command: " + JSON.stringify(cmdObject));
        if (this._isAndroid()) {
            Android.postMessage(JSON.stringify(cmdObject))
        } else {
            window.webkit.messageHandlers.NativeInterop.postMessage(cmdObject);
        }
    },

    /**
     * Private method.
     * Create a send command to swift via JS interop.
     */
    _createCommandWithParams:  function(command, params) {
        return function(reqId) {
            var cmdMessage = {
                'reqId' : reqId,
                'type' : 'asyncResponse',
                'command' : command,
                'payload' : params,
            };
            _PlayAPI._sendCommand(cmdMessage);
        };
    },


    /**
     * Private method.
     * Create a send command to swift via JS interop.
     */
    _createCommand:  function(command) {
        return function(reqId) {
            var cmdMessage = {
                'reqId' : reqId,
                'type' : 'asyncResponse',
                'command' : command,
            };
            _PlayAPI._sendCommand(cmdMessage);
        };
    },

    /**
     * Private method.
     * The native iOS handler invokes this function when iOS gets the firebase response back,
     * and resolves it asynchronously via promises.
     */
    _createSaveCommand: function(command, payload) {
        return function(reqId) {
            var cmdMessage = {
                'reqId' : reqId,
                'command' : command,
                'type' : 'asyncResponse',
                'payload' : payload
            };
            _PlayAPI._sendCommand(cmdMessage);
        };
    },

    /**
     * Private method.
     * The native iOS handler invokes this function when iOS gets the firebase response back,
     * and resolves it asynchronously via promises.
     */
    _handlePromiseReponse: function(res) {
        var response = this._parseIfNecessary(res);
        console.log("_handlePromiseReponse " + JSON.stringify(response));

        // Get reqId from swift callback response.
        var reqId = response.requestID;

        // Fetch promise from promises dictionary.
        var promised = promises[reqId];

        if (promised !== undefined) {

            if (response.status == 'SUCCESS') {
                // call promise's resolve function
                promised.resolveFunc(response.payload);
            } else {
                var error = new Error(ErrorType.ERROR_SERVER_FAILED,
                                      response.payload.error);

                promised.rejectFunc(error);
            }
            // remove the promise from our dictionary of promises
            this._removePromise(reqId);

        } else {
            // Error: We Can't find the promise to callback with.
            promised.rejectFunc("Error promised request " + reqId + " not found." + response);
        }
    },

    _parseIfNecessary: function(response){
        if(this._isAndroid()){
            return JSON.parse(response);
        } else {
            return response;
        }
    },

    _isAndroid: function(){
        try {
            return Android !== undefined;
        } catch (e) {
            return false;
        }
    },

    _subscribeToEvent: function(event, params, handler) {
        window.addEventListener(event, (result) => {
            handler(result.detail);
        }, false);

        return new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var parameters = {
                'event_type': EventType.EVENT_TYPE_SUBSCRIBE,
                'params': params
            };
            _PlayAPI._checkCommand(event, parameters, promised);
        });
    },

    _unsubscribeToEvent: function(event, handler) {
        window.removeEventListener(event, (result) => {
                                        handler(result.detail);
                                   });

        return new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var parameters = {
                'event_type': EventType.EVENT_TYPE_UNSUBSCRIBE,
            };
            _PlayAPI._checkCommand(event, parameters, promised);
        });
    }
};

//
// Private
// Logging tool for native developers to be able to see logs from JS in XCode console
//
// @example
// var _log = function(toLog){
//   PlayAPI.NativeLog.log('Trivia game log: ' + toLog);
// }
//
var _NativeLog = {
    log: function(log) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'log': log
            };
            _PlayAPI._checkCommand('NATIVE_PLATFORM_LOG', params, promised);
        });

        return myPromise;

    }
};

PlayAPI.NativeLog = _NativeLog;

var _DBeacon = {
    events: {
        /**
         * The platform is detecting a D-Beacon enabled show control is ready
         * for interaction and idle.
         * Returns a {Experience} object with the experience ready to be triggered.
         */
        SHOW_CONTROL_EFFECT_READY: 'SHOW_CONTROL_EFFECT_READY',

        /**
         * The platform is detecting a D-Beacon enabled show control is ready to trigger.
         * Returns a {Expereince} object with the experience that is ready to trigger.
         */
        SHOW_CONTROL_EFFECT_READY_TO_TRIGGER: 'SHOW_CONTROL_EFFECT_READY_TO_TRIGGER',

        /**
         * The platform is detecting a D-Beacon enabled show control is pending.
         * Returns a {Expereince} object with the experience that is pending.
         */
        SHOW_CONTROL_EFFECT_PENDING: 'SHOW_CONTROL_EFFECT_PENDING',

        /**
         * The platform is detecting a D-Beacon enabled sow control has timed out.
         * Returns a {Experience} object with the experience that has timed out.
         */
        SHOW_CONTROL_EFFECT_TIMEOUT: 'SHOW_CONTROL_EFFECT_TIMEOUT',

        /**
         * The platform is detecting a D-Beacon enabled show control is in range.
         * Returns a {Experience} object with the experience that is in range.
         */
        SHOW_CONTROL_EFFECT_IN_RANGE: 'SHOW_CONTROL_EFFECT_IN_RANGE',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * advertising that a trigger requested by the platform was successful.
         * Returns a {Experience} object with the triggered experience.
         */
        SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS: 'SHOW_CONTROL_TRIGGER_SUCCESS',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * advertising a trigger request has failed.
         * Returns a {Experience} object with the experience that failed.
         */
        SHOW_CONTROL_EFFECT_TRIGGER_FAILURE: 'SHOW_CONTROL_TRIGGER_FAILURE',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * advertising that data sent by the platform was successful.
         * Returns a {Experience} object with the triggered experience.
         */
        SHOW_CONTROL_SEND_DATA_SUCCESS: 'SHOW_CONTROL_SEND_DATA_SUCCESS',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * advertising a send data request has failed.
         * Returns a {Experience} object with the experience that failed.
         */
        SHOW_CONTROL_SEND_DATA_FAILURE: 'SHOW_CONTROL_SEND_DATA_FAILURE',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * advertising data sent to the platform.
         * Returns a {Experience} object with the experience and payload.
         */
        SHOW_CONTROL_DATA_RECEIVED: 'SHOW_CONTROL_DATA_RECEIVED',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * currently running.
         * Returns a {Experience} object with the experience.
         */
        SHOW_CONTROL_SHOW_RUNNING: 'SHOW_CONTROL_SHOW_RUNNING',

        /**
         * The platform is detecting a D-Beacon enabled show control that is
         * currently running.
         * Returns a {Experience} object with the experience.
         */
        SHOW_CONTROL_SHOW_RESETTING: 'SHOW_CONTROL_SHOW_RESETTING',

        /**
         * The platform is detecting 'Game Advance' type packet sent by a D-Beacon.
         * Returns a {GameAdvance} object with the game Id, waypoint id and current rssi.
         */
        BEACON_GAME_ADVANCE_IN_RANGE: 'BEACON_GAME_ADVANCE_IN_RANGE'
    },

    requestType: {
        SHOW_CONTROL_SESSION_INIT: 'SHOW_CONTROL_SESSION_INIT',
        SHOW_CONTROL_SESSION_END: 'SHOW_CONTROL_SESSION_END',
        SHOW_CONTROL_SEND_DATA: 'SHOW_CONTROL_SEND_DATA',
        SHOW_CONTROL_TRIGGER_EFFECT: 'SHOW_CONTROL_TRIGGER_EFFECT',
    },

    sessionInit: function() {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.DBeacon.requestType.SHOW_CONTROL_SESSION_INIT)(promised.reqId);
        });

        return myPromise;
    },

    sessionEnd: function() {
        myPromise = new Promise(function (resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.DBeacon.requestType.SHOW_CONTROL_SESSION_END)(promised.reqId);
        });

        return myPromise;
    },

    sendData: function(data) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'data': data
            };
            _PlayAPI._checkCommand(PlayAPI.DBeacon.requestType.SHOW_CONTROL_SEND_DATA, params, promised);
        });

        return myPromise;
    },

    sendString: function(str) {
        if (str.length > 7) {
            throw new Error("String must be seven characters or less.");
        }
        var charArray = [];
        for (var i = 0; i < str.length; i++) {
            charArray[i] = str.charCodeAt(i);
        }
        return sendData(charArray);
    },

    sendInt: function(d) {
        if (d > 255 || d < 0) {
            throw new Error("Integer must be in byte range of 0 - 255.");
        }
        return sendData([d]);
    },

    sendInts: function(integers) {
        if (integers.constructor !== Array || integers.count <= 7) {
            throw new Error("Integers must be an array of seven or fewer integers valued 0 - 255.");
        }
        for (var i = 0; i < integers.count; i++) {
            if (integers[i] > 255 || integers[i] < 0) {
                throw new Error("Integer must be in byte range of 0 - 255.");
            }
        }

        return sendData(integers);
    },

    triggerEffect: function(effectName, payload, timeout) {
        myPromise = new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'effectName': effectName,
                'payload': payload,
                'timeout': timeout
            };
            _PlayAPI._checkCommand(PlayAPI.DBeacon.requestType.SHOW_CONTROL_TRIGGER_EFFECT, params, promised);
        });

        return myPromise;
    }
};

PlayAPI.DBeacon = _DBeacon;

/**
 * @deprecated use PlayAPI.DBeacon
 */
PlayAPI.ShowControl = _DBeacon;

class PlayARCamera extends THREE.PerspectiveCamera {
    constructor(fov, aspect, near, far) {
        super(fov, aspect, near, far);
        this.isARCamera = true;
        this.updateProjectionMatrix();
    }

    updateMatrixWorld() {
        var arSession = PlayAPI.AugmentedReality._arsession;
        if (!arSession || !arSession.matrices) {
            super.updateMatrixWorld();
            return;
        }

        const viewMatrix = arSession.matrices.viewMatrix;
        this.matrixWorldInverse.fromArray(viewMatrix);
        this.matrixWorld.getInverse(this.matrixWorldInverse);
        this.updateProjectionMatrix();
    }
}

var _augmentedReality = {
    _arsession: {
        matrices: null
    },
    
    _renderTiming: {
        _fps: 60.0,
        _now: 0,
        _delta: 0,
        _then: Date.now(),
        _interval: 1000.0 / 60.0
    },

    events: {
        AR_SESSION_DID_RECOGNIZE_IMAGE: 'AR_SESSION_DID_RECOGNIZE_IMAGE',
        AR_SESSION_DID_UPDATE_IMAGE:    'AR_SESSION_DID_UPDATE_IMAGE',
        AR_SESSION_DID_UPDATE_WORLD:    'AR_SESSION_DID_UPDATE_WORLD',
        AR_SESSION_DID_REMOVE_IMAGE:    'AR_SESSION_DID_REMOVE_IMAGE',
        AR_SESSION_DID_ERROR:           'AR_SESSION_DID_ERROR',
        AR_SESSION_DID_PAUSE:           'AR_SESSION_DID_PAUSE',
        AR_SESSION_DID_RESUME:          'AR_SESSION_DID_RESUME'
    },

    requestType: {
        AR_SESSION_SESSION_INIT:   'AR_SESSION_SESSION_INIT',
        AR_SESSION_AVAILABLE:      'AR_SESSION_AVAILABLE',
        AR_SESSION_SESSION_END:    'AR_SESSION_SESSION_END',
        AR_SESSION_REGISTER_IMAGE: 'AR_SESSION_REGISTER_IMAGE'
    },

    camera: new PlayARCamera(70.0, window.innerWidth / window.innerHeight, 0.001, 1000.0),
    renderer: null,
    scene: null,

    _updateWorld: function(event) {
        var arSession = PlayAPI.AugmentedReality._arsession;
        arSession.matrices = {
            viewMatrix: event.viewMatrix,
            projectionMatrix: event.projectionMatrix
        };

        PlayAPI.AugmentedReality.camera.position.set(event.cameraPosition.x, event.cameraPosition.y, event.cameraPosition.z);
    },

    isAvailable: function() {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.AugmentedReality.requestType.AR_SESSION_AVAILABLE)(promised.reqId);
        });

        return myPromise;
    },

    sessionStart: function(canvasElement) {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.AugmentedReality.requestType.AR_SESSION_SESSION_INIT)(promised.reqId);
        });

        PlayAPI.subscribeToEvent(PlayAPI.AugmentedReality.events.AR_SESSION_DID_UPDATE_WORLD, this._updateWorld);

        this.renderer = new THREE.WebGLRenderer({ alpha: true, canvas: canvasElement });
        this.scene = new THREE.Scene();

        this.renderer.setSize(window.innerWidth, window.innerHeight);
        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.shadowMap.enabled = false;

        return myPromise;
    },

    sessionEnd: function() {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.AugmentedReality.requestType.AR_SESSION_SESSION_END)(promised.reqId);
        });

        PlayAPI.unsubscribeToEvent(PlayAPI.AugmentedReality.events.AR_SESSION_DID_UPDATE_WORLD, this._updateWorld);

        this.renderer = null;
        this.scene = null;

        return myPromise;
    },

    registerImage: function(imageName, pathToImageInGameBundle, imageWidthInMeters) {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'imageName': imageName,
                'imagePath': pathToImageInGameBundle,
                'imageWidth': imageWidthInMeters
            };
            _PlayAPI._checkCommand(PlayAPI.AugmentedReality.requestType.AR_SESSION_REGISTER_IMAGE, params, promised);
        });

        return myPromise;
    },

    render: function() {
        if (this.renderer == null || this.camera == null || this.scene == null) {
            return;
        }

        this._renderTiming._now = Date.now();
        this._renderTiming._delta = this._renderTiming._now - this._renderTiming._then;

        if (this._renderTiming._delta > this._renderTiming._interval) {
            this._renderTiming._then = this._renderTiming._now - (this._renderTiming._delta % this._renderTiming._interval);

            this.renderer.clearColor();
            this.renderer.clearDepth();
            this.renderer.render(this.scene, this.camera);
        }
    }
};

PlayAPI.AugmentedReality = _augmentedReality;

_localStore = {
    requestType: {
        FILE_STORE_LIST:   'FILE_STORE_LIST',
        FILE_STORE_READ:   'FILE_STORE_READ',
        FILE_STORE_STORE:  'FILE_STORE_STORE',
        FILE_STORE_DELETE: 'FILE_STORE_DELETE'
    },

    list: function() {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            _PlayAPI._createCommand(PlayAPI.LocalStorage.requestType.FILE_STORE_LIST)(promised.reqId);
        });

        return myPromise;
    },

    read: function(filename) {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'fileName': filename
            };
            _PlayAPI._checkCommand(PlayAPI.LocalStorage.requestType.FILE_STORE_READ, params, promised);
        });

        return myPromise;
    },

    write: function(filename, data) {
        var myPromise =new Promise(function(resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'fileName': filename,
                'data': data
            };
            _PlayAPI._checkCommand(PlayAPI.LocalStorage.requestType.FILE_STORE_STORE, params, promised);
        });

        return myPromise;
    },

    unlink: function(filename, data) {
        var myPromise = new Promise(function (resolve, reject) {
            var promised = _PlayAPI._createPromise(resolve, reject);
            var params = {
                'fileName': filename
            };
            _PlayAPI._checkCommand(PlayAPI.LocalStorage.requestType.FILE_STORE_DELETE, params, promised);
        });

        return myPromise;
    }
};

PlayAPI.LocalStorage = _localStore;