var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @author nick@cerebralfix.com
 */
define("switchback-common/component/Signal", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Signal {
        constructor() {
            this.active = true;
            this.bindings = [];
        }
        add(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        addOnce(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this, true);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        dispatch(...params) {
            if (this.active) {
                for (let i = 0; this.active && i < this.bindings.length; i++) {
                    const binding = this.bindings[i];
                    if (binding.addOnce) {
                        this.bindings.splice(i, 1);
                        i--;
                    }
                    if (binding.context) {
                        binding.listener.call(binding.context, ...params);
                    }
                    else {
                        binding.listener(...params);
                    }
                }
            }
        }
        removeBinding(binding) {
            if (this.active) {
                const index = this.bindings.indexOf(binding);
                if (index >= 0) {
                    this.bindings.splice(index, 1);
                }
            }
        }
        dispose() {
            this.active = false;
            delete this.bindings;
        }
    }
    exports.Signal = Signal;
    // tslint:disable-next-line:max-classes-per-file
    class SignalBinding {
        constructor(listener, context, signal, addOnce = false) {
            this.listener = listener;
            this.context = context;
            this.signal = signal;
            this.addOnce = addOnce;
        }
        remove() {
            this.signal.removeBinding(this);
        }
    }
    exports.SignalBinding = SignalBinding;
});
define("switchback-common/util/PromiseUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to provide helpful Promise wrapper functions
     * @author joel@cerebralfix.com
     * @author steven@cerebralfix.com
     */
    class PromiseUtil {
        /**
         * Phaser time
         * Smallest time you can wait is the length of one frame (16ms @ 60fps)
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static time(pGame, pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timer;
                    const cancelCallback = () => {
                        if (timer) {
                            // stop timer as we've cancelled it
                            pGame.time.events.remove(timer);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timer = pGame.time.events.add(pTime, resolveCallback);
                });
            });
        }
        /**
         * Real time
         * Can update in smaller increments, but does not pause when the game loses focus
         * Use with caution
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static realTime(pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timeoutId = -1;
                    const cancelCallback = () => {
                        if (timeoutId >= 0) {
                            // stop timeout as we've cancelled it
                            clearTimeout(timeoutId);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timeoutId = setTimeout(resolveCallback, pTime);
                });
            });
        }
        /*
         * Awaits a tween to complete
         *
         * @param pTween The tween to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will cancel the tween and release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static tween2(pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pTween) {
                            // stop listening to the tween and stop it as we've cancelled it
                            pTween.onComplete.remove(resolveCallback);
                            pTween.stop();
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pTween.onComplete.addOnce(resolveCallback);
                });
            });
        }
        static tween(pGame, pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield PromiseUtil.tween2(pTween, cancelSignal);
            });
        }
        static animation(pAnimation, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pAnimation) {
                            // stop listening to callback as we've cancelled it
                            pAnimation.onComplete.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pAnimation.onComplete.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static signal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pSignal.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a common signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static commonSignal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let resolveCallbackBinding;
                    let cancelCallbackBinding;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.removeBinding(resolveCallbackBinding);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.removeBinding(cancelCallbackBinding);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelCallbackBinding = cancelSignal.addOnce(cancelCallback);
                    }
                    resolveCallbackBinding = pSignal.addOnce(resolveCallback);
                });
            });
        }
        static load(pLoader) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    pLoader.onLoadComplete.addOnce(resolve);
                    pLoader.start();
                });
            });
        }
        /**
         * Loops through all markers in @param audioSprite. If any are
         * playing, this function will block until they're finished.
         */
        static audioSpriteMarkers(audioSprite) {
            return __awaiter(this, void 0, void 0, function* () {
                const stopPromises = Object.keys(audioSprite.sounds).map((key) => {
                    const sfx = audioSprite.sounds[key];
                    if (sfx.isPlaying) {
                        return PromiseUtil.signal(sfx.onStop);
                    }
                });
                yield Promise.all(stopPromises);
            });
        }
        /**
         * Wait for a sound that is already playing to finish playing.
         *
         * @param sound The sound that you wish to wait to end
         * @param offsetDuration Handles the situation where you want to wait for a remainder of the time,
         * useful if this sound has already played for a while before being awaited.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static sound(sound, offsetDurationMs = 0, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (sound && sound.game && sound.isPlaying) {
                    // it has been possible for sounds to break and never stop,
                    // hense the additional time safety check
                    yield Promise.race([
                        PromiseUtil.signal(sound.onStop, cancelSignal),
                        PromiseUtil.time(sound.game, sound.durationMS - offsetDurationMs, cancelSignal),
                    ]);
                }
            });
        }
    }
    exports.PromiseUtil = PromiseUtil;
});
define("switchback-common/state/BaseState", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * BaseState is our extension of Phaser.State that all states within games should extend.
     * This is to allow us to handle things like state transitions and display object hierarchies in a common way.
     * @author joel@cerebralfix.com
     */
    class BaseState extends Phaser.State {
        constructor() {
            super(...arguments);
            // Duration of the fade graphic alpha in and out, in milliseconds
            this.FADE_DURATION_MS = 300;
            this.stateName = "BaseState";
            this.firstPointerDownTime = 0;
            this.tapCount = 0;
        }
        init(...args) {
            super.init(...args);
            console.log(`[${this.stateName}] init(${JSON.stringify(args)})`);
            this.shutdownSignal = new Phaser.Signal();
            this.isShutdown = false;
            // Hijack the state's add function to ensure all game objects are
            // added to the correct group (e.g. under the fade overlay)
            this.add.existing = this.addToGameLayer.bind(this);
            // Create our game group here
            this.gameGroup = new Phaser.Group(this.game);
            // Create our dev group here, goes on top of the game group
            this.createDevGroup();
            // Then immediately add our fade overlay on top. This ensures the
            // overlay is present for the duration of the loading
            this.fadeImage = this.add.graphics(0, 0);
            this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
            this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
            this.fadeImage.endFill();
            this.fadeImage.fixedToCamera = true;
            this.fadeImage.inputEnabled = true;
            this.game.input.onDown.add(this.onPointerDown, this);
        }
        create() {
            console.log(`[${this.stateName}] create()`);
            super.create();
        }
        removeChildFromGameGroup(child) {
            this.gameGroup.remove(child);
        }
        createDevGroup() {
            this.devGroup = new Phaser.Group(this.game);
            this.devGroup.visible = false;
            const data = this.game.cache.getJSON("build-info");
            if (data) {
                let versionString = `${data.hash}`;
                if (data.version) {
                    versionString = "v" + `${data.version}` + "." + `${data.build}` + "-" + `${data.agent}` + " (" + `${data.hash}` + ")";
                }
                const text = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height, versionString, {
                    fontSize: 50,
                    fill: "white",
                    stroke: "black",
                    strokeThickness: 5,
                    align: "center"
                });
                text.anchor.set(0.5, 1);
                this.game.input.onDown.add(() => { this.devGroup.visible = false; }, this);
                text.inputEnabled = true;
                this.devGroup.add(text);
            }
        }
        onPointerDown(pointer) {
            // Was it in the right place?
            if (pointer.positionDown.x < this.game.width * 0.1 &&
                pointer.positionDown.y > this.game.height * 0.9) {
                // If it's been too long since the last tap in this area, start the timer and counter again
                if (this.game.time.now > this.firstPointerDownTime + 1000) {
                    this.firstPointerDownTime = this.game.time.now;
                    this.tapCount = 0;
                }
                this.tapCount++;
                if (this.tapCount === 5) {
                    this.devGroup.visible = true;
                    this.tapCount = 0;
                }
            }
            else {
                // If it wasn't in the right place, reset the counter.
                this.tapCount = 0;
            }
        }
        preload() {
            super.preload();
            console.log(`[${this.stateName}] preload()`);
            // Add a listener so we can see which files are loading
            this.load.onFileComplete.add((progress, key, success, loaded, total) => {
                console.log(`[${this.stateName}] preload() [${progress}% | ${loaded}/${total}] loading: ${key}`);
            });
            // Load the game's common assets into this state
            if (BaseState.loadCommonAssets) {
                BaseState.loadCommonAssets();
            }
        }
        ensurePointers(num) {
            // check if we can even do that
            if (num > Phaser.Input.MAX_POINTERS) {
                console.log(`[${this.stateName}] Can't add pointers. `, num, "/", Phaser.Input.MAX_POINTERS);
                return false;
            }
            for (let i = 1; i < num + 1; i++) {
                // if pointer doesn't exists, add it
                if (!this.game.input["pointer" + i]) {
                    this.game.input.addPointer();
                }
                // if it hasnt been added, explode
                if (!this.game.input["pointer" + i]) {
                    return false;
                }
            }
            // fix reduced inputs
            this.game.input.maxPointers = num;
            return true;
        }
        fadeToState(state, ...args) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                this.game.state.start(state, true, false, ...args);
            });
        }
        /*
         * Shows the input-blocking overlay on top of the game, then tweens it to full alpha.
         * This function should be awaited before transitioning away to a loading screen.
         */
        showOverlay(alpha = 1, fadeDurationMS = this.FADE_DURATION_MS, newColor = undefined) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] showOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 0;
                this.fadeImage.visible = true;
                if (newColor) {
                    BaseState.OVERLAY_COLOR = newColor;
                    this.fadeImage.clear();
                    this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
                    this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
                    this.fadeImage.endFill();
                }
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                }
                else {
                    this.fadeImage.alpha = 0;
                    const fadeInTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    yield PromiseUtil_1.PromiseUtil.tween2(fadeInTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        /*
         * Tweens the overlay out to alpha 0 and then sets it to invisible (to remove input-blocking).
         * This function should be called when the state's create() function is complete
         * and everything has been setup on-screen.
         */
        dismissOverlay(alpha = 0, fadeDurationMS = this.FADE_DURATION_MS) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] dismissOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 1;
                this.fadeImage.visible = true;
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                    this.fadeImage.visible = false;
                }
                else {
                    this.fadeImage.alpha = 1;
                    const fadeOutTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    fadeOutTween.onComplete.addOnce(() => {
                        this.fadeImage.visible = false;
                        this.overlayDismissed();
                    });
                    yield PromiseUtil_1.PromiseUtil.tween2(fadeOutTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        shutdown() {
            console.log(`[${this.stateName}] shutdown()`);
            this.isShutdown = true;
            if (this.shutdownSignal) {
                this.shutdownSignal.dispatch();
                this.shutdownSignal.dispose();
            }
            this.game.input.onDown.remove(this.onPointerDown, this);
            // Remove our file load listener
            this.load.onFileComplete.removeAll();
            this.fadeImage.destroy();
            this.devGroup.destroy();
            this.gameGroup.destroy();
            super.shutdown();
            this.shutdownSignal = null;
            this.fadeImage = null;
            this.gameGroup = null;
            this.devGroup = null;
        }
        /*
         * Overridden by subclasses to determine when overlay has finished
         * dismissing
         */
        overlayDismissed() {
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToGameLayer(object) {
            this.gameGroup.add(object);
            return object;
        }
    }
    BaseState.OVERLAY_COLOR = 0x000000;
    exports.BaseState = BaseState;
});
define("switchback-common/localization/LocalizationManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Localization singleton. Parsed from json.
     * @author joel@cerebralfix.com
     */
    class LocalizationManager {
        static get Instance() {
            return this.instance || (this.instance = new this());
        }
        init(pLocale, data) {
            if (data === null) {
                throw new Error("Localization data not found");
            }
            this.polyglot = new Polyglot({
                locale: pLocale,
                phrases: data,
            });
            this.isReversed = false;
        }
        text(phrase) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase));
            }
            return this.polyglot.t(phrase);
        }
        textWithCount(phrase, smartCount) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, smartCount));
            }
            return this.polyglot.t(phrase, smartCount);
        }
        textWithReplacements(phrase, interpolationOptions) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, interpolationOptions));
            }
            return this.polyglot.t(phrase, interpolationOptions);
        }
        setReversed(reversed) {
            this.isReversed = reversed;
        }
        /**
         * Please note that this method of reversal should only be used for
         * debugging purposes, and won't properly handle non-ASCII chars
         */
        reverseString(source) {
            return source.split("").reverse().join("");
        }
    }
    exports.LocalizationManager = LocalizationManager;
});
/**
 * This class exists to support on-desktop testing of data saving and loading.
 * It should only be used by the MockPlayAPI.
 * @author joel@cerebralfix.com
 * @author nick@cerebralfix.com
 */
define("switchback-common/util/CookieUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CookieUtil {
        static get(name) {
            const nameEQ = name + "=";
            const ca = document.cookie.split(";");
            for (let i = 0; i < ca.length; i++) {
                let c = ca[i];
                while (c.charAt(0) === " ") {
                    c = c.substring(1, c.length);
                }
                if (c.indexOf(nameEQ) === 0) {
                    return decodeURIComponent(c.substring(nameEQ.length, c.length));
                }
            }
            return null;
        }
        static set(name, value, daysTilExpiry = 1) {
            const expiry = new Date();
            expiry.setTime(expiry.getTime() + daysTilExpiry * 24 * 60 * 60 * 1000);
            document.cookie = name + "=" + encodeURIComponent(value) + "; expires=" + expiry.toUTCString() + "; path = /";
        }
        static erase(name) {
            CookieUtil.set(name, "", -1);
        }
    }
    exports.CookieUtil = CookieUtil;
});
define("switchback-common/play/api/MockPlayAPI", ["require", "exports", "switchback-common/util/CookieUtil", "switchback-common/util/PromiseUtil", "switchback-common/component/Signal"], function (require, exports, CookieUtil_1, PromiseUtil_2, Signal_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class exists to support on-desktop testing of the PlayAPI.
     * It returns a bunch of fake data, and can save and load data via browser storage.
     * @author joel@cerebralfix.com
     */
    class MockPlayAPI {
        static init() {
            // Overwrite the PlayAPI functions
            PlayAPI.gameInit = MockPlayAPI.gameInit;
            PlayAPI.locationRead = MockPlayAPI.locationRead;
            PlayAPI.locationWaitTimeRead = MockPlayAPI.locationWaitTimeRead;
            PlayAPI.gameStateRead = MockPlayAPI.gameStateRead;
            PlayAPI.gameStateSave = MockPlayAPI.gameStateSave;
            PlayAPI.gameStatsSave = MockPlayAPI.gameStatsSave;
            PlayAPI.gameStatsRead = MockPlayAPI.gameStatsRead;
            PlayAPI.openPlayerManagement = MockPlayAPI.openPlayerManagement;
            PlayAPI.captureImage = MockPlayAPI.captureImage;
            PlayAPI.saveToPhotos = MockPlayAPI.saveToPhotos;
            PlayAPI.shareImage = MockPlayAPI.shareImage;
            PlayAPI.startCamera = MockPlayAPI.startCamera;
            PlayAPI.cameraAccessStatus = MockPlayAPI.cameraAccessStatus;
            PlayAPI.endCamera = MockPlayAPI.endCamera;
            PlayAPI.closeGame = MockPlayAPI.closeGame;
            PlayAPI.grantAchievementToPlayers = MockPlayAPI.grantAchievementToPlayers;
            PlayAPI.analyticsStateLog = MockPlayAPI.analyticsStateLog;
            PlayAPI.analyticsActionLog = MockPlayAPI.analyticsActionLog;
            PlayAPI.analyticsActionObjectLog = MockPlayAPI.analyticsActionObjectLog;
            PlayAPI.analyticsTimedActionStart = MockPlayAPI.analyticsTimedActionStart;
            PlayAPI.analyticsTimedActionEnd = MockPlayAPI.analyticsTimedActionEnd;
            PlayAPI.vibrateDevice = MockPlayAPI.vibrateDevice;
            PlayAPI.subscribeToEvent = MockPlayAPI.subscribeToEvent;
            PlayAPI.unsubscribeToEvent = MockPlayAPI.unsubscribeToEvent;
            PlayAPI.DBeacon.sessionInit = MockPlayAPI.dbSessionInit;
            PlayAPI.DBeacon.sessionEnd = MockPlayAPI.dbSessionEnd;
            PlayAPI.DBeacon.triggerEffect = MockPlayAPI.triggerEffect;
            PlayAPI.AugmentedReality.sessionStart = MockPlayAPI.arSessionStart;
            PlayAPI.AugmentedReality.sessionEnd = MockPlayAPI.arSessionEnd;
            PlayAPI.AugmentedReality.registerImage = MockPlayAPI.arRegisterImage;
            PlayAPI.AugmentedReality.isAvailable = MockPlayAPI.arIsAvailable;
            PlayAPI.LocalStorage.list = MockPlayAPI.localStorageList;
            PlayAPI.LocalStorage.read = MockPlayAPI.localStorageRead;
            PlayAPI.LocalStorage.write = MockPlayAPI.localStorageWrite;
            PlayAPI.LocalStorage.unlink = MockPlayAPI.localStorageUnlink;
            PlayAPI.cancelSpeech = MockPlayAPI.cancelSpeech;
            PlayAPI.speak = MockPlayAPI.speak;
            PlayAPI.listAvailableVoices = MockPlayAPI.listAvailableVoices;
            // TODO: Figure out why this causes TSM to not load
            // (window as any).webkit = {messageHandlers:{NativeInterop:{postMessage: function (msg) {}}}};
        }
        static subscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] subscribeToEvent()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static unsubscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] unsubscribeToEvent()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static vibrateDevice() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] vibrateDevice()");
                return {};
            });
        }
        static startCamera(cameraType) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] startCamera()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    if (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS) {
                        reject();
                    }
                    else {
                        resolve({});
                    }
                }));
            });
        }
        static cameraAccessStatus() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] cameraAccessStatus()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {
                    status: (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS ? CameraAccessStatus.DENIED : CameraAccessStatus.AUTHORIZED)
                };
            });
        }
        static endCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] endCamera()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionLog(action, category) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionLog(${action}, ${category})`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionObjectLog(action, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionObjectLog(${action}, ${JSON.stringify(data)})`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsStateLog(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsStateLog(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionStart(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionStart(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionEnd(state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionEnd(${state}})`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static grantAchievementToPlayers(achievementId, playersIds, showNotification) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] grantAchievementToPlayers()");
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return {};
            });
        }
        static gameInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameInit()`);
                yield PromiseUtil_2.PromiseUtil.realTime(500);
                return MockPlayAPI.getFakeGameInitResult();
            });
        }
        static locationRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationRead()`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeLocation();
            });
        }
        static locationWaitTimeRead(location) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationWaitTimeRead()`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeWaitTime();
            });
        }
        static gameStateRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateRead()`);
                return MockPlayAPI.doGameStateRead();
            });
        }
        static gameStateSave(status, state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateSave()`);
                // CookieUtil.set(MockPlayAPI.GAME_STATE_TAG, JSON.stringify(state));
                return {};
            });
        }
        static dbSessionInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionInit()`);
                return {};
            });
        }
        static triggerEffect(effectName, data, timeout) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] triggerEffect()`);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PromiseUtil_2.PromiseUtil.realTime(timeout * 1000);
                    reject(); // The MockAPI should never succeed here
                }));
            });
        }
        static dbSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionEnd()`);
                return {};
            });
        }
        static arSessionStart(canvasElement) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionStart()`);
                PlayAPI.AugmentedReality.renderer = new THREE.WebGLRenderer({ alpha: true, canvas: canvasElement });
                PlayAPI.AugmentedReality.scene = new THREE.Scene();
                PlayAPI.AugmentedReality.renderer.setSize(window.innerWidth, window.innerHeight);
                PlayAPI.AugmentedReality.renderer.setPixelRatio(window.devicePixelRatio);
                PlayAPI.AugmentedReality.renderer.shadowMap.enabled = false;
                return {};
            });
        }
        static arSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionEnd()`);
                return {};
            });
        }
        static arRegisterImage(imageName, pathToImageInGameBundle, imageWidthInMeters) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arRegisterImage(${imageName}, ${pathToImageInGameBundle}, ${imageWidthInMeters})`);
                return {};
            });
        }
        static arIsAvailable() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arIsAvailable()`);
                return true;
            });
        }
        static localStorageList() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageList()`);
                return {
                    contents: []
                };
            });
        }
        static localStorageRead(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageRead()`);
                return {};
            });
        }
        static localStorageWrite(fileName, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageWrite()`);
                return {};
            });
        }
        static localStorageUnlink(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageUnlink()`);
                return {};
            });
        }
        static gameStatsRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsRead()`);
                return MockPlayAPI.doGameStatsRead();
            });
        }
        static gameStatsSave(gameStats) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsSave()`);
                CookieUtil_1.CookieUtil.set(MockPlayAPI.GAME_STATS_TAG, JSON.stringify(gameStats));
                return {};
            });
        }
        static openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.getFakePlayerManagementData();
            });
        }
        static captureImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] captureImage()`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return null;
            });
        }
        static saveToPhotos() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] saveToPhotos()`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return null;
            });
        }
        static shareImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] shareImage()`);
                yield PromiseUtil_2.PromiseUtil.realTime(300);
                return true;
            });
        }
        static closeGame() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.fakeCloseGame();
            });
        }
        static getFakeLocation() {
            const response = {
                attractionName: "Little Foot's Wild Ride",
                landName: "Land Before Time",
                latitude: 66.33,
                locationID: "beacon_" + Math.floor(Math.random() * 20),
                longitude: 100.22,
                parkName: "Animal Kingdom",
            };
            return response;
        }
        static doGameStateRead() {
            console.log(`[MockPlayAPI] doGameStateRead()`);
            let gameState;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATE_TAG));
            if (cookieData == null) {
                gameState = MockPlayAPI.getFakeGameState();
            }
            else {
                gameState = {
                    created: cookieData.created || "",
                    state: cookieData.state || {},
                    status: cookieData.status || "1",
                    updated: cookieData.updated || "",
                };
            }
            return gameState;
        }
        static doGameStatsRead() {
            console.log(`[MockPlayAPI] doGameStatsRead()`);
            let gameStats;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATS_TAG));
            if (cookieData == null) {
                gameStats = MockPlayAPI.getFakeGameStats();
            }
            else {
                gameStats = {
                    score: cookieData.score || 0,
                };
            }
            return gameStats;
        }
        static listAvailableVoices() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    let voicesList = window.speechSynthesis.getVoices();
                    if (voicesList.length === 0) {
                        const voicesLoadedSignal = new Signal_1.Signal();
                        window.speechSynthesis.onvoiceschanged = () => {
                            voicesLoadedSignal.dispatch();
                        };
                        yield PromiseUtil_2.PromiseUtil.commonSignal(voicesLoadedSignal);
                        voicesList = window.speechSynthesis.getVoices();
                    }
                    voicesList.forEach(element => {
                        element["identifier"] = element.voiceURI;
                    });
                    return { "voices": voicesList };
                }
                else {
                    return { "voices": [] };
                }
            });
        }
        static speak(text, voiceId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    const textToRead = new SpeechSynthesisUtterance(text);
                    const voice = window.speechSynthesis.getVoices().find((singleVoice) => {
                        return singleVoice.voiceURI === voiceId;
                    });
                    textToRead.voice = voice;
                    window.speechSynthesis.speak(textToRead);
                }
                return {};
            });
        }
        static cancelSpeech() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    window.speechSynthesis.cancel();
                }
                return {};
            });
        }
        /////////////////////////////////
        // FAKE DATA GENERATION
        /////////////////////////////////
        static getFakeGameInitResult() {
            const response = {
                gameDefinitionRead: MockPlayAPI.getFakeGameDefinition(),
                gameStateRead: MockPlayAPI.doGameStateRead(),
                playerInfoRead: MockPlayAPI.getFakePlayerInformation(),
            };
            return response;
        }
        static getFakeWaitTime() {
            const response = {
                waitTime: Math.floor(Math.random() * 150),
            };
            return response;
        }
        static getFakePlayerInformation() {
            const response = {
                avatar: "",
                email: "example@gmail.com",
                name: "Joe A"
            };
            return response;
        }
        static getFakeGameDefinition() {
            const response = {
                stateSchema: {},
                statsSchema: {},
                title: "game_definition_title",
            };
            return response;
        }
        static getFakeGameState() {
            const response = {
                created: "game_state_created",
                state: "game_state_state",
                status: "game_state_status",
                updated: "game_state_updated",
            };
            return response;
        }
        static getFakeGameStats() {
            const response = {
                score: 0,
            };
            return response;
        }
        static getFakePlayerManagementData(limit = null) {
            const players = [{ name: "Cinderella", type: "TYPE_PRIMARY", identifier: "" },
                { name: "Elsa", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Snow White", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Leia", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Tinker Bell", type: "TYPE_SECONDARY", identifier: "" },
            ];
            if (limit != null) {
                players.splice(limit);
            }
            return { players };
        }
        static fakeCloseGame() {
            console.log("The game got closed using mockplayApi!");
            return {};
        }
    }
    MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS = false;
    MockPlayAPI.GAME_STATE_TAG = "game-state";
    MockPlayAPI.GAME_STATS_TAG = "game-stats";
    exports.MockPlayAPI = MockPlayAPI;
});
define("switchback-common/play/AchievementManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AchievementManager {
        initialise(players, achievementPrefix) {
            this.playerIds = players.map((player) => player.identifier);
            this.achievementPrefix = achievementPrefix;
            console.log(`[AchievementManager] initialise(${this.playerIds}, ${achievementPrefix}`);
        }
        grantAchievement(achievementId) {
            return __awaiter(this, void 0, void 0, function* () {
                const achievementString = `${this.achievementPrefix}-${achievementId}`;
                yield PlayAPI.grantAchievementToPlayers(achievementString, this.playerIds, true).then(() => {
                    console.log(`achievement ${achievementString} grant success`);
                }, (reason) => {
                    console.log(`achievement ${achievementString} grant fail`);
                    console.error(JSON.stringify(reason));
                });
            });
        }
    }
    exports.AchievementManager = AchievementManager;
});
define("model/LayoutType", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LayoutType {
    }
    LayoutType.INTRO = "INTRO";
    LayoutType.TEXT_WITH_IMAGE = "TEXT_WITH_IMAGE";
    LayoutType.JOKE_PAGE = "JOKE_PAGE";
    LayoutType.OPEN_QUESTION = "OPEN_QUESTION";
    LayoutType.CHARADES = "CHARADES";
    LayoutType.PICK_ONE_OF_TWO = "PICK_ONE_OF_TWO";
    LayoutType.COUNTER = "COUNTER";
    LayoutType.MULTI_OPTIONS = "MULTI_OPTIONS";
    LayoutType.TWO_TRUTHS = "TWO_TRUTHS";
    LayoutType.MULTI_PAGE = "MULTI_PAGE";
    exports.LayoutType = LayoutType;
});
define("logic/PPAchievementManager", ["require", "exports", "switchback-common/play/AchievementManager", "model/LayoutType", "logic/GameController"], function (require, exports, AchievementManager_1, LayoutType_1, GameController_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PPAchievement {
    }
    PPAchievement.ALL_ACTIVITY_TYPES = "peterPanHappyThoughts";
    PPAchievement.REACH_MERGE = "peterPanReadyToFly";
    PPAchievement.COLLECT_TRINKETS_SMALL = "peterPanTinkerAway";
    PPAchievement.COLLECT_TRINKETS_MEDIUM = "peterPanDaydreamer";
    PPAchievement.COLLECT_TRINKETS_LARGE = "peterPanVividImagination";
    PPAchievement.COLLECT_TRINKETS_HUGE = "peterPanFantastical";
    PPAchievement.HAPPY_THOUGHTS_SMALL = "peterPanMagicalMusings";
    PPAchievement.HAPPY_THOUGHTS_LARGE = "peterPanMemoriesMade";
    PPAchievement.SPEND_TIME_ON_ACTIVITY = "peterPanDeepDiscussion";
    PPAchievement.TRIGGER_LANTERN_INSTALLATION = "peterPanPixieDust";
    PPAchievement.UNANIMOUS_DECISION = "peterPanHarmony";
    class PPAchievementManager extends AchievementManager_1.AchievementManager {
        onActivityCompleted(LayoutType, timeSpentMs) {
            return __awaiter(this, void 0, void 0, function* () {
                if (LayoutType) {
                    GameController_1.GameController.Instance.state.addLayoutTypeCompleted(LayoutType);
                }
                let completedAll = true;
                PPAchievementManager.TYPES_REQUIRED_FOR_ALL_ACTIVITY_TYPES_ACHIEVEMENT.forEach((type) => {
                    if (GameController_1.GameController.Instance.state.layoutTypesCompleted.indexOf(type) === -1) {
                        completedAll = false;
                    }
                });
                if (completedAll) {
                    yield this.grantAchievement(PPAchievement.ALL_ACTIVITY_TYPES);
                }
                if (timeSpentMs > PPAchievementManager.TIME_FOR_ACHIEVEMENT_MS) {
                    yield this.grantAchievement(PPAchievement.SPEND_TIME_ON_ACTIVITY);
                }
                GameController_1.GameController.Instance.state.numberOfHappyThoughtsCreated++;
                if (GameController_1.GameController.Instance.state.numberOfHappyThoughtsCreated > PPAchievementManager.HAPPY_THOUGHTS_FOR_SMALL) {
                    yield this.grantAchievement(PPAchievement.HAPPY_THOUGHTS_SMALL);
                }
                if (GameController_1.GameController.Instance.state.numberOfHappyThoughtsCreated > PPAchievementManager.HAPPY_THOUGHTS_FOR_LARGE) {
                    yield this.grantAchievement(PPAchievement.HAPPY_THOUGHTS_LARGE);
                }
            });
        }
        onLanternTrigger() {
            return __awaiter(this, void 0, void 0, function* () {
                yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.TRIGGER_LANTERN_INSTALLATION);
            });
        }
        onUnanimousDecision() {
            return __awaiter(this, void 0, void 0, function* () {
                yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.UNANIMOUS_DECISION);
            });
        }
        onMergeReached() {
            return __awaiter(this, void 0, void 0, function* () {
                yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.REACH_MERGE);
            });
        }
        onTrinketFound() {
            return __awaiter(this, void 0, void 0, function* () {
                if (GameController_1.GameController.Instance.state.selectedTrinkets.length >= PPAchievementManager.TRINKETS_FOR_SMALL) {
                    yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.COLLECT_TRINKETS_SMALL);
                }
                if (GameController_1.GameController.Instance.state.selectedTrinkets.length >= PPAchievementManager.TRINKETS_FOR_MEDIUM) {
                    yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.COLLECT_TRINKETS_MEDIUM);
                }
                if (GameController_1.GameController.Instance.state.selectedTrinkets.length >= PPAchievementManager.TRINKETS_FOR_LARGE) {
                    yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.COLLECT_TRINKETS_LARGE);
                }
                if (GameController_1.GameController.Instance.state.selectedTrinkets.length >= PPAchievementManager.TRINKETS_FOR_HUGE) {
                    yield GameController_1.GameController.Instance.achievementManager.grantAchievement(PPAchievement.COLLECT_TRINKETS_HUGE);
                }
            });
        }
    }
    PPAchievementManager.TYPES_REQUIRED_FOR_ALL_ACTIVITY_TYPES_ACHIEVEMENT = [
        LayoutType_1.LayoutType.JOKE_PAGE,
        LayoutType_1.LayoutType.OPEN_QUESTION,
        LayoutType_1.LayoutType.CHARADES,
        LayoutType_1.LayoutType.PICK_ONE_OF_TWO,
        LayoutType_1.LayoutType.COUNTER,
        LayoutType_1.LayoutType.MULTI_OPTIONS
    ];
    PPAchievementManager.TIME_FOR_ACHIEVEMENT_MS = 300000;
    PPAchievementManager.TRINKETS_FOR_SMALL = 5;
    PPAchievementManager.TRINKETS_FOR_MEDIUM = 10;
    PPAchievementManager.TRINKETS_FOR_LARGE = 15;
    PPAchievementManager.TRINKETS_FOR_HUGE = 20;
    PPAchievementManager.HAPPY_THOUGHTS_FOR_SMALL = 10;
    PPAchievementManager.HAPPY_THOUGHTS_FOR_LARGE = 30;
    exports.PPAchievementManager = PPAchievementManager;
});
define("switchback-common/enum/Park", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Listing of Park 'types'
     *
     * I know it's not an enum, but this is to prevent
     * people with older TS versions having issues.
     */
    class Park {
    }
    Park.DLR = "disneylandResort";
    Park.WDW = "waltDisneyWorld";
    exports.Park = Park;
});
define("model/PeterPanConfig", ["require", "exports", "switchback-common/enum/Park"], function (require, exports, Park_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Config singleton. Parsed from json.
     * TODO: Make this not a singleton and find a good spot for it
     * @author joel@cerebralfix.com
     */
    class PeterPanConfig {
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        parse(data) {
            if (typeof data === 'string') {
                try {
                    data = JSON.parse(data);
                }
                catch (e) {
                    console.log('Unable to parse config.');
                }
            }
            this.enableConsoleLogs = data.enableConsoleLogs === undefined ? false : data.enableConsoleLogs;
            this.enableFPS = data.enableFPS === undefined ? false : data.enableFPS;
            this.usePlayAPI = data.usePlayAPI === undefined ? true : data.usePlayAPI;
            this.debug = data.debug === undefined ? false : data.debug;
            this.showDebugMenu = data.showDebugMenu === undefined ? false : data.showDebugMenu;
            this.lanternEffectDLRQueueATag = data.lanternEffectDLRQueueATag === undefined ? "dlr-dl-pan-lantern-queue" : data.lanternEffectDLRQueueATag;
            this.lanternEffectDLRQueueBTag = data.lanternEffectDLRQueueBTag === undefined ? "dlr-dl-pan-lantern-ada" : data.lanternEffectDLRQueueBTag;
            // Defaulting this to DLR now that we cannot guarantee the gameInit response will give us something we expect
            this.park = data.park || Park_1.Park.DLR;
        }
    }
    exports.PeterPanConfig = PeterPanConfig;
});
define("enum/Queue", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Queue {
    }
    Queue.PRIMARY = "1";
    Queue.SECONDARY = "2";
    exports.Queue = Queue;
});
define("logic/sequence-controllers/TinksTrinketsSequenceController", ["require", "exports", "logic/GameController", "enum/Queue"], function (require, exports, GameController_2, Queue_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Perhaps the second most important controller - this one
     * returns a Tinks Trinkets struct for the book to show.
     *
     * TODO: Add more logic around the specific trinkets to show
     *
     * @author joel@cerebralfix.com
     */
    class TinksTrinketsSequenceController {
        constructor() {
            this.maxPromptFrequency = 300000; // 5 mins
            this.lastDisplay = 0;
        }
        init(tinksTrinketsJson) {
            this.tinksTrinketsJson = tinksTrinketsJson[GameController_2.GameController.Instance.parkPrefix];
        }
        get hasOptedOutOfTT() {
            return this._hasOptedOutOfTT;
        }
        set hasOptedOutOfTT(optedOut) {
            this._hasOptedOutOfTT = optedOut;
        }
        get hasFinishedTT() {
            return GameController_2.GameController.Instance.tinksTrinketsSequenceController.hasOptedOutOfTT ||
                GameController_2.GameController.Instance.state.selectedTrinkets.length === this.totalTrinketsToFind;
        }
        set lastDisplayTime(time) {
            this.lastDisplay = time;
        }
        get totalTrinketsToFind() {
            return GameController_2.GameController.Instance.state.selectedQueueType === Queue_1.Queue.SECONDARY ?
                this.tinksTrinketsJson.secondary_queue.trinkets.length :
                this.tinksTrinketsJson.primary_queue.trinkets.length;
        }
        canInterrupt() {
            if (this.hasFinishedTT) {
                return false;
            }
            const currentTime = new Date().getTime();
            if (this.lastDisplay === 0 || currentTime > this.lastDisplay + this.maxPromptFrequency) {
                return true;
            }
            return false;
        }
        get tinksTrinketsRawData() {
            // use the correct data for the queue
            let data = this.tinksTrinketsJson.primary_queue;
            if (GameController_2.GameController.Instance.state.selectedQueueType === Queue_1.Queue.SECONDARY) {
                // Assign the secondary queue data (defaulting to primary if we somehow are missing that data)
                data = this.tinksTrinketsJson.secondary_queue || this.tinksTrinketsJson.primary_queue;
            }
            return data;
        }
    }
    exports.TinksTrinketsSequenceController = TinksTrinketsSequenceController;
});
define("switchback-common/play/AnalyticsManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AnalyticsManager {
        constructor() {
            // Needs to be overwritten on the extending class
            this.gaNameMap = new Map();
            this.timings = new Map();
            this.gaCommonNameMap = new Map([
                ["park.name", "dimension1"],
                ["line.type", "dimension2"],
                ["attraction.estimatedWaitTime", "dimension3"],
                // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
                // ['attraction.actualWaitTime', 'dimension4'],
                ["party.size", "dimension5"],
                ["party.visits", "dimension6"],
            ]);
            this.ga = null;
        }
        /**
         * "A" = Standard
         * "B" = Other
         * "N/A" = No queue distinctions
         */
        set lineType(type) {
            this.baseData["line.type"] = type;
        }
        set estimatedWaitTime(time) {
            this.baseData["attraction.estimatedWaitTime"] = time;
        }
        set numPartyVisits(numVisits) {
            this.baseData["party.visits"] = numVisits;
        }
        set actualWaitTime(time) {
            // Disabled because we were advised that this parameter should not be in adobe metrics
            // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
            // after that this code can be cleaned up
            // this.baseData['attraction.actualWaitTime'] = time;
        }
        /**
         * Starts a timing event.
         * For Adobe, this is automatic.
         * For Google analytics, we record the time ourselves.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         */
        startTiming(adobeTimingKey, data = {}) {
            this.timings.set(adobeTimingKey, new Date().getTime());
            PlayAPI.analyticsTimedActionStart(adobeTimingKey, Object.assign({}, this.baseData, data));
        }
        /**
         * Ends and sends a timing event.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         * @param gaCategory The category of this event e.g. 'Installation'
         * @param gaLabel Additional information about the action e.g. 'Poster'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        endTiming(adobeTimingKey, gaTimingKey, gaCategory, gaLabel, data = {}) {
            return __awaiter(this, void 0, void 0, function* () {
                const prevTime = this.timings.get(adobeTimingKey);
                if (prevTime) {
                    const totalTime = new Date().getTime() - prevTime;
                    this.ga("send", "timing", Object.assign({ timingCategory: gaCategory, timingLabel: gaLabel, timingValue: totalTime, timingVar: gaTimingKey }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
                }
                yield PlayAPI.analyticsTimedActionEnd(adobeTimingKey).then((result) => {
                    console.log(result);
                }).catch((error) => {
                    console.error(error);
                });
            });
        }
        /**
         * Tracks an Adobe 'Action' or a Google 'event'.
         *
         * @param gaCategory The category of action e.g. 'Game'
         * @param gaAction The specific action they took e.g. 'Play'
         * @param gaLabel Further information about that action e.g. 'Woody'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackAction(adobeAction, gaCategory, gaAction, gaLabel, data = {}) {
            PlayAPI.analyticsActionObjectLog(adobeAction, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ eventAction: gaAction, eventCategory: gaCategory, eventLabel: gaLabel, hitType: "event" }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        /**
         * Tracks an Adobe 'state' or a Google 'pageview'
         *
         * @param adobeStatePath e.g. 'content/game/inqueue/resume'
         * @param gaStateId e.g. 'Resume'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackState(adobeStatePath, gaStateId, data = {}) {
            PlayAPI.analyticsStateLog(adobeStatePath, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ hitType: "pageview", page: gaStateId }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        initialise(category, landName, attractionName, gameId, gameName, parkName, lineType, hasPrimaryPlayer, partySize, partyVisits, useGA = false) {
            this.baseData = {
                "attraction.name": attractionName,
                "game.id": gameId,
                "game.name": gameName,
                "land.name": landName,
                "line.type": lineType,
                "link.category": category,
                "park.name": parkName,
                "party.size": partySize,
                "party.visits": partyVisits,
                "primary.player": hasPrimaryPlayer ? 1 : 0,
            };
            // Combine both our name maps
            this.gaNameMap = new Map([
                ...Array.from(this.gaNameMap.entries()),
                ...Array.from(this.gaCommonNameMap.entries()),
            ]);
            if (useGA) {
                this.ga = window.ga;
            }
            else {
                this.ga = (...params) => { console.log(...params); };
            }
        }
        /**
         * Extract custom data into a format Google Analytics will accept
         */
        transformFieldsObject(data) {
            const transformedData = {};
            Object.keys(data).forEach((key) => {
                const mapping = this.gaNameMap.get(key);
                if (mapping && data[key] !== undefined && data[key] !== null) {
                    if (mapping.startsWith("dimension")) {
                        data[key] = data[key].toString(); // GA requires custom dimensions pass through as strings
                    }
                    transformedData[mapping] = data[key];
                }
            });
            return transformedData;
        }
    }
    AnalyticsManager.PARK_WDW = "WDW";
    AnalyticsManager.PARK_DLR = "DLR";
    AnalyticsManager.LAND_HS = "HS";
    AnalyticsManager.LAND_MK = "MK";
    AnalyticsManager.LAND_DCA = "DCA";
    AnalyticsManager.LAND_DL = "DL";
    AnalyticsManager.LAND_AK = "AK";
    AnalyticsManager.QUEUE_A = "A";
    AnalyticsManager.QUEUE_B = "B";
    AnalyticsManager.QUEUE_NONE = "N/A";
    exports.AnalyticsManager = AnalyticsManager;
});
define("logic/PPAnalyticsManager", ["require", "exports", "switchback-common/play/AnalyticsManager", "switchback-common/enum/Park", "logic/GameController"], function (require, exports, AnalyticsManager_1, Park_2, GameController_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PPAnalyticsManager extends AnalyticsManager_1.AnalyticsManager {
        constructor() {
            super(...arguments);
            this.gaNameMap = new Map([
                [PPAnalyticsManager.GAME_NAME, 'dimension7'],
                [PPAnalyticsManager.GAME_PLAYS, 'dimension8'],
                [PPAnalyticsManager.GAME_ALLPLAYS, 'dimension9'],
                [PPAnalyticsManager.ACTIVITY_SET_INDEX, 'dimension10'],
                [PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX, 'dimension11'],
                [PPAnalyticsManager.ACTIVITY_SET_THEME, 'dimension12'],
                [PPAnalyticsManager.ACTIVITY_SET_THEME_COMPLETED, 'dimension13'],
                [PPAnalyticsManager.ITEM_ID, 'dimension14'],
                [PPAnalyticsManager.ITEM_COMPLETED, 'dimension15'],
                [PPAnalyticsManager.CUTSCENE, 'dimension16'],
                [PPAnalyticsManager.INSTALLATION_TRIGGER, 'dimension17'],
                [PPAnalyticsManager.INSTALLATION_NAME, 'dimension18'],
            ]);
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init(park, parkString, hasPrimaryPlayer, partySize, partyVisits) {
            let landString = park === Park_2.Park.WDW ? AnalyticsManager_1.AnalyticsManager.LAND_MK : AnalyticsManager_1.AnalyticsManager.LAND_DL;
            super.initialise("InQueue", landString, "Peter Pan", "peterPansFlight", "Off to Never Land", parkString, AnalyticsManager_1.AnalyticsManager.QUEUE_NONE, hasPrimaryPlayer, partySize, partyVisits);
        }
        trackState(adobeStatePath, gaStateId, data = {}) {
            super.trackState(adobeStatePath, gaStateId, Object.assign({}, data, { [PPAnalyticsManager.GAME_ALLPLAYS]: GameController_3.GameController.Instance.state.numActivitiesCompleted, [PPAnalyticsManager.ITEM_COMPLETED]: GameController_3.GameController.Instance.state.selectedTrinkets.length }));
        }
        startTiming(adobeTimingKey, data = {}) {
            super.startTiming(adobeTimingKey, Object.assign({}, data, { [PPAnalyticsManager.GAME_ALLPLAYS]: GameController_3.GameController.Instance.state.numActivitiesCompleted, [PPAnalyticsManager.ITEM_COMPLETED]: GameController_3.GameController.Instance.state.selectedTrinkets.length }));
        }
        endTiming(adobeTimingKey, gaTimingKey, gaCategory, gaLabel, data = {}) {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("endTiming").call(this, adobeTimingKey, gaTimingKey, gaCategory, gaLabel, Object.assign({}, data, { [PPAnalyticsManager.GAME_ALLPLAYS]: GameController_3.GameController.Instance.state.numActivitiesCompleted, [PPAnalyticsManager.ITEM_COMPLETED]: GameController_3.GameController.Instance.state.selectedTrinkets.length }));
            });
        }
        trackAction(adobeAction, gaCategory, gaAction, gaLabel, data = {}) {
            super.trackAction(adobeAction, gaCategory, gaAction, gaLabel, Object.assign({}, data, { [PPAnalyticsManager.GAME_ALLPLAYS]: GameController_3.GameController.Instance.state.numActivitiesCompleted, [PPAnalyticsManager.ITEM_COMPLETED]: GameController_3.GameController.Instance.state.selectedTrinkets.length }));
        }
    }
    PPAnalyticsManager.GAME_NAME = "game.name.mini";
    PPAnalyticsManager.GAME_PLAYS = "game.plays";
    PPAnalyticsManager.GAME_ALLPLAYS = "game.allplays";
    PPAnalyticsManager.ACTIVITY_SET_INDEX = "activity.set.index";
    PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX = "activity.set.activityindex";
    PPAnalyticsManager.ACTIVITY_SET_THEME = "activity.set.theme";
    PPAnalyticsManager.ACTIVITY_SET_THEME_COMPLETED = "activity.set.theme.completed";
    PPAnalyticsManager.ITEM_ID = "item.id";
    PPAnalyticsManager.ITEM_COMPLETED = "item.completed";
    PPAnalyticsManager.CUTSCENE = "cutscene";
    PPAnalyticsManager.INSTALLATION_TRIGGER = "installation.trigger";
    PPAnalyticsManager.INSTALLATION_NAME = "installation.name";
    exports.PPAnalyticsManager = PPAnalyticsManager;
});
define("switchback-common/util/TextUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextUtil {
        /**
         * Calculates the font size modifier used when getting the fontsize based on
         * the devices aspect ratio. This is used in getFontSize(). For wider devices the
         * fontSizeModifier is 600 and for narrow devices is uses 400. These values were
         * choosen because they make the text look nice.
         * @param gameWidth Canvas width
         * @param gameHeight Canvas height
         */
        static calculateFontSizeModifier(gameWidth, gameHeight) {
            TextUtil.fontSizeModifier = 400;
            let aspectRatio = (gameHeight / gameWidth);
            // Finding the aspect ratio of the biggest edge to the smallest edge. Using
            // height / width as the base check because most games should be run in portrait
            if (aspectRatio < 1) {
                // Invert aspect ratio if less than 1.
                aspectRatio = 1 / aspectRatio;
            }
            if (aspectRatio <= 1.5) {
                TextUtil.fontSizeModifier = 600;
            }
        }
        static leftPad(text, padCharacter, width) {
            return width - text.length > 0 ? padCharacter.repeat(width - text.length) + text : text;
        }
        static rightPad(text, padCharacter, width) {
            return width - text.length > 0 ? text + padCharacter.repeat(width - text.length) : text;
        }
        // Returns a font size value that is usable by Phaser at the scale of the desktop font.
        static getFontSize(gameWidth, fontPtSize) {
            return (fontPtSize / TextUtil.fontSizeModifier) * gameWidth;
        }
        static formattedTime(seconds) {
            const min = Math.max(0, Math.floor(seconds / 60)).toString();
            const sec = (seconds % 60).toFixed(0);
            return TextUtil.leftPad(min, "0", 2) + ":" + TextUtil.leftPad(sec, "0", 2);
        }
        static getOrdinalFormattedString(n) {
            const s = ["th", "st", "nd", "rd"];
            const v = n % 100;
            return n + (s[(v - 20) % 10] || s[v] || s[0]);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the width.
         * This is used to ensure that single line textfields don't overflow.
         *
         * Will not work if word-wrap is turned on (as word wrap ensures it never overflows width)
         */
        static scaleDownSingleLineTextToWidth(textField, width) {
            const condition = (textField, sizeRestriction) => {
                return textField.width > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, width, condition);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the height.
         */
        static scaleDownTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, height, condition);
        }
        static scaleUpTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height < sizeRestriction;
            };
            TextUtil.scaleText(textField, height, condition, true);
        }
        /**
         * Passing in an array of @interface IFontSize @param textObjects to this function
         * will attempt to equalize their font sizes to either the largest,
         * or smallest, depending on @param equalizeOnSmallest.
         */
        static equalizeFontSizes(textObjects, equalizeOnSmallest) {
            let size = equalizeOnSmallest ? Infinity : 0;
            textObjects.forEach((obj) => {
                size = equalizeOnSmallest ? Math.min(size, obj.fontSize) : Math.max(size, obj.fontSize);
            });
            textObjects.forEach((obj) => {
                obj.fontSize = size;
            });
        }
        static scaleText(textField, sizeRestriction, condition, increment = false) {
            const alignment = textField.align;
            let fontSize = textField.fontSize;
            while (condition(textField, sizeRestriction)) {
                textField.fontSize = increment ? fontSize++ : fontSize--;
            }
            // this is required as the text doesn't repaint automatically (phaser quirk)
            textField.align = alignment;
        }
    }
    TextUtil.fontSizeModifier = 400;
    exports.TextUtil = TextUtil;
});
define("switchback-common/save/SaveState", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SaveState {
        constructor() {
            this._isSaveEnabled = true;
        }
        save() {
            if (this._isSaveEnabled) {
                this.forceSave();
            }
        }
        clear() {
            console.log("[SaveState] clear");
            this.parse([]);
            this.forceSave();
        }
        /**
         * This disables all saves from this point. Clear will still function as expected.
         */
        disableSave() {
            this._isSaveEnabled = false;
        }
        forceSave() {
            PlayAPI.gameStateSave(GameStatus.GAME_STATUS_IN_PROGRESS, this).then(SaveState.onSaveSucceed, SaveState.onSaveFailed).catch(SaveState.onSaveExcept);
        }
        static onSaveSucceed() {
            console.log("[SaveState] save() success");
        }
        static onSaveFailed(err) {
            console.error("[SaveState] save() failed " + JSON.stringify(err));
        }
        static onSaveExcept(err) {
            console.error("[SaveState] save() inner exception" + JSON.stringify(err));
        }
    }
    exports.SaveState = SaveState;
});
define("model/PanSaveState", ["require", "exports", "switchback-common/save/SaveState"], function (require, exports, SaveState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PanSaveState extends SaveState_1.SaveState {
        get sessionStarted() {
            return this._sessionStarted;
        }
        set sessionStarted(started) {
            this._sessionStarted = started;
            this.save();
        }
        get layoutTypesCompleted() {
            return this._layoutTypesCompleted;
        }
        addLayoutTypeCompleted(layoutType) {
            if (this._layoutTypesCompleted.indexOf(layoutType) === -1) {
                this._layoutTypesCompleted.push(layoutType);
            }
            this.save();
        }
        get numberOfHappyThoughtsCreated() {
            return this._numberOfHappyThoughtsCreated;
        }
        set numberOfHappyThoughtsCreated(num) {
            this._numberOfHappyThoughtsCreated = num;
            this.save();
        }
        get playerNames() {
            return this._playerNames;
        }
        // No save here because it gave timming problems
        // also it gave the fake sense of security, because we need to reset and change it later,
        // so anyway, we needed to call .save after pushing all the elements
        set playerNames(playerNames) {
            this._playerNames = playerNames;
        }
        get hasSeenIntroSet() {
            return this._hasSeenIntroSet;
        }
        set hasSeenIntroSet(seenIntro) {
            this._hasSeenIntroSet = seenIntro;
            this.save();
        }
        get currentSetID() {
            return this._currentSetID;
        }
        set currentSetID(setID) {
            this._currentSetID = setID;
            this.save();
        }
        getSetViews(setID) {
            // not a get because getters cant have parameters and returning the whole map will introduce
            // bugs where the map is modified outside this class and no .save() gets called
            // or gets modified with unintentional side effects
            // also it's needed to be able to return default values for not saved sets (fail-proof)
            return this._setViews[setID] || 0;
        }
        setSetViews(setID, views) {
            console.log("[Savestate] setView set", setID, "to", views);
            this._setViews[setID] = views;
            this.save();
        }
        getActivityPlays(activityType) {
            return this._activityPlays[activityType] || 0;
        }
        incrementActivityPlays(activityType) {
            this._activityPlays[activityType] = this._activityPlays[activityType] + 1 || 1;
            return this._activityPlays[activityType];
        }
        get currentActivityIndex() {
            return this._currentActivityIndex;
        }
        set currentActivityIndex(page) {
            this._currentActivityIndex = page;
            this.save();
        }
        get currentDrawingPromptIndex() {
            return this._currentDrawingPromptIndex;
        }
        set currentDrawingPromptIndex(index) {
            this._currentDrawingPromptIndex = index;
        }
        get numVisits() {
            return this._numVisits;
        }
        set numVisits(num) {
            this._numVisits = num;
            this.save();
        }
        get numActivitiesCompleted() {
            return this._numActivitiesCompleted;
        }
        set numActivitiesCompleted(num) {
            this._numActivitiesCompleted = num;
            this.save();
        }
        get showTinksTrinkets() {
            return this._showTinksTrinkets;
        }
        set showTinksTrinkets(showTinksTrinkets) {
            this._showTinksTrinkets = showTinksTrinkets;
            this.save();
        }
        get hasSeenTinksTrinketsIntro() {
            return this._hasSeenTinksTrinketsIntro;
        }
        set hasSeenTinksTrinketsIntro(seenTinksTrinketsIntro) {
            this._hasSeenTinksTrinketsIntro = seenTinksTrinketsIntro;
            this.save();
        }
        get selectedQueueType() {
            return this._selectedQueueType;
        }
        set selectedQueueType(queue) {
            this._selectedQueueType = queue;
            this.save();
        }
        get hasSeenIntro() {
            return this._hasSeenIntro;
        }
        set hasSeenIntro(seenIntro) {
            this._hasSeenIntro = seenIntro;
            this.save();
        }
        get selectedTrinkets() {
            return this._selectedTrinkets;
        }
        isTrinketSelected(trinket) {
            return this._selectedTrinkets.indexOf(trinket) > -1;
        }
        setTrinketValue(trinket, selected) {
            if (selected) {
                if (this._selectedTrinkets.indexOf(trinket) < 0) {
                    this._selectedTrinkets.push(trinket);
                }
            }
            else {
                const index = this._selectedTrinkets.indexOf(trinket);
                this._selectedTrinkets.splice(index, 1);
            }
            this.save();
        }
        get sessionPlayTime() {
            return this._sessionPlayTime;
        }
        set sessionPlayTime(time) {
            this._sessionPlayTime = time;
        }
        parse(data) {
            console.log("[Savestate] parse", data);
            if (!data) {
                data = {};
            }
            this._numActivitiesCompleted = data._numActivitiesCompleted || 0;
            this._selectedTrinkets = data._selectedTrinkets || [];
            this._showTinksTrinkets = data._showTinksTrinkets || false;
            this._hasSeenIntro = data._hasSeenIntro || false;
            this._hasSeenTinksTrinketsIntro = data._hasSeenTinksTrinketsIntro || false;
            this._selectedQueueType = data._selectedQueueType; // If this is undefined, it hasn't been selected - no default needed
            this._playerNames = data._playerNames || [];
            this._sessionStarted = data._sessionStarted || false;
            this._layoutTypesCompleted = data._layoutTypesCompleted || [];
            this._numberOfHappyThoughtsCreated = data._numberOfHappyThoughtsCreated || 0;
            this._sessionPlayTime = data._sessionPlayTime || 0;
            this._numVisits = data._numVisits || 0;
            // Sequence/Set/Theme stuff
            this._currentActivityIndex = data._currentActivityIndex || 0;
            this._currentDrawingPromptIndex = data._currentDrawingPromptIndex || 1;
            this._currentSetID = data._currentSetID || -1;
            this._hasSeenIntroSet = data._hasSeenIntroSet || false;
            this._setViews = {};
            for (let k in (data._setViews || {})) {
                let v = data._setViews[k];
                if (k !== null && v !== null) {
                    this._setViews[k] = v;
                }
            }
            this._activityPlays = {};
            for (let k in (data._activityPlays || {})) {
                let v = data._activityPlays[k];
                if (k !== null && v !== null) {
                    this._activityPlays[k] = v;
                }
            }
            // ~
            return this;
        }
    }
    exports.PanSaveState = PanSaveState;
});
define("switchback-common/play/ShowEffectController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ShowEffectResult;
    (function (ShowEffectResult) {
        ShowEffectResult[ShowEffectResult["FAILURE"] = 0] = "FAILURE";
        ShowEffectResult[ShowEffectResult["SUCCESS"] = 1] = "SUCCESS";
        ShowEffectResult[ShowEffectResult["IN_USE"] = 2] = "IN_USE";
    })(ShowEffectResult = exports.ShowEffectResult || (exports.ShowEffectResult = {}));
    class ShowEffectController {
        constructor() {
            this.showControlResult = new Phaser.Signal();
        }
        init() {
            console.log('[ShowEffectController] init()');
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.FAILURE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.SUCCESS, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.DBeacon.sessionInit().then((result) => {
                console.log("PlayAPI.DBeacon.sessionInit successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionInit error:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            // N.B. We have to subscribe to this in order for the success events to work!
            // Do not remove!!
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE, (response) => { return; });
            console.log('[ShowEffectController] init() complete');
        }
        shutdown() {
            PlayAPI.DBeacon.sessionEnd().then((result) => {
                console.log("PlayAPI.DBeacon.sessionEnd successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionEnd error:\n" + JSON.stringify(error));
            });
        }
        get defaultShowEffectName() {
            return this._defaultShowEffectName;
        }
        set defaultShowEffectName(effectName) {
            this._defaultShowEffectName = effectName;
        }
        triggerDefaultEffect(timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this._defaultShowEffectName) {
                    return ShowEffectResult.FAILURE;
                }
                return yield this.triggerEffect(this._defaultShowEffectName, timeoutS);
            });
        }
        /**
         * Triggers an effect via the Play API.
         * with forced local timeout
         */
        triggerEffect(effectName, timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ShowEffectController] triggerEffect() " + effectName);
                return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    if (this.isInitializeFailed) {
                        resolve(ShowEffectResult.FAILURE);
                        return;
                    }
                    let secondaryTimeoutId;
                    this.showControlResult.add((result, response) => {
                        console.log("[ShowEffectController] triggerEffect() this.showControlResult.add(): " + result + " " + response.experienceName);
                        // ensure we heard the show control message for the effect we are waiting for
                        if (response.experienceName === effectName) {
                            if (secondaryTimeoutId !== undefined) {
                                clearTimeout(secondaryTimeoutId);
                            }
                            this.showControlResult.removeAll();
                            resolve(result);
                        }
                    });
                    PlayAPI.DBeacon.triggerEffect(effectName, {}, timeoutS).then((result) => {
                        console.log("PlayAPI.DBeacon.triggerEffect successfully called:\n" + JSON.stringify(result));
                        // additional safety check incase any of the show controller events fail to respond
                        // this is double the time that the PlayAPI says that requires to complete its calls
                        // this is only required as in somecases the PlayAPI has not been returning anything
                        secondaryTimeoutId = setTimeout(() => {
                            console.warn("PlayAPI.ShowControl.triggerEffect forced local timeout");
                            this.showControlResult.removeAll();
                            resolve(ShowEffectResult.FAILURE);
                        }, timeoutS * 2000);
                    }, (error) => {
                        console.error("PlayAPI.ShowControl.triggerEffect error:\n" + error);
                        if (secondaryTimeoutId !== undefined) {
                            clearTimeout(secondaryTimeoutId);
                        }
                        this.showControlResult.removeAll();
                        resolve(ShowEffectResult.FAILURE);
                    });
                }));
            });
        }
    }
    exports.ShowEffectController = ShowEffectController;
});
define("logic/GameController", ["require", "exports", "switchback-common/play/api/MockPlayAPI", "logic/PPAchievementManager", "switchback-common/enum/Park", "model/PeterPanConfig", "logic/sequence-controllers/TinksTrinketsSequenceController", "logic/PPAnalyticsManager", "enum/Queue", "switchback-common/play/AnalyticsManager", "switchback-common/util/TextUtil", "Game", "model/PanSaveState", "switchback-common/play/ShowEffectController"], function (require, exports, MockPlayAPI_1, PPAchievementManager_1, Park_3, PeterPanConfig_1, TinksTrinketsSequenceController_1, PPAnalyticsManager_1, Queue_2, AnalyticsManager_2, TextUtil_1, Game_1, PanSaveState_1, ShowEffectController_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    class GameController {
        constructor() {
            this.players = [];
            this.numTTToFind = 0; // store the TrinkTrinkets for the current session
            this.playerTurnCounter = 0;
            this.exitConfirmed = false;
            this.reachedTrinketRoom = false;
            // A cache to be able to correctly reset the state upon cleanup. Updated by BookSequenceController upon parse
            this.introSetIDs = [];
            // Flag to indicate if it needs to show the prompt to resume the game, this happens when the app opens, 
            // so the flag does not need to be saved and it's always true when the app initializes
            this.shouldPromptToResume = true;
            if (!PeterPanConfig_1.PeterPanConfig.Instance.usePlayAPI) {
                MockPlayAPI_1.MockPlayAPI.init();
            }
            this.achievementManager = new PPAchievementManager_1.PPAchievementManager();
            this.showEffectController = new ShowEffectController_1.ShowEffectController();
            this.tinksTrinketsSequenceController = new TinksTrinketsSequenceController_1.TinksTrinketsSequenceController();
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init(tinksTrinketsJson) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[GameController] init()`);
                this.state = new PanSaveState_1.PanSaveState();
                try {
                    yield PlayAPI.gameInit().then((initResult) => {
                        console.log(JSON.stringify(initResult));
                        if (initResult && initResult.gameStateRead && initResult.gameStateRead.state) {
                            this.state.parse(initResult.gameStateRead.state);
                        }
                        else {
                            this.state.parse({});
                        }
                        // Figure out which park this app got launched from!
                        // Longer form, but safer than 'PeterPanConfig.Instance.park = initResult.playerLocation'
                        if (initResult.playerLocation === Park_3.Park.DLR) {
                            PeterPanConfig_1.PeterPanConfig.Instance.park = Park_3.Park.DLR;
                        }
                        if (initResult.playerLocation === Park_3.Park.WDW) {
                            PeterPanConfig_1.PeterPanConfig.Instance.park = Park_3.Park.WDW;
                        }
                    }, (reason) => {
                        console.log(JSON.stringify(reason));
                        this.state.parse({});
                    });
                }
                catch (error) {
                    console.log(error);
                    this.state.parse({});
                }
                if (PeterPanConfig_1.PeterPanConfig.Instance.park === Park_3.Park.DLR) {
                    this.parkPrefix = 'dlr';
                }
                else if (PeterPanConfig_1.PeterPanConfig.Instance.park === Park_3.Park.WDW) {
                    this.parkPrefix = 'wdw';
                }
                yield this.getPlayers();
                this.achievementManager.initialise(this.players, `peterPansFlight${this.parkPrefix.toUpperCase()}`);
                PPAnalyticsManager_1.PPAnalyticsManager.Instance.init(PeterPanConfig_1.PeterPanConfig.Instance.park, this.parkPrefix.toUpperCase(), this.hasPrimaryPlayer(), this.players.length, this.state.numVisits);
                this.getEstimatedWaitTime().then((waitTime) => {
                    PPAnalyticsManager_1.PPAnalyticsManager.Instance.estimatedWaitTime = waitTime;
                });
                // Try to pull the queue out here
                switch (this.state.selectedQueueType) {
                    case Queue_2.Queue.PRIMARY: {
                        PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_A;
                        break;
                    }
                    case Queue_2.Queue.SECONDARY: {
                        PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_B;
                        break;
                    }
                    default: {
                        PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_NONE;
                    }
                }
                // Every second, increase our counter for time spent playing.
                setInterval(() => {
                    this.state.sessionPlayTime++;
                    PPAnalyticsManager_1.PPAnalyticsManager.Instance.actualWaitTime = this.state.sessionPlayTime;
                }, 1000);
                this.showEffectController.init();
                this.tinksTrinketsSequenceController.init(tinksTrinketsJson);
                this.numTTToFind = this.tinksTrinketsSequenceController.totalTrinketsToFind;
            });
        }
        // Has the player hit the exit beacon?
        get exitTriggered() {
            return this._exitTriggered;
        }
        set exitTriggered(exitTriggered) {
            this._exitTriggered = exitTriggered;
        }
        setSelectedQueueType(queueType) {
            this.state.selectedQueueType = queueType;
            switch (queueType) {
                case Queue_2.Queue.PRIMARY: {
                    PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_A;
                    this.showEffectController.defaultShowEffectName = PeterPanConfig_1.PeterPanConfig.Instance.lanternEffectDLRQueueATag;
                    break;
                }
                case Queue_2.Queue.SECONDARY: {
                    PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_B;
                    this.showEffectController.defaultShowEffectName = PeterPanConfig_1.PeterPanConfig.Instance.lanternEffectDLRQueueBTag;
                    break;
                }
                default: {
                    PPAnalyticsManager_1.PPAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_NONE;
                    this.showEffectController.defaultShowEffectName = "";
                }
            }
        }
        getEstimatedWaitTime() {
            return __awaiter(this, void 0, void 0, function* () {
                const location = yield PlayAPI.locationRead();
                const wait = yield PlayAPI.locationWaitTimeRead(location);
                return wait.waitTime;
            });
        }
        hasPrimaryPlayer() {
            for (let player of this.players) {
                if (player.type === "TYPE_PRIMARY") {
                    return true;
                }
            }
            return false;
        }
        allTrinketsSelected() {
            return (this.numTTToFind === this.state.selectedTrinkets.length);
        }
        /* Creates a NEW state to be parsed.
            In order to clear the state, while preserving specific values.
            Called when the ride ends.
            It has the values that must not be cleared, hence some values persists acros rides.
        */
        recycleState() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[GameController] recycleState()");
                // Is much better to create a new object rather than stripping the old one (has implications over reference sharing)
                let newData = {};
                // store views for intro themed sets
                let newSetViews = {};
                for (let setID of this.introSetIDs) {
                    newSetViews[setID] = this.state.getSetViews(setID);
                }
                newData._setViews = newSetViews;
                // Reset the current players, but keep their names
                newData._playerNames = this.state.playerNames.slice();
                // Setting to false will avoid triggering the confirmation dialog by mistake
                newData._sessionStarted = false;
                // Persist the number of happy thoughts they've made
                newData._numberOfHappyThoughtsCreated = this.state.numberOfHappyThoughtsCreated;
                // and how many times they've visited the ride
                newData._numVisits = this.state.numVisits;
                this.state.parse(newData);
                this.state.save();
            });
        }
        // Loads the players from the playApi
        getPlayers() {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    let result = yield PlayAPI.openPlayerManagement();
                    // if result contains no players, populate the players array with dummy players
                    if (result && result.players && result.players.length > 0) {
                        GameController.Instance.players = result.players;
                    }
                    else {
                        GameController.Instance.players = [
                            { name: "Player 1", type: "", identifier: "" },
                            { name: "Player 2", type: "", identifier: "" }
                        ];
                    }
                    // this relies on the savestate being loaded before calling this function
                    // and also relies on this function being always called upon init
                    // stores the previous player names for comparison
                    if (this.state.sessionStarted) {
                        this.previousSessionPlayerNames = this.state.playerNames.slice();
                    }
                    // saves the current players
                    this.state.playerNames = [];
                    for (let player of GameController.Instance.players) {
                        this.state.playerNames.push(player.name);
                    }
                    this.state.save();
                }
                catch (error) {
                    console.log("OpenPlayerManagement Error " + error);
                    // Initialize with fake data just so the app doesn' break
                    GameController.Instance.players = [
                        { name: "Player 1", type: "", identifier: "" },
                        { name: "Player 2", type: "", identifier: "" }
                    ];
                }
            });
        }
        /**
         * Listens for the eject signal from the PlayAPI.
         */
        createOutroSubscription() {
            if (!this.outroSubscription) {
                this.outroSubscription = () => {
                    GameController.Instance.exitTriggered = true;
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_EJECT, this.outroSubscription);
            }
        }
        getCurrentPlayer() {
            return this.players[this.playerTurnCounter].name;
        }
        // Cycle through the players
        passPlayer() {
            // if all players have had a turn, reset to the first player
            this.playerTurnCounter = (this.playerTurnCounter + 1) % this.players.length;
        }
        calculateDefaultTextSizes(gameWidth, gameHeight) {
            TextUtil_1.TextUtil.calculateFontSizeModifier(gameWidth, gameHeight);
            Game_1.Game.FONT_SIZE_LARGE = TextUtil_1.TextUtil.getFontSize(gameWidth, 45);
            Game_1.Game.FONT_SIZE_MEDIUM = TextUtil_1.TextUtil.getFontSize(gameWidth, 30);
            Game_1.Game.FONT_SIZE_SMALL = TextUtil_1.TextUtil.getFontSize(gameWidth, 20);
        }
    }
    exports.GameController = GameController;
});
define("switchback-common/util/SoundUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SoundUtil {
        /**
         * Add sounds loaded via loadSound or loadSoundAll will be prefixed by this path
         */
        static setRootSoundPath(value) {
            SoundUtil._rootSoundPath = value;
        }
        /**
         * @param tag Phaser SoundManager tag for this sound
         * @param filename Doesn't include the filename
         */
        static loadSound(load, tag, filename) {
            load.audio(tag, [SoundUtil._rootSoundPath + filename + ".m4a",
                SoundUtil._rootSoundPath + filename + ".ogg"]);
        }
        /**
         * Loads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound (will have the index appended)
         * @param filename Doesn't include the filename
         * @param count How many sounds to load
         */
        static loadSoundAll(load, tag, filename, count) {
            for (let i = 0; i < count; ++i) {
                load.audio(tag + i, [SoundUtil._rootSoundPath + filename + i + ".m4a",
                    SoundUtil._rootSoundPath + filename + i + ".ogg"]);
            }
        }
        /**
         * Unloads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound
         * @param count How many sounds to unload
         */
        static unloadSoundAll(cache, tag, count) {
            for (let i = 0; i < count; ++i) {
                cache.removeSound(tag + i);
            }
        }
        /**
         * Play a sound at random from a set of sounds with the same base tag
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param count How many sounds in this set
         */
        static playRandom(game, soundTag, count) {
            return game.sound.play(soundTag + Math.floor(Math.random() * count));
        }
        /**
         * Play a sound on a fixed interval
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param interval How freqently to play the sound
         * @param volume Volume to play the sound
         * @returns An id that is used to stop this loop from playing
         */
        static playSoundOnInterval(game, soundTag, interval, volume) {
            return setInterval(() => {
                game.sound.play(soundTag, volume);
            }, interval);
        }
        /**
         *
         * @param intervalId This id generated by playSoundOnInterval used to stop the loop
         */
        static stopSoundOnInterval(intervalId) {
            clearInterval(intervalId);
        }
        /**
         * Allows to resume the audio context. Specially after a focus loss.
         * This gets called automatically by PauseUtil, which is the preferred method.
         * Accessible here for the special cases where you want this functionality and skipping PauseUtil.
         */
        static resumeAudio(game) {
            // you can't re-enable the sounds until the user has had an input and
            // the sounds aren't already running
            game.input.onDown.addOnce(() => {
                try {
                    if (game.sound.context.state !== "running") {
                        game.sound.context.resume();
                    }
                }
                catch (e) {
                    console.log("[SoundUtil.resumeAudio] Error attempting to resume sounds.");
                }
            }, Number.MAX_SAFE_INTEGER);
        }
    }
    SoundUtil._rootSoundPath = "";
    exports.SoundUtil = SoundUtil;
});
define("switchback-common/util/PauseUtil", ["require", "exports", "switchback-common/util/SoundUtil"], function (require, exports, SoundUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PauseUtil {
        /*
         * Use this function to listen for when the game has resumed changes visibility.
         * On iOS backgrounding the game can kill sound context and you'll
         * permanently lose audio. This function resumes it.
         * Function should be hooked once as the game loads.
         *
         * This needs to be called as the last part on initialization.
         * As in Android the player management card will trigger 2 onBlur.
         */
        static addHooks(game) {
            console.log("[PauseUtil] addHooks");
            PauseUtil.removeHooks(game);
            PauseUtil._isBlurred = false;
            PauseUtil.onBlur = new Phaser.Signal();
            PauseUtil.onFocus = new Phaser.Signal();
            PauseUtil._onBlurListener = PauseUtil.onBlurred.bind(null, game);
            PauseUtil._onFocusListener = PauseUtil.onFocused.bind(null, game);
            // These two are the only ones reliable on ios and android
            // onBlur and onFocus are not triggering on android for some reason
            // also document.addEventListener("visibilitychange") seems to be reliable too
            game.onPause.add(PauseUtil._onBlurListener);
            game.onResume.add(PauseUtil._onFocusListener);
            // Make sure the audio gets activated. Because the engine might have been stopped before this hook gets called.
            // Or in new updates the engine always starts stopped.
            SoundUtil_1.SoundUtil.resumeAudio(game);
        }
        static removeHooks(game) {
            if (PauseUtil.onBlur) {
                PauseUtil.onBlur.dispose();
                PauseUtil.onBlur = null;
            }
            if (PauseUtil.onFocus) {
                PauseUtil.onFocus.dispose();
                PauseUtil.onFocus = null;
            }
            if (PauseUtil._onBlurListener) {
                game.onBlur.remove(this._onBlurListener, null);
            }
            if (PauseUtil._onFocusListener) {
                game.onFocus.remove(PauseUtil._onFocusListener, null);
            }
        }
        static onBlurred(game) {
            console.log("[PauseUtil] onBlur. isBlurred=", PauseUtil._isBlurred);
            if (PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = true;
            PauseUtil.onBlur.dispatch();
        }
        static onFocused(game) {
            console.log("[PauseUtil] onFocus. isBlurred=", PauseUtil._isBlurred);
            if (!PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = false;
            SoundUtil_1.SoundUtil.resumeAudio(game);
            PauseUtil.onFocus.dispatch();
        }
    }
    exports.PauseUtil = PauseUtil;
});
define("state/LoadingState", ["require", "exports", "switchback-common/state/BaseState", "switchback-common/localization/LocalizationManager", "logic/GameController", "model/PeterPanConfig", "switchback-common/util/PauseUtil"], function (require, exports, BaseState_1, LocalizationManager_1, GameController_4, PeterPanConfig_2, PauseUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class LoadingState extends BaseState_1.BaseState {
        init(...params) {
            super.init(params);
            this.stateName = "Load";
            console.log(`[${this.stateName}] init()`);
            this.scale.scaleMode = Phaser.ScaleManager.RESIZE;
            this.game.time.advancedTiming = true;
            this.game.input.maxPointers = 1;
        }
        preload() {
            super.preload();
            console.log(`[${this.stateName}] preload()`);
            this.game.load.json("config", "assets/json/config.json");
            this.game.load.json("i18n", "assets/lang/en.json");
            this.game.load.json('pages', 'assets/json/pages.json');
            this.game.load.json('tinks-trinkets', 'assets/json/tinks-trinkets.json');
            this.game.load.json('build-info', 'assets/json/build-info.json');
            this.game.load.image('cover', 'assets/common/cover.jpg');
            this.game.load.image('room-background', 'assets/common/room_intro.jpg');
            this.game.load.image('page-fake-corner', 'assets/common/fake_corner.png');
            this.game.load.image('page-with-border', 'assets/common/page.jpg');
            this.game.load.image('page-tinks-trinkets', 'assets/common/tt_page.jpg');
            this.game.load.image('page-corner', 'assets/common/corner.png');
            this.game.load.image('page-bend', 'assets/common/page_bend.png');
            this.game.load.spritesheet('lantern-spritesheet', 'assets/lantern/spritesheet.png', 300, 629);
            this.game.load.image('lantern-background', 'assets/lantern/background.png');
            this.game.load.image('lantern-glow-ring', 'assets/lantern/glow_scale.png');
            this.game.load.image('lantern-glow-lines', 'assets/lantern/glow_rotate.png');
            this.game.load.image('lantern-sparkles', 'assets/lantern/sparkles.png');
            this.game.load.image('header-graphic', 'assets/common/header_graphic.png');
            this.game.load.image('header-graphic-gold', 'assets/common/header_graphic_gold.png');
            this.game.load.image('pic_neverland', 'assets/pictures/neverland.png');
            this.game.load.image('drawing-title', 'assets/pictures/drawing-title.png');
            this.game.load.image('happy_thoughts_1', 'assets/pictures/happy_thoughts1.png');
            this.game.load.image('happy_thoughts_2', 'assets/pictures/happy_thoughts2.png');
            this.game.load.image('joke_1', 'assets/pictures/joke_1.png');
            this.game.load.image('joke_2', 'assets/pictures/joke_2.png');
            this.game.load.image('selection_tick', 'assets/pictures/selection_tick.png');
            this.game.load.image('checkbox_background', 'assets/common/checkbox_background.png');
            this.game.load.image('plus', 'assets/common/plus.png');
            this.game.load.image('tinks_trinkets_title', 'assets/pictures/tinks_trinkets_title.png');
            this.game.load.image('tinks_trinkets_outro', 'assets/pictures/tinks_trinkets_outro.png');
            this.game.load.image('tinks_trinkets_bottom_banner', 'assets/common/trinkets_bottom_banner.png');
            this.game.load.image('load_illustration', 'assets/pictures/load_illustration.png');
            this.game.load.image('outro_background', 'assets/pictures/outro_background.png');
            this.game.load.image('button-pagination-arrow', 'assets/common/button/pagination_arrow.png');
            this.game.load.image('button-pattern-large', 'assets/common/button/pattern_largebutton.png');
            this.game.load.image('button-pattern-medium', 'assets/common/button/pattern_mediumbutton.png');
            this.game.load.spritesheet('option-button-large', 'assets/common/button/option_button_large.png', 697, 125);
            this.game.load.spritesheet('option-button-medium', 'assets/common/button/option_button_medium.png', 419, 125);
            this.game.load.image('dialog-bg', 'assets/common/dialog/dialog_bg.png');
            this.game.load.image('button-corner-red', 'assets/common/button/button_corner_red.png');
            this.game.load.image('button-corner-green', 'assets/common/button/button_corner_green.png');
            this.game.load.image('button-corner-shadow', 'assets/common/button/button_cornershadow.png');
            this.game.load.image('button-icon-quit', 'assets/common/button/icon_quit.png');
            this.game.load.image('button-icon-tink', 'assets/common/button/icon_tink.png');
            this.game.load.image('button-icon-arrow', 'assets/common/button/tab_arrow.png');
            this.game.load.image('dialog-button-negative', 'assets/common/button/dialog-button-negative.png');
            this.game.load.image('dialog-button-positive', 'assets/common/button/dialog-button-positive.png');
            this.game.load.image('down-arrow', 'assets/common/button/button_downarrow.png');
            this.game.load.spritesheet('next-button', 'assets/common/button/arrow_button.png', 392, 216);
            this.game.load.atlas('numbers', 'assets/common/button/numbers.png', 'assets/common/button/numbers.json');
            this.game.load.image('arrow-corner', 'assets/common/arrow_corner.png');
            this.game.load.image('dialog-large', 'assets/common/dialog/dialog_large.png');
            this.game.load.spritesheet('button-basic', 'assets/common/button-basic.png', 100, 60, 2);
            this.game.load.atlas('button-medium', 'assets/common/button/button_medium.png', 'assets/common/button/button_medium.json');
            this.game.load.spritesheet('button-large', 'assets/common/button/button_large.png', 949, 171);
            this.game.load.audio('pageturn1', ['assets/sound/pageturn1.ogg', 'assets/sound/pageturn1.m4a']);
            this.game.load.audio('pageturn2', ['assets/sound/pageturn2.ogg', 'assets/sound/pageturn2.m4a']);
            this.game.load.audio('pageturn3', ['assets/sound/pageturn3.ogg', 'assets/sound/pageturn3.m4a']);
            this.game.load.audio('pixie-dust-collect', ['assets/sound/pixie-dust-collect.ogg', 'assets/sound/pixie-dust-collect.m4a']);
            this.game.load.audio('tink-fly-in', ['assets/sound/tink-fly-in.ogg', 'assets/sound/tink-fly-in.m4a']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                console.log(`[${this.stateName}] create()`);
                this.game.input.maxPointers = 1;
                // As soon as we possibly can, we should load up the config file
                PeterPanConfig_2.PeterPanConfig.Instance.parse(this.game.cache.getJSON("config"));
                // And the localization strings
                LocalizationManager_1.LocalizationManager.Instance.init("en", this.game.cache.getJSON("i18n"));
                this.dismissOverlay();
                // Initialise the game controller
                yield GameController_4.GameController.Instance.init(this.cache.getJSON('tinks-trinkets'));
                PauseUtil_1.PauseUtil.addHooks(this.game);
                GameController_4.GameController.Instance.calculateDefaultTextSizes(this.game.width, this.game.height);
                yield this.showOverlay();
                this.game.state.start("BookState");
            });
        }
    }
    exports.LoadingState = LoadingState;
});
define("switchback-common/util/DeviceUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to support device-specific functions.
     * @author nick@cerebralfix.com
     */
    class DeviceUtil {
        /*
        * This function detects whether the device is running a Mali-400 GPU.
        * This is required to apply a phaser render setting that fixes a
        * canvas strobing issue present in these GPUs.
        */
        static isMaliGPU() {
            const canv = document.createElement("canvas");
            canv.setAttribute("width", "1");
            canv.setAttribute("height", "1");
            document.body.appendChild(canv);
            const canvas = document.getElementsByTagName("canvas");
            const gl = canvas[0].getContext("webgl", { stencil: true });
            canvas[0].parentNode.removeChild(canvas[0]);
            if (!gl) {
                return false;
            }
            const dbgRenderInfo = gl.getExtension("WEBGL_debug_renderer_info");
            let renderer;
            if (dbgRenderInfo !== null) {
                renderer = gl.getParameter(dbgRenderInfo.UNMASKED_RENDERER_WEBGL);
            }
            else {
                return false;
            }
            return renderer.search("Mali-400") !== -1;
        }
        /**
         * Checks the userAgent to try to figure out
         * which platform we're running on.
         */
        static isIos() {
            return /iPad|iPhone|iPod/.test(navigator.userAgent);
        }
    }
    exports.DeviceUtil = DeviceUtil;
});
define("pages/ITurnable", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("pages/component/HeaderText", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_2, TextUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HeaderText extends Phaser.Group {
        constructor(game, pageTypeTitle, headerGraphic = 'header-graphic') {
            super(game);
            const dropCapBackground = new Phaser.Image(this.game, 0, 0, headerGraphic);
            dropCapBackground.width = this.game.width * 0.16;
            dropCapBackground.height = this.game.width * 0.16;
            this.add(dropCapBackground);
            const dropCapLetter = new Phaser.Text(this.game, 0, 0, pageTypeTitle.charAt(0), {
                fontSize: TextUtil_2.TextUtil.getFontSize(game.width, 70),
                fill: Game_2.Game.TEXT_COLOR_RED,
                font: Game_2.Game.DEFAULT_FONT,
                stroke: Game_2.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_2.Game.TEXT_STROKE_THICKNESS
            });
            dropCapLetter.setShadow(Game_2.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_2.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_2.Game.TEXT_SHADOW_SOFT);
            dropCapLetter.centerX = dropCapBackground.centerX;
            dropCapLetter.centerY = dropCapBackground.centerY;
            this.add(dropCapLetter);
            const standardText = new Phaser.Text(this.game, 0, 0, pageTypeTitle.slice(1), {
                fontSize: TextUtil_2.TextUtil.getFontSize(game.width, 30),
                fill: Game_2.Game.TEXT_COLOR_RED,
                align: 'left',
                wordWrap: true,
                wordWrapWidth: game.width * 0.43,
                font: Game_2.Game.DEFAULT_FONT
            });
            standardText.left = dropCapBackground.right;
            standardText.top = dropCapBackground.top + dropCapBackground.height * 0.3;
            standardText.lineSpacing = -15;
            this.add(standardText);
        }
    }
    exports.HeaderText = HeaderText;
});
define("switchback-common/util/ScaleUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function scaleToBounds(object, widthBound, heightBound, enlargeToFit = true) {
        if (!enlargeToFit && object.width <= widthBound && object.height <= heightBound) {
            return;
        }
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio > yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToBounds = scaleToBounds;
    // Usage: scaleToSingleBound(object, this.game.width * 0.3); width will be set to bound, and height in proportion
    // Usage: scaleToSingleBound(object, this.game.width * 0.3, false); height will be set to bound, and width in proportion
    function scaleToSingleBound(object, bound, isWidth = true) {
        if (isWidth) {
            object.width = bound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = bound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToSingleBound = scaleToSingleBound;
    // Scales the object such that its smallest dimension covers the specified bounds. Its
    // largest dimension will likely exceed the other bound.
    // Useful for ensuring that a background covers the screen without stretching
    function scaleToCover(object, widthBound, heightBound) {
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio < yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToCover = scaleToCover;
    class ScaleUtil {
        static getVerticalProportionalPosition(pt1, pt2, proportion = 0.5) {
            return pt1 + ((pt2 - pt1) * proportion);
        }
    }
    exports.ScaleUtil = ScaleUtil;
});
define("model/ActivityType", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ActivityType {
    }
    ActivityType.INSTRUCTIONS = "Instructions";
    ActivityType.HAPPY_THOUGHTS = "HappyThoughts";
    ActivityType.POPULAR_OPINION = "PopularOpinion";
    ActivityType.WOULD_YOU_RATHER = "WouldYouRather";
    ActivityType.THIS_OR_THAT = "ThisOrThat";
    ActivityType.COMPLIMENT = "Compliment";
    ActivityType.JOKE = "Joke";
    ActivityType.FRIENDSHIP = "Friendship";
    ActivityType.TWENTY_QUESTIONS = "20Questions";
    ActivityType.TWO_TRUTHS = "TwoTruths";
    ActivityType.CHARADES = "Charades";
    ActivityType.DRAWING = "Drawing";
    ActivityType.DRAWING_PROMPT = "DrawingPrompt";
    exports.ActivityType = ActivityType;
});
define("logic/sequence-controllers/ISequenceController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("pages/TextPicturePageData", ["require", "exports", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil"], function (require, exports, PageData_1, Page_1, ScaleUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextPicturePageData extends PageData_1.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key) {
                return null;
            }
            this.imageKey = data.image_key;
            return this;
        }
        build(game) {
            return [new TextPicturePage(game, this.activityType, this.titleString, this.text, this.isFinalPageOfActivity, this.imageKey)];
        }
    }
    exports.TextPicturePageData = TextPicturePageData;
    class TextPicturePage extends Page_1.Page {
        constructor(game, activityType, titleString, text, isFinalPageOfActivity, pictureTag) {
            super(game, activityType, Page_1.PageType.CORNER, titleString, text, isFinalPageOfActivity);
            this.image = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_1.scaleToBounds(this.image, game.width * 0.9, game.height * 0.6);
            this.image.centerX = game.width * 0.5;
            this.text.top = game.height * 0.15;
            this.image.top = this.text.bottom;
            this.add(this.image);
        }
    }
    exports.TextPicturePage = TextPicturePage;
});
define("pages/JokePageData", ["require", "exports", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "model/LayoutType", "model/ActivityType"], function (require, exports, PageData_2, Page_2, ScaleUtil_2, LocalizationManager_2, LayoutType_2, ActivityType_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class JokePageData extends PageData_2.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key) {
                return null;
            }
            this.imageKey = data.image_key;
            return this;
        }
        build(game) {
            return [new JokePage(game, this.text, this.isFinalPageOfActivity, this.imageKey)];
        }
    }
    exports.JokePageData = JokePageData;
    class JokePage extends Page_2.Page {
        constructor(game, text, isFinalPageOfActivity, pictureTag) {
            super(game, ActivityType_1.ActivityType.JOKE, Page_2.PageType.CORNER, "questions.joke.title", text, isFinalPageOfActivity);
            this.layoutType = LayoutType_2.LayoutType.JOKE_PAGE;
            this.image = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_2.scaleToBounds(this.image, game.width * 0.9, game.height * 0.6);
            this.image.centerX = game.width * 0.5;
            this.image.top = game.height * 0.3;
            this.text.centerY = game.height * 0.23;
            this.add(this.image);
        }
        addText(game, text, textWidthPercent = 0.9, textHeightPercent = 0.185) {
            if (!this.isFinalPageOfActivity) {
                text = LocalizationManager_2.LocalizationManager.Instance.text("questions.joke.joke_prefix") + text;
            }
            super.addText(game, text, textWidthPercent, textHeightPercent);
        }
    }
    exports.JokePage = JokePage;
});
define("pages/PageUtility", ["require", "exports", "switchback-common/util/TextUtil", "Game", "logic/GameController", "switchback-common/localization/LocalizationManager"], function (require, exports, TextUtil_3, Game_3, GameController_5, LocalizationManager_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PageUtility {
        static createPassToPlayerText(game) {
            const passPromptText = new Phaser.Text(game, 0, 0, LocalizationManager_3.LocalizationManager.Instance.textWithReplacements("instructions.pass_to_player", { name: GameController_5.GameController.Instance.getCurrentPlayer() }), {
                fontSize: TextUtil_3.TextUtil.getFontSize(game.width, 40),
                fill: Game_3.Game.TEXT_COLOR_GREEN,
                align: 'center',
                wordWrap: false,
                wordWrapWidth: game.width * 0.85,
                font: Game_3.Game.DEFAULT_FONT,
                stroke: Game_3.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_3.Game.TEXT_STROKE_THICKNESS
            });
            passPromptText.setShadow(Game_3.Game.TEXT_SHADOW_OFFSET_MED_X, Game_3.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_3.Game.TEXT_SHADOW_SOFT, 1);
            TextUtil_3.TextUtil.scaleDownSingleLineTextToWidth(passPromptText, game.width * 0.85);
            passPromptText.centerX = game.width * 0.5;
            GameController_5.GameController.Instance.passPlayer();
            return passPromptText;
        }
        static nextBackground() {
            PageUtility.backgroundIndex++;
            // Alternates between the two page backgrounds
            return PageUtility.backgroundIndex % 2 ? 'happy_thoughts_1' : 'happy_thoughts_2';
        }
    }
    PageUtility.backgroundIndex = 0;
    exports.PageUtility = PageUtility;
});
define("pages/OpenQuestionPageData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "pages/PageUtility", "model/LayoutType"], function (require, exports, Game_4, PageData_3, Page_3, ScaleUtil_3, TextUtil_4, LocalizationManager_4, PageUtility_1, LayoutType_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class OpenQuestionPageData extends PageData_3.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key) {
                return null;
            }
            this.layoutType = LayoutType_3.LayoutType.OPEN_QUESTION;
            this.imageKey = PageUtility_1.PageUtility.nextBackground();
            this.instructionKey = data.instruction_key || "";
            return this;
        }
        build(game) {
            return [new OpenQuestionPage(game, this.activityType, this.titleString, this.text, this.isFinalPageOfActivity, this.imageKey, this.layoutType, this.instructionKey)];
        }
    }
    exports.OpenQuestionPageData = OpenQuestionPageData;
    class OpenQuestionPage extends Page_3.Page {
        constructor(game, activityType, titleString, text, isFinalPageOfActivity, pictureTag, layoutType, instructionKey) {
            super(game, activityType, Page_3.PageType.CORNER, titleString, text, isFinalPageOfActivity);
            this.layoutType = layoutType;
            this.backgroundImage = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_3.scaleToBounds(this.backgroundImage, game.width * 1.1, game.height * 0.8);
            this.backgroundImage.centerX = game.width * 0.5;
            this.backgroundImage.centerY = this.game.height * 0.51;
            // add the background above the page texture but below all other elements on the page
            this.add(this.backgroundImage, false, 2);
            const passPromptText = PageUtility_1.PageUtility.createPassToPlayerText(this.game);
            this.add(passPromptText);
            passPromptText.top = this.backgroundImage.top + this.backgroundImage.height * 0.15;
            let instructionBottom;
            if (instructionKey && instructionKey !== "") {
                const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_4.LocalizationManager.Instance.text(instructionKey), {
                    fontSize: TextUtil_4.TextUtil.getFontSize(game.width, 25),
                    fill: Game_4.Game.TEXT_COLOR,
                    align: 'center',
                    wordWrap: true,
                    wordWrapWidth: this.backgroundImage.width * 0.75,
                    font: Game_4.Game.DEFAULT_FONT,
                    stroke: Game_4.Game.TEXT_STROKE_COLOR,
                    strokeThickness: Game_4.Game.TEXT_STROKE_THICKNESS
                });
                instructionText.centerX = this.game.width * 0.5;
                instructionText.top = passPromptText.bottom;
                this.add(instructionText);
                instructionBottom = instructionText.bottom;
            }
            else {
                // postition under pass prompt if there is no instruction
                instructionBottom = passPromptText.bottom;
            }
            this.text.wordWrapWidth = this.backgroundImage.width * 0.75;
            this.text.centerX = this.game.width * 0.5;
            // Center in the remaining space within the image
            let remainingSpace = this.backgroundImage.bottom * 0.8 - instructionBottom - this.text.height;
            if (remainingSpace < 0) {
                remainingSpace = 0;
            }
            this.text.top = instructionBottom + remainingSpace * 0.5;
            this.add(this.text);
        }
        addText(game, text, wordWrapWidthPercent = 0.85) {
            super.addText(game, text, 0.8);
        }
    }
    exports.OpenQuestionPage = OpenQuestionPage;
});
define("pages/component/LargeOptionButton", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_5, ScaleUtil_4, TextUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LargeOptionButton extends Phaser.Group {
        constructor(game, text) {
            super(game);
            this.onSelectedSignal = new Phaser.Signal();
            this.ticks = [];
            const backgroundImage = new Phaser.Image(this.game, 0, 0, "button-pattern-large");
            ScaleUtil_4.scaleToBounds(backgroundImage, this.game.width * 0.83, this.game.height * 0.27);
            backgroundImage.inputEnabled = true;
            this.add(backgroundImage);
            this.buttonImage = new Phaser.Image(this.game, 0, 0, "option-button-large", 0);
            ScaleUtil_4.scaleToBounds(this.buttonImage, this.game.width * 0.9, this.game.height * 0.1);
            this.buttonImage.centerX = backgroundImage.centerX;
            this.buttonImage.top = backgroundImage.bottom - backgroundImage.height * 0.05;
            this.buttonImage.inputEnabled = true;
            this.add(this.buttonImage);
            const questionText = new Phaser.Text(this.game, 0, 0, text, {
                fontSize: TextUtil_5.TextUtil.getFontSize(game.width, 40),
                fill: Game_5.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.78,
                font: Game_5.Game.DEFAULT_FONT,
                stroke: Game_5.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_5.Game.TEXT_STROKE_THICKNESS
            });
            questionText.setShadow(Game_5.Game.TEXT_SHADOW_OFFSET_MED_X, Game_5.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_5.Game.TEXT_SHADOW_SOFT, 1);
            TextUtil_5.TextUtil.scaleDownTextToHeight(questionText, backgroundImage.height * 0.88);
            questionText.centerX = backgroundImage.centerX;
            questionText.centerY = backgroundImage.centerY + backgroundImage.height * 0.044;
            this.add(questionText);
            this.tickGroup = new Phaser.Group(this.game);
            this.tickGroup.centerX = backgroundImage.centerX;
            this.tickGroup.centerY = this.buttonImage.centerY;
            this.add(this.tickGroup);
            this.onChildInputUp.add(() => {
                this.game.sound.play("pp-sfx-ui-click");
                this.onSelectedSignal.dispatch(this);
            }, this);
        }
        get tickCount() {
            return this.ticks.length;
        }
        addTick() {
            let tick = new Phaser.Image(this.game, 0, 0, "selection_tick", 0);
            let spacing = 0.025 * this.game.width;
            ScaleUtil_4.scaleToBounds(tick, this.width * 0.1, this.height * 0.4);
            // space the ticks evenly without overlapping and increase their size
            // scaleToBounds(tick, this.buttonImage.width, this.buttonImage.height * 0.60);
            // const spacing = tick.width;
            tick.x = spacing * (this.ticks.length + 1);
            tick.centerY = 0;
            this.tickGroup.add(tick);
            this.ticks.push(tick);
            // Re-center
            this.tickGroup.centerX = this.buttonImage.centerX;
        }
        removeTick() {
            if (this.ticks.length < 1) {
                return;
            }
            this.ticks.pop().destroy();
            // Re-center
            this.tickGroup.centerX = this.buttonImage.centerX;
        }
    }
    exports.LargeOptionButton = LargeOptionButton;
});
define("pages/PickOneOfTwoPageData", ["require", "exports", "Game", "logic/GameController", "pages/PageData", "pages/Page", "pages/component/LargeOptionButton", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "model/LayoutType"], function (require, exports, Game_6, GameController_6, PageData_4, Page_4, LargeOptionButton_1, TextUtil_6, LocalizationManager_5, LayoutType_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PickOneOfTwoPageData extends PageData_4.PageData {
        parse(data) {
            super.parse(data);
            if (!data.title_key) {
                return null;
            }
            this.optionOneText = LocalizationManager_5.LocalizationManager.Instance.text(data.answer_1_key);
            this.optionTwoText = LocalizationManager_5.LocalizationManager.Instance.text(data.answer_2_key);
            return this;
        }
        build(game) {
            return [new PickOneOfTwoPage(game, this.activityType, this.titleString, this.text, this.isFinalPageOfActivity, this.optionOneText, this.optionTwoText)];
        }
    }
    exports.PickOneOfTwoPageData = PickOneOfTwoPageData;
    class PickOneOfTwoPage extends Page_4.Page {
        constructor(game, activityType, titleString, text, isFinalPageOfActivity, optionOneText, optionTwoText, picture = null) {
            super(game, activityType, Page_4.PageType.CORNER, titleString, text, isFinalPageOfActivity);
            this.options = [];
            this.selected = -1;
            this.layoutType = LayoutType_4.LayoutType.PICK_ONE_OF_TWO;
            this.titleText.top = game.height * 0.02;
            const optionOneButton = new LargeOptionButton_1.LargeOptionButton(game, optionOneText);
            optionOneButton.centerX = game.width * 0.5;
            optionOneButton.top = this.game.height * 0.2;
            this.add(optionOneButton);
            this.options.push(optionOneButton);
            optionOneButton.onSelectedSignal.add(() => { this.setTicks(optionOneButton); });
            const orText = new Phaser.Text(game, 0, 0, LocalizationManager_5.LocalizationManager.Instance.text("instructions.or"), {
                fontSize: TextUtil_6.TextUtil.getFontSize(game.width, 40),
                fill: Game_6.Game.TEXT_COLOR_GREEN,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.85,
                font: Game_6.Game.DEFAULT_FONT,
                stroke: Game_6.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_6.Game.TEXT_STROKE_THICKNESS
            });
            orText.setShadow(Game_6.Game.TEXT_SHADOW_OFFSET_MED_X, Game_6.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_6.Game.TEXT_SHADOW_SOFT, 1);
            orText.centerX = game.width * 0.5;
            orText.top = optionOneButton.bottom + this.game.height * 0.01;
            this.add(orText);
            const optionTwoButton = new LargeOptionButton_1.LargeOptionButton(game, optionTwoText);
            optionTwoButton.centerX = game.width * 0.5;
            optionTwoButton.top = orText.bottom + this.game.height * 0.01;
            this.add(optionTwoButton);
            this.options.push(optionTwoButton);
            optionTwoButton.onSelectedSignal.add(() => { this.setTicks(optionTwoButton); });
            const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_5.LocalizationManager.Instance.text('instructions.everybody_add_your_vote'), {
                fontSize: TextUtil_6.TextUtil.getFontSize(game.width, 25),
                fill: Game_6.Game.TEXT_COLOR_RED,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.6,
                font: Game_6.Game.DEFAULT_FONT
            });
            instructionText.centerX = game.width * 0.5;
            let remainingSpace = game.height * 0.9 - optionTwoButton.bottom;
            instructionText.top = optionTwoButton.bottom + remainingSpace * 0.4;
            this.add(instructionText);
        }
        setTicks(option) {
            let total = this.options.reduce((total, current) => { return total + current.tickCount; }, 0);
            if (total < GameController_6.GameController.Instance.players.length) {
                option.addTick();
            }
            else {
                option.removeTick();
            }
            if (option.tickCount === GameController_6.GameController.Instance.players.length) {
                GameController_6.GameController.Instance.achievementManager.onUnanimousDecision();
            }
        }
    }
    exports.PickOneOfTwoPage = PickOneOfTwoPage;
});
define("pages/CounterPageData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "model/LayoutType", "pages/PageUtility"], function (require, exports, Game_7, PageData_5, Page_5, ScaleUtil_5, TextUtil_7, LocalizationManager_6, LayoutType_5, PageUtility_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CounterPageData extends PageData_5.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key) {
                return null;
            }
            this.count = data.counter_number;
            return this;
        }
        build(game) {
            return [
                new CounterInstructionPage(game, this.activityType),
                new CounterPage(game, this.activityType, this.text, this.isFinalPageOfActivity, this.count)
            ];
        }
    }
    exports.CounterPageData = CounterPageData;
    class CounterInstructionPage extends Page_5.Page {
        constructor(game, activityType) {
            super(game, activityType, Page_5.PageType.CORNER, "questions.21_questions.title", "questions.21_questions.how_to_play", false);
            const passPromptText = PageUtility_2.PageUtility.createPassToPlayerText(this.game);
            this.add(passPromptText);
            passPromptText.centerY = game.height * 0.19;
            this.text.top = passPromptText.bottom + game.height * 0.01;
        }
        addText(game, text) {
            super.addText(game, text, 0.85, 0.6);
        }
    }
    exports.CounterInstructionPage = CounterInstructionPage;
    class CounterPage extends Page_5.Page {
        constructor(game, activityType, text, isFinalActivityPage, count) {
            super(game, activityType, Page_5.PageType.CORNER, "questions.21_questions.title", text, isFinalActivityPage);
            this.layoutType = LayoutType_5.LayoutType.COUNTER;
            // Main question text
            this.text.top = game.height * 0.17;
            const counterBg = new Phaser.Image(game, 0, 0, "button-pattern-medium");
            ScaleUtil_5.scaleToBounds(counterBg, game.width * 0.6, game.height * 0.4);
            counterBg.centerX = game.width * 0.5;
            counterBg.centerY = game.height * 0.5;
            this.add(counterBg);
            const counterNumber = new Phaser.Image(game, 0, 0, "numbers", count - 1);
            ScaleUtil_5.scaleToBounds(counterNumber, counterBg.width * 0.7, counterBg.height * 0.8);
            counterNumber.centerX = counterBg.centerX;
            counterNumber.top = counterBg.top + counterBg.height * 0.05;
            this.add(counterNumber);
            // "Questions" text underneath count
            const questionsText = new Phaser.Text(game, 0, 0, LocalizationManager_6.LocalizationManager.Instance.textWithCount("questions.21_questions.questions", count), {
                fontSize: TextUtil_7.TextUtil.getFontSize(game.width, 30),
                fill: Game_7.Game.TEXT_COLOR_GREEN,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: counterBg.width * 0.9,
                font: Game_7.Game.DEFAULT_FONT,
                stroke: Game_7.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_7.Game.TEXT_STROKE_THICKNESS
            });
            questionsText.setShadow(Game_7.Game.TEXT_SHADOW_OFFSET_MED_X, Game_7.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_7.Game.TEXT_SHADOW_SOFT, 1);
            questionsText.centerX = game.width * 0.5;
            questionsText.top = counterNumber.bottom;
            this.add(questionsText);
            const counterButton = new Phaser.Image(game, 0, 0, "option-button-medium", 0);
            ScaleUtil_5.scaleToBounds(counterButton, game.width * 0.65, game.height * 0.2);
            counterButton.centerX = this.game.width * 0.5;
            counterButton.top = counterBg.bottom - counterBg.height * 0.05;
            this.add(counterButton);
            const counterArrow = new Phaser.Image(game, 0, 0, "down-arrow");
            ScaleUtil_5.scaleToBounds(counterArrow, counterButton.width * 0.25, counterButton.height * 0.9);
            counterArrow.centerX = counterButton.centerX;
            counterArrow.centerY = counterButton.centerY;
            this.add(counterArrow);
            const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_6.LocalizationManager.Instance.text("questions.21_questions.twenty_questions_to_guess"), {
                fontSize: TextUtil_7.TextUtil.getFontSize(game.width, 25),
                fill: Game_7.Game.TEXT_COLOR_RED,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.8,
                font: Game_7.Game.DEFAULT_FONT
            });
            instructionText.centerX = game.width * 0.5;
            instructionText.centerY = game.height * 0.85;
            this.add(instructionText);
            let tapCallback = () => {
                this.game.sound.play("pp-sfx-ui-click");
                count = Math.max(count - 1, 1);
                questionsText.text = LocalizationManager_6.LocalizationManager.Instance.textWithCount("questions.21_questions.questions", count);
                questionsText.centerX = game.width * 0.5;
                // Frames are 0-indexed, so need to use count - 1
                counterNumber.frame = Math.max(count - 1, 0);
                counterNumber.centerX = game.width * 0.5;
            };
            // The actual logic for this class - hitting the button decreases the counter, and updates text fields accordingly.
            counterButton.inputEnabled = true;
            counterButton.events.onInputDown.add(tapCallback, this);
            counterBg.inputEnabled = true;
            counterBg.events.onInputDown.add(tapCallback, this);
        }
        addText(game, text, textWidthPercent = 0.9, textHeightPercent = 0.14) {
            super.addText(game, text, textWidthPercent, textHeightPercent);
        }
    }
    exports.CounterPage = CounterPage;
});
define("pages/component/OptionButton", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_8, ScaleUtil_6, TextUtil_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class OptionButton extends Phaser.Group {
        constructor(game, text) {
            super(game);
            this.onSelectedSignal = new Phaser.Signal();
            // List of ticks, for easy removal
            this.ticks = [];
            this.image = new Phaser.Image(game, 0, 0, "option-button-large", 0);
            ScaleUtil_6.scaleToBounds(this.image, game.width * 0.875, game.height * 0.2);
            this.image.anchor.set(0.5);
            this.image.inputEnabled = true;
            this.image.events.onInputDown.add(() => {
                this.game.sound.play("pp-sfx-ui-click");
                this.onSelectedSignal.dispatch(this);
            }, this);
            this.add(this.image);
            this.tickGroup = new Phaser.Group(game);
            this.add(this.tickGroup);
            const textField = new Phaser.Text(game, this.image.centerX, this.image.centerY, text.toUpperCase(), {
                fontSize: TextUtil_8.TextUtil.getFontSize(game.width, 26),
                fill: Game_8.Game.TEXT_COLOR_YELLOW,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.71,
                font: Game_8.Game.DEFAULT_FONT
            });
            // Anchor to the right, so the ticks can be added easily.
            textField.anchor.set(1, 0.5);
            TextUtil_8.TextUtil.scaleDownTextToHeight(textField, this.image.height * 0.85);
            // Adds textfield to tickGroup so it can be centered
            this.tickGroup.add(textField);
            this.tickGroup.centerX = this.image.centerX;
        }
        set selected(selected) {
            this.image.frame = selected ? 1 : 0;
        }
        get tickCount() {
            return this.ticks.length;
        }
        // Sets how many ticks should a button show.
        // It's an absolute value because its easier to manage
        setTicks(count) {
            // Clamps to 0, this allow to remove all the ticks, but avoid an infinite loop on the 
            // second loop
            count = Math.max(count, 0);
            // tries to add ticks if that's the case
            // notice the while counts as an if
            while (this.ticks.length < count) {
                this.addTick();
            }
            // or tries to remove them
            while (this.ticks.length > count) {
                this.removeTick();
            }
        }
        addTick() {
            let tick = new Phaser.Image(this.game, 0, 0, "selection_tick", 0);
            let spacing = 0.025 * this.game.width;
            ScaleUtil_6.scaleToBounds(tick, this.width * 0.1, this.height * 0.4);
            tick.x = -0.02 * this.game.width + spacing * (this.ticks.length + 1);
            tick.centerY = 0;
            this.tickGroup.add(tick);
            this.ticks.push(tick);
            // Re-center
            this.tickGroup.centerX = this.image.centerX;
        }
        removeTick() {
            if (this.ticks.length < 1) {
                return;
            }
            this.ticks.pop().destroy(); // technically should remove itself from the this.tickGroup
            // Re-center
            this.tickGroup.centerX = this.image.centerX;
        }
        destroy() {
            this.image.events.onInputDown.removeAll();
            this.onSelectedSignal.removeAll();
            super.destroy();
        }
    }
    exports.OptionButton = OptionButton;
});
define("pages/component/OptionResult", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_9, ScaleUtil_7, TextUtil_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class OptionResult extends Phaser.Group {
        constructor(game, text, result) {
            super(game);
            this.image = new Phaser.Image(game, 0, 0, "option-button-large", 0);
            ScaleUtil_7.scaleToBounds(this.image, game.width * 0.875, game.height * 0.2);
            this.add(this.image);
            this.resultField = new Phaser.Text(game, 0, 0, result + ' -', {
                fontSize: TextUtil_9.TextUtil.getFontSize(game.width, 25),
                fill: Game_9.Game.TEXT_COLOR_WHITE,
                align: 'right',
                wordWrap: false,
                font: Game_9.Game.DEFAULT_FONT
            });
            this.resultField.left = this.image.left + this.image.width * 0.09;
            this.resultField.centerY = this.image.centerY;
            this.add(this.resultField);
            this.textField = new Phaser.Text(game, 0, 0, text.toUpperCase(), {
                fontSize: TextUtil_9.TextUtil.getFontSize(game.width, 25),
                fill: Game_9.Game.TEXT_COLOR_WHITE,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.66,
                font: Game_9.Game.DEFAULT_FONT
            });
            TextUtil_9.TextUtil.scaleDownTextToHeight(this.textField, this.image.height * 0.85);
            const remainingSpace = this.image.width - this.resultField.right;
            this.textField.centerX = this.resultField.right + remainingSpace * 0.5;
            this.textField.centerY = this.image.centerY;
            this.add(this.textField);
        }
        set selected(selected) {
            this.image.frame = selected ? 2 : 1;
            this.resultField.fill = selected ? Game_9.Game.TEXT_COLOR_YELLOW : Game_9.Game.TEXT_COLOR_WHITE;
            this.textField.fill = selected ? Game_9.Game.TEXT_COLOR_YELLOW : Game_9.Game.TEXT_COLOR_WHITE;
        }
    }
    exports.OptionResult = OptionResult;
});
define("pages/MultiPageOptionData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "pages/component/OptionButton", "pages/component/OptionResult", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "logic/GameController", "model/LayoutType"], function (require, exports, Game_10, PageData_6, Page_6, OptionButton_1, OptionResult_1, TextUtil_10, LocalizationManager_7, GameController_7, LayoutType_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MultiSelectPageResult extends Page_6.PageResult {
        constructor() {
            super(...arguments);
            // Stores how many votes each options has
            this.votesForOption = [];
        }
    }
    exports.MultiSelectPageResult = MultiSelectPageResult;
    // Each of the options in the tallie 
    class Option {
        parse(data) {
            if (!data.key) {
                return null;
            }
            this.key = LocalizationManager_7.LocalizationManager.Instance.text(data.key);
            this.result = data.result;
            return this;
        }
    }
    class MultiPageOptionData extends PageData_6.PageData {
        constructor() {
            super(...arguments);
            this.options = [];
        }
        parse(data) {
            super.parse(data);
            // Set this as the final activity page. 
            // This value only gets passed through to the MultiPageOptionResultPage constructor
            // The MultiPageOptionAskPage is always set to false, as we don't want any page other than
            // the final result page to increment the activity count 
            this.isFinalPageOfActivity = true;
            data.options.forEach((option) => {
                const parsedOption = new Option().parse(option);
                if (parsedOption) {
                    this.options.push(parsedOption);
                }
            }, this);
            return this;
        }
        build(game) {
            const pages = [];
            // N.B. This page awards no pixie dust when viewed,
            // and always passes through a isFinalPageOfActivity value of false
            const askPage = new MultiPageOptionAskPage(game, this.activityType, this.text, 0, false, this.options);
            pages.push(askPage);
            const page2TitleTextString = "questions.quiz.others_in_the_queue";
            const resultPage = new MultiPageOptionResultPage(game, this.activityType, page2TitleTextString, this.isFinalPageOfActivity, this.options);
            askPage.pageResultSignal.add((result) => {
                resultPage.setResult(result);
            });
            pages.push(resultPage);
            return pages;
        }
    }
    exports.MultiPageOptionData = MultiPageOptionData;
    // This is the page that shows the options and allow the players to vote
    class MultiPageOptionAskPage extends Page_6.Page {
        constructor(game, activityType, text, pixieDustAmount, isFinalPageOfActivity, options) {
            super(game, activityType, Page_6.PageType.CORNER, "questions.quiz.title", text, isFinalPageOfActivity);
            this.optionButtons = [];
            // initialize the resultset
            this.result = new MultiSelectPageResult();
            this.result.votesForOption = [];
            let yPos = game.height * 0.35;
            const yInc = game.height * 0.1;
            options.forEach((option) => {
                const optionButton = new OptionButton_1.OptionButton(game, option.key);
                optionButton.centerX = game.width * 0.5;
                optionButton.centerY = yPos;
                yPos += yInc;
                optionButton.onSelectedSignal.add(this.onOptionSelected, this);
                this.add(optionButton);
                this.optionButtons.push(optionButton);
                this.result.votesForOption.push(0); // each option starts with 0 votes
            }, this);
            this.text.bottom = this.optionButtons[0].top;
            const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_7.LocalizationManager.Instance.text('instructions.everybody_add_your_vote'), {
                fontSize: TextUtil_10.TextUtil.getFontSize(game.width, 25),
                fill: Game_10.Game.TEXT_COLOR_RED,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.6,
                font: Game_10.Game.DEFAULT_FONT
            });
            instructionText.centerX = game.width * 0.5;
            instructionText.top = game.height * 0.82;
            const lastButton = this.optionButtons[this.optionButtons.length - 1];
            let remainingSpace = game.height * 0.9 - lastButton.bottom;
            instructionText.top = lastButton.bottom + remainingSpace * 0.4;
            this.add(instructionText);
        }
        addText(game, text, textWidthPercent = 0.85, textHeightPercent = 0.15) {
            super.addText(game, text, textWidthPercent, textHeightPercent);
        }
        // gets called when a button has been pressed
        onOptionSelected(optionButton) {
            let btnIndex = this.optionButtons.indexOf(optionButton);
            // If there's an error, quits
            if (btnIndex < 0) {
                return;
            }
            // Count total votes
            let totalVotes = 0;
            for (let votes of this.result.votesForOption) {
                totalVotes += votes;
            }
            // calculate new value
            let newVotes = this.result.votesForOption[btnIndex];
            if (totalVotes < GameController_7.GameController.Instance.players.length) {
                // Add only if possible
                newVotes++;
            }
            else {
                // Otherwise, try to remove one
                if (newVotes > 0) {
                    newVotes--;
                }
            }
            // Sets new value
            this.result.votesForOption[btnIndex] = newVotes;
            optionButton.setTicks(newVotes);
            if (optionButton.tickCount === GameController_7.GameController.Instance.players.length) {
                GameController_7.GameController.Instance.achievementManager.onUnanimousDecision();
            }
            // notice this gets signaled everytime a button is pressed, 
            // so the listener should always accept multiple calls discarding previous values
            this.pageResultSignal.dispatch(this.result);
        }
    }
    exports.MultiPageOptionAskPage = MultiPageOptionAskPage;
    class MultiPageOptionResultPage extends Page_6.Page {
        constructor(game, activityType, text, isFinalPageOfActivity, optionData) {
            super(game, activityType, Page_6.PageType.CORNER, "questions.quiz.title", text, isFinalPageOfActivity);
            this.options = [];
            this.result = new MultiSelectPageResult();
            this.layoutType = LayoutType_6.LayoutType.MULTI_OPTIONS;
            let yPos = game.height * 0.35;
            const yInc = game.height * 0.1;
            optionData.forEach((option) => {
                const optionButton = new OptionResult_1.OptionResult(game, option.key, option.result);
                optionButton.centerX = game.width * 0.5;
                optionButton.centerY = yPos;
                yPos += yInc;
                this.add(optionButton);
                this.options.push(optionButton);
            }, this);
            this.text.bottom = this.options[0].top;
        }
        addText(game, text, textWidthPercent = 0.85, textHeightPercent = 0.15, fontSize = 25) {
            super.addText(game, text, textWidthPercent, textHeightPercent, fontSize);
        }
        // Sets the result for this page
        setResult(result) {
            this.result = result;
            this.options.forEach((option, index) => {
                // Marks the option as selected (notice that option.selected is a property and has no "get")
                option.selected = result.votesForOption[index] > 0;
            }, this);
        }
    }
    exports.MultiPageOptionResultPage = MultiPageOptionResultPage;
});
define("pages/IntroPageData", ["require", "exports", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil"], function (require, exports, PageData_7, Page_7, ScaleUtil_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroPageData extends PageData_7.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key) {
                return null;
            }
            return this;
        }
        build(game) {
            return [new IntroPage(game, this.text, this.isFinalPageOfActivity)];
        }
    }
    exports.IntroPageData = IntroPageData;
    class IntroPage extends Page_7.Page {
        constructor(game, text, isFinalPageOfActivity) {
            super(game, '', Page_7.PageType.CORNER, null, text, isFinalPageOfActivity);
            this.text.centerX = this.game.width * 0.5;
            this.text.centerY = this.game.height * 0.325;
            this.image = new Phaser.Image(game, 0, 0, "pic_neverland");
            ScaleUtil_8.scaleToBounds(this.image, game.width * 0.7, game.height * 0.6);
            this.image.centerX = game.width * 0.5;
            this.image.top = this.text.bottom + this.game.height * 0.075;
            this.add(this.image);
        }
        addText(game, text, textWidthPercent = 0.85, textHeightPercent = 0.35, fontSize = 30) {
            super.addText(game, text, textWidthPercent, textHeightPercent, fontSize);
        }
    }
    exports.IntroPage = IntroPage;
});
define("model/BookData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /* Basic structure
    - Page: is one of the shown pages. Smallest unit.
    - Sequence (Activity): Page[] A sorted list of pages, usually list of 1, but some (like joke) have 2 or more.
        This is the smallest unit of interest for exchanging. getNextSequence returns 1 Activity.
        An activity is not interrupted by TT.
    - Set: Activity[] A Sorted list of Activities, always more than 1. Will be interrupted by TT.
        Should be completed as a whole. That means, the game should remember which Set is currently showing
        and return the next one until the set is completed. Also, upon loading, it should recover which set, and
        which activity IN the set was last viewed to continue from that.
        Once a set is done, it will THEN change to another one.
        Possibly in the future, TT will only interrupt AFTER the set (which is not implemented, but then i wold recomend revisit the
        use of sequences).
    - Theme: Set[]. Collection of sets, though is more easy to think of as a property of a set.
        Upon showing two consecutives sets returned should have different themes.
    - Book: Theme[].
    */
    // Extra data types.
    /// JSON
    // One Activity /Sequence
    // Currently the smaller unit in the json is a sequence
    class JSONSequenceData {
    }
    exports.JSONSequenceData = JSONSequenceData;
});
define("pages/CharadesPageData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "model/LayoutType", "pages/PageUtility", "model/ActivityType"], function (require, exports, Game_11, PageData_8, Page_8, ScaleUtil_9, TextUtil_11, LocalizationManager_8, LayoutType_7, PageUtility_3, ActivityType_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CharadesPageData extends PageData_8.PageData {
        parse(data) {
            super.parse(data);
            this.imageKey = PageUtility_3.PageUtility.nextBackground();
            return this;
        }
        build(game) {
            if (!this.isFinalPageOfActivity) {
                return [new CharadesIntroPage(game, this.activityType, this.text, this.isFinalPageOfActivity, this.imageKey)];
            }
            else {
                return [new CharadesCluePage(game, this.activityType, this.text, this.isFinalPageOfActivity, this.imageKey)];
            }
        }
    }
    exports.CharadesPageData = CharadesPageData;
    class CharadesIntroPage extends Page_8.Page {
        constructor(game, activityType, text, isFinalPageOfActivity, pictureTag) {
            super(game, activityType, Page_8.PageType.CORNER, "questions.charades.title", text, isFinalPageOfActivity);
            this.layoutType = LayoutType_7.LayoutType.CHARADES;
            this.activityType = ActivityType_2.ActivityType.CHARADES;
            const image = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_9.scaleToBounds(image, game.width * 1.1, game.height * 0.8);
            image.centerX = game.width * 0.5;
            image.centerY = this.game.height * 0.5;
            this.add(image);
            const passPromptText = PageUtility_3.PageUtility.createPassToPlayerText(this.game);
            this.add(passPromptText);
            passPromptText.top = image.top + image.height * 0.15;
            const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_8.LocalizationManager.Instance.text('questions.charades.instruction_secret'), {
                fontSize: TextUtil_11.TextUtil.getFontSize(game.width, 30),
                fill: Game_11.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.85,
                font: Game_11.Game.DEFAULT_FONT,
                stroke: Game_11.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS
            });
            const remainingSpace = image.bottom * 0.8 - passPromptText.bottom;
            instructionText.centerX = this.game.width * 0.5;
            instructionText.centerY = passPromptText.bottom + remainingSpace * 0.5;
            this.add(instructionText);
        }
    }
    exports.CharadesIntroPage = CharadesIntroPage;
    class CharadesCluePage extends Page_8.Page {
        constructor(game, activityType, text, isFinalPageOfActivity, pictureTag) {
            super(game, activityType, Page_8.PageType.CORNER, "questions.charades.title", text, isFinalPageOfActivity);
            this.layoutType = LayoutType_7.LayoutType.CHARADES;
            this.activityType = ActivityType_2.ActivityType.CHARADES;
            this.text.centerY = game.height * 0.4;
            const image = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_9.scaleToBounds(image, game.width * 1.1, game.height);
            image.centerX = game.width * 0.5;
            image.centerY = this.game.height * 0.5;
            this.add(image);
            const cluePromptText = new Phaser.Text(game, 0, 0, LocalizationManager_8.LocalizationManager.Instance.text('questions.charades.clue_prompt'), {
                fontSize: TextUtil_11.TextUtil.getFontSize(game.width, 25),
                fill: Game_11.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.85,
                font: Game_11.Game.DEFAULT_FONT,
                stroke: Game_11.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS
            });
            cluePromptText.centerX = this.game.width * 0.5;
            cluePromptText.bottom = this.text.top;
            this.add(cluePromptText);
            const instructionText = new Phaser.Text(game, 0, 0, LocalizationManager_8.LocalizationManager.Instance.text('questions.charades.instruction_everyone_answer'), {
                fontSize: TextUtil_11.TextUtil.getFontSize(game.width, 25),
                fill: Game_11.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.8,
                font: Game_11.Game.DEFAULT_FONT,
                stroke: Game_11.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS
            });
            instructionText.centerX = this.game.width * 0.5;
            instructionText.bottom = image.bottom - image.height * 0.25;
            this.add(instructionText);
            // Put the text on top of the border image
            this.text.parent.bringToTop(this.text);
        }
        addText(game, text) {
            // use Green text for Clue
            this.text = new Phaser.Text(game, 0, 0, text, {
                fontSize: TextUtil_11.TextUtil.getFontSize(game.width, 40),
                fill: Game_11.Game.TEXT_COLOR_GREEN,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.85,
                font: Game_11.Game.DEFAULT_FONT,
                stroke: Game_11.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS
            });
            this.text.setShadow(Game_11.Game.TEXT_SHADOW_OFFSET_MED_X, Game_11.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_11.Game.TEXT_SHADOW_SOFT, 1);
            this.text.centerX = this.game.width * 0.5;
            // Center in the remaining space within the image
            this.text.centerY = this.game.height * 0.5;
            this.add(this.text);
        }
    }
    exports.CharadesCluePage = CharadesCluePage;
});
define("logic/sequence-controllers/BookSequenceController", ["require", "exports", "pages/TextPicturePageData", "pages/JokePageData", "pages/OpenQuestionPageData", "pages/PickOneOfTwoPageData", "pages/CounterPageData", "pages/MultiPageOptionData", "pages/IntroPageData", "logic/GameController", "model/LayoutType", "pages/CharadesPageData", "logic/PPAnalyticsManager"], function (require, exports, TextPicturePageData_1, JokePageData_1, OpenQuestionPageData_1, PickOneOfTwoPageData_1, CounterPageData_1, MultiPageOptionData_1, IntroPageData_1, GameController_8, LayoutType_8, CharadesPageData_1, PPAnalyticsManager_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * The most important sequence controller!
     *
     * This guy parses a bunch of page JSON data into specific
     * types of PageData.
     *
     * It also returns the next sequence (set of pages) to show.
     *
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class BookSequenceController {
        constructor(bookData) {
            this.onSetFinished = new Phaser.Signal();
            this.parseBook(bookData);
        }
        /*
         Returns an array of randomized BUT prioritized set ids
         Handles saved states, viewed times, intro or not intro.
        */
        getPrioSetIds() {
            // Store the ids in different lists so they can be shuffled and mixed separatedly (hence priority)
            // Set-IDs for Intro themed sets
            let introThemes = [];
            // Set-IDs for the current theme
            let currentThemes = [];
            // Set-IDs for the rest
            let commonSetIds = [];
            const currentTheme = this.getCurrentTheme();
            // Make use of the book map to traverse the themes
            for (let themeName of this.book.keys()) { // notice of
                // get a theme
                let theme = this.book.get(themeName);
                // SetIds for a theme
                let setIds = [];
                for (let setId of theme.keys()) { // notice of
                    setIds.push(setId);
                }
                // is important that the IF-block has this format
                // due to having many intro-themed sets, and how the flag behaves
                if (themeName === BookSequenceController.THEME_INTRO) { // If its an intro
                    introThemes = setIds;
                }
                else {
                    if (themeName === currentTheme) { // if it's the same theme as the current one
                        currentThemes = setIds;
                    }
                    else {
                        // append to the common pool of sets
                        commonSetIds = commonSetIds.concat(setIds);
                    }
                }
            }
            // Helper function to sort an array of setIds by the viewing time (ascending)
            let sortSetByViews = (a, b) => {
                return GameController_8.GameController.Instance.state.getSetViews(a) - GameController_8.GameController.Instance.state.getSetViews(b);
            };
            // By shuffling separatedly, then joining in order, then sorting, we get a list that
            // has in order, the least viewed items first, then different theme, but also keeping the randomness
            Phaser.ArrayUtils.shuffle(commonSetIds);
            Phaser.ArrayUtils.shuffle(currentThemes);
            // Extend the list of common with the currentTheme at the _end_
            // Adding it at the end will make them usable only if necessary, in case the commons have more views than the current theme.
            commonSetIds = commonSetIds.concat(currentThemes);
            // Sort does not work as supposed so we need to do it before inserting the intro theme
            commonSetIds.sort(sortSetByViews);
            // Adds the intro if necessary
            if (!GameController_8.GameController.Instance.state.hasSeenIntroSet) {
                console.log("[BookController].getRandomSetId Intro themes inserted first");
                // Ticket description says: "Intro sets are not randomized, they cycle through in order across different sessions"
                // Also has to be sorted after random to achieve the same effect
                introThemes.sort(sortSetByViews);
                // Intros goes at the begining, because, if they are to be shown, they have to be shown first
                commonSetIds = introThemes.concat(commonSetIds);
            }
            return commonSetIds;
        }
        // Returns a set for given setID, if it doesnt exists it returns an empty array. 
        // This makes the code safer and failsafe, and more logical to work with
        getSafeSet(setID) {
            return this.sets.get(setID) || [];
        }
        // Returns true when the current is the last sequence of the set. That means, calling getNextSequence will trigger a new set
        isAtSetEnd() {
            let state = GameController_8.GameController.Instance.state;
            return state.currentActivityIndex >= this.getSafeSet(state.currentSetID).length - 1;
        }
        /* Changes to a new set.
         - If there was a valid set previously, mark it as finished.
            Be careful because it dispatches a signal. So be careful which data you try to access/modify on that listener.
         - Sets currentActivityIndex and currentSetID on the savestate, and the currentTheme in this class.
        */
        setNewSet() {
            let state = GameController_8.GameController.Instance.state;
            let setID = state.currentSetID;
            let theme = this.getCurrentTheme();
            // After some consideration i've found that it's important to have this here (and not in another function (though it could be split later))
            /// Finish the previous set (if it's valid)
            if (this.getSafeSet(setID).length > 0) {
                // If it was a valid set, count a view because it has ended
                state.setSetViews(setID, state.getSetViews(setID) + 1);
                // if it was an intro, mark intro as viewed
                if (theme === BookSequenceController.THEME_INTRO) {
                    state.hasSeenIntroSet = true;
                }
                // dispatch the signal
                this.onSetFinished.dispatch(setID, theme);
                PPAnalyticsManager_2.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Mini:Set', 'Set', 'Activity', theme, {
                    [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_8.GameController.Instance.state.currentSetID,
                    [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_THEME]: theme,
                    [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_THEME_COMPLETED]: GameController_8.GameController.Instance.state.getActivityPlays(theme)
                });
            }
            /// ~
            // Note returns a list, in case we want to add extra logic to it
            let prioSetsIds = this.getPrioSetIds();
            // Get our possible sets, and take the first one
            state.currentSetID = prioSetsIds[0];
            // new set, start over
            state.currentActivityIndex = 0;
            // gets the current theme for this set and set it
            const newTheme = this.getCurrentTheme();
            PPAnalyticsManager_2.PPAnalyticsManager.Instance.startTiming('OffToNeverLand:Mini:Set', {
                [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_8.GameController.Instance.state.currentSetID,
                [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_THEME]: newTheme,
                [PPAnalyticsManager_2.PPAnalyticsManager.ACTIVITY_SET_THEME_COMPLETED]: GameController_8.GameController.Instance.state.getActivityPlays(newTheme)
            });
            return state.currentSetID;
        }
        /*
         API, used by BookState, returns the next sequence to be shown.
         Internally chooses which set has to show.
        */
        getNextSequence() {
            let state = GameController_8.GameController.Instance.state;
            // Use the last set saved. if none, then the if below will account for it
            let currentSet = state.currentSetID;
            // Gets current data. Accounts for an invalid set, in which case will force to get a new set
            let currentSetData = this.getSafeSet(currentSet);
            // Goes to the next activity if any (invalid number if none)
            state.currentActivityIndex++;
            // If we passed the limit (or has an invalid set), load a new one
            if (state.currentActivityIndex >= currentSetData.length) {
                // Gets a new set (notice it will modify state.currentSetID)
                this.setNewSet();
                // Update the data
                currentSetData = this.sets.get(state.currentSetID);
            }
            let currentSequenceData = currentSetData[state.currentActivityIndex];
            console.log("[BookController].getNextSequence currentActivityIndex", state.currentActivityIndex);
            return currentSequenceData.slice(); // slice is important, the consumer deletes the items
        }
        // Returns the currentTheme. It's difficult to get the current theme so it's better to have it as a function.
        getCurrentTheme() {
            // Important: Also, as the savestate could be deleted (recycled) after this class has been initialized
            // because the player choose to start over again, it's not recomendable to store the current theme as
            // an attribute of this class, and rather we should try to calculate it when it's needed; using the save data.
            // Tries to get the theme for the current activity. or "" if fails.
            return this.getThemeForSetId(GameController_8.GameController.Instance.state.currentSetID);
        }
        getThemeForSetId(setID) {
            let set = (this.sets.get(setID) || []);
            return (set.length < 1) ? "" : set[0][0].themeID;
        }
        /*
         API entrypoint, parses the whole JSON file for the book.
         Also does some needed initialization. To handle restoring view position.
        */
        parseBook(bookData) {
            // Create new objects
            this.book = new Map();
            this.sets = new Map();
            let newIntroSetIDs = [];
            // Iterate the sequences of the json
            for (let jsPage of bookData) {
                // Try to parse it
                let seq = this.parseSequence(jsPage);
                if (seq.length === 0) {
                    continue;
                } // This aren't the pages you're looking for
                // this would assume that all the pages from a parseSequence belong to 
                // the same sequence, which is basically true (by design) and because the value is actually redundant
                // makes the code faster and easier
                // Store it to ensure that the data is correct
                let page0 = seq[0];
                let setId = page0.setID;
                let themeId = page0.themeID;
                // get the sequence from the sets map or add it
                if (!this.sets.has(setId)) {
                    this.sets.set(setId, []);
                }
                let thisSet = this.sets.get(setId);
                thisSet.push(seq); // no flattening!
                ///// Also index in the book hierarchy
                // get the theme
                if (!this.book.has(themeId)) {
                    // new theme
                    this.book.set(themeId, new Map());
                }
                let theme = this.book.get(themeId);
                // update the sequence
                // optimization note. as we are storing references, and only pushing to the set, we only should need to do this once
                // probably when the set if first created, but i rather be safe than sorry
                theme.set(setId, thisSet);
                // If its an intro, cache it
                if (themeId === BookSequenceController.THEME_INTRO && newIntroSetIDs.indexOf(setId) < 0) {
                    newIntroSetIDs.push(setId);
                }
            }
            // Updates the cache of introSets for clearing states
            GameController_8.GameController.Instance.introSetIDs = newIntroSetIDs;
            let state = GameController_8.GameController.Instance.state;
            // If it haven't seen the intro set, invalidate the current set so it will load the proper one
            if (!state.hasSeenIntroSet) {
                state.currentSetID = -1;
                state.currentActivityIndex = -1;
            }
            else {
                // In case it has, continue from the last set, and THE SAME PAGE (as requested) (must decrease one)
                state.currentActivityIndex = Math.max(-1, state.currentActivityIndex - 1);
            }
        }
        // Parses a JSON Sequence data into a SequenceData (AKA Activity AKA PageData[])
        parseSequence(page) {
            let responseArray = [];
            let pageData;
            // Defaults
            let setID = page.setID || 0;
            let themeID = page.themeID || 'personal';
            switch (page.type) {
                case LayoutType_8.LayoutType.INTRO:
                    pageData = new IntroPageData_1.IntroPageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.TEXT_WITH_IMAGE:
                    pageData = new TextPicturePageData_1.TextPicturePageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.JOKE_PAGE:
                    pageData = new JokePageData_1.JokePageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.OPEN_QUESTION:
                    pageData = new OpenQuestionPageData_1.OpenQuestionPageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.CHARADES:
                    pageData = new CharadesPageData_1.CharadesPageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.PICK_ONE_OF_TWO:
                    pageData = new PickOneOfTwoPageData_1.PickOneOfTwoPageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.COUNTER:
                    pageData = new CounterPageData_1.CounterPageData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.MULTI_OPTIONS:
                    pageData = new MultiPageOptionData_1.MultiPageOptionData().parse(page.details);
                    break;
                case LayoutType_8.LayoutType.MULTI_PAGE:
                    page.details.pages.forEach((page) => {
                        let pageData = this.parseSequence(page)[0];
                        pageData.isFinalPageOfActivity = false;
                        responseArray = responseArray.concat(pageData);
                    }, this);
                    responseArray[responseArray.length - 1].isFinalPageOfActivity = true;
                    break;
            }
            if (pageData) {
                responseArray.push(pageData);
            }
            // set the correct setID and themeID as that is not in .details (ensure data correctness)
            // using ´for (page of responseArray)´ here will make the playApp crash (but not the browser), because tightscript
            for (let i = 0; i < responseArray.length; i++) {
                let page = responseArray[i];
                page.setID = setID;
                page.themeID = themeID;
            }
            return responseArray;
        }
    }
    BookSequenceController.THEME_INTRO = 'intro';
    exports.BookSequenceController = BookSequenceController;
});
define("pages/LoadInstructionsPageData", ["require", "exports", "Game", "pages/Page", "pages/TextPicturePageData", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "logic/GameController", "logic/PPAnalyticsManager", "switchback-common/util/ScaleUtil"], function (require, exports, Game_12, Page_9, TextPicturePageData_2, TextUtil_12, LocalizationManager_9, GameController_9, PPAnalyticsManager_3, ScaleUtil_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LoadInstructionsPageData extends TextPicturePageData_2.TextPicturePageData {
        parse(data) {
            super.parse(data);
            if (!data.text_top
                || !data.picture
                || !data.text_bottom) {
                return null;
            }
            this.textTop = LocalizationManager_9.LocalizationManager.Instance.text(data.text_top);
            this.picTag = data.picture;
            this.textBottom = LocalizationManager_9.LocalizationManager.Instance.text(data.text_bottom);
            return this;
        }
        build(game) {
            return [new LoadInstructionsPage(game, this.text, this.isFinalPageOfActivity, this.textTop, this.textBottom, this.picTag)];
        }
    }
    exports.LoadInstructionsPageData = LoadInstructionsPageData;
    class LoadInstructionsPage extends Page_9.Page {
        constructor(game, text, isFinalPageOfActivity, mainTextString, subTextString, picTag) {
            super(game, '', Page_9.PageType.NO_CORNER, null, text, isFinalPageOfActivity, true);
            const image = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.1, picTag);
            image.anchor.set(0.5, 0);
            ScaleUtil_10.scaleToBounds(image, this.game.width * 0.9, this.game.height * 0.6);
            this.add(image);
            /* Disney requested we remove load instruction safety text.
               I've left this here incase they change their mind back again.
            const mainText: Phaser.Text = new Phaser.Text(game, this.game.width * 0.5, image.bottom, mainTextString, {
                font: Game.DEFAULT_FONT,
                fill: Game.TEXT_COLOR,
                fontSize: Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center',
                stroke: Game.TEXT_STROKE_COLOR,
                strokeThickness: Game.TEXT_STROKE_THICKNESS
            });
            mainText.anchor.set(0.5, 0);
            mainText.setShadow(Game.TEXT_SHADOW_OFFSET_SMALL_X, Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game.TEXT_SHADOW_SOFT);
            this.add(mainText);
            TextUtil.scaleDownTextToHeight(mainText, this.game.height * 0.25);
            */
            const subText = new Phaser.Text(game, this.game.width * 0.5, image.bottom + this.height * 0.02, subTextString, {
                font: Game_12.Game.DEFAULT_FONT,
                fill: Game_12.Game.TEXT_COLOR_GREEN,
                fontSize: Game_12.Game.FONT_SIZE_LARGE,
                stroke: Game_12.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_12.Game.TEXT_STROKE_THICKNESS,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
            });
            subText.anchor.set(0.5, 0);
            subText.setShadow(Game_12.Game.TEXT_SHADOW_OFFSET_MED_X, Game_12.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_12.Game.TEXT_SHADOW_SOFT);
            TextUtil_12.TextUtil.scaleDownTextToHeight(subText, this.game.height - image.height - this.height * 0.02);
            this.add(subText);
        }
        // This page can't be turned - never resolve this!
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                PPAnalyticsManager_3.PPAnalyticsManager.Instance.trackState('content/game/inqueue/outro/loadinstructions/1_1', 'LoadInstructions1_1', {
                    [PPAnalyticsManager_3.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_9.GameController.Instance.state.currentSetID,
                    [PPAnalyticsManager_3.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_9.GameController.Instance.state.currentActivityIndex
                });
                // Give them an achievement for hitting merge
                GameController_9.GameController.Instance.achievementManager.onMergeReached();
                // When we get here, we can wipe their save.
                yield GameController_9.GameController.Instance.recycleState();
                return new Promise((resolve, reject) => { });
            });
        }
    }
    exports.LoadInstructionsPage = LoadInstructionsPage;
});
define("view/MediumButton", ["require", "exports", "Game"], function (require, exports, Game_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var RibbonWidth;
    (function (RibbonWidth) {
        RibbonWidth[RibbonWidth["MEDIUM"] = 0] = "MEDIUM";
        RibbonWidth[RibbonWidth["LARGE"] = 1] = "LARGE";
    })(RibbonWidth = exports.RibbonWidth || (exports.RibbonWidth = {}));
    class MediumButton extends Phaser.Button {
        constructor(game, frame, text, size = RibbonWidth.MEDIUM) {
            let buttonAssetKey = 'button-medium';
            if (size === RibbonWidth.LARGE) {
                buttonAssetKey = 'button-large'; // TODO: change the asset key here when you have 
            }
            super(game, 0, 0, buttonAssetKey, null, null, frame, frame, frame, frame);
            this.events.onInputDown.add(this.onPress, this);
            this.anchor.set(0.5);
            this.label = new Phaser.Text(game, 0, 0, text, {
                font: Game_13.Game.DEFAULT_FONT,
                fill: Game_13.Game.TEXT_BUTTON_COLOR,
                fontSize: Game_13.Game.FONT_SIZE_LARGE,
            });
            this.label.anchor.set(0.5);
            this.label.setShadow(Game_13.Game.TEXT_SHADOW_OFFSET_MED_X, Game_13.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_13.Game.TEXT_SHADOW_HARD);
            this.addChild(this.label);
        }
        set text(text) {
            this.label.text = text;
        }
        onPress() {
            this.game.sound.play("pp-sfx-ui-click");
        }
    }
    exports.MediumButton = MediumButton;
});
define("view/ConfirmDialog", ["require", "exports", "switchback-common/util/ScaleUtil", "Game", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil"], function (require, exports, ScaleUtil_11, Game_14, PromiseUtil_3, TextUtil_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ConfirmDialog extends Phaser.Group {
        constructor(game, titleText, descriptionText, confirmText, denyText, confirmIsPositive = true) {
            super(game);
            this.clickedSignal = new Phaser.Signal();
            this.shade = new Phaser.Graphics(this.game);
            this.shade.beginFill(0x031220, 0.7);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.inputEnabled = false;
            this.shade.alpha = 0;
            this.add(this.shade);
            this.bgGroup = new Phaser.Group(this.game);
            this.add(this.bgGroup);
            this.bg = new Phaser.Image(game, this.game.width * 0.5, this.game.height * 0.5, "dialog-bg");
            this.bg.anchor.set(0.5);
            this.bg.width = this.game.width * 0.9;
            this.bg.height = this.game.height * 0.5;
            this.bgGroup.add(this.bg);
            if (titleText) {
                this.titleText = new Phaser.Text(game, this.bg.centerX, this.bg.top + this.bg.height * 0.13, titleText, {
                    font: Game_14.Game.DEFAULT_FONT,
                    fill: Game_14.Game.TEXT_COLOR_RED,
                    fontSize: Game_14.Game.FONT_SIZE_LARGE
                });
                this.titleText.anchor.set(0.5, 0);
                this.bgGroup.add(this.titleText);
            }
            let descriptionPos = this.bg.top + this.bg.height * 0.2;
            if (this.titleText) {
                descriptionPos = this.titleText.bottom;
            }
            this.descriptionText = new Phaser.Text(game, this.bg.centerX, descriptionPos, descriptionText, {
                font: Game_14.Game.DEFAULT_FONT,
                fill: Game_14.Game.TEXT_COLOR,
                fontSize: TextUtil_13.TextUtil.getFontSize(this.game.width, 25),
                wordWrap: true,
                wordWrapWidth: this.bg.width * 0.8,
                align: 'center',
                stroke: Game_14.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_14.Game.TEXT_STROKE_THICKNESS
            });
            this.descriptionText.setShadow(Game_14.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_14.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_14.Game.TEXT_SHADOW_SOFT);
            this.descriptionText.anchor.set(0.5, 0);
            this.bgGroup.add(this.descriptionText);
            this.confirmButton = new Phaser.Button(game, this.bg.right, this.bg.top + this.bg.height * 0.8, confirmIsPositive ? 'dialog-button-positive' : 'dialog-button-negative');
            ScaleUtil_11.scaleToBounds(this.confirmButton, this.bg.width * 0.525, this.bg.height * 0.35);
            this.confirmButton.anchor.set(0.975, 0.5);
            this.confirmButton.events.onInputDown.add(() => {
                this.game.sound.play("pp-sfx-ui-click");
                this.clickedSignal.dispatch(true);
            });
            this.confirmButton.x = this.bg.right;
            if (confirmIsPositive) {
                this.confirmButton.width *= -1;
                this.confirmButton.x = this.bg.right + (this.confirmButton.width * 0.95);
            }
            this.bgGroup.add(this.confirmButton);
            const confirmLabel = new Phaser.Text(game, this.confirmButton.centerX + Math.abs(this.confirmButton.width) * 0.05, this.confirmButton.top + this.confirmButton.height * 0.425, confirmText, {
                font: Game_14.Game.DEFAULT_FONT,
                fill: Game_14.Game.TEXT_BUTTON_COLOR,
                fontSize: Game_14.Game.FONT_SIZE_LARGE,
            });
            confirmLabel.anchor.set(0.5);
            confirmLabel.setShadow(Game_14.Game.TEXT_SHADOW_OFFSET_MED_X, Game_14.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_14.Game.TEXT_SHADOW_HARD);
            this.bgGroup.add(confirmLabel);
            this.denyButton = new Phaser.Button(game, this.bg.left, this.bg.top + this.bg.height * 0.85, confirmIsPositive ? 'dialog-button-negative' : 'dialog-button-positive');
            ScaleUtil_11.scaleToBounds(this.denyButton, this.bg.width * 0.525, this.bg.height * 0.35);
            this.denyButton.anchor.set(0.025, 0.5);
            this.denyButton.events.onInputDown.add(() => {
                this.game.sound.play("pp-sfx-ui-click");
                this.clickedSignal.dispatch(false);
            });
            this.denyButton.x = this.bg.left;
            if (confirmIsPositive) {
                this.denyButton.width *= -1;
                this.denyButton.x = this.bg.left - (this.denyButton.width * 0.95);
            }
            this.bgGroup.add(this.denyButton);
            const denyLabel = new Phaser.Text(game, this.denyButton.centerX - Math.abs(this.denyButton.width) * 0.05, this.denyButton.top + this.denyButton.height * 0.425, denyText, {
                font: Game_14.Game.DEFAULT_FONT,
                fill: Game_14.Game.TEXT_BUTTON_COLOR,
                fontSize: Game_14.Game.FONT_SIZE_LARGE,
            });
            denyLabel.anchor.set(0.5);
            denyLabel.setShadow(Game_14.Game.TEXT_SHADOW_OFFSET_MED_X, Game_14.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_14.Game.TEXT_SHADOW_HARD);
            this.bgGroup.add(denyLabel);
            this.bgGroup.top = this.game.height;
            this.bgGroup.alpha = 0;
            this.visible = false;
        }
        setDescription(description) {
            this.descriptionText.text = description;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.shade.inputEnabled = true;
                this.visible = true;
                this.game.add.tween(this.bgGroup).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.shade).to({ alpha: 1 }, ConfirmDialog.ANIMATION_DURATION, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_3.PromiseUtil.tween(this.game, this.game.add.tween(this.bgGroup).to({ centerY: this.game.height * 0.5 }, ConfirmDialog.ANIMATION_DURATION, Phaser.Easing.Sinusoidal.Out, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.bgGroup).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.shade).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_3.PromiseUtil.tween(this.game, this.game.add.tween(this.bgGroup).to({ top: this.game.height }, ConfirmDialog.ANIMATION_DURATION, Phaser.Easing.Sinusoidal.Out, true));
                this.visible = false;
                this.shade.inputEnabled = false;
            });
        }
    }
    ConfirmDialog.ANIMATION_DURATION = 500;
    exports.ConfirmDialog = ConfirmDialog;
});
define("pages/ExitPromptPageData", ["require", "exports", "Game", "pages/Page", "switchback-common/localization/LocalizationManager", "pages/PickOneOfTwoPageData", "view/MediumButton", "view/ConfirmDialog", "switchback-common/util/PromiseUtil", "logic/GameController", "logic/PPAnalyticsManager", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_15, Page_10, LocalizationManager_10, PickOneOfTwoPageData_2, MediumButton_1, ConfirmDialog_1, PromiseUtil_4, GameController_10, PPAnalyticsManager_4, ScaleUtil_12, TextUtil_14) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ExitPromptPageData extends PickOneOfTwoPageData_2.PickOneOfTwoPageData {
        build(game) {
            return [
                new ExitPromptPage(game, this.text, this.isFinalPageOfActivity, this.titleString, this.optionOneText, this.optionTwoText)
            ];
        }
    }
    exports.ExitPromptPageData = ExitPromptPageData;
    class ExitPromptPage extends Page_10.Page {
        constructor(game, descriptionString, isFinalPageOfActivity, titleString, optionOneText, optionTwoText) {
            super(game, '', Page_10.PageType.NO_CORNER, null, null, isFinalPageOfActivity, false);
            const aspectRatio = this.game.width / this.game.height;
            let aspectRatioMultiplier;
            if (aspectRatio > (9.0 / 16.0)) {
                aspectRatioMultiplier = (9.0 / 16.0) / aspectRatio;
            }
            else {
                aspectRatioMultiplier = 1;
            }
            const queueImage = new Phaser.Image(game, 0, 0, 'wdw-merge');
            ScaleUtil_12.scaleToBounds(queueImage, this.game.width * 0.9 * aspectRatioMultiplier, this.game.height * 0.5 * aspectRatioMultiplier);
            queueImage.centerX = this.game.width * 0.5;
            queueImage.centerY = this.game.height * 0.4;
            this.add(queueImage);
            const titleText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_10.LocalizationManager.Instance.text(titleString), {
                font: Game_15.Game.DEFAULT_FONT,
                fill: Game_15.Game.TEXT_COLOR_GREEN,
                fontSize: Game_15.Game.FONT_SIZE_LARGE,
                stroke: Game_15.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_15.Game.TEXT_STROKE_THICKNESS
            });
            titleText.anchor.set(0.5, 0);
            titleText.setShadow(Game_15.Game.TEXT_SHADOW_OFFSET_MED_X, Game_15.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_15.Game.TEXT_SHADOW_SOFT);
            this.add(titleText);
            titleText.top = this.game.height * 0.02;
            const descriptionText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_10.LocalizationManager.Instance.text(descriptionString), {
                font: Game_15.Game.DEFAULT_FONT,
                fill: Game_15.Game.TEXT_COLOR,
                fontSize: Game_15.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                align: 'center',
                stroke: Game_15.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_15.Game.TEXT_STROKE_THICKNESS
            });
            descriptionText.anchor.set(0.5, 0);
            descriptionText.setShadow(Game_15.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_15.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_15.Game.TEXT_SHADOW_SOFT);
            this.add(descriptionText);
            TextUtil_14.TextUtil.scaleDownTextToHeight(descriptionText, queueImage.top - titleText.bottom - this.game.height * 0.01);
            descriptionText.centerY = titleText.bottom + (queueImage.top - titleText.bottom) * 0.5;
            const buttonGroup = new Phaser.Group(this.game, this);
            const bgDetail = new Phaser.Image(this.game, 0, 0, 'outro_background');
            bgDetail.anchor.set(0.5);
            bgDetail.width = this.game.width;
            bgDetail.scale.y = bgDetail.scale.x;
            buttonGroup.add(bgDetail);
            this.yesButton = new MediumButton_1.MediumButton(this.game, 0, optionOneText);
            this.yesButton.x = bgDetail.centerX;
            this.yesButton.y = bgDetail.top + bgDetail.height * 0.32;
            buttonGroup.add(this.yesButton);
            this.noButton = new MediumButton_1.MediumButton(this.game, 1, optionTwoText);
            this.noButton.x = bgDetail.centerX;
            this.noButton.y = bgDetail.top + bgDetail.height * 0.68;
            buttonGroup.add(this.noButton);
            buttonGroup.width *= aspectRatioMultiplier;
            buttonGroup.scale.y = buttonGroup.scale.x;
            buttonGroup.x = this.game.width * 0.5;
            buttonGroup.top = queueImage.bottom;
            this.add(buttonGroup);
            this.confirmDialog = new ConfirmDialog_1.ConfirmDialog(this.game, LocalizationManager_10.LocalizationManager.Instance.text('outro.confirm_dialog.title'), LocalizationManager_10.LocalizationManager.Instance.text('outro.confirm_dialog.text'), LocalizationManager_10.LocalizationManager.Instance.text('outro.confirm_dialog.yes'), LocalizationManager_10.LocalizationManager.Instance.text('outro.confirm_dialog.no'), true);
            this.add(this.confirmDialog);
        }
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    PPAnalyticsManager_4.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'OutroAreYouHere', {
                        [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_10.GameController.Instance.state.currentSetID,
                        [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_10.GameController.Instance.state.currentActivityIndex
                    });
                    this.yesButton.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                        // Get them to confirm their 'Yes' choice
                        yield this.confirmDialog.show();
                        const confirmed = yield PromiseUtil_4.PromiseUtil.signal(this.confirmDialog.clickedSignal);
                        yield this.confirmDialog.hide();
                        if (confirmed) {
                            // Stop listening for the no
                            this.noButton.events.onInputDown.removeAll();
                            // Tell the game controller they want to exit
                            GameController_10.GameController.Instance.exitConfirmed = true;
                            PPAnalyticsManager_4.PPAnalyticsManager.Instance.trackAction('Declare_Yes', 'Location', 'Yes', 'Outro', {
                                [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_10.GameController.Instance.state.currentSetID,
                                [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_10.GameController.Instance.state.currentActivityIndex
                            });
                            // And resolve, they've finished 'answering' this page
                            resolve();
                        }
                    }), this);
                    this.noButton.events.onInputDown.addOnce(() => {
                        // Stop listening for the yes
                        this.yesButton.events.onInputDown.removeAll();
                        PPAnalyticsManager_4.PPAnalyticsManager.Instance.trackAction('Declare_No', 'Location', 'No', 'Outro', {
                            [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_10.GameController.Instance.state.currentSetID,
                            [PPAnalyticsManager_4.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_10.GameController.Instance.state.currentActivityIndex
                        });
                        // And resolve, they've finished 'answering' this page
                        resolve();
                    });
                });
            });
        }
        // As soon as this is set to be turnable, 
        // fake a click - it should turn immediately
        set turnable(turnable) {
            PromiseUtil_4.PromiseUtil.time(this.game, 100).then(() => {
                this.onPageClicked.dispatch();
            });
        }
    }
    exports.ExitPromptPage = ExitPromptPage;
});
define("logic/sequence-controllers/OutroSequenceController", ["require", "exports", "pages/LoadInstructionsPageData", "logic/GameController", "pages/ExitPromptPageData"], function (require, exports, LoadInstructionsPageData_1, GameController_11, ExitPromptPageData_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Similar to the intro controller - returns a specific outro sequence.
     *
     * @author joel@cerebralfix.com
     */
    class OutroSequenceController {
        getNextSequence() {
            if (!GameController_11.GameController.Instance.exitConfirmed) {
                return [
                    new ExitPromptPageData_1.ExitPromptPageData().parse({
                        title_key: 'outro.confirm_page.title',
                        text_key: 'outro.confirm_page.text',
                        answer_1_key: "outro.confirm_page.yes",
                        answer_2_key: "outro.confirm_page.no",
                        is_final_activity_page: false
                    })
                ];
            }
            else {
                return [
                    new LoadInstructionsPageData_1.LoadInstructionsPageData().parse({
                        text_top: "outro.main_text",
                        picture: "load_illustration",
                        text_bottom: "outro.sub_text",
                        is_final_activity_page: false
                    })
                ];
            }
        }
    }
    exports.OutroSequenceController = OutroSequenceController;
});
define("pages/TextPicTextPageData", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "pages/Page", "Game", "pages/PageData"], function (require, exports, ScaleUtil_13, TextUtil_15, LocalizationManager_11, Page_11, Game_16, PageData_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextPicTextPageData extends PageData_9.PageData {
        parse(data) {
            super.parse(data);
            if (!data.text_key
                || data.picture === null
                || !data.text_bottom_key) {
                return null;
            }
            this.picTag = data.picture;
            this.textBottom = LocalizationManager_11.LocalizationManager.Instance.text(data.text_bottom_key);
            return this;
        }
        build(game) {
            return [new TextPicTextPage(game, this.activityType, this.titleString, this.text, this.isFinalPageOfActivity, this.textBottom, this.picTag)];
        }
    }
    exports.TextPicTextPageData = TextPicTextPageData;
    class TextPicTextPage extends Page_11.Page {
        constructor(game, activityType, titleString, text, isFinalPageOfActivity, bottomTextString, pictureTag) {
            super(game, activityType, Page_11.PageType.CORNER, titleString, text, isFinalPageOfActivity);
            this.text.centerY = this.game.height * 0.22;
            const picture = new Phaser.Image(game, 0, 0, pictureTag);
            ScaleUtil_13.scaleToBounds(picture, game.width * 0.9, game.height * 0.3);
            picture.centerX = game.width * 0.5;
            picture.centerY = game.height * 0.5;
            this.add(picture);
            const textBottom = new Phaser.Text(game, 0, 0, bottomTextString, {
                fontSize: TextUtil_15.TextUtil.getFontSize(game.width, 25),
                fill: Game_16.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.9,
                font: Game_16.Game.DEFAULT_FONT
            });
            textBottom.top = (game.height * 0.8) - (textBottom.height * 0.5);
            textBottom.left = (game.width - textBottom.width) * 0.5;
            this.add(textBottom);
        }
    }
    exports.TextPicTextPage = TextPicTextPage;
});
define("pages/component/CheckBox", ["require", "exports", "switchback-common/util/ScaleUtil"], function (require, exports, ScaleUtil_14) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CheckBox extends Phaser.Group {
        constructor(game) {
            super(game);
            const boxBackground = new Phaser.Image(this.game, 0, 0, 'checkbox_background');
            boxBackground.anchor.set(0.5);
            this.add(boxBackground);
            this.checkMark = new Phaser.Image(this.game, 0, 0, 'selection_tick');
            this.checkMark.anchor.set(0.5);
            this.checkMark.visible = false;
            ScaleUtil_14.scaleToBounds(this.checkMark, boxBackground.width, boxBackground.height);
            this.add(this.checkMark);
        }
        set selected(s) {
            this.checkMark.visible = s;
        }
    }
    exports.CheckBox = CheckBox;
});
define("pages/QueueSelectionPageData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "view/ConfirmDialog", "logic/GameController", "enum/Queue", "view/MediumButton", "pages/component/CheckBox", "logic/PPAnalyticsManager"], function (require, exports, Game_17, PageData_10, Page_12, ScaleUtil_15, LocalizationManager_12, PromiseUtil_5, ConfirmDialog_2, GameController_12, Queue_3, MediumButton_2, CheckBox_1, PPAnalyticsManager_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class QueueSelectionPageData extends PageData_10.PageData {
        build(game) {
            return [
                new QueueSelectionPage(game)
            ];
        }
    }
    exports.QueueSelectionPageData = QueueSelectionPageData;
    class QueueSelectionPage extends Page_12.Page {
        constructor(game) {
            super(game, '', Page_12.PageType.NO_CORNER, null, null, false, false);
            const descriptionText = new Phaser.Text(game, this.game.width * 0.5, this.game.height * 0.06, LocalizationManager_12.LocalizationManager.Instance.text('queue_select.text'), {
                font: Game_17.Game.DEFAULT_FONT,
                fill: Game_17.Game.TEXT_COLOR,
                fontSize: Game_17.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                align: 'center',
                stroke: Game_17.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_17.Game.TEXT_STROKE_THICKNESS
            });
            descriptionText.anchor.set(0.5, 0);
            descriptionText.setShadow(Game_17.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_17.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_17.Game.TEXT_SHADOW_SOFT);
            this.add(descriptionText);
            this.queue1 = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.38, 'queue-1');
            this.queue1.anchor.set(0.5);
            ScaleUtil_15.scaleToBounds(this.queue1, this.game.width * 0.95, this.game.height * 0.325);
            this.queue1.inputEnabled = true;
            this.add(this.queue1);
            this.queue2 = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.7, 'queue-2');
            this.queue2.anchor.set(0.5);
            ScaleUtil_15.scaleToBounds(this.queue2, this.game.width * 0.95, this.game.height * 0.325);
            this.queue2.inputEnabled = true;
            this.add(this.queue2);
            this.queue1Box = new CheckBox_1.CheckBox(this.game);
            this.queue1Box.position.set(this.queue1.left + this.queue1.width * 0.05, this.queue1.top + this.queue1.height * 0.2);
            ScaleUtil_15.scaleToBounds(this.queue1Box, this.queue1.width * 0.15, this.queue1.height * 0.15);
            this.add(this.queue1Box);
            this.queue2Box = new CheckBox_1.CheckBox(this.game);
            this.queue2Box.position.set(this.queue2.left + this.queue2.width * 0.05, this.queue2.top + this.queue2.height * 0.2);
            ScaleUtil_15.scaleToBounds(this.queue2Box, this.queue2.width * 0.15, this.queue2.height * 0.15);
            this.add(this.queue2Box);
            this.button = new MediumButton_2.MediumButton(this.game, 0, LocalizationManager_12.LocalizationManager.Instance.text("button.continue"), MediumButton_2.RibbonWidth.LARGE);
            this.button.centerX = this.game.width * 0.5;
            this.button.bottom = this.game.height * 0.96;
            ScaleUtil_15.scaleToBounds(this.button, this.game.width * 0.9, this.game.height * 0.1);
            this.button.alpha = 0;
            this.button.visible = false;
            this.add(this.button);
            this.confirmDialog = new ConfirmDialog_2.ConfirmDialog(this.game, LocalizationManager_12.LocalizationManager.Instance.text('queue_select.confirm_dialog.title'), LocalizationManager_12.LocalizationManager.Instance.text('queue_select.confirm_dialog.queueAText'), LocalizationManager_12.LocalizationManager.Instance.text('queue_select.confirm_dialog.yes'), LocalizationManager_12.LocalizationManager.Instance.text('queue_select.confirm_dialog.no'), true);
            this.add(this.confirmDialog);
        }
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                PPAnalyticsManager_5.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'QueueSelect');
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    this.pageResolutionFunction = resolve;
                    // Set up listeners on each image that will perform the selection
                    this.queue1.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                        this.selectedQueue = Queue_3.Queue.PRIMARY;
                        this.game.sound.play("pp-sfx-ui-click");
                        this.queue1Box.selected = true;
                        this.queue2Box.selected = false;
                        PPAnalyticsManager_5.PPAnalyticsManager.Instance.trackAction('Declare_LineA', 'QueueSelect', 'QueueA', '');
                    }));
                    this.queue2.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                        this.selectedQueue = Queue_3.Queue.SECONDARY;
                        this.game.sound.play("pp-sfx-ui-click");
                        this.queue1Box.selected = false;
                        this.queue2Box.selected = true;
                        PPAnalyticsManager_5.PPAnalyticsManager.Instance.trackAction('Declare_LineB', 'QueueSelect', 'QueueB', '');
                    }));
                    // Wait for one of them to be clicked.
                    yield Promise.race([
                        PromiseUtil_5.PromiseUtil.signal(this.queue1.events.onInputDown),
                        PromiseUtil_5.PromiseUtil.signal(this.queue2.events.onInputDown)
                    ]);
                    // Show the button and wait for it to be clicked
                    this.button.visible = true;
                    this.game.add.tween(this.button).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                    this.button.onInputDown.add(this.onContinueClicked, this);
                }));
            });
        }
        onContinueClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                const queueDescription = (this.selectedQueue === Queue_3.Queue.PRIMARY) ?
                    ('queue_select.confirm_dialog.queueAText') : ('queue_select.confirm_dialog.queueBText');
                this.confirmDialog.setDescription(LocalizationManager_12.LocalizationManager.Instance.text(queueDescription));
                yield this.confirmDialog.show();
                const confirmed = yield PromiseUtil_5.PromiseUtil.signal(this.confirmDialog.clickedSignal);
                this.confirmDialog.hide();
                PPAnalyticsManager_5.PPAnalyticsManager.Instance.trackAction(confirmed ? 'Declare_Yes' : 'Declare_No', 'QueueSelect', confirmed ? 'Yes' : 'No', this.selectedQueue === Queue_3.Queue.PRIMARY ? 'QueueA' : 'QueueB');
                if (confirmed) {
                    // Stop listening to any changes to the queue type
                    this.queue1.events.onInputDown.removeAll();
                    this.queue2.events.onInputDown.removeAll();
                    // Set the currently selected queue as the one we'll be using.
                    GameController_12.GameController.Instance.setSelectedQueueType(this.selectedQueue);
                    // And resolve, they've finished 'answering' this page
                    this.pageResolutionFunction();
                }
            });
        }
        // As soon as this is set to be turnable, 
        // fake a click - it should turn immediately
        set turnable(turnable) {
            PromiseUtil_5.PromiseUtil.time(this.game, 100).then(() => {
                this.onPageClicked.dispatch();
            });
        }
    }
    exports.QueueSelectionPage = QueueSelectionPage;
});
define("logic/sequence-controllers/IntroSequenceController", ["require", "exports", "pages/TextPicTextPageData", "pages/QueueSelectionPageData", "logic/PPAnalyticsManager"], function (require, exports, TextPicTextPageData_1, QueueSelectionPageData_1, PPAnalyticsManager_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Pretty basic controller - returns a specific intro sequence.
     * Probably only called once per execution.
     *
     * @author joel@cerebralfix.com
     */
    class IntroSequenceController {
        getNextSequence() {
            PPAnalyticsManager_6.PPAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene', 'Welcome', {
                [PPAnalyticsManager_6.PPAnalyticsManager.CUTSCENE]: "Welcome"
            });
            return [
                new TextPicTextPageData_1.TextPicTextPageData().parse({
                    text_key: "book.intro_1_text_1",
                    picture: "pic_neverland",
                    text_bottom_key: "book.intro_1_text_2",
                    is_final_activity_page: false
                })
            ];
        }
        getQueueSelectSequence() {
            return [
                new QueueSelectionPageData_1.QueueSelectionPageData()
            ];
        }
    }
    exports.IntroSequenceController = IntroSequenceController;
});
define("pages/ResumePromptPageData", ["require", "exports", "Game", "pages/PageData", "pages/Page", "switchback-common/localization/LocalizationManager", "view/MediumButton", "view/ConfirmDialog", "switchback-common/util/PromiseUtil", "logic/GameController", "logic/PPAnalyticsManager"], function (require, exports, Game_18, PageData_11, Page_13, LocalizationManager_13, MediumButton_3, ConfirmDialog_3, PromiseUtil_6, GameController_13, PPAnalyticsManager_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ResumePromptPageData extends PageData_11.PageData {
        build(game) {
            return [
                new ResumePromptPage(game)
            ];
        }
    }
    exports.ResumePromptPageData = ResumePromptPageData;
    class ResumePromptPage extends Page_13.Page {
        constructor(game) {
            super(game, '', Page_13.PageType.NO_CORNER, null, null, false, false);
            const contentGroup = new Phaser.Group(game);
            this.add(contentGroup);
            let playerNamesText = null;
            let playerChanged = this.didPlayerChange();
            ResumePromptPage.METRIC_LABEL = playerChanged ? "NewGroup" : "SameGroup";
            // Only show the names if they've changed (AC)
            if (playerChanged) {
                let title = LocalizationManager_13.LocalizationManager.Instance.text('resume.title');
                const titleText = new Phaser.Text(game, this.game.width * 0.5, 0, title, {
                    font: Game_18.Game.DEFAULT_FONT,
                    fill: Game_18.Game.TEXT_COLOR,
                    fontSize: Game_18.Game.FONT_SIZE_MEDIUM,
                    wordWrap: true,
                    wordWrapWidth: this.game.width * 0.8,
                    align: 'center',
                    stroke: Game_18.Game.TEXT_STROKE_COLOR,
                    strokeThickness: Game_18.Game.TEXT_STROKE_THICKNESS
                });
                titleText.anchor.set(0.5, 0);
                titleText.setShadow(Game_18.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_18.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_18.Game.TEXT_SHADOW_SOFT);
                contentGroup.add(titleText);
                let playerNamesString = GameController_13.GameController.Instance.previousSessionPlayerNames.join("\n");
                playerNamesText = new Phaser.Text(game, this.game.width * 0.5, titleText.bottom + this.game.height * 0.013, playerNamesString, {
                    font: Game_18.Game.DEFAULT_FONT,
                    fill: Game_18.Game.TEXT_COLOR_GREEN,
                    fontSize: Game_18.Game.FONT_SIZE_MEDIUM,
                    stroke: Game_18.Game.TEXT_STROKE_COLOR,
                    strokeThickness: Game_18.Game.TEXT_STROKE_THICKNESS,
                    align: "center"
                });
                playerNamesText.anchor.set(0.5, 0);
                playerNamesText.width = Math.min(playerNamesText.width, game.width * 0.9);
                playerNamesText.scale.y = playerNamesText.scale.x;
                playerNamesText.setShadow(Game_18.Game.TEXT_SHADOW_OFFSET_MED_X, Game_18.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_18.Game.TEXT_SHADOW_SOFT);
                contentGroup.add(playerNamesText);
            }
            let description = LocalizationManager_13.LocalizationManager.Instance.text('resume.description');
            let yPosition = playerNamesText === null ? 0 : playerNamesText.bottom + this.game.height * 0.013;
            const descriptionText = new Phaser.Text(game, this.game.width * 0.5, yPosition, description, {
                font: Game_18.Game.DEFAULT_FONT,
                fill: Game_18.Game.TEXT_COLOR,
                fontSize: Game_18.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                align: 'center',
                stroke: Game_18.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_18.Game.TEXT_STROKE_THICKNESS
            });
            descriptionText.anchor.set(0.5, 0);
            descriptionText.setShadow(Game_18.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_18.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_18.Game.TEXT_SHADOW_SOFT);
            contentGroup.add(descriptionText);
            const bgDetail = new Phaser.Image(this.game, this.game.width * 0.5, descriptionText.bottom, 'outro_background');
            bgDetail.anchor.set(0.5, 0);
            bgDetail.width = this.game.width;
            bgDetail.scale.y = bgDetail.scale.x;
            contentGroup.add(bgDetail);
            let yesText = LocalizationManager_13.LocalizationManager.Instance.text('resume.yes').toUpperCase();
            this.yesButton = new MediumButton_3.MediumButton(this.game, 0, yesText, MediumButton_3.RibbonWidth.LARGE);
            this.yesButton.width = this.game.width * 0.9683673469387755;
            this.yesButton.scale.y = this.yesButton.scale.x;
            this.yesButton.x = bgDetail.centerX;
            this.yesButton.y = bgDetail.top + bgDetail.height * 0.32;
            contentGroup.add(this.yesButton);
            let noText = LocalizationManager_13.LocalizationManager.Instance.text('resume.no').toUpperCase();
            this.noButton = new MediumButton_3.MediumButton(this.game, 1, noText, MediumButton_3.RibbonWidth.LARGE);
            this.noButton.width = this.game.width * 0.9683673469387755;
            this.noButton.scale.y = this.noButton.scale.x;
            this.noButton.x = bgDetail.centerX;
            this.noButton.y = bgDetail.top + bgDetail.height * 0.68;
            contentGroup.add(this.noButton);
            // centers all the content in the middle of the screen
            contentGroup.centerY = this.game.height * 0.5;
            this.confirmDialog = new ConfirmDialog_3.ConfirmDialog(this.game, LocalizationManager_13.LocalizationManager.Instance.text('resume.confirm_dialog.title'), LocalizationManager_13.LocalizationManager.Instance.text('resume.confirm_dialog.text'), LocalizationManager_13.LocalizationManager.Instance.text('resume.confirm_dialog.yes'), LocalizationManager_13.LocalizationManager.Instance.text('resume.confirm_dialog.no'), true);
            this.add(this.confirmDialog);
            if (playerChanged) {
                PPAnalyticsManager_7.PPAnalyticsManager.Instance.trackState('content/game/inqueue/resume/newgroup', 'ResumeNewGroup');
            }
            else {
                PPAnalyticsManager_7.PPAnalyticsManager.Instance.trackState('content/game/inqueue/resume', 'Resume');
            }
        }
        // Returns true if the player names changed
        didPlayerChange() {
            const previous = GameController_13.GameController.Instance.previousSessionPlayerNames;
            const current = GameController_13.GameController.Instance.state.playerNames;
            // Test for length
            if (previous.length !== current.length) {
                return true;
            }
            // Test for the names
            for (let name of previous) {
                if (current.indexOf(name) < 0) {
                    return true;
                }
            }
            return false;
        }
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                // Once showed, set the flag to not ask again.
                GameController_13.GameController.Instance.shouldPromptToResume = false;
                // Show a dialog for confirmation and automatically turn the page when the user chooses yes
                return new Promise((resolve, reject) => {
                    this.noButton.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                        PPAnalyticsManager_7.PPAnalyticsManager.Instance.trackAction("Exit", ResumePromptPage.METRIC_CATEGORY, "NewGame", ResumePromptPage.METRIC_LABEL);
                        // Get them to confirm their 'delete' choice
                        yield this.confirmDialog.show();
                        const confirmed = yield PromiseUtil_6.PromiseUtil.signal(this.confirmDialog.clickedSignal);
                        yield this.confirmDialog.hide();
                        PPAnalyticsManager_7.PPAnalyticsManager.Instance.trackAction(confirmed ? "Exit_Confirm" : "Exit_Cancel", ResumePromptPage.METRIC_CATEGORY, confirmed ? "NewGameConfirm" : "NewGameCancel", ResumePromptPage.METRIC_LABEL);
                        if (confirmed) {
                            // Stop listening
                            // I'm not listening, not anymore...
                            this.yesButton.events.onInputDown.removeAll();
                            this.noButton.events.onInputDown.removeAll();
                            // N.B. We end this timer before we start it. This is because a 
                            // new game can be started here, which should end the
                            // timer on the previous game before starting a new one.
                            PPAnalyticsManager_7.PPAnalyticsManager.Instance.endTiming("OffToNeverLand", "OffToNeverLand", '', '');
                            GameController_13.GameController.Instance.state.numVisits++;
                            PPAnalyticsManager_7.PPAnalyticsManager.Instance.startTiming("OffToNeverLand");
                            // Delete the old savedata so the players can start again
                            yield GameController_13.GameController.Instance.recycleState();
                            // And resolve, they've finished 'answering' this page
                            resolve();
                        }
                    }), this);
                    this.yesButton.events.onInputDown.addOnce(() => {
                        // Stop listening
                        this.yesButton.events.onInputDown.removeAll();
                        this.noButton.events.onInputDown.removeAll();
                        PPAnalyticsManager_7.PPAnalyticsManager.Instance.trackAction("ContinueGame", ResumePromptPage.METRIC_CATEGORY, "ContinueGame", ResumePromptPage.METRIC_LABEL);
                        // And resolve, they've finished 'answering' this page
                        resolve();
                    });
                });
            });
        }
        // As soon as this is set to be turnable, 
        // fake a click - it should turn immediately
        set turnable(turnable) {
            PromiseUtil_6.PromiseUtil.time(this.game, 100).then(() => {
                this.onPageClicked.dispatch();
            });
        }
    }
    ResumePromptPage.METRIC_CATEGORY = "Resume";
    exports.ResumePromptPage = ResumePromptPage;
});
define("logic/sequence-controllers/ResumeGameSequenceController", ["require", "exports", "logic/GameController", "pages/ResumePromptPageData", "logic/PPAnalyticsManager"], function (require, exports, GameController_14, ResumePromptPageData_1, PPAnalyticsManager_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Shows a dialog when the user resumes the game,
     * to choose between continuing or starting a new one
     *
     * @author qi@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class ResumeGameSequenceController {
        /*
        Returns true in case the book should show the sequence for resuming the game.
    
        Done here because it's cleaner for the BookLogic,
        and also to leverage the fact that we are using a class to control the sequence
        */
        shouldShow() {
            // If we had shown this dialog before, no need to show it again
            if (!GameController_14.GameController.Instance.shouldPromptToResume) {
                return false;
            }
            // If the players have a current session going, then we should show the dialog
            if (GameController_14.GameController.Instance.state.sessionStarted) {
                return true;
            }
            else {
                // else we don't need to show it, because it's a new game
                // and also don't try to show the dialog again
                GameController_14.GameController.Instance.shouldPromptToResume = false;
                PPAnalyticsManager_8.PPAnalyticsManager.Instance.endTiming("OffToNeverLand", "OffToNeverLand", "OffToNeverLand", '');
                PPAnalyticsManager_8.PPAnalyticsManager.Instance.startTiming("OffToNeverLand");
                return false;
            }
        }
        getNextSequence() {
            return [
                new ResumePromptPageData_1.ResumePromptPageData().parse({})
            ];
        }
    }
    exports.ResumeGameSequenceController = ResumeGameSequenceController;
});
define("pages/component/TextButton", ["require", "exports", "Game"], function (require, exports, Game_19) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextButton extends Phaser.Group {
        constructor(game, text, buttonKey = "option-button-medium", buttonFrame = 0, fontSize = Game_19.Game.FONT_SIZE_MEDIUM) {
            super(game);
            this.image = new Phaser.Image(game, 0, 0, buttonKey, buttonFrame);
            this.image.anchor.set(0.5);
            this.image.inputEnabled = true;
            this.add(this.image);
            const textField = new Phaser.Text(game, this.image.centerX, this.image.centerY, text, {
                fontSize: fontSize,
                fill: Game_19.Game.TEXT_COLOR_YELLOW,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.85,
                font: Game_19.Game.DEFAULT_FONT
            });
            textField.anchor.set(0.5);
            this.add(textField);
        }
    }
    exports.TextButton = TextButton;
});
define("pages/DrawingPageData", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "pages/Page", "Game", "pages/PageData", "pages/component/TextButton", "switchback-common/util/PromiseUtil", "model/ActivityType", "logic/PPAnalyticsManager", "logic/GameController", "logic/sequence-controllers/MinigameSequenceController", "view/ConfirmDialog"], function (require, exports, TextUtil_16, LocalizationManager_14, Page_14, Game_20, PageData_12, TextButton_1, PromiseUtil_7, ActivityType_3, PPAnalyticsManager_9, GameController_15, MinigameSequenceController_1, ConfirmDialog_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingPageData extends PageData_12.PageData {
        build(game) {
            const drawingPage = new DrawingPage(game);
            drawingPage.isFinalTurnOfGame = this.isFinalTurnOfGame;
            if (drawingPage.isFinalTurnOfGame) {
                drawingPage.onPageClicked.addOnce(() => {
                    MinigameSequenceController_1.MinigameSequenceController.lastGameDisplay = new Date().getTime();
                });
            }
            return [drawingPage];
        }
    }
    exports.DrawingPageData = DrawingPageData;
    class DrawingPage extends Page_14.Page {
        constructor(game) {
            super(game, ActivityType_3.ActivityType.DRAWING, Page_14.PageType.CORNER, "questions.drawing.title", null, false);
            this.drawingAreaWidth = this.game.width * 0.8;
            this.drawingAreaHeight = this.game.height * 0.5;
            this.drawingAreaX = this.game.width * 0.1;
            this.canDraw = false;
            this.instructionText = new Phaser.Text(game, game.width * 0.5, 0, LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.text_2a"), {
                fontSize: TextUtil_16.TextUtil.getFontSize(game.width, 25),
                fill: Game_20.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: false,
                wordWrapWidth: game.width * 0.9,
                font: Game_20.Game.DEFAULT_FONT,
                stroke: Game_20.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_20.Game.TEXT_STROKE_THICKNESS
            });
            this.instructionText.anchor.set(0.5, 0);
            TextUtil_16.TextUtil.scaleDownSingleLineTextToWidth(this.instructionText, game.width * 0.9);
            this.instructionText.y = game.height * 0.15;
            this.add(this.instructionText);
            const optionCount = this.countOptions();
            this.clueIndex = GameController_15.GameController.Instance.state.currentDrawingPromptIndex;
            GameController_15.GameController.Instance.state.currentDrawingPromptIndex = (this.clueIndex % optionCount) + 1;
            this.clueText = new Phaser.Text(game, game.width * 0.5, 0, LocalizationManager_14.LocalizationManager.Instance.text(`questions.drawing.option_${this.clueIndex}_text`), {
                fontSize: TextUtil_16.TextUtil.getFontSize(game.width, 40),
                fill: Game_20.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: false,
                wordWrapWidth: game.width * 0.9,
                font: Game_20.Game.DEFAULT_FONT,
                stroke: Game_20.Game.TEXT_STROKE_COLOR
            });
            this.clueText.anchor.set(0.5);
            TextUtil_16.TextUtil.scaleDownSingleLineTextToWidth(this.clueText, game.width * 0.9);
            this.clueText.top = this.instructionText.bottom;
            this.add(this.clueText);
            this.clearButton = new TextButton_1.TextButton(this.game, LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.clear"), "option-button-medium", 1);
            this.clearButton.width = this.game.width * 0.4;
            this.clearButton.scale.y = this.clearButton.scale.x;
            this.clearButton.centerX = this.game.width * 0.25;
            this.clearButton.centerY = this.game.height * 0.925;
            this.clearButton.onChildInputDown.add(this.onClearClicked, this);
            this.add(this.clearButton);
            this.finishButton = new TextButton_1.TextButton(this.game, LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.complete"), "option-button-medium", 0);
            this.finishButton.scale = this.clearButton.scale;
            this.finishButton.centerX = this.game.width * 0.75;
            this.finishButton.centerY = this.game.height * 0.925;
            this.add(this.finishButton);
            this.bottomText = new Phaser.Text(game, game.width * 0.5, game.height * 0.825, LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.text_2c"), {
                fontSize: TextUtil_16.TextUtil.getFontSize(game.width, 25),
                fill: Game_20.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.9,
                font: Game_20.Game.DEFAULT_FONT
            });
            this.bottomText.anchor.set(0.5, 0);
            TextUtil_16.TextUtil.scaleDownTextToHeight(this.clueText, game.width * 0.1);
            this.bottomText.bottom = this.clearButton.top;
            this.add(this.bottomText);
            // fill as much remaining space as possible with the drawing area
            const drawingYPadding = 20;
            this.drawingAreaY = this.clueText.bottom + drawingYPadding;
            this.drawingAreaHeight = this.bottomText.top - this.clueText.bottom - (drawingYPadding * 2);
            const frame = new Phaser.Graphics(this.game);
            frame.lineStyle(5, 0x000000, 1);
            frame.drawRect(this.drawingAreaX, this.drawingAreaY, this.drawingAreaWidth, this.drawingAreaHeight);
            this.add(frame);
            this.drawingArea = this.game.add.bitmapData(this.game.width, this.game.height);
            this.drawingArea.smoothed = false;
            const img = new Phaser.Image(this.game, 0, 0, this.drawingArea);
            this.add(img);
            this.stamp = this.game.make.sprite(0, 0, 'drawing-stamp');
            this.stamp.width = this.game.width * 0.02;
            this.stamp.scale.y = this.stamp.scale.x;
            this.stamp.anchor.set(0.5);
            this.backgroundImg.inputEnabled = true;
            this.backgroundImg.input.enableDrag();
            this.backgroundImg.input.setDragLock(false, false);
            this.backgroundImg.events.onDragUpdate.add(this.paint, this);
            this.backgroundImg.events.onInputUp.add(this.onPointerUp, this);
        }
        /**
         * Find the number of drawing options that have been provided for this language.
         */
        countOptions() {
            return Object.keys(this.game.cache.getJSON('i18n').questions.drawing).filter((key) => {
                return key.includes('option');
            }).length;
        }
        paint(obj, pointer, x, y) {
            if (pointer.isDown && this.canDraw) {
                if (pointer.x > this.drawingAreaX && pointer.x < this.drawingAreaX + this.drawingAreaWidth) {
                    if (pointer.y > this.drawingAreaY && pointer.y < this.drawingAreaY + this.drawingAreaHeight) {
                        if (!this.prevPoint) {
                            this.drawingArea.draw(this.stamp, pointer.x, pointer.y);
                        }
                        else {
                            this.drawingArea.line(this.prevPoint.x, this.prevPoint.y, pointer.x, pointer.y, 'black', this.game.width * 0.02);
                        }
                        this.prevPoint = pointer.position.clone();
                    }
                }
                else {
                    this.prevPoint = null;
                }
            }
        }
        onClearClicked() {
            this.drawingArea.clear();
        }
        onPointerUp() {
            this.prevPoint = null;
        }
        readyToTurn() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("readyToTurn").call(this);
                this.canDraw = true;
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    // Wait for them to finish
                    yield this.doDrawing();
                    // if the exit sequence has been triggered take the shortest path out of drawing
                    if (GameController_15.GameController.Instance.exitConfirmed) {
                        resolve();
                        return;
                    }
                    // Don't allow them to draw
                    this.canDraw = false;
                    // Then stop listening to the buttons
                    this.clearButton.ignoreChildInput = true;
                    this.finishButton.ignoreChildInput = true;
                    // Fade out the buttons and the text
                    yield Promise.all([
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.clearButton)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.finishButton)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionText)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.clueText)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomText)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                    ]);
                    this.instructionText.text = LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.text_3a");
                    // reset the font size and rescale for the new text
                    this.instructionText.fontSize = TextUtil_16.TextUtil.getFontSize(this.game.width, 25);
                    TextUtil_16.TextUtil.scaleDownSingleLineTextToWidth(this.instructionText, this.game.width * 0.9);
                    this.clueText.text = LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.text_3b");
                    // reset the font size and rescale for the new text
                    this.clueText.fontSize = TextUtil_16.TextUtil.getFontSize(this.game.width, 40);
                    TextUtil_16.TextUtil.scaleDownSingleLineTextToWidth(this.clueText, this.game.width * 0.9);
                    this.bottomText.text = LocalizationManager_14.LocalizationManager.Instance.text("questions.drawing.text_3c") +
                        LocalizationManager_14.LocalizationManager.Instance.text(`questions.drawing.option_${this.clueIndex}_text`);
                    yield Promise.all([
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionText)
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.clueText)
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomText)
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true))
                    ]);
                    // Finally resolve, this page can be turned now
                    resolve();
                }));
            });
        }
        doDrawing() {
            return __awaiter(this, void 0, void 0, function* () {
                let isStillDrawing = true;
                while (isStillDrawing) {
                    // on page clicked is only triggered when the exit sequence has been triggered
                    // in this case we want to take the shortest path out of drawing
                    yield Promise.race([
                        PromiseUtil_7.PromiseUtil.signal(this.finishButton.onChildInputDown),
                        PromiseUtil_7.PromiseUtil.signal(this.onPageClicked)
                    ]);
                    if (GameController_15.GameController.Instance.exitConfirmed) {
                        return;
                    }
                    const confirmDialog = new ConfirmDialog_4.ConfirmDialog(this.game, null, LocalizationManager_14.LocalizationManager.Instance.text('questions.drawing.confirm_dialog.text'), LocalizationManager_14.LocalizationManager.Instance.text('questions.drawing.confirm_dialog.yes'), LocalizationManager_14.LocalizationManager.Instance.text('questions.drawing.confirm_dialog.no'), true);
                    // show over the trinkets or exit button so they can't be clicked
                    this.game.add.existing(confirmDialog);
                    yield confirmDialog.show();
                    isStillDrawing = !(yield PromiseUtil_7.PromiseUtil.signal(confirmDialog.clickedSignal));
                    yield confirmDialog.hide();
                    confirmDialog.destroy();
                }
            });
        }
        doStartAnalytics() {
            const stateNameString = 'content/game/inqueue/minigame';
            this.analyticsKey = this.activityType.toString();
            this.analyticsTimingKey = 'OffToNeverLand:Mini:DrawingSolo';
            const pageActivityTypePlays = GameController_15.GameController.Instance.state.incrementActivityPlays(this.analyticsKey);
            this.analyticsData = {
                [PPAnalyticsManager_9.PPAnalyticsManager.GAME_NAME]: this.analyticsKey,
                [PPAnalyticsManager_9.PPAnalyticsManager.GAME_PLAYS]: pageActivityTypePlays,
                [PPAnalyticsManager_9.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: this.clueIndex
            };
            PPAnalyticsManager_9.PPAnalyticsManager.Instance.trackState(stateNameString, this.activityType.toString(), this.analyticsData);
            // N.B The page with actual drawing doesn't start the analytics timer, but it does end it
        }
        doEndAnalytics() {
            PPAnalyticsManager_9.PPAnalyticsManager.Instance.endTiming(this.analyticsTimingKey, this.analyticsKey, 'Activity', 'SoloPlay', this.analyticsData);
            // End the timer for the whole game if this was the final turn
            if (this.isFinalTurnOfGame) {
                PPAnalyticsManager_9.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Mini:Drawing', this.analyticsKey, 'Activity', 'AllPlay', this.analyticsData);
            }
        }
        destroy() {
            this.drawingArea.destroy();
            super.destroy();
        }
    }
    exports.DrawingPage = DrawingPage;
});
define("pages/DrawingIntroPageData", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "pages/Page", "Game", "pages/PageData", "logic/GameController", "pages/PageUtility", "model/ActivityType", "logic/PPAnalyticsManager"], function (require, exports, ScaleUtil_16, TextUtil_17, LocalizationManager_15, Page_15, Game_21, PageData_13, GameController_16, PageUtility_4, ActivityType_4, PPAnalyticsManager_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingIntroPageData extends PageData_13.PageData {
        build(game) {
            return [new DrawingIntroPage(game)];
        }
    }
    exports.DrawingIntroPageData = DrawingIntroPageData;
    class DrawingIntroPage extends Page_15.Page {
        constructor(game) {
            super(game, ActivityType_4.ActivityType.DRAWING, Page_15.PageType.CORNER, "questions.drawing.title", null, false);
            this.titleText.top = game.height * 0.02;
            const passPromptText = PageUtility_4.PageUtility.createPassToPlayerText(this.game);
            passPromptText.centerY = game.height * 0.25;
            this.add(passPromptText);
            const picture = new Phaser.Image(game, 0, 0, 'drawing-title');
            ScaleUtil_16.scaleToBounds(picture, game.width * 0.9, game.height * 0.3);
            picture.centerX = game.width * 0.5;
            picture.centerY = game.height * 0.5;
            this.add(picture);
            const textBottom = new Phaser.Text(game, 0, 0, LocalizationManager_15.LocalizationManager.Instance.text("questions.drawing.text_1a"), {
                fontSize: TextUtil_17.TextUtil.getFontSize(game.width, 30),
                fill: Game_21.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.9,
                font: Game_21.Game.DEFAULT_FONT
            });
            textBottom.centerX = game.width * 0.5;
            textBottom.centerY = game.height * 0.75;
            this.add(textBottom);
        }
        doStartAnalytics() {
            const stateNameString = 'content/game/inqueue/minigame';
            this.analyticsKey = this.activityType.toString();
            this.analyticsTimingKey = 'OffToNeverLand:Mini:Drawing';
            const pageActivityTypePlays = GameController_16.GameController.Instance.state.incrementActivityPlays(this.analyticsKey);
            this.analyticsData = {
                [PPAnalyticsManager_10.PPAnalyticsManager.GAME_NAME]: this.analyticsKey,
                [PPAnalyticsManager_10.PPAnalyticsManager.GAME_PLAYS]: pageActivityTypePlays
            };
            PPAnalyticsManager_10.PPAnalyticsManager.Instance.trackState(stateNameString, 'Drawing', this.analyticsData);
            // N.B This page doesn't end the analytics timer, but it does start it
            PPAnalyticsManager_10.PPAnalyticsManager.Instance.startTiming(this.analyticsTimingKey, this.analyticsData);
        }
        doEndAnalytics() {
            // None
        }
    }
    exports.DrawingIntroPage = DrawingIntroPage;
});
define("pages/DrawingTitlePage", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "pages/Page", "pages/PageData", "logic/GameController", "view/MediumButton", "logic/sequence-controllers/MinigameSequenceController", "switchback-common/util/PromiseUtil", "model/ActivityType", "logic/PPAnalyticsManager"], function (require, exports, ScaleUtil_17, LocalizationManager_16, Page_16, PageData_14, GameController_17, MediumButton_4, MinigameSequenceController_2, PromiseUtil_8, ActivityType_5, PPAnalyticsManager_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingTitlePageData extends PageData_14.PageData {
        build(game) {
            return [new DrawingTitlePage(game)];
        }
    }
    exports.DrawingTitlePageData = DrawingTitlePageData;
    class DrawingTitlePage extends Page_16.Page {
        constructor(game) {
            super(game, ActivityType_5.ActivityType.DRAWING_PROMPT, Page_16.PageType.CORNER, "questions.drawing.title", null, false);
            this.titleText.top = game.height * 0.02;
            this.addText(this.game, LocalizationManager_16.LocalizationManager.Instance.text("questions.drawing.text_1"), 0.85, 0.2, 40);
            this.text.top = this.titleText.bottom;
            const bgDetail = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.8, 'outro_background');
            bgDetail.anchor.set(0.5);
            ScaleUtil_17.scaleToBounds(bgDetail, this.game.width * 0.95, this.game.height * 0.35);
            this.add(bgDetail);
            const picture = new Phaser.Image(game, 0, 0, 'drawing-title');
            ScaleUtil_17.scaleToBounds(picture, game.width * 0.9, bgDetail.top - this.text.bottom);
            picture.centerX = game.width * 0.5;
            picture.centerY = this.text.bottom + (bgDetail.top - this.text.bottom) * 0.5;
            this.add(picture);
            this.yesButton = new MediumButton_4.MediumButton(this.game, 0, LocalizationManager_16.LocalizationManager.Instance.text("button.play"));
            this.yesButton.x = bgDetail.centerX;
            this.yesButton.y = bgDetail.top + bgDetail.height * 0.32;
            this.add(this.yesButton);
            this.noButton = new MediumButton_4.MediumButton(this.game, 1, LocalizationManager_16.LocalizationManager.Instance.text("button.later"));
            this.noButton.x = bgDetail.centerX;
            this.noButton.y = bgDetail.top + bgDetail.height * 0.68;
            this.add(this.noButton);
        }
        // Contains some edge cases around converting 'DrawingPrompt' -> 'Drawing'
        // for adobe analytics
        doStartAnalytics() {
            const stateNameString = 'content/game/inqueue/minigame/title';
            this.analyticsKey = this.activityType.toString();
            this.analyticsTimingKey = 'OffToNeverLand:Mini:Drawing';
            const pageActivityTypePlays = GameController_17.GameController.Instance.state.incrementActivityPlays(this.analyticsKey);
            this.analyticsData = {
                [PPAnalyticsManager_11.PPAnalyticsManager.GAME_NAME]: 'Drawing',
                [PPAnalyticsManager_11.PPAnalyticsManager.GAME_PLAYS]: pageActivityTypePlays
            };
            PPAnalyticsManager_11.PPAnalyticsManager.Instance.trackState(stateNameString, 'Drawing', this.analyticsData);
        }
        doEndAnalytics() {
            // None
        }
        readyToTurn() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("readyToTurn").call(this);
                return new Promise((resolve, reject) => {
                    this.yesButton.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                        this.noButton.events.onInputDown.removeAll();
                        MinigameSequenceController_2.MinigameSequenceController.confirmedMinigamePrompt = MinigameSequenceController_2.MinigamePromptStatus.ACCEPTED;
                        PPAnalyticsManager_11.PPAnalyticsManager.Instance.trackAction("Play_Confirm", "ActivitySelect", "Yes", "Drawing");
                        PPAnalyticsManager_11.PPAnalyticsManager.Instance.startTiming(this.analyticsTimingKey, this.analyticsData);
                        resolve();
                    }), this);
                    this.noButton.events.onInputDown.addOnce(() => {
                        this.yesButton.events.onInputDown.removeAll();
                        MinigameSequenceController_2.MinigameSequenceController.confirmedMinigamePrompt = MinigameSequenceController_2.MinigamePromptStatus.NOT_ACCEPTED;
                        MinigameSequenceController_2.MinigameSequenceController.lastGameDisplay = new Date().getTime();
                        PPAnalyticsManager_11.PPAnalyticsManager.Instance.trackAction("Play_Cancel", "ActivitySelect", "No", "Drawing");
                        resolve();
                    });
                });
            });
        }
        // As soon as this is set to be turnable, 
        // fake a click - it should turn immediately
        set turnable(turnable) {
            PromiseUtil_8.PromiseUtil.time(this.game, 100).then(() => {
                this.onPageClicked.dispatch();
            });
        }
    }
    exports.DrawingTitlePage = DrawingTitlePage;
});
define("logic/sequence-controllers/MinigameSequenceController", ["require", "exports", "logic/GameController", "pages/DrawingPageData", "pages/DrawingIntroPageData", "pages/DrawingTitlePage"], function (require, exports, GameController_18, DrawingPageData_1, DrawingIntroPageData_1, DrawingTitlePage_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var MinigamePromptStatus;
    (function (MinigamePromptStatus) {
        MinigamePromptStatus[MinigamePromptStatus["DONT_ASK"] = 0] = "DONT_ASK";
        MinigamePromptStatus[MinigamePromptStatus["NOT_ASKED"] = 1] = "NOT_ASKED";
        MinigamePromptStatus[MinigamePromptStatus["NOT_ACCEPTED"] = 2] = "NOT_ACCEPTED";
        MinigamePromptStatus[MinigamePromptStatus["ACCEPTED"] = 3] = "ACCEPTED";
    })(MinigamePromptStatus = exports.MinigamePromptStatus || (exports.MinigamePromptStatus = {}));
    /**
     * @author joel@cerebralfix.com
     */
    class MinigameSequenceController {
        constructor() {
            this.setsSinceLastPrompt = MinigameSequenceController.SETS_BETWEEN_PROMPT_THRESHOLD;
            this.maxPromptFrequencyMs = 300000;
        }
        getNextSequence() {
            console.log("[MinigameSequenceController] getNextSequence()");
            const drawingGame = [];
            for (let i = 0; i < GameController_18.GameController.Instance.players.length; i++) {
                drawingGame.push(new DrawingIntroPageData_1.DrawingIntroPageData().parse({}));
                drawingGame.push(new DrawingPageData_1.DrawingPageData().parse({}));
                // If this is the last page of the minigame, let it know.
                if (i === GameController_18.GameController.Instance.players.length - 1) {
                    drawingGame[drawingGame.length - 1].isFinalTurnOfGame = true;
                }
            }
            return drawingGame;
        }
        getConfirmationSequence() {
            return [new DrawingTitlePage_1.DrawingTitlePageData().parse({})];
        }
        canShowPrompt() {
            const setCheck = this.setsSinceLastPrompt > MinigameSequenceController.SETS_BETWEEN_PROMPT_THRESHOLD;
            const promptCheck = MinigameSequenceController.confirmedMinigamePrompt === MinigamePromptStatus.NOT_ASKED;
            const timeCheck = (MinigameSequenceController.lastGameDisplay === 0 || new Date().getTime() > MinigameSequenceController.lastGameDisplay + this.maxPromptFrequencyMs);
            return setCheck && promptCheck && timeCheck;
        }
    }
    MinigameSequenceController.lastGameDisplay = 0;
    MinigameSequenceController.SETS_BETWEEN_PROMPT_THRESHOLD = 2;
    MinigameSequenceController.confirmedMinigamePrompt = MinigamePromptStatus.DONT_ASK;
    exports.MinigameSequenceController = MinigameSequenceController;
});
define("logic/BookLogic", ["require", "exports", "logic/sequence-controllers/BookSequenceController", "logic/GameController", "logic/sequence-controllers/OutroSequenceController", "logic/sequence-controllers/IntroSequenceController", "logic/sequence-controllers/ResumeGameSequenceController", "switchback-common/enum/Park", "model/PeterPanConfig", "logic/sequence-controllers/MinigameSequenceController"], function (require, exports, BookSequenceController_1, GameController_19, OutroSequenceController_1, IntroSequenceController_1, ResumeGameSequenceController_1, Park_4, PeterPanConfig_3, MinigameSequenceController_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class is responsible for deciding what kind of content to
     * show to the players next.
     *
     * @author joel@cerebralfix.com
     */
    class BookLogic {
        constructor(bookJson) {
            this.activitiesSinceExitConfirmPrompt = 0;
            this.timeSinceLastExitReminder = 0;
            BookLogic.bookSequenceController = new BookSequenceController_1.BookSequenceController(bookJson);
            this.introSequenceController = new IntroSequenceController_1.IntroSequenceController();
            this.outroSequenceController = new OutroSequenceController_1.OutroSequenceController();
            this.resumeGameSequenceController = new ResumeGameSequenceController_1.ResumeGameSequenceController();
            this.minigameSequenceController = new MinigameSequenceController_3.MinigameSequenceController();
        }
        /**
         * This is perhaps one of the most important functions in this game.
         *
         * Basically, it performs a bunch of checks on some flags (e.g. hasSeenIntro, shouldExit)
         * and from those it determines what content should appear next in the book.
         *
         * Obviously, the checks at the top are higher priority as they all return immediately
         * when their condition is met. So intro/outro sequences take presedence over other things.
         */
        getNextSequence() {
            console.log('[BookLogic] getNextSequence()');
            if (this.resumeGameSequenceController.shouldShow()) {
                return this.resumeGameSequenceController.getNextSequence();
            }
            // Check has seen Intro and show intro
            if (!GameController_19.GameController.Instance.state.hasSeenIntro) {
                GameController_19.GameController.Instance.state.hasSeenIntro = true;
                GameController_19.GameController.Instance.state.sessionStarted = true;
                return this.introSequenceController.getNextSequence();
            }
            // Have they selected a queue type? Do they need to?
            if (GameController_19.GameController.Instance.state.selectedQueueType === undefined &&
                PeterPanConfig_3.PeterPanConfig.Instance.park === Park_4.Park.DLR) {
                return this.introSequenceController.getQueueSelectSequence();
            }
            // TODO: Check for BT events and show trigger pages
            // Show Outro if:
            // We've definitely confirmed showing the outro OR
            // We've received a beacon and need to ask the user to confirm AND we haven't shown a confirm too recently
            if (GameController_19.GameController.Instance.exitConfirmed) {
                return this.outroSequenceController.getNextSequence();
            }
            if (GameController_19.GameController.Instance.exitTriggered &&
                GameController_19.GameController.Instance.reachedTrinketRoom &&
                this.activitiesSinceExitConfirmPrompt >= BookLogic.ACTIVITIES_BETWEEN_CHECK_EXIT_CONFIRM) {
                const timeNow = new Date().getTime();
                if (timeNow > this.timeSinceLastExitReminder + BookLogic.TIME_BETWEEN_EXIT_REMINDERS) {
                    this.activitiesSinceExitConfirmPrompt = 0;
                    this.timeSinceLastExitReminder = timeNow;
                    return this.outroSequenceController.getNextSequence();
                }
            }
            // Shows a minigame prompt if they're due for it. If they accept the prompt, we'll get the
            // game content next time. If they decline it, we'll move on
            if (this.minigameSequenceController.canShowPrompt()) {
                return this.minigameSequenceController.getConfirmationSequence();
            }
            else if (MinigameSequenceController_3.MinigameSequenceController.confirmedMinigamePrompt === MinigameSequenceController_3.MinigamePromptStatus.ACCEPTED) {
                MinigameSequenceController_3.MinigameSequenceController.confirmedMinigamePrompt = MinigameSequenceController_3.MinigamePromptStatus.DONT_ASK;
                return this.minigameSequenceController.getNextSequence();
            }
            // If all else fails, show an activity
            return BookLogic.bookSequenceController.getNextSequence();
        }
        /**
         * When an activity is completed ok the BookState,
         * this function is called. It moves the SaveState's index
         * along and helps inform when we should show a TT page.
         */
        completedActivity() {
            this.activitiesSinceExitConfirmPrompt++;
            GameController_19.GameController.Instance.state.numActivitiesCompleted++;
        }
        // Returns a debug string for the current set/sequence/theme
        getCurrentDebugString() {
            let state = GameController_19.GameController.Instance.state;
            return "Set: " + state.currentSetID + "\nActivity: " + state.currentActivityIndex + "\nTheme: " + BookLogic.bookSequenceController.getCurrentTheme();
        }
    }
    BookLogic.ACTIVITIES_BETWEEN_CHECK_EXIT_CONFIRM = 4;
    BookLogic.TIME_BETWEEN_EXIT_REMINDERS = 8 * 60 * 1000; // 8 minutes
    exports.BookLogic = BookLogic;
});
define("pages/Page", ["require", "exports", "Game", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "logic/GameController", "pages/component/HeaderText", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "model/ActivityType", "logic/PPAnalyticsManager", "logic/BookLogic", "state/BookState"], function (require, exports, Game_22, PromiseUtil_9, TextUtil_18, GameController_20, HeaderText_1, LocalizationManager_17, ScaleUtil_18, ActivityType_6, PPAnalyticsManager_12, BookLogic_1, BookState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     * @author joel@cerebralfix.com
     */
    var TurnDirection;
    (function (TurnDirection) {
        TurnDirection[TurnDirection["LEFT"] = 0] = "LEFT";
        TurnDirection[TurnDirection["RIGHT"] = 1] = "RIGHT";
    })(TurnDirection = exports.TurnDirection || (exports.TurnDirection = {}));
    var PageType;
    (function (PageType) {
        PageType[PageType["COVER"] = 0] = "COVER";
        PageType[PageType["CORNER"] = 1] = "CORNER";
        PageType[PageType["NO_CORNER"] = 2] = "NO_CORNER";
    })(PageType = exports.PageType || (exports.PageType = {}));
    /**
     * This PageResult class can be dispatched by the pageResultSignal on all Pages.
     * It contains information that things might be listening for such as
     * responses to questions, completed pages etc.
     */
    class PageResult {
    }
    exports.PageResult = PageResult;
    class Page extends Phaser.Group {
        constructor(game, activityType, type, titleString, text, isFinalActivityPage, showQuit = true) {
            super(game);
            this.isTimerRunning = false;
            this.timeOnPageMs = 0;
            this.parent.removeChild(this);
            this.activityType = activityType;
            this.showQuit = showQuit;
            this.isFinalActivityPage = isFinalActivityPage;
            this.isBusy = false;
            this.onPageClicked = new Phaser.Signal();
            this.pageResultSignal = new Phaser.Signal();
            this.backgroundImg = new Phaser.Image(this.game, 0, 0, "page-with-border");
            this.backgroundImg.width = this.game.width;
            this.backgroundImg.height = this.game.height;
            this.pageBend = new Phaser.Image(this.game, 0, 0, "page-bend");
            this.pageBend.anchor.set(0);
            this.pageBend.width = this.game.width;
            this.pageBend.scale.y = this.pageBend.scale.x;
            this.pageBend.top = this.game.height;
            this.pageBend.right = this.game.width;
            this.game.add.existing(this.pageBend);
            this.pageGroup = new Phaser.Group(game);
            this.pageGroup.add(this.backgroundImg);
            this.add(this.pageGroup);
            if (type === PageType.CORNER) {
                this.pageCorner = new Phaser.Image(this.game, 0, 0, "page-corner");
                this.pageCorner.anchor.set(1);
                ScaleUtil_18.scaleToBounds(this.pageCorner, this.game.width * 0.5, this.game.height * 0.5);
                this.pageCornerScale = this.pageCorner.scale.x;
                this.pageCorner.width = 0;
                this.pageCorner.height = 0;
                this.pageCorner.bottom = this.game.height;
                this.pageCorner.right = this.game.width;
                this.pageCorner.inputEnabled = true;
                this.add(this.pageCorner);
                this.pageCorner.events.onInputDown.add((target, pointer) => {
                    if (!this.isBusy) { // disallows pages from turning if the page is busy with animations
                        this.onPageClicked.dispatch();
                    }
                });
            }
            ////////////////////
            // Main text
            ////////////////////
            this.shade = new Phaser.Graphics(this.game, 0, 0);
            this.shade.beginFill(0x000000, 0.75);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.alpha = 0;
            this.shade.inputEnabled = false;
            this.add(this.shade);
            this.add = this._add;
            ////////////////////
            // Main title & text
            ////////////////////
            if (titleString && titleString !== "") {
                this.titleText = new HeaderText_1.HeaderText(this.game, LocalizationManager_17.LocalizationManager.Instance.text(titleString));
                this.titleText.centerX = game.width * 0.5;
                this.titleText.top = game.height * 0.005;
                this.add(this.titleText);
            }
            if (text && text !== "") {
                this.addText(game, LocalizationManager_17.LocalizationManager.Instance.text(text));
            }
        }
        get isFinalPageOfActivity() {
            return this.isFinalActivityPage;
        }
        addText(game, text, textWidthPercent = 0.85, textHeightPercent = 0.3, fontSize = 40) {
            this.text = new Phaser.Text(game, 0, 0, text, {
                fontSize: TextUtil_18.TextUtil.getFontSize(game.width, fontSize),
                fill: Game_22.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * textWidthPercent,
                font: Game_22.Game.DEFAULT_FONT,
                stroke: Game_22.Game.TEXT_STROKE_COLOR
            });
            TextUtil_18.TextUtil.scaleDownTextToHeight(this.text, game.height * textHeightPercent);
            this.text.centerX = game.width * 0.5;
            this.text.centerY = game.height * 0.45;
            this.add(this.text);
        }
        _add(object, silent, index) {
            this.pageGroup.add(object, silent, index);
        }
        /**
         * Resolves immediately in most cases - subclasses can
         * override this and determine when they want to be turnable
         */
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                this.doStartAnalytics();
                this.isTimerRunning = true;
            });
        }
        doStartAnalytics() {
            // Do an analytics call here for most activities
            // Any edge cases will override this function
            if (this.activityType && this.pageIndexWithinActivity === 0) {
                const stateNameString = this.activityType === ActivityType_6.ActivityType.DRAWING_PROMPT ?
                    'content/game/inqueue/minigame/title' : 'content/game/inqueue/minigame';
                this.analyticsKey = this.activityType === ActivityType_6.ActivityType.DRAWING_PROMPT ?
                    "DrawingPrompt" : this.activityType.toString();
                this.analyticsTimingKey = this.activityType === ActivityType_6.ActivityType.DRAWING ?
                    'OffToNeverLand:Mini:DrawingSolo' : `OffToNeverLand:Mini:${this.analyticsKey}`;
                const pageTheme = BookLogic_1.BookLogic.bookSequenceController.getThemeForSetId(GameController_20.GameController.Instance.state.currentSetID);
                const pageThemePlays = GameController_20.GameController.Instance.state.incrementActivityPlays(pageTheme);
                const pageActivityTypePlays = GameController_20.GameController.Instance.state.incrementActivityPlays(this.analyticsKey);
                this.analyticsData = {
                    [PPAnalyticsManager_12.PPAnalyticsManager.GAME_NAME]: this.analyticsKey,
                    [PPAnalyticsManager_12.PPAnalyticsManager.GAME_PLAYS]: pageActivityTypePlays
                };
                // Non-drawing pages have the activity and set data tracked
                if (this.activityType !== ActivityType_6.ActivityType.DRAWING && this.activityType !== ActivityType_6.ActivityType.DRAWING_PROMPT) {
                    this.analyticsData = Object.assign({}, this.analyticsData, { [PPAnalyticsManager_12.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_20.GameController.Instance.state.currentSetID, [PPAnalyticsManager_12.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_20.GameController.Instance.state.currentActivityIndex, [PPAnalyticsManager_12.PPAnalyticsManager.ACTIVITY_SET_THEME]: pageTheme, [PPAnalyticsManager_12.PPAnalyticsManager.ACTIVITY_SET_THEME_COMPLETED]: pageThemePlays });
                }
                PPAnalyticsManager_12.PPAnalyticsManager.Instance.trackState(stateNameString, this.activityType.toString(), this.analyticsData);
                if (this.activityType === ActivityType_6.ActivityType.DRAWING) {
                    PPAnalyticsManager_12.PPAnalyticsManager.Instance.startTiming(this.analyticsTimingKey, Object.assign({}, this.analyticsData, { [PPAnalyticsManager_12.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: '' }));
                }
                else {
                    PPAnalyticsManager_12.PPAnalyticsManager.Instance.startTiming(this.analyticsTimingKey, this.analyticsData);
                }
            }
        }
        doEndAnalytics() {
            PPAnalyticsManager_12.PPAnalyticsManager.Instance.endTiming(this.analyticsTimingKey, this.analyticsKey, 'Activity', 'AllPlay', this.analyticsData);
        }
        turn(direction) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play(this.game.rnd.pick(['pageturn1', 'pageturn2', 'pageturn3']));
                this.shade.inputEnabled = true;
                direction === TurnDirection.LEFT ? yield this.doTurnLeft() : yield this.doTurnRight();
                this.shade.inputEnabled = false;
                GameController_20.GameController.Instance.achievementManager.onActivityCompleted(this.layoutType, this.timeOnPageMs);
                this.doEndAnalytics();
            });
        }
        doTurnLeft() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.pageCorner) {
                    this.game.tweens.removeFrom(this.pageCorner.scale);
                    this.game.add.tween(this.pageCorner.scale).to({ x: 0, y: 0 }, 150, Phaser.Easing.Sinusoidal.Out, true)
                        .onComplete.add(() => {
                        this.pageCorner.destroy();
                    });
                }
                this.game.add.tween(this.pageBend).to({ bottom: this.game.height }, BookState_1.BookState.TURN_ANIMATION_DURATION_MS, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.pageBend.scale).to({ x: 0 }, BookState_1.BookState.TURN_ANIMATION_DURATION_MS, Phaser.Easing.Sinusoidal.In, true);
                const flip = this.game.add.tween(this.scale).to({ x: 0 }, BookState_1.BookState.TURN_ANIMATION_DURATION_MS, Phaser.Easing.Sinusoidal.In, true, 0);
                return PromiseUtil_9.PromiseUtil.tween(this.game, flip);
            });
        }
        doTurnRight() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.pageBend).to({ bottom: this.game.height }, 150, Phaser.Easing.Linear.None, true);
                const flip = this.game.add.tween(this.scale).to({ x: 1 }, 500, Phaser.Easing.Sinusoidal.Out, true);
                return PromiseUtil_9.PromiseUtil.tween(this.game, flip);
            });
        }
        // Enables or disabled input on the page. This function also makes the page background display/hide the turnable corner
        set turnable(turnable) {
            this.backgroundImg.inputEnabled = turnable;
            // Move the page up or down (to hide or show the turning corner) depending on whether the page is turnable or not
            if (turnable) {
                this.game.add.tween(this.pageCorner.scale).to({ x: this.pageCornerScale, y: this.pageCornerScale }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0)
                    .onComplete.add(() => {
                    this.game.add.tween(this.pageCorner.scale).to({ x: this.pageCornerScale * 0.8, y: this.pageCornerScale * 0.8 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                });
            }
            else if (this.pageCorner) {
                this.game.tweens.removeFrom(this.pageCorner.scale);
                this.pageCorner.top = this.game.height;
            }
        }
        // derived classes should override this function
        // (This is probably a stupid idea)
        set clickable(clickable) { }
        update() {
            super.update();
            if (this.isTimerRunning) {
                this.timeOnPageMs += this.game.time.elapsedMS;
            }
        }
        destroy() {
            this.onPageClicked.dispose();
            this.pageResultSignal.dispose();
            super.destroy();
        }
    }
    exports.Page = Page;
});
define("pages/PageData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PageData {
        constructor() {
            this.isFinalActivityPage = true;
        }
        parse(data) {
            if (data) {
                this.text = data.text_key || "";
                this.titleString = data.title_key || "";
                this.isFinalActivityPage = data.is_final_activity_page !== undefined ? data.is_final_activity_page : true;
                this.activityType = data.activity_type;
            }
            this.setID = 0;
            this.themeID = '';
            return this;
        }
        set isFinalPageOfActivity(finalPage) {
            this.isFinalActivityPage = finalPage;
        }
        get isFinalPageOfActivity() {
            return this.isFinalActivityPage;
        }
    }
    exports.PageData = PageData;
});
define("enum/TurnDirection", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var TurnDirection;
    (function (TurnDirection) {
        TurnDirection[TurnDirection["LEFT"] = 0] = "LEFT";
        TurnDirection[TurnDirection["RIGHT"] = 1] = "RIGHT";
    })(TurnDirection = exports.TurnDirection || (exports.TurnDirection = {}));
});
define("pages/Cover", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Cover extends Phaser.Group {
        constructor(game) {
            super(game);
            this.onPageClicked = new Phaser.Signal();
            this.backgroundImg = new Phaser.Image(this.game, 0, 0, "cover");
            this.backgroundImg.width = this.game.width;
            this.backgroundImg.height = this.game.height;
            this.backgroundImg.events.onInputDown.add((target, pointer) => {
                this.onPageClicked.dispatch();
            }, this);
            this.pageGroup = new Phaser.Group(game);
            this.pageGroup.add(this.backgroundImg);
            this.add(this.pageGroup);
        }
        /**
         * Resolves immediately in most cases - subclasses can
         * override this and determine when they want to be turnable
         */
        readyToTurn() {
            return __awaiter(this, void 0, void 0, function* () { });
        }
        turn(direction) {
            return __awaiter(this, void 0, void 0, function* () {
                const tweenDuration = 1400;
                const pageBend = new Phaser.Image(this.game, 0, 0, "page-bend");
                pageBend.anchor.set(0);
                pageBend.width = this.game.width;
                pageBend.scale.y = pageBend.scale.x;
                pageBend.top = this.game.height;
                pageBend.right = this.game.width;
                this.add(pageBend);
                this.game.add.tween(pageBend).to({ bottom: this.game.height }, tweenDuration, Phaser.Easing.Sinusoidal.In, true);
                const flip = this.game.add.tween(this.scale).to({ x: 0 }, tweenDuration, Phaser.Easing.Sinusoidal.In, true);
                return PromiseUtil_10.PromiseUtil.tween(this.game, flip);
            });
        }
        // Enables or disabled input on the page. This function also makes the page background display/hide the turnable corner
        set turnable(turnable) {
            this.backgroundImg.inputEnabled = turnable;
        }
    }
    exports.Cover = Cover;
});
define("view/DebugPanel", ["require", "exports", "switchback-common/util/ScaleUtil", "logic/GameController", "model/PeterPanConfig", "switchback-common/enum/Park"], function (require, exports, ScaleUtil_19, GameController_21, PeterPanConfig_4, Park_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class DebugPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            this.showLanternOverlaySignal = new Phaser.Signal();
            const paddingX = this.game.width * 0.025;
            const debugButtonGroup = new Phaser.Group(this.game);
            debugButtonGroup.visible = false;
            this.add(debugButtonGroup);
            const debugToggle = new Phaser.Button(this.game, 0, 0, 'plus', () => {
                debugButtonGroup.visible = !debugButtonGroup.visible;
            }, this);
            ScaleUtil_19.scaleToBounds(debugToggle, this.game.width * 0.125, this.game.height * 0.1);
            this.add(debugToggle);
            const clearSaveDebugButton = new Phaser.Button(this.game, debugToggle.right + paddingX, 0, 'button-basic', () => {
                GameController_21.GameController.Instance.recycleState();
            }, this, 0, 0, 1, 0);
            ScaleUtil_19.scaleToBounds(clearSaveDebugButton, this.game.width * 0.15, this.game.height * 0.1);
            debugButtonGroup.add(clearSaveDebugButton);
            const clearSaveDebugLabel = new Phaser.Text(this.game, clearSaveDebugButton.centerX, clearSaveDebugButton.centerY, 'CLEAR');
            clearSaveDebugLabel.anchor.set(0.5);
            debugButtonGroup.add(clearSaveDebugLabel);
            if (PeterPanConfig_4.PeterPanConfig.Instance.park === Park_5.Park.WDW) {
                const outroDebugButton = new Phaser.Button(this.game, clearSaveDebugButton.right + paddingX, 0, 'button-basic', () => {
                    GameController_21.GameController.Instance.exitTriggered = true;
                }, this, 0, 0, 1, 0);
                ScaleUtil_19.scaleToBounds(outroDebugButton, this.game.width * 0.15, this.game.height * 0.1);
                debugButtonGroup.add(outroDebugButton);
                const outroDebugLabel = new Phaser.Text(this.game, outroDebugButton.centerX, outroDebugButton.centerY, 'OUTRO');
                outroDebugLabel.anchor.set(0.5);
                debugButtonGroup.add(outroDebugLabel);
            }
        }
    }
    exports.DebugPanel = DebugPanel;
});
define("view/PageTopButton", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_20, PromiseUtil_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ButtonColor;
    (function (ButtonColor) {
        ButtonColor[ButtonColor["GREEN"] = 0] = "GREEN";
        ButtonColor[ButtonColor["RED"] = 1] = "RED";
    })(ButtonColor = exports.ButtonColor || (exports.ButtonColor = {}));
    class PageTopButton extends Phaser.Group {
        constructor(game, buttonColor) {
            super(game);
            this.clickedSignal = new Phaser.Signal();
            this.shadow = new Phaser.Image(game, 0, 0, 'button-corner-shadow');
            this.shadow.anchor.set(0.5, 0);
            this.shadow.alpha = 0.5;
            this.add(this.shadow);
            const buttonImgKey = (buttonColor === ButtonColor.RED) ? 'button-corner-red' : 'button-corner-green';
            this.foregroundImage = new Phaser.Image(game, 0, 0, buttonImgKey);
            this.foregroundImage.anchor.set(0.5, 0);
            this.add(this.foregroundImage);
            this.foregroundImage.inputEnabled = true;
            this.foregroundImage.events.onInputDown.add(() => {
                this.clickedSignal.dispatch();
            });
            this.iconGroup = new Phaser.Group(this.game);
            this.add(this.iconGroup);
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.foregroundImage.scale).to({ y: 0 }, PageTopButton.ANIMATION_DURATION, Phaser.Easing.Cubic.Out, true)),
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.iconGroup.scale).to({ y: 0 }, PageTopButton.ANIMATION_DURATION, Phaser.Easing.Cubic.Out, true)),
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.shadow).to({ alpha: 0 }, PageTopButton.ANIMATION_DURATION * 0.5, Phaser.Easing.Linear.None, true))
                ]);
            });
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.foregroundImage.scale).to({ y: 1 }, PageTopButton.ANIMATION_DURATION, Phaser.Easing.Cubic.In, true)),
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.iconGroup.scale).to({ y: 1 }, PageTopButton.ANIMATION_DURATION, Phaser.Easing.Cubic.In, true)),
                    PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.shadow).to({ alpha: 0.5 }, PageTopButton.ANIMATION_DURATION, Phaser.Easing.Linear.None, true))
                ]);
            });
        }
    }
    PageTopButton.ANIMATION_DURATION = 250;
    exports.PageTopButton = PageTopButton;
    class QuitButton extends PageTopButton {
        constructor(game) {
            super(game, ButtonColor.RED);
            const quitImg = new Phaser.Image(this.game, 0, 0, 'button-icon-quit');
            ScaleUtil_20.scaleToBounds(quitImg, this.foregroundImage.width * 0.8, this.foregroundImage.height * 0.8);
            quitImg.anchor.x = 0.5;
            this.iconGroup.add(quitImg);
        }
    }
    exports.QuitButton = QuitButton;
    class TrinketsButton extends PageTopButton {
        constructor(game) {
            super(game, ButtonColor.GREEN);
            this.tinkIcon = new Phaser.Image(this.game, 0, 0, "button-icon-tink");
            ScaleUtil_20.scaleToBounds(this.tinkIcon, this.foregroundImage.width * 0.8, this.foregroundImage.height * 0.65);
            this.iconScaleX = this.tinkIcon.scale.x;
            this.tinkIcon.anchor.x = 0.5;
            this.tinkIcon.top = this.foregroundImage.height * 0.08;
            this.iconGroup.add(this.tinkIcon);
            this.arrowIcon = new Phaser.Image(this.game, 0, 0, "button-icon-arrow");
            ScaleUtil_20.scaleToBounds(this.arrowIcon, this.foregroundImage.width * 0.8, this.foregroundImage.height * 0.65);
            this.arrowIcon.anchor.set(0.5);
            this.arrowIcon.angle = 180;
            this.arrowIcon.top = this.foregroundImage.height * 0.08;
            this.iconGroup.add(this.arrowIcon);
            this.arrowIcon.width = 0;
        }
        animate() {
            return __awaiter(this, void 0, void 0, function* () {
                let icon1;
                let icon2;
                while (this.game) {
                    icon1 = (this.tinkIcon.scale.x === this.iconScaleX) ? this.tinkIcon : this.arrowIcon;
                    icon2 = (icon1 === this.tinkIcon) ? this.arrowIcon : this.tinkIcon;
                    yield PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(icon1.scale).to({ x: 0 }, 200, Phaser.Easing.Linear.None, true, 0));
                    if (this.game) {
                        yield PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(icon2.scale).to({ x: this.iconScaleX }, 200, Phaser.Easing.Linear.None, true, 0));
                    }
                    if (this.game) {
                        yield PromiseUtil_11.PromiseUtil.time(this.game, 1000);
                    }
                }
            });
        }
        flipArrow(pointDown) {
            if (pointDown) {
                this.game.add.tween(this.arrowIcon).to({ angle: 180 }, 250, Phaser.Easing.Linear.None, true);
            }
            else {
                this.game.add.tween(this.arrowIcon).to({ angle: 0 }, 250, Phaser.Easing.Linear.None, true);
            }
        }
    }
    exports.TrinketsButton = TrinketsButton;
});
define("pages/lantern-overlay/groups/LanternBasePage", ["require", "exports", "switchback-common/util/PromiseUtil", "pages/component/HeaderText", "switchback-common/localization/LocalizationManager"], function (require, exports, PromiseUtil_12, HeaderText_2, LocalizationManager_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LanternBasePage extends Phaser.Group {
        constructor(game) {
            super(game);
            this.completeSignal = new Phaser.Signal();
            this.dismissSignal = new Phaser.Signal();
            // By default, don't be accepting any input from this group.
            this.ignoreChildInput = true;
            this.header = new HeaderText_2.HeaderText(this.game, LocalizationManager_18.LocalizationManager.Instance.text("lantern.header"));
            this.header.centerX = game.width * 0.5;
            this.header.top = game.height * 0.005;
            this.add(this.header);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                yield PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                this.ignoreChildInput = false;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.ignoreChildInput = true;
                yield PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true));
                this.visible = false;
            });
        }
    }
    exports.LanternBasePage = LanternBasePage;
});
define("pages/lantern-overlay/groups/LanternLocationConfirmPage", ["require", "exports", "pages/lantern-overlay/groups/LanternBasePage", "Game", "view/MediumButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "logic/GameController", "logic/PPAnalyticsManager", "pages/lantern-overlay/LanternOverlayPage", "switchback-common/util/TextUtil", "switchback-common/util/PromiseUtil", "view/ConfirmDialog"], function (require, exports, LanternBasePage_1, Game_23, MediumButton_5, LocalizationManager_19, ScaleUtil_21, GameController_22, PPAnalyticsManager_13, LanternOverlayPage_1, TextUtil_19, PromiseUtil_13, ConfirmDialog_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LanternLocationConfirmPage extends LanternBasePage_1.LanternBasePage {
        constructor(game) {
            super(game);
            this.content = new Phaser.Group(game);
            this.add(this.content);
            LanternLocationConfirmPage.DESCRIPTION_HEIGHT = this.game.height * 0.16;
            this.descriptionText = new Phaser.Text(game, this.game.width * 0.5, this.header.bottom, LocalizationManager_19.LocalizationManager.Instance.text("lantern.location_confirm_description_1"), {
                font: Game_23.Game.DEFAULT_FONT,
                fill: Game_23.Game.TEXT_COLOR,
                fontSize: Game_23.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.96,
                align: 'center',
                stroke: Game_23.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_23.Game.TEXT_STROKE_THICKNESS
            });
            TextUtil_19.TextUtil.scaleDownTextToHeight(this.descriptionText, LanternLocationConfirmPage.DESCRIPTION_HEIGHT);
            this.descriptionText.anchor.set(0.5, 0);
            this.descriptionText.setShadow(Game_23.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_23.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_23.Game.TEXT_SHADOW_SOFT);
            this.content.add(this.descriptionText);
            this.bgDetail = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.99, 'outro_background');
            this.bgDetail.anchor.set(0.5, 1);
            this.bgDetail.width = this.game.width * 0.8;
            this.bgDetail.scale.y = this.bgDetail.scale.x;
            this.content.add(this.bgDetail);
            // Put the lantern photo between the top text and the confirmation buttons, scaled to 90% of the space
            console.log("LOAD LANTERN CONFIRM");
            this.yesButton = new MediumButton_5.MediumButton(this.game, 0, LocalizationManager_19.LocalizationManager.Instance.text("button.yes"));
            this.yesButton.x = this.bgDetail.centerX;
            this.yesButton.y = this.bgDetail.top + this.bgDetail.height * 0.32;
            this.yesButton.events.onInputDown.add(this.onDistantYesClicked, this);
            this.content.add(this.yesButton);
            this.noButton = new MediumButton_5.MediumButton(this.game, 0, LocalizationManager_19.LocalizationManager.Instance.text("button.no"));
            this.noButton.x = this.bgDetail.centerX;
            this.noButton.y = this.bgDetail.top + this.bgDetail.height * 0.68;
            this.noButton.events.onInputDown.add(this.onNoClicked, this);
            this.content.add(this.noButton);
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.lanternPhoto) {
                    // As all pages are preloaded when the game starts we can't load this image until it is known which queue is used
                    // create photo asseta
                    this.lanternPhoto = new Phaser.Image(this.game, this.game.width * 0.5, this.descriptionText.y + LanternLocationConfirmPage.DESCRIPTION_HEIGHT + ((this.bgDetail.top - this.descriptionText.y - LanternLocationConfirmPage.DESCRIPTION_HEIGHT) * 0.5), 'dlr-lantern-distant-' + GameController_22.GameController.Instance.state.selectedQueueType);
                    ScaleUtil_21.scaleToBounds(this.lanternPhoto, this.game.width * 0.96, (this.bgDetail.top - this.descriptionText.y - LanternLocationConfirmPage.DESCRIPTION_HEIGHT));
                    this.lanternPhoto.anchor.set(0.5);
                    this.content.add(this.lanternPhoto);
                }
                this.setDistantQuery();
                yield _super("show").call(this);
                PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'InstallationLanternAreYouHere1', {
                    [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                });
            });
        }
        setDistantQuery() {
            this.yesButton.events.onInputDown.remove(this.onCloseYesClicked, this);
            this.yesButton.events.onInputDown.add(this.onDistantYesClicked, this);
            this.noButton.events.onInputDown.add(this.onNoClicked, this);
            this.lanternPhoto.loadTexture('dlr-lantern-distant-' + GameController_22.GameController.Instance.state.selectedQueueType);
            this.descriptionText.text = LocalizationManager_19.LocalizationManager.Instance.text('lantern.location_confirm_description_1');
            this.descriptionText.fontSize = Game_23.Game.FONT_SIZE_MEDIUM;
            TextUtil_19.TextUtil.scaleDownTextToHeight(this.descriptionText, this.game.height * 0.16);
        }
        setCloseQuery() {
            this.yesButton.events.onInputDown.remove(this.onDistantYesClicked, this);
            this.yesButton.events.onInputDown.add(this.onCloseYesClicked, this);
            this.lanternPhoto.loadTexture('dlr-lantern-close-' + GameController_22.GameController.Instance.state.selectedQueueType);
            this.descriptionText.text = LocalizationManager_19.LocalizationManager.Instance.text('lantern.location_confirm_description_2');
            this.descriptionText.fontSize = Game_23.Game.FONT_SIZE_MEDIUM;
            TextUtil_19.TextUtil.scaleDownTextToHeight(this.descriptionText, this.game.height * 0.16);
        }
        transitionToNextQuery() {
            return __awaiter(this, void 0, void 0, function* () {
                this.yesButton.events.onInputDown.remove(this.onDistantYesClicked, this);
                const leftContent = this.content.x;
                yield PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.content).to({ right: 0 }, 500, Phaser.Easing.Sinusoidal.Out, true));
                this.setCloseQuery();
                this.content.left = this.game.width;
                yield PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.content).to({ x: leftContent }, 500, Phaser.Easing.Sinusoidal.In, true));
                PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'InstallationLanternAreYouHere2', {
                    [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                });
            });
        }
        onDistantYesClicked() {
            PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackAction('Declare_Yes', 'Location', 'Yes', LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME, {
                [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
            });
            this.transitionToNextQuery();
        }
        onCloseYesClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.yesButton.events.onInputDown.remove(this.onCloseYesClicked, this);
                this.yesButton.events.onInputDown.remove(this.onDistantYesClicked, this);
                this.noButton.events.onInputDown.remove(this.onNoClicked, this);
                PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackAction('Declare_Yes', 'Location', 'Yes', LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME, {
                    [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                });
                PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'InstallationLanternAreYouHere3', {
                    [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                });
                const confirmDialog = new ConfirmDialog_5.ConfirmDialog(this.game, LocalizationManager_19.LocalizationManager.Instance.text('outro.confirm_dialog.title'), LocalizationManager_19.LocalizationManager.Instance.text('lantern.location_confirm_warning'), LocalizationManager_19.LocalizationManager.Instance.text('button.yes'), LocalizationManager_19.LocalizationManager.Instance.text('button.no'), true);
                this.add(confirmDialog);
                yield confirmDialog.show();
                const isConfirmed = !!(yield PromiseUtil_13.PromiseUtil.signal(confirmDialog.clickedSignal));
                yield confirmDialog.hide();
                if (isConfirmed) {
                    PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackAction('Declare_Yes', 'Location', 'Yes', LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME, {
                        [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                    });
                    this.completeSignal.dispatch();
                }
                else {
                    PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackAction('Declare_No', 'Location', 'No', LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME, {
                        [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
                    });
                    this.dismissSignal.dispatch();
                }
            });
        }
        onNoClicked() {
            PPAnalyticsManager_13.PPAnalyticsManager.Instance.trackAction('Declare_No', 'Location', 'No', LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME, {
                [PPAnalyticsManager_13.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_1.LanternOverlayPage.LANTERN_NAME
            });
            this.dismissSignal.dispatch();
        }
        destroy() {
            this.yesButton.events.onInputDown.removeAll();
            this.noButton.events.onInputDown.removeAll();
            super.destroy();
        }
    }
    exports.LanternLocationConfirmPage = LanternLocationConfirmPage;
});
define("pages/lantern-overlay/groups/LanternTriggerPage", ["require", "exports", "pages/lantern-overlay/groups/LanternBasePage", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "view/MediumButton", "logic/GameController", "logic/PPAnalyticsManager", "pages/lantern-overlay/LanternOverlayPage", "switchback-common/play/ShowEffectController"], function (require, exports, LanternBasePage_2, Game_24, LocalizationManager_20, ScaleUtil_22, PromiseUtil_14, MediumButton_6, GameController_23, PPAnalyticsManager_14, LanternOverlayPage_2, ShowEffectController_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LanternTriggerPage extends LanternBasePage_2.LanternBasePage {
        constructor(game) {
            super(game);
            this.topText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_20.LocalizationManager.Instance.text("lantern.trigger_instructions"), {
                fontSize: Game_24.Game.FONT_SIZE_MEDIUM,
                fill: Game_24.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: game.width * 0.9,
                font: Game_24.Game.DEFAULT_FONT,
                stroke: Game_24.Game.TEXT_STROKE_COLOR
            });
            this.topText.anchor.set(0.5);
            this.topText.top = this.topTextOrigTopPos = this.header.bottom + this.game.height * 0.02;
            this.button = new MediumButton_6.MediumButton(this.game, 0, LocalizationManager_20.LocalizationManager.Instance.text("button.tap_here"), MediumButton_6.RibbonWidth.LARGE);
            this.button.centerX = this.game.width * 0.5;
            this.button.bottom = this.game.height * 0.975;
            // Put the lantern wireframe between the top text and the bottom text, scaled to 90% of the space
            this.lanternWireframe = new Phaser.Image(this.game, this.game.width * 0.5, this.topText.bottom + ((this.button.top - this.topText.bottom) * 0.5), 'lantern-spritesheet', 0);
            ScaleUtil_22.scaleToBounds(this.lanternWireframe, this.game.width * 0.8, (this.button.top - this.topText.bottom) * 0.9);
            this.lanternWireframe.anchor.set(0.5);
            // Add the other layers of the lantern
            this.lanternFilled = new Phaser.Image(this.game, 0, 0, 'lantern-spritesheet', 1);
            this.lanternFilledWithTink = new Phaser.Image(this.game, 0, 0, 'lantern-spritesheet', 2);
            // and ensure they line up on top of the wireframe
            this.lanternFilled.position = this.lanternFilledWithTink.position = this.lanternWireframe.position;
            this.lanternFilled.scale = this.lanternFilledWithTink.scale = this.lanternWireframe.scale;
            this.lanternFilled.anchor = this.lanternFilledWithTink.anchor = this.lanternWireframe.anchor;
            this.lanternFilled.alpha = this.lanternFilledWithTink.alpha = 0;
            // Add the light FX behind the lantern
            this.lanternGlowRing = new Phaser.Image(this.game, this.lanternWireframe.centerX, this.lanternWireframe.centerY, 'lantern-glow-ring');
            this.lanternGlowRing.width = this.lanternWireframe.width * 2;
            this.lanternGlowRing.scale.y = this.lanternGlowRing.scale.x;
            this.lanternGlowRing.anchor.set(0.5);
            this.lanternGlowLines = new Phaser.Image(this.game, this.lanternWireframe.centerX, this.lanternWireframe.centerY, 'lantern-glow-lines');
            this.lanternGlowLines.width = this.lanternWireframe.width * 2;
            this.lanternGlowLines.scale.y = this.lanternGlowLines.scale.x;
            this.lanternGlowLines.anchor.set(0.5);
            this.lanternGlowLines.alpha = this.lanternGlowRing.alpha = 0;
            // The dark background to make the lantern and the glow FX contrast nicely
            this.lanternBackground = new Phaser.Image(this.game, this.lanternWireframe.centerX, this.lanternWireframe.centerY, 'lantern-background');
            this.lanternBackground.width = this.lanternBackground.width * 1.5;
            this.lanternBackground.scale.y = this.lanternBackground.scale.x;
            this.lanternBackground.anchor.set(0.5);
            this.lanternBackground.alpha = 0;
            // Some sparkles!
            this.lanternSparkles = new Phaser.Image(this.game, this.lanternWireframe.centerX, this.lanternWireframe.centerY, 'lantern-sparkles');
            this.lanternSparkles.width = this.lanternBackground.width;
            this.lanternSparkles.scale.y = this.lanternSparkles.scale.x;
            this.lanternSparkles.anchor.set(0.5);
            this.lanternSparkles.alpha = 0;
            this.add(this.lanternBackground);
            this.add(this.lanternSparkles);
            this.add(this.lanternGlowRing);
            this.add(this.lanternGlowLines);
            this.add(this.lanternWireframe);
            this.add(this.lanternFilled);
            this.add(this.lanternFilledWithTink);
            this.add(this.topText);
            this.add(this.button);
        }
        /**
         * Only starts the 'flow' of this page once the show is complete.
         */
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.reset();
                yield _super("show").call(this);
                this.play();
            });
        }
        /**
         * Reset the page (you're unlikely to ever see this twice though)
         */
        reset() {
            // Reset the text fields
            this.topText.text = LocalizationManager_20.LocalizationManager.Instance.text("lantern.trigger_instructions");
            this.topText.fontSize = Game_24.Game.FONT_SIZE_MEDIUM;
            this.topText.anchor.set(0.5);
            this.button.text = LocalizationManager_20.LocalizationManager.Instance.text("button.tap_here");
            // Visible elements
            this.lanternBackground.alpha = 0;
            this.lanternSparkles.alpha = 0;
            this.lanternGlowRing.alpha = 0;
            this.lanternGlowLines.alpha = 0;
            this.lanternFilled.alpha = 0;
            this.lanternFilledWithTink.alpha = 0;
            // and remove any infinite tweens we may have added
            this.game.tweens.removeFrom(this.lanternGlowLines);
            this.game.tweens.removeFrom(this.lanternGlowRing);
            this.game.tweens.removeFrom(this.lanternSparkles);
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackState('content/game/inqueue/installation/trigger', 'InstallationLanternTrigger', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.startTiming('OffToNeverLand:Installation:LanternTrigger', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                yield PromiseUtil_14.PromiseUtil.signal(this.button.events.onInputDown);
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Installation:LanternTrigger', 'Trigger', 'Installation', '', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackAction('Installation_Trigger', 'Installation', 'Trigger', LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME, {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                this.game.add.tween(this.lanternWireframe).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                // Trigger the effect
                yield this.showLookAtLantern();
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackState('content/game/inqueue/installation/watch', 'InstallationLanternWatch', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                let effectResult = yield GameController_23.GameController.Instance.showEffectController.triggerDefaultEffect(5);
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackAction('Installation_Result', 'Installation', 'Result', LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME, {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME,
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_TRIGGER]: effectResult
                });
                if (effectResult === ShowEffectController_2.ShowEffectResult.SUCCESS) {
                    // If the trigger succeeded, we should wait and let them watch the effect before moving on
                    yield PromiseUtil_14.PromiseUtil.time(this.game, LanternTriggerPage.LANTERN_INSTALLATION_DURATION_MS);
                }
                else if (effectResult === ShowEffectController_2.ShowEffectResult.FAILURE) {
                    // immediately go to B-Show
                }
                else if (effectResult === ShowEffectController_2.ShowEffectResult.IN_USE) {
                    if (LanternTriggerPage.inUseAttemptCount++ < LanternTriggerPage.MAX_IN_USE_ATTEMPTS) {
                        yield this.showTryAgain();
                        // go back to the confirmation page
                        this.dismissSignal.dispatch();
                        return;
                    }
                }
                this.game.add.tween(this.topText).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    this.topText.top = this.topTextOrigTopPos;
                });
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackState('content/game/inqueue/installation/animation', 'InstallationLanternAnimation', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                // Then move to the b-show
                yield this.showLanternEffects();
                yield this.lanternRevealComplete();
                yield PromiseUtil_14.PromiseUtil.signal(this.button.events.onInputDown);
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackAction('Installation_End', 'Installation', 'End', LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME, {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                this.completeSignal.dispatch();
            });
        }
        showLookAtLantern() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.button).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.topText).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                this.button.visible = false;
                this.topText.fontSize = 85;
                this.topText.centerY = this.game.height * 0.5;
                this.topText.text = LocalizationManager_20.LocalizationManager.Instance.text("lantern.watch_description");
                yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.topText).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
            });
        }
        showTryAgain() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.topText).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true));
                this.topText.text = LocalizationManager_20.LocalizationManager.Instance.text("lantern.in_use");
                this.button.text = LocalizationManager_20.LocalizationManager.Instance.text("lantern.try_again");
                this.button.visible = true;
                yield Promise.all([
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.button).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.topText).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                PPAnalyticsManager_14.PPAnalyticsManager.Instance.trackState('content/game/inqueue/installation/inuse', 'InstallationLanternInUse', {
                    [PPAnalyticsManager_14.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage_2.LanternOverlayPage.LANTERN_NAME
                });
                yield PromiseUtil_14.PromiseUtil.signal(this.button.events.onInputDown);
            });
        }
        /**
         * B-Show animation!
         */
        showLanternEffects() {
            return __awaiter(this, void 0, void 0, function* () {
                // This is necessary as we don't await the completion of the "Watch Lantern" tweens
                // If we reach here before that is complete, it can cause the text to remain and be misaligned
                this.topText.visible = false;
                // Save off the scales of our glow FX, and set them to 0.
                // We'll be scaling them up shortly.
                const origRingScale = this.lanternGlowRing.scale.x;
                const origLineScale = this.lanternGlowLines.scale.x;
                this.lanternGlowRing.scale.set(0);
                this.lanternGlowLines.scale.set(0);
                const lanternFillAnimationDurationMs = 1500;
                const tinkRevealAnimationDurationMs = 2000;
                yield Promise.all([
                    ////////////////////////////////////////////////
                    // ANIMATION STAGE 1
                    // Adding color to the lantern
                    ////////////////////////////////////////////////
                    // Fade in the colored lantern layer
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternFilled)
                        .to({ alpha: 1 }, lanternFillAnimationDurationMs, Phaser.Easing.Linear.None, true)),
                    // And the background for contrast (do this one quickly)
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternBackground)
                        .to({ alpha: 1 }, lanternFillAnimationDurationMs, Phaser.Easing.Quintic.In, true)),
                    ////////////////////////////////////////////////
                    // ANIMATION STAGE 2
                    // Adding tink & glow FX to the lantern
                    ////////////////////////////////////////////////
                    // Fade in the final lantern layer - the one with tink
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternFilledWithTink)
                        .to({ alpha: 1 }, tinkRevealAnimationDurationMs, Phaser.Easing.Linear.None, true, lanternFillAnimationDurationMs)),
                    // Bring in the sparkles
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternSparkles)
                        .to({ alpha: 1 }, tinkRevealAnimationDurationMs * 0.5, Phaser.Easing.Linear.None, true, lanternFillAnimationDurationMs * 0.9)).then(() => {
                        this.game.add.tween(this.lanternSparkles)
                            .to({ alpha: 0.5 }, 500, Phaser.Easing.Linear.None, true, 0, Infinity, true);
                    }),
                    // Fade in the glow ring, then ping pong its alpha between 1 <-> 0.8
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternGlowRing)
                        .to({ alpha: 1 }, tinkRevealAnimationDurationMs * 0.5, Phaser.Easing.Linear.None, true, lanternFillAnimationDurationMs * 0.9)).then(() => {
                        this.game.add.tween(this.lanternGlowRing)
                            .to({ alpha: 0.8 }, 250, Phaser.Easing.Linear.None, true, 0, Infinity, true);
                    }),
                    // Not a promise, but infinitely rotate the glow lines
                    this.game.add.tween(this.lanternGlowLines).to({ angle: 360 }, 10000, Phaser.Easing.Linear.None, true, 0, Infinity),
                    // Fade in the glow lines, then ping pong their alpha between 1 <-> 0.8
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternGlowLines)
                        .to({ alpha: 1 }, tinkRevealAnimationDurationMs * 0.5, Phaser.Easing.Linear.None, true, lanternFillAnimationDurationMs * 1.1)).then(() => {
                        this.game.add.tween(this.lanternGlowLines)
                            .to({ alpha: 0.1 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                    }),
                    // Scale up the glow ring quickly, then ping pong its scale between 1 <-> 0.95
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternGlowRing.scale)
                        .to({ x: origRingScale, y: origRingScale }, tinkRevealAnimationDurationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true, lanternFillAnimationDurationMs)).then(() => {
                        this.game.add.tween(this.lanternGlowRing.scale)
                            .to({ x: origRingScale * 0.95, y: origRingScale * 0.95 }, 750, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                    }),
                    // Scale in the glow lines
                    PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.lanternGlowLines.scale)
                        .to({ x: origLineScale, y: origLineScale }, tinkRevealAnimationDurationMs * 0.5, Phaser.Easing.Exponential.Out, true, lanternFillAnimationDurationMs))
                ]);
            });
        }
        /**
         * Called when everything is done. This shows the final state
         * of the page, including the continue button to end the whole
         * lantern triggering experience.
         */
        lanternRevealComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                this.topText.visible = true;
                this.topText.anchor.set(0.5, 0);
                this.topText.top = this.topTextOrigTopPos;
                this.topText.text = LocalizationManager_20.LocalizationManager.Instance.text("lantern.tinkerbell_reveal");
                yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.topText).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                this.button.text = LocalizationManager_20.LocalizationManager.Instance.text("button.continue");
                this.button.visible = true;
                yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this.button).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                GameController_23.GameController.Instance.achievementManager.onLanternTrigger();
            });
        }
    }
    LanternTriggerPage.MAX_IN_USE_ATTEMPTS = 3;
    LanternTriggerPage.LANTERN_INSTALLATION_DURATION_MS = 5000;
    LanternTriggerPage.inUseAttemptCount = 0;
    exports.LanternTriggerPage = LanternTriggerPage;
});
define("pages/lantern-overlay/LanternOverlayPage", ["require", "exports", "switchback-common/util/PromiseUtil", "pages/lantern-overlay/groups/LanternLocationConfirmPage", "pages/lantern-overlay/groups/LanternTriggerPage", "logic/PPAnalyticsManager", "model/PeterPanConfig", "switchback-common/enum/Park"], function (require, exports, PromiseUtil_15, LanternLocationConfirmPage_1, LanternTriggerPage_1, PPAnalyticsManager_15, PeterPanConfig_5, Park_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LanternOverlayPage extends Phaser.Group {
        constructor(game) {
            super(game);
            this.clickBlocker = new Phaser.Graphics(this.game);
            this.clickBlocker.beginFill(0x000000, 0.5);
            this.clickBlocker.drawRect(0, 0, this.game.width, this.game.height);
            this.clickBlocker.inputEnabled = true;
            this.clickBlocker.alpha = 0;
            this.add(this.clickBlocker);
            this.pageGroup = new Phaser.Group(this.game);
            this.add(this.pageGroup);
            this.backgroundImg = new Phaser.Image(this.game, 0, 0, "page-tinks-trinkets");
            this.backgroundImg.width = this.game.width;
            this.backgroundImg.height = this.game.height;
            this.pageGroup.add(this.backgroundImg);
            /////////////////////////////////////////
            // LOCATION CONFIRMATION GROUP
            /////////////////////////////////////////
            if (PeterPanConfig_5.PeterPanConfig.Instance.park === Park_6.Park.DLR) {
                this.locationConfirmGroup = new LanternLocationConfirmPage_1.LanternLocationConfirmPage(this.game);
                this.locationConfirmGroup.completeSignal.add(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.locationConfirmGroup.hide();
                    this.triggerGroup.show();
                }));
                this.locationConfirmGroup.dismissSignal.add(this.hide, this);
                this.pageGroup.add(this.locationConfirmGroup);
            }
            /////////////////////////////////////////
            // SHOW EFFECT TRIGGER GROUP
            /////////////////////////////////////////
            this.triggerGroup = new LanternTriggerPage_1.LanternTriggerPage(this.game);
            this.triggerGroup.completeSignal.add(this.hide, this);
            this.triggerGroup.dismissSignal.add(this.onTriggerPageInUse, this);
            this.pageGroup.add(this.triggerGroup);
            this.pageGroup.bottom = 0;
            this.visible = false;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                // Don't show if we're already showing!
                if (this.isBeingShown) {
                    return;
                }
                PPAnalyticsManager_15.PPAnalyticsManager.Instance.startTiming('OffToNeverLand:Installation:Lantern');
                this.visible = true;
                this.isBeingShown = true;
                // Ensure all groups are hidden
                this.locationConfirmGroup.alpha = 0;
                this.locationConfirmGroup.visible = false;
                this.triggerGroup.alpha = 0;
                this.triggerGroup.visible = false;
                yield Promise.all([
                    this.locationConfirmGroup.show(),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.pageGroup).to({ bottom: this.game.height }, 800, Phaser.Easing.Sinusoidal.In, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.clickBlocker).to({ alpha: 1 }, 800, Phaser.Easing.Linear.None, true))
                ]);
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                // Don't hide if we're already hiding!
                if (!this.isBeingShown) {
                    return;
                }
                PPAnalyticsManager_15.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Installation:Lantern', 'Lantern', 'Installation', '', {
                    [PPAnalyticsManager_15.PPAnalyticsManager.INSTALLATION_NAME]: LanternOverlayPage.LANTERN_NAME
                });
                yield Promise.all([
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.pageGroup).to({ bottom: 0 }, 800, Phaser.Easing.Sinusoidal.In, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.clickBlocker).to({ alpha: 0 }, 800, Phaser.Easing.Linear.None, true))
                ]);
                this.isBeingShown = false;
                this.visible = false;
            });
        }
        onTriggerPageInUse() {
            return __awaiter(this, void 0, void 0, function* () {
                this.isBeingShown = false;
                yield this.show();
            });
        }
    }
    LanternOverlayPage.LANTERN_NAME = "Lantern";
    exports.LanternOverlayPage = LanternOverlayPage;
});
define("view/TinkWayfindConfirmation", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "logic/GameController", "model/PeterPanConfig", "switchback-common/util/ScaleUtil", "view/MediumButton", "logic/PPAnalyticsManager", "pages/TinksTrinketsPage", "switchback-common/enum/Park", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil"], function (require, exports, LocalizationManager_21, Game_25, GameController_24, PeterPanConfig_6, ScaleUtil_23, MediumButton_7, PPAnalyticsManager_16, TinksTrinketsPage_1, Park_7, PromiseUtil_16, TextUtil_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TinkWayfindConfirmation extends Phaser.Group {
        constructor(game, contentReference) {
            super(game);
            const titleText = new Phaser.Text(this.game, this.game.width * 0.5, 0, LocalizationManager_21.LocalizationManager.Instance.text("tinks_trinkets.confirm_page.title"), {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.TEXT_COLOR_GREEN,
                fontSize: Game_25.Game.FONT_SIZE_LARGE,
                stroke: Game_25.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_25.Game.TEXT_STROKE_THICKNESS
            });
            titleText.anchor.set(0.5, 0);
            titleText.setShadow(Game_25.Game.TEXT_SHADOW_OFFSET_MED_X, Game_25.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_25.Game.TEXT_SHADOW_SOFT);
            this.add(titleText);
            const descriptionText = new Phaser.Text(this.game, this.game.width * 0.5, titleText.bottom + contentReference.height * 0.005, LocalizationManager_21.LocalizationManager.Instance.text("tinks_trinkets.confirm_page.text-" + GameController_24.GameController.Instance.parkPrefix), {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.TEXT_COLOR,
                fontSize: Game_25.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.95,
                align: 'center',
                stroke: Game_25.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_25.Game.TEXT_STROKE_THICKNESS
            });
            descriptionText.anchor.set(0.5, 0);
            descriptionText.setShadow(Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_25.Game.TEXT_SHADOW_SOFT);
            TextUtil_20.TextUtil.scaleDownTextToHeight(descriptionText, contentReference.height * 0.1);
            this.add(descriptionText);
            this.noButton = new MediumButton_7.MediumButton(this.game, 1, LocalizationManager_21.LocalizationManager.Instance.text("tinks_trinkets.confirm_page.no"));
            this.noButton.x = contentReference.centerX;
            this.noButton.bottom = contentReference.height - contentReference.height * 0.02;
            ScaleUtil_23.scaleToBounds(this.noButton, contentReference.width * 0.7, contentReference.height * 0.1);
            this.noButton.events.onInputUp.addOnce(this.onNoClicked, this);
            this.add(this.noButton);
            this.yesButton = new MediumButton_7.MediumButton(this.game, 0, LocalizationManager_21.LocalizationManager.Instance.text("tinks_trinkets.confirm_page.yes"));
            this.yesButton.x = contentReference.centerX;
            this.yesButton.bottom = this.noButton.top - contentReference.height * 0.005;
            ScaleUtil_23.scaleToBounds(this.yesButton, contentReference.width * 0.7, contentReference.height * 0.1);
            this.yesButton.events.onInputUp.addOnce(this.onYesClicked, this);
            this.add(this.yesButton);
            const captionText = new Phaser.Text(this.game, this.game.width * 0.5, titleText.bottom, LocalizationManager_21.LocalizationManager.Instance.text("tinks_trinkets.confirm_page.caption"), {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.TEXT_COLOR,
                fontSize: Game_25.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.95,
                align: 'center',
                stroke: Game_25.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_25.Game.TEXT_STROKE_THICKNESS
            });
            captionText.anchor.set(0.5, 0);
            captionText.setShadow(Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_25.Game.TEXT_SHADOW_SOFT);
            TextUtil_20.TextUtil.scaleDownTextToHeight(captionText, contentReference.height * 0.1);
            captionText.bottom = this.yesButton.top - contentReference.height * 0.02;
            this.add(captionText);
            // animate through the sequence of "are you there yet" photo's at WDW
            const img = new Phaser.Sprite(this.game, 0, 0, "tt-start");
            img.anchor.set(0.5);
            if (PeterPanConfig_6.PeterPanConfig.Instance.park === Park_7.Park.WDW) {
                img.animations.add("slide", [0, 1, 2], 0.3, true);
            }
            img.play("slide");
            ScaleUtil_23.scaleToBounds(img, this.game.width * 0.95, captionText.top - descriptionText.bottom - contentReference.height * 0.02);
            img.centerY = descriptionText.bottom + (captionText.top - descriptionText.bottom) * 0.5;
            img.centerX = this.game.width * 0.5;
            this.add(img);
        }
        beginShow() {
            // use alpha rather than visible so that the width/height of the content is maintained
            this.yesButton.alpha = 0;
            this.noButton.alpha = 0;
            this.yesButton.inputEnabled = false;
            this.noButton.inputEnabled = false;
        }
        endShow() {
            return __awaiter(this, void 0, void 0, function* () {
                // delay the appearance of the buttons so that guests have time to view the confirmation content
                yield PromiseUtil_16.PromiseUtil.time(this.game, 1000);
                // slide the buttons in
                this.yesButton.alpha = 1;
                this.noButton.alpha = 1;
                this.game.add.tween(this.yesButton).from({ right: this.game.width, alpha: 0 }, 500, Phaser.Easing.Sinusoidal.In, true);
                yield PromiseUtil_16.PromiseUtil.tween(this.game, this.game.add.tween(this.noButton).from({ left: 0, alpha: 0 }, 500, Phaser.Easing.Sinusoidal.In, true));
                this.yesButton.inputEnabled = true;
                this.noButton.inputEnabled = true;
            });
        }
        beginHide() {
            this.yesButton.events.onInputUp.removeAll();
            this.noButton.events.onInputUp.removeAll();
        }
        onYesClicked() {
            PPAnalyticsManager_16.PPAnalyticsManager.Instance.trackAction('Declare_Yes', 'Location', 'Yes', 'ItemRoom', {
                [PPAnalyticsManager_16.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage_1.TinksTrinketsPage.GAME_NAME
            });
        }
        onNoClicked() {
            PPAnalyticsManager_16.PPAnalyticsManager.Instance.trackAction('Declare_No', 'Location', 'No', 'ItemRoom', {
                [PPAnalyticsManager_16.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage_1.TinksTrinketsPage.GAME_NAME
            });
        }
        destroy() {
            this.yesButton.events.onInputUp.removeAll();
            this.noButton.events.onInputUp.removeAll();
            super.destroy();
        }
    }
    exports.TinkWayfindConfirmation = TinkWayfindConfirmation;
});
define("pages/TinksTrinketsPage", ["require", "exports", "Game", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "view/PageTopButton", "switchback-common/localization/LocalizationManager", "pages/component/HeaderText", "logic/GameController", "pages/component/TextButton", "model/PeterPanConfig", "switchback-common/enum/Park", "pages/lantern-overlay/LanternOverlayPage", "enum/Queue", "logic/PPAnalyticsManager", "view/TinkWayfindConfirmation"], function (require, exports, Game_26, PromiseUtil_17, TextUtil_21, ScaleUtil_24, PageTopButton_1, LocalizationManager_22, HeaderText_3, GameController_25, TextButton_2, PeterPanConfig_7, Park_8, LanternOverlayPage_3, Queue_4, PPAnalyticsManager_17, TinkWayfindConfirmation_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var DragDirection;
    (function (DragDirection) {
        DragDirection[DragDirection["LEFT"] = 0] = "LEFT";
        DragDirection[DragDirection["RIGHT"] = 1] = "RIGHT";
        DragDirection[DragDirection["SNAP"] = 2] = "SNAP";
        DragDirection[DragDirection["NONE"] = 3] = "NONE";
    })(DragDirection = exports.DragDirection || (exports.DragDirection = {}));
    class TinksTrinketsPage extends Phaser.Group {
        constructor(game) {
            super(game);
            this.isBeingShown = false;
            this.isAnimatingInOut = false;
            this.isDisplayingTrinkets = false;
            this.trinketsContentGenerated = false;
            this.trinketImages = [];
            this.onHidePage = new Phaser.Signal();
            this.onShowPage = new Phaser.Signal();
            this.backgroundImg = new Phaser.Image(this.game, 0, 0, "page-tinks-trinkets");
            this.backgroundImg.height = this.game.height * 0.9;
            this.backgroundImg.width = this.game.width;
            this.add(this.backgroundImg);
            this.pageHeader = new HeaderText_3.HeaderText(game, LocalizationManager_22.LocalizationManager.Instance.text("tinks_trinkets.title"));
            this.pageHeader.top = 0;
            this.pageHeader.centerX = this.game.width * 0.5;
            this.pageHeader.alpha = 0;
            this.add(this.pageHeader);
            /**
             * Add bar along the bottom of the screen saying 'Search again later'
             */
            this.bannerGraphic = new Phaser.Image(this.game, 0, 0, "tinks_trinkets_bottom_banner");
            this.bannerGraphic.width = this.game.width;
            this.bannerGraphic.height = this.game.height - this.backgroundImg.height;
            this.bannerGraphic.top = this.backgroundImg.bottom;
            this.bannerGraphic.inputEnabled = true;
            this.bannerGraphic.alpha = 0;
            this.bannerGraphic.events.onInputUp.add(this.onTinkButtonClicked, this);
            this.add(this.bannerGraphic);
            /**
             * Add green ribbon bookmark button that is visible in the book
             */
            this.trinketsButton = new PageTopButton_1.TrinketsButton(this.game);
            this.trinketsButton.width = this.game.width * 0.175;
            this.trinketsButton.scale.y = this.trinketsButton.scale.x;
            if (this.trinketsButton.height > this.bannerGraphic.height) {
                this.trinketsButton.height = this.bannerGraphic.height;
                this.scale.x = this.scale.y;
            }
            this.trinketsButton.left = this.game.width * 0.025;
            this.trinketsButton.top = this.backgroundImg.bottom;
            this.trinketsButton.clickedSignal.add(this.onTinkButtonClicked, this);
            this.add(this.trinketsButton);
            this.trinketsButton.animate();
            this.bannerText = new Phaser.Text(this.game, 0, 0, LocalizationManager_22.LocalizationManager.Instance.text("tinks_trinkets.search_again_later"), {
                fontSize: TextUtil_21.TextUtil.getFontSize(game.width, 30),
                fill: Game_26.Game.TEXT_COLOR_WHITE,
                align: 'center',
                font: Game_26.Game.DEFAULT_FONT
            });
            this.bannerText.lineSpacing = -this.game.height * 0.01;
            this.bannerText.setShadow(Game_26.Game.TEXT_SHADOW_OFFSET_MED_X, Game_26.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_26.Game.TEXT_SHADOW_SOFT, 1);
            this.bannerText.left = this.trinketsButton.right + this.game.width * 0.015;
            this.bannerText.centerY = this.backgroundImg.bottom + this.bannerGraphic.height * 0.5;
            this.bannerText.alpha = 0;
            this.add(this.bannerText);
            this.contentPageCoords = [];
            this.currentContentLoc = 0;
            /**
             * Add 'Are you there yet' page
             */
            this.areYouThereYetPage = new TinkWayfindConfirmation_1.TinkWayfindConfirmation(this.game, this.backgroundImg);
            this.areYouThereYetPage.centerY = this.backgroundImg.centerY;
            this.areYouThereYetPage.centerX = this.game.width * 0.5;
            this.add(this.areYouThereYetPage);
            /**
             * Add main page content
             */
            // Create empty sprite to handle scrolling
            this.draggableContentSprite = new Phaser.Sprite(this.game, 0, 0);
            this.draggableContentSprite.events.onDragStart.add(this.onDragStart, this);
            this.draggableContentSprite.events.onDragUpdate.add(this.onDragUpdate, this);
            this.draggableContentSprite.events.onDragStop.add(this.onDragStop, this);
            // Enable drag, add draggable sprite
            this.draggableContentSprite.inputEnabled = true;
            this.draggableContentSprite.input.enableDrag();
            this.draggableContentSprite.input.allowVerticalDrag = false;
            this.draggableContentSprite.top = this.pageHeader.bottom;
            // Start off the screen, move onto screen after 'Are you there yet' screen
            this.draggableContentSprite.left = this.game.width;
            this.add(this.draggableContentSprite);
            /**
             * Add arrows for pagination
             */
            const buttonHeight = this.game.height * 0.08;
            this.leftArrow = new Phaser.Image(this.game, 0, 0, "button-pagination-arrow");
            this.leftArrow.anchor.set(1.0, 0.5);
            this.leftArrow.height = buttonHeight;
            this.leftArrow.scale.x = this.leftArrow.scale.y * -1;
            this.leftArrow.centerY = this.draggableContentSprite.centerY;
            this.leftArrow.right = 0;
            this.leftArrow.inputEnabled = true;
            this.leftArrow.events.onInputDown.add(this.goToPrevPage, this);
            this.add(this.leftArrow);
            this.rightArrow = new Phaser.Image(this.game, 0, 0, "button-pagination-arrow");
            this.rightArrow.anchor.set(1.0, 0.5);
            this.rightArrow.height = buttonHeight;
            this.rightArrow.scale.x = this.rightArrow.scale.y;
            this.rightArrow.centerY = this.draggableContentSprite.centerY;
            this.rightArrow.right = this.game.width;
            this.rightArrow.inputEnabled = true;
            this.rightArrow.events.onInputDown.add(this.goToNextPage, this);
            this.add(this.rightArrow);
            // We start on the 'Are you there yet' page, so we don't want arrows
            this.leftArrow.visible = false;
            this.leftArrow.alpha = 0;
            this.rightArrow.visible = false;
            this.rightArrow.alpha = 0;
            /////////////////////////////////
            // LANTERN PAGE (on top of trinkets)
            /////////////////////////////////
            this.lanternPage = new LanternOverlayPage_3.LanternOverlayPage(this.game);
            this.lanternPage.triggerGroup.completeSignal.add(this.onLanternTriggered, this);
            this.add(this.lanternPage);
        }
        destroy() {
            if (this.leftArrowTween != null) {
                this.leftArrowTween.stop();
            }
            if (this.rightArrowTween != null) {
                this.rightArrowTween.stop();
            }
            super.destroy();
        }
        generateMainContent() {
            const contentGroup = new Phaser.Group(this.game);
            const contentGroupHeight = this.backgroundImg.height * 0.95 - this.pageHeader.bottom;
            this.currentPage = 0;
            //////////////////////////////////////////////////////////////
            // INTRO PAGE
            //////////////////////////////////////////////////////////////
            const introPageGroup = new Phaser.Group(this.game);
            const introText = new Phaser.Text(this.game, 0, 0, LocalizationManager_22.LocalizationManager.Instance.text("tinks_trinkets.intro"), {
                fontSize: TextUtil_21.TextUtil.getFontSize(this.game.width, 30),
                fill: Game_26.Game.TEXT_COLOR,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.85,
                font: Game_26.Game.DEFAULT_FONT
            });
            introText.top = 0;
            introText.centerX = this.game.width * 0.5;
            const introImage = new Phaser.Image(this.game, 0, 0, "tinks_trinkets_title");
            ScaleUtil_24.scaleToBounds(introImage, this.game.width * 0.8, contentGroupHeight * 0.8);
            introImage.top = introText.bottom + this.game.height * 0.01;
            introImage.centerX = this.game.width * 0.5;
            introPageGroup.add(introText);
            introPageGroup.add(introImage);
            introPageGroup.centerX = this.game.width * 0.5;
            introPageGroup.centerY = contentGroupHeight * 0.5;
            contentGroup.add(introPageGroup);
            this.contentPageCoords[this.currentPage] = introPageGroup.centerX;
            //////////////////////////////////////////////////////////////
            // TRINKET IMAGES
            //////////////////////////////////////////////////////////////
            const trinkets = GameController_25.GameController.Instance.tinksTrinketsSequenceController.tinksTrinketsRawData.trinkets;
            const numRows = 4;
            const maxTrinketsPerPage = numRows * 2;
            const trinketHeight = contentGroupHeight / numRows;
            let trinketGroup;
            // Create a trinket group for each page of trinkets
            for (let i = 0; i < trinkets.length; i++) {
                const imgKey = trinkets[i];
                const isLastTrinket = i >= trinkets.length - 1;
                if (!trinketGroup) {
                    trinketGroup = new Phaser.Group(this.game);
                }
                const trinketImageFrame = GameController_25.GameController.Instance.state.isTrinketSelected(imgKey) ? 1 : 0;
                const trinketImage = new Phaser.Image(this.game, 0, 0, imgKey, trinketImageFrame);
                ScaleUtil_24.scaleToBounds(trinketImage, this.game.width * 0.45, trinketHeight);
                trinketGroup.add(trinketImage);
                this.trinketImages.push(trinketImage);
                // If we've hit the page limit, or we've reached the final trinket
                if (trinketGroup.children.length >= maxTrinketsPerPage || isLastTrinket) {
                    this.currentPage++;
                    // current group width is width of one item
                    const itemWidth = trinketGroup.width;
                    trinketGroup.align(2, numRows, itemWidth, trinketHeight);
                    // here we place the trinket group after the last page,
                    // with padding on the left, so the content is in the middle of the page
                    trinketGroup.x = (this.game.width * this.currentPage) + (this.game.width * 0.5 - itemWidth);
                    // Add special lantern trinket for DLR
                    if (isLastTrinket && PeterPanConfig_7.PeterPanConfig.Instance.park === Park_8.Park.DLR) {
                        const lantern = this.createLantern(contentGroupHeight / 2);
                        lantern.top = trinketGroup.bottom;
                        lantern.centerX = trinketGroup.width * 0.5;
                        trinketGroup.add(lantern);
                        this.trinketImages.push(lantern);
                        // Save off reference - only being used by onLanternTriggered()
                        this.lanternTrinket = lantern;
                    }
                    contentGroup.add(trinketGroup);
                    // Keep track of the center points of each page, so we 
                    // know where to transition to
                    this.contentPageCoords[this.currentPage] = trinketGroup.centerX;
                    trinketGroup = null;
                }
            }
            //////////////////////////////////////////////////////////////
            // OUTRO PAGE
            //////////////////////////////////////////////////////////////
            const outroPageGroup = new Phaser.Group(this.game);
            this.currentPage++;
            const outroImage = new Phaser.Image(this.game, 0, 0, "tinks_trinkets_outro");
            ScaleUtil_24.scaleToBounds(outroImage, this.game.width * 0.8, contentGroupHeight * 0.9);
            outroImage.centerY = contentGroupHeight * 0.5;
            outroImage.centerX = this.game.width * 0.5;
            outroPageGroup.add(outroImage);
            const finishedButton = new TextButton_2.TextButton(this.game, LocalizationManager_22.LocalizationManager.Instance.text("tinks_trinkets.finished"), "button-medium", 3);
            finishedButton.width = this.game.width * 0.7;
            finishedButton.scale.y = finishedButton.scale.x;
            finishedButton.onChildInputDown.add(this.onFinishedClicked, this);
            finishedButton.bottom = outroImage.bottom;
            finishedButton.centerX = this.game.width * 0.5;
            outroPageGroup.add(finishedButton);
            outroPageGroup.centerX = this.contentPageCoords[this.currentPage - 1] + this.game.width;
            outroPageGroup.centerY = contentGroupHeight * 0.5;
            contentGroup.add(outroPageGroup);
            this.contentPageCoords[this.currentPage] = outroPageGroup.centerX;
            // We were using currentPage during page generation
            // Reset back to 0
            this.currentPage = 0;
            // Generate group containing page content, add to scrolling sprite
            this.draggableContentSprite.addChild(contentGroup);
            // Scale the scrolling sprite to cover the whole content area
            // so that even empty space is draggable
            this.draggableContentSprite.width = contentGroup.width;
            this.draggableContentSprite.height = contentGroup.height;
            this.dragThreshold = this.game.width * 0.1;
            this.clickThreshold = this.game.width * 0.03;
            // Scaling the child group by the inverse of the sprite scale, to prevent distortion
            // This is reasonably magical/not understood
            contentGroup.scale.x = contentGroup.scale.x * (1 / this.draggableContentSprite.scale.x);
            contentGroup.scale.y = contentGroup.scale.y * (1 / this.draggableContentSprite.scale.y);
            // Now that the content is the correct height, reposition the pagination arrows
            this.leftArrow.centerY = this.draggableContentSprite.centerY;
            this.rightArrow.centerY = this.draggableContentSprite.centerY;
        }
        createLantern(heightToScale) {
            const trinketImageFrame = GameController_25.GameController.Instance.state.isTrinketSelected(TinksTrinketsPage.LANTERN_TRINKET_KEY) ? 1 : 0;
            const lanternImage = new Phaser.Image(this.game, 0, 0, TinksTrinketsPage.LANTERN_TRINKET_KEY, trinketImageFrame);
            // Scale to take up space of 4 trinkets
            ScaleUtil_24.scaleToBounds(lanternImage, this.game.width * 0.9, heightToScale);
            return lanternImage;
        }
        // Show trinkets content, hide 'Are you there yet' page
        transitionToTrinketContent() {
            return __awaiter(this, void 0, void 0, function* () {
                this.areYouThereYetPage.beginHide();
                GameController_25.GameController.Instance.reachedTrinketRoom = true;
                const hideConfirmPage = this.game.add.tween(this.areYouThereYetPage).to({ right: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                hideConfirmPage.onComplete.add(() => {
                    this.areYouThereYetPage.destroy();
                    this.areYouThereYetPage = null;
                });
                yield PromiseUtil_17.PromiseUtil.tween(this.game, hideConfirmPage);
                this.game.add.tween(this.draggableContentSprite).to({ left: 0 }, 250, Phaser.Easing.Sinusoidal.In, true);
                // Show the pagination arrow, 'Search again later' text, and page title, after we've transitioned the content onto the screen
                this.rightArrow.visible = true;
                this.game.add.tween(this.rightArrow).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 100);
                this.game.add.tween(this.bannerText).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 100);
                this.game.add.tween(this.pageHeader).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 100);
                this.isDisplayingTrinkets = true;
            });
        }
        // Show Trinkets page
        // Tween to bottom, fade in text/graphic at the bottom
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isAnimatingInOut) {
                    return;
                }
                this.isAnimatingInOut = true;
                GameController_25.GameController.Instance.state.incrementActivityPlays("Trinkets");
                if (!this.trinketsContentGenerated) {
                    // Generate group containing trinkets content, add to scrolling sprite
                    this.generateMainContent();
                    this.trinketsContentGenerated = true;
                    // If we're in the DLR secondary queue, don't show an 'are you there yet' screen
                    if (GameController_25.GameController.Instance.state.selectedQueueType === Queue_4.Queue.SECONDARY) {
                        yield this.transitionToTrinketContent();
                    }
                }
                if (!this.isDisplayingTrinkets) {
                    this.areYouThereYetPage.beginShow();
                }
                this.leftArrowTween = this.game.add.tween(this.leftArrow.scale).to({ x: this.leftArrow.scale.x * 0.75, y: this.leftArrow.scale.y * 0.75 }, 200, Phaser.Easing.Sinusoidal.InOut, true, 5000, Infinity, true);
                this.leftArrowTween.repeatDelay(1000);
                this.rightArrowTween = this.game.add.tween(this.rightArrow.scale).to({ x: this.rightArrow.scale.x * 0.75, y: this.rightArrow.scale.y * 0.75 }, 200, Phaser.Easing.Sinusoidal.InOut, true, 5000, Infinity, true);
                this.rightArrowTween.repeatDelay(1000);
                this.isBeingShown = true;
                this.onShowPage.dispatch();
                yield PromiseUtil_17.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 800, Phaser.Easing.Sinusoidal.In, true));
                this.bannerGraphic.inputEnabled = true;
                this.game.add.tween(this.bannerGraphic).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                if (this.isDisplayingTrinkets) {
                    this.game.add.tween(this.bannerText).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                }
                else {
                    this.areYouThereYetPage.endShow();
                    this.areYouThereYetPage.yesButton.events.onInputUp.addOnce(() => {
                        this.transitionToTrinketContent();
                    }, this);
                    this.areYouThereYetPage.noButton.events.onInputUp.addOnce(() => {
                        this.hide();
                    }, this);
                }
                GameController_25.GameController.Instance.tinksTrinketsSequenceController.lastDisplayTime = new Date().getTime();
                clearInterval(this.buttonPulseIntervalRef);
                this.trinketsButton.flipArrow(false);
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.startTiming('OffToNeverLand:Mini:Items');
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.startTiming('OffToNeverLand:Mini:ItemsSession');
                if (this.isDisplayingTrinkets) {
                    PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackAction('ShowItems', 'Book', 'ShowItems', 'Items', {
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                    });
                }
                else {
                    PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackAction('ShowItems', 'Book', 'ShowItems', 'Location', {
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                    });
                    PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'ItemsAreYouHere', {
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME
                    });
                }
                this.isAnimatingInOut = false;
            });
        }
        // Hide Trinkets page
        // Fade out text/graphic at the bottom, then slide to top of screen
        // Top of TT button sits at top of screen
        hide(forced = false) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isAnimatingInOut) {
                    return;
                }
                this.isAnimatingInOut = true;
                this.trinketsButton.flipArrow(true);
                if (this.leftArrowTween != null) {
                    this.leftArrowTween.stop();
                }
                if (this.rightArrowTween != null) {
                    this.rightArrowTween.stop();
                }
                this.game.add.tween(this.bannerText).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.bannerGraphic).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_17.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.bannerGraphic.height }, 800, Phaser.Easing.Sinusoidal.Out, true));
                this.bannerGraphic.inputEnabled = false;
                if (!GameController_25.GameController.Instance.tinksTrinketsSequenceController.hasFinishedTT) {
                    this.buttonPulseIntervalRef = setInterval(this.tinkButtonPulse.bind(this), TinksTrinketsPage.RIBBON_PULSE_INTERVAL);
                }
                if (!forced) {
                    this.onHidePage.dispatch();
                }
                this.isBeingShown = false;
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Mini:ItemsSession', 'Session', 'Items', '', {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: "Trinkets"
                });
                // ONLY FIRE WHEN NOT FORCED AT THE START OF THE GAME
                if (!forced) {
                    PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackAction('Back', 'Items', 'Back', '', {
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                        [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                    });
                }
                this.isAnimatingInOut = false;
            });
        }
        tinkButtonPulse() {
            // Idle pulse animation on the ribbon button
            const initScaleX = this.trinketsButton.scale.x;
            const initScaleY = this.trinketsButton.scale.y;
            this.game.add.tween(this.trinketsButton.scale).to({ x: initScaleX * 1.15, y: initScaleY * 1.15 }, 200, Phaser.Easing.Sinusoidal.InOut, true, 0, 2, true);
        }
        onTinkButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isAnimatingInOut) {
                    if (this.isDisplayingTrinkets || !this.isBeingShown) {
                        if (this.isBeingShown) {
                            yield this.hide();
                        }
                        else {
                            yield this.show();
                        }
                    }
                }
            });
        }
        onTrinketMouseUp(trinketImage) {
            // if trinket key is lantern, and we haven't already been through the lantern flow, call lantern
            if (trinketImage.key === TinksTrinketsPage.LANTERN_TRINKET_KEY) {
                if (this.lanternTrinket.frame === 0) {
                    this.onLanternMouseUp();
                }
                return;
            }
            // Flip the frame on click
            const select = trinketImage.frame !== 1;
            trinketImage.frame = select ? 1 : 0;
            // Update save state
            GameController_25.GameController.Instance.state.setTrinketValue(trinketImage.key, select);
            GameController_25.GameController.Instance.achievementManager.onTrinketFound();
            if (select) {
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackAction('FoundItem', 'Items', 'FoundItem', '', {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME),
                    [PPAnalyticsManager_17.PPAnalyticsManager.ITEM_ID]: trinketImage.key
                });
            }
            // Play audio
            if (GameController_25.GameController.Instance.allTrinketsSelected()) {
                this.game.sound.play('pp-sfx-all-trinkets');
            }
            else if (select) {
                this.game.sound.play('pp-sfx-find-trinket');
            }
        }
        onLanternMouseUp() {
            // Trigger trinkets flow
            this.lanternPage.show();
        }
        onLanternTriggered() {
            return __awaiter(this, void 0, void 0, function* () {
                // Toggle lantern icon frame
                const select = this.lanternTrinket.frame !== 1;
                this.lanternTrinket.frame = select ? 1 : 0;
                // Update save state
                GameController_25.GameController.Instance.state.setTrinketValue(TinksTrinketsPage.LANTERN_TRINKET_KEY, select);
                GameController_25.GameController.Instance.achievementManager.onTrinketFound();
                // after triggering the lantern close trinkets and go directly to the exit sequence
                GameController_25.GameController.Instance.exitConfirmed = true;
                yield this.onFinishedClicked();
            });
        }
        onDragStart() {
            // Store off the position at the start of the drag
            this.initDragXPos = this.game.input.x;
        }
        onDragUpdate() {
            // If the user is trying to scroll past the start/end of the content
            if (this.draggableContentSprite.left > this.game.width * 0.5 || this.draggableContentSprite.right < this.game.width * 0.5) {
                // Cancel the current scroll operation, snap back to current page
                this.draggableContentSprite.input.disableDrag();
                this.snapToCurrentPage();
                this.draggableContentSprite.input.enableDrag();
            }
        }
        onDragStop(sprite, pointer) {
            return __awaiter(this, void 0, void 0, function* () {
                // Check which direction we should scroll
                const dragDir = this.calcDragDirection();
                if (dragDir === DragDirection.LEFT) {
                    yield this.goToNextPage();
                }
                else if (dragDir === DragDirection.RIGHT) {
                    yield this.goToPrevPage();
                }
                else if (dragDir === DragDirection.SNAP) {
                    yield this.snapToCurrentPage();
                }
                else {
                    // Consider this drag a click
                    // Look for any trinkets that collide with the input
                    for (let trinket of this.trinketImages) {
                        if (trinket.getBounds().contains(pointer.positionUp.x, pointer.positionUp.y)) {
                            this.onTrinketMouseUp(trinket);
                            break;
                        }
                    }
                    yield this.snapToCurrentPage();
                }
            });
        }
        // Calculate the direction (and size) of the user's drag input
        // Returns 0 if the magnitude of the drag is under the threshold
        calcDragDirection() {
            const finalDragXPos = this.game.input.x;
            const movement = finalDragXPos - this.initDragXPos;
            if (Math.abs(movement) > this.dragThreshold) {
                if (movement < 0) {
                    return DragDirection.LEFT;
                }
                if (movement > 0) {
                    return DragDirection.RIGHT;
                }
            }
            else if (Math.abs(movement) < this.clickThreshold) {
                return DragDirection.NONE;
            }
            else {
                return DragDirection.SNAP;
            }
        }
        // Disable interactable elements
        disableInput() {
            this.draggableContentSprite.inputEnabled = false;
            this.leftArrow.inputEnabled = false;
            this.rightArrow.inputEnabled = false;
            this.leftArrowTween.pause();
            this.rightArrowTween.pause();
        }
        // Enable interactable elements
        enableInput() {
            this.draggableContentSprite.inputEnabled = true;
            this.leftArrow.inputEnabled = true;
            this.rightArrow.inputEnabled = true;
            this.leftArrowTween.resume();
            this.rightArrowTween.resume();
        }
        // Snap view back to current page's correct position
        snapToCurrentPage() {
            return __awaiter(this, void 0, void 0, function* () {
                this.disableInput();
                yield PromiseUtil_17.PromiseUtil.tween(this.game, this.game.add.tween(this.draggableContentSprite).to({ left: this.currentContentLoc }, 300, Phaser.Easing.Elastic.Out, true));
                this.enableInput();
            });
        }
        // Go to previous page
        goToPrevPage() {
            return __awaiter(this, void 0, void 0, function* () {
                this.disableInput();
                if (!this.contentPageCoords[this.currentPage - 1]) {
                    this.snapToCurrentPage();
                    return;
                }
                // Calculate the distance between the current and next page
                const distToMove = this.contentPageCoords[this.currentPage] - this.contentPageCoords[this.currentPage - 1];
                this.currentContentLoc += distToMove;
                yield PromiseUtil_17.PromiseUtil.tween(this.game, this.game.add.tween(this.draggableContentSprite).to({ left: this.currentContentLoc }, 200, Phaser.Easing.Back.Out, true));
                this.currentPage--;
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackState(`content/game/inqueue/trinkets/${this.currentPage}_${this.contentPageCoords.length}`, `Items${this.currentPage}_${this.contentPageCoords.length}`, {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                });
                this.rightArrow.visible = true;
                this.game.add.tween(this.rightArrow).to({ alpha: 1 }, TinksTrinketsPage.ARROW_FADE_DURATION, Phaser.Easing.Linear.None, true);
                if (!this.contentPageCoords[this.currentPage - 1]) {
                    this.game.add.tween(this.leftArrow).to({ alpha: 0 }, TinksTrinketsPage.ARROW_FADE_DURATION, Phaser.Easing.Linear.None, true)
                        .onComplete.addOnce(() => {
                        this.leftArrow.visible = false;
                    });
                }
                this.enableInput();
            });
        }
        // Go to next page
        goToNextPage() {
            return __awaiter(this, void 0, void 0, function* () {
                this.disableInput();
                if (!this.contentPageCoords[this.currentPage + 1]) {
                    this.snapToCurrentPage();
                    return;
                }
                // Calculate the distance between the current and next page
                const distToMove = this.contentPageCoords[this.currentPage + 1] - this.contentPageCoords[this.currentPage];
                this.currentContentLoc -= distToMove;
                yield PromiseUtil_17.PromiseUtil.tween(this.game, this.game.add.tween(this.draggableContentSprite).to({ left: this.currentContentLoc }, 200, Phaser.Easing.Back.Out, true));
                this.currentPage++;
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackState(`content/game/inqueue/trinkets/${this.currentPage}_${this.contentPageCoords.length}`, `Items${this.currentPage}_${this.contentPageCoords.length}`, {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                });
                this.leftArrow.visible = true;
                this.game.add.tween(this.leftArrow).to({ alpha: 1 }, TinksTrinketsPage.ARROW_FADE_DURATION, Phaser.Easing.Linear.None, true);
                if (!this.contentPageCoords[this.currentPage + 1]) {
                    this.game.add.tween(this.rightArrow).to({ alpha: 0 }, TinksTrinketsPage.ARROW_FADE_DURATION, Phaser.Easing.Linear.None, true)
                        .onComplete.addOnce(() => {
                        this.rightArrow.visible = false;
                    });
                }
                this.enableInput();
            });
        }
        onFinishedClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.hide();
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.endTiming('OffToNeverLand:Mini:Items', 'Game', 'Items', '', {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                });
                PPAnalyticsManager_17.PPAnalyticsManager.Instance.trackAction('Finished', 'Items', 'Finished', '', {
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_NAME]: TinksTrinketsPage.GAME_NAME,
                    [PPAnalyticsManager_17.PPAnalyticsManager.GAME_PLAYS]: GameController_25.GameController.Instance.state.getActivityPlays(TinksTrinketsPage.GAME_NAME)
                });
                GameController_25.GameController.Instance.tinksTrinketsSequenceController.hasOptedOutOfTT = true;
            });
        }
    }
    TinksTrinketsPage.LANTERN_TRINKET_KEY = "dlr_tt_lantern";
    TinksTrinketsPage.GAME_NAME = "Trinkets";
    TinksTrinketsPage.RIBBON_PULSE_INTERVAL = 30000; // 30 secs
    TinksTrinketsPage.ARROW_FADE_DURATION = 100;
    exports.TinksTrinketsPage = TinksTrinketsPage;
});
define("view/ReminderDialog", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "Game", "model/PeterPanConfig", "pages/component/HeaderText", "switchback-common/enum/Park", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_25, LocalizationManager_23, TextUtil_22, Game_27, PeterPanConfig_8, HeaderText_4, Park_9, PromiseUtil_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ReminderDialog extends Phaser.Group {
        constructor(game) {
            super(game);
            const dialogBackground = new Phaser.Image(this.game, 0, 0, 'dialog-large');
            ScaleUtil_25.scaleToBounds(dialogBackground, this.game.width * 0.9, this.game.height * 0.5);
            this.add(dialogBackground);
            this.confirmButton = new Phaser.Button(this.game, dialogBackground.right, dialogBackground.top + dialogBackground.height * 0.8, 'dialog-button-positive');
            ScaleUtil_25.scaleToBounds(this.confirmButton, dialogBackground.width * 0.525, dialogBackground.height * 0.35);
            this.confirmButton.anchor.set(0.975, 0.6);
            this.confirmButton.width *= -1;
            this.confirmButton.left = dialogBackground.right - dialogBackground.width * 0.018;
            this.add(this.confirmButton);
            const confirmLabel = new Phaser.Text(this.game, this.confirmButton.centerX + Math.abs(this.confirmButton.width) * 0.05, this.confirmButton.top + this.confirmButton.height * 0.425, LocalizationManager_23.LocalizationManager.Instance.text("tinks_trinkets.reminder.confirm"), {
                font: Game_27.Game.DEFAULT_FONT,
                fill: Game_27.Game.TEXT_BUTTON_COLOR,
                fontSize: Game_27.Game.FONT_SIZE_LARGE,
            });
            confirmLabel.anchor.set(0.5);
            confirmLabel.setShadow(Game_27.Game.TEXT_SHADOW_OFFSET_MED_X, Game_27.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_27.Game.TEXT_SHADOW_HARD);
            TextUtil_22.TextUtil.scaleDownSingleLineTextToWidth(confirmLabel, Math.abs(this.confirmButton.width) * 0.8);
            this.add(confirmLabel);
            const headerText = new HeaderText_4.HeaderText(this.game, LocalizationManager_23.LocalizationManager.Instance.text("tinks_trinkets.reminder.title"), 'header-graphic-gold');
            headerText.x = dialogBackground.width * 0.1;
            headerText.y = dialogBackground.height * 0.1;
            this.add(headerText);
            let descriptionTag;
            if (PeterPanConfig_8.PeterPanConfig.Instance.park === Park_9.Park.DLR) {
                descriptionTag = "tinks_trinkets.reminder.description-dlr";
            }
            else {
                descriptionTag = "tinks_trinkets.reminder.description-wdw";
            }
            const descriptionText = new Phaser.Text(this.game, dialogBackground.centerX, headerText.bottom + (this.confirmButton.top - headerText.bottom) * 0.5, LocalizationManager_23.LocalizationManager.Instance.text(descriptionTag), {
                font: Game_27.Game.DEFAULT_FONT,
                fill: Game_27.Game.TEXT_COLOR,
                fontSize: Game_27.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: dialogBackground.width * 0.8,
                align: 'center',
                stroke: Game_27.Game.TEXT_STROKE_COLOR,
                strokeThickness: Game_27.Game.TEXT_STROKE_THICKNESS
            });
            descriptionText.anchor.set(0.5);
            TextUtil_22.TextUtil.scaleDownTextToHeight(descriptionText, this.confirmButton.top - headerText.bottom);
            this.add(descriptionText);
            this.centerX = this.game.width * 0.5;
            this.visible = false;
        }
        show(targetY) {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.top = this.game.height;
                this.game.add.tween(this).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: targetY }, ReminderDialog.ANIMATION_DURATION, Phaser.Easing.Sinusoidal.Out, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: this.game.height }, ReminderDialog.ANIMATION_DURATION, Phaser.Easing.Sinusoidal.Out, true));
                this.visible = false;
            });
        }
    }
    ReminderDialog.ANIMATION_DURATION = 500;
    exports.ReminderDialog = ReminderDialog;
});
define("state/BookState", ["require", "exports", "switchback-common/state/BaseState", "logic/BookLogic", "switchback-common/util/PromiseUtil", "enum/TurnDirection", "logic/GameController", "switchback-common/util/ScaleUtil", "pages/Cover", "view/DebugPanel", "view/PageTopButton", "view/ConfirmDialog", "switchback-common/localization/LocalizationManager", "model/PeterPanConfig", "pages/TinksTrinketsPage", "switchback-common/enum/Park", "logic/sequence-controllers/MinigameSequenceController", "logic/PPAnalyticsManager", "view/ReminderDialog"], function (require, exports, BaseState_2, BookLogic_2, PromiseUtil_19, TurnDirection_1, GameController_26, ScaleUtil_26, Cover_1, DebugPanel_1, PageTopButton_2, ConfirmDialog_6, LocalizationManager_24, PeterPanConfig_9, TinksTrinketsPage_2, Park_10, MinigameSequenceController_4, PPAnalyticsManager_18, ReminderDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * The one and only state for this game
     * Runs on a core loop of something like:
     * 1. Get Pages
     * 2. While has pages, show page
     * 3. When page is done, GOTO 2
     * 4. When out of pages, GOTO 1
     *
     * Also performs a few other things like animations
     * and hosting a debug panel.
     *
     * @author joel@cerebralfix.com
     */
    class BookState extends BaseState_2.BaseState {
        preload() {
            super.preload();
            console.log(`[${this.stateName}] preload()`);
            const path = 'assets/sound/';
            let soundNames = ['ui-click', 'all-trinkets', 'find-trinket'];
            for (let sound of soundNames) {
                this.game.load.audio("pp-sfx-" + sound, [path + sound + ".m4a", path + sound + ".ogg"]);
            }
            // Load park-specific tinks trinkets images
            const parkSpecificTrinketsJson = this.cache.getJSON('tinks-trinkets')[GameController_26.GameController.Instance.parkPrefix];
            const trinketImagesToLoad = parkSpecificTrinketsJson.primary_queue.trinkets.concat(parkSpecificTrinketsJson.secondary_queue.trinkets);
            for (let imageTag of trinketImagesToLoad) {
                this.game.load.spritesheet(imageTag, 'assets/trinkets/' + imageTag + '.png', 444, 363, 2);
            }
            if (PeterPanConfig_9.PeterPanConfig.Instance.park === Park_10.Park.DLR) {
                this.game.load.spritesheet(TinksTrinketsPage_2.TinksTrinketsPage.LANTERN_TRINKET_KEY, 'assets/trinkets/dlr_tt_lantern.png', 444, 363, 2);
                this.game.load.image('queue-1', `assets/queues/dlr-queue-1.png`);
                this.game.load.image('queue-2', `assets/queues/dlr-queue-2.png`);
                // NOTE: This is loaded once before the queue type is known, so we must load all lantern photo's
                // TODO: Refactor so that only the assets that are required are loaded
                this.game.load.image('dlr-lantern-distant-1', `assets/queues/dlr-lantern-queue-1-distant.png`);
                this.game.load.image('dlr-lantern-close-1', `assets/queues/dlr-lantern-queue-1-close.png`);
                this.game.load.image('dlr-lantern-distant-2', `assets/queues/dlr-lantern-queue-2-distant.png`);
                this.game.load.image('dlr-lantern-close-2', `assets/queues/dlr-lantern-queue-2-close.png`);
            }
            else {
                this.game.load.image('wdw-merge', `assets/queues/wdw-merge.png`);
            }
            this.game.load.spritesheet('tt-start', `assets/queues/${GameController_26.GameController.Instance.parkPrefix}-tt-start.png`, 927, 682);
            this.game.load.image('drawing-stamp', 'assets/drawing/drawing-stamp.png');
        }
        init() {
            super.init();
            this.currentPageViews = [];
            this.stateName = 'BookState';
            this.canInteractWithPages = false;
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                console.log(`[${this.stateName}] create()`);
                /////////////////////////////////
                // FAKE REAR CORNER
                /////////////////////////////////
                const fakeCorner = new Phaser.Image(this.game, 0, 0, 'page-fake-corner');
                fakeCorner.width = this.game.width;
                fakeCorner.height = this.game.height;
                this.add.existing(fakeCorner);
                /////////////////////////////////
                // LAYERS
                /////////////////////////////////
                this.pageGroup = new Phaser.Group(this.game);
                this.add.existing(this.pageGroup);
                this.uiGroup = new Phaser.Group(this.game);
                this.add.existing(this.uiGroup);
                this.dialogGroup = new Phaser.Group(this.game);
                this.add.existing(this.dialogGroup);
                /////////////////////////////////
                // QUIT BUTTON
                /////////////////////////////////
                this.quitButton = new PageTopButton_2.QuitButton(this.game);
                this.quitButton.top = 0;
                this.quitButton.width = this.game.width * 0.175;
                this.quitButton.scale.y = this.quitButton.scale.x;
                this.quitButton.right = this.game.width * 0.975;
                this.quitButton.clickedSignal.add(this.onQuitClicked, this);
                this.quitButton.hide(); // it starts hidden - if a page is quittable it'll get enabled
                this.uiGroup.add(this.quitButton);
                /////////////////////////////////
                // QUIT CONFIRM DIALOG
                /////////////////////////////////
                this.quitConfirmDialog = new ConfirmDialog_6.ConfirmDialog(this.game, LocalizationManager_24.LocalizationManager.Instance.text('quit.title'), LocalizationManager_24.LocalizationManager.Instance.text('quit.text'), LocalizationManager_24.LocalizationManager.Instance.text('quit.yes'), LocalizationManager_24.LocalizationManager.Instance.text('quit.no'), false);
                this.dialogGroup.add(this.quitConfirmDialog);
                /////////////////////////////////
                // INTRO ANIMATION COMPONENTS
                /////////////////////////////////
                this.introCover = new Phaser.Image(this.game, 0, 0, 'cover');
                this.introCover.width = this.game.width * 0.35;
                this.introCover.height = this.game.height * 0.35;
                this.introCover.centerX = this.game.width * 0.5;
                this.introCover.centerY = this.game.height * 0.5;
                this.introCover.inputEnabled = true;
                this.cover = new Cover_1.Cover(this.game);
                this.cover.width = this.game.width;
                this.cover.height = this.game.height;
                this.cover.centerX = this.game.width * 0.5;
                this.cover.centerY = this.game.height * 0.5;
                this.cover.visible = false;
                this.room = new Phaser.Image(this.game, 0, 0, 'room-background');
                this.room.anchor.set(0.5);
                this.room.centerX = this.game.width * 0.5;
                this.room.centerY = this.game.height * 0.5;
                this.room.width = this.game.width;
                this.room.height = this.game.height;
                this.room.inputEnabled = true;
                this.add.existing(this.room);
                this.add.existing(this.introCover);
                this.add.existing(this.cover);
                /////////////////////////////////
                // LOGIC SETUP
                /////////////////////////////////
                this.logic = new BookLogic_2.BookLogic(this.cache.getJSON('pages'));
                this.createNextPageSequence(this.logic.getNextSequence());
                // At DLR the outro sequence is triggered by finding the Lantern and triggering it, not by beacon
                if (PeterPanConfig_9.PeterPanConfig.Instance.park === Park_10.Park.WDW) {
                    GameController_26.GameController.Instance.createOutroSubscription();
                }
                BookLogic_2.BookLogic.bookSequenceController.onSetFinished.add(this.onSetChanged, this);
                /////////////////////////////////
                // CLICK BLOCKER (enabled when the trinkets page transitions in)
                /////////////////////////////////
                this.clickBlocker = new Phaser.Graphics(this.game);
                this.clickBlocker.beginFill(0x000000, 1);
                this.clickBlocker.drawRect(0, 0, this.game.width, this.game.height);
                this.clickBlocker.inputEnabled = false;
                this.clickBlocker.alpha = 0;
                this.uiGroup.add(this.clickBlocker);
                /////////////////////////////////
                // TRINKETS PAGE (sitting above screen)
                /////////////////////////////////
                this.ttPage = new TinksTrinketsPage_2.TinksTrinketsPage(this.game);
                this.ttPage.x = 0;
                this.ttPage.trinketsButton.hide();
                this.ttPage.trinketsButton.alpha = 0;
                this.ttPage.trinketsButton.visible = false;
                this.ttPage.onHidePage.add(this.onTinksTrinketsHide, this);
                this.ttPage.onShowPage.add(this.onTinksTrinketsShow, this);
                this.uiGroup.add(this.ttPage);
                this.ttPage.hide(true);
                /////////////////////////////////
                // DEBUG BUTTONS
                /////////////////////////////////
                if (PeterPanConfig_9.PeterPanConfig.Instance.showDebugMenu) {
                    const debugPanel = new DebugPanel_1.DebugPanel(this.game);
                    this.uiGroup.add(debugPanel);
                }
                /////////////////////////////////
                // START
                /////////////////////////////////
                yield this.dismissOverlay();
                this.play();
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                PPAnalyticsManager_18.PPAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene/animation', 'BookOpen', {
                    [PPAnalyticsManager_18.PPAnalyticsManager.CUTSCENE]: "BookOpen"
                });
                this.showPages();
                yield this.playIntro();
                this.canInteractWithPages = true;
                this.currentPageView.turnable = this.canInteractWithPages;
            });
        }
        /**
         * Plays the zoom-in effect on the book cover.
         * Waits for the user to interact and open the cover.
         */
        playIntro() {
            return __awaiter(this, void 0, void 0, function* () {
                const targetRoomScale = new Phaser.Point(this.room.width / this.introCover.width, this.room.height / this.introCover.height);
                let durationMS = 500;
                // Zoom in on book, scale the room image at the same speed
                yield Promise.all([
                    PromiseUtil_19.PromiseUtil.tween(this.game, this.game.add.tween(this.introCover)
                        .to({ x: 0, y: 0, width: this.game.width, height: this.game.height }, durationMS, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_19.PromiseUtil.tween(this.game, this.game.add.tween(this.room)
                        .to({ width: targetRoomScale.x * this.game.width, height: targetRoomScale.y * this.game.height }, durationMS, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                this.cover.visible = true;
                this.introCover.destroy();
                this.cover.turnable = true;
                // Remove room surround before opening book cover
                this.room.destroy();
                // Turn the cover, then destroy it
                yield this.cover.turn(TurnDirection_1.TurnDirection.LEFT);
                this.cover.destroy();
            });
        }
        /**
         * Receives a bunch of PageData - it builds each one, adds it
         * to the screen and adds it to our member list of pages.
         *
         * This is called when we don't have any pages to show!
         *
         * @param sequence The sequence of PageData to build
         */
        createNextPageSequence(sequence) {
            let pageIndex = 0;
            while (sequence.length > 0) {
                const pageData = sequence.shift();
                const pageViews = pageData.build(this.game);
                while (pageViews.length > 0) {
                    const pageView = pageViews.shift();
                    pageView.pageIndexWithinActivity = pageIndex;
                    pageIndex++;
                    this.pageGroup.addAt(pageView, 0);
                    this.currentPageViews.push(pageView);
                }
            }
        }
        /**
         * Gets the next page to show and shows it.
         */
        showPages() {
            return __awaiter(this, void 0, void 0, function* () {
                while (this.currentPageViews.length > 0) {
                    this.currentPageView = this.currentPageViews.shift();
                    yield this.showPage(this.currentPageView);
                }
            });
        }
        /**
         * Waits for @param page to do its thing, then turns it away.
         *
         * Also repopuates the page stack when we hit the bottom.
         */
        showPage(page) {
            return __awaiter(this, void 0, void 0, function* () {
                /// DEBUG: shows the current position in progression
                if (this.currentPageInfo) {
                    this.currentPageInfo.text = this.logic.getCurrentDebugString();
                }
                // Show the quit ribbon conditionally
                if (page.showQuit) {
                    this.quitButton.show();
                    if (!GameController_26.GameController.Instance.exitConfirmed) {
                        this.ttPage.trinketsButton.show();
                    }
                }
                // Allow the page to turn and wait for it to be ready
                yield page.readyToTurn();
                page.turnable = this.canInteractWithPages;
                // Then actually wait for the page to be clicked before turning it
                if (!GameController_26.GameController.Instance.exitConfirmed) {
                    yield PromiseUtil_19.PromiseUtil.signal(page.onPageClicked);
                }
                // If it was the final page of an activity, let the logic know.
                if (page.isFinalPageOfActivity) {
                    this.logic.completedActivity();
                }
                // If that exhausted our list of pages, get the next sequence.
                if (this.currentPageViews.length === 0) {
                    this.createNextPageSequence(this.logic.getNextSequence());
                }
                yield this.doTurnPage(page);
                page.destroy();
            });
        }
        doTurnPage(page) {
            return __awaiter(this, void 0, void 0, function* () {
                this.quitButton.hide();
                this.ttPage.trinketsButton.hide();
                yield page.turn(TurnDirection_1.TurnDirection.LEFT);
            });
        }
        onSetChanged(setID, theme) {
            return __awaiter(this, void 0, void 0, function* () {
                if (GameController_26.GameController.Instance.tinksTrinketsSequenceController.canInterrupt()) {
                    if (!this.ttPage.trinketsButton.visible) {
                        this.ttPage.trinketsButton.visible = true;
                        this.ttPage.trinketsButton.alpha = 0;
                        // don't await as page can be turned in this 250ms
                        PromiseUtil_19.PromiseUtil.tween(this.game, this.game.add.tween(this.ttPage.trinketsButton)
                            .to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, BookState.TURN_ANIMATION_DURATION_MS));
                    }
                    this.ttPage.show();
                }
                if (theme !== "intro") {
                    this.logic.minigameSequenceController.setsSinceLastPrompt++;
                    MinigameSequenceController_4.MinigameSequenceController.confirmedMinigamePrompt = MinigameSequenceController_4.MinigamePromptStatus.NOT_ASKED;
                }
            });
        }
        onTinksTrinketsHide() {
            return __awaiter(this, void 0, void 0, function* () {
                // auto advance to the next page if the guest triggered the end of the experience from trinkets
                if (GameController_26.GameController.Instance.exitConfirmed) {
                    // clear any pending pages to force the load instructions to be the next page
                    this.currentPageViews = [];
                    while (this.pageGroup.children.length > 1) {
                        this.pageGroup.removeChildAt(0);
                    }
                    this.currentPageView.onPageClicked.dispatch();
                }
                else if (!GameController_26.GameController.Instance.reachedTrinketRoom) {
                    // show reminder dialog
                    yield this.showReminderDialog();
                }
                this.clickBlocker.inputEnabled = false;
            });
        }
        onTinksTrinketsShow() {
            this.clickBlocker.inputEnabled = true;
        }
        showReminderDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                this.clickBlocker.inputEnabled = true;
                this.clickBlocker.alpha = 0.75;
                this.ttPage.onHidePage.remove(this.onTinksTrinketsHide, this);
                this.ttPage.onShowPage.remove(this.onTinksTrinketsShow, this);
                this.ttPage.trinketsButton.ignoreChildInput = true;
                const arrow = new Phaser.Image(this.game, 0, 0, 'arrow-corner');
                ScaleUtil_26.scaleToBounds(arrow, this.game.width * 0.2, this.game.height * 0.2);
                arrow.anchor.set(0, 1);
                arrow.left = this.ttPage.x + this.ttPage.trinketsButton.right;
                arrow.top = this.ttPage.y + this.ttPage.trinketsButton.centerY;
                arrow.angle = 15;
                this.add.existing(arrow);
                // arrow animates toward the button
                let animFrame = 1;
                const arrowAnimationId = setInterval(() => {
                    if (animFrame++ >= 1) {
                        arrow.angle = 0;
                        animFrame = 0;
                    }
                    else {
                        arrow.angle = 15;
                    }
                }, 750);
                const reminderDialog = new ReminderDialog_1.ReminderDialog(this.game);
                this.add.existing(reminderDialog);
                // wait for the confirm button to be clicked and proceed
                yield reminderDialog.show(arrow.bottom + this.game.height * 0.02);
                arrow.visible = true;
                // wait for ok press
                yield PromiseUtil_19.PromiseUtil.signal(reminderDialog.confirmButton.events.onInputDown);
                this.game.sound.play("pp-sfx-ui-click");
                // hide dialog
                arrow.visible = false;
                yield reminderDialog.hide();
                // dismiss dialog and restore state
                clearInterval(arrowAnimationId);
                reminderDialog.destroy();
                arrow.destroy();
                this.clickBlocker.alpha = 0;
                this.ttPage.onHidePage.add(this.onTinksTrinketsHide, this);
                this.ttPage.onShowPage.add(this.onTinksTrinketsShow, this);
                this.ttPage.trinketsButton.ignoreChildInput = false;
            });
        }
        onQuitClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                // If we've already confirmed the exit, this button straight up quits.
                if (GameController_26.GameController.Instance.exitConfirmed) {
                    yield PPAnalyticsManager_18.PPAnalyticsManager.Instance.endTiming("OffToNeverLand", "OffToNeverLand", '', '', {
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_26.GameController.Instance.state.currentSetID,
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_26.GameController.Instance.state.currentActivityIndex
                    });
                    PPAnalyticsManager_18.PPAnalyticsManager.Instance.trackAction('Exit', 'Outro', 'Exit', '', {
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_26.GameController.Instance.state.currentSetID,
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_26.GameController.Instance.state.currentActivityIndex
                    });
                    GameController_26.GameController.Instance.showEffectController.shutdown();
                    PlayAPI.closeGame();
                }
                else {
                    PPAnalyticsManager_18.PPAnalyticsManager.Instance.trackAction('Exit', 'Book', 'Exit', '', {
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_26.GameController.Instance.state.currentSetID,
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_26.GameController.Instance.state.currentActivityIndex
                    });
                    // Otherwise it prompts a confirmation
                    yield this.quitConfirmDialog.show();
                    const response = yield PromiseUtil_19.PromiseUtil.signal(this.quitConfirmDialog.clickedSignal);
                    PPAnalyticsManager_18.PPAnalyticsManager.Instance.trackAction(response ? 'Exit_Confirm' : 'Exit_Cancel', 'Book', response ? 'ExitConfirm' : 'ExitCancel', '', {
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_26.GameController.Instance.state.currentSetID,
                        [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_26.GameController.Instance.state.currentActivityIndex
                    });
                    if (response) {
                        yield PPAnalyticsManager_18.PPAnalyticsManager.Instance.endTiming("OffToNeverLand", "OffToNeverLand", '', '', {
                            [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_INDEX]: GameController_26.GameController.Instance.state.currentSetID,
                            [PPAnalyticsManager_18.PPAnalyticsManager.ACTIVITY_SET_ACTIVITY_INDEX]: GameController_26.GameController.Instance.state.currentActivityIndex
                        });
                        GameController_26.GameController.Instance.showEffectController.shutdown();
                        PlayAPI.closeGame();
                    }
                    yield this.quitConfirmDialog.hide();
                }
            });
        }
        shutdown() {
            this.ttPage.onHidePage.remove(this.onTinksTrinketsHide, this);
            this.ttPage.onShowPage.remove(this.onTinksTrinketsShow, this);
            this.quitButton.clickedSignal.removeAll();
            BookLogic_2.BookLogic.bookSequenceController.onSetFinished.remove(this.onSetChanged, this);
            super.shutdown();
        }
    }
    BookState.TURN_ANIMATION_DURATION_MS = 500;
    exports.BookState = BookState;
});
define("Game", ["require", "exports", "state/LoadingState", "switchback-common/state/BaseState", "switchback-common/util/DeviceUtil", "state/BookState"], function (require, exports, LoadingState_1, BaseState_3, DeviceUtil_1, BookState_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Game extends Phaser.Game {
        constructor() {
            super('100', '100', Phaser.CANVAS);
            console.log(`[Game] constructor()`);
            Game.TEXT_STROKE_THICKNESS = this.width * 0.01;
            if (DeviceUtil_1.DeviceUtil.isMaliGPU()) {
                // Fix for strobing canvas when running Phaser on Mali-400 GPUs.
                this.preserveDrawingBuffer = true;
            }
            // Set up a reference to our loadCommonAssets function, which will be used by all classes that extend BaseState
            BaseState_3.BaseState.loadCommonAssets = () => { };
            this.state.add("Load", LoadingState_1.LoadingState, true);
            this.state.add("BookState", BookState_2.BookState, false);
        }
    }
    Game.DEFAULT_FONT = "EczarBold";
    Game.DEFAULT_FONT_ITALIC = "FontinItalic";
    Game.TEXT_COLOR = "#000000";
    Game.TEXT_STROKE_COLOR = "#eae3df";
    Game.TEXT_SHADOW_SOFT = "rgba(0, 0, 0, 0.2)";
    Game.TEXT_SHADOW_HARD = "rgba(0, 0, 0, 0.5)";
    Game.TEXT_COLOR_RED = "#B81010";
    Game.TEXT_COLOR_GREEN = "#016001";
    Game.TEXT_COLOR_YELLOW = "#FFF764";
    Game.TEXT_COLOR_WHITE = "#FFFFFF";
    Game.TEXT_BUTTON_COLOR = "#FAFC62";
    Game.FONT_SIZE_LARGE = 100;
    Game.FONT_SIZE_MEDIUM = 70;
    Game.FONT_SIZE_SMALL = 50;
    Game.TEXT_SHADOW_OFFSET_SMALL_X = 4;
    Game.TEXT_SHADOW_OFFSET_SMALL_Y = 4;
    Game.TEXT_SHADOW_OFFSET_MED_X = 8;
    Game.TEXT_SHADOW_OFFSET_MED_Y = 8;
    exports.Game = Game;
});
/// <reference path="../node_modules/phaser/typescript/phaser.comments.d.ts" />
/// <reference path="./switchback-common/localization/typing/polyglot.d.ts" />
define("index", ["require", "exports", "Game"], function (require, exports, Game_28) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    new Game_28.Game();
});
define("switchback-common/component/RadialProgressBar", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialProgressBar extends Phaser.Graphics {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100, startAngleOffset = 0) {
            super(game, x, y);
            this.angleMin = 0;
            this.angleMax = 0;
            this.color = color;
            this.lineThicknessPx = lineThicknessPx;
            this.radiusPx = radiusPx;
            this.angleMin = this.angleMax = startAngleOffset;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_20.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: this.angleMin + 360 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
        update() {
            super.update();
            this.clear();
            this.lineStyle(this.lineThicknessPx, this.color);
            this.arc(0, 0, this.radiusPx, Phaser.Math.degToRad(this.angleMin), Phaser.Math.degToRad(this.angleMax), false);
            this.endFill();
        }
    }
    exports.RadialProgressBar = RadialProgressBar;
});
define("switchback-common/component/RadialCountdownBar", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/component/RadialProgressBar"], function (require, exports, PromiseUtil_21, RadialProgressBar_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialCountdownBar extends RadialProgressBar_1.RadialProgressBar {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100) {
            super(game, x, y, color, lineThicknessPx, radiusPx);
            // Start full
            this.angleMax = 360;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_21.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: 0 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
    }
    exports.RadialCountdownBar = RadialCountdownBar;
});
define("switchback-common/component/ThreePieceImage", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var JoinMethod;
    (function (JoinMethod) {
        JoinMethod[JoinMethod["STRETCHED_VERTICALLY"] = 0] = "STRETCHED_VERTICALLY";
        JoinMethod[JoinMethod["STRETCHED_HORIZONTALLY"] = 1] = "STRETCHED_HORIZONTALLY";
        JoinMethod[JoinMethod["TILED_VERTICALLY"] = 2] = "TILED_VERTICALLY";
        JoinMethod[JoinMethod["TILED_HORIZONTALLY"] = 3] = "TILED_HORIZONTALLY";
    })(JoinMethod = exports.JoinMethod || (exports.JoinMethod = {}));
    /**
     * This class creates a resizable three piece tiled image that can be used as a backplate.
     *
     * HOW TO USE:
     * this.backplate = new ThreePieceImage(game, ['topscore-leftplate', 'topscore-midplate',
     * 'topscore-rightplate'], JoinMethod.TILED_HORIZONTALLY)
     * this.backplate.height = this.game.height * 0.1; ~~~~~~~~~~~~~~~~~~~~> if it is joined
     * HORIZONTALLY, scale the backplate's height, and vice versa.
     * this.backplate.scale.x = this.backplate.scale.y;
     *
     * 	-- After this.content is created and filled --
     * this.backplate.resize(this.content.width);
     * this.content.centerX = this.backplate.centerX;
     * this.content.centerY = this.backplate.centerY;
     *
     * An example that uses this can be found in the MinigameTitleDialog class
     *
     * @author qi@cerebralfix.com
     */
    class ThreePieceImage extends Phaser.Group {
        constructor(game, assetKeys, joinMethod) {
            super(game);
            this.joinMethod = joinMethod;
            this.firstPiece = new Phaser.Image(game, 0, 0, assetKeys[0]);
            this.add(this.firstPiece);
            // we need to create this as a Phaser Image first to get the Original Width and Height of
            // the asset so we can use it later
            this.middlePiece = new Phaser.Image(game, 0, 0, assetKeys[1]);
            this.lastPiece = new Phaser.Image(game, 0, 0, assetKeys[2]);
            // if join method is Stretch, create the middle piece as a Phaser Image
            // if join method is Tile, create the middle piece as a Phaser TileSprite
            if (joinMethod === JoinMethod.TILED_HORIZONTALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePiece = new Phaser.TileSprite(game, 0, 0, this.middlePiece.width, this.middlePiece.height, assetKeys[1]);
            }
            // if method is Vertical (tile / stretch) place the pieces below each other
            if (joinMethod === JoinMethod.STRETCHED_VERTICALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.height; // save this off for resizing later
                this.middlePiece.top = this.firstPiece.bottom;
                this.add(this.middlePiece);
                this.lastPiece.top = this.middlePiece.bottom;
                this.add(this.lastPiece);
            }
            else if (joinMethod === JoinMethod.STRETCHED_HORIZONTALLY || joinMethod === JoinMethod.TILED_HORIZONTALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.width; // save this off for resizing later
                this.middlePiece.left = this.firstPiece.right;
                this.add(this.middlePiece);
                this.lastPiece.left = this.middlePiece.right;
                this.add(this.lastPiece);
            }
        }
        /**
         * Resizes the image to be the @param widthOrHeight that is passed in by adding middle tiles
         *
         * @param widthOrHeight width or height that is needed (if the join method is horizontal
         * this will be the width needed, and vice versa)
         * @param padding (optional) extra padding on both sides (multiplied by 2)
         * @param joinMethod (optional) allows the altering of the join method
         */
        resize(widthOrHeight, padding = 0) {
            // removes all extra pieces and resets the size of the group
            // based on the old join method
            this.removeBetween(3, undefined, true); // removes everything except the first 3 pieces added
            switch (this.joinMethod) {
                case JoinMethod.TILED_VERTICALLY: {
                    this.tileVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.TILED_HORIZONTALLY: {
                    this.tileHorizontally(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_VERTICALLY: {
                    this.stretchVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_HORIZONTALLY: {
                    this.stretchHorizontally(widthOrHeight, padding);
                    break;
                }
                default: {
                    break;
                }
            }
        }
        /////////////////////////////////////////////////////////////////////////////
        // TILE:
        // duplicates and tiles the middle piece to fill the missing piece or height
        /////////////////////////////////////////////////////////////////////////////
        tileHorizontally(width, padding) {
            // calculates the number of extra middle pieces needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingWidth / (this.middlePieceOrigWidthOrHeight * this.scale.x));
            this.middlePiece.width = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.left = this.middlePiece.right;
        }
        tileVertically(height, padding) {
            // calculates the number of extra middle pieces needed
            const heightOfFirsttToLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirsttToLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingHeight / (this.middlePieceOrigWidthOrHeight * this.scale.y));
            this.middlePiece.height = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.top = this.middlePiece.bottom;
        }
        ////////////////////////////////////////////////////////////////////////
        // STRETCH:
        // stretches the middle piece to fill the missing width or height
        ////////////////////////////////////////////////////////////////////////
        stretchHorizontally(width, padding) {
            // calculates the amount of missing width needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.width = missingWidth / this.scale.x;
            this.lastPiece.left = this.middlePiece.right;
        }
        stretchVertically(height, padding) {
            // calculates the amount of missing height needed
            const heightOfFirstAndLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.height = missingHeight / this.scale.y;
            this.lastPiece.top = this.middlePiece.bottom;
        }
    }
    exports.ThreePieceImage = ThreePieceImage;
});
define("switchback-common/component/button/Button", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * This class adds an optional text label to the standard Phaser.Button.
     * The Text and TextStyle can be set on construction, or later.
     */
    class Button extends Phaser.Button {
        constructor(game, x, y, key, callback, callbackContext, text, style) {
            super(game, x, y, key, callback, callbackContext, "over", "out", "down", "out");
            this.textStyle = {};
            this.textStyle = style || {};
            if (text !== null && text !== "") {
                this.text = text;
            }
        }
        get text() {
            return this.textField.text;
        }
        set text(text) {
            if (!this.textField) {
                this.textField = new Phaser.Text(this.game, (this.width * 0.5), (this.height * 0.5), text, this.textStyle);
                this.textField.anchor.set(0.5);
                this.addChild(this.textField);
            }
            else {
                this.textField.text = text;
            }
        }
        get style() {
            return this.textStyle;
        }
        set style(style) {
            this.textStyle = style;
            if (this.textField) {
                this.textField.setStyle(this.textStyle, true);
            }
        }
        // This ensures that the text remains centered within the button when the button's anchor changes
        setAnchor(x, y) {
            // This allows developers to only pass in one argument if they want them both to be the same (as Phaser does)
            y = y || x;
            // Set the button's anchor as normal
            this.anchor.set(x, y);
            // Reposition the textField to remain centered in the button
            if (this.textField) {
                this.textField.position.set((this.width * 0.5) - this.offsetX, (this.height * 0.5) - this.offsetY);
            }
        }
    }
    exports.Button = Button;
});
define("switchback-common/component/button/DisablingButton", ["require", "exports", "switchback-common/component/button/Button"], function (require, exports, Button_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A disableable Phaser.Button with default button frame names
     */
    class DisablingButton extends Button_1.Button {
        constructor(game, x, y, key, callback, callbackContext, overFrame, outFrame, downFrame, upFrame, disabledFrame, disabled) {
            super(game, x, y, key, callback, callbackContext);
            this.dBool = false;
            this.originalOverFrame = overFrame || "over";
            this.originalOutFrame = outFrame || "out";
            this.originalDownFrame = downFrame || "down";
            this.originalUpFrame = upFrame || "out";
            this.disabledFrame = disabledFrame || "disabled";
            this.dBool = disabled;
        }
        get disabled() {
            return this.dBool;
        }
        set disabled(val) {
            this.dBool = val;
            this.updateState();
        }
        changeFrames(overFrame, outFrame, downFrame, upFrame, disabledFrame) {
            this.originalOverFrame = overFrame;
            this.originalOutFrame = outFrame;
            this.originalDownFrame = downFrame;
            this.originalUpFrame = upFrame;
            this.disabledFrame = disabledFrame;
            this.updateState();
        }
        updateState() {
            // Hack to work around issue where phaser does not correctly update the input handlers in constructors
            this.game.time.events.add(1, () => {
                this.inputEnabled = !this.dBool;
            });
            if (this.dBool) {
                super.setFrames(this.disabledFrame, this.disabledFrame, this.disabledFrame, this.disabledFrame);
            }
            else {
                super.setFrames(this.originalOverFrame, this.originalOutFrame, this.originalDownFrame, this.originalUpFrame);
            }
        }
    }
    exports.DisablingButton = DisablingButton;
});
define("switchback-common/component/button/GenericButton", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil"], function (require, exports, PromiseUtil_22, TextUtil_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GenericButton extends Phaser.Group {
        constructor(game, config) {
            super(game);
            this.onClickedSignal = new Phaser.Signal();
            this.enabled = true;
            this._cooldownMS = 0;
            this._buttonEffectInProgress = false;
            if (!config.backgroundKey) {
                console.error("[Generic Button] 'backgroundKey' will soon be a required parameter.");
            }
            this._shutdownSignal = config.shutdownSignal;
            this._cooldownMS = config.waitTime ? config.waitTime : 0;
            this._upTint = config.upTint ? config.upTint : 0xffffff;
            this._downTint = config.downTint ? config.downTint : 0xcccccc;
            this.pivot.set(0.5, 0.5);
            this.image = new Phaser.Image(this.game, 0, 0, config.backgroundKey || "common-button");
            this.image.inputEnabled = true;
            this.add(this.image);
            const buttonText = new Phaser.Text(this.game, 0, 0, config.text, config.fontStyle || {
                fill: 0xff0000,
                fontSize: 72,
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.8,
            });
            buttonText.anchor.set(0.5);
            buttonText.scale.y = buttonText.scale.x;
            buttonText.position.set(this.image.width / 2, this.image.height / 2);
            buttonText.centerY -= this.height * 0.033;
            this.text = buttonText;
            this.add(buttonText);
            if (config.onClickSoundKey) {
                this._clickSound = this.game.add.sound(config.onClickSoundKey, 1, false);
            }
            this.image.events.onInputUp.add(this.onClick, this);
            this.recenterText();
        }
        static preload(loader, url) {
            console.warn("[Generic Button] static preload is now deprecated and will be removed in future");
            loader.image("common-button", url);
        }
        static unload(cache) {
            console.warn("[Generic Button] static unload is now deprecated and will be removed in future");
            cache.removeImage("common-button");
        }
        destroy() {
            this.image.events.onInputUp.removeAll();
            this.onClickedSignal.dispose();
            super.destroy();
            this._shutdownSignal = null;
        }
        delayedFadeIn(delay) {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 0;
                this.enabled = false; // makes sure that once the button is visible it still cant be clicked
                this.visible = true;
                yield PromiseUtil_22.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 1 }, 250, Phaser.Easing.Cubic.InOut, true, delay), this._shutdownSignal).catch(() => undefined);
                this.enabled = true;
            });
        }
        recenterText() {
            this.text.centerX = this.image.centerX;
            this.text.centerY = this.image.centerY - this.image.height * 0.033;
            TextUtil_23.TextUtil.scaleDownTextToHeight(this.text, this.image.height * 0.9);
        }
        fadeOut() {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 1;
                let isShutdown = false;
                yield PromiseUtil_22.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 0 }, 200, Phaser.Easing.Cubic.InOut, true), this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this.visible = false;
                this.enabled = false;
            });
        }
        onClick() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this._buttonEffectInProgress || !this.enabled || this.onClickedSignal.getNumListeners() <= 0) {
                    return;
                }
                if (this._clickSound) {
                    this._clickSound.play();
                }
                if (this.text) {
                    this.text.tint = this._downTint;
                }
                this.image.tint = this._downTint;
                this._buttonEffectInProgress = true;
                // made new function in case need for tweens
                this.onClickedSignal.dispatch();
                let isShutdown = false;
                yield PromiseUtil_22.PromiseUtil.time(this.game, this._cooldownMS, this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this._buttonEffectInProgress = false;
                this.image.tint = this._upTint;
                if (this.text) {
                    this.text.tint = this._upTint;
                }
            });
        }
    }
    exports.GenericButton = GenericButton;
});
define("switchback-common/component/button/ToggleButton", ["require", "exports", "switchback-common/component/button/DisablingButton"], function (require, exports, DisablingButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A Phaser.Button with 2 toggle states. Can be disabled.
     */
    class ToggleButton extends DisablingButton_1.DisablingButton {
        constructor(game, x, y, key, callback, callbackContext, upFrame, downFrame, overFrame, disabledFrame, toggledUpFrame, toggledDownFrame, toggledOverFrame, toggledDisabledFrame) {
            super(game, x, y, key, callback, callbackContext, overFrame, upFrame, downFrame, upFrame);
            this.overFrame = overFrame || "over";
            this.upFrame = upFrame || "out";
            this.downFrame = downFrame || "down";
            this.normalDisabledFrame = disabledFrame || "disabled";
            this.toggledUpFrame = toggledUpFrame || "toggleout";
            this.toggledDownFrame = toggledDownFrame || "toggledown";
            this.toggledOverFrame = toggledOverFrame || "toggleover";
            this.toggledDisabledFrame = toggledDisabledFrame || "toggledisabled";
            this.t = false;
            this.onInputDown.add(() => {
                this.toggled = !this.t;
            });
        }
        get toggled() {
            return this.t;
        }
        set toggled(value) {
            this.t = value;
            if (this.t) {
                this.changeFrames(this.toggledOverFrame, this.toggledUpFrame, this.toggledDownFrame, this.toggledUpFrame, this.toggledDisabledFrame);
            }
            else {
                this.changeFrames(this.overFrame, this.upFrame, this.downFrame, this.upFrame, this.normalDisabledFrame);
            }
        }
        set disabled(val) {
            this.toggled = this.t;
            super.disabled = val;
        }
    }
    exports.ToggleButton = ToggleButton;
});
define("switchback-common/play/PlayCameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCameraUtil {
        /**
         * The transparent class references the project style.css file, it MUST be set in there
         * for the default value to work. If not, the camera stream can be blacked out.
         * .transparent {
         *     background: rgba(0, 0, 0, 0);
         * }
         * This allows the bg and transparency to be specified in css and not in the code base.
         * The bg color can also be changed throughout the play session and closing of the camera
         * doesn't need to care about that information.
         */
        static startCamera(cameraType, transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PlayAPI.startCamera(cameraType).then((apiResult) => {
                        document.body.classList.add(transparentClass);
                        resolve(true);
                        console.log("[PlayCameraUtil] StartCamera accepted: " + JSON.stringify(apiResult));
                    }, (apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera rejected: " + JSON.stringify(apiResult));
                    }).catch((apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera exception: " + JSON.stringify(apiResult));
                    });
                }));
            });
        }
        static captureImage(game, timeout = 5000, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[PlayCameraUtil] captureImage()");
                yield Promise.race([
                    PlayAPI.captureImage(),
                    PromiseUtil_23.PromiseUtil.time(game, timeout),
                ]).then((result) => {
                    if (!result) {
                        return;
                    }
                    if (result.imageData) {
                        return PlayCameraUtil.loadCaptureImage(game, result.imageData, imageName);
                    }
                    else {
                        console.log("[PlayCameraUtil.captureCamera] Unable to get imageData from result");
                    }
                });
            });
        }
        static loadCaptureImage(game, imageData, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    const loader = new Phaser.Loader(game);
                    loader.image(imageName, imageData);
                    loader.onLoadComplete.addOnce(resolve, this);
                    loader.start();
                });
            });
        }
        static closeCamera(transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                document.body.classList.remove(transparentClass);
                return PlayAPI.endCamera();
            });
        }
        static checkCameraAccess(promptForCameraPermissionCallback) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[PlayCameraUtil] cameraAccessStep()`);
                let cameraStatus = CameraAccessStatus.NOT_DETERMINED;
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 1: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    return CameraAccessStatus.DENIED;
                });
                if (promptForCameraPermissionCallback) {
                    yield promptForCameraPermissionCallback(cameraStatus);
                }
                // Recheck camera permissions to see if they've accepted them now
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 2: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    cameraStatus = CameraAccessStatus.DENIED;
                });
                return cameraStatus;
            });
        }
    }
    exports.PlayCameraUtil = PlayCameraUtil;
});
define("switchback-common/component/camera/PlayCamera", ["require", "exports", "switchback-common/play/PlayCameraUtil", "switchback-common/util/DeviceUtil", "switchback-common/util/PromiseUtil"], function (require, exports, PlayCameraUtil_1, DeviceUtil_2, PromiseUtil_24) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCamera extends Phaser.Group {
        constructor(game, overlayAsset, parent = null) {
            super(game, parent, null, false);
            this._capturedImage = new Phaser.Image(this.game, 0, 0, "");
            this._capturedImage.visible = false;
            this.add(this._capturedImage);
            if (overlayAsset) {
                this._overlay = new Phaser.Image(this.game, 0, 0, overlayAsset);
                this._overlay.width = this.game.width;
                this._overlay.height = this.game.height;
                this.add(this._overlay);
            }
        }
        openCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!PlayCamera.cameraPermissionsChecked) {
                    yield PlayCameraUtil_1.PlayCameraUtil.checkCameraAccess(this.checkCameraCallBack.bind(this));
                    PlayCamera.cameraPermissionsChecked = true;
                }
                // We can't trust the CameraAccessStatus, so we'll attempt to start the camera regardless
                const isSuccess = PlayCameraUtil_1.PlayCameraUtil.startCamera(CameraType.BACK_CAMERA);
                if (isSuccess) {
                    // This is necessary so the ui actually process the changes
                    // otherwise the player can tap to take the image too quickly causing a black photo
                    yield PromiseUtil_24.PromiseUtil.time(this.game, 100);
                }
                return isSuccess;
            });
        }
        /**
         * PlayAPI.endCamera() was not returning on Android devices as at 03/09/2018
         * so could not be awaited on as was the original scanner flow. Safe bet is to
         * ignore the result and handle any situation where the camera may still be open.
         */
        closeCamera(hideCapturedImage = true) {
            this._capturedImage.visible = !hideCapturedImage;
            PlayCameraUtil_1.PlayCameraUtil.closeCamera();
            PlayAPI.endCamera();
        }
        capture(timeout = 5000) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PlayCameraUtil_1.PlayCameraUtil.captureImage(this.game, timeout, PlayCamera.CAPTURE_TEXTURE);
                this.loadTexture();
                // Give some time for the frame to render before performing any other actions.
                // Calling endCamera too soon for example can cause a black flash between camera feed and photo
                yield PromiseUtil_24.PromiseUtil.time(this.game, 100);
            });
        }
        hideCaptureImage() {
            this._capturedImage.visible = false;
        }
        destroy() {
            this.game.cache.removeImage(PlayCamera.CAPTURE_TEXTURE);
            super.destroy();
        }
        onShowCameraPermissionsWarning() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        onShowCameraEnablePermissionsDialog(isIos) {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        loadTexture() {
            if (this.game.cache.checkImageKey(PlayCamera.CAPTURE_TEXTURE)) {
                this._capturedImage.loadTexture(PlayCamera.CAPTURE_TEXTURE);
                // Adjust it because the data comes back rotated...
                this._capturedImage.height = this.game.width;
                this._capturedImage.scale.x = this._capturedImage.scale.y;
                this._capturedImage.angle = 90;
                this._capturedImage.anchor.set(0.5);
                this._capturedImage.position.set(0, 0);
                this._capturedImage.centerX = this.game.width * 0.5;
                this._capturedImage.centerY = this.game.height * 0.5;
                this._capturedImage.visible = true;
            }
        }
        checkCameraCallBack(status) {
            return __awaiter(this, void 0, void 0, function* () {
                // Show the appropriate dialog
                if (status === CameraAccessStatus.AUTHORIZED) {
                    // We don't need to do anything. They've accepted the permissions already.
                    return;
                }
                else if (status === CameraAccessStatus.NOT_DETERMINED) {
                    // This device hasn't been asked for camera perms before.
                    // We should show a dialog to let them know they should accept them.
                    yield this.onShowCameraPermissionsWarning();
                }
                else if (status === CameraAccessStatus.DENIED) {
                    // Fall back to assuming they've declined.
                    // This dialog has the most info about enabling permissions.
                    yield this.onShowCameraEnablePermissionsDialog(DeviceUtil_2.DeviceUtil.isIos());
                }
                else if (status === CameraAccessStatus.RESTRICTED) {
                    // If the camera is Restricted, the user can't change it. We must handle it as if there were no camera.
                    // We won't show anything, and B-Show needs to be shown
                }
            });
        }
    }
    PlayCamera.CAPTURE_TEXTURE = "camera-image-capture";
    PlayCamera.cameraPermissionsChecked = false;
    exports.PlayCamera = PlayCamera;
});
define("switchback-common/component/keyboard/SpecialChars", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var SpecialChar;
    (function (SpecialChar) {
        SpecialChar["DELETE"] = "";
        SpecialChar["SUBMIT"] = "OK";
        SpecialChar["CANCEL"] = "CANCEL";
    })(SpecialChar = exports.SpecialChar || (exports.SpecialChar = {}));
});
define("switchback-common/component/keyboard/KeyCode", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/keyboard/Key", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, TextUtil_24, SpecialChars_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Key extends Phaser.Group {
        constructor(game, keyCode, keyTexture, width, height, fontStyle) {
            super(game);
            this.pressedSignal = new Phaser.Signal();
            const keyBg = new Phaser.Button(this.game, 0, 0, keyTexture, () => { this.pressedSignal.dispatch(keyCode); }, this, 0, 0, 1, 0);
            keyBg.width = width * Key.keyWidthMultiplier(keyCode);
            keyBg.height = height;
            this.add(keyBg);
            this.keyChar = new Phaser.Text(this.game, keyBg.centerX, keyBg.centerY, keyCode, fontStyle);
            this.keyChar.anchor.set(0.5, 0.45);
            TextUtil_24.TextUtil.scaleDownSingleLineTextToWidth(this.keyChar, keyBg.width * 0.95);
            this.add(this.keyChar);
        }
        static keyWidthMultiplier(char) {
            switch (char) {
                case SpecialChars_1.SpecialChar.DELETE:
                    return 1.56;
                case SpecialChars_1.SpecialChar.CANCEL:
                    return 1.83;
                case SpecialChars_1.SpecialChar.SUBMIT:
                    return 3.125;
                default:
                    return 1;
            }
        }
        get keyFontSize() {
            return this.keyChar.fontSize;
        }
        set keyFontSize(size) {
            this.keyChar.fontSize = size;
        }
    }
    exports.Key = Key;
});
define("switchback-common/component/keyboard/KeyLayoutData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayoutData {
    }
    exports.KeyLayoutData = KeyLayoutData;
});
define("switchback-common/component/keyboard/KeyLayouts", ["require", "exports", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, SpecialChars_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayouts {
    }
    KeyLayouts.LETTERS = [
        ["Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", SpecialChars_2.SpecialChar.DELETE],
        ["A", "S", "D", "F", "G", "H", "J", "K", "L", SpecialChars_2.SpecialChar.CANCEL],
        ["Z", "X", "C", "V", "B", "N", "M", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    KeyLayouts.NUMBERS = [
        ["7", "8", "9"],
        ["4", "5", "6"],
        ["1", "2", "3"],
        [SpecialChars_2.SpecialChar.DELETE, "0", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    exports.KeyLayouts = KeyLayouts;
});
define("switchback-common/component/keyboard/KeyboardOptions", ["require", "exports", "switchback-common/component/keyboard/KeyLayouts"], function (require, exports, KeyLayouts_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardOptions {
        constructor(params) {
            params = params || {};
            this.bgTexture = params.bgTexture || "kb-bg";
            this.cancelTexture = params.cancelTexture || "kb-cancel";
            this.charLimit = params.charLimit || 13;
            this.deleteTexture = params.deleteTexture || "kb-delete";
            this.keys = params.keys || KeyLayouts_1.KeyLayouts.LETTERS;
            this.keysTexture = params.keysTexture || "kb-regular";
            this.labelText = params.labelText || "";
            this.output = (params.output === undefined) ? true : params.output;
            this.outputBgTexture = params.outputBgTexture || "kb-output";
            this.submitTexture = params.submitTexture || "kb-submit";
            this.xPaddingProportion = (params.xPaddingProportion === undefined) ? 0.15 : params.xPaddingProportion;
            this.keysStyle = params.keyStyle || { fill: "#FFF", fontSize: 44 };
            this.labelStyle = params.labelStyle || { fill: "#FFF", fontSize: 44 };
            this.outputStyle = params.outputStyle || { fill: "#FFF", fontSize: 44 };
        }
    }
    exports.KeyboardOptions = KeyboardOptions;
});
define("switchback-common/component/keyboard/KeyboardController", ["require", "exports", "switchback-common/component/Signal", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, Signal_2, SpecialChars_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardController {
        constructor(options) {
            this._currentString = "";
            this.stringChangedSignal = new Signal_2.Signal();
            this.submitSignal = new Signal_2.Signal();
            this.cancelSignal = new Signal_2.Signal();
            this._options = options;
        }
        get options() {
            return this._options;
        }
        getTextureForKey(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    return this.options.submitTexture;
                }
                case SpecialChars_3.SpecialChar.DELETE: {
                    return this.options.deleteTexture;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    return this.options.cancelTexture;
                }
                default: {
                    return this.options.keysTexture;
                }
            }
        }
        keyPressed(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.DELETE: {
                    if (this._currentString.length > 0) {
                        this._currentString = this._currentString.substr(0, this._currentString.length - 1);
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                    break;
                }
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    this.submitSignal.dispatch(this._currentString);
                    break;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    this.cancelSignal.dispatch();
                    break;
                }
                default: {
                    if (this._currentString.length < this._options.charLimit) {
                        this._currentString += keyCode;
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                }
            }
        }
        get currentString() {
            return this._currentString;
        }
        clearString() {
            this._currentString = "";
        }
        destroy() {
            this.stringChangedSignal.dispose();
            this.submitSignal.dispose();
            this.cancelSignal.dispose();
        }
    }
    exports.KeyboardController = KeyboardController;
});
define("switchback-common/component/keyboard/Keyboard", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/Key", "switchback-common/util/ScaleUtil"], function (require, exports, KeyboardController_1, Key_1, ScaleUtil_27) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Keyboard extends Phaser.Group {
        get onSubmit() { return this.controller.submitSignal; }
        get onCancel() { return this.controller.cancelSignal; }
        get onChange() { return this.controller.stringChangedSignal; }
        constructor(game, bounds, options) {
            super(game);
            this.keysStyle = options.keysStyle;
            this.bounds = bounds;
            this.controller = new KeyboardController_1.KeyboardController(options);
            const label = new Phaser.Text(this.game, this.bounds.centerX, this.bounds.height * -0.3, options.labelText, options.labelStyle);
            label.anchor.set(0.5);
            this.add(label);
            this.keys = new Phaser.Group(this.game);
            this.add(this.keys);
            let keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.1, this.bounds.width, this.bounds.height);
            if (options.output) {
                const outputBG = new Phaser.Image(game, 0, 0, options.outputBgTexture);
                ScaleUtil_27.scaleToSingleBound(outputBG, bounds.width * 0.67);
                outputBG.centerX = this.bounds.width * 0.5;
                outputBG.centerY = this.bounds.height * -0.075;
                this.add(outputBG);
                this.outputString = new Phaser.Text(this.game, outputBG.centerX, outputBG.top + outputBG.height * 0.55, "", options.outputStyle);
                this.outputString.anchor.set(0.5);
                this.add(this.outputString);
                this.controller.stringChangedSignal.add(this.onStringChanged, this);
                keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.2, this.bounds.width, this.bounds.height * 0.75);
            }
            this.showKeys(this.controller.options.keys, keyBounds);
            this.bg = new Phaser.Image(game, 0, 0, options.bgTexture);
            this.bg.width = bounds.width;
            this.bg.height = bounds.height * 1.2;
            if (options.output) {
                this.bg.height = bounds.height * 1.5;
            }
            this.addAt(this.bg, 0);
        }
        destroy() {
            this.controller.destroy();
            super.destroy();
        }
        get currentString() {
            return this.controller.currentString;
        }
        clearString() {
            this.controller.clearString();
            this.outputString.text = this.controller.currentString;
        }
        showKeys(keys, keyBounds) {
            const layout = this.calculateKeyLayout(keys, keyBounds);
            let xPos;
            let yPos = keyBounds.y;
            keys.forEach((row) => {
                let rowWidthMulti = 0;
                for (let i = 0; i < row.length; i++) {
                    rowWidthMulti += Key_1.Key.keyWidthMultiplier(row[i]);
                }
                const totalKeyWidth = rowWidthMulti * layout.width;
                const totalSpaceWidth = (row.length - 1) * layout.xSpacing;
                const xOffset = (keyBounds.width - totalKeyWidth - totalSpaceWidth) * 0.5;
                xPos = xOffset + keyBounds.x + (layout.xSpacing * 0.5);
                row.forEach((key) => {
                    const keySprite = new Key_1.Key(this.game, key, this.controller.getTextureForKey(key), layout.width, layout.height, this.keysStyle);
                    keySprite.position.set(xPos, yPos);
                    this.keys.add(keySprite);
                    keySprite.pressedSignal.add((k) => {
                        this.controller.keyPressed(k);
                    });
                    xPos += layout.width + layout.xSpacing;
                }, this);
                yPos += layout.height + layout.ySpacing;
            }, this);
            /**
             * Now that we've added all the keys, we need to scale their
             * text down uniformly so that they fit on the key.
             */
            let minSize = Infinity;
            this.keys.children.forEach((child) => {
                if (child.keyFontSize) {
                    minSize = Math.min(child.keyFontSize, minSize);
                }
            });
            this.keys.children.forEach((child) => {
                child.keyFontSize = minSize;
            });
        }
        calculateKeyLayout(keys, keyBounds) {
            let longestRow = 0;
            keys.forEach((row) => {
                longestRow = Math.max(row.length, longestRow);
            });
            longestRow++; // Give us some extra room for special characters
            const width = (keyBounds.width * (1 - this.controller.options.xPaddingProportion)) / longestRow;
            const xSpacing = (keyBounds.width * this.controller.options.xPaddingProportion) / longestRow;
            const height = (keyBounds.height * 0.9) / keys.length;
            const ySpacing = (keyBounds.height * 0.15) / keys.length;
            return {
                width: width,
                height: height,
                xSpacing: xSpacing,
                ySpacing: ySpacing,
            };
        }
        onStringChanged(newString) {
            this.outputString.text = newString;
        }
    }
    exports.Keyboard = Keyboard;
});
define("switchback-common/component/scrolling/IScrollableGroupSettings", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/scrolling/ScrollableGroup", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ScrollableGroup extends Phaser.Group {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, parent, null, false);
            this._settings = {
                horizontalScroll: false,
                paddingBottom: 0,
                paddingRight: 0,
                timeConstantScroll: 325,
                verticalScroll: true,
            };
            this._hasAddedListeners = false;
            this.settings = settings;
            this._previousPosition = new Phaser.Point();
            this._velocity = new Phaser.Point();
            this._autoScrollTarget = new Phaser.Point();
            this._autoScrollAmplitude = new Phaser.Point();
            this._contents = new Phaser.Group(this.game, null, null, false);
            this._displayMask = new Phaser.Graphics(this.game);
            this._displayMask.beginFill(0x000000);
            this._displayMask.drawRect(0, 0, viewableWidth, viewableHeight);
            this._displayMask.alpha = 0;
            this._displayMask.endFill();
            this._inputId = -1;
            this.add(this._displayMask);
            this.add(this._contents);
            // Hijack the group's add function to ensure all game objects are
            // added to the content layer that gets scrolled
            this.add = this.addToContentLayer.bind(this);
        }
        static addTapInputSignal(game, obj, clickSignal) {
            const tapDistance = game.width * 0.05;
            let startClickPosition = null;
            let endClickPosition = null;
            obj.inputEnabled = true;
            obj.events.onInputDown.add(() => {
                startClickPosition = new Phaser.Point(game.input.x, game.input.y);
            }, this);
            obj.events.onInputUp.add(() => {
                endClickPosition = new Phaser.Point(game.input.x, game.input.y);
                const distance = Phaser.Point.distance(startClickPosition, endClickPosition, true);
                if (distance < tapDistance) {
                    clickSignal.dispatch();
                }
            }, this);
        }
        set settings(settings) {
            for (const property in settings) {
                if (this._settings.hasOwnProperty(property)) {
                    this._settings[property] = settings[property];
                }
            }
        }
        onContentsUpdated() {
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
        }
        start() {
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
            this._contents.position.set(0, 0);
        }
        updateListeners() {
            const isHorizontalScrollable = this._settings.horizontalScroll &&
                this._contentLocalBounds.x + this._contentLocalBounds.width > this._displayMask.width;
            const isVerticalScrollable = this._settings.verticalScroll &&
                this._contentLocalBounds.y + this._contentLocalBounds.height > this._displayMask.height;
            // If it should be scrollable (in either direction), but has no listeners - add 'em all
            if (isHorizontalScrollable || isVerticalScrollable && !this._hasAddedListeners) {
                this.addListeners();
            }
            else if (!isHorizontalScrollable && !isVerticalScrollable && this._hasAddedListeners) {
                // Otherwise if it shouldn't be scrollable (in both directions) but has listeners - remove them!
                this.removeListeners();
            }
        }
        // TODO: Find a way to smooth this. Tween doesn't work 
        // when content is undersized due to bounds check
        moveToBottom() {
            this._contents.y = -(this._contents.height - this._displayMask.height);
        }
        stop() {
            this.removeListeners();
            this.reset();
        }
        update() {
            super.update();
            if (this._inputId >= 0) {
                return;
            }
            const elapsed = Date.now() - this._timestamp;
            if (this._autoScrollAmplitude.x !== 0) {
                const delta = -this._autoScrollAmplitude.x * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.x = this._autoScrollTarget.x + delta;
                }
                else {
                    this._autoScrollAmplitude.x = 0;
                }
            }
            if (this._autoScrollAmplitude.y !== 0) {
                const delta = -this._autoScrollAmplitude.y * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.y = this._autoScrollTarget.y + delta;
                }
                else {
                    this._autoScrollAmplitude.y = 0;
                }
            }
            this.limitMovement();
        }
        reset() {
            this._inputId = -1;
            this._timestamp = Date.now();
            this._velocity.set(0, 0);
            this._autoScrollTarget.set(0, 0);
            this._autoScrollAmplitude.set(0, 0);
        }
        addListeners() {
            if (!this._hasAddedListeners) {
                this.game.input.onDown.add(this.beginMove, this);
                this.game.input.addMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.add(this.endMove, this);
                this._displayMask.inputEnabled = true;
                this._hasAddedListeners = true;
            }
        }
        removeListeners() {
            if (this._hasAddedListeners) {
                this.game.input.onDown.remove(this.beginMove, this);
                this.game.input.deleteMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.remove(this.endMove, this);
                this._displayMask.inputEnabled = false;
                this._hasAddedListeners = false;
            }
        }
        beginMove(pointer) {
            if (this._inputId < 0 && this._displayMask.getBounds().contains(pointer.x, pointer.y)) {
                this.reset();
                this._inputId = pointer.id;
                this._previousPosition.set(pointer.x, pointer.y);
            }
        }
        endMove(pointer) {
            if (this._inputId === pointer.id) {
                if (this.game.input.activePointer.withinGame) {
                    const elapsed = Date.now() - this._timestamp;
                    if (elapsed <= 500) {
                        if (this._velocity.x > 20 || this._velocity.x < -20) {
                            this._autoScrollAmplitude.x = this._velocity.x;
                            this._autoScrollTarget.x = Math.round(this._contents.x + this._autoScrollAmplitude.x);
                        }
                        if (this._velocity.y > 20 || this._velocity.y < -20) {
                            this._autoScrollAmplitude.y = this._velocity.y;
                            this._autoScrollTarget.y = Math.round(this._contents.y + this._autoScrollAmplitude.y);
                        }
                    }
                }
                this._inputId = -1;
            }
        }
        moveCanvas(pointer, x, y) {
            if (this._inputId !== pointer.id) {
                return;
            }
            const now = Date.now();
            const elapsed = now - this._timestamp;
            this._timestamp = now;
            // If horizontal scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.horizontalScroll) {
                const delta = x - this._previousPosition.x;
                this._previousPosition.x = x;
                this._contents.position.x += delta;
                this._velocity.x = 100 * (delta / elapsed);
            }
            // If vertical scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.verticalScroll) {
                const delta = y - this._previousPosition.y;
                this._previousPosition.y = y;
                this._contents.position.y += delta;
                this._velocity.y = 100 * (delta / elapsed);
            }
            this.limitMovement();
        }
        limitMovement() {
            if (this._settings.horizontalScroll) {
                this.limitHorizontal();
            }
            if (this._settings.verticalScroll) {
                this.limitVertical();
            }
        }
        limitHorizontal() {
            if (this._contents.position.x >= 0) {
                this._contents.position.x = 0;
            }
            else {
                const contentWidth = this._contentLocalBounds.width + this._contentLocalBounds.x;
                const limit = this._displayMask.width - contentWidth - this._settings.paddingRight;
                if (this._contents.position.x <= limit) {
                    this._contents.position.x = limit;
                }
            }
        }
        limitVertical() {
            if (this._contents.position.y >= 0) {
                this._contents.position.y = 0;
            }
            else {
                const contentHeight = this._contentLocalBounds.height + this._contentLocalBounds.y;
                const limit = this._displayMask.height - contentHeight - this._settings.paddingBottom;
                if (this._contents.position.y <= limit) {
                    this._contents.position.y = limit;
                }
            }
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToContentLayer(object) {
            this._contents.add(object);
        }
    }
    exports.ScrollableGroup = ScrollableGroup;
});
define("switchback-common/component/scrolling/ScrollableMaskGroup", ["require", "exports", "switchback-common/component/scrolling/ScrollableGroup"], function (require, exports, ScrollableGroup_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A scrollable group whose contents outside the display area are masked out.
     */
    class ScrollableMaskGroup extends ScrollableGroup_1.ScrollableGroup {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, viewableWidth, viewableHeight, settings, parent);
            const mask = this.game.add.graphics(0, 0);
            mask.beginFill(0x000000, 0);
            mask.drawRect(0, 0, this._displayMask.width, this._displayMask.height);
            mask.endFill();
            this._contents.mask = mask;
        }
        /**
         * As the mask is added globally, we have to ensure we're masking from the correct location.
         * This must be called after this object moves.
         */
        updateMaskPosition() {
            this._contents.mask.x = this.worldPosition.x;
            this._contents.mask.y = this.worldPosition.y;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        beginMove(pointer) {
            super.beginMove(pointer);
            this._contents.mask.dirty = true;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        onContentsUpdated() {
            super.onContentsUpdated();
            this._contents.mask.dirty = true;
        }
    }
    exports.ScrollableMaskGroup = ScrollableMaskGroup;
});
define("switchback-common/component/text/TypewriterText", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_25) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TypewriterText extends Phaser.Text {
        constructor() {
            super(...arguments);
            this.isSkipping = false;
            this.softSkip = false;
        }
        typewriter(letterDelay, soundTag = null, softSkipParent = null, shutdownSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (softSkipParent && !this.isSkipping) {
                    softSkipParent.events.onInputDown.addOnce(this.onSoftSkip, this);
                }
                let sndTypewriterLoop;
                let soundVolumeTween;
                if (soundTag && !this.isSkipping) {
                    sndTypewriterLoop = this.game.sound.play(soundTag, 0.0, true);
                    sndTypewriterLoop.play();
                    sndTypewriterLoop.volume = 0;
                    soundVolumeTween = this.game.add.tween(sndTypewriterLoop).to({ volume: 0.5 }, 500, Phaser.Easing.Linear.None, true);
                }
                const numChars = this.text.length;
                this.softSkip = false;
                let isShutdown = false;
                for (let i = 0; !this.isSkipping && !this.softSkip && i < numChars; i++) {
                    this.clearColors();
                    this.addColor("rgba(0,0,0,0)", i);
                    yield PromiseUtil_25.PromiseUtil.realTime(letterDelay, shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        if (sndTypewriterLoop) {
                            soundVolumeTween.stop();
                            sndTypewriterLoop.stop();
                        }
                        return;
                    }
                }
                if (softSkipParent) {
                    softSkipParent.events.onInputDown.remove(this.onSoftSkip, this);
                }
                this.clearColors();
                // Adding colors offsets the text kerning differently, so this has to be here for consistency even though it doesn't color anything
                this.addColor("rgba(0,0,0,0)", numChars);
                this.softSkip = false;
                if (sndTypewriterLoop) {
                    soundVolumeTween.stop();
                    sndTypewriterLoop.stop();
                }
            });
        }
        clearText() {
            this.addColor("rgba(0,0,0,0)", 0);
        }
        onSoftSkip() {
            this.softSkip = true;
        }
    }
    exports.TypewriterText = TypewriterText;
});
define("switchback-common/extension/Easing", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function shake(aPeriod1, aPeriod2) {
        return (k) => {
            const current1 = k * Math.PI * 2 * aPeriod1;
            const current2 = k * (Math.PI * 2 * aPeriod2 + Math.PI / 2);
            return Math.sin(current1) * Math.cos(current2);
        };
    }
    exports.shake = shake;
});
/**
 * I can't pretend I understand this...
 * Found @ http://tiny.cc/uppary
 *
 * Modified:
 * 1. to fix retrieving the image from Phaser cache
 * 2. added frame default as 0
 *
 * Note: Don't configure a NineSlice to be smaller than its borders
 * e.g. if it has 100px borders make sure it's at least 200x200
 */
define("switchback-common/extension/NineSlice", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NineSlice {
        static create(aGame, aWidth, aHeight, aKey, aFrame = 0, aTop = 0, aLeft = 0, aBottom = 0, aRight = 0, aRepeats = false) {
            // image
            NineSlice.image = new Phaser.Image(aGame, 0, 0, aKey);
            // get frame
            if (typeof aFrame === "string") {
                NineSlice.frame = aGame.cache.getFrameByName(aKey, aFrame);
            }
            else {
                NineSlice.frame = aGame.cache.getFrameByIndex(aKey, aFrame);
            }
            // calculate dims
            NineSlice.calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats);
            // create nine image bitmap data
            NineSlice.nineImage = new Phaser.BitmapData(aGame, "NineImage" + (NineSlice.textureKey++), NineSlice.width, NineSlice.height);
            // render into bitmap data
            NineSlice.renderNineImage();
            return NineSlice.nineImage;
        }
        // -------------------------------------------------------------------------
        static calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats) {
            const frame = NineSlice.frame;
            NineSlice.centralWidth = frame.width - aLeft - aRight;
            NineSlice.centralHeight = frame.height - aTop - aBottom;
            if (aRepeats) {
                NineSlice.horizontalRepeats = aWidth;
                NineSlice.verticalRepeats = aHeight;
                NineSlice.width = aLeft + aRight + NineSlice.centralWidth * aWidth;
                NineSlice.height = aTop + aBottom + NineSlice.centralHeight * aHeight;
                NineSlice.lastWidth = 0;
                NineSlice.lastHeight = 0;
            }
            else {
                const w = aWidth - aLeft - aRight;
                NineSlice.horizontalRepeats = Math.floor(w / NineSlice.centralWidth);
                NineSlice.lastWidth = w % NineSlice.centralWidth;
                const h = aHeight - aTop - aBottom;
                NineSlice.verticalRepeats = Math.floor(h / NineSlice.centralHeight);
                NineSlice.lastHeight = h % NineSlice.centralHeight;
                NineSlice.width = aWidth;
                NineSlice.height = aHeight;
            }
            NineSlice.leftWidth = aLeft;
            NineSlice.rightWidth = aRight;
            NineSlice.topHeight = aTop;
            NineSlice.bottomHeight = aBottom;
        }
        static renderNineImage() {
            let sourceY = NineSlice.frame.y;
            let destY = 0;
            // top row
            if (NineSlice.topHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.topHeight);
                sourceY += NineSlice.topHeight;
                destY += NineSlice.topHeight;
            }
            // centrals
            for (let i = 0; i < NineSlice.verticalRepeats; i++) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.centralHeight);
                destY += NineSlice.centralHeight;
            }
            // last height
            if (NineSlice.lastHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.lastHeight);
                destY += NineSlice.lastHeight;
            }
            sourceY += NineSlice.centralHeight;
            // bottom
            if (NineSlice.bottomHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.bottomHeight);
            }
        }
        static renderNineImageRow(aImage, aSourceY, aDestY, aHeight) {
            let sourceX = NineSlice.frame.x;
            let destX = 0;
            // left
            if (NineSlice.leftWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.leftWidth, aHeight, destX, aDestY);
                destX += NineSlice.leftWidth;
                sourceX += NineSlice.leftWidth;
            }
            // centrals
            for (let i = 0; i < NineSlice.horizontalRepeats; i++) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.centralWidth, aHeight, destX, aDestY);
                destX += NineSlice.centralWidth;
            }
            // last width
            if (NineSlice.lastWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.lastWidth, aHeight, destX, aDestY);
                destX += NineSlice.lastWidth;
            }
            sourceX += NineSlice.centralWidth;
            // right
            if (NineSlice.rightWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.rightWidth, aHeight, destX, aDestY);
            }
        }
    }
    NineSlice.textureKey = 0;
    exports.NineSlice = NineSlice;
});
const pDisplayObjCont = PIXI.DisplayObjectContainer;
const pDisplayObj = PIXI.DisplayObject;
pDisplayObjCont.prototype.displayObjectUpdateTransform = pDisplayObj.prototype.updateTransform = function (parent) {
    let a;
    let b;
    let c;
    let d;
    let p;
    let pivot;
    let position;
    let pt;
    let ref;
    let rotation;
    let scale;
    let skew;
    let skewX;
    let skewY;
    let tx;
    let ty;
    let wt;
    if (!parent && !this.parent && !this.game) {
        return this;
    }
    p = this.parent;
    if (parent) {
        p = parent;
    }
    else if (!this.parent) {
        p = this.game.world;
    }
    pt = p.worldTransform;
    wt = this.worldTransform;
    ref = this, pivot = ref.pivot, position = ref.position, rotation = ref.rotation, scale = ref.scale, skew = ref.skew;
    a = b = c = d = tx = ty = null;
    if ((rotation % PIXI.PI_2) || skew) {
        skewX = (skew && skew.x) || 0;
        skewY = (skew && skew.y) || 0;
        a = scale.x * Math.cos(rotation + skewY);
        b = scale.x * Math.sin(rotation + skewY);
        c = scale.y * Math.sin(-rotation - skewX);
        d = scale.y * Math.cos(rotation + skewX);
        tx = position.x;
        ty = position.y;
        if (pivot.x || pivot.y) {
            tx -= pivot.x * a + pivot.y * c;
            ty -= pivot.x * b + pivot.y * d;
        }
        wt.a = a * pt.a + b * pt.c;
        wt.b = a * pt.b + b * pt.d;
        wt.c = c * pt.a + d * pt.c;
        wt.d = c * pt.b + d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    else {
        a = scale.x;
        d = scale.y;
        tx = position.x - (pivot.x * a);
        ty = position.y - (pivot.y * d);
        wt.a = a * pt.a;
        wt.b = a * pt.b;
        wt.c = d * pt.c;
        wt.d = d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    this.worldAlpha = this.alpha * p.worldAlpha;
    this.worldPosition.x = wt.tx;
    this.worldPosition.y = wt.ty;
    this.worldScale.x = scale.x * Math.sqrt(wt.a * wt.a + wt.c * wt.c);
    this.worldScale.y = scale.y * Math.sqrt(wt.b * wt.b + wt.d * wt.d);
    this.worldRotation = Math.atan2(-wt.c, wt.d);
    this._currentBounds = null;
    if (this.transformCallback) {
        this.transformCallback.call(this.transformCallbackContext, wt, pt);
    }
    return this;
};
define("switchback-common/play/PlayController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class PlayController {
        constructor() {
            window.webkit = {};
            window.webkit.messageHandlers = {};
            window.webkit.messageHandlers.NativeInterop = {};
            window.webkit.messageHandlers.NativeInterop.postMessage = (obj) => { return; };
        }
        start() {
            console.log("[PlayController] start()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
        eject() {
            console.log("[PlayController] eject()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        movePlayer(location) {
            console.log(`[PlayController] movePlayer(${location})`);
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: location,
            }));
        }
    }
    exports.PlayController = PlayController;
});
define("switchback-common/play/DummyApiRunner", ["require", "exports", "switchback-common/play/PlayController", "switchback-common/util/PromiseUtil"], function (require, exports, PlayController_1, PromiseUtil_26) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class DummyApiRunner {
        constructor() {
            this.controller = new PlayController_1.PlayController();
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_26.PromiseUtil.realTime(3000);
                this.controller.start();
                this.controller.movePlayer({
                    attractionName: "Little Foot's Wild Ride",
                    landName: "Land Before Time",
                    latitude: 66.33,
                    locationID: "beacon_" + Math.floor(Math.random() * 20),
                    longitude: 100.22,
                    parkName: "Animal Kingdom",
                });
                yield PromiseUtil_26.PromiseUtil.realTime(1500);
                this.controller.eject();
            });
        }
    }
    exports.DummyApiRunner = DummyApiRunner;
});
define("switchback-common/util/CollectionUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CollectionUtil {
        /**
         * Swaps two values of @param array by value (in place)
         */
        static swapByValue(array, x, y) {
            const xIndex = array.indexOf(x);
            const yIndex = array.indexOf(y);
            CollectionUtil.swapByIndex(array, xIndex, yIndex);
        }
        /**
         * Swaps two values of @param array by index (in place)
         */
        static swapByIndex(array, xIndex, yIndex) {
            if (xIndex >= 0 && yIndex >= 0) {
                let a = array[xIndex];
                array[xIndex] = array[yIndex];
                array[yIndex] = a;
            }
            else {
                console.warn(`[CollectionUtil] swap(): could not swap elements - one or more elements not present in array`);
            }
        }
    }
    exports.CollectionUtil = CollectionUtil;
    function shuffle(array) {
        const arrayCopy = array;
        for (let i = array.length; i; i--) {
            const j = Math.floor(Math.random() * i);
            [arrayCopy[i - 1], arrayCopy[j]] = [array[j], array[i - 1]];
        }
        return arrayCopy;
    }
    exports.shuffle = shuffle;
    // puts the last index in first place before shuffling the rest
    function shuffleNonRepeatingArray(originalArray) {
        if (originalArray.length <= 1) {
            return originalArray;
        }
        // removes the last item in the array
        const lastItem = originalArray.pop();
        // shuffle the array without the last item in it
        originalArray = shuffle(originalArray);
        /*
        * inserts the last item back into the middle of the array
        * so that the it will not end up as lastItem forever
        */
        originalArray.splice(Math.ceil(originalArray.length * 0.5), 0, lastItem);
        return originalArray;
    }
    exports.shuffleNonRepeatingArray = shuffleNonRepeatingArray;
    /**
     * Finds the 2D index of an item in a 2D array
     * @param arr Array to search
     * @param k Item to look for
     */
    function getIndexIn2DArray(arr, k) {
        for (let i = 0; i < arr.length; i++) {
            const index = arr[i].indexOf(k);
            if (index > -1) {
                return { row: i, col: index };
            }
        }
    }
    exports.getIndexIn2DArray = getIndexIn2DArray;
    /**
     * Returns an array of numbers from 0 to length exclusive eg. getNumberRange(3) returns [0, 1, 2]
     * @param length number, the end of the range
     */
    function getNumberRange(length) {
        return Array.apply(null, { length: length }).map(Number.call, Number);
    }
    exports.getNumberRange = getNumberRange;
    /**
     * Returns an array of numbers from start (inclusive) to start + length (exclusive) eg. getNumberRange(2, 3) returns [2, 3, 4]
     * @param start number, the first number in the range
     * @param length number, the end of the range
     */
    function getNumberRangeFrom(start, length) {
        let range = [];
        getNumberRange(length).forEach((n) => {
            range.push(n + start);
        });
        return range;
    }
    exports.getNumberRangeFrom = getNumberRangeFrom;
    function cloneArray(arr) {
        let i, copy;
        if (Array.isArray(arr)) {
            copy = arr.slice(0);
            for (i = 0; i < copy.length; i++) {
                copy[i] = cloneArray(copy[i]);
            }
            return copy;
        }
        else if (typeof arr === 'object') {
            throw 'Cannot clone array containing an object!';
        }
        else {
            return arr;
        }
    }
    exports.cloneArray = cloneArray;
    function exclusiveLeftJoinArrays(leftList, rightList) {
        const exclusiveLeftJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) === -1;
        });
        return exclusiveLeftJoin;
    }
    exports.exclusiveLeftJoinArrays = exclusiveLeftJoinArrays;
    /**
     * Return an array containing the matching elements from two arrays
     */
    function innerJoinArrays(leftList, rightList) {
        const innerJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) >= 0;
        });
        return innerJoin;
    }
    exports.innerJoinArrays = innerJoinArrays;
    function getRandomItem(list) {
        const randomIndex = Math.floor(Math.random() * Math.floor(list.length));
        return list.length === 0 ? null : list[randomIndex];
    }
    exports.getRandomItem = getRandomItem;
});
define("switchback-common/play/MiniGamePlaySession", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGamePlaySession {
        constructor(playerList) {
            this._playGroups = [];
            this._playerList = playerList;
        }
        startMiniGame(playerCount) {
            switch (playerCount) {
                case MiniGamePlaySession.PlayerCount.ALL:
                    this._playGroups = MiniGamePlaySession.generateAllPlayerGroup(this._playerList);
                    break;
                case MiniGamePlaySession.PlayerCount.SINGLE:
                    this._playGroups = MiniGamePlaySession.generateSinglePlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
                case MiniGamePlaySession.PlayerCount.TWO:
                    this._playGroups = MiniGamePlaySession.generateTwoPlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
            }
            this._currentPlayGroupIndex = 0;
        }
        playGroupFinished() {
            this._currentPlayGroupIndex++;
        }
        get roundIndex() {
            return this._currentPlayGroupIndex;
        }
        get isMiniGameSessionFinished() {
            return this._currentPlayGroupIndex >= this._playGroups.length;
        }
        get currentPlayGroup() {
            return this._playGroups[this._currentPlayGroupIndex];
        }
        get previousMiniGameLastPlayers() {
            return this._playGroups.length === 0 ? [] : this._playGroups[this._playGroups.length - 1].players;
        }
        static generateAllPlayerGroup(playerList) {
            const playGroups = [];
            playGroups.push({ players: playerList });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will only contain a single player.
         * The first group will not include any player from the list of players who should not play first
         * - Exception to this rule is if we only have one player
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateSinglePlayerGroups(playerList, shouldNotPlayFirstList) {
            const playGroups = [];
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_1.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_1.getRandomItem(playersAvailableToBeFirst);
            if (randomFirstPlayer) {
                playGroups.push({ players: [randomFirstPlayer] });
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
            }
            CollectionUtil_1.shuffle(playerPool);
            playerPool.forEach((player) => {
                playGroups.push({ players: [player] });
            });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will contain two players
         * The first group will have at least one player who is not in the list of players who should not play first
         * - Exception to this rule if we only have 2 or 3 players
         * NOTE: If we have less than two players then an empty mini game session is returned
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateTwoPlayerGroups(playerList, shouldNotPlayFirstList) {
            if (playerList.length < 2) {
                console.error(`[MiniGamePlaySession] Unable to generate play groups for two players when we don't have that many players`);
                return [];
            }
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_1.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_1.getRandomItem(playersAvailableToBeFirst);
            CollectionUtil_1.shuffle(playerPool);
            if (randomFirstPlayer) {
                // Remove the randomly selected player from the player pool,
                // Then insert them second in the array so that they get the first two plays
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
                playerPool.splice(1, 0, randomFirstPlayer);
            }
            if (playerPool.length === 2) {
                return [{ players: playerPool }];
            }
            const playSession = [];
            for (let i = 0; i < playerPool.length; i++) {
                const playerOne = playerPool[i];
                const playerTwo = (i + 1 === playerPool.length) ? playerPool[0] : playerPool[i + 1];
                const group = [playerOne, playerTwo];
                if (i % 2 === 1) {
                    group.reverse();
                }
                playSession.push({ players: group });
            }
            return playSession;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the current play group index to be set
         * @param index
         */
        TEST_setCurrentPlayGroupIndex(index) {
            this._currentPlayGroupIndex = index;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the last know players to be set
         * @param lastKnowPlayers
         */
        TEST_setLastKnownPlayers(lastKnowPlayers) {
            this._playGroups = [{ players: lastKnowPlayers }];
        }
    }
    exports.MiniGamePlaySession = MiniGamePlaySession;
    (function (MiniGamePlaySession) {
        let PlayerCount;
        (function (PlayerCount) {
            PlayerCount[PlayerCount["ALL"] = 0] = "ALL";
            PlayerCount[PlayerCount["SINGLE"] = 1] = "SINGLE";
            PlayerCount[PlayerCount["TWO"] = 2] = "TWO";
        })(PlayerCount = MiniGamePlaySession.PlayerCount || (MiniGamePlaySession.PlayerCount = {}));
    })(MiniGamePlaySession = exports.MiniGamePlaySession || (exports.MiniGamePlaySession = {}));
});
define("switchback-common/play/PlayerManager", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayerManager {
        openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                let players;
                try {
                    const result = yield PlayAPI.openPlayerManagement();
                    // if result contains no players, populate the players array with dummy players
                    if (result && result.players && result.players.length > 0) {
                        players = result.players.map((player) => {
                            return Object.assign({}, player, { instructionsViewed: new Set() });
                        });
                    }
                    else {
                        players = [
                            { name: "Player 1", type: "", identifier: "" },
                            { name: "Player 2", type: "", identifier: "" },
                        ];
                    }
                }
                catch (error) {
                    console.log("openPlayerManagement Error " + JSON.stringify(error));
                    // Initialize with fake data just so the app doesn't break
                    players = [
                        { name: "Player 1", type: "", identifier: "" },
                        { name: "Player 2", type: "", identifier: "" },
                    ];
                }
                this._players = players;
                return players;
            });
        }
        get players() {
            return this._players;
        }
        set players(p) {
            this._players = p;
        }
        /**
         * Player grouping
         * Returns a session, which is a list of groups. A group is a list of players that plays in a round.
         * @param GroupSize number = 1 : how many players per group. =1 individual game, =2 pair game, =more for bigger groups
         * If groupSize is bigger than the number of available players this will be set to this.players.length
         * @param rounds number = null : hom many rounds are needed, =null as many as possible without repeating,
         * if there aren't enough players it will start repeating
         * @param fixedPairs boolean = false : If true, for pair games, avoids making the players switch sides
         * after sequential rounds.
         * @param isShuffle boolean = false : If true, the player pool array is shuffled
         * @return list of groups. group=list of players : [ [A, B], [B, C], [C, A]]
         */
        // currently not being used in TSM 
        getPlayerSession(groupSize = 1, rounds = null, fixedPairs = false, isShuffle = false) {
            const result = [];
            if (rounds == null) {
                rounds = this.players.length;
            }
            groupSize = Math.min(groupSize, this.players.length);
            if (isShuffle) {
                CollectionUtil_2.shuffleNonRepeatingArray(this.players);
            }
            // set the pool of players
            const pool = [];
            for (const player of this.players) {
                pool.push(player);
            }
            for (let i = 0; i < rounds; i++) {
                const group = pool.slice(0, groupSize);
                // In case the game is in pairs and we want static positions, revert every other group
                if (fixedPairs && (groupSize === 2) && (i % 2 === 0)) {
                    group.reverse();
                }
                pool.push(pool.shift());
                result.push(group);
            }
            return result;
        }
        set prevPlayerNames(prevPlayers) {
            // stores the previous player names for comparison
            if (prevPlayers) {
                this.previousSessionPlayerNames = prevPlayers.slice();
            }
        }
        get prevPlayerNames() {
            return this.previousSessionPlayerNames;
        }
    }
    exports.PlayerManager = PlayerManager;
});
/**
 * Wrapper for Play Disney Parks AR Kit
 * @author joel@cerebralfix.com
 */
define("switchback-common/play/augmented-reality/ThreeARScene", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThreeARScene {
        constructor(canvas) {
            this._sessionStarted = false;
            this.markerDetectedSignal = new Phaser.Signal();
            console.log("[ThreeARScene] new()");
            this._canvasTarget = canvas;
        }
        get renderer() {
            return this._renderer;
        }
        get scene() {
            return this._scene;
        }
        get canvas() {
            return this._canvasTarget;
        }
        get camera() {
            return this._camera;
        }
        registerImage(name, pathInBundle, realWorldWidth) {
            if (!this._sessionStarted) {
                console.error("[ThreeARScene] registerImage() cannot register images before AR session is started");
                return;
            }
            PlayAPI.AugmentedReality.registerImage(name, pathInBundle, realWorldWidth).then(function () {
                console.log(`[ThreeARScene] registerImage(): successfully registered ${name}`);
            }).catch((e) => {
                console.warn(`[ThreeARScene] registerImage(): failed to register ${name}`);
                console.error(JSON.stringify(e));
            });
        }
        sessionStart() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ThreeARScene] startSession()");
                PlayAPI.AugmentedReality.sessionStart(this._canvasTarget).then(() => {
                    console.log(`[ThreeARScene] startSession(): success`);
                    this._sessionStarted = true;
                }).catch((e) => {
                    console.warn(`[ThreeARScene] startSession(): error`);
                    console.error(JSON.stringify(e));
                    this._sessionStarted = false;
                });
                // TODO: Figure out why sessionStart isn't returning, then remove this hack
                this._sessionStarted = true;
                if (!this._sessionStarted) {
                    return false;
                }
                else {
                    document.body.classList.add("transparent");
                }
                // Subscribe to PlayAPI image recognize events
                PlayAPI.subscribeToEvent(PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE, (event) => {
                    console.log("[ThreeARScene] PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE");
                    // Let any listeners know the details of the marker we found.
                    this.markerDetectedSignal.dispatch(event);
                });
                this._renderer = PlayAPI.AugmentedReality.renderer;
                this._scene = PlayAPI.AugmentedReality.scene;
                this._camera = PlayAPI.AugmentedReality.camera;
                return this._sessionStarted;
            });
        }
        endSession() {
            console.log("[ThreeARScene] endSession()");
            let response;
            PlayAPI.AugmentedReality.sessionEnd().then(() => {
                console.log(`[ThreeARScene] endSession(): success`);
                response = true;
            }).catch((e) => {
                console.warn(`[ThreeARScene] endSession(): error`);
                console.error(JSON.stringify(e));
                response = false;
            });
            this._sessionStarted = this._sessionStarted && response;
            document.body.classList.remove("transparent");
            return response;
        }
        destroy() {
            this._renderer.clear();
            this.markerDetectedSignal.dispose();
        }
    }
    exports.ThreeARScene = ThreeARScene;
});
define("switchback-common/play/debug/PlayDebugPanel", ["require", "exports", "switchback-common/play/api/MockPlayAPI"], function (require, exports, MockPlayAPI_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebugPanel can trigger events that the PlayAPI will respond to.
     * @author joel@cerebralfix.com
     */
    class PlayDebugPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            const BUTTON_PADDING = 5;
            const BUTTON_WIDTH = 200;
            const BUTTON_HEIGHT = 100;
            // EJECT beacon button
            const beaconEject = new Phaser.Button(this.game, BUTTON_PADDING, BUTTON_PADDING, "button-basic", this.onBeaconEjectButtonClicked, this, "over", "out", "down", "out");
            beaconEject.width = BUTTON_WIDTH;
            beaconEject.height = BUTTON_HEIGHT;
            beaconEject.alpha = 0.5;
            // EJECT beacon button text
            const beaconEjectText = new Phaser.Text(game, beaconEject.centerX, beaconEject.centerY, "EJECT\nBEACON");
            beaconEjectText.anchor.set(0.5);
            // START beacon button
            const beaconStart = new Phaser.Button(this.game, beaconEject.x + beaconEject.width + BUTTON_PADDING, beaconEject.y, "button-basic", this.onBeaconStartButtonClicked, this, "over", "out", "down", "out");
            beaconStart.width = BUTTON_WIDTH;
            beaconStart.height = BUTTON_HEIGHT;
            beaconStart.alpha = 0.5;
            // START beacon button text
            const beaconStartText = new Phaser.Text(game, beaconStart.centerX, beaconStart.centerY, "START\nBEACON");
            beaconStartText.anchor.set(0.5);
            // LOCATION change button
            const locationChange = new Phaser.Button(this.game, beaconStart.x + beaconStart.width + BUTTON_PADDING, beaconStart.y, "button-basic", this.onLocationChangeButtonClicked, this, "over", "out", "down", "out");
            locationChange.width = BUTTON_WIDTH;
            locationChange.height = BUTTON_HEIGHT;
            locationChange.alpha = 0.5;
            // LOCATION change button text
            const locationChangeText = new Phaser.Text(game, locationChange.centerX, locationChange.centerY, "RANDOM\nLOCATION");
            locationChangeText.anchor.set(0.5);
            // Background panel graphic
            const background = new Phaser.Image(game, 0, 0, "button-basic");
            background.alpha = 0.2;
            this.add(background);
            this.add(beaconEject);
            this.add(beaconStart);
            this.add(locationChange);
            this.add(beaconEjectText);
            this.add(beaconStartText);
            this.add(locationChangeText);
            // Size the background to sit behind everything that has been added
            background.width = this.width + BUTTON_PADDING;
            background.height = this.height + BUTTON_PADDING;
        }
        onLocationChangeButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: MockPlayAPI_2.MockPlayAPI.getFakeLocation(),
            }));
        }
        onBeaconEjectButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        onBeaconStartButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
    }
    exports.PlayDebugPanel = PlayDebugPanel;
});
define("switchback-common/play/debug/PlayDebug", ["require", "exports", "switchback-common/play/debug/PlayDebugPanel"], function (require, exports, PlayDebugPanel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebug creates a PlayDebugPanel that can be used to trigger events via the PlayAPI.
     * @author joel@cerebralfix.com
     */
    class PlayDebug {
        constructor(game) {
            // Toggle button
            const toggleButton = new Phaser.Button(game, 0, 0, "button-basic", this.onDebugButtonClicked, this, "over", "out", "down", "out");
            toggleButton.bottom = game.height;
            toggleButton.alpha = 0.3;
            // Toggle button text
            const toggleButtonText = new Phaser.Text(game, toggleButton.centerX, toggleButton.centerY, "DEBUG");
            toggleButtonText.anchor.set(0.5);
            // Debug panel
            this.debugPanel = new PlayDebugPanel_1.PlayDebugPanel(game);
            this.debugPanel.centerX = game.world.centerX;
            this.debugPanel.bottom = game.height;
            this.debugPanel.visible = false;
            game.add.existing(toggleButton);
            game.add.existing(toggleButtonText);
        }
        onDebugButtonClicked() {
            this.debugPanel.visible = !this.debugPanel.visible;
        }
    }
    exports.PlayDebug = PlayDebug;
});
define("switchback-common/test/keyboardController.spec", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/KeyboardOptions", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, KeyboardController_2, KeyboardOptions_1, SpecialChars_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("keyboardController", () => {
        let keyboardController;
        beforeEach(() => {
            keyboardController = new KeyboardController_2.KeyboardController(new KeyboardOptions_1.KeyboardOptions());
        });
        it("can add letters to a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            expect(keyboardController.currentString).toBe("TEST");
        });
        it("can submit a string", (done) => {
            keyboardController.submitSignal.addOnce((response) => {
                expect(response).toBe("TEST");
                done();
            });
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.SUBMIT);
        });
        it("can delete letters from a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.DELETE);
            expect(keyboardController.currentString).toBe("TES");
        });
    });
});
define("switchback-common/test/playerManager.spec", ["require", "exports", "switchback-common/play/PlayerManager"], function (require, exports, PlayerManager_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("PlayerManager", () => {
        let playerManager;
        beforeEach(() => {
            playerManager = new PlayerManager_1.PlayerManager();
        });
        describe("Getters and Setters", () => {
            describe("handles null parameters", () => {
                [null, undefined].forEach((value) => {
                    it(`prevPlayerNames handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
                [null, undefined].forEach((value) => {
                    it(`players handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
            });
            it("overwrites existing previous player names", () => {
                playerManager.prevPlayerNames = ["n1", "n2"];
                playerManager.prevPlayerNames = ["n3", "n4"];
                expect(playerManager.prevPlayerNames).toEqual(["n3", "n4"]);
            });
        });
        describe("Player Management", () => {
            it("has players", () => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
                expect(playerManager.players).toBeTruthy();
            });
        });
        describe("Player Sessions", () => {
            beforeEach(() => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
            });
            [2, 3, 4, 5].forEach((numberOfRounds) => {
                it("generates the correct number of rounds", () => {
                    const session = playerManager.getPlayerSession(2, numberOfRounds);
                    expect(session.length).toBe(numberOfRounds);
                });
            });
            [1, 2].forEach((numberOfRounds) => {
                it("generates rounds of the correct size", () => {
                    const session = playerManager.getPlayerSession(numberOfRounds);
                    session.forEach((round) => {
                        expect(round.length).toBe(numberOfRounds);
                    });
                });
            });
            it("has different players in each round", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                session.forEach((round) => {
                    expect(round[0]).not.toEqual(round[1]);
                });
            });
            it("prevents players from swapping sides", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                expect(session[0][1]).toEqual(session[1][1]);
            });
        });
    });
});
define("switchback-common/test/signal.spec", ["require", "exports", "switchback-common/component/Signal"], function (require, exports, Signal_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("Signal", () => {
        it("a listener is called each time a signal is dispatched", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("addOnce causes a listener to be called only once", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("dispatch calls every attached listener", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("disposing a signal stops a listener being called", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispose();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("listeners are called in the order they are added", () => {
            const signal = new Signal_3.Signal();
            const results = [];
            signal.addOnce(() => {
                results.push(1);
            });
            signal.addOnce(() => {
                results.push(2);
            });
            signal.dispatch();
            expect(results[0]).toBe(1);
            expect(results[1]).toBe(2);
        });
    });
});
define("switchback-common/test/unit/CollectionUtil.spec", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`CollectionUtil`, () => {
        describe(`Exclusive Left Join`, () => {
            it(`Excluded array contains 1 item found in the array`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual(["a", "b", "c"]);
            });
            it(`Excluded array contains 2 items found in the array`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["a"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["b"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Excluded array is empty`, () => {
                const result = CollectionUtil_3.exclusiveLeftJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual(["a", "b", "c"]);
            });
        });
        describe(`Inner Join`, () => {
            it(`Second array contains 1 item found in the array`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual([]);
            });
            it(`Second array contains 2 items found in the array`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["a"]);
            });
            it(`Second array is empty`, () => {
                const result = CollectionUtil_3.innerJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual([]);
            });
        });
    });
});
define("switchback-common/test/unit/MiniGamePlaySession.spec", ["require", "exports", "switchback-common/play/MiniGamePlaySession"], function (require, exports, MiniGamePlaySession_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`MiniGamePlaySession`, () => {
        const player0 = { identifier: "0", name: "player 0", type: "" };
        const player1 = { identifier: "1", name: "player 1", type: "" };
        const player2 = { identifier: "2", name: "player 2", type: "" };
        const player3 = { identifier: "3", name: "player 3", type: "" };
        const player4 = { identifier: "4", name: "player 4", type: "" };
        describe(`Player group generation`, () => {
            describe(`Single Player Mini Games`, () => {
                it(`The correct number of play session for a single player are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`First player is not one of the last known players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const lastKnownPlayers = [player0, player1, player2, player3];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, lastKnownPlayers);
                    expect(result[0].players).toEqual([player4]);
                });
            });
            describe(`Two Player Mini Games`, () => {
                it(`The correct number of play sessions for two people are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`Player sequence is correct for 5 players. One player plays twice in a row and in the same position`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result[0].players[1]).toEqual(result[1].players[1]);
                    expect(result[1].players[0]).toEqual(result[2].players[0]);
                    expect(result[2].players[1]).toEqual(result[3].players[1]);
                    expect(result[3].players[0]).toEqual(result[4].players[0]);
                    expect(result[0].players[0]).toEqual(result[4].players[1]);
                });
                it(`Unable to generate player groups for two player game with only one player`, () => {
                    const players = [player0];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(0);
                });
                it(`Only generates one play group when there is only two players`, () => {
                    const players = [player0, player1];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(1);
                });
            });
        });
        describe(`Player Group Cycling`, () => {
            let playSession;
            beforeEach(() => {
                playSession = new MiniGamePlaySession_1.MiniGamePlaySession([player0, player1, player2, player3]);
            });
            it(`Starting a single player mini game returns a single player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players.length).toBe(1);
            });
            it(`Starting a two player mini game returns a two player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players.length).toBe(2);
            });
            it(`The last group to finish should trigger the mini game session to be finished`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                playSession.TEST_setCurrentPlayGroupIndex(3);
                playSession.playGroupFinished();
                expect(playSession.isMiniGameSessionFinished).toBe(true);
            });
            it(`Starting a single player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players).toEqual([player1]);
            });
            it(`Starting a two player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players[1]).toEqual(player1);
            });
        });
    });
});
define("switchback-common/util/Animation", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function popSpin(x, y, width, imageKey, trailKey, game, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = parameters || {};
            const parent = params.parent || game.add.group();
            const tweenMS = params.tweenMS || 600;
            const spinMS = params.spinMS || 750;
            const bezierCurve = [
                [[0, 0], [-0.287, -0.413], [-0.705, -0.508], [-1, -0.539]],
                [[0, 0], [-0.609, 0.025], [-0.882, 0.280], [-1, 0.593]],
                [[0, 0], [0.287, -0.413], [0.705, -0.508], [1, -0.539]],
            ];
            const startPositions = [
                [x - width * 0.2, y - width * 0.1],
                [x - width * 0.25, y + width * 0.05],
                [x + width * 0.2, y],
            ];
            for (let i = 0; i < bezierCurve.length; i++) {
                const pos = startPositions[i];
                const bezier = bezierCurve[i];
                const star = new Phaser.Image(game, pos[0], pos[1], imageKey);
                const curveScale = width * 0.4;
                star.anchor.set(0.5);
                star.pivot.set(0.5, 0.5);
                star.position.set(pos[0], pos[1]);
                const starRatio = star.width / star.height;
                star.width = params.iconWidth || star.width;
                star.height = star.width * starRatio;
                star.visible = true;
                game.add.tween(star).to({ angle: 360 }, spinMS, Phaser.Easing.Linear.None, true, 0 - 1).start();
                const tween = game.add.tween(star).to({
                    x: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    y: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                const emitter = game.add.emitter(star.x, star.y, 50);
                emitter.maxParticleScale = 0.5;
                parent.addChild(emitter);
                parent.addChild(star);
                emitter.makeParticles(trailKey);
                emitter.setRotation(0, 0);
                emitter.setXSpeed(0, 0);
                emitter.setYSpeed(0, 0);
                emitter.alpha = 0.8;
                emitter.start(false, 5000, 10);
                emitter.emitX = star.position.x;
                emitter.emitY = star.position.y;
                emitter.gravity = 0;
                const emitTween = game.add.tween(emitter).to({
                    emitX: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    emitY: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                tween.interpolation(Phaser.Math.bezierInterpolation);
                emitTween.interpolation(Phaser.Math.bezierInterpolation);
                tween.onComplete.add(() => {
                    const fade = game.add.tween(star).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        star.destroy();
                    });
                    fade.start();
                }, this);
                emitTween.onComplete.addOnce(() => {
                    const fade = game.add.tween(emitter).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        // if (emitter) {
                        // 	emitter.destroy();
                        // }
                    });
                    emitter.start();
                }, this);
                tween.start();
                emitTween.start();
            }
        });
    }
    exports.popSpin = popSpin;
    function wiggle(game, tweenObj, angle = 0, offsetAngle = 15, duration = 500, autoStart = true) {
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: angle + offsetAngle }, duration, Phaser.Easing.Cubic.apply, autoStart, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: angle - offsetAngle }, duration, Phaser.Easing.Cubic.apply, false, 0, 0, true);
        tween.chain(tween2);
        return tween2;
    }
    exports.wiggle = wiggle;
    function attentionWiggle(game, tweenObj, angle = 0, duration = 500) {
        const startAngle = tweenObj.angle;
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: startAngle - angle }, duration * 0.25, Phaser.Easing.Cubic.apply, true, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: startAngle + angle }, duration * 0.5, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        const tween3 = game.add.tween(tweenObj);
        tween3.to({ angle: startAngle }, duration * 0.25, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        tween.chain(tween2);
        tween2.chain(tween3);
        return tween;
    }
    exports.attentionWiggle = attentionWiggle;
    function tintTween(game, tweenObj, tint, duration = 500, ease = Phaser.Easing.Linear.None, autoStart = false, delay = 0, repeat = 0, yoyo = false) {
        const colorBlend = { step: 0 };
        const startColor = tweenObj.tint;
        const colorTween = game.add.tween(colorBlend).to({ step: 100 }, duration, ease, autoStart, delay, repeat, yoyo);
        colorTween.onUpdateCallback(function () {
            tweenObj.tint = Phaser.Color.interpolateColor(startColor, tint, 100, colorBlend.step);
        });
        return colorTween.start();
    }
    exports.tintTween = tintTween;
    function fadeIn(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 0;
        object.visible = true;
        return game.add.tween(object).to({
            alpha: 1,
        }, duration, easing, false);
    }
    exports.fadeIn = fadeIn;
    function fadeOut(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 1;
        return game.add.tween(object).to({
            alpha: 0,
        }, duration, easing, false);
    }
    exports.fadeOut = fadeOut;
    function bounce(game, object, repeat = 0, bounceHeight) {
        bounceHeight = bounceHeight || this.game.height * 0.1;
        const destination = object.position.y - bounceHeight;
        const origin = object.position.y;
        const tween = game.add.tween(object).to({
            y: destination,
        }, 200, Phaser.Easing.Linear.None, false, 0).to({
            y: origin,
        }, 200, Phaser.Easing.Linear.None, false, 0);
        tween.repeatAll(repeat);
        return tween;
    }
    exports.bounce = bounce;
    function slideTo(game, object, targetPosition, duration = 200, easing = Phaser.Easing.Linear.None) {
        const tween = game.add.tween(object).to({
            x: targetPosition.x,
            y: targetPosition.y,
        }, duration, easing, false, 0, 0);
        return tween;
    }
    exports.slideTo = slideTo;
    function pulse(game, object, targetScale, duration = 200, easing = Phaser.Easing.Linear.None) {
        const originalWidth = object.width;
        const originalHeight = object.height;
        const tweenIn = game.add.tween(object).to({
            width: originalWidth * targetScale,
            height: originalHeight * targetScale,
        }, duration, easing, false, 0, 0);
        const tweenOut = game.add.tween(object).to({
            width: originalWidth,
            height: originalHeight,
        }, duration, easing, false, 0, 0);
        tweenOut.onComplete.add(() => {
            tweenIn.start();
        });
        tweenIn.chain(tweenOut);
        return tweenIn;
    }
    exports.pulse = pulse;
    function shake(game, object, amount, severity) {
        const times = amount || game.rnd.integerInRange(5, 7);
        const maxOffset = game.width * (severity || 0.02);
        const orgPos = object.position.clone();
        let firstTween;
        let prevTween;
        for (let i = 0; i < times; ++i) {
            const tween = game.add.tween(object).to({
                x: orgPos.x + game.rnd.integerInRange(-maxOffset, maxOffset),
                y: orgPos.y + game.rnd.integerInRange(-maxOffset, maxOffset),
            }, 20, Phaser.Easing.Linear.None, false);
            if (prevTween) {
                prevTween.chain(tween);
            }
            else {
                firstTween = tween;
            }
            prevTween = tween;
        }
        const tween = game.add.tween(object).to({
            x: orgPos.x,
            y: orgPos.y,
        }, 20, Phaser.Easing.Linear.None, false);
        prevTween.chain(tween);
        return firstTween;
    }
    exports.shake = shake;
    function blink(game, object, duration, times, autostart = true) {
        const tween = game.add.tween(object);
        tween.to({}, duration, Phaser.Easing.Linear.None, autostart, 0, times);
        tween.onRepeat.add(() => {
            object.visible = !object.visible;
        });
        return tween;
    }
    exports.blink = blink;
    function pulseFade(game, object, finalScale, duration, times = 0, autoStart = true) {
        const tweens = [
            // scale
            game.add.tween(object.scale).to({ x: finalScale, y: finalScale }, duration, Phaser.Easing.Linear.None, autoStart, 0, times),
            // tween alphas
            game.add.tween(object).to({ alpha: 0 }, duration, Phaser.Easing.Quartic.In, autoStart, 0, times)
        ];
        return tweens;
    }
    exports.pulseFade = pulseFade;
});
define("switchback-common/util/CameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_27) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function cameraShake(game, intensity = 0.005, durationMS = 200) {
        return __awaiter(this, void 0, void 0, function* () {
            game.camera.shake(intensity, durationMS);
            yield PromiseUtil_27.PromiseUtil.time(game, durationMS);
        });
    }
    exports.cameraShake = cameraShake;
    /**
     * Scales up/down the given group so that it appears as though the camera is zooming in/out respectively.
     * Set duration > 0 to make the zoom a tween.
     *
     * @param camGroup: Phaser.Group - Should contain everything in the scene you wish to zoom
     * @param zoomLevel: number - 1.0 is standard, higher (eg. 1.5 or 2.0) will zoom in, use < 1.0 with caution.
     * @param focusPnt: Phaser.Point (Scale 0-1) - Centre of the screen once the zoom is complete, (0.5, 0.5) would be game world centre
     * @param duration: number - Default is instant, higher numbers will tween the zoom for the given time.
     */
    function cameraZoom(game, camGroup, zoomLevel, focusPnt, duration = -1, easing = Phaser.Easing.Linear.None) {
        return __awaiter(this, void 0, void 0, function* () {
            const movementProps = {};
            const camW = game.width * zoomLevel;
            const camH = game.height * zoomLevel;
            movementProps.x = game.width * 0.5 - camW * focusPnt.x;
            movementProps.y = game.height * 0.5 - camH * focusPnt.y;
            const scaleProps = {
                x: zoomLevel,
                y: zoomLevel,
            };
            if (duration > 0) {
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween2(game.add.tween(camGroup).to(movementProps, duration, easing, true)),
                    PromiseUtil_27.PromiseUtil.tween2(game.add.tween(camGroup.scale).to(scaleProps, duration, easing, true)),
                ]);
            }
            else {
                camGroup.scale.set(scaleProps.x, scaleProps.y);
                camGroup.position.set(movementProps.x, movementProps.y);
            }
        });
    }
    exports.cameraZoom = cameraZoom;
});
define("switchback-common/util/LiquidScaler", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class LiquidScaler {
        static init(game, minWidth, minHeight, maxScale) {
            if (this.instance) {
                console.error("LiquidScaler already initialized.");
                return;
            }
            this.instance = new LiquidScaler(game, minWidth, minHeight, maxScale);
        }
        static getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            return this.Instance.getScaledSize(widthPercent, heightPercent, maintainAspectRatio, useMaxScale);
        }
        static get Instance() {
            if (!this.instance) {
                console.error("LiquidScaler not initialized.");
            }
            return this.instance;
        }
        constructor(game, minWidth, minHeight, maxScale) {
            this.game = game;
            this.minWidth = minWidth;
            this.minHeight = minHeight;
            this.maxScale = maxScale;
        }
        getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            let width = widthPercent * this.minWidth;
            let height = heightPercent * this.minHeight;
            let verticalScale = this.game.height / this.minHeight;
            let horizontalScale = this.game.width / this.minWidth;
            if (maintainAspectRatio) {
                const isLong = horizontalScale > verticalScale;
                if (isLong) {
                    if (useMaxScale) {
                        verticalScale = Math.min(verticalScale, this.maxScale);
                    }
                    width *= verticalScale;
                    height *= verticalScale;
                }
                else {
                    if (useMaxScale) {
                        horizontalScale = Math.min(horizontalScale, this.maxScale);
                    }
                    width *= horizontalScale;
                    height *= horizontalScale;
                }
            }
            else {
                width = widthPercent * this.game.width;
                height = widthPercent * this.game.height;
            }
            return new Phaser.Rectangle(0, 0, width, height);
        }
    }
    exports.LiquidScaler = LiquidScaler;
});
define("switchback-common/util/MathUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Returns a random number between min (inclusive) and max (exclusive)
     */
    function getRandomArbitrary(min, max) {
        return Math.random() * (max - min) + min;
    }
    exports.getRandomArbitrary = getRandomArbitrary;
    /**
     * Returns a random integer between min (inclusive) and max (inclusive)
     * Using Math.round() will give you a non-uniform distribution!
     */
    function getRandomInt(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
    exports.getRandomInt = getRandomInt;
    /**
     * Generate a nomally distibuted number based on the central limit theorem between 0 and 1
     * https://stackoverflow.com/questions/25582882/javascript-math-random-normal-distribution-gaussian-bell-curve/39187274#39187274
     */
    function normalRandom(numSamples = 7) {
        let rand = 0;
        for (let i = 0; i < numSamples; i++) {
            rand += Math.random();
        }
        return rand / numSamples;
    }
    exports.normalRandom = normalRandom;
    /*
        Utility function to get the shortest difference between two angles
        Used for tweens, it works, given the fact that phaser wraps the angle to the
        correct space whenever we assign it (which allows us to overflow the range by doing current+dif)
        And also because the tweens don't seem to rely on re-reading the value of the variable (which is awesome)
    */
    function getShorterAngleDif(currentAngle, newAngle) {
        if (currentAngle < 0) {
            currentAngle += 360;
        }
        if (newAngle < 0) {
            newAngle += 360;
        }
        let dif = newAngle - currentAngle;
        if (dif < -180) {
            dif += 360;
        }
        else if (dif > 180) {
            dif -= 360;
        }
        return dif;
    }
    exports.getShorterAngleDif = getShorterAngleDif;
    function base64ToByteArray(base64) {
        const binary_string = window.atob(base64);
        const len = binary_string.length;
        const bytes = new Uint8Array(len);
        for (var i = 0; i < len; i++) {
            bytes[i] = binary_string.charCodeAt(i);
        }
        return bytes;
    }
    exports.base64ToByteArray = base64ToByteArray;
    function lerp2D(pointA, pointB, amount) {
        const dX = pointA.x + (pointB.x - pointA.x) * amount;
        const dY = pointA.y + (pointB.y - pointA.y) * amount;
        return { x: dX, y: dY };
    }
    exports.lerp2D = lerp2D;
    class MathUtil {
        /**
         * Tests to see if @param n is between angles @param a and @param b.
         * Code as per https://www.xarg.org/2010/06/is-an-angle-between-two-other-angles/
         * TODO: Write tests
         */
        static isAngleBetween(n, a, b) {
            n = (360 + (n % 360)) % 360;
            a = (3600000 + a) % 360;
            b = (3600000 + b) % 360;
            if (a < b) {
                return a <= n && n <= b;
            }
            return a <= n || n <= b;
        }
        /**
         * Returns a point located at @param distance along a @param line.
         * Formula as per https://math.stackexchange.com/a/409721
         */
        static getPointAlongLine(line, distance) {
            const x1 = line.start.x;
            const y1 = line.start.y;
            const x2 = line.end.x;
            const y2 = line.end.x;
            const d = distance;
            const d1 = line.length;
            return new Phaser.Point((x1 + (d / d1) * (x2 - x1)), y1 + (d / d1) * (y2 - y1));
        }
    }
    exports.MathUtil = MathUtil;
});
define("switchback-common/util/MemoryUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MemoryUtil {
        /*
            Checks that all the child objects contained as attributes have been destroyed properly.
            Does not descend into the tree (childs of childs).
            Works for Phaser objects and Signals.
            To be used for debugging, and profiling only.
            How to use:
                When trying to profile certain class, call this function between destroy and nulling. ie:
    
                ```
                super.destroy();
                MemoryUtil.checkAllDestroyed(this);
    
                this.prop1 = null;
                this.prop2 = null;
                ```
    
                If certain attribute has not been destroyed properly it will be reported on the console.
        */
        static checkAllDestroyed(obj) {
            if (obj.stateName) {
                console.warn("Checking state " + obj.stateName);
            }
            const toIgnore = ["game", "make", "add", "camera", "cache", "input", "sound",
                "scale", "stage", "state", "time", "tweens", "world", "particles", "physics", "load"];
            let ret = true;
            for (const name in obj) {
                if (!obj.hasOwnProperty(name)) {
                    continue;
                }
                if (toIgnore.indexOf(name) > -1) {
                    continue;
                }
                const child = obj[name];
                if (!child) {
                    continue;
                }
                // if the object contains one of this properties and is not null, then it hasn't been destroyed properly
                if (child.parent || child.game || child.exists) {
                    ret = false;
                    console.warn("Object not properly destroyed. '" + name + "'", child);
                }
                // signals
                if (child._bindings || child._prevParams) {
                    ret = false;
                    console.warn("Signal not properly disposed. '" + name + "'", child);
                }
                // sounds
                if (child._buffer || child.externalNode) {
                    ret = false;
                    console.warn("Sound not properly disposed. '" + name + "'", child);
                }
            }
            return ret;
        }
    }
    exports.MemoryUtil = MemoryUtil;
});
define("switchback-common/util/MotionUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_28) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MotionUtil {
        static checkGyroAvailabililty() {
            return __awaiter(this, void 0, void 0, function* () {
                MotionUtil.isAccelerometerAvailable = false;
                MotionUtil.isGyroAvailable = false;
                const motionCallback = (result) => {
                    const acceleration = result.userAcceleration;
                    if (acceleration.x !== 0 || acceleration.y !== 0 || acceleration.z !== 0) {
                        MotionUtil.isAccelerometerAvailable = true;
                    }
                    const roll = result.attitude.roll;
                    if (roll !== 0) {
                        MotionUtil.isGyroAvailable = true;
                    }
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
                yield PromiseUtil_28.PromiseUtil.realTime(1000);
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
            });
        }
    }
    MotionUtil.isAccelerometerAvailable = false;
    MotionUtil.isGyroAvailable = false;
    exports.MotionUtil = MotionUtil;
});
define("switchback-common/util/PhysicsUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PhysicsUtil {
        /**
         * Custom physics data doesn't cope with being scaled.
         *
         * This function found at https://stackoverflow.com/a/43302120/1343176 fixes this
         * by scaling all vertices on the shape and rewriting them back to the cache at runtime.
         * @param originalPhysicsKey the key in cache of your physics data
         * @param newPhysicsKey the key you want the new physics data to have
         * @param shapeKey the specific shape in the original data to scale
         * @param scale the scale value
         */
        static resizePolygonData(game, originalPhysicsKey, newPhysicsKey, shapeKey, scale) {
            const newData = [];
            const data = game.cache.getPhysicsData(originalPhysicsKey, shapeKey);
            for (let i = 0; i < data.length; i++) {
                const vertices = [];
                for (let j = 0; j < data[i].shape.length; j += 2) {
                    vertices[j] = data[i].shape[j] * scale;
                    vertices[j + 1] = data[i].shape[j + 1] * scale;
                }
                newData.push({ shape: vertices });
            }
            const item = {};
            item[shapeKey] = newData;
            game.load.physics(newPhysicsKey, "", item);
        }
    }
    exports.PhysicsUtil = PhysicsUtil;
});
/**
 * A simple timer that avoids many of the problems with the timer from Phaser
 * Even though it looks unnervingly simple, that's actually the design of it.
 * The code is solid, and works logically as it would be expected to,
 * and it has as minimal overhead as possible. So it's efficient, lightweight
 * and easy to understand, debug and maintain.
 * It doesn't depend on Phaser nor es6 or anything fancy, that'll make it reusable,
 * solid and predictable as it possible.
 *
 * @author Joel@CerebralFix.com
 * @author Jeronimo@CerebralFix.com
 */
define("switchback-common/util/SimpleTimer", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SimpleTimer {
        constructor(delay = 0, callback = null) {
            this.onTimer = null;
            this.delay = 0;
            this.elapsed = 0;
            this.stopped = true;
            this.delay = delay;
            this.onTimer = callback;
        }
        stop() {
            this.stopped = true;
        }
        start() {
            this.elapsed = 0;
            this.stopped = false;
        }
        update(dt) {
            if (this.stopped) {
                return;
            }
            this.elapsed += dt;
            // Doing it this way ensures that it will be triggered only once per frame,
            // but overall the accuracy of the timer is maximized even during frame drops
            // which is the whole reason of using dt
            if (this.elapsed > this.delay) {
                this.elapsed -= this.delay;
                if (this.onTimer) {
                    this.onTimer();
                }
            }
        }
    }
    exports.SimpleTimer = SimpleTimer;
});
define("switchback-common/util/SpriteUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SpriteUtil {
        static greyscale(game, frame) {
            const bmd = game.make.bitmapData();
            bmd.load(frame);
            bmd.processPixelRGB(SpriteUtil.forEachPixel, this);
            return bmd;
        }
        static forEachPixel(pixel) {
            const gray = (pixel.r + pixel.g + pixel.b) / 3;
            pixel.r = gray;
            pixel.g = gray;
            pixel.b = gray;
            return pixel;
        }
    }
    exports.SpriteUtil = SpriteUtil;
});
define("switchback-common/util/TimeUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class TimeUtil {
        /**
         * Outputs time as a string in minutes:seconds format.
         * Seconds are padded, minutes are not
         *
         * @param timeMS In milliseconds
         */
        static toDigitalTime(timeMS) {
            timeMS = Math.round(timeMS * 0.001);
            const minutes = Math.floor(timeMS / 60);
            const seconds = timeMS % 60;
            return minutes + ":" + (seconds < 10 ? "0" + seconds : seconds).toString();
        }
    }
    exports.TimeUtil = TimeUtil;
});
define("switchback-common/util/WebUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Helper functions for web-specific tasks
     * @author joel@cerebralfix.com
     * @author nick@cerebralfix.com
     */
    class WebUtil {
        /**
         * Grabs the URL parameters and returns them as a dictionary.
         */
        static getQueryParams() {
            if (WebUtil.queryDict) {
                return WebUtil.queryDict;
            }
            const params = {};
            location.search.substr(1).split("&").forEach((item) => {
                const s = item.split("=");
                const key = s[0];
                const value = s[1] && decodeURIComponent(s[1]);
                if (!(key in params)) {
                    params[key] = new Array();
                }
                params[key].push(value);
            });
            WebUtil.queryDict = params;
            return WebUtil.queryDict;
        }
        // Fast syntax lookup for a specific query parametre
        static getQueryFor(key, index = 0) {
            const dict = WebUtil.getQueryParams();
            if (dict[key]) {
                return dict[key][index];
            }
            return null;
        }
    }
    exports.WebUtil = WebUtil;
});
