var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * This class exists to support on-desktop testing of data saving and loading.
 * It should only be used by the MockPlayAPI.
 * @author joel@cerebralfix.com
 * @author nick@cerebralfix.com
 */
define("switchback-common/util/CookieUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CookieUtil {
        static get(name) {
            const nameEQ = name + "=";
            const ca = document.cookie.split(";");
            for (let i = 0; i < ca.length; i++) {
                let c = ca[i];
                while (c.charAt(0) === " ") {
                    c = c.substring(1, c.length);
                }
                if (c.indexOf(nameEQ) === 0) {
                    return decodeURIComponent(c.substring(nameEQ.length, c.length));
                }
            }
            return null;
        }
        static set(name, value, daysTilExpiry = 1) {
            const expiry = new Date();
            expiry.setTime(expiry.getTime() + daysTilExpiry * 24 * 60 * 60 * 1000);
            document.cookie = name + "=" + encodeURIComponent(value) + "; expires=" + expiry.toUTCString() + "; path = /";
        }
        static erase(name) {
            CookieUtil.set(name, "", -1);
        }
    }
    exports.CookieUtil = CookieUtil;
});
/**
 * @author nick@cerebralfix.com
 */
define("switchback-common/component/Signal", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Signal {
        constructor() {
            this.active = true;
            this.bindings = [];
        }
        add(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        addOnce(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this, true);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        dispatch(...params) {
            if (this.active) {
                for (let i = 0; this.active && i < this.bindings.length; i++) {
                    const binding = this.bindings[i];
                    if (binding.addOnce) {
                        this.bindings.splice(i, 1);
                        i--;
                    }
                    if (binding.context) {
                        binding.listener.call(binding.context, ...params);
                    }
                    else {
                        binding.listener(...params);
                    }
                }
            }
        }
        removeBinding(binding) {
            if (this.active) {
                const index = this.bindings.indexOf(binding);
                if (index >= 0) {
                    this.bindings.splice(index, 1);
                }
            }
        }
        dispose() {
            this.active = false;
            delete this.bindings;
        }
    }
    exports.Signal = Signal;
    // tslint:disable-next-line:max-classes-per-file
    class SignalBinding {
        constructor(listener, context, signal, addOnce = false) {
            this.listener = listener;
            this.context = context;
            this.signal = signal;
            this.addOnce = addOnce;
        }
        remove() {
            this.signal.removeBinding(this);
        }
    }
    exports.SignalBinding = SignalBinding;
});
define("switchback-common/util/PromiseUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to provide helpful Promise wrapper functions
     * @author joel@cerebralfix.com
     * @author steven@cerebralfix.com
     */
    class PromiseUtil {
        /**
         * Phaser time
         * Smallest time you can wait is the length of one frame (16ms @ 60fps)
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static time(pGame, pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timer;
                    const cancelCallback = () => {
                        if (timer) {
                            // stop timer as we've cancelled it
                            pGame.time.events.remove(timer);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timer = pGame.time.events.add(pTime, resolveCallback);
                });
            });
        }
        /**
         * Real time
         * Can update in smaller increments, but does not pause when the game loses focus
         * Use with caution
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static realTime(pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timeoutId = -1;
                    const cancelCallback = () => {
                        if (timeoutId >= 0) {
                            // stop timeout as we've cancelled it
                            clearTimeout(timeoutId);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timeoutId = setTimeout(resolveCallback, pTime);
                });
            });
        }
        /*
         * Awaits a tween to complete
         *
         * @param pTween The tween to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will cancel the tween and release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static tween2(pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pTween) {
                            // stop listening to the tween and stop it as we've cancelled it
                            pTween.onComplete.remove(resolveCallback);
                            pTween.stop();
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pTween.onComplete.addOnce(resolveCallback);
                });
            });
        }
        static tween(pGame, pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield PromiseUtil.tween2(pTween, cancelSignal);
            });
        }
        static animation(pAnimation, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pAnimation) {
                            // stop listening to callback as we've cancelled it
                            pAnimation.onComplete.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pAnimation.onComplete.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static signal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pSignal.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a common signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static commonSignal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let resolveCallbackBinding;
                    let cancelCallbackBinding;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.removeBinding(resolveCallbackBinding);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.removeBinding(cancelCallbackBinding);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelCallbackBinding = cancelSignal.addOnce(cancelCallback);
                    }
                    resolveCallbackBinding = pSignal.addOnce(resolveCallback);
                });
            });
        }
        static load(pLoader) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    pLoader.onLoadComplete.addOnce(resolve);
                    pLoader.start();
                });
            });
        }
        /**
         * Loops through all markers in @param audioSprite. If any are
         * playing, this function will block until they're finished.
         */
        static audioSpriteMarkers(audioSprite) {
            return __awaiter(this, void 0, void 0, function* () {
                const stopPromises = Object.keys(audioSprite.sounds).map((key) => {
                    const sfx = audioSprite.sounds[key];
                    if (sfx.isPlaying) {
                        return PromiseUtil.signal(sfx.onStop);
                    }
                });
                yield Promise.all(stopPromises);
            });
        }
        /**
         * Wait for a sound that is already playing to finish playing.
         *
         * @param sound The sound that you wish to wait to end
         * @param offsetDuration Handles the situation where you want to wait for a remainder of the time,
         * useful if this sound has already played for a while before being awaited.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static sound(sound, offsetDurationMs = 0, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (sound && sound.game && sound.isPlaying) {
                    // it has been possible for sounds to break and never stop,
                    // hense the additional time safety check
                    yield Promise.race([
                        PromiseUtil.signal(sound.onStop, cancelSignal),
                        PromiseUtil.time(sound.game, sound.durationMS - offsetDurationMs, cancelSignal),
                    ]);
                }
            });
        }
    }
    exports.PromiseUtil = PromiseUtil;
});
define("switchback-common/play/api/MockPlayAPI", ["require", "exports", "switchback-common/util/CookieUtil", "switchback-common/util/PromiseUtil", "switchback-common/component/Signal"], function (require, exports, CookieUtil_1, PromiseUtil_1, Signal_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class exists to support on-desktop testing of the PlayAPI.
     * It returns a bunch of fake data, and can save and load data via browser storage.
     * @author joel@cerebralfix.com
     */
    class MockPlayAPI {
        static init() {
            // Overwrite the PlayAPI functions
            PlayAPI.gameInit = MockPlayAPI.gameInit;
            PlayAPI.locationRead = MockPlayAPI.locationRead;
            PlayAPI.locationWaitTimeRead = MockPlayAPI.locationWaitTimeRead;
            PlayAPI.gameStateRead = MockPlayAPI.gameStateRead;
            PlayAPI.gameStateSave = MockPlayAPI.gameStateSave;
            PlayAPI.gameStatsSave = MockPlayAPI.gameStatsSave;
            PlayAPI.gameStatsRead = MockPlayAPI.gameStatsRead;
            PlayAPI.openPlayerManagement = MockPlayAPI.openPlayerManagement;
            PlayAPI.captureImage = MockPlayAPI.captureImage;
            PlayAPI.saveToPhotos = MockPlayAPI.saveToPhotos;
            PlayAPI.shareImage = MockPlayAPI.shareImage;
            PlayAPI.startCamera = MockPlayAPI.startCamera;
            PlayAPI.cameraAccessStatus = MockPlayAPI.cameraAccessStatus;
            PlayAPI.endCamera = MockPlayAPI.endCamera;
            PlayAPI.closeGame = MockPlayAPI.closeGame;
            PlayAPI.grantAchievementToPlayers = MockPlayAPI.grantAchievementToPlayers;
            PlayAPI.analyticsStateLog = MockPlayAPI.analyticsStateLog;
            PlayAPI.analyticsActionLog = MockPlayAPI.analyticsActionLog;
            PlayAPI.analyticsActionObjectLog = MockPlayAPI.analyticsActionObjectLog;
            PlayAPI.analyticsTimedActionStart = MockPlayAPI.analyticsTimedActionStart;
            PlayAPI.analyticsTimedActionEnd = MockPlayAPI.analyticsTimedActionEnd;
            PlayAPI.vibrateDevice = MockPlayAPI.vibrateDevice;
            PlayAPI.subscribeToEvent = MockPlayAPI.subscribeToEvent;
            PlayAPI.unsubscribeToEvent = MockPlayAPI.unsubscribeToEvent;
            PlayAPI.DBeacon.sessionInit = MockPlayAPI.dbSessionInit;
            PlayAPI.DBeacon.sessionEnd = MockPlayAPI.dbSessionEnd;
            PlayAPI.DBeacon.triggerEffect = MockPlayAPI.triggerEffect;
            PlayAPI.AugmentedReality.sessionStart = MockPlayAPI.arSessionStart;
            PlayAPI.AugmentedReality.sessionEnd = MockPlayAPI.arSessionEnd;
            PlayAPI.AugmentedReality.registerImage = MockPlayAPI.arRegisterImage;
            PlayAPI.AugmentedReality.isAvailable = MockPlayAPI.arIsAvailable;
            PlayAPI.LocalStorage.list = MockPlayAPI.localStorageList;
            PlayAPI.LocalStorage.read = MockPlayAPI.localStorageRead;
            PlayAPI.LocalStorage.write = MockPlayAPI.localStorageWrite;
            PlayAPI.LocalStorage.unlink = MockPlayAPI.localStorageUnlink;
            PlayAPI.cancelSpeech = MockPlayAPI.cancelSpeech;
            PlayAPI.speak = MockPlayAPI.speak;
            PlayAPI.listAvailableVoices = MockPlayAPI.listAvailableVoices;
            // TODO: Figure out why this causes TSM to not load
            // (window as any).webkit = {messageHandlers:{NativeInterop:{postMessage: function (msg) {}}}};
        }
        static subscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] subscribeToEvent()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static unsubscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] unsubscribeToEvent()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static vibrateDevice() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] vibrateDevice()");
                return {};
            });
        }
        static startCamera(cameraType) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] startCamera()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    if (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS) {
                        reject();
                    }
                    else {
                        resolve({});
                    }
                }));
            });
        }
        static cameraAccessStatus() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] cameraAccessStatus()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {
                    status: (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS ? CameraAccessStatus.DENIED : CameraAccessStatus.AUTHORIZED)
                };
            });
        }
        static endCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] endCamera()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionLog(action, category) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionLog(${action}, ${category})`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionObjectLog(action, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionObjectLog(${action}, ${JSON.stringify(data)})`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsStateLog(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsStateLog(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionStart(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionStart(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionEnd(state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionEnd(${state}})`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static grantAchievementToPlayers(achievementId, playersIds, showNotification) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] grantAchievementToPlayers()");
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return {};
            });
        }
        static gameInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameInit()`);
                yield PromiseUtil_1.PromiseUtil.realTime(500);
                return MockPlayAPI.getFakeGameInitResult();
            });
        }
        static locationRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationRead()`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeLocation();
            });
        }
        static locationWaitTimeRead(location) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationWaitTimeRead()`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeWaitTime();
            });
        }
        static gameStateRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateRead()`);
                return MockPlayAPI.doGameStateRead();
            });
        }
        static gameStateSave(status, state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateSave()`);
                // CookieUtil.set(MockPlayAPI.GAME_STATE_TAG, JSON.stringify(state));
                return {};
            });
        }
        static dbSessionInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionInit()`);
                return {};
            });
        }
        static triggerEffect(effectName, data, timeout) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] triggerEffect()`);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PromiseUtil_1.PromiseUtil.realTime(timeout * 1000);
                    reject(); // The MockAPI should never succeed here
                }));
            });
        }
        static dbSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionEnd()`);
                return {};
            });
        }
        static arSessionStart(canvasElement) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionStart()`);
                PlayAPI.AugmentedReality.renderer = new THREE.WebGLRenderer({ alpha: true, canvas: canvasElement });
                PlayAPI.AugmentedReality.scene = new THREE.Scene();
                PlayAPI.AugmentedReality.renderer.setSize(window.innerWidth, window.innerHeight);
                PlayAPI.AugmentedReality.renderer.setPixelRatio(window.devicePixelRatio);
                PlayAPI.AugmentedReality.renderer.shadowMap.enabled = false;
                return {};
            });
        }
        static arSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionEnd()`);
                return {};
            });
        }
        static arRegisterImage(imageName, pathToImageInGameBundle, imageWidthInMeters) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arRegisterImage(${imageName}, ${pathToImageInGameBundle}, ${imageWidthInMeters})`);
                return {};
            });
        }
        static arIsAvailable() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arIsAvailable()`);
                return true;
            });
        }
        static localStorageList() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageList()`);
                return {
                    contents: []
                };
            });
        }
        static localStorageRead(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageRead()`);
                return {};
            });
        }
        static localStorageWrite(fileName, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageWrite()`);
                return {};
            });
        }
        static localStorageUnlink(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageUnlink()`);
                return {};
            });
        }
        static gameStatsRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsRead()`);
                return MockPlayAPI.doGameStatsRead();
            });
        }
        static gameStatsSave(gameStats) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsSave()`);
                CookieUtil_1.CookieUtil.set(MockPlayAPI.GAME_STATS_TAG, JSON.stringify(gameStats));
                return {};
            });
        }
        static openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.getFakePlayerManagementData();
            });
        }
        static captureImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] captureImage()`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return null;
            });
        }
        static saveToPhotos() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] saveToPhotos()`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return null;
            });
        }
        static shareImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] shareImage()`);
                yield PromiseUtil_1.PromiseUtil.realTime(300);
                return true;
            });
        }
        static closeGame() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.fakeCloseGame();
            });
        }
        static getFakeLocation() {
            const response = {
                attractionName: "Little Foot's Wild Ride",
                landName: "Land Before Time",
                latitude: 66.33,
                locationID: "beacon_" + Math.floor(Math.random() * 20),
                longitude: 100.22,
                parkName: "Animal Kingdom",
            };
            return response;
        }
        static doGameStateRead() {
            console.log(`[MockPlayAPI] doGameStateRead()`);
            let gameState;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATE_TAG));
            if (cookieData == null) {
                gameState = MockPlayAPI.getFakeGameState();
            }
            else {
                gameState = {
                    created: cookieData.created || "",
                    state: cookieData.state || {},
                    status: cookieData.status || "1",
                    updated: cookieData.updated || "",
                };
            }
            return gameState;
        }
        static doGameStatsRead() {
            console.log(`[MockPlayAPI] doGameStatsRead()`);
            let gameStats;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATS_TAG));
            if (cookieData == null) {
                gameStats = MockPlayAPI.getFakeGameStats();
            }
            else {
                gameStats = {
                    score: cookieData.score || 0,
                };
            }
            return gameStats;
        }
        static listAvailableVoices() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    let voicesList = window.speechSynthesis.getVoices();
                    if (voicesList.length === 0) {
                        const voicesLoadedSignal = new Signal_1.Signal();
                        window.speechSynthesis.onvoiceschanged = () => {
                            voicesLoadedSignal.dispatch();
                        };
                        yield PromiseUtil_1.PromiseUtil.commonSignal(voicesLoadedSignal);
                        voicesList = window.speechSynthesis.getVoices();
                    }
                    voicesList.forEach(element => {
                        element["identifier"] = element.voiceURI;
                    });
                    return { "voices": voicesList };
                }
                else {
                    return { "voices": [] };
                }
            });
        }
        static speak(text, voiceId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    const textToRead = new SpeechSynthesisUtterance(text);
                    const voice = window.speechSynthesis.getVoices().find((singleVoice) => {
                        return singleVoice.voiceURI === voiceId;
                    });
                    textToRead.voice = voice;
                    window.speechSynthesis.speak(textToRead);
                }
                return {};
            });
        }
        static cancelSpeech() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    window.speechSynthesis.cancel();
                }
                return {};
            });
        }
        /////////////////////////////////
        // FAKE DATA GENERATION
        /////////////////////////////////
        static getFakeGameInitResult() {
            const response = {
                gameDefinitionRead: MockPlayAPI.getFakeGameDefinition(),
                gameStateRead: MockPlayAPI.doGameStateRead(),
                playerInfoRead: MockPlayAPI.getFakePlayerInformation(),
                isGuestOnSite: true,
            };
            return response;
        }
        static getFakeWaitTime() {
            const response = {
                waitTime: Math.floor(Math.random() * 150),
            };
            return response;
        }
        static getFakePlayerInformation() {
            const response = {
                avatar: "",
                email: "example@gmail.com",
                name: "Joe A"
            };
            return response;
        }
        static getFakeGameDefinition() {
            const response = {
                stateSchema: {},
                statsSchema: {},
                title: "game_definition_title",
            };
            return response;
        }
        static getFakeGameState() {
            const response = {
                created: "game_state_created",
                state: "game_state_state",
                status: "game_state_status",
                updated: "game_state_updated",
            };
            return response;
        }
        static getFakeGameStats() {
            const response = {
                score: 0,
            };
            return response;
        }
        static getFakePlayerManagementData(limit = null) {
            const players = [{ name: "Cinderella", type: "TYPE_PRIMARY", identifier: "" },
                { name: "Elsa", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Snow White", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Leia", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Tinker Bell", type: "TYPE_SECONDARY", identifier: "" },
            ];
            if (limit != null) {
                players.splice(limit);
            }
            return { players };
        }
        static fakeCloseGame() {
            console.log("The game got closed using mockplayApi!");
            return {};
        }
    }
    MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS = false;
    MockPlayAPI.GAME_STATE_TAG = "game-state";
    MockPlayAPI.GAME_STATS_TAG = "game-stats";
    exports.MockPlayAPI = MockPlayAPI;
});
define("switchback-common/util/CollectionUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CollectionUtil {
        /**
         * Swaps two values of @param array by value (in place)
         */
        static swapByValue(array, x, y) {
            const xIndex = array.indexOf(x);
            const yIndex = array.indexOf(y);
            CollectionUtil.swapByIndex(array, xIndex, yIndex);
        }
        /**
         * Swaps two values of @param array by index (in place)
         */
        static swapByIndex(array, xIndex, yIndex) {
            if (xIndex >= 0 && yIndex >= 0) {
                let a = array[xIndex];
                array[xIndex] = array[yIndex];
                array[yIndex] = a;
            }
            else {
                console.warn(`[CollectionUtil] swap(): could not swap elements - one or more elements not present in array`);
            }
        }
        /**
         * Recursively merges 2 objects. The spread (...) operator doesn't act on
         * nested properties - this does.
         * https://stackoverflow.com/a/40276429/1343176
         * { "test": { "val1": 1 } }
         * +
         * { "test": { "val2": 2 } }
         * =
         * { "test": { "val1": 1, "val2": 2 } }
         */
        static mergeObjects(obj1, obj2) {
            for (var key in obj2) {
                if (typeof (obj1[key]) === 'object') {
                    CollectionUtil.mergeObjects(obj1[key], obj2[key]);
                }
                else {
                    if (obj1[key] || typeof (obj2[key]) === 'boolean') {
                        obj1[key] = obj2[key];
                    }
                }
            }
            return obj1;
        }
        /**
         * Returns an array pre-filled with sequential values from 0...(length - 1)
         * i.e. @param length = 5 will return [0, 1, 2, 3, 4]
         */
        static generateArrayWithSequentialNumbers(length) {
            return Array.from(Array(length).keys());
        }
        /**
         * Shuffles an array and then checks to see how 'shuffled' it is.
         * If more than half of the original members are in the same index
         * as the shuffled members, it shuffles again. A safeguard
         * counter is in place to prevent it infinitely looping.
         */
        static guaranteeShuffle(array) {
            let shuffledCopy = shuffle(Object.assign([], array));
            let safeguard = 0;
            while (CollectionUtil.checkArrayShuffleCoefficient(array, shuffledCopy) > 0.5 && safeguard < 1000) {
                shuffledCopy = shuffle(Object.assign([], array));
                safeguard++;
            }
            ;
            return shuffledCopy;
        }
        static checkArrayShuffleCoefficient(array, copy) {
            let numSame = 0;
            array.forEach((val, index) => {
                if (val === copy[index]) {
                    numSame++;
                }
            });
            return numSame / array.length;
        }
    }
    exports.CollectionUtil = CollectionUtil;
    function shuffle(array) {
        const arrayCopy = array;
        for (let i = array.length; i; i--) {
            const j = Math.floor(Math.random() * i);
            [arrayCopy[i - 1], arrayCopy[j]] = [array[j], array[i - 1]];
        }
        return arrayCopy;
    }
    exports.shuffle = shuffle;
    // puts the last index in first place before shuffling the rest
    function shuffleNonRepeatingArray(originalArray) {
        if (originalArray.length <= 1) {
            return originalArray;
        }
        // removes the last item in the array
        const lastItem = originalArray.pop();
        // shuffle the array without the last item in it
        originalArray = shuffle(originalArray);
        /*
        * inserts the last item back into the middle of the array
        * so that the it will not end up as lastItem forever
        */
        originalArray.splice(Math.ceil(originalArray.length * 0.5), 0, lastItem);
        return originalArray;
    }
    exports.shuffleNonRepeatingArray = shuffleNonRepeatingArray;
    /**
     * Finds the 2D index of an item in a 2D array
     * @param arr Array to search
     * @param k Item to look for
     */
    function getIndexIn2DArray(arr, k) {
        for (let i = 0; i < arr.length; i++) {
            const index = arr[i].indexOf(k);
            if (index > -1) {
                return { row: i, col: index };
            }
        }
    }
    exports.getIndexIn2DArray = getIndexIn2DArray;
    /**
     * Returns an array of numbers from 0 to length exclusive eg. getNumberRange(3) returns [0, 1, 2]
     * @param length number, the end of the range
     */
    function getNumberRange(length) {
        return Array.apply(null, { length: length }).map(Number.call, Number);
    }
    exports.getNumberRange = getNumberRange;
    /**
     * Returns an array of numbers from start (inclusive) to start + length (exclusive) eg. getNumberRange(2, 3) returns [2, 3, 4]
     * @param start number, the first number in the range
     * @param length number, the end of the range
     */
    function getNumberRangeFrom(start, length) {
        let range = [];
        getNumberRange(length).forEach((n) => {
            range.push(n + start);
        });
        return range;
    }
    exports.getNumberRangeFrom = getNumberRangeFrom;
    function cloneArray(arr) {
        let i, copy;
        if (Array.isArray(arr)) {
            copy = arr.slice(0);
            for (i = 0; i < copy.length; i++) {
                copy[i] = cloneArray(copy[i]);
            }
            return copy;
        }
        else if (typeof arr === 'object') {
            throw 'Cannot clone array containing an object!';
        }
        else {
            return arr;
        }
    }
    exports.cloneArray = cloneArray;
    function exclusiveLeftJoinArrays(leftList, rightList) {
        const exclusiveLeftJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) === -1;
        });
        return exclusiveLeftJoin;
    }
    exports.exclusiveLeftJoinArrays = exclusiveLeftJoinArrays;
    /**
     * Return an array containing the matching elements from two arrays
     */
    function innerJoinArrays(leftList, rightList) {
        const innerJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) >= 0;
        });
        return innerJoin;
    }
    exports.innerJoinArrays = innerJoinArrays;
    function getRandomItem(list) {
        const randomIndex = Math.floor(Math.random() * Math.floor(list.length));
        return list.length === 0 ? null : list[randomIndex];
    }
    exports.getRandomItem = getRandomItem;
});
define("switchback-common/enum/Park", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Listing of Park 'types'
     *
     * I know it's not an enum, but this is to prevent
     * people with older TS versions having issues.
     */
    class Park {
    }
    Park.DLR = "disneylandResort";
    Park.WDW = "waltDisneyWorld";
    exports.Park = Park;
});
define("switchback-common/play/AnalyticsManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AnalyticsManager {
        constructor() {
            // Needs to be overwritten on the extending class
            this.gaNameMap = new Map();
            this.timings = new Map();
            this.gaCommonNameMap = new Map([
                ["park.name", "dimension1"],
                ["line.type", "dimension2"],
                ["attraction.estimatedWaitTime", "dimension3"],
                // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
                // ['attraction.actualWaitTime', 'dimension4'],
                ["party.size", "dimension5"],
                ["party.visits", "dimension6"],
            ]);
            this.ga = null;
        }
        /**
         * "A" = Standard
         * "B" = Other
         * "N/A" = No queue distinctions
         */
        set lineType(type) {
            this.baseData["line.type"] = type;
        }
        set estimatedWaitTime(time) {
            this.baseData["attraction.estimatedWaitTime"] = time;
        }
        set numPartyVisits(numVisits) {
            this.baseData["party.visits"] = numVisits;
        }
        set actualWaitTime(time) {
            // Disabled because we were advised that this parameter should not be in adobe metrics
            // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
            // after that this code can be cleaned up
            // this.baseData['attraction.actualWaitTime'] = time;
        }
        /**
         * Starts a timing event.
         * For Adobe, this is automatic.
         * For Google analytics, we record the time ourselves.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         */
        startTiming(adobeTimingKey, data = {}) {
            this.timings.set(adobeTimingKey, new Date().getTime());
            PlayAPI.analyticsTimedActionStart(adobeTimingKey, Object.assign({}, this.baseData, data));
        }
        /**
         * Ends and sends a timing event.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         * @param gaCategory The category of this event e.g. 'Installation'
         * @param gaLabel Additional information about the action e.g. 'Poster'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        endTiming(adobeTimingKey, gaTimingKey, gaCategory, gaLabel, data = {}) {
            return __awaiter(this, void 0, void 0, function* () {
                const prevTime = this.timings.get(adobeTimingKey);
                if (prevTime) {
                    const totalTime = new Date().getTime() - prevTime;
                    this.ga("send", "timing", Object.assign({ timingCategory: gaCategory, timingLabel: gaLabel, timingValue: totalTime, timingVar: gaTimingKey }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
                }
                yield PlayAPI.analyticsTimedActionEnd(adobeTimingKey).then((result) => {
                    console.log(result);
                }).catch((error) => {
                    console.error(error);
                });
            });
        }
        /**
         * Tracks an Adobe 'Action' or a Google 'event'.
         *
         * @param gaCategory The category of action e.g. 'Game'
         * @param gaAction The specific action they took e.g. 'Play'
         * @param gaLabel Further information about that action e.g. 'Woody'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackAction(adobeAction, gaCategory, gaAction, gaLabel, data = {}) {
            PlayAPI.analyticsActionObjectLog(adobeAction, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ eventAction: gaAction, eventCategory: gaCategory, eventLabel: gaLabel, hitType: "event" }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        /**
         * Tracks an Adobe 'state' or a Google 'pageview'
         *
         * @param adobeStatePath e.g. 'content/game/inqueue/resume'
         * @param gaStateId e.g. 'Resume'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackState(adobeStatePath, gaStateId, data = {}) {
            PlayAPI.analyticsStateLog(adobeStatePath, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ hitType: "pageview", page: gaStateId }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        initialise(category, landName, attractionName, gameId, gameName, parkName, lineType, hasPrimaryPlayer, partySize, partyVisits, useGA = false) {
            this.baseData = {
                "attraction.name": attractionName,
                "game.id": gameId,
                "game.name": gameName,
                "land.name": landName,
                "line.type": lineType,
                "link.category": category,
                "park.name": parkName,
                "party.size": partySize,
                "party.visits": partyVisits,
                "primary.player": hasPrimaryPlayer ? 1 : 0,
            };
            // Combine both our name maps
            this.gaNameMap = new Map([
                ...Array.from(this.gaNameMap.entries()),
                ...Array.from(this.gaCommonNameMap.entries()),
            ]);
            if (useGA) {
                this.ga = window.ga;
            }
            else {
                this.ga = (...params) => { console.log(...params); };
            }
        }
        /**
         * Extract custom data into a format Google Analytics will accept
         */
        transformFieldsObject(data) {
            const transformedData = {};
            Object.keys(data).forEach((key) => {
                const mapping = this.gaNameMap.get(key);
                if (mapping && data[key] !== undefined && data[key] !== null) {
                    if (mapping.startsWith("dimension")) {
                        data[key] = data[key].toString(); // GA requires custom dimensions pass through as strings
                    }
                    transformedData[mapping] = data[key];
                }
            });
            return transformedData;
        }
    }
    AnalyticsManager.PARK_WDW = "WDW";
    AnalyticsManager.PARK_DLR = "DLR";
    AnalyticsManager.LAND_HS = "HS";
    AnalyticsManager.LAND_MK = "MK";
    AnalyticsManager.LAND_DCA = "DCA";
    AnalyticsManager.LAND_DL = "DL";
    AnalyticsManager.LAND_AK = "AK";
    AnalyticsManager.QUEUE_A = "A";
    AnalyticsManager.QUEUE_B = "B";
    AnalyticsManager.QUEUE_NONE = "N/A";
    exports.AnalyticsManager = AnalyticsManager;
});
define("config/SpaceMountainConfig", ["require", "exports", "switchback-common/enum/Park"], function (require, exports, Park_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Config singleton. Parsed from json.
     * @author joel@cerebralfix.com
     */
    class SpaceMountainConfig {
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        parse(data) {
            if (typeof data === 'string') {
                try {
                    data = JSON.parse(data);
                }
                catch (e) {
                    console.log('Unable to parse config.');
                }
            }
            this.enableConsoleLogs = data.enableConsoleLogs === undefined ? false : data.enableConsoleLogs;
            this.enableFPS = data.enableFPS === undefined ? false : data.enableFPS;
            this.usePlayAPI = data.usePlayAPI === undefined ? true : data.usePlayAPI;
            this.debug = data.debug === undefined ? false : data.debug;
            this.quickTest = data.quickTest === undefined ? false : data.quickTest;
            this.showDebugMenu = data.showDebugMenu === undefined ? false : data.showDebugMenu;
            this.skipGames = data.skipGames === undefined ? false : data.skipGames;
            this.skipTutorials = data.skipTutorials === undefined ? false : data.skipTutorials;
            this.showCodeRedeem = data.showCodeRedeem === undefined ? true : data.showCodeRedeem;
            this.forceAllPipesLevels = data.forceAllPipesLevels === undefined ? false : data.forceAllPipesLevels;
            this.showStateNames = data.showStateNames === undefined ? false : data.showStateNames;
            this.canRaceBeforeUnlock = data.canRaceBeforeUnlock === undefined ? false : data.canRaceBeforeUnlock;
            this.showAsteroidsTimer = data.showAsteroidsTimer === undefined ? false : data.showAsteroidsTimer;
            this.raceDebugBeacons = data.raceDebugBeacons === undefined ? false : data.raceDebugBeacons;
            this.raceFilterXPID = data.raceFilterXPID; // undefined is ok
            this.raceExperienceName = data.raceExperienceName === undefined ? "wdw-mk-spacemtn-race" : data.raceExperienceName;
            // TODO this value needs to be set in accordance with the Videos and the way the installation works
            // PlayAPI only notify us when the video actually starts, so when we get the beacon the video already started
            // we use a padding on the video to accommodate for user notification, but needs to be done with the videos
            // it cant be too high, otherwise it might delay the user if the PreRaceState accounts for the time "before" the race
            this.raceBeaconWindow = data.raceBeaconWindow === undefined ? 60 : data.raceBeaconWindow;
            // Defaulting this to DLR now that we cannot guarantee the gameInit response will give us something we expect
            this.park = data.park || Park_1.Park.DLR;
        }
    }
    exports.SpaceMountainConfig = SpaceMountainConfig;
});
define("logic/mission/MissionType", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionType {
    }
    MissionType.PARTS_GATHERED = "PARTS_GATHERED";
    MissionType.RACE_WIN = "RACE_WIN";
    MissionType.RACE_PARTICIPATION = "RACE_PARTICIPATION";
    MissionType.GAME_PLAYED = "GAME_PLAYED";
    exports.MissionType = MissionType;
});
define("switchback-common/localization/LocalizationManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Localization singleton. Parsed from json.
     * @author joel@cerebralfix.com
     */
    class LocalizationManager {
        static get Instance() {
            return this.instance || (this.instance = new this());
        }
        init(pLocale, data) {
            if (data === null) {
                throw new Error("Localization data not found");
            }
            this.polyglot = new Polyglot({
                locale: pLocale,
                phrases: data,
            });
            this.isReversed = false;
        }
        text(phrase) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase));
            }
            return this.polyglot.t(phrase);
        }
        textWithCount(phrase, smartCount) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, smartCount));
            }
            return this.polyglot.t(phrase, smartCount);
        }
        textWithReplacements(phrase, interpolationOptions) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, interpolationOptions));
            }
            return this.polyglot.t(phrase, interpolationOptions);
        }
        setReversed(reversed) {
            this.isReversed = reversed;
        }
        /**
         * Please note that this method of reversal should only be used for
         * debugging purposes, and won't properly handle non-ASCII chars
         */
        reverseString(source) {
            return source.split("").reverse().join("");
        }
    }
    exports.LocalizationManager = LocalizationManager;
});
define("logic/mission/MissionRewardType", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionRewardType {
    }
    MissionRewardType.RACE = "RACE";
    MissionRewardType.PARTS = "PARTS";
    exports.MissionRewardType = MissionRewardType;
});
define("logic/mission/Mission", ["require", "exports", "logic/mission/MissionType", "switchback-common/localization/LocalizationManager", "logic/mission/MissionRewardType", "logic/GameController"], function (require, exports, MissionType_1, LocalizationManager_1, MissionRewardType_1, GameController_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Mission {
        constructor(data) {
            this.id = data.id;
            this.prerequisites = data.prerequisites;
            this.type = data.type;
            this.target = data.target;
            this.rewardType = data.rewardType;
            this.rewardValue = data.rewardValue;
        }
        get description() {
            switch (this.type) {
                case MissionType_1.MissionType.PARTS_GATHERED: {
                    return LocalizationManager_1.LocalizationManager.Instance.textWithCount("mission.description_parts", this.target);
                }
                case MissionType_1.MissionType.RACE_WIN: {
                    return LocalizationManager_1.LocalizationManager.Instance.text("mission.description_race_win");
                }
                case MissionType_1.MissionType.RACE_PARTICIPATION: {
                    return LocalizationManager_1.LocalizationManager.Instance.textWithCount("mission.description_race_participation", this.target);
                }
                case MissionType_1.MissionType.GAME_PLAYED: {
                    return LocalizationManager_1.LocalizationManager.Instance.textWithCount("mission.description_games", this.target);
                }
            }
        }
        get title() {
            return LocalizationManager_1.LocalizationManager.Instance.text(`mission.name_${this.id}`);
        }
        get rewardDescription() {
            switch (this.rewardType) {
                case MissionRewardType_1.MissionRewardType.PARTS: {
                    return LocalizationManager_1.LocalizationManager.Instance.textWithCount("mission.reward_parts", this.rewardValue);
                }
                case MissionRewardType_1.MissionRewardType.RACE: {
                    return LocalizationManager_1.LocalizationManager.Instance.text("mission.reward_race");
                }
            }
        }
        get isActive() {
            return GameController_1.GameController.Instance.state.getIsMissionActive(this.id);
        }
        set isActive(isActive) {
            GameController_1.GameController.Instance.state.setIsMissionActive(this.id, isActive);
            if (isActive && this.type === MissionType_1.MissionType.PARTS_GATHERED) {
                this.progress = GameController_1.GameController.Instance.getTotalShipParts();
            }
        }
        get progressProportion() {
            return this.progress / this.target;
        }
        get pendingProgressProportion() {
            return this.pendingProgress / this.target;
        }
        awardPendingProgress() {
            this.progress += this.pendingProgress;
            this.pendingProgress = 0;
            if (this.progress >= this.target) {
                this.hasBeenCompleted = true;
            }
        }
        /**
         * The progress and pendingProgress getter/setters are routed to the save state
         */
        get progress() {
            return GameController_1.GameController.Instance.state.getProgressForMission(this.id);
        }
        set progress(progress) {
            GameController_1.GameController.Instance.state.setProgressForMission(this.id, progress);
        }
        get pendingProgress() {
            return GameController_1.GameController.Instance.state.getPendingProgressForMission(this.id);
        }
        set pendingProgress(pendingProgress) {
            GameController_1.GameController.Instance.state.setPendingProgressForMission(this.id, pendingProgress);
        }
        get hasBeenCompleted() {
            return GameController_1.GameController.Instance.state.getHasBeenCompletedForMission(this.id);
        }
        set hasBeenCompleted(hasBeenCompleted) {
            GameController_1.GameController.Instance.state.setHasBeenCompletedForMission(this.id, hasBeenCompleted);
        }
    }
    exports.Mission = Mission;
});
define("switchback-common/save/SaveState", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SaveState {
        constructor() {
            this._isSaveEnabled = true;
        }
        save() {
            if (this._isSaveEnabled) {
                this.forceSave();
            }
        }
        clear() {
            console.log("[SaveState] clear");
            this.parse([]);
            this.forceSave();
        }
        /**
         * This disables all saves from this point. Clear will still function as expected.
         */
        disableSave() {
            this._isSaveEnabled = false;
        }
        forceSave() {
            PlayAPI.gameStateSave(GameStatus.GAME_STATUS_IN_PROGRESS, this).then(SaveState.onSaveSucceed, SaveState.onSaveFailed).catch(SaveState.onSaveExcept);
        }
        static onSaveSucceed() {
            console.log("[SaveState] save() success");
        }
        static onSaveFailed(err) {
            console.error("[SaveState] save() failed " + JSON.stringify(err));
        }
        static onSaveExcept(err) {
            console.error("[SaveState] save() inner exception" + JSON.stringify(err));
        }
    }
    exports.SaveState = SaveState;
});
define("model/SpaceSaveState", ["require", "exports", "logic/GameController", "switchback-common/save/SaveState", "config/SpaceMountainConfig", "switchback-common/enum/Park"], function (require, exports, GameController_2, SaveState_1, SpaceMountainConfig_1, Park_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MinigameScore {
        constructor() {
            this.highScore = 0;
            this.currentScore = 0;
            this.wasHighScoreInLastPlay = false;
        }
    }
    exports.MinigameScore = MinigameScore;
    class MissionSaveData {
        constructor() {
            this.id = 0;
            this.progress = 0;
            this.pendingProgress = 0;
            this.isActive = false;
            this.hasBeenCompleted = false;
        }
    }
    exports.MissionSaveData = MissionSaveData;
    var Queue;
    (function (Queue) {
        Queue[Queue["PRIMARY"] = 0] = "PRIMARY";
        Queue[Queue["SECONDARY"] = 1] = "SECONDARY"; // ADA Entrance
    })(Queue = exports.Queue || (exports.Queue = {}));
    class SpaceSaveState extends SaveState_1.SaveState {
        get numRaces() {
            return this._numRaces;
        }
        // Sets when the user has participated in a race
        // currently set at PreRaceState while transitioning
        set numRaces(value) {
            this._numRaces = value;
        }
        get hasMerged() {
            return this._hasMerged;
        }
        // Sets when the user has reached the merge point (end of queue)
        set hasMerged(merged) {
            this._hasMerged = merged;
        }
        // Returns true if the user can keep playing
        // (usually after or before the end of the queue)
        get canResumePostMerge() {
            return (this._hasMerged &&
                this.numRaces === 0 &&
                SpaceMountainConfig_1.SpaceMountainConfig.Instance.park === Park_2.Park.WDW &&
                GameController_2.GameController.Instance.state.queueType === Queue.SECONDARY);
        }
        get shouldForceOutroAfterRace() {
            return (this._hasMerged &&
                this.numRaces > 0 &&
                SpaceMountainConfig_1.SpaceMountainConfig.Instance.park === Park_2.Park.WDW &&
                GameController_2.GameController.Instance.state.queueType === Queue.SECONDARY);
        }
        get queueType() {
            return this._queueType;
        }
        set queueType(queue) {
            this._queueType = queue;
            this.save();
        }
        get ftueCodeTutorial() {
            return this._ftueCodeTutorial;
        }
        set ftueCodeTutorial(value) {
            this._ftueCodeTutorial = value;
        }
        get ftueCompleted() {
            return this._ftueCompleted;
        }
        set ftueCompleted(ftueCompleted) {
            this._ftueCompleted = ftueCompleted;
        }
        get sessionStarted() {
            return this._sessionStarted;
        }
        set sessionStarted(value) {
            this._sessionStarted = value;
        }
        get isInForcedProgression() {
            return this._onForcedProgression;
        }
        set isInForcedProgression(value) {
            this._onForcedProgression = value;
        }
        get players() {
            return this._players;
        }
        set players(value) {
            this._players = value;
        }
        get shipBaseParts() {
            return this._shipBaseParts;
        }
        set shipBaseParts(n) {
            this._shipBaseParts = n;
        }
        get shipPendingBaseParts() {
            return this._shipPendingBaseParts;
        }
        set shipPendingBaseParts(n) {
            this._shipPendingBaseParts = n;
        }
        get shipBonusParts() {
            return this._shipBonusParts;
        }
        set shipBonusParts(n) {
            this._shipBonusParts = n;
        }
        get shipPendingBonusParts() {
            return this._shipPendingBonusParts;
        }
        set shipPendingBonusParts(n) {
            this._shipPendingBonusParts = n;
        }
        get isSpaceRaceUnlocked() {
            return this._isSpaceRaceUnlocked;
        }
        set isSpaceRaceUnlocked(b) {
            this._isSpaceRaceUnlocked = b;
        }
        get lastRacePlacement() {
            return this._previousRacePlacement;
        }
        set lastRacePlacement(value) {
            this._previousRacePlacement = value;
        }
        /**
         * This is the order in which games have been (or are about to be) played.
         * It's different to currentHubLayout, and contains kind of a superset of information.
         *
         * This is used to ensure players don't get the same game
         * a bunch of times in a row, or miss out on games entirely.
         *
         * It is regenerated when players finish going through it.
         */
        set gameOrder(order) {
            this._gameOrder = order;
        }
        get gameOrder() {
            return this._gameOrder;
        }
        get progressThroughGameOrder() {
            return this._progressThroughGameOrder;
        }
        set progressThroughGameOrder(index) {
            this._progressThroughGameOrder = index;
        }
        get numGamePlays() {
            return this._numGamePlays;
        }
        set numGamePlays(plays) {
            this._numGamePlays = plays;
        }
        get numVisits() {
            return this._numVisits;
        }
        set numVisits(num) {
            this._numVisits = num;
        }
        get sessionPlayTime() {
            return this._sessionPlayTime;
        }
        set sessionPlayTime(time) {
            this._sessionPlayTime = time;
        }
        get minigameScores() {
            return this._minigameScores;
        }
        /**
         * How many times has a minigame been played?
         */
        getMinigamePlays(minigameName) {
            return this._minigamePlays[minigameName] || 0;
        }
        incrementMinigamePlays(minigameName) {
            this._minigamePlays[minigameName] = this._minigamePlays[minigameName] + 1 || 1;
            return this._minigamePlays[minigameName];
        }
        // Returns the score for a minigame.
        // If it doesn't exists, it creates a new one.
        // Any change to the returned MinigameScore object should be saved by calling this.save
        getMinigameScore(gameName) {
            let score = this._minigameScores.find(obj => obj.gameName === gameName);
            if (score == null) {
                score = new MinigameScore();
                score.currentScore = 0;
                score.highScore = 0;
                score.gameName = gameName;
                this.addMinigameScore(score);
            }
            return score;
        }
        // Adds a minigame score to the internal list, Notice that adding a score is considered "playing the game"
        // It can trigger the achievement onAllMinigamesPlayed
        addMinigameScore(score) {
            this._minigameScores.push(score);
            // if we have all game scores in here
            // award Astronaut achievement
            if (this._minigameScores.length === GameController_2.GameController.Instance.getGameCount()) {
                GameController_2.GameController.Instance.achievementManager.onAllMinigamesPlayed();
            }
        }
        set minigameScores(scores) {
            this._minigameScores = scores;
        }
        get redeemedCodes() {
            return this._redeemedCodes;
        }
        addRedeemedCode(code) {
            this._redeemedCodes.push(code);
            GameController_2.GameController.Instance.addPendingBonusShipParts(1);
        }
        getProgressForMission(id) {
            const missionData = this.getMissionById(id);
            return missionData ? missionData.progress : 0;
        }
        getPendingProgressForMission(id) {
            const missionData = this.getMissionById(id);
            return missionData ? missionData.pendingProgress : 0;
        }
        getHasBeenCompletedForMission(id) {
            const missionData = this.getMissionById(id);
            return missionData ? missionData.hasBeenCompleted : false;
        }
        getMissionById(id) {
            return this._missionProgressData.find((mission) => {
                return mission.id === id;
            }, this);
        }
        getIsMissionActive(id) {
            const missionData = this.getMissionById(id);
            return missionData ? missionData.isActive : false;
        }
        getOrCreateMission(id) {
            let missionData = this.getMissionById(id);
            // We didn't find a mission with that id? Create it!
            if (!missionData) {
                missionData = new MissionSaveData();
                missionData.id = id;
                this._missionProgressData.push(missionData);
            }
            return missionData;
        }
        setIsMissionActive(id, isActive) {
            this.getOrCreateMission(id).isActive = isActive;
        }
        setHasBeenCompletedForMission(id, hasBeenCompleted) {
            this.getOrCreateMission(id).hasBeenCompleted = hasBeenCompleted;
        }
        setPendingProgressForMission(id, pendingProgress) {
            this.getOrCreateMission(id).pendingProgress = pendingProgress;
        }
        setProgressForMission(id, progress) {
            this.getOrCreateMission(id).progress = progress;
        }
        parse(data) {
            console.log("[SaveState] Data is: ", JSON.stringify(data));
            if (!data) {
                data = [];
            }
            this._ftueCompleted = data._ftueCompleted || false;
            this._ftueCodeTutorial = data._ftueCodeTutorial || false;
            this._sessionStarted = data._sessionStarted || false;
            this._onForcedProgression = data._onForcedProgression == null ? true : data._onForcedProgression;
            this._players = data._players || [];
            this._shipBaseParts = data._shipBaseParts || 0;
            this._shipPendingBaseParts = data._shipPendingBaseParts || 0;
            this._shipBonusParts = data._shipBonusParts || 0;
            this._shipPendingBonusParts = data._shipPendingBonusParts || 0;
            this._minigameScores = data._minigameScores || [];
            this._gameOrder = data._gameOrder || [];
            this._progressThroughGameOrder = data._progressThroughGameOrder || 0;
            this._redeemedCodes = data._redeemedCodes || [];
            this._queueType = data._queueType; // If this is undefined, it hasn't been selected - no default needed
            this._missionProgressData = data._missionProgressData || [];
            this._isSpaceRaceUnlocked = data._isSpaceRaceUnlocked;
            this._hasMerged = data._hasMerged || false;
            this._numRaces = data._numRaces || 0;
            this._sessionPlayTime = data._sessionPlayTime || 0;
            this._numVisits = data._numVisits || 0;
            this._numGamePlays = data._numGamePlays || 0;
            this._previousRacePlacement = data._previousRacePlacement; // This can be undefined
            this._minigamePlays = {};
            for (let k in (data._minigamePlays || {})) {
                let v = data._minigamePlays[k];
                if (k !== null && v !== null) {
                    this._minigamePlays[k] = v;
                }
            }
            return this;
        }
    }
    exports.SpaceSaveState = SpaceSaveState;
});
define("StateNames", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class StateNames {
    }
    StateNames.LOAD = "Load";
    StateNames.INTRO = "Intro";
    StateNames.NARRATIVE_BRIEFING = "NarrativeBriefing";
    StateNames.NARRATIVE_DEBRIEFING = "NarrativeDebriefing";
    StateNames.HUB = "Hub";
    StateNames.OUTRO = "Outro";
    StateNames.REDEEM_CODE = "Redeem Code";
    StateNames.PRE_RACE = "PreRace";
    StateNames.POST_RACE_PLACEMENT = "PostRacePlacement";
    StateNames.POST_RACE_REVIEW = "PostRaceReview";
    StateNames.INSTALLATION_RACE_IN_PROGRESS = "InstallationRaceInProgress";
    StateNames.DEVICE_RACE = "DeviceRace";
    /* These values are being used as localization keys */
    StateNames.GAME_ASTEROIDS = "game_asteroids"; // Pilot
    StateNames.GAME_MEMORY = "game_memory"; // Commander
    StateNames.GAME_RELAY_INTRO = "game_relay"; // Maintenance
    StateNames.GAME_ENERGY_BALLS = "game_energy_balls"; // Engineer
    StateNames.GAME_RELAY_OUTRO = "game_relay_outro";
    StateNames.GAME_RELAY_SWITCHES = "game_relay_switches";
    StateNames.GAME_RELAY_SCANNER = "game_relay_scanner";
    StateNames.GAME_RELAY_WRENCH = "game_relay_wrench";
    StateNames.GAME_RELAY_LOCKS = "game_relay_locks";
    StateNames.GAME_PIPES = "game_pipes"; // Navigator
    StateNames.GAME_DOCKING = "game_docking"; // Specialist
    exports.StateNames = StateNames;
});
define("switchback-common/play/AchievementManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AchievementManager {
        initialise(players, achievementPrefix) {
            this.playerIds = players.map((player) => player.identifier);
            this.achievementPrefix = achievementPrefix;
            console.log(`[AchievementManager] initialise(${this.playerIds}, ${achievementPrefix}`);
        }
        grantAchievement(achievementId) {
            return __awaiter(this, void 0, void 0, function* () {
                const achievementString = `${this.achievementPrefix}-${achievementId}`;
                yield PlayAPI.grantAchievementToPlayers(achievementString, this.playerIds, true).then(() => {
                    console.log(`achievement ${achievementString} grant success`);
                }, (reason) => {
                    console.log(`achievement ${achievementString} grant fail`);
                    console.error(JSON.stringify(reason));
                });
            });
        }
    }
    exports.AchievementManager = AchievementManager;
});
define("logic/SMAchievementManager", ["require", "exports", "switchback-common/play/AchievementManager", "logic/GameController"], function (require, exports, AchievementManager_1, GameController_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SMAchievement {
    }
    SMAchievement.PLAY_ALL = "spaceMountainAstronaut";
    SMAchievement.LAUNCH_SHIP = "spaceMountainMaidenVoyage";
    SMAchievement.SHIP_PARTS_1 = "spaceMountainInterstellar";
    SMAchievement.SHIP_PARTS_2 = "spaceMountainIntergalactic";
    SMAchievement.REDEEM_CODES = "spaceMountainSalvager";
    SMAchievement.MEMORY = "spaceMountainCommander";
    SMAchievement.ENERGY_BALLS = "spaceMountainEngineer";
    SMAchievement.ASTEROIDS = "spaceMountainPilot";
    SMAchievement.RELAY = "spaceMountainMaintenance";
    SMAchievement.PIPES = "spaceMountainNavigator";
    SMAchievement.DOCKING = "spaceMountainSpecialist";
    SMAchievement.WIN_SPACE_RACE = "spaceMountainLightspeed";
    class SMAchievementManager extends AchievementManager_1.AchievementManager {
        constructor() {
            super(...arguments);
            this.shipPartTarget1 = 40;
            this.shipPartTarget2 = 60;
            this.queueCodeTarget = 10;
            this.memoryRoundTarget = 4; // round number
            this.energyBallsTargetTimePerRound = 10; // secs per round
            this.asteroidsTargetScore = 20; // score per player
            this.relayRepairTargetPerPerson = 3; // no. microgames completed
            this.pipesTargetTimePerRound = 12; // secs per round
            this.dockingTargetTimePerRound = 10; // secs per round
        }
        onShipPartAwarded(numRedeemedParts) {
            return __awaiter(this, void 0, void 0, function* () {
                if (numRedeemedParts >= this.shipPartTarget2) {
                    yield this.grantAchievement(SMAchievement.SHIP_PARTS_2);
                }
                else if (numRedeemedParts >= this.shipPartTarget1) {
                    yield this.grantAchievement(SMAchievement.SHIP_PARTS_1);
                }
            });
        }
        onQueueCodeRedeemed() {
            return __awaiter(this, void 0, void 0, function* () {
                const numCodesRedeemed = GameController_3.GameController.Instance.state.redeemedCodes.length;
                if (numCodesRedeemed >= this.queueCodeTarget) {
                    yield this.grantAchievement(SMAchievement.REDEEM_CODES);
                }
            });
        }
        onAllMinigamesPlayed() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.grantAchievement(SMAchievement.PLAY_ALL);
            });
        }
        onShipLaunch() {
            return __awaiter(this, void 0, void 0, function* () {
                // The "Maiden Voyage" achievement. Not currently being triggered
                yield this.grantAchievement(SMAchievement.LAUNCH_SHIP);
            });
        }
        onSpaceRaceWin() {
            return __awaiter(this, void 0, void 0, function* () {
                // The "Lightspeed" achievement. Not currently being triggered
                yield this.grantAchievement(SMAchievement.WIN_SPACE_RACE);
            });
        }
        onEnergyBallsEndGame(avgSecsPerRound) {
            return __awaiter(this, void 0, void 0, function* () {
                if (avgSecsPerRound <= this.energyBallsTargetTimePerRound) {
                    yield this.grantAchievement(SMAchievement.ENERGY_BALLS);
                }
            });
        }
        onAsteroidsEndGame(avgPointsPerPlayer) {
            return __awaiter(this, void 0, void 0, function* () {
                if (avgPointsPerPlayer >= this.asteroidsTargetScore) {
                    yield this.grantAchievement(SMAchievement.ASTEROIDS);
                }
            });
        }
        onRelayEndGame(avgNumTasksComplete) {
            return __awaiter(this, void 0, void 0, function* () {
                if (avgNumTasksComplete >= this.relayRepairTargetPerPerson) {
                    yield this.grantAchievement(SMAchievement.RELAY);
                }
            });
        }
        onPipesEndGame(avgSecsPerRound) {
            return __awaiter(this, void 0, void 0, function* () {
                if (avgSecsPerRound <= this.pipesTargetTimePerRound) {
                    yield this.grantAchievement(SMAchievement.PIPES);
                }
            });
        }
        onDockingEndGame(avgSecsPerRound) {
            return __awaiter(this, void 0, void 0, function* () {
                if (avgSecsPerRound <= this.dockingTargetTimePerRound) {
                    yield this.grantAchievement(SMAchievement.DOCKING);
                }
            });
        }
        onMemoryEndGame(currentRound) {
            return __awaiter(this, void 0, void 0, function* () {
                if (currentRound > this.memoryRoundTarget) {
                    yield this.grantAchievement(SMAchievement.MEMORY);
                }
            });
        }
    }
    exports.SMAchievementManager = SMAchievementManager;
});
define("logic/comms/Message", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Message {
        constructor(type, isBroadcast, targetId = '') {
            this.type = type;
            this.isBroadcast = isBroadcast;
            this.targetId = targetId;
            this.details = {};
        }
    }
    exports.Message = Message;
});
define("logic/comms/MessageTypes", ["require", "exports", "logic/comms/Message"], function (require, exports, Message_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var MessageType;
    (function (MessageType) {
        MessageType["RACE_ID"] = "race_id";
    })(MessageType = exports.MessageType || (exports.MessageType = {}));
    class RaceIdMessage extends Message_1.Message {
        constructor(raceId, raceTime) {
            super(MessageType.RACE_ID, true);
            this.details = {
                raceId,
                raceTime
            };
        }
    }
    exports.RaceIdMessage = RaceIdMessage;
});
define("switchback-common/util/MathUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Returns a random number between min (inclusive) and max (exclusive)
     */
    function getRandomArbitrary(min, max) {
        return Math.random() * (max - min) + min;
    }
    exports.getRandomArbitrary = getRandomArbitrary;
    /**
     * Returns a random integer between min (inclusive) and max (inclusive)
     * Using Math.round() will give you a non-uniform distribution!
     */
    function getRandomInt(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
    exports.getRandomInt = getRandomInt;
    /**
     * Generate a nomally distibuted number based on the central limit theorem between 0 and 1
     * https://stackoverflow.com/questions/25582882/javascript-math-random-normal-distribution-gaussian-bell-curve/39187274#39187274
     */
    function normalRandom(numSamples = 7) {
        let rand = 0;
        for (let i = 0; i < numSamples; i++) {
            rand += Math.random();
        }
        return rand / numSamples;
    }
    exports.normalRandom = normalRandom;
    /*
        Utility function to get the shortest difference between two angles
        Used for tweens, it works, given the fact that phaser wraps the angle to the
        correct space whenever we assign it (which allows us to overflow the range by doing current+dif)
        And also because the tweens don't seem to rely on re-reading the value of the variable (which is awesome)
    */
    function getShorterAngleDif(currentAngle, newAngle) {
        if (currentAngle < 0) {
            currentAngle += 360;
        }
        if (newAngle < 0) {
            newAngle += 360;
        }
        let dif = newAngle - currentAngle;
        if (dif < -180) {
            dif += 360;
        }
        else if (dif > 180) {
            dif -= 360;
        }
        return dif;
    }
    exports.getShorterAngleDif = getShorterAngleDif;
    function base64ToByteArray(base64) {
        const binary_string = window.atob(base64);
        const len = binary_string.length;
        const bytes = new Uint8Array(len);
        for (var i = 0; i < len; i++) {
            bytes[i] = binary_string.charCodeAt(i);
        }
        return bytes;
    }
    exports.base64ToByteArray = base64ToByteArray;
    function lerp2D(pointA, pointB, amount) {
        const dX = pointA.x + (pointB.x - pointA.x) * amount;
        const dY = pointA.y + (pointB.y - pointA.y) * amount;
        return { x: dX, y: dY };
    }
    exports.lerp2D = lerp2D;
    class MathUtil {
        /**
         * Tests to see if @param n is between angles @param a and @param b.
         * Code as per https://www.xarg.org/2010/06/is-an-angle-between-two-other-angles/
         * TODO: Write tests
         */
        static isAngleBetween(n, a, b) {
            n = (360 + (n % 360)) % 360;
            a = (3600000 + a) % 360;
            b = (3600000 + b) % 360;
            if (a < b) {
                return a <= n && n <= b;
            }
            return a <= n || n <= b;
        }
        /**
         * Returns a point located at @param distance along a @param line.
         * Formula as per https://math.stackexchange.com/a/409721
         */
        static getPointAlongLine(line, distance) {
            const x1 = line.start.x;
            const y1 = line.start.y;
            const x2 = line.end.x;
            const y2 = line.end.x;
            const d = distance;
            const d1 = line.length;
            return new Phaser.Point((x1 + (d / d1) * (x2 - x1)), y1 + (d / d1) * (y2 - y1));
        }
    }
    exports.MathUtil = MathUtil;
});
define("logic/RaceController", ["require", "exports", "logic/comms/MessageTypes", "logic/GameController", "config/SpaceMountainConfig", "switchback-common/enum/Park", "switchback-common/util/MathUtil", "model/SpaceSaveState"], function (require, exports, MessageTypes_1, GameController_4, SpaceMountainConfig_2, Park_3, MathUtil_1, SpaceSaveState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var RaceState;
    (function (RaceState) {
        RaceState[RaceState["NOT_QUEUED"] = 0] = "NOT_QUEUED";
        RaceState[RaceState["QUEUED_NOT_STARTED"] = 1] = "QUEUED_NOT_STARTED";
        RaceState[RaceState["STARTABLE_NOT_STARTED"] = 2] = "STARTABLE_NOT_STARTED";
        RaceState[RaceState["RACE_FLOW_STARTED"] = 3] = "RACE_FLOW_STARTED";
    })(RaceState = exports.RaceState || (exports.RaceState = {}));
    var RaceType;
    (function (RaceType) {
        RaceType[RaceType["BSHOW"] = 0] = "BSHOW";
        RaceType[RaceType["BEACON"] = 1] = "BEACON";
    })(RaceType = exports.RaceType || (exports.RaceType = {}));
    class RaceDetails {
    }
    exports.RaceDetails = RaceDetails;
    /**
     * A RaceShipMovement describes the movement of a ship during
     * the race. For example, a race ship movement of
     * { t: 0.5, d: 0.2, x: 0.25, y: 0.2}
     * Describes the following:
     * t - halfway through the race
     * d - for a duration of 20% of the race time
     * x - the ship moves to 25% screen width
     * y - and 20% screen height
     */
    class RaceShipMovement {
    }
    exports.RaceShipMovement = RaceShipMovement;
    class RaceInfo {
    }
    exports.RaceInfo = RaceInfo;
    class RaceResult {
    }
    exports.RaceResult = RaceResult;
    /**
     * A RaceConfig is a list of ships participating in the races,
     * and some RacePlacing objects detailing which position those ships come in.
     *
     * PlacingThresholds are the number of parts required to complete a race at that position.
     * This is constant across all races.
     * For example, if it was [20, 10, 0] you would need
     * >= 20 parts to win 1st place
     * >= 10 for second
     * and any other number of parts is 3rd places
     */
    class RaceConfig {
    }
    exports.RaceConfig = RaceConfig;
    /**
     * A RaceShipVisualInfo class contains information about how
     * the ship should look on the device
     */
    class RaceShipVisualInfo {
    }
    exports.RaceShipVisualInfo = RaceShipVisualInfo;
    class RaceController {
        constructor() {
            this.transitionToRaceSignal = new Phaser.Signal();
            this.raceState = RaceState.NOT_QUEUED;
            this.hasReceivedInstallationRaceMessage = false;
        }
        triggerDebugBeaconRace() {
            this.onShowControlDataReceived(RaceController.SAMPLE_RACE_DATA[35]);
        }
        init(raceConfig) {
            this.raceConfig = raceConfig;
            RaceController.RACE_DURATION_MS = this.raceConfig.raceDurationMs || 60000;
            RaceController.NUM_RACES = this.raceConfig.races.length;
            console.log(this.raceConfig);
            this.isFixedIntervalRaceScheduled = false;
            this.timeOfLastRaceComplete = null;
            setInterval(this.refreshRaceStatus.bind(this), 1000);
            // Initialise and shuffle the race order
            this.backupRaceIndex = 0;
            console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE");
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE, (result) => {
                this.onShowControlInRange(result);
            }).then((r) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE success");
                console.log(JSON.stringify(r));
            }).catch((e) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE error");
                console.log(JSON.stringify(e));
            });
            console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING");
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING, (result) => {
                this.onShowControlShowRunning(result);
            }).then((r) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING success");
                console.log(JSON.stringify(r));
            }).catch((e) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING error");
                console.log(JSON.stringify(e));
            });
            console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_DATA_RECEIVED");
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_DATA_RECEIVED, (result) => {
                this.onShowControlDataReceived(result);
            }).then((r) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_DATA_RECEIVED success");
                console.log(JSON.stringify(r));
            }).catch((e) => {
                console.log("Subscribing to PlayAPI.DBeacon.events.SHOW_CONTROL_DATA_RECEIVED error");
                console.log(JSON.stringify(e));
            });
            window.t = this;
        }
        addRaceMovementData(raceId, raceMovementJSON) {
            this.raceConfig.races[raceId].movements = raceMovementJSON;
            console.log(raceMovementJSON);
        }
        /**
         * This is called when our request to transition to a race has been received.
         * We can now stop asking for the game to transition!
         *
         * We cache off whatever race was queued at the time it
         * was called. This prevents the Race flow from receiving a new
         * race message halfway through and getting out of sorts. From this
         * point on the game should reference the active race
         * getters for information about the race.
         */
        beginRaceFlow() {
            this.raceState = RaceState.RACE_FLOW_STARTED;
        }
        /**
         *
         */
        raceCompleted() {
            this.timeOfLastRaceComplete = new Date();
        }
        /**
         * Called after the race results have been processed.
         *
         * Used to reset the state.
         */
        postRaceComplete() {
            this.scheduledRace = null;
            this.raceState = RaceState.NOT_QUEUED;
        }
        /**
         * Checks the status of the current race, updating variables where necessary.
         *
         * Also responsible for queueing an on-device race if we haven't received anything from
         * the show controller yet.
         */
        refreshRaceStatus() {
            if (this.raceState === RaceState.NOT_QUEUED && this.canShipParticipateInRace()) {
                if (this.hasReceivedInstallationRaceMessage) {
                    // We have receieved a message from the show controller. We expect the next show controller race to begin
                    // at the normal scheduled race interval with up to a 1 minute error (due to the way the video's are synced)
                    // if a new race is recieved from the show controller within this time this device race will be overwritten
                    // TODO: Jay is going to pass the roofRaceScheduleInterval value via PlayAPI::game_init. 
                    // This gets it from Firebase to ensure the game is synced with any changes made to the show controller
                    this.scheduleOnDeviceRaceFromLast(this.raceConfig.roofRaceScheduleInterval);
                }
                else {
                    // We haven't received any data from the show controller. We'll use device races that are syncronized 
                    // with the clock on the device at every x minutes past the hour
                    this.scheduleOnDeviceRaceFixedInterval(this.raceConfig.deviceRaceScheduleMinutesPastHour);
                }
                return;
            }
            // If the race is in progress, we don't need to do anything here.
            if (this.raceState === RaceState.RACE_FLOW_STARTED) {
                return;
            }
            const remainingTime = this.getTimeUntilScheduledRaceMS();
            // don't start the race if it's not the right time
            // this affects both, fake and beacon races
            if (remainingTime <= 0 || remainingTime > RaceController.TRANSITION_TO_PRE_RACE_THRESHOLD_MS) {
                return;
            }
            // by this point is startable
            this.raceState = RaceState.STARTABLE_NOT_STARTED;
            // You can only be interrupted by a race in the WDW Primary queue.
            const isInInterruptibleLocation = this.inInRangeOfInstallationBeacon();
            if (isInInterruptibleLocation &&
                this.canShipParticipateInRace() && // are they eligible to compete?
                this.raceState === RaceState.STARTABLE_NOT_STARTED) { // have we already sent them there?
                console.log("[RaceController] Dispatching transition to race signal.");
                this.transitionToRaceSignal.dispatch();
            }
        }
        onShowControlInRange(message) {
            // this.hasReceivedInstallationRaceMessage = true;
            // TODO handle if necessary later (if not being handled elsewhere)
            // nothing to do atm
            // console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE:\n" + JSON.stringify(message));
        }
        onShowControlShowRunning(message) {
            // this.hasReceivedInstallationRaceMessage = true;
            // TODO handle if necessary later (if not being handled elsewhere)
            // nothing atm
            // console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING:\n" + JSON.stringify(message));
        }
        // see data diagram at https://cerebralfix.jira.com/browse/SWB-1369
        // use the script genRaceFakeData.py to generate a sample data
        // parses the message from `SHOW_CONTROL_DATA_RECEIVED` and triggers `onRaceMessageReceived` if valid
        onShowControlDataReceived(message) {
            let result = null;
            const config = SpaceMountainConfig_2.SpaceMountainConfig.Instance;
            if (!(message && message.data && message.experienceName === SpaceMountainConfig_2.SpaceMountainConfig.Instance.raceExperienceName)) {
                if (config.raceDebugBeacons) {
                    console.log("[RaceController.onShowControlDataReceived] ERROR Erroneous packet received. Discarding.\n" + JSON.stringify(message));
                }
                return null;
            }
            /* data
                examples
                    {"experienceName":"wdw-mk-spacemtn-race", "data":"ADsABQI=" }
                    {"experienceName":"wdw-mk-spacemtn-race", "data":"AAIAAS8=" }
                    {"experienceName":"wdw-mk-spacemtn-race","data":"ADsABQI="} 0 0 true 27 5 false 2
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAAAA="} 0 0 false 2 0 false 0
                    {"experienceName":"wdw-mk-spacemtn-race","data":"ADsABQI="} 0 0 true 27 5 false 2
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAAAA="} 0 0 false 2 0 false 0
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAASc="} 0 0 false 2 1 false 39
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAASg="} 0 0 false 2 1 false 40
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAATg="} 0 0 false 2 1 false 56
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAARw="} 0 0 false 2 1 false 28
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAAR0="} 0 0 false 2 1 false 29
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAAR4="} 0 0 false 2 1 false 30
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAAR8="} 0 0 false 2 1 false 31
                    {"experienceName":"wdw-mk-spacemtn-race","data":"AAIAASA="} 0 0 false 2 1 false 32
                base 64 encoded byte array
                octet	1	Data Type
                octet	1
                    bits	2	RFU
                    bits	1	Secured Flag
                    bits	5	Length
                octet	2	Interaction ID
                octet	1
                    bits	1	directed
                    bits	7	RFU
                octet	1	Sequence ID
                octet	1	Sequence Time
    
                on SHOW_CONTROL_DATA_RECEIVED , the message is stripped of its header,
                we can assume (but should not trust) that it will send only
                packets of type ArbitraryData, with no headers. that is, only the last 5 octets
    
                ATM the experience ID gets inverted
                    should be 0002 not 0200
            */
            try {
                const data = message.data;
                const xpName = message.experienceName;
                const binData = MathUtil_1.base64ToByteArray(data);
                const interID = (binData[0] << 8) | (binData[1]);
                const directed = ((binData[2] & parseInt("10000000", 2)) >> 7) !== 0;
                const RFU = binData[2] & parseInt("1111111", 2);
                const seqID = binData[3];
                const seqTime = binData[4];
                const tOffset = (config.raceBeaconWindow - seqTime) * 1000;
                const raceTime = (new Date().getTime()) + tOffset;
                let packet = {}; // packet is for debug purposes
                packet.data = data;
                packet.xpName = xpName;
                packet.binData = binData;
                packet.RFU = RFU;
                packet.interID = interID;
                packet.directed = directed;
                packet.seqID = seqID;
                packet.seqTime = seqTime;
                packet.raceTime = raceTime;
                if (config.raceDebugBeacons) {
                    console.log("[RaceController.onShowControlDataReceived] Packet:\n" + JSON.stringify(packet));
                }
                if (!seqID) { // catches null, undefined and 0 (which is invalid time/race)
                    // When idle the installation will broadcast seqID== 0 & seqTime==0
                    // if a device race past the hour is sheduled, replace with a device race time from now
                    // so that on load if we are able to play a roof race give enough time for that packet to be received
                    // this will cause the timer to the next race to increase from what time was remaining to the full duration
                    // the race will actually happen at time indeterminate time (mostly likely a lot earlier than the time shown)
                    // this will only happen once as the game is opened or when they transition into range of the beacon
                    if (seqID === 0 && (this.raceState === RaceState.NOT_QUEUED || this.isFixedIntervalRaceScheduled)) {
                        this.scheduleOnDeviceRaceFromLast(this.raceConfig.roofRaceScheduleInterval);
                    }
                    return null;
                }
                if (seqTime > config.raceBeaconWindow) {
                    // the actual race already started, so skip the message to avoid problems
                    if (config.raceDebugBeacons) {
                        console.log("[RaceController.onShowControlDataReceived] Race already started, discarding.");
                    }
                    return null;
                }
                if (config.raceFilterXPID !== null && config.raceFilterXPID !== undefined) {
                    // interID and xpID are the same, i resort to the one in the packet as it should always be there
                    if (config.raceFilterXPID !== interID) {
                        if (config.raceDebugBeacons) {
                            console.log("[RaceController.onShowControlDataReceived] Incorrect experience id, discarding. " + config.raceFilterXPID + "!=" + interID);
                        }
                        return null;
                    }
                }
                // important, seqID is 1 based on playAPI end, we use 0 based
                result = new MessageTypes_1.RaceIdMessage(seqID - 1, raceTime);
                this.onRaceMessageReceived(result);
                return result;
            }
            catch (e) {
                if (SpaceMountainConfig_2.SpaceMountainConfig.Instance.raceDebugBeacons) {
                    console.log("[RaceController.onShowControlDataReceived] Error decoding packet:\n" + JSON.stringify(e));
                }
            }
            return null;
        }
        /**
         * Handles new race messages. Doesn't accept new messages once the race
         * flow has started, and only schedules races that are in the future.
         * Notice that only WDW park enforces a race, DLR doesn't.
         * @param raceMessage RaceIdMessage Message that was received
         */
        onRaceMessageReceived(raceMessage) {
            if (raceMessage.details.raceId < 0 || raceMessage.details.raceId >= this.raceConfig.races.length) {
                console.log("[RaceController.onRaceMessageReceived] Invalid race id, got " + raceMessage.details.raceId + ", max is " + (this.raceConfig.races.length - 1));
                return;
            }
            this.hasReceivedInstallationRaceMessage = true;
            if (this.raceState !== RaceState.RACE_FLOW_STARTED &&
                raceMessage.details.raceTime > new Date().getTime()) {
                this.isFixedIntervalRaceScheduled = false;
                this.scheduleRace({
                    id: raceMessage.details.raceId,
                    time: raceMessage.details.raceTime,
                    type: RaceType.BEACON
                });
                if (SpaceMountainConfig_2.SpaceMountainConfig.Instance.raceDebugBeacons) {
                    console.log("[RaceController.onRaceMessageReceived] New race scheduled:" + JSON.stringify(this.scheduledRace));
                }
            }
        }
        canShipParticipateInRace() {
            return GameController_4.GameController.Instance.state.isSpaceRaceUnlocked || SpaceMountainConfig_2.SpaceMountainConfig.Instance.canRaceBeforeUnlock;
        }
        getShipList() {
            return this.raceConfig.ships;
        }
        getColors() {
            return this.raceConfig.colors.slice();
        }
        getRaceState() {
            return this.raceState;
        }
        getRaceMovementsByShipAndRaceId(shipId, raceId) {
            return this.raceConfig.races[raceId].movements.filter((movement) => {
                return movement.ship === shipId;
            });
        }
        /**
         * Returns the epoch timestamp of the scheduled race
         */
        getEpochTimeOfScheduledRace() {
            return this.scheduledRace ? this.scheduledRace.time : Infinity;
        }
        /**
         * Calculates the time until the next race, in MS
         */
        getTimeUntilScheduledRaceMS() {
            return this.getEpochTimeOfScheduledRace() - new Date().getTime();
        }
        /**
         * Returns the id of the upcoming race
         */
        getIdOfScheduledRace() {
            return this.scheduledRace.id;
        }
        /**
         * Can the player see the installation? This is affected
         * by a number of variables, including:
         * 1. Is the installation up and running (have we heard from it)?
         * 2. Are we in a park with an installation?
         * 3. Are we in a queue with an installation?
         *
         * Note. there's another flag `state.isAtRaceRoom` that should be taken into account
         */
        inInRangeOfInstallationBeacon() {
            // adding state.isAtRaceRoom will break this logic
            return SpaceMountainConfig_2.SpaceMountainConfig.Instance.park === Park_3.Park.WDW &&
                GameController_4.GameController.Instance.state.queueType === SpaceSaveState_1.Queue.PRIMARY &&
                this.hasReceivedInstallationRaceMessage;
        }
        isRoofRaceScheduled() {
            return this.inInRangeOfInstallationBeacon() &&
                this.scheduledRace && this.scheduledRace.type === RaceType.BEACON;
        }
        getShipIdForRace(raceId, numParts) {
            console.log(`[RaceController] getShipIdForRace(${raceId}, ${numParts})`);
            // What place are they going to get in this race?
            this.shipPlacement = this.getShipPlacingForNumParts(numParts);
            // Ensure they can't perform worse than their previous race.
            if (GameController_4.GameController.Instance.state.lastRacePlacement !== undefined
                && GameController_4.GameController.Instance.state.lastRacePlacement <= this.shipPlacement) {
                this.shipPlacement = Math.max(GameController_4.GameController.Instance.state.lastRacePlacement, 0);
            }
            // Given the race id, what ship is in that position?
            return this.raceConfig.races[raceId].placings[this.shipPlacement];
        }
        getShipVisualsForRace(raceId, numParts) {
            console.log(`[RaceController] getShipVisualsForRace(${raceId}, ${numParts})`);
            const shipId = this.getShipIdForRace(raceId, numParts);
            // What does that ship look like?
            const shipInfo = this.raceConfig.ships[shipId];
            return shipInfo;
        }
        /**
         * Finds the ship's placement in the race based on how many parts it has
         * @param numParts The number of parts the ship has
         */
        getShipPlacingForNumParts(numParts) {
            console.log(`[RaceController] getShipPlacingForNumParts(${numParts})`);
            // SWB-958 After merging the queue, they will always be first :)
            if (GameController_4.GameController.Instance.state.hasMerged) {
                return 0;
            }
            // By default, they come last.
            let tempPlacing = this.raceConfig.placingThresholds.length - 1;
            // Look through the config's placement thresholds until we find one we can surpass (or match)
            for (let i = 0; i < this.raceConfig.placingThresholds.length; i++) {
                if (numParts >= this.raceConfig.placingThresholds[i]) {
                    tempPlacing = i;
                    break;
                }
            }
            return tempPlacing;
        }
        /**
         * Given a specific @param placing, how many parts would
         * a ship require to get that placing? Placing is 1 based.
         */
        getRandomPartsForPlacing(placing) {
            console.log(`[RaceController] getRandomPartsForPlacing(${(placing)})`);
            let parts;
            if (placing > 0) {
                let minPlacing = this.raceConfig.placingThresholds[placing] + 1; // need to add one otherwise we can get a draw with the guest
                let maxPlacing = this.raceConfig.placingThresholds[placing - 1] - 1;
                parts = MathUtil_1.getRandomInt(minPlacing, maxPlacing);
            }
            else {
                parts = this.raceConfig.placingThresholds[placing];
            }
            return parts;
        }
        /**
         * Given a @param shipId and @param raceId, which position
         * is this ship going to come in?
         */
        getShipPlacingForId(shipId, raceId) {
            console.log(`[RaceController] getShipPlacingForId(${shipId}, ${raceId})`);
            return this.raceConfig.races[raceId].placings.indexOf(shipId);
        }
        getRaceResults() {
            return this.raceResults;
        }
        setRaceResult(raceResults) {
            this.raceResults = raceResults;
        }
        /**
         * Schedules a race for x minutes past the hour.
         * Also cycles through the list of race configurations we have.
         * This fake race still must occur because of DLR and WDW ADA
         * @param interval How many minutes past the hour to trigger at. '15' will occur at 15, 30, 45 and 00.
         */
        scheduleOnDeviceRaceFixedInterval(interval) {
            this.isFixedIntervalRaceScheduled = true;
            const currentTime = new Date();
            const targetTime = new Date(currentTime.getFullYear(), currentTime.getMonth(), currentTime.getDate(), currentTime.getHours(), (currentTime.getMinutes() - (currentTime.getMinutes() % interval)) + interval, 0, 0);
            this.scheduleOnDeviceRace(targetTime);
        }
        /**
         * Schedules a race for x minutes from the last race.
         * Also cycles through the list of race configurations we have.
         * This fake race still must occur because of DLR and WDW ADA
         * @param interval How many minutes from now
         */
        scheduleOnDeviceRaceFromLast(interval) {
            this.isFixedIntervalRaceScheduled = false;
            // if there was a previous race, schedule the next one from the end of the last, otherwise schedule from now
            let currentTime;
            if (this.timeOfLastRaceComplete !== null) {
                currentTime = this.timeOfLastRaceComplete;
            }
            else {
                currentTime = new Date();
            }
            const targetTime = new Date(currentTime.getFullYear(), currentTime.getMonth(), currentTime.getDate(), currentTime.getHours(), currentTime.getMinutes() + interval, currentTime.getSeconds(), currentTime.getMilliseconds());
            this.scheduleOnDeviceRace(targetTime);
        }
        scheduleOnDeviceRace(targetTime) {
            this.scheduleRace({
                id: this.backupRaceIndex,
                time: targetTime.getTime(),
                type: RaceType.BSHOW
            });
            this.backupRaceIndex++;
            // If we need to, reset and reshuffle the race order array for another run-through
            if (this.backupRaceIndex >= this.raceConfig.races.length) {
                this.backupRaceIndex = 0;
            }
        }
        scheduleRace(details) {
            this.scheduledRace = details;
            this.raceState = RaceState.QUEUED_NOT_STARTED;
        }
    }
    RaceController.NUM_SHIPS_PER_RACE = 8;
    // Updated from JSON but needs to be initialized because Loading needs this even before parsing the races config
    RaceController.NUM_RACES = 8;
    // How many seconds before the scheduled race to actually going starting the race dialogs flow
    RaceController.TRANSITION_TO_PRE_RACE_THRESHOLD_MS = 60000;
    RaceController.SAMPLE_RACE_DATA = [
        // {'data': 'AAIACgM=', "experienceName": "wdw-mk-spacemtn-race"}, // race id 10 (invalid)
        // {"experienceName": "wdw-mk-spacemtn-race", "data":"ADsABQI="} // 2 seconds exp59
        { 'data': 'AAIAAQA=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQE=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQI=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQM=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQQ=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQU=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQY=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQc=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQg=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQk=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQo=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQs=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQw=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQ0=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQ4=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAQ8=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARA=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARE=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARI=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARM=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARQ=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARU=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARY=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARc=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARg=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARk=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARo=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARs=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAARw=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAR0=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAR4=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAAR8=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASA=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASE=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASI=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASM=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASQ=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASU=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASY=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASc=', 'experienceName': "wdw-mk-spacemtn-race" },
        { 'data': 'AAIAASg=', 'experienceName': "wdw-mk-spacemtn-race" },
    ];
    exports.RaceController = RaceController;
});
define("logic/mission/MissionManager", ["require", "exports", "logic/mission/Mission"], function (require, exports, Mission_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionManager {
        constructor() {
            this.missions = [];
        }
        init(missionJSON) {
            missionJSON.forEach((data) => {
                const newMission = new Mission_1.Mission(data);
                this.missions[newMission.id] = newMission;
            }, this);
            // Retrieve any active missions, then get new missions until we have up to MAX_NUM_SIMULTANEOUS_MISSIONS
            const activeMissions = this.getActiveMissions();
            this.getNewMissions(Math.min(0, MissionManager.MAX_NUM_SIMULTANEOUS_MISSIONS - activeMissions.length));
        }
        getMissionById(id) {
            return this.missions[id];
        }
        getNumMissionsCompleted() {
            return this.missions.filter((mission) => {
                return mission.hasBeenCompleted;
            }).length;
        }
        hasPendingMissionProgressToAward() {
            return this.getActiveMissions().find((mission) => {
                return mission.pendingProgress > 0;
            }, this) !== undefined;
        }
        getActiveMissions() {
            return this.missions.filter((mission) => {
                return mission.isActive;
            }, this);
        }
        getNewMissions(numberToGet) {
            let eligibleMissions = this.missions.filter((mission) => {
                return (!mission.hasBeenCompleted && this.hasMetPrerequisites(mission.id) && !mission.isActive);
            }, this);
            eligibleMissions = eligibleMissions.slice(0, MissionManager.MAX_NUM_SIMULTANEOUS_MISSIONS);
            eligibleMissions.forEach((mission) => {
                mission.isActive = true;
            }, this);
            return eligibleMissions;
        }
        hasMetPrerequisites(id) {
            const mission = this.missions[id];
            return mission.prerequisites.every((id) => {
                return this.missions[id].hasBeenCompleted;
            }, this);
        }
        addMissionPendingProgress(type, amount) {
            this.getActiveMissions().forEach((mission) => {
                if (mission.type === type) {
                    mission.pendingProgress += amount;
                }
            });
        }
    }
    MissionManager.MAX_NUM_SIMULTANEOUS_MISSIONS = 2;
    exports.MissionManager = MissionManager;
});
define("logic/SMAnalyticsManager", ["require", "exports", "switchback-common/play/AnalyticsManager", "switchback-common/enum/Park", "logic/GameController", "StateNames"], function (require, exports, AnalyticsManager_1, Park_4, GameController_5, StateNames_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SMAnalyticsManager extends AnalyticsManager_1.AnalyticsManager {
        constructor() {
            super(...arguments);
            this.gaNameMap = new Map([
                [SMAnalyticsManager.GAME_NAME_MINI, 'dimension7'],
                [SMAnalyticsManager.GAME_NAME_MICRO, 'dimension8'],
                [SMAnalyticsManager.GAME_SCORE, 'dimension9'],
                [SMAnalyticsManager.GAME_PLAYS, 'dimension10'],
                [SMAnalyticsManager.GAME_ALLPLAYS, 'dimension11'],
                [SMAnalyticsManager.GAME_PROGRESS, 'dimension12'],
                [SMAnalyticsManager.ITEM_ID, 'dimension13'],
                [SMAnalyticsManager.ITEM_COMPLETED, 'dimension14'],
                [SMAnalyticsManager.CUTSCENE, 'dimension15'],
                [SMAnalyticsManager.INSTALLATION_TRIGGER, 'dimension16'],
                [SMAnalyticsManager.INSTALLATION_NAME, 'dimension17'],
            ]);
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init(park, parkString, hasPrimaryPlayer, partySize, partyVisits) {
            let landString = park === Park_4.Park.WDW ? AnalyticsManager_1.AnalyticsManager.LAND_MK : AnalyticsManager_1.AnalyticsManager.LAND_DL;
            super.initialise("InQueue", landString, "Space Mountain", "spaceMountain", "Rocket Race", parkString, AnalyticsManager_1.AnalyticsManager.QUEUE_NONE, hasPrimaryPlayer, partySize, partyVisits);
        }
        trackState(adobeStatePath, gaStateId, data = {}) {
            super.trackState(adobeStatePath, gaStateId, Object.assign({}, data, { [SMAnalyticsManager.GAME_ALLPLAYS]: GameController_5.GameController.Instance.state.numGamePlays, [SMAnalyticsManager.GAME_PROGRESS]: `${GameController_5.GameController.Instance.getTotalShipParts()}_70`, [SMAnalyticsManager.ITEM_COMPLETED]: GameController_5.GameController.Instance.state.redeemedCodes.length }));
        }
        startTiming(adobeTimingKey, data = {}) {
            super.startTiming(adobeTimingKey, Object.assign({}, data, { [SMAnalyticsManager.GAME_ALLPLAYS]: GameController_5.GameController.Instance.state.numGamePlays, [SMAnalyticsManager.GAME_PROGRESS]: `${GameController_5.GameController.Instance.getTotalShipParts()}_70`, [SMAnalyticsManager.ITEM_COMPLETED]: GameController_5.GameController.Instance.state.redeemedCodes.length }));
        }
        endTiming(adobeTimingKey, gaTimingKey, gaCategory, gaLabel, data = {}) {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("endTiming").call(this, adobeTimingKey, gaTimingKey, gaCategory, gaLabel, Object.assign({}, data, { [SMAnalyticsManager.GAME_ALLPLAYS]: GameController_5.GameController.Instance.state.numGamePlays, [SMAnalyticsManager.GAME_PROGRESS]: `${GameController_5.GameController.Instance.getTotalShipParts()}_70`, [SMAnalyticsManager.ITEM_COMPLETED]: GameController_5.GameController.Instance.state.redeemedCodes.length }));
            });
        }
        trackAction(adobeAction, gaCategory, gaAction, gaLabel = '', data = {}) {
            super.trackAction(adobeAction, gaCategory, gaAction, gaLabel, Object.assign({}, data, { [SMAnalyticsManager.GAME_ALLPLAYS]: GameController_5.GameController.Instance.state.numGamePlays, [SMAnalyticsManager.GAME_PROGRESS]: `${GameController_5.GameController.Instance.getTotalShipParts()}_70`, [SMAnalyticsManager.ITEM_COMPLETED]: GameController_5.GameController.Instance.state.redeemedCodes.length }));
        }
        trackPassAndPlay(gameName) {
            const metricsGameName = SMAnalyticsManager.gameNameMap.get(gameName);
            this.trackState('content/game/inqueue/minigame/passandplay', `${metricsGameName}PassAndPlay`, {
                [SMAnalyticsManager.GAME_NAME_MINI]: metricsGameName,
                [SMAnalyticsManager.GAME_PLAYS]: GameController_5.GameController.Instance.state.getMinigamePlays(gameName) + 1
            });
        }
        trackInstructions(gameName) {
            const metricsGameName = SMAnalyticsManager.gameNameMap.get(gameName);
            this.trackState('content/game/inqueue/minigame/instructions', `${metricsGameName}Instructions`, {
                [SMAnalyticsManager.GAME_NAME_MINI]: metricsGameName,
                [SMAnalyticsManager.GAME_PLAYS]: GameController_5.GameController.Instance.state.getMinigamePlays(gameName) + 1
            });
        }
        trackGamePlay(gameName) {
            const metricsGameName = SMAnalyticsManager.gameNameMap.get(gameName);
            this.trackState('content/game/inqueue/minigame', `${metricsGameName}Game`, {
                [SMAnalyticsManager.GAME_NAME_MINI]: metricsGameName,
                [SMAnalyticsManager.GAME_PLAYS]: GameController_5.GameController.Instance.state.getMinigamePlays(gameName) + 1
            });
        }
        trackScore(gameName, score) {
            const metricsGameName = SMAnalyticsManager.gameNameMap.get(gameName);
            this.trackState('content/game/inqueue/minigame/score', `${metricsGameName}Score`, {
                [SMAnalyticsManager.GAME_NAME_MINI]: metricsGameName,
                [SMAnalyticsManager.GAME_PLAYS]: GameController_5.GameController.Instance.state.getMinigamePlays(gameName),
                [SMAnalyticsManager.GAME_SCORE]: score
            });
        }
    }
    SMAnalyticsManager.GAME_NAME_MINI = "game.name.mini";
    SMAnalyticsManager.GAME_NAME_MICRO = "game.name.micro";
    SMAnalyticsManager.GAME_SCORE = "game.score";
    SMAnalyticsManager.GAME_PLAYS = "game.plays";
    SMAnalyticsManager.GAME_ALLPLAYS = "game.allplays";
    SMAnalyticsManager.GAME_PROGRESS = "game.progress";
    SMAnalyticsManager.ITEM_ID = "item.id";
    SMAnalyticsManager.ITEM_COMPLETED = "item.completed";
    SMAnalyticsManager.CUTSCENE = "cutscene";
    SMAnalyticsManager.INSTALLATION_TRIGGER = "installation.trigger";
    SMAnalyticsManager.INSTALLATION_NAME = "installation.name";
    SMAnalyticsManager.gameNameMap = new Map([
        [StateNames_1.StateNames.GAME_ASTEROIDS, 'Pilot'],
        [StateNames_1.StateNames.GAME_DOCKING, 'Specialist'],
        [StateNames_1.StateNames.GAME_ENERGY_BALLS, 'Engineer'],
        [StateNames_1.StateNames.GAME_MEMORY, 'Commander'],
        [StateNames_1.StateNames.GAME_PIPES, 'Navigator'],
        [StateNames_1.StateNames.GAME_RELAY_INTRO, 'Maintenance'],
        [StateNames_1.StateNames.GAME_RELAY_LOCKS, 'Airlock'],
        [StateNames_1.StateNames.GAME_RELAY_OUTRO, 'Maintenance'],
        [StateNames_1.StateNames.GAME_RELAY_SCANNER, 'Scanner'],
        [StateNames_1.StateNames.GAME_RELAY_SWITCHES, 'Switches'],
        [StateNames_1.StateNames.GAME_RELAY_WRENCH, 'Wrench'],
        [StateNames_1.StateNames.DEVICE_RACE, 'SpaceRace']
    ]);
    exports.SMAnalyticsManager = SMAnalyticsManager;
});
define("switchback-common/util/TextUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextUtil {
        /**
         * Calculates the font size modifier used when getting the fontsize based on
         * the devices aspect ratio. This is used in getFontSize(). For wider devices the
         * fontSizeModifier is 600 and for narrow devices is uses 400. These values were
         * choosen because they make the text look nice.
         * @param gameWidth Canvas width
         * @param gameHeight Canvas height
         */
        static calculateFontSizeModifier(gameWidth, gameHeight) {
            TextUtil.fontSizeModifier = 400;
            let aspectRatio = (gameHeight / gameWidth);
            // Finding the aspect ratio of the biggest edge to the smallest edge. Using
            // height / width as the base check because most games should be run in portrait
            if (aspectRatio < 1) {
                // Invert aspect ratio if less than 1.
                aspectRatio = 1 / aspectRatio;
            }
            if (aspectRatio <= 1.5) {
                TextUtil.fontSizeModifier = 600;
            }
        }
        static leftPad(text, padCharacter, width) {
            return width - text.length > 0 ? padCharacter.repeat(width - text.length) + text : text;
        }
        static rightPad(text, padCharacter, width) {
            return width - text.length > 0 ? text + padCharacter.repeat(width - text.length) : text;
        }
        // Returns a font size value that is usable by Phaser at the scale of the desktop font.
        static getFontSize(gameWidth, fontPtSize) {
            return (fontPtSize / TextUtil.fontSizeModifier) * gameWidth;
        }
        static formattedTime(seconds) {
            const min = Math.max(0, Math.floor(seconds / 60)).toString();
            const sec = (seconds % 60).toFixed(0);
            return TextUtil.leftPad(min, "0", 2) + ":" + TextUtil.leftPad(sec, "0", 2);
        }
        static getOrdinalFormattedString(n) {
            const s = ["th", "st", "nd", "rd"];
            const v = n % 100;
            return n + (s[(v - 20) % 10] || s[v] || s[0]);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the width.
         * This is used to ensure that single line textfields don't overflow.
         *
         * Will not work if word-wrap is turned on (as word wrap ensures it never overflows width)
         */
        static scaleDownSingleLineTextToWidth(textField, width) {
            const condition = (textField, sizeRestriction) => {
                return textField.width > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, width, condition);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the height.
         */
        static scaleDownTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, height, condition);
        }
        static scaleUpTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height < sizeRestriction;
            };
            TextUtil.scaleText(textField, height, condition, true);
        }
        /**
         * Passing in an array of @interface IFontSize @param textObjects to this function
         * will attempt to equalize their font sizes to either the largest,
         * or smallest, depending on @param equalizeOnSmallest.
         */
        static equalizeFontSizes(textObjects, equalizeOnSmallest) {
            let size = equalizeOnSmallest ? Infinity : 0;
            textObjects.forEach((obj) => {
                size = equalizeOnSmallest ? Math.min(size, obj.fontSize) : Math.max(size, obj.fontSize);
            });
            textObjects.forEach((obj) => {
                obj.fontSize = size;
            });
        }
        static scaleText(textField, sizeRestriction, condition, increment = false) {
            const alignment = textField.align;
            let fontSize = textField.fontSize;
            while (condition(textField, sizeRestriction)) {
                textField.fontSize = increment ? fontSize++ : fontSize--;
            }
            // this is required as the text doesn't repaint automatically (phaser quirk)
            textField.align = alignment;
        }
    }
    TextUtil.fontSizeModifier = 400;
    exports.TextUtil = TextUtil;
});
define("switchback-common/util/MotionUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MotionUtil {
        static checkGyroAvailability() {
            return __awaiter(this, void 0, void 0, function* () {
                MotionUtil.isAccelerometerAvailable = false;
                MotionUtil.isGyroAvailable = false;
                const motionCallback = (result) => {
                    const acceleration = result.userAcceleration;
                    if (acceleration.x !== 0 || acceleration.y !== 0 || acceleration.z !== 0) {
                        MotionUtil.isAccelerometerAvailable = true;
                    }
                    const roll = result.attitude.roll;
                    if (roll !== 0) {
                        MotionUtil.isGyroAvailable = true;
                    }
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
                yield PromiseUtil_2.PromiseUtil.realTime(1000);
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
            });
        }
    }
    MotionUtil.isAccelerometerAvailable = false;
    MotionUtil.isGyroAvailable = false;
    exports.MotionUtil = MotionUtil;
});
define("logic/GameController", ["require", "exports", "switchback-common/play/api/MockPlayAPI", "switchback-common/util/CollectionUtil", "switchback-common/enum/Park", "switchback-common/play/AnalyticsManager", "config/SpaceMountainConfig", "model/SpaceSaveState", "StateNames", "logic/SMAchievementManager", "logic/RaceController", "logic/mission/MissionManager", "logic/mission/MissionType", "logic/SMAnalyticsManager", "switchback-common/util/TextUtil", "Game", "model/SpaceSaveState", "switchback-common/util/MotionUtil"], function (require, exports, MockPlayAPI_1, CollectionUtil_1, Park_5, AnalyticsManager_2, SpaceMountainConfig_3, SpaceSaveState_2, StateNames_2, SMAchievementManager_1, RaceController_1, MissionManager_1, MissionType_2, SMAnalyticsManager_1, TextUtil_1, Game_1, SpaceSaveState_3, MotionUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SMPlayer {
        constructor() {
            this.relayPossibleGames = [];
        }
    }
    exports.SMPlayer = SMPlayer;
    class GameController {
        constructor() {
            this.isInstallationFirstWayfindDialogConfirmed = false;
            this.isInRangeOfInstallationPlayerConfirmed = false;
            // Flag for queuing the exit beacon
            this._isExitBeaconTriggered = false;
            this.lastCodeReminderGamePlayCount = 0;
            this.games = [
                StateNames_2.StateNames.GAME_RELAY_INTRO,
                StateNames_2.StateNames.GAME_ASTEROIDS,
                StateNames_2.StateNames.GAME_ENERGY_BALLS,
                StateNames_2.StateNames.GAME_DOCKING,
                StateNames_2.StateNames.GAME_PIPES,
                StateNames_2.StateNames.GAME_MEMORY
            ];
            if (!SpaceMountainConfig_3.SpaceMountainConfig.Instance.usePlayAPI) {
                MockPlayAPI_1.MockPlayAPI.init();
            }
            this.players = [];
            this.previousPlayers = [];
            this.achievementManager = new SMAchievementManager_1.SMAchievementManager();
            this.raceController = new RaceController_1.RaceController();
            this.missionManager = new MissionManager_1.MissionManager();
            this._exitBeaconTriggeredSignal = new Phaser.Signal();
            this.timeOfLastExitReminder = new Date().getTime();
            MotionUtil_1.MotionUtil.checkGyroAvailability();
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[GameController] init()`);
                this.state = new SpaceSaveState_3.SpaceSaveState();
                try {
                    yield PlayAPI.gameInit().then((initResult) => {
                        console.log(JSON.stringify(initResult));
                        if (initResult && initResult.gameStateRead && initResult.gameStateRead.state) {
                            this.state.parse(initResult.gameStateRead.state);
                        }
                        else {
                            this.state.parse({});
                        }
                        // Figure out which park this app got launched from!
                        // Longer form, but safer than 'SpaceMountainConfig.Instance.park = initResult.playerLocation'
                        if (initResult.playerLocation === Park_5.Park.DLR) {
                            SpaceMountainConfig_3.SpaceMountainConfig.Instance.park = Park_5.Park.DLR;
                        }
                        else if (initResult.playerLocation === Park_5.Park.WDW) {
                            SpaceMountainConfig_3.SpaceMountainConfig.Instance.park = Park_5.Park.WDW;
                        }
                    }, (reason) => {
                        console.log(JSON.stringify(reason));
                        this.state.parse({});
                    });
                }
                catch (error) {
                    console.log(error);
                    this.state.parse({});
                }
                this.validateState();
                yield GameController.Instance.fetchNewPlayers();
                if (SpaceMountainConfig_3.SpaceMountainConfig.Instance.park === Park_5.Park.DLR) {
                    this.parkPrefix = 'dlr';
                }
                else if (SpaceMountainConfig_3.SpaceMountainConfig.Instance.park === Park_5.Park.WDW) {
                    this.parkPrefix = 'wdw';
                }
                else {
                    // make sure the value is valid
                    SpaceMountainConfig_3.SpaceMountainConfig.Instance.park = Park_5.Park.DLR;
                    this.parkPrefix = 'dlr';
                }
                SMAnalyticsManager_1.SMAnalyticsManager.Instance.init(SpaceMountainConfig_3.SpaceMountainConfig.Instance.park, this.parkPrefix.toUpperCase(), this.hasPrimaryPlayer(), this.players.length, this.state.numVisits);
                this.getEstimatedWaitTime().then((waitTime) => {
                    SMAnalyticsManager_1.SMAnalyticsManager.Instance.estimatedWaitTime = waitTime;
                });
                this.setAnalyticsQueueType(GameController.Instance.state.queueType);
                this._exitBeaconCallbackFunction = () => {
                    this.onExitBeaconTriggered();
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_EJECT, this._exitBeaconCallbackFunction);
                // Every second, increase our counter for time spent playing.
                setInterval(() => {
                    this.state.sessionPlayTime++;
                    SMAnalyticsManager_1.SMAnalyticsManager.Instance.actualWaitTime = this.state.sessionPlayTime;
                }, 1000);
            });
        }
        setAnalyticsQueueType(queueType) {
            // Try to pull the queue out here
            switch (queueType) {
                case SpaceSaveState_2.Queue.PRIMARY: {
                    SMAnalyticsManager_1.SMAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_A;
                    break;
                }
                case SpaceSaveState_2.Queue.SECONDARY: {
                    SMAnalyticsManager_1.SMAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_B;
                    break;
                }
                default: {
                    SMAnalyticsManager_1.SMAnalyticsManager.Instance.lineType = AnalyticsManager_2.AnalyticsManager.QUEUE_NONE;
                }
            }
        }
        getEstimatedWaitTime() {
            return __awaiter(this, void 0, void 0, function* () {
                const location = yield PlayAPI.locationRead();
                const wait = yield PlayAPI.locationWaitTimeRead(location);
                return wait.waitTime;
            });
        }
        hasPrimaryPlayer() {
            for (let player of this.players) {
                if (player.type === "TYPE_PRIMARY") {
                    return true;
                }
            }
            return false;
        }
        getGameCount() {
            return this.games.length;
        }
        getNextGame() {
            if (!this.state.gameOrder || this.state.gameOrder.length === 0) {
                this.state.gameOrder = this.games;
            }
            if (this.state.progressThroughGameOrder >= this.state.gameOrder.length) {
                this.state.gameOrder = CollectionUtil_1.shuffleNonRepeatingArray(this.state.gameOrder);
                this.state.progressThroughGameOrder = 0;
                this.state.isInForcedProgression = false;
            }
            return this.state.gameOrder[this.state.progressThroughGameOrder];
        }
        hasActiveSession() {
            return this.state.sessionStarted;
        }
        get codeReminderTimer() {
            return this.codeReminderTimerMs;
        }
        set codeReminderTimer(n) {
            this.codeReminderTimerMs = n;
        }
        get gamePlayCountAtLastCodeReminder() {
            return this.lastCodeReminderGamePlayCount;
        }
        set gamePlayCountAtLastCodeReminder(n) {
            this.lastCodeReminderGamePlayCount = n;
        }
        startNewGame() {
            return __awaiter(this, void 0, void 0, function* () {
                const recycledData = {};
                recycledData.players = this.state.players.slice();
                recycledData.numVisits = this.state.numVisits;
                this.state.parse(recycledData);
                this.state.sessionStarted = true;
                this.state.save();
            });
        }
        isPlayingWithNewParty() {
            let isNew = false;
            if (this.players.length !== this.previousPlayers.length) {
                isNew = true;
            }
            else {
                for (let i = 0; i < this.players.length && !isNew; i++) {
                    isNew = this.players[i].name !== this.previousPlayers[i].name;
                }
            }
            return isNew;
        }
        fetchNewPlayers() {
            return __awaiter(this, void 0, void 0, function* () {
                this.previousPlayers = this.state.players;
                // Get new player list from PlayAPI
                let result = null;
                try {
                    result = yield PlayAPI.openPlayerManagement();
                }
                catch (error) {
                    console.log("OpenPlayerManagement Error: " + error);
                }
                let resultPlayers = [];
                // Get data from playAPI or use mock data
                if (result && result.players && result.players.length > 0) {
                    this.state.players = result.players;
                    resultPlayers = result.players;
                }
                else {
                    resultPlayers = [
                        { name: "Player 1", type: "", identifier: "" },
                        { name: "Player 2", type: "", identifier: "" }
                    ];
                }
                // Add custom fields
                this.players = resultPlayers.map(player => {
                    return Object.assign({}, player, { relayPossibleGames: [] });
                });
            });
        }
        /*
        Player grouping
        Returns a session, which is a list of groups. A group is a list of players that plays in a round.
        The first player in a group is important because is the captain.
        Everybody will play at least twice.
        A, B, C, D, etc are randomized, but not the rest of the pattern.
        That means that the initial player list is randomized but not the session pattern, which makes it
        more fair, predictable and AC compliant (cooler?)
        @return list of groups. group=list of players : [ [A, B], [B, C], [C, A]]
        @param GroupSize number = 1 : how many players per group. =1 individual game, =2 pair game, =more for bigger groups
            If groupSize is bigger than the number of available players this will return []
        @param rounds number = null : hom many rounds are needed, =null as many as possible without repeating,
            if there aren't enough players it will start repeating
        @param fixedPairs boolean = false : If true, for pair games, avoids making the players switch sides
            after sequential rounds.
        */
        getPlayerSession(groupSize = 1, rounds = null, fixedPairs = false) {
            let result = [];
            // if no rounds is specified, use as many rounds as necessary to make everybody play AS CAPTAIN (if applicable, or once)
            if (rounds == null) {
                rounds = this.players.length;
            }
            if (groupSize > this.players.length) {
                console.log("[GameController] Error, Group Size is bigger than player."); // Not removing in case this return introduces bugs when not taken into account
                return [];
            }
            // set the pool of players
            let pool = [];
            for (let i in this.players) {
                pool.push(Number(i));
            }
            // pool = this.players.map((_, i) => i); // could use this but reduces the readability
            // let pool: number[] = this.players.keys().map(i => Number(i)); // this one too but same with readability
            // Make sure the group is different per game (AC)
            Phaser.ArrayUtils.shuffle(pool);
            for (let i = 0; i < rounds; i++) {
                let group = pool.slice(0, groupSize);
                // In case the game is in pairs and we want static positions, revert every other group 
                if (fixedPairs && (groupSize === 2) && (i % 2 === 0)) {
                    group.reverse();
                }
                pool.push(pool.splice(0, 1)[0]);
                result.push(group);
            }
            console.log("[GameController] Players session is ", result);
            return result;
            /* TODO after implementing other games,
            consider using an enum instead of groupSize & fixed pairs, so call is more clear
                enum GameType {
                    INDIVIDUAL,
                    PAIR_ALTERNATING,
                    PAIR_ROTATING,
                    GROUP
                }
            */
        }
        isTimeBasedGame(gameName) {
            return GameController.TIMED_GAMES.indexOf(gameName) > -1;
        }
        /**
         *
         * @param stateName The state they scored in
         * @param rawScore The score they achieved
         * @param wasInterrupted Did they complete this game normally, or were they interrupted?
         */
        postMinigameScore(stateName, rawScore, wasInterrupted = false) {
            this.state.numGamePlays++;
            // races don't impact the game order
            if (stateName !== StateNames_2.StateNames.DEVICE_RACE) {
                this.state.progressThroughGameOrder++;
            }
            const numGamePlays = this.state.incrementMinigamePlays(stateName);
            const metricGameName = SMAnalyticsManager_1.SMAnalyticsManager.gameNameMap.get(stateName);
            // Adds an exception for Relay, as it already 
            // tracks this metric on RelayOutroState.
            if (stateName !== StateNames_2.StateNames.GAME_RELAY_INTRO) {
                SMAnalyticsManager_1.SMAnalyticsManager.Instance.trackScore(stateName, rawScore);
            }
            SMAnalyticsManager_1.SMAnalyticsManager.Instance.endTiming(`RocketRace:Mini:${metricGameName}`, metricGameName, 'Game', '', {
                [SMAnalyticsManager_1.SMAnalyticsManager.GAME_NAME_MINI]: metricGameName,
                [SMAnalyticsManager_1.SMAnalyticsManager.GAME_PLAYS]: numGamePlays
            });
            let mgScore = this.state.getMinigameScore(stateName);
            let isTime = GameController.Instance.isTimeBasedGame(stateName);
            if (SpaceMountainConfig_3.SpaceMountainConfig.Instance.skipGames) {
                // apply default scores if the games are being skipped
                rawScore = (isTime ? 75 : 10);
            }
            mgScore.currentScore = rawScore;
            let isBetter;
            if (mgScore.highScore === 0) {
                isBetter = (rawScore !== 0);
            }
            else if (isTime) {
                isBetter = rawScore < mgScore.highScore;
            }
            else {
                isBetter = rawScore > mgScore.highScore;
            }
            mgScore.wasHighScoreInLastPlay = isBetter;
            if (isBetter) {
                mgScore.highScore = rawScore;
            }
            // Space Race has its own logic for earning ship parts
            if (stateName !== StateNames_2.StateNames.DEVICE_RACE) {
                let parts = this.getNumberPartsEarnedForLastPlayOf(stateName);
                this.addPendingShipParts(parts);
                // space race isn't concidered a game and doesn't increment mission
                this.missionManager.addMissionPendingProgress(MissionType_2.MissionType.GAME_PLAYED, 1);
            }
        }
        getNumberPartsEarnedForLastPlayOf(gameState) {
            const mgScore = this.state.getMinigameScore(gameState);
            let numParts;
            // SWB-1069 Forced progression: 2 per mini-game (regardless of performance)
            // SWB-1069 Subsequently: 1 per mini-game, +1 if they beat their previous highscore/best time
            if (GameController.Instance.state.isInForcedProgression || mgScore.wasHighScoreInLastPlay) {
                numParts = 2;
            }
            else {
                numParts = 1;
            }
            return numParts;
        }
        getTotalShipParts() {
            return this.state.shipBaseParts
                + this.state.shipPendingBaseParts
                + this.state.shipBonusParts
                + this.state.shipPendingBonusParts;
        }
        getShipLevelText() {
            // The ship's level is how many ship parts have been redeemed, + 1
            // (because we don't want to start on level 0)
            return (this.state.shipBaseParts
                + this.state.shipBonusParts
                + 1).toString();
        }
        getShipState() {
            return {
                baseParts: this.state.shipBaseParts,
                pendingBaseParts: this.state.shipPendingBaseParts,
                bonusParts: this.state.shipBonusParts,
                pendingBonusParts: this.state.shipPendingBonusParts
            };
        }
        /**
         * Called when a game is started.
         *
         * Used to set up variables for the NarrativeStates to reference.
         * @param gameName
         */
        startGame(gameName) {
            GameController.Instance.isInGame = true;
            GameController.Instance.activeGameState = gameName;
            GameController.Instance.isNarrativeIntro = true;
        }
        /**
         * Called when we've finished playing a game (N.B. post-game, but pre-narrative!)
         * If the game didn't finish normally (i.e. was interrupted) then
         * this function call is skipped. If this function call has been hit,
         * (and no new startGame() function has been called since) then any
         * interruptions aren't really interruptions - players aren't doing
         * anything important.
         * @param gameName
         */
        finishGame() {
            GameController.Instance.isInGame = false;
            GameController.Instance.isNarrativeIntro = false;
        }
        get hasShipPartsToAward() {
            return this.numShipPartsToAward > 0;
        }
        get numShipPartsToAward() {
            const baseParts = Math.min(this.state.shipPendingBaseParts, GameController.MAX_NUM_SHIP_PARTS - this.state.shipBaseParts);
            const bonusParts = Math.min(this.state.shipPendingBonusParts, GameController.MAX_NUM_BONUS_PARTS - this.state.shipBonusParts);
            return baseParts + bonusParts;
        }
        addPendingShipParts(num) {
            // don't allow the number of ship parts to exceed the maximum
            const currentPartCount = this.state.shipBaseParts + this.state.shipPendingBaseParts;
            if (currentPartCount + num > GameController.MAX_NUM_SHIP_PARTS) {
                num = GameController.MAX_NUM_SHIP_PARTS - currentPartCount;
            }
            if (num > 0) {
                this.state.shipPendingBaseParts += num;
                this.missionManager.addMissionPendingProgress(MissionType_2.MissionType.PARTS_GATHERED, num);
            }
        }
        redeemSinglePendingShipPart() {
            this.state.shipBaseParts++;
            this.state.shipPendingBaseParts--;
            // See if we've redeemed enough parts for an achievement
            GameController.Instance.achievementManager.onShipPartAwarded(this.state.shipBaseParts + this.state.shipBonusParts);
        }
        addPendingBonusShipParts(num) {
            // don't allow the number of ship parts to exceed the maximum
            const currentBonusPartCount = this.state.shipBonusParts + this.state.shipPendingBonusParts;
            if (currentBonusPartCount + num > GameController.MAX_NUM_BONUS_PARTS) {
                num = GameController.MAX_NUM_BONUS_PARTS - currentBonusPartCount;
            }
            if (num > 0) {
                this.state.shipPendingBonusParts += num;
                this.missionManager.addMissionPendingProgress(MissionType_2.MissionType.PARTS_GATHERED, num);
            }
        }
        redeemSinglePendingBonusShipPart() {
            this.state.shipBonusParts++;
            this.state.shipPendingBonusParts--;
            // See if we've redeemed enough parts for an achievement
            GameController.Instance.achievementManager.onShipPartAwarded(this.state.shipBaseParts + this.state.shipBonusParts);
        }
        hasCompletedMinigame(gameName) {
            return this.state.minigameScores.find(obj => obj.gameName === gameName) != null;
        }
        shouldShowExitReminder() {
            if (this._isExitBeaconTriggered) {
                const now = new Date().getTime();
                if (now > this.timeOfLastExitReminder + GameController.TIME_BETWEEN_EXIT_REMINDERS_MS) {
                    this.timeOfLastExitReminder = now;
                    return true;
                }
            }
            return false;
        }
        set isExitBeaconTriggered(value) {
            if (!this._isExitBeaconTriggered) {
                this._isExitBeaconTriggered = value;
                this.exitBeaconTriggeredSignal.dispatch();
            }
        }
        get exitBeaconTriggeredSignal() {
            return this._exitBeaconTriggeredSignal;
        }
        onExitBeaconTriggered() {
            GameController.Instance.isExitBeaconTriggered = true;
            // Can unsubscribe to the exit beacon event once it's been triggered.
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.GAME_EJECT, this._exitBeaconCallbackFunction);
        }
        /**
         * Calculates the default text sizes and sets up the modifier for the TextUtils
         * @param gameWidth Game width
         * @param gameHeight Game Height
         */
        calculateDefaultTextSizes(gameWidth, gameHeight) {
            TextUtil_1.TextUtil.calculateFontSizeModifier(gameWidth, gameHeight);
            Game_1.Game.FONT_SIZE_EXTRA_LARGE = TextUtil_1.TextUtil.getFontSize(gameWidth, 45);
            Game_1.Game.FONT_SIZE_LARGE = TextUtil_1.TextUtil.getFontSize(gameWidth, 35);
            Game_1.Game.FONT_SIZE_MEDIUM = TextUtil_1.TextUtil.getFontSize(gameWidth, 25);
            Game_1.Game.FONT_SIZE_SMALL = TextUtil_1.TextUtil.getFontSize(gameWidth, 15);
            Game_1.Game.DEFAULT_FONT_STYLE = {
                font: Game_1.Game.DEFAULT_FONT,
                fill: Game_1.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_1.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_1.Game.FONT_STROKE_LARGE
            };
            Game_1.Game.FONT_STYLE_TITLE = {
                font: Game_1.Game.DEFAULT_FONT,
                fill: Game_1.Game.FONT_COLOUR_BLUE,
                stroke: Game_1.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_1.Game.FONT_STROKE_SMALL,
                fontSize: Game_1.Game.FONT_SIZE_LARGE,
            };
        }
        /**
         * This is required to fix the save state for guests playing on versions prior to #134 rev: aebe93f3
         * This fixed an issue where the ship parts could exceed their maximum and cause the ships to display
         * incorrectly.
         */
        validateState() {
            this.state.shipBaseParts = Math.min(this.state.shipBaseParts, GameController.MAX_NUM_SHIP_PARTS);
            this.state.shipBonusParts = Math.min(this.state.shipBonusParts, GameController.MAX_NUM_BONUS_PARTS);
            const totalParts = this.state.shipBaseParts + this.state.shipPendingBaseParts;
            if (totalParts > GameController.MAX_NUM_SHIP_PARTS) {
                this.state.shipPendingBaseParts = Math.max(0, totalParts - GameController.MAX_NUM_SHIP_PARTS - 1);
            }
            const totalBonusParts = this.state.shipBonusParts + this.state.shipPendingBonusParts;
            if (totalBonusParts > GameController.MAX_NUM_BONUS_PARTS) {
                this.state.shipPendingBonusParts = Math.max(0, totalBonusParts - GameController.MAX_NUM_BONUS_PARTS - 1);
            }
        }
    }
    GameController.MAX_NUM_SHIP_PARTS = 49;
    GameController.MAX_NUM_BONUS_PARTS = 20;
    GameController.TIMED_GAMES = [StateNames_2.StateNames.GAME_DOCKING, StateNames_2.StateNames.GAME_ENERGY_BALLS, StateNames_2.StateNames.GAME_PIPES];
    GameController.TIME_BETWEEN_EXIT_REMINDERS_MS = 8 * 60 * 1000; // 8 minutes
    exports.GameController = GameController;
});
define("model/PlanetStruct", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var SpecialPlanetEnum;
    (function (SpecialPlanetEnum) {
        SpecialPlanetEnum[SpecialPlanetEnum["NONE"] = 0] = "NONE";
        SpecialPlanetEnum[SpecialPlanetEnum["SPACE_MOUNTAIN"] = 1] = "SPACE_MOUNTAIN";
        SpecialPlanetEnum[SpecialPlanetEnum["HYPERION"] = 2] = "HYPERION";
        SpecialPlanetEnum[SpecialPlanetEnum["STATION"] = 3] = "STATION";
    })(SpecialPlanetEnum = exports.SpecialPlanetEnum || (exports.SpecialPlanetEnum = {}));
    class PlanetStruct {
        parse(data) {
            if (data.id === undefined) {
                console.error('Planet found with no id');
                return;
            }
            this.id = data.id;
            this.name = data.name || "";
            this.tint = data.tint || 0;
            this.texture = data.texture || 0;
            this.feature = data.feature || -1;
            this.specialId = data.specialId || SpecialPlanetEnum.NONE;
            return this;
        }
    }
    exports.PlanetStruct = PlanetStruct;
});
define("logic/MapController", ["require", "exports", "model/PlanetStruct"], function (require, exports, PlanetStruct_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MapController {
        constructor() {
            this.planets = new Map();
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        parse(mapData) {
            // Load planets
            mapData.forEach((planetData) => {
                const parsedPlanet = new PlanetStruct_1.PlanetStruct().parse(planetData);
                if (parsedPlanet) {
                    if (this.planets.has(planetData.id)) {
                        console.error(`Duplicate id found: ${planetData.id}`);
                    }
                    else {
                        this.planets.set(planetData.id, parsedPlanet);
                    }
                }
            });
        }
    }
    exports.MapController = MapController;
});
define("switchback-common/component/ThreePieceImage", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var JoinMethod;
    (function (JoinMethod) {
        JoinMethod[JoinMethod["STRETCHED_VERTICALLY"] = 0] = "STRETCHED_VERTICALLY";
        JoinMethod[JoinMethod["STRETCHED_HORIZONTALLY"] = 1] = "STRETCHED_HORIZONTALLY";
        JoinMethod[JoinMethod["TILED_VERTICALLY"] = 2] = "TILED_VERTICALLY";
        JoinMethod[JoinMethod["TILED_HORIZONTALLY"] = 3] = "TILED_HORIZONTALLY";
    })(JoinMethod = exports.JoinMethod || (exports.JoinMethod = {}));
    /**
     * This class creates a resizable three piece tiled image that can be used as a backplate.
     *
     * HOW TO USE:
     * this.backplate = new ThreePieceImage(game, ['topscore-leftplate', 'topscore-midplate',
     * 'topscore-rightplate'], JoinMethod.TILED_HORIZONTALLY)
     * this.backplate.height = this.game.height * 0.1; ~~~~~~~~~~~~~~~~~~~~> if it is joined
     * HORIZONTALLY, scale the backplate's height, and vice versa.
     * this.backplate.scale.x = this.backplate.scale.y;
     *
     * 	-- After this.content is created and filled --
     * this.backplate.resize(this.content.width);
     * this.content.centerX = this.backplate.centerX;
     * this.content.centerY = this.backplate.centerY;
     *
     * An example that uses this can be found in the MinigameTitleDialog class
     *
     * @author qi@cerebralfix.com
     */
    class ThreePieceImage extends Phaser.Group {
        constructor(game, assetKeys, joinMethod) {
            super(game);
            this.joinMethod = joinMethod;
            this.firstPiece = new Phaser.Image(game, 0, 0, assetKeys[0]);
            this.add(this.firstPiece);
            // we need to create this as a Phaser Image first to get the Original Width and Height of
            // the asset so we can use it later
            this.middlePiece = new Phaser.Image(game, 0, 0, assetKeys[1]);
            this.lastPiece = new Phaser.Image(game, 0, 0, assetKeys[2]);
            // if join method is Stretch, create the middle piece as a Phaser Image
            // if join method is Tile, create the middle piece as a Phaser TileSprite
            if (joinMethod === JoinMethod.TILED_HORIZONTALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePiece = new Phaser.TileSprite(game, 0, 0, this.middlePiece.width, this.middlePiece.height, assetKeys[1]);
            }
            // if method is Vertical (tile / stretch) place the pieces below each other
            if (joinMethod === JoinMethod.STRETCHED_VERTICALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.height; // save this off for resizing later
                this.middlePiece.top = this.firstPiece.bottom;
                this.add(this.middlePiece);
                this.lastPiece.top = this.middlePiece.bottom;
                this.add(this.lastPiece);
            }
            else if (joinMethod === JoinMethod.STRETCHED_HORIZONTALLY || joinMethod === JoinMethod.TILED_HORIZONTALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.width; // save this off for resizing later
                this.middlePiece.left = this.firstPiece.right;
                this.add(this.middlePiece);
                this.lastPiece.left = this.middlePiece.right;
                this.add(this.lastPiece);
            }
        }
        /**
         * Resizes the image to be the @param widthOrHeight that is passed in by adding middle tiles
         *
         * @param widthOrHeight width or height that is needed (if the join method is horizontal
         * this will be the width needed, and vice versa)
         * @param padding (optional) extra padding on both sides (multiplied by 2)
         * @param joinMethod (optional) allows the altering of the join method
         */
        resize(widthOrHeight, padding = 0) {
            // removes all extra pieces and resets the size of the group
            // based on the old join method
            this.removeBetween(3, undefined, true); // removes everything except the first 3 pieces added
            switch (this.joinMethod) {
                case JoinMethod.TILED_VERTICALLY: {
                    this.tileVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.TILED_HORIZONTALLY: {
                    this.tileHorizontally(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_VERTICALLY: {
                    this.stretchVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_HORIZONTALLY: {
                    this.stretchHorizontally(widthOrHeight, padding);
                    break;
                }
                default: {
                    break;
                }
            }
        }
        /////////////////////////////////////////////////////////////////////////////
        // TILE:
        // duplicates and tiles the middle piece to fill the missing piece or height
        /////////////////////////////////////////////////////////////////////////////
        tileHorizontally(width, padding) {
            // calculates the number of extra middle pieces needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingWidth / (this.middlePieceOrigWidthOrHeight * this.scale.x));
            this.middlePiece.width = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.left = this.middlePiece.right;
        }
        tileVertically(height, padding) {
            // calculates the number of extra middle pieces needed
            const heightOfFirsttToLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirsttToLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingHeight / (this.middlePieceOrigWidthOrHeight * this.scale.y));
            this.middlePiece.height = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.top = this.middlePiece.bottom;
        }
        ////////////////////////////////////////////////////////////////////////
        // STRETCH:
        // stretches the middle piece to fill the missing width or height
        ////////////////////////////////////////////////////////////////////////
        stretchHorizontally(width, padding) {
            // calculates the amount of missing width needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.width = missingWidth / this.scale.x;
            this.lastPiece.left = this.middlePiece.right;
        }
        stretchVertically(height, padding) {
            // calculates the amount of missing height needed
            const heightOfFirstAndLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.height = missingHeight / this.scale.y;
            this.lastPiece.top = this.middlePiece.bottom;
        }
    }
    exports.ThreePieceImage = ThreePieceImage;
});
define("component/dialog/BasicSpaceDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "Game", "switchback-common/component/ThreePieceImage"], function (require, exports, PromiseUtil_3, Game_2, ThreePieceImage_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BasicSpaceDialog extends Phaser.Group {
        constructor(game, shutdownSignal, descriptionText, customScale, extra = null) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            // shade
            this.shade = new Phaser.Graphics(this.game, 0, 0);
            this.shade.beginFill(0x000000, 0.75);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.alpha = 0;
            this.add(this.shade);
            // group that slides in
            // contains this.content and this.backplate
            this.slidingGroup = new Phaser.Group(this.game);
            this.add(this.slidingGroup);
            // All text and buttons in the dialog (not including backplate)
            // The height of this group is used to calculate backplate height
            // Everything in a dialog should be added to this group
            this.content = new Phaser.Group(this.game);
            this.add(this.content);
            // backplate that can be resized by calling this.resizeBackground()
            this.backplate = new ThreePieceImage_1.ThreePieceImage(this.game, ["dialog-top", "dialog-mid", "dialog-bot"], ThreePieceImage_1.JoinMethod.TILED_VERTICALLY);
            this.backplate.width = this.game.width * 0.95;
            this.backplate.scale.y = this.backplate.scale.x;
            // generic dialog text field that can be used 
            if (descriptionText) {
                this.dialogText = new Phaser.Text(game, 0, 0, descriptionText, {
                    font: Game_2.Game.DEFAULT_FONT,
                    fill: Game_2.Game.DEFAULT_FONT_COLOUR,
                    fontSize: Game_2.Game.FONT_SIZE_MEDIUM,
                    wordWrap: true,
                    wordWrapWidth: this.backplate.width * 0.8,
                    align: 'center',
                    stroke: Game_2.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_2.Game.DEFAULT_STROKE_THICKNESS
                });
                if (!customScale) {
                    this.dialogText.setScaleMinMax(1);
                }
                this.content.add(this.dialogText);
            }
            if (extra) {
                this.content.add(extra);
                extra.centerX = this.dialogText.centerX;
                extra.top = this.dialogText.bottom + this.game.height * 0.05;
            }
            this.slidingGroup.add(this.backplate);
            this.slidingGroup.add(this.content);
            // resizes the backplate based on the content height
            // and positions the content group in the middle of the backplate
            this.resizeBackground();
            // moves the entire sliding group to the left side of the screen to be ready for sliding in
            // this does not include the shade which is left in the middle of the screen
            this.slidingGroup.right = 0;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.game.sound.play("dialog-entry");
                // absorbs all clicks on screen so that nothing else can be clicked in game
                this.shade.inputEnabled = true;
                this.game.add.tween(this.shade).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true);
                this.slidingGroup.right = 0;
                this.slidingGroup.centerY = this.game.height * 0.5;
                const slideInTween = this.game.add.tween(this.slidingGroup).to({ centerX: this.game.width * 0.5 }, 500, Phaser.Easing.Sinusoidal.Out, true, 0);
                yield PromiseUtil_3.PromiseUtil.tween(this.game, slideInTween, this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        hide(selfDestroy = false) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.shade).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                const slideOutTween = this.game.add.tween(this.slidingGroup).to({ left: this.game.height * 1.1 }, 500, Phaser.Easing.Sinusoidal.In, true, 0);
                yield PromiseUtil_3.PromiseUtil.tween(this.game, slideOutTween, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                this.shade.inputEnabled = false;
                this.visible = false;
                if (selfDestroy) {
                    this.destroy();
                }
            });
        }
        // Used by the BasicSpaceLandscapeDialog
        setTextWidth(width) {
            this.dialogText.wordWrapWidth = width;
            this.resizeBackground();
        }
        // Called by subclasses that are modifying the dialog's content
        resizeBackground(resizeByContentHeight = true, paddingProportionOverride) {
            if (resizeByContentHeight) {
                // resizes the backplate based on the content's height
                let paddingBothSides = this.game.height * 0.075;
                if (paddingProportionOverride !== undefined) {
                    paddingBothSides = this.game.height * paddingProportionOverride;
                }
                this.backplate.resize(this.content.height, paddingBothSides);
            }
            else {
                // resizes the backplate based on the content's width
                let paddingBothSides = this.game.width * 0.1;
                if (paddingProportionOverride !== undefined) {
                    paddingBothSides = this.game.width * paddingProportionOverride;
                }
                this.backplate.resize(this.content.width, paddingBothSides);
            }
            // repositions contents to center of backplate
            this.content.centerY = this.backplate.centerY;
            this.content.centerX = this.backplate.centerX;
        }
        destroy(destroyChildren, soft) {
            this.shutdownSignal = null;
            super.destroy(destroyChildren, soft);
        }
    }
    exports.BasicSpaceDialog = BasicSpaceDialog;
});
define("switchback-common/util/ScaleUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function scaleToBounds(object, widthBound, heightBound, enlargeToFit = true) {
        if (!enlargeToFit && object.width <= widthBound && object.height <= heightBound) {
            return;
        }
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio > yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToBounds = scaleToBounds;
    // Usage: scaleToSingleBound(object, this.game.width * 0.3); width will be set to bound, and height in proportion
    // Usage: scaleToSingleBound(object, this.game.width * 0.3, false); height will be set to bound, and width in proportion
    function scaleToSingleBound(object, bound, isWidth = true) {
        if (isWidth) {
            object.width = bound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = bound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToSingleBound = scaleToSingleBound;
    // Scales the object such that its smallest dimension covers the specified bounds. Its
    // largest dimension will likely exceed the other bound.
    // Useful for ensuring that a background covers the screen without stretching
    function scaleToCover(object, widthBound, heightBound) {
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio < yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToCover = scaleToCover;
    class ScaleUtil {
        static getVerticalProportionalPosition(pt1, pt2, proportion = 0.5) {
            return pt1 + ((pt2 - pt1) * proportion);
        }
    }
    exports.ScaleUtil = ScaleUtil;
});
define("component/dialog/ConfirmDialog", ["require", "exports", "switchback-common/util/ScaleUtil", "Game", "component/dialog/BasicSpaceDialog"], function (require, exports, ScaleUtil_1, Game_3, BasicSpaceDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ConfirmDialog extends BasicSpaceDialog_1.BasicSpaceDialog {
        constructor(game, descriptionText, confirmButtonText, declineButtonText, shutdownSignal, extra = null) {
            super(game, shutdownSignal, descriptionText, null, extra);
            // aligns the text to the middle
            this.dialogText.align = 'center';
            this.onButtonClickedSignal = new Phaser.Signal();
            this.buttonGroup = new Phaser.Group(this.game);
            const buttonFontStyle = {
                font: Game_3.Game.DEFAULT_FONT,
                fontSize: Game_3.Game.FONT_SIZE_LARGE,
                fill: Game_3.Game.DEFAULT_FONT_COLOUR,
                align: 'center',
                stroke: Game_3.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_3.Game.DEFAULT_STROKE_THICKNESS,
            };
            // left button (back)
            this.declineButton = new Phaser.Button(this.game, 0, 0, "back-button-small", () => {
                this.isConfirmed = false;
                this.onButtonClickedSignal.dispatch(false);
            }, this, 0, 0, 1, 0);
            ScaleUtil_1.scaleToBounds(this.declineButton, this.backplate.width * 0.48, this.height * 0.5);
            const declineButtonLabel = new Phaser.Text(this.game, this.declineButton.centerX, this.declineButton.centerY, declineButtonText, buttonFontStyle);
            declineButtonLabel.setScaleMinMax(1);
            declineButtonLabel.anchor.set(0.5);
            ScaleUtil_1.scaleToBounds(declineButtonLabel, this.declineButton.width * 0.8, this.declineButton.height * 0.8);
            declineButtonLabel.setShadow(2, 2, Game_3.Game.DEFAULT_FONT_STROKE, 2, true, false);
            // right button (quit)
            this.confirmButton = new Phaser.Button(this.game, 0, 0, "confirm-button-small", () => {
                this.isConfirmed = true;
                this.onButtonClickedSignal.dispatch(true);
            }, this, 0, 0, 1, 0);
            this.confirmButton.width = this.declineButton.width;
            this.confirmButton.scale.y = this.confirmButton.scale.x;
            this.confirmButton.left = this.declineButton.right;
            const confirmButtonLabel = new Phaser.Text(this.game, this.confirmButton.centerX, this.confirmButton.centerY, confirmButtonText, buttonFontStyle);
            confirmButtonLabel.setScaleMinMax(1);
            confirmButtonLabel.anchor.set(0.5);
            ScaleUtil_1.scaleToBounds(confirmButtonLabel, this.confirmButton.width * 0.8, this.confirmButton.height * 0.8);
            confirmButtonLabel.setShadow(2, 2, Game_3.Game.DEFAULT_FONT_STROKE, 2, true, false);
            this.buttonGroup.add(this.confirmButton);
            this.buttonGroup.add(confirmButtonLabel);
            this.buttonGroup.add(this.declineButton);
            this.buttonGroup.add(declineButtonLabel);
            this.buttonGroup.centerX = this.dialogText.centerX;
            this.buttonGroup.top = this.content.bottom;
            this.content.add(this.buttonGroup);
            this.resizeBackground();
        }
        hide(selfDestroy = false) {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.confirmButton.inputEnabled = false;
                this.declineButton.inputEnabled = false;
                yield _super("hide").call(this, selfDestroy);
            });
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.isConfirmed = false;
                this.confirmButton.inputEnabled = true;
                this.declineButton.inputEnabled = true;
                yield _super("show").call(this);
            });
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.ConfirmDialog = ConfirmDialog;
});
define("component/dialog/ContinueGameDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/dialog/BasicSpaceDialog", "Game", "switchback-common/localization/LocalizationManager", "component/dialog/ConfirmDialog", "logic/SMAnalyticsManager", "logic/GameController"], function (require, exports, PromiseUtil_4, BasicSpaceDialog_2, Game_4, LocalizationManager_2, ConfirmDialog_1, SMAnalyticsManager_2, GameController_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ContinueGameDialog extends BasicSpaceDialog_2.BasicSpaceDialog {
        constructor(game, shutdownSignal, previousPlayers) {
            super(game, shutdownSignal);
            this.onOptionSelected = new Phaser.Signal();
            const textFontStyle = Object.assign({}, Game_4.Game.DEFAULT_FONT_STYLE, { align: 'center', strokeThickness: Game_4.Game.DEFAULT_STROKE_THICKNESS });
            this.isNewGroup = previousPlayers !== undefined;
            this.gaLabel = this.isNewGroup ? "NewGroup" : "SameGroup";
            if (previousPlayers) {
                const prompt1 = new Phaser.Text(game, 0, 0, LocalizationManager_2.LocalizationManager.Instance.text('continue_game.playing_with'), Object.assign({}, textFontStyle, { fontSize: Game_4.Game.FONT_SIZE_MEDIUM, fill: Game_4.Game.FONT_COLOUR_BLUE, wordWrap: true, wordWrapWidth: this.backplate.width * 0.9 }));
                prompt1.centerX = 0;
                prompt1.top = 0;
                this.content.add(prompt1);
                const playerNames = new Phaser.Group(game);
                // add player names
                previousPlayers.forEach(name => {
                    const nameField = new Phaser.Text(game, 0, 0, name, Object.assign({}, textFontStyle, { fontSize: Game_4.Game.FONT_SIZE_LARGE }));
                    nameField.centerX = 0;
                    nameField.top = playerNames.height;
                    playerNames.add(nameField);
                });
                playerNames.width = Math.min(playerNames.width, this.backplate.width * 0.9);
                playerNames.scale.y = playerNames.scale.x;
                playerNames.centerX = 0;
                playerNames.top = this.content.height;
                this.content.add(playerNames);
            }
            const prompt2 = new Phaser.Text(game, 0, 0, LocalizationManager_2.LocalizationManager.Instance.text('continue_game.continue'), Object.assign({}, textFontStyle, { fontSize: Game_4.Game.FONT_SIZE_MEDIUM, fill: Game_4.Game.FONT_COLOUR_BLUE, wordWrap: true, wordWrapWidth: this.backplate.width * 0.9 }));
            prompt2.centerX = 0;
            prompt2.top = this.content.height;
            this.content.add(prompt2);
            const buttonFontStyle = {
                font: Game_4.Game.DEFAULT_FONT,
                fontSize: Game_4.Game.FONT_SIZE_LARGE,
                fill: Game_4.Game.DEFAULT_FONT_COLOUR,
                align: 'center',
                stroke: Game_4.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_4.Game.DEFAULT_STROKE_THICKNESS,
            };
            this.continueButton = new Phaser.Image(game, 0, 0, 'button-continue');
            this.continueButton.width = this.backplate.width * 0.9;
            this.continueButton.scale.y = this.continueButton.scale.x;
            this.continueButton.inputEnabled = true;
            this.continueButton.top = prompt2.bottom;
            this.continueButton.centerX = 0;
            this.content.add(this.continueButton);
            const continueButtonLabel = new Phaser.Text(this.game, this.continueButton.centerX, this.continueButton.centerY, LocalizationManager_2.LocalizationManager.Instance.text('buttons.continue'), buttonFontStyle);
            continueButtonLabel.setScaleMinMax(1);
            continueButtonLabel.anchor.set(0.5);
            continueButtonLabel.setShadow(2, 2, Game_4.Game.DEFAULT_FONT_STROKE, 2, true, false);
            this.content.add(continueButtonLabel);
            this.startAgainButton = new Phaser.Image(game, 0, 0, 'button-startagain');
            this.startAgainButton.width = this.backplate.width * 0.9;
            this.startAgainButton.scale.y = this.startAgainButton.scale.x;
            this.startAgainButton.inputEnabled = true;
            this.startAgainButton.top = this.continueButton.bottom;
            this.startAgainButton.centerX = 0;
            this.content.add(this.startAgainButton);
            const startAgainButtonLabel = new Phaser.Text(this.game, this.startAgainButton.centerX, this.startAgainButton.centerY, LocalizationManager_2.LocalizationManager.Instance.text('buttons.start_again'), buttonFontStyle);
            startAgainButtonLabel.setScaleMinMax(1);
            startAgainButtonLabel.anchor.set(0.5);
            startAgainButtonLabel.setShadow(2, 2, Game_4.Game.DEFAULT_FONT_STROKE, 2, true, false);
            this.content.add(startAgainButtonLabel);
            this.continueButton.events.onInputDown.addOnce(() => {
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.trackAction("ContinueGame", "Resume", "ContinueGame", this.gaLabel);
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.endTiming("RocketRace", "RocketRace", 'Game', '');
                this.selectOption(true);
            });
            this.startAgainButton.events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.trackAction("Exit", "Resume", "NewGame", this.gaLabel);
                const confirmDialog = new ConfirmDialog_1.ConfirmDialog(this.game, LocalizationManager_2.LocalizationManager.Instance.text('continue_game.confirm'), LocalizationManager_2.LocalizationManager.Instance.text('buttons.yes'), LocalizationManager_2.LocalizationManager.Instance.text('buttons.no'), this.shutdownSignal);
                yield confirmDialog.show();
                if (confirmDialog.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                const confirmed = yield PromiseUtil_4.PromiseUtil.signal(confirmDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.trackAction(confirmed ? "Exit_Confirm" : "Exit_Cancel", "Resume", confirmed ? "NewGameConfirm" : "NewGameCancel", this.gaLabel);
                if (confirmed) {
                    this.selectOption(false);
                }
                yield confirmDialog.hide();
                if (confirmDialog.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                confirmDialog.destroy();
            }));
            this.resizeBackground();
            if (this.isNewGroup) {
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.trackState('content/game/inqueue/resume/newgroup', 'ResumeNewGroup');
            }
            else {
                SMAnalyticsManager_2.SMAnalyticsManager.Instance.trackState('content/game/inqueue/resume', 'Resume');
            }
        }
        selectOption(shouldContinue) {
            this.continueButton.inputEnabled = false;
            this.startAgainButton.inputEnabled = false;
            SMAnalyticsManager_2.SMAnalyticsManager.Instance.endTiming("RocketRace", "RocketRace", 'Game', '');
            GameController_6.GameController.Instance.state.numVisits++;
            SMAnalyticsManager_2.SMAnalyticsManager.Instance.numPartyVisits = GameController_6.GameController.Instance.state.numVisits;
            SMAnalyticsManager_2.SMAnalyticsManager.Instance.startTiming("RocketRace");
            this.onOptionSelected.dispatch(shouldContinue);
        }
        destroy(destroyChildren, soft) {
            this.onOptionSelected.dispose();
            super.destroy(destroyChildren, soft);
        }
    }
    exports.ContinueGameDialog = ContinueGameDialog;
});
define("switchback-common/state/BaseState", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * BaseState is our extension of Phaser.State that all states within games should extend.
     * This is to allow us to handle things like state transitions and display object hierarchies in a common way.
     * @author joel@cerebralfix.com
     */
    class BaseState extends Phaser.State {
        constructor() {
            super(...arguments);
            // Duration of the fade graphic alpha in and out, in milliseconds
            this.FADE_DURATION_MS = 300;
            this.stateName = "BaseState";
            this.firstPointerDownTime = 0;
            this.tapCount = 0;
        }
        init(...args) {
            super.init(...args);
            console.log(`[${this.stateName}] init(${JSON.stringify(args)})`);
            this.shutdownSignal = new Phaser.Signal();
            this.isShutdown = false;
            // Hijack the state's add function to ensure all game objects are
            // added to the correct group (e.g. under the fade overlay)
            this.add.existing = this.addToGameLayer.bind(this);
            // Create our game group here
            this.gameGroup = new Phaser.Group(this.game);
            // Create our dev group here, goes on top of the game group
            this.createDevGroup();
            // Then immediately add our fade overlay on top. This ensures the
            // overlay is present for the duration of the loading
            this.fadeImage = this.add.graphics(0, 0);
            this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
            this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
            this.fadeImage.endFill();
            this.fadeImage.fixedToCamera = true;
            this.fadeImage.inputEnabled = true;
            this.game.input.onDown.add(this.onPointerDown, this);
        }
        create() {
            console.log(`[${this.stateName}] create()`);
            super.create();
        }
        removeChildFromGameGroup(child) {
            this.gameGroup.remove(child);
        }
        createDevGroup() {
            this.devGroup = new Phaser.Group(this.game);
            this.devGroup.visible = false;
            const data = this.game.cache.getJSON("build-info");
            if (data) {
                let versionString = `${data.hash}`;
                if (data.version) {
                    versionString = "v" + `${data.version}` + "." + `${data.build}` + "-" + `${data.agent}` + " (" + `${data.hash}` + ")";
                }
                const text = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height, versionString, {
                    fontSize: 50,
                    fill: "white",
                    stroke: "black",
                    strokeThickness: 5,
                    align: "center"
                });
                text.anchor.set(0.5, 1);
                this.game.input.onDown.add(this.onInputDown, this);
                text.inputEnabled = true;
                this.devGroup.add(text);
            }
        }
        onInputDown() {
            this.devGroup.visible = false;
        }
        onPointerDown(pointer) {
            // Was it in the right place?
            if (pointer.positionDown.x < this.game.width * 0.1 &&
                pointer.positionDown.y > this.game.height * 0.9) {
                // If it's been too long since the last tap in this area, start the timer and counter again
                if (this.game.time.now > this.firstPointerDownTime + 1000) {
                    this.firstPointerDownTime = this.game.time.now;
                    this.tapCount = 0;
                }
                this.tapCount++;
                if (this.tapCount === 5) {
                    this.devGroup.visible = true;
                    this.tapCount = 0;
                }
            }
            else {
                // If it wasn't in the right place, reset the counter.
                this.tapCount = 0;
            }
        }
        preload() {
            super.preload();
            console.log(`[${this.stateName}] preload()`);
            // Add a listener so we can see which files are loading
            this.load.onFileComplete.add((progress, key, success, loaded, total) => {
                console.log(`[${this.stateName}] preload() [${progress}% | ${loaded}/${total}] loading: ${key}`);
            });
            // Load the game's common assets into this state
            if (BaseState.loadCommonAssets) {
                BaseState.loadCommonAssets();
            }
        }
        ensurePointers(num) {
            // check if we can even do that
            if (num > Phaser.Input.MAX_POINTERS) {
                console.log(`[${this.stateName}] Can't add pointers. `, num, "/", Phaser.Input.MAX_POINTERS);
                return false;
            }
            for (let i = 1; i < num + 1; i++) {
                // if pointer doesn't exists, add it
                if (!this.game.input["pointer" + i]) {
                    this.game.input.addPointer();
                }
                // if it hasnt been added, explode
                if (!this.game.input["pointer" + i]) {
                    return false;
                }
            }
            // fix reduced inputs
            this.game.input.maxPointers = num;
            return true;
        }
        fadeToState(state, ...args) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                this.game.state.start(state, true, false, ...args);
            });
        }
        /*
         * Shows the input-blocking overlay on top of the game, then tweens it to full alpha.
         * This function should be awaited before transitioning away to a loading screen.
         */
        showOverlay(alpha = 1, fadeDurationMS = this.FADE_DURATION_MS, newColor = undefined) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] showOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 0;
                this.fadeImage.visible = true;
                if (newColor) {
                    BaseState.OVERLAY_COLOR = newColor;
                    this.fadeImage.clear();
                    this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
                    this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
                    this.fadeImage.endFill();
                }
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                }
                else {
                    this.fadeImage.alpha = 0;
                    const fadeInTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    yield PromiseUtil_5.PromiseUtil.tween2(fadeInTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        /*
         * Tweens the overlay out to alpha 0 and then sets it to invisible (to remove input-blocking).
         * This function should be called when the state's create() function is complete
         * and everything has been setup on-screen.
         */
        dismissOverlay(alpha = 0, fadeDurationMS = this.FADE_DURATION_MS) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] dismissOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 1;
                this.fadeImage.visible = true;
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                    this.fadeImage.visible = false;
                }
                else {
                    this.fadeImage.alpha = 1;
                    const fadeOutTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    fadeOutTween.onComplete.addOnce(() => {
                        this.fadeImage.visible = false;
                        this.overlayDismissed();
                    });
                    yield PromiseUtil_5.PromiseUtil.tween2(fadeOutTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        shutdown() {
            console.log(`[${this.stateName}] shutdown()`);
            this.isShutdown = true;
            if (this.shutdownSignal) {
                this.shutdownSignal.dispatch();
                this.shutdownSignal.dispose();
            }
            this.game.input.onDown.remove(this.onPointerDown, this);
            // Remove our file load listener
            this.load.onFileComplete.removeAll();
            this.fadeImage.destroy();
            this.devGroup.destroy();
            this.gameGroup.destroy();
            super.shutdown();
            this.shutdownSignal = null;
            this.fadeImage = null;
            this.gameGroup = null;
            this.devGroup = null;
        }
        /*
         * Overridden by subclasses to determine when overlay has finished
         * dismissing
         */
        overlayDismissed() {
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToGameLayer(object) {
            this.gameGroup.add(object);
            return object;
        }
    }
    BaseState.OVERLAY_COLOR = 0x000000;
    exports.BaseState = BaseState;
});
define("switchback-common/util/SoundUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SoundUtil {
        /**
         * Add sounds loaded via loadSound or loadSoundAll will be prefixed by this path
         */
        static setRootSoundPath(value) {
            SoundUtil._rootSoundPath = value;
        }
        /**
         * @param tag Phaser SoundManager tag for this sound
         * @param filename Doesn't include the filename
         */
        static loadSound(load, tag, filename) {
            load.audio(tag, [SoundUtil._rootSoundPath + filename + ".m4a",
                SoundUtil._rootSoundPath + filename + ".ogg"]);
        }
        /**
         * Loads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound (will have the index appended)
         * @param filename Doesn't include the filename
         * @param count How many sounds to load
         */
        static loadSoundAll(load, tag, filename, count) {
            for (let i = 0; i < count; ++i) {
                load.audio(tag + i, [SoundUtil._rootSoundPath + filename + i + ".m4a",
                    SoundUtil._rootSoundPath + filename + i + ".ogg"]);
            }
        }
        /**
         * Unloads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound
         * @param count How many sounds to unload
         */
        static unloadSoundAll(cache, tag, count) {
            for (let i = 0; i < count; ++i) {
                cache.removeSound(tag + i);
            }
        }
        /**
         * Play a sound at random from a set of sounds with the same base tag
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param count How many sounds in this set
         */
        static playRandom(game, soundTag, count) {
            return game.sound.play(soundTag + Math.floor(Math.random() * count));
        }
        /**
         * Play a sound on a fixed interval
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param interval How freqently to play the sound
         * @param volume Volume to play the sound
         * @returns An id that is used to stop this loop from playing
         */
        static playSoundOnInterval(game, soundTag, interval, volume) {
            return setInterval(() => {
                game.sound.play(soundTag, volume);
            }, interval);
        }
        /**
         *
         * @param intervalId This id generated by playSoundOnInterval used to stop the loop
         */
        static stopSoundOnInterval(intervalId) {
            clearInterval(intervalId);
        }
        /**
         * Allows to resume the audio context. Specially after a focus loss.
         * This gets called automatically by PauseUtil, which is the preferred method.
         * Accessible here for the special cases where you want this functionality and skipping PauseUtil.
         */
        static resumeAudio(game) {
            // you can't re-enable the sounds until the user has had an input and
            // the sounds aren't already running
            game.input.onDown.addOnce(() => {
                try {
                    if (game.sound.context.state !== "running") {
                        game.sound.context.resume();
                    }
                }
                catch (e) {
                    console.log("[SoundUtil.resumeAudio] Error attempting to resume sounds.");
                }
            }, Number.MAX_SAFE_INTEGER);
        }
    }
    SoundUtil._rootSoundPath = "";
    exports.SoundUtil = SoundUtil;
});
define("switchback-common/util/PauseUtil", ["require", "exports", "switchback-common/util/SoundUtil"], function (require, exports, SoundUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PauseUtil {
        /*
         * Use this function to listen for when the game has resumed changes visibility.
         * On iOS backgrounding the game can kill sound context and you'll
         * permanently lose audio. This function resumes it.
         * Function should be hooked once as the game loads.
         *
         * This needs to be called as the last part on initialization.
         * As in Android the player management card will trigger 2 onBlur.
         */
        static addHooks(game) {
            console.log("[PauseUtil] addHooks");
            PauseUtil.removeHooks(game);
            PauseUtil._isBlurred = false;
            PauseUtil.onBlur = new Phaser.Signal();
            PauseUtil.onFocus = new Phaser.Signal();
            PauseUtil._onBlurListener = PauseUtil.onBlurred.bind(null, game);
            PauseUtil._onFocusListener = PauseUtil.onFocused.bind(null, game);
            // These two are the only ones reliable on ios and android
            // onBlur and onFocus are not triggering on android for some reason
            // also document.addEventListener("visibilitychange") seems to be reliable too
            game.onPause.add(PauseUtil._onBlurListener);
            game.onResume.add(PauseUtil._onFocusListener);
            // Make sure the audio gets activated. Because the engine might have been stopped before this hook gets called.
            // Or in new updates the engine always starts stopped.
            SoundUtil_1.SoundUtil.resumeAudio(game);
        }
        static removeHooks(game) {
            if (PauseUtil.onBlur) {
                PauseUtil.onBlur.dispose();
                PauseUtil.onBlur = null;
            }
            if (PauseUtil.onFocus) {
                PauseUtil.onFocus.dispose();
                PauseUtil.onFocus = null;
            }
            if (PauseUtil._onBlurListener) {
                game.onBlur.remove(this._onBlurListener, null);
            }
            if (PauseUtil._onFocusListener) {
                game.onFocus.remove(PauseUtil._onFocusListener, null);
            }
        }
        static onBlurred(game) {
            console.log("[PauseUtil] onBlur. isBlurred=", PauseUtil._isBlurred);
            if (PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = true;
            PauseUtil.onBlur.dispatch();
        }
        static onFocused(game) {
            console.log("[PauseUtil] onFocus. isBlurred=", PauseUtil._isBlurred);
            if (!PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = false;
            SoundUtil_1.SoundUtil.resumeAudio(game);
            PauseUtil.onFocus.dispatch();
        }
    }
    exports.PauseUtil = PauseUtil;
});
define("state/LoadingState", ["require", "exports", "Game", "logic/GameController", "StateNames", "logic/MapController", "component/dialog/ContinueGameDialog", "config/SpaceMountainConfig", "switchback-common/state/BaseState", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "logic/RaceController", "switchback-common/util/PauseUtil"], function (require, exports, Game_5, GameController_7, StateNames_3, MapController_1, ContinueGameDialog_1, SpaceMountainConfig_4, BaseState_1, PromiseUtil_6, LocalizationManager_3, RaceController_2, PauseUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class LoadingState extends BaseState_1.BaseState {
        init(...params) {
            this.stateName = StateNames_3.StateNames.LOAD;
            super.init(params);
            this.scale.scaleMode = Phaser.ScaleManager.RESIZE;
        }
        preload() {
            super.preload();
            this.game.load.json("config", "assets/json/config.json");
            this.game.load.json("mission-config", "assets/json/mission-config.json");
            this.game.load.json("race-config", "assets/json/race-config.json");
            // TODO polish?
            // Notice this is loaded before race-config is being parsed, hence, it's a hardcoded value
            for (let i = 0; i < RaceController_2.RaceController.NUM_RACES; i++) {
                this.game.load.json("race-" + i, "assets/json/race-" + i + ".json");
            }
            this.game.load.json("i18n", "assets/lang/strings.json");
            this.game.load.json("map", "assets/map/map.json");
            this.game.load.json('build-info', 'assets/json/build-info.json');
            this.game.load.image("logo", "assets/cutscene/sm_logo.png");
            this.game.load.image("logo-text", "assets/cutscene/logo-text.png");
            this.game.load.image("sparkle", "assets/cutscene/sparkle.png");
            this.game.load.audio("cta-button", ["assets/sound/ui/cta-button.ogg", "assets/sound/ui/cta-button.m4a"]);
            this.game.load.audio("dialog-entry", ["assets/sound/hub/mission-dialog.ogg", "assets/sound/hub/mission-dialog.m4a"]);
            this.game.load.audio("doors_close", ["assets/sound/race/doors_close.ogg", "assets/sound/race/doors_close.m4a"]);
            this.game.load.audio("doors_open", ["assets/sound/race/doors_open.ogg", "assets/sound/race/doors_open.m4a"]);
            // Continue dialog
            this.game.load.image("button-continue", "assets/button/continue_button.png");
            this.game.load.image("button-startagain", "assets/button/startagain_button.png");
            this.game.load.image("back-button-small", "assets/button/back_button_small.png");
            this.game.load.image("confirm-button-small", "assets/button/confirm_button_small.png");
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                // Initialize our localization manager
                LocalizationManager_3.LocalizationManager.Instance.init("en", this.game.cache.getJSON("i18n"));
                // As soon as we possibly can, we should load up the config file
                SpaceMountainConfig_4.SpaceMountainConfig.Instance.parse(this.game.cache.getJSON("config"));
                if (!SpaceMountainConfig_4.SpaceMountainConfig.Instance.enableConsoleLogs) {
                    console.log = () => { };
                }
                // Connect to the installation
                GameController_7.GameController.Instance.calculateDefaultTextSizes(this.game.width, this.game.height);
                // Turn on advancedTiming if we're in debug mode
                if (SpaceMountainConfig_4.SpaceMountainConfig.Instance.enableFPS) {
                    this.game.time.advancedTiming = true;
                }
                // Disable pause on off-screen for quicktest
                this.stage.disableVisibilityChange = SpaceMountainConfig_4.SpaceMountainConfig.Instance.quickTest;
                let loadingText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.25, LocalizationManager_3.LocalizationManager.Instance.text("intro.loading_text"), {
                    font: Game_5.Game.DEFAULT_FONT,
                    fontSize: Game_5.Game.FONT_SIZE_LARGE,
                    fill: Game_5.Game.DEFAULT_FONT_COLOUR,
                    align: 'center'
                });
                loadingText.setScaleMinMax(1);
                loadingText.anchor.set(0.5);
                loadingText.width = this.game.width * 0.3;
                loadingText.scale.y = loadingText.scale.x;
                this.add.existing(loadingText);
                this.dismissOverlay();
                // Wait for our initialisation, with a minimum time of 1 second to prevent the screen flashing
                yield Promise.all([PromiseUtil_6.PromiseUtil.time(this.game, 1000),
                    GameController_7.GameController.Instance.init()]);
                // This fixes sound being lost on iOS, it must be hooked after player management
                PauseUtil_1.PauseUtil.addHooks(this.game);
                // Parse all map data here
                // TODO: Find a better spot that takes into account the timePromise
                MapController_1.MapController.Instance.parse(this.game.cache.getJSON("map"));
                const hasPreviousSession = GameController_7.GameController.Instance.hasActiveSession();
                const achievementPrefix = "spaceMountain" + (GameController_7.GameController.Instance.parkPrefix || "").toUpperCase();
                GameController_7.GameController.Instance.achievementManager.initialise(GameController_7.GameController.Instance.players, achievementPrefix);
                // Ask player to resume session
                if (hasPreviousSession) {
                    if (GameController_7.GameController.Instance.isPlayingWithNewParty()) {
                        this.continueDialog = new ContinueGameDialog_1.ContinueGameDialog(this.game, this.shutdownSignal, GameController_7.GameController.Instance.previousPlayers.map(player => player.name));
                    }
                    else {
                        this.continueDialog = new ContinueGameDialog_1.ContinueGameDialog(this.game, this.shutdownSignal);
                    }
                    this.continueDialog.visible = false;
                    this.add.existing(this.continueDialog);
                    yield this.continueDialog.show();
                    const shouldContinue = yield PromiseUtil_6.PromiseUtil.signal(this.continueDialog.onOptionSelected);
                    yield this.continueDialog.hide();
                    if (!shouldContinue) {
                        yield GameController_7.GameController.Instance.startNewGame();
                    }
                }
                GameController_7.GameController.Instance.state.sessionStarted = true;
                // Initialise the race controller - N.B. We have to do it after we have save states
                GameController_7.GameController.Instance.raceController.init(this.game.cache.getJSON("race-config"));
                // TODO polish get num json races from raceController.raceConfig
                for (let i = 0; i < RaceController_2.RaceController.NUM_RACES; i++) {
                    GameController_7.GameController.Instance.raceController.addRaceMovementData(i, this.game.cache.getJSON(`race-` + i));
                }
                GameController_7.GameController.Instance.missionManager.init(this.game.cache.getJSON('mission-config'));
                yield this.showOverlay();
                if (!GameController_7.GameController.Instance.state.ftueCompleted) {
                    this.game.state.start(StateNames_3.StateNames.INTRO);
                }
                else {
                    this.game.state.start(StateNames_3.StateNames.HUB);
                }
            });
        }
        shutdown() {
            this.continueDialog = null;
            super.shutdown();
        }
    }
    exports.LoadingState = LoadingState;
});
define("component/DebugPanel", ["require", "exports", "config/SpaceMountainConfig", "StateNames", "logic/GameController", "Game"], function (require, exports, SpaceMountainConfig_5, StateNames_4, GameController_8, Game_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DebugPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            if (SpaceMountainConfig_5.SpaceMountainConfig.Instance.enableFPS) {
                this.fpsText = this.game.add.text(this.game.width, 0, "", { fill: "white", align: 'right', fontSize: 50 });
                this.fpsText.anchor.set(1, 0);
                this.fpsText.fixedToCamera = true;
            }
            this.interruptButtonGroup = new Phaser.Group(this.game);
            this.interruptButtonGroup.fixedToCamera = true;
            ///////////////////////////////
            // HEAD TO OUTRO
            ///////////////////////////////
            const exitGlobalButton = this.createButton(this.game.width * 0.1, 0, "O");
            exitGlobalButton.onInputDown.add(() => {
                GameController_8.GameController.Instance.isExitBeaconTriggered = true;
            }, this);
            this.interruptButtonGroup.add(exitGlobalButton);
            let padding = exitGlobalButton.width * 1.1;
            ///////////////////////////////
            // CLEAR SAVE STATE
            ///////////////////////////////
            const clearState = this.createButton(exitGlobalButton.left + padding, 0, "🗑");
            clearState.onInputDown.add(() => {
                GameController_8.GameController.Instance.state.clear();
            }, this);
            this.interruptButtonGroup.add(clearState);
            ///////////////////////////////
            // BEGIN PRE-RACE
            ///////////////////////////////
            const raceButton = this.createButton(clearState.left + padding, 0, "R");
            raceButton.onInputDown.addOnce(() => {
                GameController_8.GameController.Instance.raceController.transitionToRaceSignal.dispatch();
            });
            this.interruptButtonGroup.add(raceButton);
            ///////////////////////////////
            // FAKE RACE MESSAGE
            ///////////////////////////////
            const fakeRaceMessage = this.createButton(raceButton.left + padding, 0, "Rx");
            fakeRaceMessage.onInputDown.add(() => {
                GameController_8.GameController.Instance.raceController.triggerDebugBeaconRace();
            }, this);
            this.interruptButtonGroup.add(fakeRaceMessage);
            ///////////////////////////////
            // AWARD PART
            ///////////////////////////////
            const addPartButton = this.createButton(fakeRaceMessage.left + padding, 0, "+P");
            addPartButton.onInputDown.add(() => {
                GameController_8.GameController.Instance.addPendingShipParts(1);
            });
            this.interruptButtonGroup.add(addPartButton);
            ///////////////////////////////
            // AWARD BONUS PART
            ///////////////////////////////
            const addBonusPartButton = this.createButton(addPartButton.left + padding, 0, "+BP");
            addBonusPartButton.onInputDown.add(() => {
                GameController_8.GameController.Instance.addPendingBonusShipParts(1);
            });
            this.interruptButtonGroup.add(addBonusPartButton);
            ///////////////////////////////
            // MINIGAME BUTTONS
            ///////////////////////////////
            const games = [
                StateNames_4.StateNames.GAME_DOCKING,
                StateNames_4.StateNames.GAME_PIPES,
                StateNames_4.StateNames.GAME_MEMORY,
                StateNames_4.StateNames.GAME_RELAY_INTRO,
                StateNames_4.StateNames.GAME_ASTEROIDS,
                StateNames_4.StateNames.GAME_ENERGY_BALLS
            ];
            const tags = [
                "D", "P", "M", "R", "A", "E"
            ];
            games.forEach((state, i) => {
                const btn = this.createButton(i * padding, exitGlobalButton.height * 1.5, tags[i]);
                btn.onInputDown.addOnce(() => {
                    this.onMinigameButtonClicked(state);
                });
                this.interruptButtonGroup.add(btn);
            });
            ///////////////////////////////
            // BUTTON TOGGLING
            ///////////////////////////////
            this.interruptButtonGroup.children.forEach((intButt) => {
                intButt.visible = false;
            }, this);
            const toggleButton = new Phaser.Button(this.game, 0, 0, 'plus', () => {
                for (let i = 0; i < this.interruptButtonGroup.children.length - 1; i++) {
                    this.interruptButtonGroup.children[i].visible = !this.interruptButtonGroup.children[i].visible;
                }
            }, this);
            toggleButton.width = toggleButton.height = this.game.width * 0.1;
            this.interruptButtonGroup.add(toggleButton);
        }
        onMinigameButtonClicked(stateName) {
            GameController_8.GameController.Instance.startGame(stateName);
            this.game.state.start(StateNames_4.StateNames.NARRATIVE_BRIEFING);
        }
        createButton(x, y, text) {
            const interruptButton = new Phaser.Button(this.game, x, y, "back_button_small");
            interruptButton.width = this.game.width * 0.13;
            interruptButton.scale.y = interruptButton.scale.x;
            interruptButton.anchor.set(0.5);
            interruptButton.top = y;
            interruptButton.left = x;
            const interruptButtonText = new Phaser.Text(this.game, 0, 0, text, {
                fill: Game_6.Game.DEFAULT_FONT_COLOUR,
                font: Game_6.Game.DEFAULT_FONT,
                fontSize: 50,
                wordWrap: true,
                wordWrapWidth: interruptButton.width,
                align: 'center',
                stroke: Game_6.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_6.Game.DEFAULT_STROKE_THICKNESS
            });
            interruptButtonText.setScaleMinMax(1);
            interruptButtonText.anchor.set(0.5);
            interruptButton.addChild(interruptButtonText);
            return interruptButton;
        }
        update() {
            super.update();
            if (SpaceMountainConfig_5.SpaceMountainConfig.Instance.enableFPS && this.fpsText) {
                this.fpsText.text = this.game.time.fps.toString();
            }
        }
    }
    exports.DebugPanel = DebugPanel;
});
define("switchback-common/util/CameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function cameraShake(game, intensity = 0.005, durationMS = 200) {
        return __awaiter(this, void 0, void 0, function* () {
            game.camera.shake(intensity, durationMS);
            yield PromiseUtil_7.PromiseUtil.time(game, durationMS);
        });
    }
    exports.cameraShake = cameraShake;
    /**
     * Scales up/down the given group so that it appears as though the camera is zooming in/out respectively.
     * Set duration > 0 to make the zoom a tween.
     *
     * @param camGroup: Phaser.Group - Should contain everything in the scene you wish to zoom
     * @param zoomLevel: number - 1.0 is standard, higher (eg. 1.5 or 2.0) will zoom in, use < 1.0 with caution.
     * @param focusPnt: Phaser.Point (Scale 0-1) - Centre of the screen once the zoom is complete, (0.5, 0.5) would be game world centre
     * @param duration: number - Default is instant, higher numbers will tween the zoom for the given time.
     */
    function cameraZoom(game, camGroup, zoomLevel, focusPnt, duration = -1, easing = Phaser.Easing.Linear.None) {
        return __awaiter(this, void 0, void 0, function* () {
            const movementProps = {};
            const camW = game.width * zoomLevel;
            const camH = game.height * zoomLevel;
            movementProps.x = game.width * 0.5 - camW * focusPnt.x;
            movementProps.y = game.height * 0.5 - camH * focusPnt.y;
            const scaleProps = {
                x: zoomLevel,
                y: zoomLevel,
            };
            if (duration > 0) {
                yield Promise.all([
                    PromiseUtil_7.PromiseUtil.tween2(game.add.tween(camGroup).to(movementProps, duration, easing, true)),
                    PromiseUtil_7.PromiseUtil.tween2(game.add.tween(camGroup.scale).to(scaleProps, duration, easing, true)),
                ]);
            }
            else {
                camGroup.scale.set(scaleProps.x, scaleProps.y);
                camGroup.position.set(movementProps.x, movementProps.y);
            }
        });
    }
    exports.cameraZoom = cameraZoom;
});
define("state/SpaceBaseState", ["require", "exports", "Game", "logic/GameController", "StateNames", "switchback-common/state/BaseState", "config/SpaceMountainConfig", "component/DebugPanel", "switchback-common/util/PromiseUtil", "switchback-common/util/CameraUtil", "logic/SMAnalyticsManager"], function (require, exports, Game_7, GameController_9, StateNames_5, BaseState_2, SpaceMountainConfig_6, DebugPanel_1, PromiseUtil_8, CameraUtil_1, SMAnalyticsManager_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author qi@cerebralfix.com
     */
    class SpaceBaseState extends BaseState_2.BaseState {
        init(startWithDoorsClosed = false, startWithFade = true) {
            super.init();
            this.startWithDoorsClosed = startWithDoorsClosed;
            this.startWithFade = startWithFade;
            GameController_9.GameController.Instance.raceController.transitionToRaceSignal.add(this.transitionToPreRaceState, this);
        }
        preload() {
            super.preload();
            this.load.image('plus', 'assets/common/plus.png');
            this.load.image("door-cover", "assets/common/door-cover.jpg");
        }
        showDebugMenu() {
            new DebugPanel_1.DebugPanel(this.game);
        }
        create() {
            super.create();
            if (SpaceMountainConfig_6.SpaceMountainConfig.Instance.showStateNames) {
                const stateNameText = this.game.add.text(this.game.width * 0.5, 0, this.stateName, {
                    font: Game_7.Game.DEFAULT_FONT,
                    fill: Game_7.Game.DEFAULT_FONT_COLOUR,
                    stroke: Game_7.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_7.Game.DEFAULT_STROKE_THICKNESS,
                    fontSize: Game_7.Game.FONT_SIZE_MEDIUM
                });
                stateNameText.anchor.set(0.5, 0);
            }
            this.input.maxPointers = 1;
            ///////////////////////////////////////
            // SCREEN TRANSITION BLAST DOORS
            ///////////////////////////////////////
            this.doorGroup = new Phaser.Group(this.game);
            this.doorClickBlocker = new Phaser.Graphics(this.game);
            this.doorClickBlocker.beginFill(0xffffff, 0);
            this.doorClickBlocker.drawRect(0, 0, this.game.width, this.game.height);
            this.doorClickBlocker.endFill();
            this.doorClickBlocker.inputEnabled = true;
            this.doorClickBlocker.visible = false;
            this.doorGroup.add(this.doorClickBlocker);
            this.doorL = this.game.add.image(0, 0, "door-cover");
            this.doorL.width = this.game.width * 0.5;
            this.doorL.height = this.game.height;
            this.doorL.scale.x *= -1;
            this.doorGroup.add(this.doorL);
            this.doorR = this.game.add.image(this.game.width, 0, "door-cover");
            this.doorR.width = this.game.width * 0.5;
            this.doorR.height = this.game.height;
            this.doorGroup.add(this.doorR);
            if (this.startWithDoorsClosed) {
                this.closeDoors(0);
            }
            if (!this.startWithFade) {
                this.dismissOverlay(0, 0);
            }
            if (SpaceMountainConfig_6.SpaceMountainConfig.Instance.showDebugMenu) {
                this.showDebugMenu();
            }
        }
        closeDoors(durationMs = 1000) {
            return __awaiter(this, void 0, void 0, function* () {
                this.doorClickBlocker.visible = true;
                if (durationMs === 0) {
                    this.doorL.left = this.game.width * 0.5;
                    this.doorR.left = this.game.width * 0.5;
                }
                else {
                    this.game.sound.play('doors_close');
                    yield Promise.all([
                        PromiseUtil_8.PromiseUtil.tween2(this.game.add.tween(this.doorL).to({ left: this.game.width * 0.5 }, durationMs, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal),
                        PromiseUtil_8.PromiseUtil.tween2(this.game.add.tween(this.doorR).to({ left: this.game.width * 0.5 }, durationMs, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal)
                    ]).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    yield CameraUtil_1.cameraShake(this.game, 0.005, 200);
                }
            });
        }
        openDoors(durationMs = 1000) {
            return __awaiter(this, void 0, void 0, function* () {
                if (durationMs === 0) {
                    this.doorL.left = 0;
                    this.doorR.left = this.game.width;
                }
                else {
                    this.game.sound.play('doors_open');
                    yield Promise.all([
                        PromiseUtil_8.PromiseUtil.tween2(this.game.add.tween(this.doorL).to({ left: 0 }, durationMs, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal),
                        PromiseUtil_8.PromiseUtil.tween2(this.game.add.tween(this.doorR).to({ left: this.game.width }, durationMs, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal)
                    ]).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                }
                this.doorClickBlocker.visible = false;
            });
        }
        /**
         * Called when the state responds to a race interruption request
         * It can be called in the middle of an ongoing minigame
         */
        transitionToPreRaceState() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[SpaceBaseState] transitionToPreRaceState()");
                GameController_9.GameController.Instance.raceController.transitionToRaceSignal.removeAll();
                // Tell the Race controller we've received its request and are acting upon it.
                GameController_9.GameController.Instance.raceController.beginRaceFlow();
                SMAnalyticsManager_3.SMAnalyticsManager.Instance.trackAction('Installation_Trigger', 'Installation', 'Trigger', 'SpaceRace', {
                    [SMAnalyticsManager_3.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_3.SMAnalyticsManager.GAME_PLAYS]: GameController_9.GameController.Instance.state.getMinigamePlays(StateNames_5.StateNames.DEVICE_RACE) + 1
                });
                SMAnalyticsManager_3.SMAnalyticsManager.Instance.trackAction('Installation_GameInterruption', 'Installation', 'GameInterruption', 'SpaceRace', {
                    [SMAnalyticsManager_3.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_3.SMAnalyticsManager.GAME_PLAYS]: GameController_9.GameController.Instance.state.getMinigamePlays(StateNames_5.StateNames.DEVICE_RACE) + 1,
                    [SMAnalyticsManager_3.SMAnalyticsManager.GAME_NAME_MINI]: SMAnalyticsManager_3.SMAnalyticsManager.gameNameMap.get(this.stateName)
                });
                // Transition to the race.
                yield this.closeDoors();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(StateNames_5.StateNames.PRE_RACE);
            });
        }
        showOutroConfirmation() {
            return __awaiter(this, void 0, void 0, function* () { });
        }
        shutdown() {
            GameController_9.GameController.Instance.raceController.transitionToRaceSignal.removeAll();
            GameController_9.GameController.Instance.state.save();
            this.doorGroup = null;
            this.doorClickBlocker = null;
            this.doorL = null;
            this.doorR = null;
            super.shutdown();
        }
    }
    exports.SpaceBaseState = SpaceBaseState;
});
define("component/button/IButton", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("component/CutsceneScreen", ["require", "exports", "switchback-common/util/PromiseUtil", "logic/SMAnalyticsManager"], function (require, exports, PromiseUtil_9, SMAnalyticsManager_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /*
     * You can set `button` attribute in the child class
     * Showing, hiding the button and binding the correct signal is handled in this class.
     */
    class CutsceneScreen extends Phaser.Group {
        constructor() {
            super(...arguments);
            this.button = null;
            this.onButtonClickedSignal = new Phaser.Signal();
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_9.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: 0 }, 300, Phaser.Easing.Sinusoidal.InOut, true));
                if (this.cutsceneName) {
                    SMAnalyticsManager_4.SMAnalyticsManager.Instance.trackState('content/game/inqueue/outro/cutscene', `Outro${this.cutsceneName}`, {
                        [SMAnalyticsManager_4.SMAnalyticsManager.CUTSCENE]: this.cutsceneName
                    });
                }
                if (this.button) {
                    yield this.button.show();
                    this.button.onButtonClickedSignal.addOnce(() => {
                        this.onButtonClickedSignal.dispatch();
                    });
                }
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.button) {
                    this.button.onButtonClickedSignal.removeAll();
                    yield this.button.hide();
                }
                this.game.add.tween(this).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_9.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ right: 0 }, 300, Phaser.Easing.Sinusoidal.InOut, true));
            });
        }
        destroy() {
            this.button = null;
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.CutsceneScreen = CutsceneScreen;
});
define("component/button/ArrowButton", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil"], function (require, exports, Game_8, ScaleUtil_2, LocalizationManager_4, PromiseUtil_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ArrowButton extends Phaser.Group {
        constructor(game, buttonTextKey, facingRight = true, textSize, customScale, shutdownSignal = null) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.shutdownSignal = shutdownSignal;
            this.inputEnableChildren = true;
            this.ignoreChildInput = true;
            this.circle = new Phaser.Image(this.game, 0, 0, "next-circle");
            ScaleUtil_2.scaleToBounds(this.circle, game.width * 0.15, game.width * 0.15);
            this.circleScale = this.circle.scale.clone();
            this.circle.anchor.set(0.5);
            this.circle.inputEnabled = true;
            this.game.add.tween(this.circle).to({ angle: 360 }, 2000, Phaser.Easing.Linear.None, true, 0, Infinity);
            this.arrow = new Phaser.Image(this.game, this.circle.left + this.circle.width * 0.4, this.circle.centerY, "next-arrow");
            ScaleUtil_2.scaleToBounds(this.arrow, this.circle.width * 0.7, this.circle.height * 0.7);
            this.arrow.anchor.set(0.5);
            this.arrow.angle = facingRight ? 0 : 180;
            this.game.add.tween(this.arrow).to({ x: this.circle.right - this.circle.width * 0.4 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.line = new Phaser.Image(this.game, this.circle.centerX, 0, "next-line");
            ScaleUtil_2.scaleToBounds(this.line, game.width * 0.4, game.width);
            this.lineScale = this.line.scale.clone();
            this.line.anchor.set(1, 0.5);
            this.line.angle = facingRight ? 0 : 180;
            facingRight ? this.line.right = this.circle.centerX : this.line.x = this.circle.centerX;
            this.add(this.line);
            this.add(this.circle);
            this.add(this.arrow);
            if (buttonTextKey) {
                this.text = new Phaser.Text(this.game, 0, 0, LocalizationManager_4.LocalizationManager.Instance.text(buttonTextKey), {
                    font: Game_8.Game.DEFAULT_FONT,
                    fill: Game_8.Game.DEFAULT_FONT_COLOUR,
                    fontSize: textSize || Game_8.Game.FONT_SIZE_MEDIUM,
                    stroke: Game_8.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_8.Game.FONT_STROKE_LARGE
                });
                /*
                    HACK HACK HACK
                    We are using setScaleMinMax() to ensure font sizes aren't effected by
                    the scale of parent objects. For some reason, setScaleMinMax() prevents
                    text from being rotated, so we need to allow rotated screens to specify
                    how they want to handle this
                */
                if (!customScale) {
                    this.text.setScaleMinMax(1);
                }
                this.text.anchor.set(1, 0.5);
                const padding = this.game.width * 0.02;
                if (!facingRight) {
                    this.text.left = this.circle.right + padding;
                }
                else {
                    this.text.right = this.circle.left - padding;
                }
                this.add(this.text);
            }
            this.reset();
            this.onChildInputUp.add(() => {
                this.onButtonClickedSignal.dispatch();
                if (!this.ignoreChildInput) {
                    this.game.sound.play("cta-button");
                }
            });
            this.ignoreChildInput = true;
        }
        show(duration = ArrowButton.ANIMATION_DURATION) {
            return __awaiter(this, void 0, void 0, function* () {
                this.reset();
                let isShutdown = false;
                yield Promise.all([
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.text).to({ alpha: 1 }, duration * 0.6, Phaser.Easing.Linear.None, true).chain(this.game.add.tween(this.text).to({ alpha: 0 }, duration * 0.1, Phaser.Easing.Linear.None), this.game.add.tween(this.text).to({ alpha: 1 }, duration * 0.1, Phaser.Easing.Linear.None), this.game.add.tween(this.text).to({ alpha: 0 }, duration * 0.1, Phaser.Easing.Linear.None), this.game.add.tween(this.text).to({ alpha: 1 }, duration * 0.1, Phaser.Easing.Linear.None)), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.arrow).to({ alpha: 1 }, duration * 0.5, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.circle.scale).to({ x: this.circleScale.x, y: this.circleScale.y }, duration, Phaser.Easing.Back.Out, true, 0), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.line.scale).to({ x: this.lineScale.x, y: this.lineScale.y }, duration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                ]).catch(() => { isShutdown = true; });
                this.ignoreChildInput = false;
                return !isShutdown;
            });
        }
        hide(duration = ArrowButton.ANIMATION_DURATION) {
            return __awaiter(this, void 0, void 0, function* () {
                this.ignoreChildInput = true;
                let isShutdown = false;
                yield Promise.all([
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.text).to({ alpha: 0 }, duration * 0.5, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.arrow).to({ alpha: 0 }, duration * 0.5, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.circle.scale).to({ x: 0, y: 0 }, duration, Phaser.Easing.Back.In, true, 0), this.shutdownSignal),
                    PromiseUtil_10.PromiseUtil.tween2(this.game.add.tween(this.line.scale).to({ x: 0, y: 0 }, duration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                ]).catch(() => { isShutdown = true; });
                return !isShutdown;
            });
        }
        reset() {
            this.text.alpha = 0;
            this.arrow.alpha = 0;
            this.circle.scale.set(0);
            this.line.scale.set(0);
        }
        destroy() {
            this.shutdownSignal = null;
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    ArrowButton.ANIMATION_DURATION = 500;
    exports.ArrowButton = ArrowButton;
});
define("component/Logo", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Logo extends Phaser.Group {
        constructor(game) {
            super(game);
            this.logoArrows = new Phaser.Image(this.game, 0, 0, "logo");
            this.logoArrows.width = this.game.width * 0.8;
            this.logoArrows.scale.y = this.logoArrows.scale.x;
            this.logoArrows.anchor.set(0.5);
            this.add(this.logoArrows);
            this.sparkle1 = new Phaser.Image(this.game, this.logoArrows.right - this.logoArrows.width * 0.03, this.logoArrows.top + this.logoArrows.height * 0.2, "sparkle");
            this.sparkle1.width = this.game.width * 0.15;
            this.sparkle1.scale.y = this.sparkle1.scale.x;
            this.sparkle1.anchor.set(0.5);
            this.sparkle1.visible = false;
            this.add(this.sparkle1);
            this.sparkle2 = new Phaser.Image(this.game, this.sparkle1.x, this.logoArrows.bottom - this.logoArrows.height * 0.2, "sparkle");
            this.sparkle2.width = this.game.width * 0.1;
            this.sparkle2.scale.y = this.sparkle2.scale.x;
            this.sparkle2.anchor.set(0.5);
            this.sparkle2.visible = false;
            this.add(this.sparkle2);
            this.logoText = new Phaser.Image(this.game, 0, 0, "logo-text");
            this.logoText.width = this.game.width * 0.8;
            this.logoText.scale.y = this.logoText.scale.x;
            this.logoText.anchor.set(0.5);
            this.add(this.logoText);
            this.visible = false;
        }
        show(shouldAnimateSparkles) {
            return __awaiter(this, void 0, void 0, function* () {
                const origLogoScale = this.logoText.scale.clone();
                this.logoArrows.scale.y = 0;
                this.logoText.scale.set(0);
                this.visible = true;
                this.game.add.tween(this.logoText.scale).to({ x: origLogoScale.x, y: origLogoScale.y }, 350, Phaser.Easing.Back.Out, true);
                yield PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.logoArrows.scale).to({ y: origLogoScale.y }, 350, Phaser.Easing.Back.Out, true, 300));
                if (shouldAnimateSparkles) {
                    this.animateSparkles();
                }
            });
        }
        animateSparkles() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_11.PromiseUtil.time(this.game, 1000);
                this.animateSparkle(this.sparkle1, this.logoArrows.top + this.logoArrows.height * 0.1, 4000);
                this.animateSparkle(this.sparkle2, this.logoArrows.bottom - this.logoArrows.height * 0.1, 3500, 500);
            });
        }
        animateSparkle(sparkle, yTarget, durationMS, delayMS) {
            return __awaiter(this, void 0, void 0, function* () {
                // Save off some values for resetting
                const xTarget = this.logoArrows.left + this.logoArrows.width * 0.03;
                const origPos = sparkle.position.clone();
                const origScale = sparkle.scale.clone();
                // Constant rotation
                this.game.add.tween(sparkle).to({ angle: 360 }, 3000, Phaser.Easing.Linear.None, true, 0, Infinity);
                // Movement
                const movementTween1 = this.game.add.tween(sparkle).to({ x: this.logoArrows.centerX, y: yTarget }, durationMS * 0.5, Phaser.Easing.Sinusoidal.In, false, delayMS);
                const movementTween2 = this.game.add.tween(sparkle).to({ x: xTarget, y: origPos.y }, durationMS * 0.5, Phaser.Easing.Sinusoidal.Out);
                // Scale in / out
                const scaleInTween = this.game.add.tween(sparkle.scale).to({ x: origScale.x, y: origScale.y }, durationMS * 0.25, Phaser.Easing.Linear.None);
                const scaleOutTween = this.game.add.tween(sparkle.scale).to({ x: 0, y: 0 }, durationMS * 0.25, Phaser.Easing.Linear.None, false, durationMS * 0.25);
                // Movement to the tip of the logo is followed 
                // by movement back down the other side
                movementTween1.chain(movementTween2);
                // The first movement tween should also make the thing start scaling up
                movementTween1.onStart.add(() => {
                    scaleInTween.start();
                });
                // And the second one should make it scale down
                movementTween2.onStart.add(() => {
                    scaleOutTween.start();
                });
                // When the second is complete, reset the position and go again!
                movementTween2.onComplete.add(() => {
                    sparkle.position.set(origPos.x, origPos.y);
                    movementTween1.start();
                });
                // Kick everything off here after setting some values
                sparkle.scale.set(0);
                sparkle.visible = true;
                movementTween1.start();
            });
        }
    }
    exports.Logo = Logo;
});
define("intro/IntroScreen1", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "component/Logo", "logic/SMAnalyticsManager", "switchback-common/util/TextUtil"], function (require, exports, Game_9, CutsceneScreen_1, ArrowButton_1, LocalizationManager_5, PromiseUtil_12, Logo_1, SMAnalyticsManager_5, TextUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroScreen1 extends CutsceneScreen_1.CutsceneScreen {
        constructor(game) {
            super(game);
            this.background = new Phaser.Image(game, 0, 0, "background");
            this.background.width = this.game.width;
            this.background.height = this.game.height;
            this.add(this.background);
            this.logo = new Logo_1.Logo(game);
            this.logo.x = this.game.width * 0.5;
            this.logo.y = this.game.height * 0.20;
            this.add(this.logo);
            this.text = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_5.LocalizationManager.Instance.text("intro.instructions"), Object.assign({}, Game_9.Game.DEFAULT_FONT_STYLE, { fontSize: Game_9.Game.FONT_SIZE_LARGE, wordWrap: true, wordWrapWidth: this.game.width * 0.90, align: 'center' }));
            TextUtil_2.TextUtil.scaleDownTextToHeight(this.text, this.game.width * 0.74);
            this.text.anchor.set(0.5);
            this.text.y = this.game.height * 0.56;
            this.add(this.text);
            const playButton = new ArrowButton_1.ArrowButton(game, "buttons.play", true, Game_9.Game.FONT_SIZE_LARGE);
            playButton.width = this.game.width * 0.7;
            playButton.scale.y = playButton.scale.x;
            playButton.x = this.game.width * 0.8;
            playButton.centerY = this.game.height * 0.85;
            this.button = playButton;
            this.add(this.button);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.text.alpha = 0;
                yield PromiseUtil_12.PromiseUtil.time(this.game, 500);
                // Animate in all the main components
                this.logo.show(true);
                this.game.add.tween(this.text).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true, 500);
                yield PromiseUtil_12.PromiseUtil.time(this.game, 500);
                SMAnalyticsManager_5.SMAnalyticsManager.Instance.trackState("content/game/inqueue", "TitleScreen");
                yield this.button.show();
                this.button.onButtonClickedSignal.addOnce(() => {
                    SMAnalyticsManager_5.SMAnalyticsManager.Instance.trackAction("Play", "TitleScreen", "Play");
                    this.onButtonClickedSignal.dispatch();
                });
            });
        }
    }
    exports.IntroScreen1 = IntroScreen1;
});
define("intro/IntroScreen2", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "logic/SMAnalyticsManager", "config/SpaceMountainConfig", "switchback-common/enum/Park"], function (require, exports, Game_10, CutsceneScreen_2, ArrowButton_2, LocalizationManager_6, ScaleUtil_3, PromiseUtil_13, SMAnalyticsManager_6, SpaceMountainConfig_7, Park_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroScreen2 extends CutsceneScreen_2.CutsceneScreen {
        constructor(game) {
            super(game);
            const welcomeTag = "intro.welcome-" + (SpaceMountainConfig_7.SpaceMountainConfig.Instance.park === Park_6.Park.DLR ? "dlr" : "wdw");
            const topText = new Phaser.Text(game, 0, 0, LocalizationManager_6.LocalizationManager.Instance.text(welcomeTag), {
                font: Game_10.Game.DEFAULT_FONT,
                fill: Game_10.Game.FONT_COLOUR_BLUE,
                fontSize: Game_10.Game.FONT_SIZE_LARGE,
                stroke: Game_10.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center'
            });
            topText.anchor.set(0.5);
            topText.top = this.game.height * 0.1;
            this.add(topText);
            const spaceStation = new Phaser.Image(this.game, 0, 0, "space-station");
            ScaleUtil_3.scaleToBounds(spaceStation, this.game.width, this.game.width, true);
            spaceStation.anchor.set(0.5);
            spaceStation.centerY = this.game.height * 0.5;
            this.add(spaceStation);
            if (SpaceMountainConfig_7.SpaceMountainConfig.Instance.park === Park_6.Park.DLR) {
                const bottomText = new Phaser.Text(game, 0, 0, LocalizationManager_6.LocalizationManager.Instance.text("intro.space_station"), {
                    font: Game_10.Game.DEFAULT_FONT,
                    fill: Game_10.Game.DEFAULT_FONT_COLOUR,
                    fontSize: Game_10.Game.FONT_SIZE_LARGE,
                    stroke: Game_10.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: this.game.width * 0.01
                });
                bottomText.anchor.set(0.5);
                bottomText.top = spaceStation.bottom;
                this.add(bottomText);
            }
            const nextButton = new ArrowButton_2.ArrowButton(game, "buttons.next", true, Game_10.Game.FONT_SIZE_LARGE);
            nextButton.onButtonClickedSignal.addOnce(() => {
                this.onButtonClickedSignal.dispatch();
            });
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.3;
            nextButton.bottom = this.game.height * 0.95;
            this.button = nextButton;
            this.add(this.button);
            this.left = this.game.width;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: this.game.width * 0.5 }, 300, Phaser.Easing.Sinusoidal.In, true));
                this.button.show();
                SMAnalyticsManager_6.SMAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene/1_4', 'IntroCutscene1_4', {
                    [SMAnalyticsManager_6.SMAnalyticsManager.CUTSCENE]: "Intro"
                });
            });
        }
    }
    exports.IntroScreen2 = IntroScreen2;
});
define("intro/IntroScreen3", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "logic/SMAnalyticsManager"], function (require, exports, Game_11, CutsceneScreen_3, ArrowButton_3, LocalizationManager_7, ScaleUtil_4, PromiseUtil_14, SMAnalyticsManager_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroScreen3 extends CutsceneScreen_3.CutsceneScreen {
        constructor(game) {
            super(game);
            const hyperion = new Phaser.Image(this.game, 0, 0, 'hyperion');
            ScaleUtil_4.scaleToBounds(hyperion, this.game.width, this.game.width, true);
            hyperion.anchor.set(0.5);
            hyperion.centerY = this.game.height * 0.375;
            this.add(hyperion);
            const topText = new Phaser.Text(this.game, 0, 0, LocalizationManager_7.LocalizationManager.Instance.text("intro.intro_screen_3a"), {
                font: Game_11.Game.DEFAULT_FONT,
                fill: Game_11.Game.FONT_COLOUR_BLUE,
                fontSize: Game_11.Game.FONT_SIZE_LARGE,
                stroke: Game_11.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01
            });
            topText.anchor.set(0.5);
            topText.bottom = hyperion.top;
            this.add(topText);
            const bottomText = new Phaser.Text(this.game, 0, 0, LocalizationManager_7.LocalizationManager.Instance.text("intro.intro_screen_3b"), {
                font: Game_11.Game.DEFAULT_FONT,
                fill: Game_11.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_11.Game.FONT_SIZE_LARGE,
                stroke: Game_11.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            bottomText.anchor.set(0.5);
            bottomText.top = hyperion.bottom;
            this.add(bottomText);
            const nextButton = new ArrowButton_3.ArrowButton(game, "buttons.next", true, Game_11.Game.FONT_SIZE_LARGE);
            nextButton.onButtonClickedSignal.addOnce(() => {
                this.onButtonClickedSignal.dispatch();
            });
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.3;
            nextButton.bottom = this.game.height - this.game.height * 0.05;
            this.button = nextButton;
            this.add(this.button);
            this.left = this.game.width;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: this.game.width * 0.5 }, 300, Phaser.Easing.Sinusoidal.In, true));
                this.button.show();
                SMAnalyticsManager_7.SMAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene/2_4', 'IntroCutscene2_4', {
                    [SMAnalyticsManager_7.SMAnalyticsManager.CUTSCENE]: "Intro"
                });
            });
        }
    }
    exports.IntroScreen3 = IntroScreen3;
});
define("intro/IntroScreen4", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "logic/SMAnalyticsManager"], function (require, exports, Game_12, CutsceneScreen_4, ArrowButton_4, LocalizationManager_8, ScaleUtil_5, PromiseUtil_15, SMAnalyticsManager_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroScreen4 extends CutsceneScreen_4.CutsceneScreen {
        constructor(game) {
            super(game);
            const ship = new Phaser.Image(this.game, 0, 0, "ship-wireframe");
            ScaleUtil_5.scaleToBounds(ship, this.game.width * 0.85, this.game.width * 0.85, true);
            ship.anchor.set(0.5);
            ship.centerY = this.game.height * 0.55;
            this.add(ship);
            const topText = new Phaser.Text(this.game, 0, 0, LocalizationManager_8.LocalizationManager.Instance.text("intro.intro_screen_4a"), {
                font: Game_12.Game.DEFAULT_FONT,
                fill: Game_12.Game.FONT_COLOUR_BLUE,
                fontSize: Game_12.Game.FONT_SIZE_MEDIUM,
                stroke: Game_12.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            topText.anchor.set(0.5);
            topText.bottom = ship.top - this.game.height * 0.05;
            this.add(topText);
            const nextButton = new ArrowButton_4.ArrowButton(game, "buttons.next", true, Game_12.Game.FONT_SIZE_LARGE);
            nextButton.onButtonClickedSignal.addOnce(() => {
                this.onButtonClickedSignal.dispatch();
            });
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.3;
            nextButton.bottom = this.game.height - this.game.height * 0.05;
            this.button = nextButton;
            this.add(this.button);
            // Sits at top left
            const jargonMain = new Phaser.Image(this.game, 0, 0, 'jargon2');
            ScaleUtil_5.scaleToBounds(jargonMain, ship.width * 0.5, ship.width);
            jargonMain.anchor.set(0.5);
            jargonMain.left = ship.x - ship.width * 0.47;
            jargonMain.top = ship.y - ship.height * 0.48;
            this.add(jargonMain);
            // Alien text with lines pointing to various parts of the ship
            this.jargon1 = new Phaser.Image(this.game, 0, 0, 'jargon1');
            ScaleUtil_5.scaleToBounds(this.jargon1, ship.width * 0.5, ship.width);
            this.jargon1.x = this.game.width * -0.4;
            this.jargon1.y = this.game.height * 0.45;
            this.jargon1.alpha = 0;
            this.add(this.jargon1);
            this.jargon2 = new Phaser.Image(this.game, 0, 0, 'jargon1');
            ScaleUtil_5.scaleToBounds(this.jargon2, ship.width * 0.5, ship.width);
            this.jargon2.angle = 180;
            this.jargon2.x = this.game.width * 0.4;
            this.jargon2.y = this.game.height * 0.7;
            this.jargon2.alpha = 0;
            this.add(this.jargon2);
            this.jargon3 = new Phaser.Image(this.game, 0, 0, 'jargon1');
            ScaleUtil_5.scaleToBounds(this.jargon3, ship.width * 0.5, ship.width);
            this.jargon3.scale.y *= -1;
            this.jargon3.x = this.game.width * -0.4;
            this.jargon3.y = this.game.height * 0.77;
            this.jargon3.alpha = 0;
            this.add(this.jargon3);
            this.jargon4 = new Phaser.Image(this.game, 0, 0, 'jargon1');
            ScaleUtil_5.scaleToBounds(this.jargon4, ship.width * 0.5, ship.width);
            this.jargon4.scale.x *= -1;
            this.jargon4.x = this.game.width * 0.4;
            this.jargon4.y = this.game.height * 0.41;
            this.jargon4.alpha = 0;
            this.add(this.jargon4);
            this.left = this.game.width;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: this.game.width * 0.5 }, 300, Phaser.Easing.Sinusoidal.In, true));
                this.button.show();
                // flash jargon 1
                this.game.add.tween(this.jargon1).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 100).chain(this.game.add.tween(this.jargon1).to({ alpha: 0 }, 75, Phaser.Easing.Linear.None, false, 0, 1, true));
                // flash jargon 2
                this.game.add.tween(this.jargon2).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 250).chain(this.game.add.tween(this.jargon2).to({ alpha: 0 }, 75, Phaser.Easing.Linear.None, false, 0, 1, true));
                // flash jargon 3
                this.game.add.tween(this.jargon3).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 450).chain(this.game.add.tween(this.jargon3).to({ alpha: 0 }, 75, Phaser.Easing.Linear.None, false, 0, 1, true));
                // flash jargon 4
                this.game.add.tween(this.jargon4).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 500).chain(this.game.add.tween(this.jargon4).to({ alpha: 0 }, 75, Phaser.Easing.Linear.None, false, 0, 1, true));
                yield this.button.show();
                SMAnalyticsManager_8.SMAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene/3_4', 'IntroCutscene3_4', {
                    [SMAnalyticsManager_8.SMAnalyticsManager.CUTSCENE]: "Intro"
                });
            });
        }
    }
    exports.IntroScreen4 = IntroScreen4;
});
define("component/button/CTAButton", ["require", "exports", "switchback-common/util/ScaleUtil", "Game", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_6, Game_13, PromiseUtil_16) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    class CTAButton extends Phaser.Group {
        constructor(game, buttonText, textSize, customScale, shutdownSignal = null) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.onButtonClickedSignal = new Phaser.Signal();
            this.inputEnableChildren = true;
            this.onChildInputDown.add(this.onClicked, this);
            // create the lightbeam behind the green background
            this.lightBeam = new Phaser.Image(game, 0, 0, "lightbeam");
            ScaleUtil_6.scaleToBounds(this.lightBeam, game.width * 0.8, game.height * 0.15);
            this.lightBeam.anchor.set(0.5);
            this.add(this.lightBeam);
            // create the green backgrond
            this.background = new Phaser.Image(game, 0, 0, "start-button-background");
            ScaleUtil_6.scaleToBounds(this.background, game.width * 0.5, game.height * 0.5);
            this.background.anchor.setTo(0.5);
            this.add(this.background);
            // create the next arrow
            this.nextArrow = new Phaser.Image(game, 0, 0, "next-arrow");
            ScaleUtil_6.scaleToBounds(this.nextArrow, game.width * 0.1, game.height * 0.1);
            this.nextArrow.anchor.setTo(0.5);
            this.add(this.nextArrow);
            // create the button's text
            this.text = new Phaser.Text(this.game, 0, 0, buttonText, {
                font: Game_13.Game.DEFAULT_FONT,
                fill: Game_13.Game.DEFAULT_FONT_COLOUR,
                fontSize: textSize || Game_13.Game.FONT_SIZE_MEDIUM,
                stroke: Game_13.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01
            });
            if (!customScale) {
                this.text.setScaleMinMax(1);
            }
            this.text.anchor.set(1, 0.5);
            this.add(this.text);
            // Resize the background on X to accomodate the text
            const padding = this.game.width * 0.05;
            this.background.width = this.text.width + this.nextArrow.width + (padding * 4);
            this.nextArrow.right = this.background.right - padding;
            this.text.right = this.nextArrow.left;
            this.alpha = 0;
            this.game.add.tween(this.nextArrow).to({ x: this.nextArrow.x + this.game.width * 0.02 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.ignoreChildInput = true;
        }
        reset() {
            this.alpha = 1;
            this.text.alpha = 1;
            this.nextArrow.alpha = 1;
            this.lightBeam.scale.x = this.lightBeam.scale.y;
            this.background.scale.y = this.background.scale.x;
        }
        show(durationMS = 500) {
            return __awaiter(this, void 0, void 0, function* () {
                this.reset();
                this.game.add.tween(this.lightBeam.scale).from({ x: 0 }, durationMS * 0.4, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.background.scale).from({ y: 0 }, durationMS * 0.4, Phaser.Easing.Quintic.In, true, durationMS * 0.2);
                this.game.add.tween(this.text).from({ alpha: 0 }, durationMS * 0.4, Phaser.Easing.Linear.None, true, durationMS * 0.4).chain(this.game.add.tween(this.text).to({ alpha: 0 }, durationMS * 0.1, Phaser.Easing.Linear.None, false, 0, 1, true));
                this.game.add.tween(this.nextArrow).from({ alpha: 0 }, durationMS * 0.4, Phaser.Easing.Linear.None, true, durationMS * 0.6);
                let isShutdown = false;
                // Instead of wrapping each of these in a promise, we can just await the duration.
                yield PromiseUtil_16.PromiseUtil.time(this.game, durationMS, this.shutdownSignal).catch(() => { isShutdown = true; });
                this.ignoreChildInput = false;
                return !isShutdown;
            });
        }
        hide(durationMS = 350) {
            return __awaiter(this, void 0, void 0, function* () {
                this.ignoreChildInput = true;
                this.game.add.tween(this.text).to({ alpha: 0 }, durationMS * 0.4, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.nextArrow).to({ alpha: 0 }, durationMS * 0.4, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.background.scale).to({ y: 0 }, durationMS * 0.6, Phaser.Easing.Quintic.Out, true, durationMS * 0.4);
                this.game.add.tween(this.lightBeam.scale).to({ x: 0 }, durationMS, Phaser.Easing.Linear.None, true);
                let isShutdown = false;
                yield PromiseUtil_16.PromiseUtil.time(this.game, durationMS, this.shutdownSignal).catch(() => { isShutdown = true; });
                return !isShutdown;
            });
        }
        onClicked() {
            if (!this.ignoreChildInput) {
                this.game.sound.play("cta-button");
            }
            this.onButtonClickedSignal.dispatch();
        }
        destroy() {
            this.shutdownSignal = null;
            this.onChildInputDown.remove(this.onClicked, this);
            this.onButtonClickedSignal.dispose();
            this.onButtonClickedSignal = null;
            super.destroy();
        }
    }
    exports.CTAButton = CTAButton;
});
define("component/dialog/TwoTextDialog", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "component/dialog/BasicSpaceDialog", "component/button/CTAButton"], function (require, exports, LocalizationManager_9, Game_14, BasicSpaceDialog_3, CTAButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TwoTextDialog extends BasicSpaceDialog_3.BasicSpaceDialog {
        constructor(game, descriptionText, instructionText, buttonTextKey, shutdownSignal, fontSize, secondFontSize, secondTextAlignLeft = true) {
            super(game, shutdownSignal);
            this.onButtonClickedSignal = new Phaser.Signal();
            const padding = this.game.width * 0.05;
            const dialogWidth = this.game.width * 0.9;
            this.button = new CTAButton_1.CTAButton(this.game, LocalizationManager_9.LocalizationManager.Instance.text(buttonTextKey), null, null, this.shutdownSignal);
            this.button.onButtonClickedSignal.addOnce(() => {
                this.onButtonClickedSignal.dispatch();
            });
            let mainText;
            if (descriptionText) {
                mainText = new Phaser.Text(game, 0, 0, descriptionText, {
                    font: Game_14.Game.DEFAULT_FONT,
                    fill: Game_14.Game.FONT_COLOUR_BLUE,
                    fontSize: fontSize || Game_14.Game.FONT_SIZE_MEDIUM,
                    stroke: Game_14.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_14.Game.FONT_STROKE_SMALL,
                    wordWrap: true,
                    wordWrapWidth: dialogWidth - (padding * 2),
                    align: 'center'
                });
                mainText.anchor.set(0.5, 0);
                mainText.top = padding;
                mainText.setScaleMinMax(1);
                this.content.add(mainText);
            }
            const textWidth = dialogWidth - padding * 2;
            const subText = new Phaser.Text(game, 0, 0, instructionText, {
                font: Game_14.Game.DEFAULT_FONT,
                fill: Game_14.Game.DEFAULT_FONT_COLOUR,
                fontSize: secondFontSize || fontSize || Game_14.Game.FONT_SIZE_MEDIUM,
                stroke: Game_14.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_14.Game.FONT_STROKE_SMALL,
                wordWrap: true,
                wordWrapWidth: textWidth,
                align: secondTextAlignLeft ? 'left' : 'center'
            });
            subText.setScaleMinMax(1);
            subText.anchor.x = 0.5;
            if (mainText) {
                subText.top = mainText.bottom + padding;
            }
            else {
                subText.top = padding;
            }
            this.content.add(subText);
            this.button.top = subText.bottom + padding;
            this.content.add(this.button);
            this.resizeBackground();
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                this.button.show();
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                let isShutdown = !(yield this.button.hide());
                if (isShutdown) {
                    return;
                }
                yield _super("hide").call(this);
            });
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.TwoTextDialog = TwoTextDialog;
});
define("intro/IntroScreenQueueSelection", ["require", "exports", "Game", "component/CutsceneScreen", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "component/dialog/ConfirmDialog", "model/SpaceSaveState", "logic/GameController", "component/button/ArrowButton", "logic/SMAnalyticsManager", "component/dialog/TwoTextDialog"], function (require, exports, Game_15, CutsceneScreen_5, LocalizationManager_10, ScaleUtil_7, PromiseUtil_17, ConfirmDialog_2, SpaceSaveState_4, GameController_10, ArrowButton_5, SMAnalyticsManager_9, TwoTextDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /*
     * @author ellie@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class IntroScreenQueueSelection extends CutsceneScreen_5.CutsceneScreen {
        constructor(game) {
            super(game);
            const centerX = this.game.width * 0.5;
            const text = LocalizationManager_10.LocalizationManager.Instance.text("intro.queue_title");
            const style = Object.assign({}, Game_15.Game.FONT_STYLE_TITLE, { wordWrap: true, wordWrapWidth: this.game.width * 0.9, align: 'center' });
            const topText = new Phaser.Text(this.game, centerX, this.game.height * 0.1, text, style);
            topText.anchor.set(0.5);
            this.add(topText);
            /// Queue panels
            const queuesY = [this.game.height * 0.39, this.game.height * 0.77];
            const queuesKey = ['queue-1', 'queue-2'];
            this.queues = [];
            this.groups = [];
            this.buttons = [];
            for (let i = 0; i < queuesY.length; i++) {
                const group = new Phaser.Group(this.game, this);
                const queueImage = new Phaser.Image(this.game, 0, 0, queuesKey[i]);
                queueImage.anchor.set(0.5);
                queueImage.inputEnabled = true;
                ScaleUtil_7.scaleToBounds(queueImage, this.game.width * 1, this.game.height * 0.75);
                const button = new ArrowButton_5.ArrowButton(this.game, "intro." + queuesKey[i], true, Game_15.Game.FONT_SIZE_MEDIUM, true);
                ScaleUtil_7.scaleToBounds(button, queueImage.width * 0.8, queueImage.height * 0.15);
                button.bottom = queueImage.bottom - queueImage.height * 0.18;
                button.right = queueImage.right - queueImage.width * 0.16;
                group.addChild(queueImage);
                group.addChild(button);
                // set the offset on the group so the animation looks good
                group.x = centerX;
                group.y = queuesY[i];
                this.queues.push(queueImage);
                this.buttons.push(button);
                this.groups.push(group);
            }
            this.resetQueuePanels();
            // ~ Queue Panels
            this.confirmDialog = new ConfirmDialog_2.ConfirmDialog(this.game, LocalizationManager_10.LocalizationManager.Instance.text('intro.queue_dialog'), LocalizationManager_10.LocalizationManager.Instance.text('intro.queue_yes').toUpperCase(), LocalizationManager_10.LocalizationManager.Instance.text('intro.queue_no').toUpperCase(), null);
            this.confirmDialog.x = 0;
            this.confirmDialog.visible = false;
            this.add(this.confirmDialog);
            this.left = this.game.width;
            this.watchTheStairsDialog = new TwoTextDialog_1.TwoTextDialog(this.game, LocalizationManager_10.LocalizationManager.Instance.text("watch_the_stairs.title"), LocalizationManager_10.LocalizationManager.Instance.text("watch_the_stairs.body"), "buttons.continue", null);
            this.watchTheStairsDialog.x = 0;
            this.watchTheStairsDialog.visible = false;
            this.add(this.watchTheStairsDialog);
        }
        // Resets animations, positions and event listeners for both queue panels
        resetQueuePanels() {
            // better safe than sorry (in case show gets called twice)
            for (let queueIndex in this.queues) {
                let queue = this.queues[queueIndex];
                queue.events.onInputDown.removeAll();
                let group = this.groups[queueIndex];
                this.game.tweens.removeFrom(group.scale);
                group.scale.x = group.scale.y = 1;
            }
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                SMAnalyticsManager_9.SMAnalyticsManager.Instance.trackState("content/game/inqueue/declare", "QueueSelect");
                // Bind events
                // only bind the inputs after the animation stops
                this.resetQueuePanels();
                this.queues[0].events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.onInput(SpaceSaveState_4.Queue.PRIMARY);
                }));
                this.queues[1].events.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.onInput(SpaceSaveState_4.Queue.SECONDARY);
                }));
                let promises = [];
                for (let button of this.buttons) {
                    promises.push(button.show());
                }
                yield Promise.all(promises);
                for (let button of this.buttons) {
                    button.inputEnableChildren = false;
                    button.ignoreChildInput = true;
                }
                // Animations
                // Jed suggested: 2s animation loop; scale to 97% over 1s, then back to 100% over 1s
                for (let group of this.groups) {
                    let tween = this.game.add.tween(group.scale);
                    tween.to({ x: group.scale.x * 0.97, y: group.scale.y * 0.97 }, 2000, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                }
            });
        }
        onInput(queueType) {
            return __awaiter(this, void 0, void 0, function* () {
                let gaLabel = queueType === SpaceSaveState_4.Queue.PRIMARY ? "QueueA" : "QueueB";
                let gaAction = gaLabel;
                let adobeAction = queueType === SpaceSaveState_4.Queue.PRIMARY ? "Declare_LineA" : "Declare_LineB";
                SMAnalyticsManager_9.SMAnalyticsManager.Instance.trackAction(adobeAction, "QueueSelect", gaAction);
                yield this.confirmDialog.show();
                const confirmed = yield PromiseUtil_17.PromiseUtil.signal(this.confirmDialog.onButtonClickedSignal);
                yield this.confirmDialog.hide();
                gaAction = confirmed ? "Yes" : "No";
                adobeAction = confirmed ? "Declare_Yes" : "Declare_No";
                SMAnalyticsManager_9.SMAnalyticsManager.Instance.trackAction(adobeAction, "QueueSelect", gaAction, gaLabel);
                if (confirmed) {
                    if (queueType === SpaceSaveState_4.Queue.PRIMARY) {
                        yield this.watchTheStairsDialog.show();
                        yield PromiseUtil_17.PromiseUtil.signal(this.watchTheStairsDialog.onButtonClickedSignal);
                        yield this.watchTheStairsDialog.hide();
                    }
                    // Stop listening
                    this.resetQueuePanels();
                    GameController_10.GameController.Instance.state.queueType = queueType;
                    GameController_10.GameController.Instance.setAnalyticsQueueType(queueType);
                    // And resolve, they've finished 'answering' this page
                    this.onButtonClickedSignal.dispatch();
                }
            });
        }
    }
    exports.IntroScreenQueueSelection = IntroScreenQueueSelection;
});
define("intro/IntroScreen5", ["require", "exports", "Game", "component/CutsceneScreen", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "component/button/CTAButton", "logic/SMAnalyticsManager"], function (require, exports, Game_16, CutsceneScreen_6, LocalizationManager_11, ScaleUtil_8, PromiseUtil_18, CTAButton_2, SMAnalyticsManager_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroScreen5 extends CutsceneScreen_6.CutsceneScreen {
        constructor(game) {
            super(game);
            const ship = new Phaser.Image(this.game, 0, 0, "race-preview");
            ScaleUtil_8.scaleToBounds(ship, this.game.width * 0.98, this.game.width * 0.98, true);
            ship.anchor.set(0.5);
            ship.centerY = this.game.height * 0.5;
            this.add(ship);
            const topText = new Phaser.Text(this.game, 0, 0, LocalizationManager_11.LocalizationManager.Instance.text("intro.intro_screen_5a"), {
                font: Game_16.Game.DEFAULT_FONT,
                fill: Game_16.Game.FONT_COLOUR_BLUE,
                fontSize: Game_16.Game.FONT_SIZE_MEDIUM,
                stroke: Game_16.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            topText.anchor.set(0.5);
            topText.bottom = ship.top - this.game.height * 0.05;
            this.add(topText);
            const startButton = new CTAButton_2.CTAButton(game, LocalizationManager_11.LocalizationManager.Instance.text("buttons.start"), Game_16.Game.FONT_SIZE_LARGE);
            startButton.width = this.game.width * 0.7;
            startButton.scale.y = startButton.scale.x;
            startButton.x = 0;
            startButton.bottom = this.game.height - this.game.height * 0.1;
            this.button = startButton;
            this.add(this.button);
            this.left = this.game.width;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this).from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: this.game.width * 0.5 }, 300, Phaser.Easing.Sinusoidal.In, true));
                this.button.show();
                yield this.button.show();
                SMAnalyticsManager_10.SMAnalyticsManager.Instance.trackState('content/game/inqueue/cutscene/4_4', 'IntroCutscene4_4', {
                    [SMAnalyticsManager_10.SMAnalyticsManager.CUTSCENE]: "Intro"
                });
                this.button.onButtonClickedSignal.addOnce(() => {
                    this.game.sound.play('intro-end');
                    this.onButtonClickedSignal.dispatch();
                });
            });
        }
    }
    exports.IntroScreen5 = IntroScreen5;
});
define("state/IntroState", ["require", "exports", "logic/GameController", "StateNames", "state/SpaceBaseState", "intro/IntroScreen1", "intro/IntroScreen2", "intro/IntroScreen3", "intro/IntroScreen4", "switchback-common/util/PromiseUtil", "intro/IntroScreenQueueSelection", "switchback-common/enum/Park", "config/SpaceMountainConfig", "intro/IntroScreen5"], function (require, exports, GameController_11, StateNames_6, SpaceBaseState_1, IntroScreen1_1, IntroScreen2_1, IntroScreen3_1, IntroScreen4_1, PromiseUtil_19, IntroScreenQueueSelection_1, Park_7, SpaceMountainConfig_8, IntroScreen5_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ellie@cerebralfix.com
     */
    class IntroState extends SpaceBaseState_1.SpaceBaseState {
        init() {
            this.stateName = StateNames_6.StateNames.INTRO;
            super.init();
        }
        preload() {
            super.preload();
            this.game.load.audio("intro-start", ["assets/sound/ui/intro-start.m4a", "assets/sound/ui/intro-start.ogg"]);
            this.game.load.audio("intro-end", ["assets/sound/ui/intro-end.m4a", "assets/sound/ui/intro-end.ogg"]);
            this.game.load.image("galaxy-bg", "assets/cutscene/galaxy_bg.png");
            this.game.load.image("hyperion", "assets/cutscene/hyperion.png");
            this.game.load.image("jargon1", "assets/cutscene/jargon1.png");
            this.game.load.image("jargon2", "assets/cutscene/jargon2.png");
            this.game.load.image("line", "assets/cutscene/line.png");
            this.game.load.image("ship-wireframe", "assets/cutscene/ship_wireframe.png");
            this.game.load.image("space-station", "assets/cutscene/spacestation.png");
            this.game.load.image("race-preview", "assets/cutscene/race_preview.png");
            // Queue selection
            this.game.load.image("queue-1", "assets/queues/wdw-queue-1.png");
            this.game.load.image("queue-2", "assets/queues/wdw-queue-2.png");
            this.game.load.image("back-button-small", "assets/button/back_button_small.png");
            this.game.load.image("confirm-button-small", "assets/button/confirm_button_small.png");
            // ~ end queue selection
            this.game.load.spritesheet('probe', 'assets/probes/probes.png', 412, 406);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const background = new Phaser.Image(this.game, 0, 0, "galaxy-bg");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                let screens = [];
                if (!SpaceMountainConfig_8.SpaceMountainConfig.Instance.skipTutorials) {
                    screens.push(new IntroScreen1_1.IntroScreen1(this.game));
                }
                if (GameController_11.GameController.Instance.state.queueType === undefined &&
                    SpaceMountainConfig_8.SpaceMountainConfig.Instance.park === Park_7.Park.WDW) {
                    screens.push(new IntroScreenQueueSelection_1.IntroScreenQueueSelection(this.game));
                }
                if (!SpaceMountainConfig_8.SpaceMountainConfig.Instance.skipTutorials) {
                    screens.push(new IntroScreen2_1.IntroScreen2(this.game));
                    screens.push(new IntroScreen3_1.IntroScreen3(this.game));
                    screens.push(new IntroScreen4_1.IntroScreen4(this.game));
                    screens.push(new IntroScreen5_1.IntroScreen5(this.game));
                }
                this.dismissOverlay();
                this.game.sound.play('intro-start');
                for (let screen of screens) {
                    this.add.existing(screen);
                    yield screen.show();
                    yield PromiseUtil_19.PromiseUtil.signal(screen.onButtonClickedSignal);
                    yield screen.hide();
                    screen.destroy();
                }
                GameController_11.GameController.Instance.state.ftueCompleted = true;
                console.log("Intro finished, queue type is", GameController_11.GameController.Instance.state.queueType);
                this.transitionToGame();
            });
        }
        transitionToGame() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToGame()`);
                yield this.showOverlay();
                this.game.state.start(StateNames_6.StateNames.HUB);
            });
        }
        shutdown() {
            this.game.cache.removeImage("galaxy-bg");
            this.game.cache.removeImage("hyperion");
            this.game.cache.removeImage("jargon1");
            this.game.cache.removeImage("jargon2");
            this.game.cache.removeImage("line");
            this.game.cache.removeImage("ship-wireframe");
            this.game.cache.removeImage("space-station");
            this.game.cache.removeImage("race-preview");
            this.game.cache.removeImage("queue-1");
            this.game.cache.removeImage("queue-2");
            this.game.cache.removeImage("back-button-small");
            this.game.cache.removeImage("confirm-button-small");
            this.game.cache.removeImage('probe');
            super.shutdown();
        }
    }
    exports.IntroState = IntroState;
});
define("component/coderedeem/CodeSymbolInputField", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var InputState;
    (function (InputState) {
        InputState[InputState["AWAITING_INPUT"] = 0] = "AWAITING_INPUT";
        InputState[InputState["CORRECT"] = 1] = "CORRECT";
        InputState[InputState["INCORRECT"] = 2] = "INCORRECT";
    })(InputState = exports.InputState || (exports.InputState = {}));
    class CodeSymbolInputField extends Phaser.Group {
        constructor(game) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.button = new Phaser.Button(game, 0, 0, "cursor-symbol", () => {
                this.onButtonClickedSignal.dispatch();
            }, this);
            this.button.anchor.set(0.5);
            this.add(this.button);
            this.symbolImage = new Phaser.Image(game, 0, 0, "input-symbols-flat");
            this.symbolImage.anchor.set(0.5);
            this.add(this.symbolImage);
            // Default state to awaiting input
            // This will get updated on each input event
            this.state = InputState.AWAITING_INPUT;
        }
        set symbol(symbol) {
            this._symbol = symbol;
            this.symbolImage.frame = this._symbol;
            if (this._symbol === -1) {
                this.symbolImage.visible = false;
                this.button.visible = true;
            }
            else {
                this.symbolImage.visible = true;
                this.button.visible = false;
            }
        }
        set state(state) {
            this._state = state;
            switch (this._state) {
                case (InputState.AWAITING_INPUT): {
                    // Tint symbols blue
                    this.button.tint = 0x5ef9fc;
                    this.symbolImage.tint = 0x5ef9fc;
                    break;
                }
                case (InputState.CORRECT): {
                    // Tint symbols green
                    this.button.tint = 0x50d473;
                    this.symbolImage.tint = 0x50d473;
                    break;
                }
                case (InputState.INCORRECT): {
                    // Tint symbols red
                    this.button.tint = 0xb52b3d;
                    this.symbolImage.tint = 0xb52b3d;
                    break;
                }
            }
        }
        destroy() {
            if (this.onButtonClickedSignal) {
                this.onButtonClickedSignal.dispose();
            }
            super.destroy();
        }
    }
    exports.CodeSymbolInputField = CodeSymbolInputField;
});
define("component/coderedeem/CodeSymbolButton", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeSymbolButton extends Phaser.Group {
        constructor(game, symbol) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            const button = new Phaser.Button(game, 0, 0, "input-symbols", () => {
                this.onButtonClickedSignal.dispatch(symbol);
            }, symbol, symbol, symbol);
            button.anchor.set(0.5);
            this.add(button);
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.CodeSymbolButton = CodeSymbolButton;
});
define("component/coderedeem/CodeRedeemInputComponent", ["require", "exports", "Game", "component/coderedeem/CodeSymbolInputField", "component/coderedeem/CodeSymbolButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "component/coderedeem/RedeemCodeDialog"], function (require, exports, Game_17, CodeSymbolInputField_1, CodeSymbolButton_1, LocalizationManager_12, PromiseUtil_20, RedeemCodeDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeRedeemInputComponent extends Phaser.Group {
        constructor(game) {
            super(game);
            this.attemptRedeemSignal = new Phaser.Signal();
            this.inputSymbolSequence = [];
            this.inputSymbolFields = [];
            this.symbolButtons = new Phaser.Group(this.game);
            this.inputSymbolDisplay = new Phaser.Group(this.game);
            const largeDialog = new Phaser.Image(this.game, 0, 0, "large-dialog");
            largeDialog.width = this.game.width * 0.9;
            largeDialog.height = this.game.height * 0.6;
            this.add(largeDialog);
            // Black area displaying input
            const codeEntryBackdrop = new Phaser.Image(this.game, 0, 0, "code-backdrop");
            codeEntryBackdrop.inputEnabled = true;
            codeEntryBackdrop.events.onInputDown.add(this.onInputDisplayTapped, this);
            codeEntryBackdrop.width = this.width * 0.9;
            codeEntryBackdrop.height = this.height * 0.25;
            codeEntryBackdrop.centerX = this.width * 0.5;
            codeEntryBackdrop.centerY = this.height * 0.2;
            this.add(codeEntryBackdrop);
            // Defines the horizontal location of buttons
            const rowWidth = (this.width * 0.8) * 0.25;
            // Inputted symbols
            for (let i = 0; i < RedeemCodeDialog_1.RedeemCodeDialog.CODE_LENGTH; i++) {
                let inputField = new CodeSymbolInputField_1.CodeSymbolInputField(game);
                inputField.onButtonClickedSignal.add(this.onInputDisplayTapped, this);
                inputField.symbol = -1;
                inputField.centerX = rowWidth * (i + 1);
                inputField.width = this.game.width * 0.15;
                inputField.height = inputField.width;
                this.inputSymbolDisplay.add(inputField);
                this.inputSymbolFields.push(inputField);
            }
            this.inputSymbolDisplay.centerY = this.height * 0.2;
            this.add(this.inputSymbolDisplay);
            // Subheading
            const textLightbeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
            textLightbeam.width = this.width * 0.9;
            textLightbeam.height = this.height * 0.05;
            textLightbeam.centerX = this.width * 0.5;
            textLightbeam.bottom = codeEntryBackdrop.top + (textLightbeam.height * 0.2);
            this.add(textLightbeam);
            const codeEntryText = new Phaser.Text(game, 0, 0, LocalizationManager_12.LocalizationManager.Instance.text("code_redeem.enter_code"), {
                fill: '#ffffff',
                font: Game_17.Game.DEFAULT_FONT,
                fontSize: Game_17.Game.FONT_SIZE_SMALL,
                stroke: Game_17.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            codeEntryText.setScaleMinMax(1);
            codeEntryText.centerX = this.width * 0.5;
            codeEntryText.centerY = textLightbeam.centerY;
            this.add(codeEntryText);
            // Symbol entry keyboard
            let symbolIndex = 0;
            for (let i = 0; i < 3; i++) {
                for (let j = 0; j < 4; j++) {
                    const input = new CodeSymbolButton_1.CodeSymbolButton(this.game, symbolIndex);
                    input.onButtonClickedSignal.add(this.onSymbolButtonClicked, this);
                    input.centerX = rowWidth * j;
                    input.centerY = rowWidth * i;
                    input.width = this.game.width * 0.15;
                    input.height = input.width;
                    this.symbolButtons.add(input);
                    symbolIndex++;
                }
            }
            this.symbolButtons.centerX = this.width * 0.5;
            this.symbolButtons.centerY = this.height * 0.65;
            this.add(this.symbolButtons);
        }
        displayResultAnimation(correct) {
            return __awaiter(this, void 0, void 0, function* () {
                // Change the inputted symbols to red or green based on
                // whether the code entered is correct or not
                if (correct) {
                    this.updateAllInputFieldStates(CodeSymbolInputField_1.InputState.CORRECT);
                }
                else {
                    this.updateAllInputFieldStates(CodeSymbolInputField_1.InputState.INCORRECT);
                }
                // Flash the inputted symbols
                const flashingTween = this.game.add.tween(this.inputSymbolDisplay).to({ alpha: 0 }, 100, Phaser.Easing.Linear.None, true, 0, 2, true);
                yield PromiseUtil_20.PromiseUtil.tween2(flashingTween);
            });
        }
        clearInputSequence() {
            // Clear inputted symbols, next input will start a new sequence
            this.inputSymbolSequence = [];
            this.updateInputDisplay();
        }
        onInputDisplayTapped() {
            // Regardless of where on the input the user taps,
            // clear the last (rightmost) symbol (tap to backspace)
            this.inputSymbolSequence.pop();
            this.updateInputDisplay();
        }
        updateAllInputFieldStates(state) {
            // Set the state of all fields as true/false
            this.inputSymbolFields.forEach((field) => {
                field.state = state;
            });
        }
        updateInputDisplay() {
            // Update each input based on values selected by user
            for (let i = 0; i < RedeemCodeDialog_1.RedeemCodeDialog.CODE_LENGTH; i++) {
                let inputField = this.inputSymbolFields[i];
                if (this.inputSymbolSequence[i] !== undefined) {
                    inputField.symbol = this.inputSymbolSequence[i];
                }
                else {
                    inputField.symbol = -1;
                }
            }
            this.updateAllInputFieldStates(CodeSymbolInputField_1.InputState.AWAITING_INPUT);
        }
        onSymbolButtonClicked(symbolIndex) {
            this.inputSymbolSequence.push(symbolIndex);
            this.updateInputDisplay();
            if (this.inputSymbolSequence.length === RedeemCodeDialog_1.RedeemCodeDialog.CODE_LENGTH) {
                // Attempt to redeem code
                this.attemptRedeemSignal.dispatch(this.inputSymbolSequence);
            }
        }
        destroy() {
            this.attemptRedeemSignal.dispose();
            super.destroy();
        }
    }
    exports.CodeRedeemInputComponent = CodeRedeemInputComponent;
});
define("component/coderedeem/RedeemCodeSuccessDialog", ["require", "exports", "switchback-common/util/ScaleUtil", "Game", "component/dialog/BasicSpaceDialog", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager"], function (require, exports, ScaleUtil_9, Game_18, BasicSpaceDialog_4, ArrowButton_6, LocalizationManager_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RedeemCodeSuccessDialog extends BasicSpaceDialog_4.BasicSpaceDialog {
        constructor(game, found, total, shutdownSignal) {
            super(game, shutdownSignal, "");
            this.inputEnableChildren = true;
            this.onButtonClickedSignal = new Phaser.Signal();
            const successText = new Phaser.Text(this.game, 0, 0, LocalizationManager_13.LocalizationManager.Instance.text("code_redeem.success"), {
                font: Game_18.Game.DEFAULT_FONT,
                fill: Game_18.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_18.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            successText.setScaleMinMax(1);
            const infoText = new Phaser.Text(this.game, 0, 0, LocalizationManager_13.LocalizationManager.Instance.text("code_redeem.you_have_gained_part"), {
                font: Game_18.Game.DEFAULT_FONT,
                fill: Game_18.Game.FONT_COLOUR_BLUE,
                fontSize: Game_18.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                stroke: 'black',
                strokeThickness: this.game.width * 0.015
            });
            infoText.setScaleMinMax(1);
            const tripleLightbeams = new Phaser.Image(this.game, 0, 0, "lightbeams");
            ScaleUtil_9.scaleToBounds(tripleLightbeams, this.game.width, this.game.height);
            tripleLightbeams.top = 0;
            tripleLightbeams.anchor.set(0.5, 0);
            this.content.add(tripleLightbeams);
            successText.anchor.set(0.5, 0);
            successText.centerY = tripleLightbeams.centerY;
            this.content.add(successText);
            infoText.top = tripleLightbeams.bottom;
            infoText.anchor.set(0.5, 0);
            this.content.add(infoText);
            const launchImage = new Phaser.Image(this.game, 0, 0, "boost-symbol");
            ScaleUtil_9.scaleToBounds(launchImage, this.game.width * 0.5, this.game.height * 0.25);
            launchImage.top = infoText.bottom;
            launchImage.anchor.set(0.5, 0);
            this.content.add(launchImage);
            const progressText = new Phaser.Text(this.game, 0, 0, `${found} / ${total}`, {
                font: Game_18.Game.DEFAULT_FONT,
                fill: Game_18.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_18.Game.FONT_SIZE_LARGE,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            progressText.setScaleMinMax(1);
            progressText.anchor.set(0.5, 0);
            progressText.top = launchImage.bottom;
            this.content.add(progressText);
            const nextButton = new ArrowButton_6.ArrowButton(this.game, "buttons.next", true, 70);
            nextButton.inputEnableChildren = true;
            nextButton.onButtonClickedSignal.add(() => {
                this.onButtonClickedSignal.dispatch();
            }, this);
            ScaleUtil_9.scaleToBounds(nextButton, this.game.width * 0.5, this.game.height * 0.15);
            nextButton.top = progressText.bottom + (this.game.height * 0.02);
            nextButton.centerX = 0;
            this.content.add(nextButton);
            nextButton.show();
            this.resizeBackground();
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.RedeemCodeSuccessDialog = RedeemCodeSuccessDialog;
});
define("component/coderedeem/CodeRedeemDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil"], function (require, exports, PromiseUtil_21, Game_19, LocalizationManager_14, TextUtil_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeRedeemDialog extends Phaser.Group {
        constructor(game, titleTextKey, subTitleTextKey, numberOfMidDialogues) {
            super(game);
            this.inputEnableChildren = true;
            // top dialog
            const dialogTop = new Phaser.Image(this.game, 0, 0, 'dialog-top');
            dialogTop.width = this.game.world.width * 0.9;
            dialogTop.scale.y = dialogTop.scale.x;
            dialogTop.x = 0;
            dialogTop.y = 0;
            this.add(dialogTop);
            const dialogMiddle = new Phaser.Image(this.game, 0, 0, 'dialog-mid');
            dialogMiddle.width = this.game.world.width * 0.9;
            dialogMiddle.scale.y = dialogMiddle.scale.x;
            dialogMiddle.x = 0;
            dialogMiddle.y = this.height;
            this.add(dialogMiddle);
            this.middleDialogues = [];
            this.middleDialogues.push(dialogMiddle); // add the first mid dialog to the array of mid dialogues
            // add more middle dialogs if we need to make it bigger
            if (numberOfMidDialogues > 1) {
                for (let i = 0; i < numberOfMidDialogues; i++) {
                    const md = new Phaser.Image(this.game, 0, 0, 'dialog-mid');
                    md.width = this.game.world.width * 0.9;
                    md.scale.y = md.scale.x;
                    md.x = 0;
                    md.y = this.height;
                    this.middleDialogues.push(md);
                    this.add(md);
                }
            }
            // bottom dialog
            const dialogBottom = new Phaser.Image(this.game, 0, 0, 'dialog-bot');
            dialogBottom.width = this.game.world.width * 0.9;
            dialogBottom.scale.y = dialogBottom.scale.x;
            dialogBottom.x = 0;
            dialogBottom.y = this.height;
            this.add(dialogBottom);
            const title = new Phaser.Text(this.game, 0, 0, LocalizationManager_14.LocalizationManager.Instance.text(titleTextKey), {
                fill: '#ffffff',
                font: Game_19.Game.DEFAULT_FONT,
                fontSize: TextUtil_3.TextUtil.getFontSize(this.game.width, 25),
                wordWrap: true,
                wordWrapWidth: this.game.world.width * 0.75,
                align: 'center'
            });
            title.centerX = dialogMiddle.centerX;
            title.top = dialogMiddle.top + (dialogMiddle.height * 0.15);
            this.add(title);
            const subTitle = new Phaser.Text(this.game, 0, 0, LocalizationManager_14.LocalizationManager.Instance.text(subTitleTextKey), {
                fill: '#F00',
                font: Game_19.Game.DEFAULT_FONT,
                fontSize: TextUtil_3.TextUtil.getFontSize(this.game.width, 15),
                wordWrap: true,
                wordWrapWidth: this.game.world.width * 0.75,
                align: 'left'
            });
            subTitle.centerX = dialogMiddle.centerX;
            subTitle.centerY = this.middleDialogues[this.middleDialogues.length - 1].bottom;
            this.add(subTitle);
            this.pivot.set(this.width / 2, this.height / 2);
        }
        show() {
            this.visible = true;
            // scale the whole dialog 
            this.scale.y = 0.3;
            this.scale.x = 3;
            this.centerX = this.game.world.centerX;
            this.centerY = this.game.world.height * 0.4;
            // animate dialog in
            this.game.add.tween(this.scale).to({ x: 1 }, 250, Phaser.Easing.Quadratic.In, true);
            this.game.add.tween(this.scale).to({ y: 1 }, 250, Phaser.Easing.Quadratic.In, true, 50);
        }
        hide(shutdownSignal) {
            return __awaiter(this, void 0, void 0, function* () {
                // animate dialog out
                this.game.add.tween(this.scale).to({ y: 0.3 }, 250, Phaser.Easing.Exponential.In, true);
                yield PromiseUtil_21.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 2 }, 250, Phaser.Easing.Exponential.In, true, 50), shutdownSignal).catch(() => { });
                this.game.tweens.removeFrom(this.children);
                this.visible = false;
            });
        }
    }
    exports.CodeRedeemDialog = CodeRedeemDialog;
});
define("component/coderedeem/RedeemCodeDialog", ["require", "exports", "component/coderedeem/CodeRedeemInputComponent", "component/coderedeem/RedeemCodeSuccessDialog", "component/coderedeem/CodeRedeemDialog", "switchback-common/localization/LocalizationManager", "Game", "switchback-common/util/TextUtil", "component/button/ArrowButton", "logic/GameController", "logic/SMAnalyticsManager", "switchback-common/util/PromiseUtil"], function (require, exports, CodeRedeemInputComponent_1, RedeemCodeSuccessDialog_1, CodeRedeemDialog_1, LocalizationManager_15, Game_20, TextUtil_4, ArrowButton_7, GameController_12, SMAnalyticsManager_11, PromiseUtil_22) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RedeemCodeDialog extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.isShutdown = false;
            this.shutdownSignal = shutdownSignal;
            this.successDialog = null;
            this.parseCodeJson();
            const background = new Phaser.Image(this.game, 0, 0, "redeem-background");
            background.width = this.game.width;
            background.height = this.game.height;
            this.add(background);
            background.inputEnabled = true;
            // Title text
            const title = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.06, LocalizationManager_15.LocalizationManager.Instance.text("code_redeem.title"), {
                fill: Game_20.Game.DEFAULT_FONT_COLOUR,
                font: Game_20.Game.DEFAULT_FONT,
                fontSize: Game_20.Game.FONT_SIZE_LARGE,
                stroke: Game_20.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center'
            });
            title.setScaleMinMax(1);
            title.anchor.set(0.5);
            const lightBeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
            lightBeam.width = this.game.width * 0.9;
            lightBeam.height = title.height * 0.8;
            lightBeam.centerX = title.centerX;
            lightBeam.centerY = title.centerY;
            this.add(lightBeam);
            this.add(title);
            const descText = new Phaser.Text(this.game, this.game.width * 0.5, title.bottom, LocalizationManager_15.LocalizationManager.Instance.text("code_redeem.instructions"), {
                fill: Game_20.Game.FONT_COLOUR_BLUE,
                font: Game_20.Game.DEFAULT_FONT,
                fontSize: Game_20.Game.FONT_SIZE_MEDIUM,
                stroke: Game_20.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            descText.setScaleMinMax(1);
            descText.anchor.set(0.5, 0);
            this.add(descText);
            // Main input UI
            this.inputArea = new CodeRedeemInputComponent_1.CodeRedeemInputComponent(this.game);
            this.inputArea.attemptRedeemSignal.add(this.attemptRedemption, this);
            this.inputArea.centerX = this.game.width * 0.5;
            this.inputArea.centerY = this.game.height * 0.55;
            this.add(this.inputArea);
            TextUtil_4.TextUtil.scaleDownTextToHeight(descText, this.inputArea.top - title.bottom);
            descText.centerY = title.bottom + (this.inputArea.top - title.bottom) * 0.5;
            // Back button
            this.backButton = new ArrowButton_7.ArrowButton(this.game, "code_redeem.back", false);
            this.backButton.left = this.game.width * 0.05;
            this.backButton.centerY = this.game.height * 0.9;
            this.backButton.onChildInputDown.addOnce(() => {
                this.backButton.hide();
                this.onBack();
            }, this);
            this.add(this.backButton);
            this.codeProgressText = new Phaser.Text(this.game, 0, 0, '', {
                fill: Game_20.Game.DEFAULT_FONT_COLOUR,
                font: Game_20.Game.DEFAULT_FONT,
                fontSize: Game_20.Game.FONT_SIZE_MEDIUM,
                stroke: Game_20.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
            });
            this.codeProgressText.right = this.game.width * 0.9;
            this.codeProgressText.centerY = this.game.height * 0.9;
            this.updateCodeProgressText(this.codeProgressText);
            this.add(this.codeProgressText);
            // create the already redeemed dialogue
            this.alreadyRedeemedDialog = new CodeRedeemDialog_1.CodeRedeemDialog(this.game, "code_redeem.already_redeemed", "code_redeem.tap_to_close", 4);
            this.alreadyRedeemedDialog.visible = false;
        }
        static preload(game) {
            game.load.audio("sound-back", ["assets/sound/ui/back.m4a", "assets/sound/ui/back.ogg"]);
            game.load.json("codes", "assets/code-redeem/codes.json");
            game.load.image("code_btn_floating", "assets/code-redeem/code_btn_floating.png");
            game.load.image("code_btn_splitscreen", "assets/code-redeem/code_btn_splitscreen.png");
            game.load.image("redeem-background", "assets/common/background.jpg");
            // Symbol input assets
            game.load.image("code-backdrop", "assets/code-redeem/code_backdrop.png");
            game.load.image("large-dialog", "assets/code-redeem/large_dialog.png");
            game.load.image("lightbeam", "assets/common/lightbeam.png");
            game.load.image("cursor-symbol", "assets/code-redeem/cursor_symbol.png");
            // Back button assets
            game.load.image("next-circle", "assets/common/next_circle.png");
            game.load.image("next-arrow", "assets/common/next_arrow.png");
            game.load.image("next-line", "assets/common/next_line.png");
            // Success dialog assets
            game.load.image("lightbeams", "assets/common/lightbeams.png");
            game.load.image("boost-symbol", "assets/code-redeem/boost_symbol.png");
            game.load.spritesheet("input-symbols", "assets/code-redeem/queue_symbols.png", 128, 128, 12);
            game.load.spritesheet("input-symbols-flat", "assets/code-redeem/queue_symbols_flat.png", 128, 128, 12);
        }
        show() {
            // Reset the reminder timer, so that it doesn't trigger for another 5 mins
            GameController_12.GameController.Instance.codeReminderTimer = this.game.time.now;
            this.backButton.show();
            SMAnalyticsManager_11.SMAnalyticsManager.Instance.trackState('content/game/inqueue/redeem', 'Redeem');
        }
        parseCodeJson() {
            this.redeemCodes = [];
            let codeList = this.game.cache.getJSON("codes").codes;
            codeList.forEach((value) => {
                this.redeemCodes.push(value);
            });
        }
        updateCodeProgressText(textField) {
            const right = textField.right;
            textField.text = `${GameController_12.GameController.Instance.state.redeemedCodes.length} / ${this.redeemCodes.length}`;
            textField.right = right;
        }
        onBack() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('sound-back');
                if (this.successDialog) {
                    yield this.successDialog.hide();
                    if (this.isShutdown) {
                        return;
                    }
                }
                SMAnalyticsManager_11.SMAnalyticsManager.Instance.trackAction('Back', 'Redeem', 'Back');
                // Go back to hub on success
                this.onButtonClickedSignal.dispatch(false);
            });
        }
        attemptRedemption(inputSequence) {
            return __awaiter(this, void 0, void 0, function* () {
                let inputSuccessful = false;
                const stringifiedSequence = inputSequence.join();
                const codeIsValidCode = this.codeExistsInCollection(inputSequence, this.redeemCodes);
                if (codeIsValidCode) {
                    const codeAlreadyRedeemed = this.codeExistsInCollection(inputSequence, GameController_12.GameController.Instance.state.redeemedCodes);
                    if (!codeAlreadyRedeemed) {
                        inputSuccessful = true;
                        SMAnalyticsManager_11.SMAnalyticsManager.Instance.trackAction('CodeValid', 'Redeem', 'CodeValid', '', {
                            [SMAnalyticsManager_11.SMAnalyticsManager.ITEM_ID]: stringifiedSequence
                        });
                    }
                    else {
                        this.alreadyRedeemedDialog.show();
                        yield PromiseUtil_22.PromiseUtil.signal(this.alreadyRedeemedDialog.onChildInputDown, this.shutdownSignal).catch(() => { });
                        if (this.isShutdown) {
                            return;
                        }
                        yield this.alreadyRedeemedDialog.hide(this.shutdownSignal);
                        if (this.isShutdown) {
                            return;
                        }
                    }
                }
                else {
                    SMAnalyticsManager_11.SMAnalyticsManager.Instance.trackAction('CodeInvalid', 'Redeem', 'CodeInvalid', '', {
                        [SMAnalyticsManager_11.SMAnalyticsManager.ITEM_ID]: stringifiedSequence
                    });
                }
                if (inputSuccessful) {
                    // Change entered symbols to green 
                    yield this.inputArea.displayResultAnimation(true);
                    if (this.isShutdown) {
                        return;
                    }
                    // save code to save state
                    GameController_12.GameController.Instance.state.addRedeemedCode(inputSequence);
                    this.updateCodeProgressText(this.codeProgressText);
                    // display success dialog
                    this.successDialog = new RedeemCodeSuccessDialog_1.RedeemCodeSuccessDialog(this.game, GameController_12.GameController.Instance.state.redeemedCodes.length, this.redeemCodes.length, this.shutdownSignal);
                    this.successDialog.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                        yield this.successDialog.hide();
                        this.onButtonClickedSignal.dispatch(true);
                    }), this);
                    this.successDialog.show();
                    SMAnalyticsManager_11.SMAnalyticsManager.Instance.trackState('content/game/inqueue/redeem/success', 'RedeemSuccess', {
                        [SMAnalyticsManager_11.SMAnalyticsManager.ITEM_ID]: stringifiedSequence
                    });
                    yield GameController_12.GameController.Instance.achievementManager.onQueueCodeRedeemed();
                }
                else {
                    // Change entered symbols to red
                    yield this.inputArea.displayResultAnimation(false);
                    if (this.isShutdown) {
                        return;
                    }
                    this.inputArea.clearInputSequence();
                }
            });
        }
        // Checks whether a given 4-digit code exists in a collection of codes
        codeExistsInCollection(code, collection) {
            for (let i = 0; i < collection.length; i++) {
                let codeValue = collection[i];
                // if each element of the array matches
                if (code.every((value, i) => { return value === codeValue[i]; })) {
                    return true;
                }
            }
            return false;
        }
        destroy() {
            this.shutdownSignal = null;
            if (this.onButtonClickedSignal) {
                this.onButtonClickedSignal.dispose();
                this.onButtonClickedSignal = null;
            }
            this.redeemCodes = null;
            this.inputArea = null;
            this.successDialog = null;
            this.alreadyRedeemedDialog = null;
            this.codeProgressText = null;
            super.destroy();
        }
    }
    RedeemCodeDialog.CODE_LENGTH = 4;
    exports.RedeemCodeDialog = RedeemCodeDialog;
});
define("state/RedeemCodeState", ["require", "exports", "StateNames", "state/SpaceBaseState", "component/coderedeem/RedeemCodeDialog"], function (require, exports, StateNames_7, SpaceBaseState_2, RedeemCodeDialog_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RedeemCodeState extends SpaceBaseState_2.SpaceBaseState {
        init() {
            this.stateName = StateNames_7.StateNames.REDEEM_CODE;
            super.init();
        }
        preload() {
            super.preload();
            RedeemCodeDialog_2.RedeemCodeDialog.preload(this.game);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.codeRedeemDialog = new RedeemCodeDialog_2.RedeemCodeDialog(this.game, this.shutdownSignal);
                this.add.existing(this.codeRedeemDialog);
                this.codeRedeemDialog.onButtonClickedSignal.addOnce((isSuccess) => {
                    this.transitionToHub();
                }, this);
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.codeRedeemDialog.show();
            });
        }
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToGame()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(StateNames_7.StateNames.HUB);
                this.codeRedeemDialog.show();
            });
        }
        shutdown() {
            this.codeRedeemDialog = null;
            super.shutdown();
        }
    }
    exports.RedeemCodeState = RedeemCodeState;
});
define("component/outro/PanelDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/button/ArrowButton"], function (require, exports, PromiseUtil_23, ArrowButton_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Panel dialog class that is currently only used in LoadInstruction screen
     * Currently asset is only loaded in OutroState
     * Need to load backplate asset somewhere if using this dialog somewhere else
     *
     * This dialog has no shade to block input on screen
     *
     * Behaviour:
     * -showDialog() scales the dialog up to create a 'dialog opening' effect
     * -showText() fades text in and out when 'next' button is pressed
     * -When there are no more text to show, the allTextShownSignal signal is dispatched
     */
    class PanelDialog extends Phaser.Group {
        constructor(game, whiteTextStyle) {
            super(game);
            this.onComplete = new Phaser.Signal();
            this.nextSignal = new Phaser.Signal();
            // backplate of the dialog that never moves
            this.backplate = new Phaser.Image(game, 0, 0, 'instruction-panel-backplate');
            this.backplate.width = game.width;
            this.backplate.scale.y = this.backplate.scale.x;
            this.backplate.height = Math.min(game.height * 0.25, this.backplate.height);
            this.backplate.anchor.set(0.5);
            this.backplate.alpha = 0.9;
            this.add(this.backplate);
            this.origBackplateScale = this.backplate.scale.y;
            // white text that can be changed
            this.whiteText = new Phaser.Text(game, 0, 0, '', whiteTextStyle);
            this.add(this.whiteText);
            // next button 
            this.nextButton = new ArrowButton_8.ArrowButton(game, "buttons.next", true, 70);
            this.nextButton.centerX = this.backplate.centerX;
            this.nextButton.centerY = this.backplate.bottom;
            this.add(this.nextButton);
            this.isShowing = false;
            this.visible = false;
        }
        showDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                this.backplate.scale.y = this.origBackplateScale * 0.3;
                this.visible = true;
                yield PromiseUtil_23.PromiseUtil.tween(this.game, this.game.add.tween(this.backplate.scale).to({ y: this.origBackplateScale }, 500, Phaser.Easing.Back.Out, true));
            });
        }
        showText(strings) {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < strings.length; i++) {
                    yield this.fadeInText(strings[i]);
                    yield PromiseUtil_23.PromiseUtil.time(this.game, 200);
                    // TODO: show the next button;
                    yield this.nextButton.show();
                    yield PromiseUtil_23.PromiseUtil.signal(this.nextButton.onButtonClickedSignal);
                    this.nextButton.onButtonClickedSignal.removeAll();
                    yield Promise.all([
                        this.nextButton.hide(),
                        this.fadeOutText()
                    ]);
                }
                this.onComplete.dispatch();
            });
        }
        fadeInText(whiteTextString) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShowing) {
                    yield this.fadeOutText();
                }
                this.whiteText.text = whiteTextString;
                // centers entire text group to the backplate
                this.whiteText.centerX = this.backplate.centerX;
                this.whiteText.centerY = this.backplate.centerY;
                // fade in the text
                yield PromiseUtil_23.PromiseUtil.tween(this.game, this.game.add.tween(this.whiteText).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true));
                this.isShowing = true;
            });
        }
        fadeOutText() {
            return __awaiter(this, void 0, void 0, function* () {
                // fade out the text
                yield PromiseUtil_23.PromiseUtil.tween(this.game, this.game.add.tween(this.whiteText).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true));
                this.isShowing = false;
            });
        }
        destroy() {
            this.onComplete.dispose();
            this.nextSignal.dispose();
            super.destroy();
        }
    }
    exports.PanelDialog = PanelDialog;
});
define("component/outro/RideVehicle", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_10, PromiseUtil_24) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RideVehicle extends Phaser.Group {
        constructor(game) {
            super(game);
            // the bundling process automatically pulls the correct world's image into the asset folder
            // so both world's ride-car image must be named the same 
            this.car = new Phaser.Image(game, 0, 0, 'ride-car');
            ScaleUtil_10.scaleToBounds(this.car, game.width * 0.95, game.height * 0.56);
            this.car.anchor.set(0.5);
            this.add(this.car);
            this.glow = new Phaser.Image(game, 0, 0, 'ride-glow');
            this.glow.width = this.car.width * 0.9;
            this.glow.scale.y = this.glow.scale.x;
            this.glow.anchor.set(0.5);
            this.positionGlow();
            this.add(this.glow);
            this.origScaleRideGlow = this.glow.scale.x; // save this off for use in tweens later
            this.visible = false;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.glow.alpha = 0;
                this.car.alpha = 0;
                this.glow.scale.set(this.origScaleRideGlow);
                this.positionGlow();
                this.visible = true;
                // fade in and scale up glow
                this.game.add.tween(this.glow).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.glow.scale).to({ x: this.origScaleRideGlow * 1.1, y: this.origScaleRideGlow * 1.1 }, 300, Phaser.Easing.Linear.None, true));
                // fade in the car
                this.game.add.tween(this.glow.scale).to({ x: this.origScaleRideGlow * 1, y: this.origScaleRideGlow * 1 }, 750, Phaser.Easing.Cubic.Out, true);
                yield PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.car).to({ alpha: 1 }, 300, Phaser.Easing.Cubic.In, true));
                // fade out and scale down glow
                this.game.add.tween(this.glow).to({ alpha: 0 }, 1500, Phaser.Easing.Cubic.Out, true);
            });
        }
        get right() {
            return this.car.right;
        }
        // the .right property of this group needs to be overwritten because 
        // the glow takes up too much space on the right hand side, and we always want to position 
        // this object based on the car's right side and not the glow's right side.
        set right(n) {
            this.car.right = n;
            this.positionGlow();
        }
        positionGlow() {
            const paddingX = this.car.width * 0.05;
            const paddingY = this.car.height * 0.03;
            this.glow.bottom = this.car.bottom - paddingY;
            this.glow.left = this.car.left + paddingX;
        }
    }
    exports.RideVehicle = RideVehicle;
});
define("switchback-common/extension/Easing", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function shake(aPeriod1, aPeriod2) {
        return (k) => {
            const current1 = k * Math.PI * 2 * aPeriod1;
            const current2 = k * (Math.PI * 2 * aPeriod2 + Math.PI / 2);
            return Math.sin(current1) * Math.cos(current2);
        };
    }
    exports.shake = shake;
});
define("component/ship/ShipSprite", ["require", "exports", "logic/GameController", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/extension/Easing"], function (require, exports, GameController_13, PromiseUtil_25, ScaleUtil_11, Easing_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ShipSprite extends Phaser.Group {
        constructor(game, shutdownSignal = null) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.WOBBLE_OFFSET_X = this.game.width * 0.02;
            this.WOBBLE_OFFSET_Y = this.game.height * 0.01;
            let partIndex = GameController_13.GameController.Instance.getShipState().baseParts;
            partIndex = Math.max(partIndex - 1, 0);
            this.base = new Phaser.Image(this.game, 0, 0, 'ship-base');
            this.base.anchor.set(0.5);
            this.add(this.base);
            this.thrusterPassive = new Phaser.Image(this.game, 0, this.base.bottom - (this.base.height * 0.1), 'thruster-passive');
            this.thrusterPassive.width = this.base.width * 0.4;
            this.thrusterPassive.scale.y = this.thrusterPassive.scale.x;
            this.thrusterPassive.anchor.set(0.5, 0);
            this.add(this.thrusterPassive);
            this.game.add.tween(this.thrusterPassive.scale).to({ x: this.thrusterPassive.scale.x * 1.15 }, 200, Easing_1.shake(50, 50), true, 0, Infinity, true);
            this.upgradeLayerLeft = new Phaser.Image(this.game, 0, 0, 'ship-upgrades', partIndex);
            this.upgradeLayerLeft.anchor.set(1, 0.5);
            this.upgradeLayerLeft.visible = partIndex > 0;
            this.add(this.upgradeLayerLeft);
            this.upgradeLayerRight = new Phaser.Image(this.game, 0, 0, 'ship-upgrades', partIndex);
            this.upgradeLayerRight.anchor.set(1, 0.5);
            this.upgradeLayerRight.scale.x *= -1;
            this.upgradeLayerRight.visible = partIndex > 0;
            this.add(this.upgradeLayerRight);
            this.bonusUpgradeLayerLeft = new Phaser.Image(this.game, 0, 0, 'bonus-upgrades', 0);
            this.bonusUpgradeLayerLeft.anchor.set(1, 0.5);
            this.bonusUpgradeLayerLeft.top = this.upgradeLayerLeft.top;
            this.bonusUpgradeLayerLeft.alpha = 0;
            this.add(this.bonusUpgradeLayerLeft);
            this.bonusUpgradeLayerRight = new Phaser.Image(this.game, 0, 0, 'bonus-upgrades', 0);
            this.bonusUpgradeLayerRight.anchor.set(1, 0.5);
            this.bonusUpgradeLayerRight.scale.x *= -1;
            this.bonusUpgradeLayerRight.top = this.upgradeLayerRight.top;
            this.bonusUpgradeLayerRight.alpha = 0;
            this.add(this.bonusUpgradeLayerRight);
            // If we've already redeemed bonus ship upgrades
            if (GameController_13.GameController.Instance.getShipState().bonusParts > 0) {
                // Add bonus upgrades
                let bonusPartIndex = GameController_13.GameController.Instance.getShipState().bonusParts;
                bonusPartIndex = Math.max(bonusPartIndex - 1, 0);
                this.bonusUpgradeLayerLeft.frame = bonusPartIndex;
                this.bonusUpgradeLayerLeft.alpha = 1;
                this.bonusUpgradeLayerRight.frame = bonusPartIndex;
                this.bonusUpgradeLayerRight.alpha = 1;
            }
            this.thrusterLaunch = new Phaser.Image(this.game, 0, this.base.bottom - (this.base.height * 0.1), 'thruster-launch');
            this.thrusterLaunch.width = this.base.width * 0.4;
            this.thrusterLaunch.scale.y = this.thrusterLaunch.scale.x;
            this.thrusterLaunch.anchor.set(0.5, 0);
            this.add(this.thrusterLaunch);
            this.game.add.tween(this.thrusterLaunch.scale).to({ x: this.thrusterLaunch.scale.x * 1.15 }, 200, Easing_1.shake(50, 50), true, 0, Infinity, true);
            const slipStreamEffect = new Phaser.Image(this.game, 0, this.base.height * 0.15, 'slipstream-effect');
            slipStreamEffect.anchor.set(0.5, 0);
            slipStreamEffect.alpha = 0.7;
            ScaleUtil_11.scaleToBounds(slipStreamEffect, this.base.width * 1.5, this.base.height);
            slipStreamEffect.animations.add('loop', null, 24, true).play();
            this.add(slipStreamEffect);
        }
        startWobble() {
            const randX = this.game.rnd.integerInRange(this.x - this.WOBBLE_OFFSET_X, this.x + this.WOBBLE_OFFSET_X);
            const randY = this.game.rnd.integerInRange(this.y - this.WOBBLE_OFFSET_Y, this.y + this.WOBBLE_OFFSET_Y);
            this.wobbleTween = this.game.add.tween(this)
                .to({ x: randX, y: randY }, 300, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
            this.wobbleTween.onComplete.addOnce(this.startWobble, this);
        }
        animateLaunch(duration = 350, delay = 0) {
            return __awaiter(this, void 0, void 0, function* () {
                // If we've redeemed one of the final two bonus ship upgrades,
                // move the thruster down so it can be seen
                let yPos = this.thrusterLaunch.y;
                if (GameController_13.GameController.Instance.state.shipBonusParts >= GameController_13.GameController.MAX_NUM_BONUS_PARTS - 2) {
                    yPos = this.bonusUpgradeLayerLeft.bottom - (this.bonusUpgradeLayerLeft.height * 0.4);
                }
                if (GameController_13.GameController.Instance.state.shipBonusParts >= GameController_13.GameController.MAX_NUM_BONUS_PARTS - 1) {
                    yPos = this.bonusUpgradeLayerLeft.bottom - (this.bonusUpgradeLayerLeft.height * 0.25);
                }
                this.thrusterLaunch.y = yPos;
                this.thrusterPassive.y = yPos;
                this.thrusterLaunch.visible = true;
                this.thrusterLaunch.alpha = 0;
                yield PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.thrusterLaunch).to({ alpha: 1 }, duration, Phaser.Easing.Linear.None, true, delay), this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        stopAnimateLaunch(duration = 350, delay = 0) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.thrusterLaunch).to({ alpha: 0 }, duration, Phaser.Easing.Linear.None, true, delay), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.thrusterLaunch.visible = false;
                this.thrusterLaunch.alpha = 0;
            });
        }
        performUpgrade() {
            return __awaiter(this, void 0, void 0, function* () {
                if (GameController_13.GameController.Instance.state.shipPendingBaseParts > 0) {
                    yield this.performBaseUpgrade();
                }
                else if (GameController_13.GameController.Instance.state.shipPendingBonusParts > 0) {
                    // Apply bonus upgrades afterwards, as they may obscure
                    // the base upgrades
                    yield this.performBonusUpgrade();
                }
            });
        }
        performBaseUpgrade() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_13.GameController.Instance.redeemSinglePendingShipPart();
                let partIndex = GameController_13.GameController.Instance.getShipState().baseParts - 1;
                this.upgradeLayerLeft.frame = partIndex;
                this.upgradeLayerRight.frame = partIndex;
                this.upgradeLayerLeft.visible = true;
                this.upgradeLayerRight.visible = true;
                this.game.sound.play('upgrade_applied' + this.game.rnd.integerInRange(1, 3));
                yield PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.scale)
                    .from({ x: this.scale.x * 1.15, y: this.scale.y * 1.15 }, 200, Phaser.Easing.Cubic.In, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        performBonusUpgrade() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_13.GameController.Instance.redeemSinglePendingBonusShipPart();
                let partIndex = GameController_13.GameController.Instance.getShipState().bonusParts - 1;
                this.bonusUpgradeLayerLeft.frame = partIndex;
                this.bonusUpgradeLayerRight.frame = partIndex;
                this.bonusUpgradeLayerLeft.alpha = 1;
                this.bonusUpgradeLayerRight.alpha = 1;
                this.game.sound.play('upgrade_applied' + this.game.rnd.integerInRange(1, 3));
                yield PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.scale)
                    .from({ x: this.scale.x * 1.15, y: this.scale.y * 1.15 }, 200, Phaser.Easing.Cubic.In, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                if (partIndex >= GameController_13.GameController.MAX_NUM_BONUS_PARTS - 2) {
                    // Move thruster down, because the length of the ship has just changed
                    // This only needs to happen for the final 2 bonus upgrades
                    this.game.add.tween(this.thrusterPassive).to({ y: this.bonusUpgradeLayerRight.bottom - (this.bonusUpgradeLayerRight.height * 0.25) }, 300, Phaser.Easing.Linear.None, true);
                }
            });
        }
        destroy() {
            if (this.wobbleTween) {
                this.wobbleTween.stop();
                this.wobbleTween = null;
            }
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.ShipSprite = ShipSprite;
});
define("switchback-common/component/RadialProgressBar", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_26) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialProgressBar extends Phaser.Graphics {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100, startAngleOffset = 0) {
            super(game, x, y);
            this.angleMin = 0;
            this.angleMax = 0;
            this.color = color;
            this.lineThicknessPx = lineThicknessPx;
            this.radiusPx = radiusPx;
            this.angleMin = this.angleMax = startAngleOffset;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: this.angleMin + 360 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
        update() {
            super.update();
            this.clear();
            this.lineStyle(this.lineThicknessPx, this.color);
            this.arc(0, 0, this.radiusPx, Phaser.Math.degToRad(this.angleMin), Phaser.Math.degToRad(this.angleMax), false);
            this.endFill();
        }
    }
    exports.RadialProgressBar = RadialProgressBar;
});
define("component/outro/LoadInstruction", ["require", "exports", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "component/button/CTAButton", "component/CutsceneScreen", "Game", "component/Logo", "component/outro/PanelDialog", "component/outro/RideVehicle", "logic/GameController", "component/ship/ShipSprite", "switchback-common/component/RadialProgressBar", "switchback-common/util/TextUtil", "logic/SMAnalyticsManager"], function (require, exports, LocalizationManager_16, PromiseUtil_27, ScaleUtil_12, CTAButton_3, CutsceneScreen_7, Game_21, Logo_2, PanelDialog_1, RideVehicle_1, GameController_14, ShipSprite_1, RadialProgressBar_1, TextUtil_5, SMAnalyticsManager_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LoadInstruction extends CutsceneScreen_7.CutsceneScreen {
        constructor(game) {
            super(game);
            const aspectRatio = (game.width / game.height);
            this.isWideDevice = aspectRatio > 9 / 16;
            // SWB-1169 Polished after game
            this.wasMerged = GameController_14.GameController.Instance.state.hasMerged;
            // SWB-958 This two are important, and in this order
            GameController_14.GameController.Instance.state.hasMerged = true;
            GameController_14.GameController.Instance.state.save();
            this.canResume = GameController_14.GameController.Instance.state.canResumePostMerge;
            const largeYellowTextStyle = {
                font: Game_21.Game.DEFAULT_FONT,
                fill: Game_21.Game.FONT_COLOUR_YELLOW,
                fontSize: Game_21.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            };
            const medWhiteTextStyle = {
                font: Game_21.Game.DEFAULT_FONT,
                fill: Game_21.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_21.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                valign: 'center',
                align: 'center',
                stroke: Game_21.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_21.Game.DEFAULT_STROKE_THICKNESS
            };
            const lgeWhiteTextStyle = Object.assign({}, medWhiteTextStyle, { wordWrapWidth: game.width * 0.99, fontSize: 75 });
            ///////////////////////////////////
            // ride image and glow
            ///////////////////////////////////
            if (this.wasMerged) {
                this.rideVehicle = null;
                this.ship = new ShipSprite_1.ShipSprite(this.game);
                ScaleUtil_12.scaleToBounds(this.ship, this.game.width * 0.8, this.game.height * 0.8);
                this.ship.centerX = this.game.width / 2.0;
                this.ship.top = this.game.height * 0.3;
                this.ship.alpha = 0;
                this.add(this.ship);
            }
            else {
                this.ship = null;
                this.rideVehicle = new RideVehicle_1.RideVehicle(game);
                this.rideVehicle.centerY = this.game.height * 0.45;
                this.rideVehicle.left = 0;
                this.add(this.rideVehicle);
            }
            ///////////////////////////////////
            // yellow text above ride 
            // fades out and in
            ///////////////////////////////////
            this.titleText = new Phaser.Text(game, 0, 0, '', largeYellowTextStyle);
            this.titleText.anchor.set(0.5);
            this.titleText.centerY = this.game.height * 0.25;
            this.titleText.setShadow(0, this.game.height * 0.005, 'rgba(128, 0, 128, 1)', 0, true, true);
            this.add(this.titleText);
            ///////////////////////////////////
            // logo
            ///////////////////////////////////
            this.logo = new Logo_2.Logo(game);
            ScaleUtil_12.scaleToBounds(this.logo, game.width * 0.5, game.height * 0.15);
            this.logo.centerX = this.game.width * 0.5;
            this.logo.bottom = this.titleText.top + this.game.height * 0.01;
            this.add(this.logo);
            if (this.isWideDevice) {
                this.titleText.centerY = this.game.height * 0.22;
                this.logo.bottom = this.titleText.top + this.logo.height * 0.2;
            }
            ///////////////////////////////////
            // text dialog with next button
            ///////////////////////////////////
            this.panelDialog = new PanelDialog_1.PanelDialog(game, medWhiteTextStyle);
            this.panelDialog.left = 0;
            this.panelDialog.top = this.game.height * 0.68;
            this.add(this.panelDialog);
            ///////////////////////////////////
            // quit button
            ///////////////////////////////////
            const quitText = this.canResume ? 'buttons.continue' : 'buttons.quit';
            this.bottomGreenButton = new CTAButton_3.CTAButton(this.game, LocalizationManager_16.LocalizationManager.Instance.text(quitText));
            this.bottomGreenButton.centerX = this.game.width * 0.5;
            if (this.isWideDevice) {
                this.bottomGreenButton.bottom = this.game.height * 0.98;
            }
            else {
                this.bottomGreenButton.bottom = this.game.height * 0.95;
            }
            this.bottomGreenButton.visible = false;
            this.add(this.bottomGreenButton);
            // Destination text (last page)
            this.destinationTextGroup = new Phaser.Group(this.game);
            if (this.canResume) {
                this.createContinueText(lgeWhiteTextStyle);
            }
            else {
                this.createHyperionText(lgeWhiteTextStyle, largeYellowTextStyle);
            }
            this.add(this.destinationTextGroup);
            this.destinationTextGroup.centerX = this.panelDialog.centerX;
            this.destinationTextGroup.centerY = this.panelDialog.centerY;
            this.destinationTextGroup.visible = false;
            //////////////////////////////////////
            // QUIT BUTTON
            //////////////////////////////////////
            this.redQuitButtonText = new Phaser.Text(this.game, 0, 0, LocalizationManager_16.LocalizationManager.Instance.text("buttons.quit"), {
                font: Game_21.Game.DEFAULT_FONT,
                fill: Game_21.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_21.Game.FONT_SIZE_SMALL,
                stroke: Game_21.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_21.Game.FONT_STROKE_SMALL
            });
            this.redQuitButtonText.setScaleMinMax(1);
            this.redQuitButton = new Phaser.Image(this.game, 0, 0, "quit-button");
            this.redQuitButton.width = this.game.width * 0.25;
            this.redQuitButton.scale.y = this.redQuitButton.scale.x;
            this.redQuitButton.right = this.game.width * 0.99;
            this.redQuitButton.top = this.game.height * 0.01;
            this.redQuitButton.inputEnabled = true;
            this.redQuitButton.visible = false;
            this.redQuitButtonText.visible = false;
            this.redQuitButtonText.centerX = this.redQuitButton.centerX;
            this.redQuitButtonText.centerY = this.redQuitButton.centerY;
            this.redQuitButton.events.onInputUp.add(() => {
                this.game.sound.play("cta-button");
                SMAnalyticsManager_12.SMAnalyticsManager.Instance.trackAction('Exit', 'Outro', 'Exit');
                SMAnalyticsManager_12.SMAnalyticsManager.Instance.endTiming("RocketRace", "RocketRace", 'Game', '');
                PlayAPI.closeGame();
            }, this);
            this.add(this.redQuitButton);
            this.add(this.redQuitButtonText);
        }
        createContinueText(lgeWhite) {
            this.destinationText = new Phaser.Text(this.game, 0, 0, LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.load_continue_later"), lgeWhite);
            this.destinationTextGroup.add(this.destinationText);
            TextUtil_5.TextUtil.scaleDownTextToHeight(this.destinationText, this.panelDialog.height * 0.65);
        }
        createHyperionText(lgeWhite, lgeYellow) {
            ///////////////////////////////////
            // White text above hyperion text
            ///////////////////////////////////
            this.destinationTextGroup = new Phaser.Group(this.game);
            this.destinationText = new Phaser.Text(this.game, 0, 0, LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.your_destination").toUpperCase(), lgeWhite);
            this.destinationText.anchor.set(0.5);
            this.destinationTextGroup.add(this.destinationText);
            ///////////////////////////////////
            // Big yellow hyperion text
            ///////////////////////////////////
            this.hyperionText = new Phaser.Text(this.game, 0, 0, LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.Hyperion").toUpperCase(), Object.assign({}, lgeYellow, { fontSize: 150 }));
            TextUtil_5.TextUtil.scaleDownSingleLineTextToWidth(this.hyperionText, this.game.width * 0.90);
            this.hyperionText.anchor.set(0.5);
            this.hyperionText.top = this.destinationText.bottom;
            this.hyperionText.centerX = this.destinationText.centerX;
            this.hyperionText.setShadow(0, this.game.height * 0.005, 'rgba(128, 0, 128, 1)', 0, true, true);
            this.textLineEffect = new Phaser.Image(this.game, 0, 0, 'title-lightbar');
            this.textLineEffect.width = this.hyperionText.width * 1.5;
            this.textLineEffect.scale.y = this.textLineEffect.scale.x;
            this.textLineEffect.anchor.set(0.5);
            this.textLineEffect.centerX = this.hyperionText.centerX;
            this.textLineEffect.centerY = this.hyperionText.centerY;
            // save off the original size for use in tweens later
            this.origLineEffectScale = this.textLineEffect.scale.x;
            this.destinationTextGroup.add(this.textLineEffect);
            this.destinationTextGroup.add(this.hyperionText);
        }
        showVehicle() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.rideVehicle) {
                    yield this.rideVehicle.show();
                }
                else {
                    yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.ship).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true));
                    this.ship.startWobble();
                }
            });
        }
        showNarrativePanel() {
            return __awaiter(this, void 0, void 0, function* () {
                // scales the panelDialog's y from 0 to 1
                yield this.panelDialog.showDialog();
                SMAnalyticsManager_12.SMAnalyticsManager.Instance.trackState(`content/game/inqueue/outro/loadinstructions`, `LoadInstructions`);
                if (this.canResume) {
                    yield this.showContinueText();
                }
                else {
                    yield this.showDestinationText();
                }
                this.showTitleText(LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.enjoy_your_trip").toUpperCase());
            });
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 0;
                this.visible = true;
                // fade this whole screen in
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true));
                yield this.logo.show(true);
                if (this.wasMerged) {
                    this.showTitleText(LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.godspeed").toUpperCase());
                }
                yield this.showVehicle();
                if (!this.wasMerged) {
                    yield this.showNarrativePanel();
                }
                if (this.canResume) {
                    this.showCountdownRadial();
                }
                else {
                    this.showRegularFinishButton();
                }
            });
        }
        /**
         * Changes the title text, then fades and scales it in
         * @param s the string to be shown
         */
        showTitleText(s) {
            return __awaiter(this, void 0, void 0, function* () {
                this.titleText.text = s;
                this.titleText.centerX = this.logo.centerX;
                this.titleText.alpha = 0;
                this.titleText.top = this.logo.bottom;
                this.titleText.scale.set(0.5);
                this.titleText.visible = true;
                // scale in and fade in the title text
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.titleText).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.titleText.scale).to({ x: 1, y: 1 }, 300, Phaser.Easing.Back.Out, true))
                ]);
            });
        }
        showContinueText() {
            return __awaiter(this, void 0, void 0, function* () {
                this.destinationText.alpha = 0;
                this.destinationTextGroup.visible = true;
                this.destinationTextGroup.centerX = this.panelDialog.centerX;
                this.destinationTextGroup.top = this.panelDialog.top + this.panelDialog.height * 0.05;
                this.timerIntervalId = setInterval(this.onTimeUpdate.bind(this), 1000);
                this.startTimerTime = new Date().getTime();
                this.onTimeUpdate();
                // scale in and fade in the destination text
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.destinationText).to({ alpha: 1 }, 600, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.destinationText.scale).to({ x: 1, y: 1 }, 600, Phaser.Easing.Back.Out, true)),
                ]);
            });
        }
        onTimeUpdate() {
            return __awaiter(this, void 0, void 0, function* () {
                const elapsedTime = new Date().getTime() - this.startTimerTime;
                const timeRemaining = Math.round((LoadInstruction.WAIT_FOR_CONTINUE_MS - elapsedTime) / 1000);
                if (timeRemaining > 0) {
                    const minutes = Math.floor(timeRemaining / 60);
                    const seconds = timeRemaining % 60;
                    const secondsString = (seconds < 10 ? "0" + seconds : seconds).toString();
                    this.destinationText.text = LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.load_continue_later") + minutes + ":" + secondsString;
                }
                else {
                    clearInterval(this.timerIntervalId);
                    this.destinationText.text = LocalizationManager_16.LocalizationManager.Instance.text("narrative_outro.load_continue_now");
                    this.destinationTextGroup.centerX = this.game.width * 0.5;
                    this.destinationTextGroup.centerY = this.panelDialog.top + this.panelDialog.height * 0.4;
                    this.showPostMergeContinue();
                }
            });
        }
        /**
         * Fades and scales in the destination and hyperion text,
         * as well as the line effect that appears behind the word "Hyperion"
         */
        showDestinationText() {
            return __awaiter(this, void 0, void 0, function* () {
                this.destinationText.alpha = 0;
                this.hyperionText.alpha = 0;
                this.textLineEffect.alpha = 0;
                this.destinationText.scale.set(0.5);
                this.hyperionText.scale.set(0.4);
                // this must use its original scale as it is a phaser.image which have been scaled based on screen width/height on creation
                this.textLineEffect.scale.set(this.origLineEffectScale * 0.2);
                this.destinationTextGroup.centerX = this.panelDialog.centerX;
                this.destinationTextGroup.top = this.panelDialog.top + this.panelDialog.height * 0.05;
                this.destinationTextGroup.visible = true;
                // scale in and fade in the destination text 	
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.destinationText).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.destinationText.scale).to({ x: 1, y: 1 }, 500, Phaser.Easing.Back.Out, true)),
                ]);
                yield PromiseUtil_27.PromiseUtil.time(this.game, 900);
                // these Easing effects have been art audited to make sure they look good
                const easingTextLineEffectIn = Phaser.Easing.Quintic.Out;
                const easingTextLineEffectOut = Phaser.Easing.Cubic.In;
                // scale in and fade in the hyperion text and line effect
                this.game.add.tween(this.textLineEffect).to({ alpha: 1 }, 600, easingTextLineEffectIn, true);
                this.game.add.tween(this.textLineEffect.scale).to({ x: this.origLineEffectScale * 1.3, y: this.origLineEffectScale * 1.3 }, 600, easingTextLineEffectIn, true);
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.hyperionText).to({ alpha: 1 }, 400, easingTextLineEffectIn, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.hyperionText.scale).to({ x: 1.05, y: 1.05 }, 400, easingTextLineEffectIn, true)),
                ]);
                // scale and fade out the line effect
                yield Promise.all([
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.hyperionText.scale).to({ x: 1, y: 1 }, 500, easingTextLineEffectOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.textLineEffect).to({ alpha: 0 }, 1000, easingTextLineEffectOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.textLineEffect.scale).to({ x: 0.2, y: 0.2 }, 1000, easingTextLineEffectOut, true))
                ]);
            });
        }
        showCountdownRadial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bottomGreenButton.alpha = 0;
                this.bottomGreenButton.visible = true;
                this.countGroup = new Phaser.Group(this.game);
                const countRadius = this.game.width * 0.025;
                const timerBackground = new Phaser.Graphics(this.game);
                timerBackground.beginFill(0x16427f, 1);
                timerBackground.drawCircle(0, 0, this.game.width * 0.115);
                timerBackground.endFill();
                this.countGroup.add(timerBackground);
                const countdown = new RadialProgressBar_1.RadialProgressBar(this.game, 0, 0, 0xFFFFFF, this.game.width * 0.05, countRadius, -90);
                this.countGroup.add(countdown);
                this.countGroup.scale.set(1.9);
                this.countGroup.x = this.bottomGreenButton.centerX;
                this.countGroup.top = this.bottomGreenButton.top;
                this.countGroup.alpha = 0;
                this.add(this.countGroup);
                this.redQuitButton.visible = true;
                this.redQuitButtonText.visible = true;
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.countGroup).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true));
                countdown.loop(LoadInstruction.WAIT_FOR_CONTINUE_MS);
            });
        }
        showPostMergeContinue() {
            return __awaiter(this, void 0, void 0, function* () {
                this.redQuitButton.destroy();
                this.redQuitButtonText.destroy();
                this.countGroup.destroy();
                yield this.bottomGreenButton.show();
                this.bottomGreenButton.onButtonClickedSignal.addOnce(() => {
                    console.log("Schedule and directing to race");
                    SMAnalyticsManager_12.SMAnalyticsManager.Instance.trackAction('Continue', 'Outro', 'Continue');
                    GameController_14.GameController.Instance.raceController.scheduleOnDeviceRaceFromLast(0);
                    this.game.state.getCurrentState().transitionToPreRaceState();
                });
            });
        }
        /**
         * show the quit button
         * adds a click signal that closes the game when the quit button is clicked
         */
        showRegularFinishButton() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_27.PromiseUtil.time(this.game, 2000);
                this.bottomGreenButton.visible = true;
                yield this.bottomGreenButton.show();
                this.bottomGreenButton.onButtonClickedSignal.addOnce(() => {
                    SMAnalyticsManager_12.SMAnalyticsManager.Instance.trackAction('Exit', 'Outro', 'Exit');
                    SMAnalyticsManager_12.SMAnalyticsManager.Instance.endTiming("RocketRace", "RocketRace", 'Game', '');
                    GameController_14.GameController.Instance.state.clear();
                    PlayAPI.closeGame();
                });
            });
        }
    }
    LoadInstruction.WAIT_FOR_CONTINUE_MS = 5 * 60 * 1000; // 5 minutes
    exports.LoadInstruction = LoadInstruction;
});
define("component/outro/Outro1", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_22, CutsceneScreen_8, ArrowButton_9, LocalizationManager_17, ScaleUtil_13, TextUtil_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Outro1 extends CutsceneScreen_8.CutsceneScreen {
        constructor(game) {
            super(game);
            this.cutsceneName = "MissionComplete";
            const lightbarTop = new Phaser.Image(game, 0, 0, "lightbar");
            ScaleUtil_13.scaleToBounds(lightbarTop, this.game.width * 0.8, this.game.height * 0.1);
            lightbarTop.centerX = this.game.width * 0.5;
            lightbarTop.centerY = this.game.height * 0.1;
            this.add(lightbarTop);
            const smIcon = new Phaser.Image(game, 0, 0, "sm-icon");
            ScaleUtil_13.scaleToBounds(smIcon, this.game.width * 0.25, this.game.width * 0.25);
            smIcon.centerX = lightbarTop.centerX;
            smIcon.centerY = lightbarTop.centerY;
            this.add(smIcon);
            const missionComplete = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_17.LocalizationManager.Instance.text("outro.mission_complete"), {
                font: Game_22.Game.DEFAULT_FONT,
                fill: '#fce100',
                stroke: Game_22.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_22.Game.FONT_STROKE_SMALL,
                fontSize: 110,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                align: 'center'
            });
            TextUtil_6.TextUtil.scaleDownTextToHeight(missionComplete, this.game.height * 0.2);
            missionComplete.centerX = lightbarTop.centerX;
            missionComplete.top = smIcon.bottom;
            this.add(missionComplete);
            const lightbarBottom = new Phaser.Image(game, 0, 0, "lightbar");
            ScaleUtil_13.scaleToBounds(lightbarBottom, this.game.width * 0.8, this.game.height * 0.1);
            lightbarBottom.centerX = lightbarTop.centerX;
            lightbarBottom.top = missionComplete.bottom;
            this.add(lightbarBottom);
            const hyperionGlow = new Phaser.Image(game, 0, 0, "hyperion-glow");
            ScaleUtil_13.scaleToBounds(hyperionGlow, this.game.width, this.game.height * 0.7);
            hyperionGlow.centerX = lightbarTop.centerX;
            hyperionGlow.centerY = this.game.height * 0.59;
            this.add(hyperionGlow);
            const hyperion = new Phaser.Image(game, 0, 0, "hyperion-backlit");
            ScaleUtil_13.scaleToBounds(hyperion, this.game.width * 0.6, this.game.height * 0.4);
            hyperion.centerX = hyperionGlow.centerX;
            hyperion.centerY = hyperionGlow.centerY;
            this.add(hyperion);
            const nextButton = new ArrowButton_9.ArrowButton(game, "buttons.next", true, Game_22.Game.FONT_SIZE_LARGE);
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.8;
            nextButton.centerY = this.game.height * 0.9;
            this.button = nextButton;
            this.add(this.button);
            this.left = this.game.width;
        }
    }
    exports.Outro1 = Outro1;
});
define("component/outro/Outro2", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_23, CutsceneScreen_9, ArrowButton_10, LocalizationManager_18, ScaleUtil_14, TextUtil_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Outro2 extends CutsceneScreen_9.CutsceneScreen {
        constructor(game) {
            super(game);
            this.cutsceneName = "FastestShip";
            const hyperion = new Phaser.Image(game, 0, 0, "hyperion-travel");
            ScaleUtil_14.scaleToBounds(hyperion, this.game.width * 0.8, this.game.height * 0.5);
            hyperion.centerX = this.game.width * 0.5;
            hyperion.top = this.game.height * 0.17;
            this.add(hyperion);
            const topText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_18.LocalizationManager.Instance.text("outro.outro2a"), {
                font: Game_23.Game.DEFAULT_FONT,
                fill: Game_23.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_23.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_23.Game.FONT_STROKE_SMALL,
                fontSize: Game_23.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_7.TextUtil.scaleDownTextToHeight(topText, this.game.height * 0.2);
            topText.centerX = hyperion.centerX;
            topText.top = this.game.height * 0.04;
            this.add(topText);
            const nextButton = new ArrowButton_10.ArrowButton(game, "buttons.next", true, Game_23.Game.FONT_SIZE_LARGE);
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.8;
            nextButton.centerY = this.game.height * 0.9;
            this.button = nextButton;
            this.add(this.button);
            const bottomText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_18.LocalizationManager.Instance.text("outro.outro2b"), {
                font: Game_23.Game.DEFAULT_FONT,
                fill: Game_23.Game.FONT_COLOUR_BLUE,
                stroke: Game_23.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_23.Game.FONT_STROKE_SMALL,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                fontSize: Game_23.Game.FONT_SIZE_LARGE,
                align: 'center'
            });
            TextUtil_7.TextUtil.scaleDownTextToHeight(bottomText, this.game.height * 0.2);
            bottomText.centerX = hyperion.centerX;
            bottomText.bottom = nextButton.top - this.game.height * 0.03;
            this.add(bottomText);
            this.left = this.game.width;
        }
    }
    exports.Outro2 = Outro2;
});
define("component/outro/Outro3", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "component/ship/ShipSprite", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, Game_24, CutsceneScreen_10, ArrowButton_11, ShipSprite_2, LocalizationManager_19, ScaleUtil_15, TextUtil_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Outro3 extends CutsceneScreen_10.CutsceneScreen {
        constructor(game) {
            super(game);
            this.cutsceneName = "ReadyForTakeOff";
            const topText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_19.LocalizationManager.Instance.text("outro.your_rocket_ship"), {
                font: Game_24.Game.DEFAULT_FONT,
                fill: Game_24.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_24.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_24.Game.FONT_STROKE_SMALL,
                fontSize: Game_24.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_8.TextUtil.scaleDownTextToHeight(topText, this.game.height * 0.20);
            topText.centerX = this.game.width * 0.5;
            topText.top = this.game.height * 0.04;
            this.add(topText);
            const nextButton = new ArrowButton_11.ArrowButton(game, "buttons.next", true, Game_24.Game.FONT_SIZE_LARGE);
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.8;
            nextButton.centerY = this.game.height * 0.9;
            this.button = nextButton;
            this.add(this.button);
            const bottomText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_19.LocalizationManager.Instance.text("outro.at_launch_platform"), {
                font: Game_24.Game.DEFAULT_FONT,
                fill: Game_24.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_24.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_24.Game.FONT_STROKE_SMALL,
                fontSize: Game_24.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_8.TextUtil.scaleDownTextToHeight(topText, this.game.height * 0.2);
            bottomText.centerX = topText.centerX;
            bottomText.bottom = nextButton.top - this.game.height * 0.03;
            this.add(bottomText);
            const shipSprite = new ShipSprite_2.ShipSprite(game);
            ScaleUtil_15.scaleToBounds(shipSprite, this.game.width * 0.9, bottomText.top - topText.bottom - this.game.height * 0.02);
            shipSprite.centerX = topText.centerX;
            shipSprite.centerY = topText.bottom + (bottomText.top - topText.bottom) * 0.5;
            this.add(shipSprite);
            shipSprite.startWobble();
            shipSprite.animateLaunch(500, 0);
            this.left = this.game.width;
        }
    }
    exports.Outro3 = Outro3;
});
define("component/outro/Outro4", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "logic/SMAnalyticsManager", "switchback-common/util/TextUtil"], function (require, exports, LocalizationManager_20, Game_25, CutsceneScreen_11, ArrowButton_12, SMAnalyticsManager_13, TextUtil_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Outro4 extends CutsceneScreen_11.CutsceneScreen {
        constructor(game) {
            super(game);
            const topText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_20.LocalizationManager.Instance.text("outro.well_done_team"), {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.FONT_COLOUR_BLUE,
                stroke: Game_25.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_25.Game.FONT_STROKE_SMALL,
                fontSize: 100,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_9.TextUtil.scaleDownTextToHeight(topText, this.game.height * 0.3);
            topText.centerX = this.game.width * 0.5;
            topText.top = this.game.height * 0.2;
            this.add(topText);
            const bottomText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_20.LocalizationManager.Instance.text("outro.you_will_be_assigned"), {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_25.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_25.Game.FONT_STROKE_SMALL,
                fontSize: Game_25.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_9.TextUtil.scaleDownTextToHeight(bottomText, this.game.height * 0.3);
            bottomText.centerX = topText.centerX;
            bottomText.top = topText.bottom + this.game.height * 0.01;
            this.add(bottomText);
            const nextButton = new ArrowButton_12.ArrowButton(game, "buttons.next", true, Game_25.Game.FONT_SIZE_LARGE);
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.8;
            nextButton.centerY = this.game.height * 0.9;
            this.button = nextButton;
            this.add(this.button);
            this.left = this.game.width;
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_13.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/instructions', 'RoleAssignmentInstructions', {
                    [SMAnalyticsManager_13.SMAnalyticsManager.GAME_NAME_MINI]: 'RoleAssignment'
                });
                yield _super("show").call(this);
            });
        }
    }
    exports.Outro4 = Outro4;
});
define("component/outro/OutroRoleAssignment", ["require", "exports", "Game", "component/CutsceneScreen", "component/button/ArrowButton", "switchback-common/localization/LocalizationManager", "logic/SMAnalyticsManager", "switchback-common/util/TextUtil"], function (require, exports, Game_26, CutsceneScreen_12, ArrowButton_13, LocalizationManager_21, SMAnalyticsManager_14, TextUtil_10) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class OutroRoleAssignment extends CutsceneScreen_12.CutsceneScreen {
        constructor(game, playerName, roleNameKey, roleDescriptionKey) {
            super(game);
            const nextButton = new ArrowButton_13.ArrowButton(game, "buttons.next", true, Game_26.Game.FONT_SIZE_LARGE);
            nextButton.width = this.game.width * 0.7;
            nextButton.scale.y = nextButton.scale.x;
            nextButton.x = this.game.width * 0.8;
            nextButton.centerY = this.game.height * 0.9;
            this.button = nextButton;
            this.add(this.button);
            const playerNameText = new Phaser.Text(game, this.game.width * 0.5, 0, playerName, {
                font: Game_26.Game.DEFAULT_FONT,
                fill: Game_26.Game.FONT_COLOUR_BLUE,
                stroke: Game_26.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_26.Game.FONT_STROKE_SMALL,
                fontSize: 110,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_10.TextUtil.scaleDownTextToHeight(playerNameText, this.game.height * 0.15);
            playerNameText.top = this.game.height * 0.1;
            playerNameText.centerX = this.game.width * 0.5;
            this.add(playerNameText);
            const youHaveBeenAssigned = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_21.LocalizationManager.Instance.text("outro.you_have_been_assigned"), {
                font: Game_26.Game.DEFAULT_FONT,
                fill: Game_26.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_26.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_26.Game.FONT_STROKE_SMALL,
                fontSize: 70,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_10.TextUtil.scaleDownTextToHeight(youHaveBeenAssigned, this.game.height * 0.2);
            youHaveBeenAssigned.centerX = playerNameText.centerX;
            youHaveBeenAssigned.top = playerNameText.bottom + this.game.height * 0.05;
            this.add(youHaveBeenAssigned);
            const roleNameText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_21.LocalizationManager.Instance.text(roleNameKey), {
                font: Game_26.Game.DEFAULT_FONT,
                fill: Game_26.Game.FONT_COLOUR_BLUE,
                stroke: Game_26.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_26.Game.FONT_STROKE_SMALL,
                fontSize: 100,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_10.TextUtil.scaleDownTextToHeight(roleNameText, this.game.height * 0.2);
            roleNameText.centerX = playerNameText.centerX;
            roleNameText.top = youHaveBeenAssigned.bottom + this.game.height * 0.015;
            this.add(roleNameText);
            const roleDescriptionText = new Phaser.Text(game, this.game.width * 0.5, 0, LocalizationManager_21.LocalizationManager.Instance.text(roleDescriptionKey), {
                font: Game_26.Game.DEFAULT_FONT,
                fill: Game_26.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_26.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_26.Game.FONT_STROKE_SMALL,
                fontSize: 70,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            TextUtil_10.TextUtil.scaleDownTextToHeight(roleDescriptionText, this.game.height * 0.2);
            roleDescriptionText.centerX = playerNameText.centerX;
            roleDescriptionText.centerY = roleNameText.bottom + (nextButton.top - roleNameText.bottom) * 0.4;
            this.add(roleDescriptionText);
            this.left = this.game.width;
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_14.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame', 'RoleAssignmentGame', {
                    [SMAnalyticsManager_14.SMAnalyticsManager.GAME_NAME_MINI]: 'RoleAssignment'
                });
                SMAnalyticsManager_14.SMAnalyticsManager.Instance.startTiming('RocketRace:Mini:RoleAssignment', {
                    [SMAnalyticsManager_14.SMAnalyticsManager.GAME_NAME_MINI]: 'RoleAssignment'
                });
                yield _super("show").call(this);
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_14.SMAnalyticsManager.Instance.endTiming('RocketRace:Mini:RoleAssignment', 'RoleAssignment', 'Game', '', {
                    [SMAnalyticsManager_14.SMAnalyticsManager.GAME_NAME_MINI]: 'RoleAssignment'
                });
                yield _super("hide").call(this);
            });
        }
    }
    exports.OutroRoleAssignment = OutroRoleAssignment;
});
define("state/OutroState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "StateNames", "logic/GameController", "component/outro/LoadInstruction", "component/outro/Outro1", "component/outro/Outro2", "component/outro/Outro3", "component/outro/Outro4", "component/outro/OutroRoleAssignment", "state/SpaceBaseState", "config/SpaceMountainConfig", "switchback-common/enum/Park"], function (require, exports, PromiseUtil_28, ScaleUtil_16, StateNames_8, GameController_15, LoadInstruction_1, Outro1_1, Outro2_1, Outro3_1, Outro4_1, OutroRoleAssignment_1, SpaceBaseState_3, SpaceMountainConfig_9, Park_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class OutroState extends SpaceBaseState_3.SpaceBaseState {
        init() {
            this.stateName = StateNames_8.StateNames.OUTRO;
            super.init();
        }
        preload() {
            super.preload();
            // Galaxy BG
            this.game.load.image("galaxy-bg", "assets/cutscene/galaxy_bg.png");
            this.game.load.image("stars", "assets/outro/stars.png");
            // Screen 1
            this.game.load.image("sm-icon", "assets/outro/sm_icon.png");
            this.game.load.image("hyperion-backlit", "assets/outro/hyperion_backlit.png");
            this.game.load.image("hyperion-glow", "assets/outro/hyperion_glow.png");
            this.game.load.image("lightbar", "assets/outro/lightbar_title.png");
            // Screen 2
            this.game.load.image("hyperion-travel", "assets/outro/hyperion_travel.png");
            // Load instructions
            this.game.load.image("instruction-panel-backplate", "assets/outro/instructionpanel.png");
            this.game.load.image("title-lightbar", "assets/outro/lightbar_title.png");
            this.game.load.image("ride-car", `assets/outro/${SpaceMountainConfig_9.SpaceMountainConfig.Instance.park}/ridecar.png`);
            this.game.load.image("ride-glow", `assets/outro/${SpaceMountainConfig_9.SpaceMountainConfig.Instance.park}/ridecar_glow.png`);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const hasMerged = GameController_15.GameController.Instance.state.hasMerged; // This will be set on `LoadingInstructions.ts`
                const hasRaced = GameController_15.GameController.Instance.state.numRaces > 0;
                const roleDetails = [];
                roleDetails.push({ title: "outro.role_pilot", description: "outro.role_pilot_description" });
                roleDetails.push({ title: "outro.role_engineer", description: "outro.role_engineer_description" });
                roleDetails.push({ title: "outro.role_commander", description: "outro.role_commander_description" });
                roleDetails.push({ title: "outro.role_maintenance", description: "outro.role_maintenance_description" });
                roleDetails.push({ title: "outro.role_navigator", description: "outro.role_navigator_description" });
                // shuffle so each guest gets something different each play through
                Phaser.ArrayUtils.shuffle(roleDetails);
                const background = new Phaser.Image(this.game, 0, 0, "galaxy-bg");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                const stars = new Phaser.Image(this.game, 0, 0, "stars");
                stars.width = this.game.width;
                stars.height = this.game.height;
                this.add.existing(stars);
                const hyperionLarge = new Phaser.Image(this.game, 0, 0, "hyperion-backlit");
                ScaleUtil_16.scaleToBounds(hyperionLarge, this.game.width * 0.9, this.game.width * 0.9);
                hyperionLarge.centerX = this.game.width * 0.5;
                hyperionLarge.centerY = this.game.height * 0.98;
                hyperionLarge.alpha = 0;
                this.add.existing(hyperionLarge);
                let screens = [];
                if (hasRaced) {
                    // SWB-1169 Roles are only shown when the user finishes the complete experience
                    // ie. He cannot continue (haven't raced or have merged)
                    const screen1 = new Outro1_1.Outro1(this.game); // Mission complete!
                    this.add.existing(screen1);
                    screens.push(screen1);
                    const screen2 = new Outro2_1.Outro2(this.game); // You have the fastester ship eva
                    screen2.onButtonClickedSignal.addOnce(() => {
                        this.game.add.tween(hyperionLarge).to({ alpha: 0.4 }, 200, Phaser.Easing.Linear.None, true, 1000);
                    });
                    this.add.existing(screen2);
                    screens.push(screen2);
                }
                else {
                    hyperionLarge.alpha = 0.4;
                }
                if (!hasMerged) {
                    // This one shows when the game has been completed for the first time. (So if it already has merged and continued don't show it again)
                    const screen3 = new Outro3_1.Outro3(this.game); // Your ship is ready for take off
                    this.add.existing(screen3);
                    screens.push(screen3);
                }
                if (hasRaced || SpaceMountainConfig_9.SpaceMountainConfig.Instance.park === Park_8.Park.DLR) {
                    // SWB-1169 Roles are only shown when the user finishes the complete experience
                    // ie. He cannot continue (haven't raced or have merged)
                    const screen4 = new Outro4_1.Outro4(this.game);
                    this.add.existing(screen4);
                    screens.push(screen4);
                    for (let player of GameController_15.GameController.Instance.players) {
                        const role = roleDetails.pop();
                        const roleAssignmentScreen = new OutroRoleAssignment_1.OutroRoleAssignment(this.game, player.name, role.title, role.description);
                        screens.push(roleAssignmentScreen);
                    }
                }
                const loadScreen = new LoadInstruction_1.LoadInstruction(this.game);
                this.add.existing(loadScreen);
                screens.push(loadScreen);
                this.dismissOverlay();
                for (let screen of screens) {
                    yield screen.show();
                    yield PromiseUtil_28.PromiseUtil.signal(screen.onButtonClickedSignal);
                    yield screen.hide();
                    screen.destroy();
                }
            });
        }
        transitionToPreRaceState() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                if (GameController_15.GameController.Instance.state.hasMerged) {
                    _super("transitionToPreRaceState").call(this);
                }
                else {
                    // Do nothing
                }
            });
        }
    }
    exports.OutroState = OutroState;
});
define("component/dialog/ISpaceButtonDialog", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("component/dialog/BasicSpaceButtonDialog", ["require", "exports", "component/button/CTAButton", "component/dialog/BasicSpaceDialog"], function (require, exports, CTAButton_4, BasicSpaceDialog_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BasicSpaceButtonDialog extends BasicSpaceDialog_5.BasicSpaceDialog {
        constructor(game, descriptionText, buttonText, shutdownSignal, fontSize, customScale, extra = null) {
            super(game, shutdownSignal, descriptionText, customScale, extra);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.button = new CTAButton_4.CTAButton(this.game, buttonText, fontSize, customScale, this.shutdownSignal);
            this.button.centerX = this.dialogText.centerX;
            this.button.top = this.content.bottom + (this.game.height * 0.025);
            this.content.add(this.button);
            this.resizeBackground();
        }
        setTextWidth(width) {
            super.setTextWidth(width);
            this.button.centerX = this.dialogText.centerX;
            this.button.top = this.dialogText.bottom + this.game.height * 0.05;
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                this.button.show();
                this.button.onButtonClickedSignal.addOnce(() => {
                    this.onButtonClickedSignal.dispatch();
                    this.game.sound.play("cta-button");
                });
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.button.hide();
                this.button.onButtonClickedSignal.removeAll();
                yield _super("hide").call(this);
            });
        }
        destroy() {
            this.button.onButtonClickedSignal.removeAll();
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.BasicSpaceButtonDialog = BasicSpaceButtonDialog;
});
define("component/dialog/BasicSpaceLandscapeDialog", ["require", "exports", "component/dialog/BasicSpaceButtonDialog"], function (require, exports, BasicSpaceButtonDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BasicSpaceLandscapeDialog extends BasicSpaceButtonDialog_1.BasicSpaceButtonDialog {
        constructor(game, descriptionText, buttonText, fontSize, shutdownSignal) {
            super(game, descriptionText, buttonText, shutdownSignal, fontSize, true);
            this.content.angle = -90;
            this.dialogText.align = 'center';
            this.setTextWidth(this.game.height * 0.6);
            this.dialogText.centerX = this.button.centerX;
            const resizeByContentHeight = false; // we want to resize by content width
            this.resizeBackground(resizeByContentHeight);
        }
    }
    exports.BasicSpaceLandscapeDialog = BasicSpaceLandscapeDialog;
});
/**
 * @author ewan@Cerebralfix.com
 */
define("minigame/asteroids/LaneObject", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LaneObject extends Phaser.Group {
    }
    exports.LaneObject = LaneObject;
});
define("minigame/asteroids/Lane", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "Game"], function (require, exports, PromiseUtil_29, ScaleUtil_17, LocalizationManager_22, Game_27) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@Cerebralfix.com
     * @author Jeronimo@CerebralFix.com
     */
    class Lane extends Phaser.Group {
        constructor(game, startY) {
            super(game);
            this.laneObjects = []; // list of asteroids in the lane
            this.startingY = startY;
            this.warningIcon = new Phaser.Image(this.game, this.game.width * 0.27, this.startingY + (this.game.height / 3) / 2, "asteroid-incoming-alert");
            ScaleUtil_17.scaleToBounds(this.warningIcon, this.game.width * 0.3, this.game.height * 0.3);
            this.warningIcon.anchor.setTo(0.5);
            this.warningIcon.angle = -90;
            this.warningIcon.visible = false;
            // this.game.add.existing(this.warningIcon);
            this.warningText = new Phaser.Text(this.game, 0, 0, LocalizationManager_22.LocalizationManager.Instance.text("asteroids.tutorial_incoming").toUpperCase(), {
                font: Game_27.Game.DEFAULT_FONT,
                fill: "#ff4e00",
                fontSize: 55,
                align: 'center',
            });
            this.warningText.anchor.set(0.5);
            this.warningText.angle = -90;
            this.warningText.visible = false;
            this.warningText.centerY = this.warningIcon.centerY;
            this.warningText.left = this.warningIcon.right - this.game.width * 0.15;
            const laneImage = new Phaser.Image(this.game, 0, 0, 'asteroids-lane');
            laneImage.angle = -90;
            laneImage.y = this.startingY;
            laneImage.width = this.game.width * 0.03;
            laneImage.height = this.game.width;
            this.add(this.warningText);
            this.add(this.warningIcon);
            this.add(laneImage);
            this.timer = this.game.time.create(false);
        }
        stopWarning() {
            this.timer.stop();
            // ensure the warning icon remains off
            this.warningText.visible = this.warningIcon.visible = false;
        }
        /*
         * Create and add the warning icon, rotate it, etc.
         * Start flickering loop event which flickers the warning icon on and off every 250 ms.
         * After a given time, flickering is over and the asteroid can move.
         * @param duration the duration of the warning in ms
         */
        showWarning(duration) {
            return __awaiter(this, void 0, void 0, function* () {
                // The following flow allows for multiple calls to show warning and still:
                // 1 have a nice and solid animation
                // 2 can be awaited for the exact time so the items are synced
                // 3 can be called multiple times and the warning will show until the last element is done.
                // In case there was a previous showing, clears it
                this.stopWarning();
                this.warningText.visible = this.warningIcon.visible = true;
                const flickTime = 150;
                const loops = Math.max(0, Math.floor(duration / flickTime));
                // add a NEW event that will stop in the correct time but don't bind the waiting here
                this.timer.repeat(flickTime, loops, this.flickerWarning, this);
                // This is important, otherwise we will have a bug.
                // 2 asteroids spawned too close might leave the second one with the lane with warning = false
                // wait a little before disabling the warning icon - this prevents stars from spawning too close to the asteroids
                // notice that this is not being awaited
                this.timer.add(duration + 100, this.stopWarning, this);
                this.timer.start();
                // await this, to ensure that the return time of this function is constant, while allowing to trigger this
                // function many times before one ends
                yield PromiseUtil_29.PromiseUtil.time(this.game, duration);
            });
        }
        flickerWarning() {
            this.warningText.visible = this.warningIcon.visible = !this.warningIcon.visible;
        }
        /**
         * Adds an item to the list of lane objects in this lane
         *
         * @param item - the item to add
         */
        addItem(item) {
            this.laneObjects.push(item);
            this.add(item);
        }
        /**
         * Destroys the object in the lane
         *
         * @param laneObject - The object to destroy
         */
        destroyItem(laneObject) {
            let index = this.laneObjects.indexOf(laneObject);
            if (index !== -1) {
                this.laneObjects.splice(index, 1);
            }
            laneObject.destroy();
        }
        clear() {
            for (let lo of this.laneObjects) {
                lo.destroy();
            }
            this.laneObjects = [];
        }
        destroy() {
            if (this.timer) {
                this.timer.destroy();
                this.timer = null;
            }
            if (this.warningIcon) {
                this.warningIcon.destroy();
                this.warningIcon = null;
            }
            this.laneObjects = [];
            this.warningText = null;
            super.destroy();
        }
    }
    exports.Lane = Lane;
});
/**
 * @author ewan@Cerebralfix.com
 * @author joel@Cerebralfix.com
 */
define("minigame/asteroids/AsteroidsShip", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/extension/Easing", "component/ship/ShipSprite"], function (require, exports, PromiseUtil_30, ScaleUtil_18, Easing_2, ShipSprite_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ShipState;
    (function (ShipState) {
        ShipState[ShipState["VULNERABLE"] = 0] = "VULNERABLE";
        ShipState[ShipState["IMMUNE"] = 1] = "IMMUNE";
    })(ShipState = exports.ShipState || (exports.ShipState = {}));
    class AsteroidsShip extends Phaser.Group {
        constructor(game, tint, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.scoreUpdateSignal = new Phaser.Signal();
            this.ship = new ShipSprite_3.ShipSprite(game);
            this.ship.width = this.game.height * 0.125;
            this.ship.scale.y = this.ship.scale.x;
            this._hitbox = new Phaser.Image(game, 0, 0, '');
            this._hitbox.anchor.setTo(0.5);
            this.add(this._hitbox);
            this._hitbox.width = this.ship.width * 0.75;
            this._hitbox.height = this.ship.height * 0.75;
            this.setupShield(tint);
            this.trail = new Phaser.Sprite(game, 0, 0, 'asteroids-trail', 0);
            this.trail.animations.add('fuelcell-trail');
            this.trail.play('fuelcell-trail', 24, true);
            this.trail.width = this.ship.width * 0.1;
            this.trail.angle = 180;
            this.trail.anchor.set(0.5, 1);
            this.trail.tint = tint;
            this.add(this.trail);
            this.add(this.ship);
            this.add(this.shield);
            this.trail.x = this.ship.centerX;
            this.trail.y = this.ship.centerY;
            this.streak = new Phaser.Image(game, 0, 0, 'asteroids-streak');
            this.streak.anchor.set(0.5, 0);
            this.streak.alpha = 0.5;
            this.streak.top = this.ship.top;
            this.streak.x = this.ship.centerX;
            this.streak.width = this.ship.width * 1.2;
            this.streak.height = this.ship.height * 3;
            this.add(this.streak);
            this.currentState = ShipState.VULNERABLE; // set the ship to vulnerable
            this.angle = -90;
            this.health = AsteroidsShip.BASE_HEALTH;
            this._score = 0;
            this.game.add.tween(this.streak.scale).to({ x: this.streak.scale.x * 1.1 }, 100, Easing_2.shake(100, 100), true, 0, Infinity, true);
        }
        setupShield(tint) {
            this.shield = new Phaser.Sprite(this.game, 0, 0, 'ship-shield');
            this.shield.anchor.set(0.5);
            this.shield.tint = tint;
            this.shieldActive = true;
            ScaleUtil_18.scaleToBounds(this.shield, this.ship.width * 1.4, this.ship.height * 1.4);
            this.game.add.tween(this.shield).to({ angle: this.shield.angle + 360 }, 5000, Phaser.Easing.Linear.None, true, 0, Infinity);
        }
        /**
         * Flicker the ship every 250ms
         *
         * @param totalTime - the duration of flicker
         */
        flicker(totalTime) {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < totalTime / 250; i++) {
                    this.visible = !this.visible;
                    yield PromiseUtil_30.PromiseUtil.time(this.game, 250, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                }
                this.visible = true;
            });
        }
        /**
         * If the ship is in a vulnerable state, decrement its health,
         * enable temporary immunity, begin flickering for 1.5 seconds and
         * then remove immunity.
         */
        takeDamage() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.currentState !== ShipState.VULNERABLE) {
                    return;
                }
                this.health--;
                const hitSound = this.game.rnd.integerInRange(0, AsteroidsShip.NUM_SFX_HIT - 1);
                this.game.sound.play('asteroids-sfx-hit-asteroid-' + hitSound); // not really sure this still needs to be played, so i kept it just in case,
                // the problem is that it's gonna play along with the shield down, so probably will not sound ok
                // if the ship is dead, don't don't set immunity, just end it
                if (this.health === 0) {
                    return;
                }
                this.game.sound.play('asteroids-sfx-shield-down');
                this.shieldActive = false;
                this.shield.destroy();
                this.setTemporaryImmunity();
            });
        }
        /**
         * This puts the ship in a temporary state of immunity.
         * Used when the ship collides with an asteroid to prevent constant damage.
         */
        setTemporaryImmunity() {
            return __awaiter(this, void 0, void 0, function* () {
                this.currentState = ShipState.IMMUNE;
                this.visible = false;
                yield this.flicker(1500);
                if (this.isShutdown) {
                    return;
                }
                this.removeImmunity();
            });
        }
        /**
         * Lift the immunity of the ship after immunity timer runs out.
         * Puts the ship in a state of vulnerability.
         */
        removeImmunity() {
            this.currentState = ShipState.VULNERABLE; // set the ship back to vulnerable state
            this.game.time.events.remove(this.flickerTimerEvent); // remove the flicker event loop
            this.visible = true;
        }
        respawn(tint) {
            this.setupShield(tint);
            this.add(this.shield);
            this.trail.tint = tint; // change tint of the trail too
            this.health = AsteroidsShip.BASE_HEALTH;
        }
        /**
         * Returns true if the ship has run out of health.
         */
        isDead() {
            return this.health === 0;
        }
        /**
         * Increment the Ship's score
         */
        increaseScore() {
            this._score++;
            this.scoreUpdateSignal.dispatch();
        }
        /**
         * Returns the current score
         */
        get score() {
            return this._score;
        }
        getShip() {
            return this.ship;
        }
        get isShieldActive() {
            return this.shieldActive;
        }
        get hitbox() {
            return this._hitbox;
        }
        getCurrentLane() {
            return this.currentLane;
        }
        setCurrentLane(lane) {
            this.currentLane = lane;
        }
        destroy() {
            this.shutdownSignal = null;
            this.scoreUpdateSignal.dispose();
            this.scoreUpdateSignal = null;
            super.destroy();
        }
    }
    AsteroidsShip.NUM_SFX_HIT = 3; // How many sfxs for hitting we have
    AsteroidsShip.BASE_HEALTH = 2;
    exports.AsteroidsShip = AsteroidsShip;
});
/**
 * @author ewan@Cerebralfix.com
 */
define("minigame/asteroids/Asteroid", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "minigame/asteroids/LaneObject", "switchback-common/extension/Easing"], function (require, exports, PromiseUtil_31, ScaleUtil_19, LaneObject_1, Easing_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Asteroid extends LaneObject_1.LaneObject {
        constructor(game, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.visible = false;
            this.asteroidArray = [];
            this.asteroidRotationTweens = [];
            this.asteroid1 = new Phaser.Image(game, 0, 0, 'asteroids-ast1');
            this.asteroid2 = new Phaser.Image(game, 0, 0, 'asteroids-ast2');
            this.asteroid3 = new Phaser.Image(game, 0, 0, 'asteroids-ast2');
            this.asteroid4 = new Phaser.Image(game, 0, 0, 'asteroids-ast3');
            this.asteroid5 = new Phaser.Image(game, 0, 0, 'asteroids-ast4');
            this.asteroidArray.push(this.asteroid1);
            this.asteroidArray.push(this.asteroid2);
            this.asteroidArray.push(this.asteroid3);
            this.asteroidArray.push(this.asteroid4);
            this.asteroidArray.push(this.asteroid5);
            for (let ast of this.asteroidArray) {
                ast.anchor.set(0.5);
                ast.angle = game.rnd.integerInRange(0, -90);
                this.asteroidRotationTweens.push(game.add.tween(ast).to({ angle: ast.angle + 360 }, 10000, Phaser.Easing.Linear.None, true, 0, Infinity));
            }
            this.streak = new Phaser.Image(game, 0, 0, 'asteroids-streak');
            this.streak.anchor.set(0.5);
            this.streak.angle = 90;
            this.streak.tint = 0xf11217;
            this.hitbox = new Phaser.Image(game, 0, 0, '');
            this.add(this.hitbox);
            this.add(this.streak);
            this.add(this.asteroid1);
            this.add(this.asteroid2);
            this.add(this.asteroid3);
            this.add(this.asteroid4);
            this.add(this.asteroid5);
            // set the position of the cluster
            this.asteroid2.right = this.asteroid1.width * 0.3;
            this.asteroid2.bottom = this.asteroid1.top + this.asteroid1.height * 0.3;
            this.asteroid3.right = this.asteroid1.width * 0.1;
            this.asteroid3.top = this.asteroid1.bottom - this.asteroid1.height * 0.2;
            this.asteroid4.right = this.asteroid3.left + this.asteroid3.width * 0.1;
            this.asteroid4.centerY = this.asteroid1.height * 0.4;
            this.asteroid5.right = this.asteroid3.left;
            this.asteroid5.y = this.asteroid2.bottom;
            this.streak.width = this.width * 0.5;
            this.streak.height = this.height * 3;
            this.streak.right = this.asteroid5.left - this.width * 0.09;
            this.streakTween = this.game.add.tween(this.streak.scale)
                .to({ x: this.streak.scale.x * 1.1 }, 100, Easing_3.shake(100, 100), true, 0, Infinity, true);
            this.hitbox.width = this.width * 0.3;
            this.hitbox.height = this.height * 0.6;
            this.hitbox.x = this.streak.right;
            this.hitbox.centerY = this.streak.centerY;
        }
        /**
         * Calls the takeDamage function of the ship
         * that this asteroid collided with
         *
         * @param ship - the ship that this asteroid collided with
         */
        collision(ship, intersectionPoint) {
            this.showBurst(intersectionPoint);
            ship.takeDamage();
            this.game.camera.shake(0.01, 250);
        }
        /**
         * Stops all the animations on this asteroid.
         * This function is used for the tutorial when
         * freezing the asteroid while the player makes a decision.
         */
        removeAnimations() {
            this.streakTween.pause();
            this.asteroidRotationTweens.forEach((tween) => {
                tween.pause();
            });
        }
        /**
         * Continues the animations on this asteroid.
         * This function is used during the tutorial
         * after the player makes a decision and the asteroid
         * passes by.
         */
        unpauseAnimations() {
            this.streakTween.resume();
            this.asteroidRotationTweens.forEach((tween) => {
                tween.resume();
            });
        }
        showBurst(intersectionPoint) {
            return __awaiter(this, void 0, void 0, function* () {
                const burst = new Phaser.Image(this.game, intersectionPoint.x, intersectionPoint.y, "asteroid-collision");
                ScaleUtil_19.scaleToBounds(burst, 0, 0);
                burst.anchor.setTo(0.5);
                this.game.add.existing(burst);
                let isShutdown = false;
                yield PromiseUtil_31.PromiseUtil.tween(this.game, this.game.add.tween(burst.scale).to({ x: 1.5, y: 1.5 }, 60, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                burst.destroy();
            });
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.Asteroid = Asteroid;
});
/**
 * A simple timer that avoids many of the problems with the timer from Phaser
 * Even though it looks unnervingly simple, that's actually the design of it.
 * The code is solid, and works logically as it would be expected to,
 * and it has as minimal overhead as possible. So it's efficient, lightweight
 * and easy to understand, debug and maintain.
 * It doesn't depend on Phaser nor es6 or anything fancy, that'll make it reusable,
 * solid and predictable as it possible.
 *
 * @author Joel@CerebralFix.com
 * @author Jeronimo@CerebralFix.com
 */
define("switchback-common/util/SimpleTimer", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SimpleTimer {
        constructor(delay = 0, callback = null) {
            this.onTimer = null;
            this.delay = 0;
            this.elapsed = 0;
            this.stopped = true;
            this.delay = delay;
            this.onTimer = callback;
        }
        stop() {
            this.stopped = true;
        }
        start() {
            this.elapsed = 0;
            this.stopped = false;
        }
        update(dt) {
            if (this.stopped) {
                return;
            }
            this.elapsed += dt;
            // Doing it this way ensures that it will be triggered only once per frame,
            // but overall the accuracy of the timer is maximized even during frame drops
            // which is the whole reason of using dt
            if (this.elapsed > this.delay) {
                this.elapsed -= this.delay;
                if (this.onTimer) {
                    this.onTimer();
                }
            }
        }
    }
    exports.SimpleTimer = SimpleTimer;
});
define("minigame/asteroids/FuelCellPool", ["require", "exports", "minigame/asteroids/FuelCell"], function (require, exports, FuelCell_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A pool for animating fuel cells
     */
    class FuelCellPool extends Phaser.Group {
        getNewFuelCellIcon() {
            let newFuelCellIcon = this.getFirstDead();
            if (newFuelCellIcon) {
                newFuelCellIcon.revive();
            }
            else {
                newFuelCellIcon = new FuelCell_1.FuelCellPickUpIcon(this.game, 0, 0, "asteroids-pickup");
                newFuelCellIcon.anchor.set(0.5);
                newFuelCellIcon.angle = -90;
                this.add(newFuelCellIcon);
            }
            return newFuelCellIcon;
        }
    }
    exports.FuelCellPool = FuelCellPool;
});
/**
 * @author ewan@Cerebralfix.com
 */
define("minigame/asteroids/FuelCell", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "minigame/asteroids/LaneObject", "minigame/asteroids/AsteroidsGameState"], function (require, exports, PromiseUtil_32, ScaleUtil_20, LaneObject_2, AsteroidsGameState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class FuelCell extends LaneObject_2.LaneObject {
        constructor(game, fuelCellPool, shutdownSignal) {
            super(game);
            this.fuelCellPool = fuelCellPool;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.fuelCell = new Phaser.Sprite(game, 0, 0, 'asteroids-pickup');
            this.fuelCell.anchor.setTo(0.5);
            const fuelCellScanner = new Phaser.Sprite(game, 0, 0, 'asteroids-fuelcell-scanner');
            fuelCellScanner.anchor.set(0.5);
            game.add.tween(fuelCellScanner).to({ angle: fuelCellScanner.angle + 360 }, 15000, Phaser.Easing.Linear.None, true, 0, Infinity);
            this.add(fuelCellScanner);
            this.add(this.fuelCell);
            fuelCellScanner.centerX = this.fuelCell.centerX;
            fuelCellScanner.centerY = this.fuelCell.centerY;
            this.trail = new Phaser.Sprite(game, 0, 0, 'asteroids-trail', 0);
            this.trail.width = game.width * 0.01;
            this.trail.tint = 0x27c7f9;
            this.add(this.trail);
            this.trail.bottom = this.fuelCell.top;
            this.trail.centerX = this.fuelCell.centerX;
            this.trail.animations.add('fuelcell-trail');
            this.trail.play('fuelcell-trail', 24, true);
            this.angle = -90;
            this.shouldPulse = true;
            this.hitbox = new Phaser.Image(game, 0, 0, '');
            this.hitbox.anchor.setTo(0.5);
            this.add(this.hitbox);
            this.hitbox.width = this.fuelCell.width;
            this.hitbox.height = this.fuelCell.height;
            this.pulse();
        }
        pulse() {
            return __awaiter(this, void 0, void 0, function* () {
                let originalScaleX = this.fuelCell.scale.x;
                let originalScaleY = this.fuelCell.scale.y;
                while (this.shouldPulse) {
                    let tweens = [
                        PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(this.fuelCell.scale).to({ x: originalScaleX * 1.1, y: originalScaleY * 1.1 }, 30, Phaser.Easing.Linear.None, true, 0, 0, true), this.shutdownSignal),
                        PromiseUtil_32.PromiseUtil.time(this.game, FuelCell.PULSE_DELAY_MS, this.shutdownSignal)
                    ];
                    yield Promise.all(tweens).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                }
            });
        }
        collision(ship, intersectionPoint) {
            return __awaiter(this, void 0, void 0, function* () {
                this.shouldPulse = false; // disable pulsing
                this.game.sound.play('asteroids-sfx-collect-fuel'); // play the sound of collision
                let tempFuelCell = this.fuelCellPool.getNewFuelCellIcon();
                tempFuelCell.init(this.x, this.y);
                this.showBurst(intersectionPoint);
                this.isShutdown = !(yield tempFuelCell.tweenToScore(this.game, this.shutdownSignal));
                if (this.isShutdown) {
                    return;
                }
                ship.increaseScore();
            });
        }
        showBurst(intersectionPoint) {
            return __awaiter(this, void 0, void 0, function* () {
                const burst = new Phaser.Image(this.game, intersectionPoint.x, intersectionPoint.y, "fuelcell-collision");
                ScaleUtil_20.scaleToBounds(burst, 0, 0);
                burst.anchor.setTo(0.5);
                this.game.add.existing(burst);
                yield PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(burst.scale).to({ x: 1.5, y: 1.5 }, 60, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                burst.destroy();
            });
        }
        destroy() {
            this.fuelCellPool = null;
            this.shouldPulse = false;
            this.shutdownSignal = null;
            super.destroy();
        }
        /**
         * Pauses the animations and tweens on this fuelcell.
         * This function is used for the tutorial.
         */
        removeAnimations() {
            this.shouldPulse = false;
            this.game.tweens.removeFrom(this);
            this.trail.animations.stop();
        }
    }
    FuelCell.PULSE_DELAY_MS = 600;
    exports.FuelCell = FuelCell;
    class FuelCellPickUpIcon extends Phaser.Image {
        init(x, y) {
            this.x = x;
            this.y = y;
        }
        tweenToScore(game, shutdownSignal) {
            return __awaiter(this, void 0, void 0, function* () {
                let isShutdown = false;
                yield PromiseUtil_32.PromiseUtil.tween(game, game.add.tween(this).to({ x: game.width * AsteroidsGameState_1.AsteroidsGameState.SCORE_X_POS, y: game.height * AsteroidsGameState_1.AsteroidsGameState.SCORE_Y_POS }, 500, Phaser.Easing.Linear.None, true), shutdownSignal).catch(() => { isShutdown = true; });
                this.kill();
                return !isShutdown;
            });
        }
    }
    exports.FuelCellPickUpIcon = FuelCellPickUpIcon;
});
define("minigame/asteroids/ItemSpawner", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/util/SimpleTimer", "minigame/asteroids/Asteroid", "minigame/asteroids/FuelCell"], function (require, exports, PromiseUtil_33, ScaleUtil_21, SimpleTimer_1, Asteroid_1, FuelCell_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@Cerebralfix.com
     * @author Jeronimo@CerebralFix.com
     */
    /*
    The flow is quite simple, yet important to get it right in order to tweak correctly.
    
        So timing works like this:
        Time |----5|====|--------10|     >
             attemptSpawn
             +TimeToShowWarning (for all the items)
                   +Random Delay
                        +TimeOnScreen
             |--------10|----5|=|--------10|
             +SpawnTime |
                        |--------10|----5|==|--------10|
                                   |--------10|
    
    The idea is that every chunk is spawned next to each other. Consecutively.
    That's why initially the TimeOnScreen === SpawnTime.
    
    Before visually spawning, the warning time is added to every item.
    By being all the items offset by the same time, sync and distance is correctly maintained and controlled.
    The warning time is constant. This way it can be a huge or small time without affecting the spacing of items (which is essential).
    Because the warning time will be larger than the spawning time sooner.
    Allowing this way to reduce spawn time to its minimum, while being controllable and with no bugs.
     So warning time can be managed separately and have huge times without affecting really the spacing of items.
    
    Then a random delay is added, to each item. That offsets its initial position. The amount of items per time will stay the same
    just some items will start behind. Its important that the delay time will be smaller than the spawn time,
    otherwise, items will overlap.
    */
    var ItemType;
    (function (ItemType) {
        ItemType[ItemType["NONE"] = 0] = "NONE";
        ItemType[ItemType["ASTEROID"] = 1] = "ASTEROID";
        ItemType[ItemType["STAR"] = 2] = "STAR";
    })(ItemType = exports.ItemType || (exports.ItemType = {}));
    class ItemSpawner {
        constructor(game, lanes, difficultyLevel, fuelCellPool, shutdownSignal) {
            // Difficulty factor [0..1..] 0 = Min, 1 = max, can grow further
            this.difficulty = 0;
            this.warningTimeMS = ItemSpawner.MAX_WARNING_TIME_MS;
            // Used to calculate the score, as there's pause and resume in the game this is the most reliable place i've found
            // without having to create a new timer
            this.spawnClockCount = 0;
            this.lastTimeStarDrop = 0;
            this.game = game;
            this.lanes = lanes;
            this.difficultyLevel = difficultyLevel;
            this.fuelCellPool = fuelCellPool;
            this.shutdownSignal = shutdownSignal;
            this.spawnClockCount = 0;
            this.spawningEnabled = false;
            this.lastTimeStarDrop = 0;
            this.elapsed = 0;
            this.timer = new SimpleTimer_1.SimpleTimer(0, this.onTimer.bind(this));
            this.resetDifficulty();
        }
        //////////////////
        //// Timer stuff
        onTimer() {
            this.tryToSpawnRow();
        }
        stopSpawning() {
            this.spawningEnabled = false;
            this.timer.stop();
            for (let lane of this.lanes) {
                lane.stopWarning();
            }
        }
        // Starts spawning stuff. Purposely doesn't reset the difficulty
        startSpawning() {
            this.stopSpawning(); // clear previous events to avoid bugs
            this.elapsed = 0;
            this.spawningEnabled = true;
            // This will trigger as soon as the next update.
            // Otherwise it would wait one `delay` before the first trigger, effectively making the user wait for 1 spawn
            this.timer.delay = 0;
            this.timer.start();
        }
        update() {
            const dt = this.game.time.elapsedMS;
            this.elapsed += dt;
            this.lastTimeStarDrop += dt;
            this.timer.update(dt);
        }
        //// End Timer stuff
        //////////////////
        getElapsed() {
            return this.elapsed;
        }
        /**
         * Add the item to the game and move it across the screen, then, destroys it.
         * It dispatches the itemSpawned signal with the item received as parameter
         * @param item : Item to add
         * @param laneNumber : lane to add to
         */
        addSpawnedItem(item, laneNumber) {
            return __awaiter(this, void 0, void 0, function* () {
                // important because of the way FuelCell creates it's sprite (it adds them directly to game on the constructor)
                item.visible = false;
                let lane = this.lanes[laneNumber];
                // set a delay so that they don't all spawn at once
                // by using this.spawnTimeMS we ensure that no items will step over each other
                // spawnTimeMS already accounts for ITEMS_ON_LANE
                // so this is perfect, we need to change less variables to keep all the relations in order
                let delay = this.game.rnd.between(0, this.spawnTimeMS * ItemSpawner.ITEM_DELAY_FACTOR);
                yield PromiseUtil_33.PromiseUtil.time(this.game, delay, this.shutdownSignal).catch(() => { this.isShutdown = true; }); // wait a little before spawning next item
                if (this.isShutdown) {
                    return;
                }
                if (!this.spawningEnabled) {
                    item.destroy();
                    return;
                }
                lane.addItem(item); // add the item to the lane
                item.visible = true;
                item.x = 0;
                item.centerY = lane.startingY + ((this.game.height / 3) / 2);
                // tween the item across the lane
                yield PromiseUtil_33.PromiseUtil.tween(this.game, this.game.add.tween(item).to({ x: this.game.width + 100 }, this.itemTimeOnScreenMS, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // destroy the item from the lane
                lane.destroyItem(item);
            });
        }
        /*
            Spawns an asteroid
            Shows a temporal warning then shows the asteroid moving across and finally will be destroyed
        */
        spawnAsteroid(laneNumber) {
            return __awaiter(this, void 0, void 0, function* () {
                let lane = this.lanes[laneNumber];
                // create a warning in the lane before spawning the asteroid
                yield lane.showWarning(this.warningTimeMS);
                if (this.isShutdown) {
                    return;
                }
                // Check that the player didn't die while we were waiting
                if (!this.spawningEnabled) {
                    return;
                }
                let ast = new Asteroid_1.Asteroid(this.game, this.shutdownSignal);
                ScaleUtil_21.scaleToBounds(ast, this.game.height * 0.55, this.game.width * 0.6);
                yield this.addSpawnedItem(ast, laneNumber);
            });
        }
        /*
            Spawns an asteroid
            Unless the warning sign is active
            Shows a temporal warning then shows the asteroid moving across and finally will be destroyed
        */
        spawnStar(laneNumber) {
            return __awaiter(this, void 0, void 0, function* () {
                // Synchronize with asteroids
                // notice we do this before checking for the warning icon
                yield PromiseUtil_33.PromiseUtil.time(this.game, this.warningTimeMS);
                // if there isn't a warning icon active, spawn the star
                let star = new FuelCell_2.FuelCell(this.game, this.fuelCellPool, this.shutdownSignal);
                ScaleUtil_21.scaleToBounds(star, this.game.height * 0.4, this.game.width * 0.4);
                yield this.addSpawnedItem(star, laneNumber);
            });
        }
        /*
            Attempts to spawn a full row.
            Increases the difficulty.
            This is a core function on the loop
         */
        tryToSpawnRow() {
            // Counts the clock (for score keeping (a.k.a. distance)) 
            // notice this doesn't care about if it should spawn or not, its used only to measure the played time
            this.spawnClockCount++;
            this.setDifficulty();
            /// choose which row configuration to spawn
            // a list of how many possible spawns. how many configurations are there
            const numSpawns = ItemSpawner.POSSIBLE_SPAWNS_PER_DIFFICULTY.length - 1; // -1 for maxItemSpawnCount = []
            // how many can we use now
            let possibleSpawnsCount = Math.floor(this.difficulty * numSpawns);
            possibleSpawnsCount = Math.min(numSpawns, Math.max(0, possibleSpawnsCount)); // wrapped
            // how many possible spawns can we actually use
            let maxItemSpawnCount = ItemSpawner.POSSIBLE_SPAWNS_PER_DIFFICULTY[possibleSpawnsCount];
            // which rows are actually possible to use
            let possibleSpawns = ItemSpawner.POSSIBLE_ASTEROIDS_SPAWNS.slice(0, maxItemSpawnCount);
            // choose a spawn config
            // bug-fix: slice here is super important, otherwise the original array will be filled with stars
            let itemSpawns = this.game.rnd.pick(possibleSpawns).slice();
            /// ~ end row configuration
            // loop over the itemSpawns and decide whether we can replace the empty lanes with stars (based on star drop rates)
            // itemOrder holds the indexes of itemSpawns in a random order, so we can traverse it randomly while not
            // shuffling the original array which would mess with the difficulty
            let itemOrder = Phaser.ArrayUtils.numberArray(0, itemSpawns.length - 1);
            Phaser.ArrayUtils.shuffle(itemOrder);
            for (let i of itemOrder) {
                if (itemSpawns[i] === ItemType.NONE && this.shouldSpawnStar()) {
                    this.lastTimeStarDrop = 0;
                    itemSpawns[i] = ItemType.STAR;
                }
            }
            // and spawn the asteroids in the lane they're supposed to be spawned in (according to the chosen spawns)
            for (let i = 0; i < itemSpawns.length; i++) {
                // Check that the player didn't die while we were waiting
                if (!this.spawningEnabled) {
                    return;
                }
                // notice the spawn functions are NOT being awaited (important to keep the sync)
                if (itemSpawns[i] === ItemType.ASTEROID) {
                    this.spawnAsteroid(i);
                }
                else if (itemSpawns[i] === ItemType.STAR) {
                    this.spawnStar(i);
                }
            }
        }
        /**
         * Decide whether we should spawn a star
         */
        shouldSpawnStar() {
            if (this.lastTimeStarDrop < ItemSpawner.STAR_MIN_DROP_TIME_MS) {
                return false;
            }
            if (this.lastTimeStarDrop > ItemSpawner.STAR_MAX_DROP_TIME_MS) {
                return true;
            }
            return this.game.rnd.frac() < ItemSpawner.STAR_DROP_RATE;
        }
        /**
         * Sets the correct difficulty based on the time spent
         * Sets the speed and next spawn
         */
        setDifficulty() {
            // Steven said: Difficulty shouldn't be capped, if a person is good enough to play at highest difficulty, he will be playing forever
            this.difficulty = (this.elapsed + ItemSpawner.GAME_DURATION_OFFSET[this.difficultyLevel]) / ItemSpawner.TIME_TO_MAX_DIFFICULTY_MS;
            this.warningTimeMS = Math.max(0, Phaser.Math.linear(ItemSpawner.MAX_WARNING_TIME_MS, ItemSpawner.MIN_WARNING_TIME_MS, this.difficulty));
            this.itemTimeOnScreenMS = Math.max(0, Phaser.Math.linear(ItemSpawner.MAX_TIME_ON_SCREEN_MS, ItemSpawner.MIN_TIME_ON_SCREEN_MS, this.difficulty));
            this.spawnTimeMS = this.itemTimeOnScreenMS / ItemSpawner.ITEMS_ON_LANE;
            this.timer.delay = this.spawnTimeMS;
        }
        /**
         * Resets the difficulty to the base.
         * Call this function when the player dies and passes to next.
         */
        resetDifficulty() {
            this.difficulty = 0;
        }
        /**
         * Destroys the things that aren't needed anymore
         * e.g. TimerEvents
         */
        shutdown() {
            this.shutdownSignal = null;
            this.timer = null;
            this.fuelCellPool = null;
            for (let lane of this.lanes) {
                lane.destroy();
            }
            this.lanes = [];
            this.game = null;
            this.difficultyLevel = null;
        }
    }
    // How long to show a warning for before the asteroid enters the screen
    ItemSpawner.MIN_WARNING_TIME_MS = 500; // Tweak ME Please!
    ItemSpawner.MAX_WARNING_TIME_MS = ItemSpawner.MIN_WARNING_TIME_MS + 2500; // Tweak ME Please!
    // This controls how close is each spawn, this determines how many items will be at the same lane at the same time.
    // Higher number will spawn items more tightly, but careful, it could conflict with the extra spawn delay
    // Lower value will make items more spaced out.
    ItemSpawner.ITEMS_ON_LANE = 1.3333; // Tweak ME Please!
    // Used to offset the items within a random space, should be <1 otherwise it could collide with the next spawned item
    ItemSpawner.ITEM_DELAY_FACTOR = 0.5;
    // Controls the speed of each item
    // The fastest an item can move across the screen
    ItemSpawner.MIN_TIME_ON_SCREEN_MS = 1000; // Tweak ME Please!
    // The slowest an item can move across the screen
    ItemSpawner.MAX_TIME_ON_SCREEN_MS = ItemSpawner.MIN_TIME_ON_SCREEN_MS + 3000; // Tweak ME Please!
    // Percentage chance to spawn a star on an empty spot (0-1)
    ItemSpawner.STAR_DROP_RATE = 0.45; // Tweak ME Please!
    // Minimum time before a star spawns
    ItemSpawner.STAR_MIN_DROP_TIME_MS = 500; // Tweak ME Please!
    // Maximum time before a star is forced to spawn
    ItemSpawner.STAR_MAX_DROP_TIME_MS = ItemSpawner.STAR_MIN_DROP_TIME_MS + 1500; // Tweak ME Please!
    // This array is sorted strategically to work with POSSIBLE_SPAWNS_PER_DIFFICULTY
    ItemSpawner.POSSIBLE_ASTEROIDS_SPAWNS = [
        [ItemType.NONE, ItemType.ASTEROID, ItemType.NONE],
        [ItemType.NONE, ItemType.NONE, ItemType.NONE],
        [ItemType.NONE, ItemType.NONE, ItemType.ASTEROID],
        [ItemType.ASTEROID, ItemType.NONE, ItemType.NONE],
        [ItemType.ASTEROID, ItemType.NONE, ItemType.ASTEROID],
        [ItemType.ASTEROID, ItemType.ASTEROID, ItemType.NONE],
        [ItemType.NONE, ItemType.ASTEROID, ItemType.ASTEROID]
    ];
    // This array sets how many possibilities from the POSSIBLE_ASTEROIDS_SPAWNS array, as the difficulty increases more of those
    // combinations will be used. That's why the easiest combinations are first
    ItemSpawner.POSSIBLE_SPAWNS_PER_DIFFICULTY = [2, 4, 5, 5, 7, 7]; // Tweak ME Please!
    // AC: The best guy should be like [60] seconds
    ItemSpawner.TIME_TO_MAX_DIFFICULTY_MS = 120000; // Tweak ME Please!
    ItemSpawner.GAME_DURATION_OFFSET = [5000, 15000, 30000];
    exports.ItemSpawner = ItemSpawner;
});
define("state/SpaceGameBaseState", ["require", "exports", "state/SpaceBaseState"], function (require, exports, SpaceBaseState_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SpaceGameBaseState extends SpaceBaseState_4.SpaceBaseState {
        constructor() {
            super(...arguments);
            this.MAX_PLAYTIME_BEFORE_STUCK = Infinity;
        }
        init() {
            super.init();
            this.isScorePosted = false;
        }
        endGame(isInterrupted = false) {
            if (!this.isScorePosted) {
                this.isScorePosted = true;
                this.onEndGame();
                this.postScore(isInterrupted);
            }
        }
        onEndGame() { }
        transitionToPreRaceState() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isScorePosted) {
                    this.endGame(true);
                }
                yield _super("transitionToPreRaceState").call(this);
            });
        }
    }
    exports.SpaceGameBaseState = SpaceGameBaseState;
});
define("component/coderedeem/CodeButton", ["require", "exports", "component/coderedeem/RedeemCodeDialog", "switchback-common/util/PromiseUtil", "Game", "switchback-common/util/TextUtil"], function (require, exports, RedeemCodeDialog_3, PromiseUtil_34, Game_28, TextUtil_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeButton extends Phaser.Group {
        constructor(game, shutdownSignal, buttonAssetTag = "code_btn_splitscreen") {
            super(game);
            this.shutdownSignal = shutdownSignal;
            const codeButton = new Phaser.Button(this.game, 0, 0, buttonAssetTag, this.onCodeButtonClicked, this);
            this.add(codeButton);
            const codeText = new Phaser.Text(this.game, codeButton.centerX, codeButton.centerY, 'Code', {
                fill: Game_28.Game.DEFAULT_FONT_COLOUR,
                font: Game_28.Game.DEFAULT_FONT,
                stroke: Game_28.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_28.Game.FONT_STROKE_SMALL,
                fontSize: Game_28.Game.FONT_SIZE_MEDIUM,
                align: 'center'
            });
            codeText.anchor.set(0.5);
            this.add(codeText);
            TextUtil_11.TextUtil.scaleDownSingleLineTextToWidth(codeText, codeButton.width * 0.8);
        }
        get toggleCodeScreen() {
            return this._toggleCodeScreen || (this._toggleCodeScreen = new Phaser.Signal());
        }
        onCodeButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this._toggleCodeScreen) {
                    this._toggleCodeScreen.dispatch(true);
                }
                const codeRedeemDialog = new RedeemCodeDialog_3.RedeemCodeDialog(this.game, this.shutdownSignal);
                this.game.add.existing(codeRedeemDialog);
                codeRedeemDialog.show();
                yield PromiseUtil_34.PromiseUtil.signal(codeRedeemDialog.onButtonClickedSignal);
                if (codeRedeemDialog.isShutdown) {
                    return;
                }
                codeRedeemDialog.destroy();
                if (this._toggleCodeScreen) {
                    this._toggleCodeScreen.dispatch(false);
                }
            });
        }
        destroy() {
            if (this._toggleCodeScreen) {
                this._toggleCodeScreen.dispose();
                this._toggleCodeScreen = null;
            }
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.CodeButton = CodeButton;
});
define("minigame/asteroids/AsteroidsHalfDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/button/CTAButton", "Game", "switchback-common/localization/LocalizationManager", "component/coderedeem/CodeButton"], function (require, exports, PromiseUtil_35, CTAButton_5, Game_29, LocalizationManager_23, CodeButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AsteroidsHalfDialog extends Phaser.Group {
        constructor(game, isTop, shutdownSignal) {
            super(game);
            this.isTop = isTop;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            const bg = new Phaser.Image(this.game, 0, 0, 'eb-instruction-panel');
            bg.inputEnabled = true;
            bg.anchor.set(0, 1);
            bg.width = this.game.width;
            bg.height = this.game.height * 0.5;
            this.add(bg);
            this.codeButton = new CodeButton_1.CodeButton(this.game, this.shutdownSignal);
            this.codeButton.top = bg.top + this.game.height * 0.005;
            this.add(this.codeButton);
            this.codeButton.visible = false;
            this.readyButton = new CTAButton_5.CTAButton(this.game, LocalizationManager_23.LocalizationManager.Instance.text('buttons.ready'), Game_29.Game.FONT_SIZE_MEDIUM, true, this.shutdownSignal);
            this.readyButton.centerX = bg.centerX;
            this.readyButton.bottom = bg.bottom - this.game.height * 0.02;
            this.add(this.readyButton);
            this.nameText = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.15, '', {
                fill: Game_29.Game.FONT_COLOUR_BLUE,
                font: Game_29.Game.DEFAULT_FONT,
                stroke: Game_29.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_29.Game.FONT_STROKE_LARGE,
                fontSize: Game_29.Game.FONT_SIZE_MEDIUM,
                align: 'center'
            });
            this.nameText.anchor.set(0.5);
            this.add(this.nameText);
            const instructionText = new Phaser.Text(this.game, 0, 0, LocalizationManager_23.LocalizationManager.Instance.text("asteroids.intro_top_and_tail"), {
                fill: Game_29.Game.DEFAULT_FONT_COLOUR,
                font: Game_29.Game.DEFAULT_FONT,
                stroke: Game_29.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_29.Game.FONT_STROKE_LARGE,
                fontSize: Game_29.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            instructionText.centerX = bg.centerX;
            instructionText.top = this.nameText.bottom;
            this.add(instructionText);
        }
        set playerName(name) {
            this.nameText.text = name;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                if (this.isTop) {
                    yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                if (this.isShutdown) {
                    return;
                }
                this.codeButton.visible = true;
                this.game.add.tween(this.codeButton).from({ right: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield this.readyButton.show();
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_35.PromiseUtil.signal(this.readyButton.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield this.readyButton.hide();
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isTop) {
                    yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                this.visible = false;
            });
        }
        destroy() {
            this.shutdownSignal = null;
            this.readyButton = null;
            this.nameText = null;
            super.destroy();
        }
    }
    exports.AsteroidsHalfDialog = AsteroidsHalfDialog;
});
define("minigame/asteroids/AsteroidsGameState", ["require", "exports", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "component/dialog/BasicSpaceLandscapeDialog", "Game", "logic/GameController", "minigame/asteroids/AsteroidsShip", "minigame/asteroids/Lane", "minigame/asteroids/Asteroid", "minigame/asteroids/ItemSpawner", "StateNames", "minigame/asteroids/FuelCellPool", "minigame/asteroids/FuelCell", "config/SpaceMountainConfig", "logic/SMAnalyticsManager", "state/SpaceGameBaseState", "switchback-common/enum/Park", "component/coderedeem/CodeButton", "component/coderedeem/RedeemCodeDialog", "switchback-common/util/MotionUtil", "minigame/asteroids/AsteroidsHalfDialog"], function (require, exports, LocalizationManager_24, TextUtil_12, ScaleUtil_22, PromiseUtil_36, BasicSpaceLandscapeDialog_1, Game_30, GameController_16, AsteroidsShip_1, Lane_1, Asteroid_2, ItemSpawner_1, StateNames_9, FuelCellPool_1, FuelCell_3, SpaceMountainConfig_10, SMAnalyticsManager_15, SpaceGameBaseState_1, Park_9, CodeButton_2, RedeemCodeDialog_4, MotionUtil_2, AsteroidsHalfDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@Cerebralfix.com
     * @author Jeronimo@CerebralFix.com
     */
    var TutorialAction;
    (function (TutorialAction) {
        TutorialAction[TutorialAction["RIGHT"] = 0] = "RIGHT";
        TutorialAction[TutorialAction["LEFT"] = 1] = "LEFT";
        TutorialAction[TutorialAction["AWAITING_ACTION"] = 2] = "AWAITING_ACTION";
        TutorialAction[TutorialAction["AWAITING_RESET"] = 3] = "AWAITING_RESET";
    })(TutorialAction = exports.TutorialAction || (exports.TutorialAction = {}));
    var GameState;
    (function (GameState) {
        GameState[GameState["AWAITING_INPUT"] = 0] = "AWAITING_INPUT";
        GameState[GameState["DEAD"] = 1] = "DEAD";
        GameState[GameState["OUTRO"] = 2] = "OUTRO";
        GameState[GameState["PLAYER_PASS"] = 3] = "PLAYER_PASS";
        GameState[GameState["TUTORIAL"] = 4] = "TUTORIAL";
    })(GameState = exports.GameState || (exports.GameState = {}));
    var MovingDirection;
    (function (MovingDirection) {
        MovingDirection[MovingDirection["LEFT"] = 0] = "LEFT";
        MovingDirection[MovingDirection["RIGHT"] = 1] = "RIGHT";
        MovingDirection[MovingDirection["STRAIGHT"] = 2] = "STRAIGHT";
    })(MovingDirection = exports.MovingDirection || (exports.MovingDirection = {}));
    class AsteroidsGameState extends SpaceGameBaseState_1.SpaceGameBaseState {
        constructor() {
            super(...arguments);
            this.lanes = [];
            this.timerText = null; // debug, shown when config sets .showAsteroidsTimer
        }
        init() {
            this.stateName = StateNames_9.StateNames.GAME_ASTEROIDS;
            super.init();
            this.tutorialActionSignal = new Phaser.Signal();
            this.reorientDeviceSignal = new Phaser.Signal();
            const numPlays = GameController_16.GameController.Instance.state.getMinigamePlays(StateNames_9.StateNames.GAME_ASTEROIDS);
            this.difficultyLevel = Math.min(numPlays, 2); // difficulty is based on number of plays and is caped at 3
            this.currentRound = 0;
            this.isCodeScreenVisible = false;
            this.wentRight = false;
            this.isFuelCellStage = false;
            if (MotionUtil_2.MotionUtil.isGyroAvailable) {
                this.playersSession = GameController_16.GameController.Instance.getPlayerSession(1);
            }
            else {
                const numRounds = GameController_16.GameController.Instance.players.length;
                this.playersSession = GameController_16.GameController.Instance.getPlayerSession(2, numRounds, true);
                // Make sure we don't have to play through difficulty level 1 if 
                // we're not playing with gyro, b/c it's too easy
                this.difficultyLevel = Math.max(this.difficultyLevel, 2);
            }
            // It's tutorial time if they've never completed this game.
            this.shouldShowTutorial = !GameController_16.GameController.Instance.hasCompletedMinigame(this.stateName) && !SpaceMountainConfig_10.SpaceMountainConfig.Instance.skipTutorials;
            if (this.shouldShowTutorial) {
                this.gameState = GameState.TUTORIAL; // set the gamestate to tutorial so other game events don't get triggered
            }
        }
        preload() {
            super.preload();
            RedeemCodeDialog_4.RedeemCodeDialog.preload(this.game);
            const path = 'assets/sound/asteroids/';
            let soundNames = ['collect-fuel', 'game-start', 'hit-asteroid-0', 'hit-asteroid-1', 'hit-asteroid-2',
                'shield-down', 'shield-up'];
            for (let sound of soundNames) {
                this.game.load.audio("asteroids-sfx-" + sound, [path + sound + ".m4a", path + sound + ".ogg"]);
            }
            if (SpaceMountainConfig_10.SpaceMountainConfig.Instance.park === Park_9.Park.DLR) {
                this.load.image('ship-base', 'assets/ship/ship-base-dlr.png');
            }
            else if (SpaceMountainConfig_10.SpaceMountainConfig.Instance.park === Park_9.Park.WDW) {
                this.load.image('ship-base', 'assets/ship/ship-base-wdw.png');
            }
            this.load.atlas('slipstream-effect', 'assets/hub/slipstream-effect.png', 'assets/hub/slipstream-effect.json');
            this.load.atlas("ship-upgrades", "assets/ship/ship-upgrades.png", "assets/ship/ship-upgrades.json");
            this.load.atlas("bonus-upgrades", "assets/ship/bonus-upgrades.png", "assets/ship/bonus-upgrades.json");
            this.load.image('thruster-passive', 'assets/ship/thruster-passive.png');
            this.load.image('thruster-launch', 'assets/ship/thruster-launch.png');
            // load background assets
            this.game.load.image("asteroids-background", "assets/minigame/asteroids/asteroids_bg.jpg");
            this.game.load.image("asteroids-grid", "assets/minigame/asteroids/grid.png");
            this.game.load.image("asteroids-overlay", "assets/common/overlay.png");
            this.game.load.image("asteroids-topui", "assets/common/frame-header.png");
            this.game.load.image("asteroid-incoming-alert", "assets/minigame/asteroids/incoming.png");
            this.game.load.image("asteroids-pickup", "assets/minigame/asteroids/pickup.png");
            this.game.load.image("ship-shield", "assets/minigame/asteroids/shield.png");
            this.game.load.spritesheet('probe', 'assets/probes/probes.png', 412, 406);
            this.game.load.image("progress-bar-border", "assets/common/progress-bar-border.png");
            this.game.load.image("progress-bar-inner", "assets/common/progress-bar-inner.png");
            // load the lanes line
            this.game.load.image("asteroids-lane", "assets/minigame/asteroids/lanes.png");
            // load the streak for the prove/ship and asteroid
            this.game.load.image("asteroids-streak", "assets/minigame/asteroids/streak.png");
            // load asteroid assets
            this.game.load.image("asteroids-ast1", "assets/minigame/asteroids/asteroid1.png");
            this.game.load.image("asteroids-ast2", "assets/minigame/asteroids/asteroid2.png");
            this.game.load.image("asteroids-ast3", "assets/minigame/asteroids/asteroid3.png");
            this.game.load.image("asteroids-ast4", "assets/minigame/asteroids/asteroid4.png");
            // load fuel cell assets
            this.game.load.image("asteroids-fuelcell-scanner", "assets/minigame/asteroids/scanner.png");
            // load the trail spritesheet
            this.game.load.atlas('asteroids-trail', 'assets/minigame/asteroids/anim_dottedtrail.png', 'assets/minigame/asteroids/anim_dottedtrail.json');
            // load the collision asset
            this.game.load.image("asteroid-collision", "assets/minigame/asteroids/asteroid_collision.png");
            this.game.load.image("fuelcell-collision", "assets/minigame/asteroids/fuelcell_collision.png");
            // load the probes (with outlines)
            this.game.load.atlas('asteroids-probes-active', 'assets/common/probes_active.png', 'assets/common/probes_active.json');
            // load assets for the tutorial
            this.game.load.image("fingerprint", "assets/minigame/asteroids/tutorial/fingerprint.png");
            this.game.load.image('eb-instruction-panel', 'assets/minigame/energy-ball/eb-instruction-panel.jpg');
            this.game.load.image("asteroids-tutorial-topbar", "assets/minigame/asteroids/tutorial/top_bar.png");
            this.game.load.image("asteroids-tutorial-person-holding-phone", "assets/minigame/asteroids/tutorial/hold_guy.png");
            this.game.load.image("asteroids-tutorial-bottombar", "assets/minigame/asteroids/tutorial/bottom_bar.png");
            this.game.load.image("asteroids-tutorial-pause-glow", "assets/minigame/asteroids/tutorial/pause_glow.png");
            this.game.load.atlas('asteroids-tutorial-phone-icon', 'assets/minigame/asteroids/tutorial/phone_icon.png', 'assets/minigame/asteroids/tutorial/phone_icon.json');
            this.game.load.image("asteroids-tutorial-beam", "assets/common/lightbeam.png");
            this.game.load.image("asteroids-tutorial-arrow", "assets/minigame/asteroids/tutorial/arrow.png");
            this.game.load.spritesheet('asteroids-tutorial-group', 'assets/minigame/asteroids/tutorial/group.png', 1128, 600);
            this.game.load.spritesheet('asteroids-tutorial-player', 'assets/minigame/asteroids/tutorial/player.png', 533, 586);
        }
        create() {
            super.create();
            this.movementCallback = (result) => {
                this.onGyro(result);
            };
            // create background
            const background = new Phaser.Image(this.game, 0, 0, "asteroids-background");
            // set anchor to center and rotate background
            background.anchor.setTo(0.5);
            background.angle = -90;
            // change width to screen height and height to screen width (simulate landscape)
            background.width = this.game.height;
            background.height = this.game.width;
            // move background to center of screen
            background.x = this.game.width * 0.5;
            background.y = this.game.height * 0.5;
            this.add.existing(background);
            this.grid = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, "asteroids-grid");
            this.add.existing(this.grid);
            const overlay = new Phaser.Image(this.game, 0, 0, "asteroids-overlay");
            overlay.anchor.setTo(0.5);
            overlay.angle = -90;
            overlay.width = this.game.height;
            overlay.height = this.game.width;
            overlay.x = this.game.width * 0.5;
            overlay.y = this.game.height * 0.5;
            this.add.existing(overlay);
            // create the lanes
            this.lanes = [];
            for (let i = 0; i < 3; i++) {
                const lane = new Lane_1.Lane(this.game, i * this.game.height / 3); // We're using height here because background is flipped
                this.add.existing(lane);
                this.lanes.push(lane);
            }
            // Create score icon and text
            this.scoreIcon = new Phaser.Image(this.game, 0, 0, "asteroids-pickup");
            ScaleUtil_22.scaleToBounds(this.scoreIcon, this.game.width * 0.2, this.game.height * 0.2);
            this.scoreIcon.anchor.setTo(0.5);
            this.scoreIcon.angle = -90;
            this.scoreIcon.x = this.game.width * AsteroidsGameState.SCORE_X_POS;
            this.scoreIcon.y = this.game.height * AsteroidsGameState.SCORE_Y_POS;
            this.scoreIcon.visible = false;
            this.add.existing(this.scoreIcon);
            // add score text
            this.scoreText = new Phaser.Text(this.game, this.game.width * 0.07, this.game.height * 0.87, '', {
                font: Game_30.Game.DEFAULT_FONT,
                fontSize: TextUtil_12.TextUtil.getFontSize(this.game.width, 30),
                fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                align: 'center',
                stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_30.Game.DEFAULT_STROKE_THICKNESS
            });
            this.scoreText.anchor.set(0.5, 0);
            this.scoreText.angle = -90;
            this.scoreText.visible = false;
            this.add.existing(this.scoreText);
            // add the tutorial layer
            this.tutorialLayer = new Phaser.Group(this.game);
            this.add.existing(this.tutorialLayer);
            // add the items layer
            this.itemLayer = new Phaser.Group(this.game);
            this.add.existing(this.itemLayer);
            // add the top ui menu bar that indicates when the shield is down
            this.topUI = new Phaser.Image(this.game, 0, 0, "asteroids-topui");
            this.topUI.anchor.setTo(0.5);
            this.topUI.angle = -90;
            this.topUI.x = this.game.width * 0.05;
            this.topUI.y = this.game.height * 0.5;
            this.add.existing(this.topUI);
            this.shieldStatusText = new Phaser.Text(this.game, 0, 0, LocalizationManager_24.LocalizationManager.Instance.text("asteroids.shield_active").toUpperCase(), {
                font: Game_30.Game.DEFAULT_FONT,
                fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_30.Game.FONT_SIZE_MEDIUM,
                stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            this.shieldStatusText.anchor.set(0.5);
            this.shieldStatusText.angle = -90;
            this.shieldStatusText.centerX = this.topUI.centerX;
            this.shieldStatusText.centerY = this.topUI.centerY;
            this.add.existing(this.shieldStatusText);
            this.ship = new AsteroidsShip_1.AsteroidsShip(this.game, Game_30.Game.PLAYER_TINTS[this.playersSession[this.currentRound][0]], this.shutdownSignal);
            this.add.existing(this.ship);
            this.shieldAlreadyFlickered = false;
            if (SpaceMountainConfig_10.SpaceMountainConfig.Instance.showAsteroidsTimer) {
                this.timerText = new Phaser.Text(this.game, this.game.width * 0.07, this.game.height * 0.07, '', {
                    font: Game_30.Game.DEFAULT_FONT,
                    fontSize: TextUtil_12.TextUtil.getFontSize(this.game.width, 30),
                    fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                    align: 'right',
                    stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_30.Game.DEFAULT_STROKE_THICKNESS
                });
                this.timerText.anchor.set(1, 0);
                this.timerText.angle = -90;
                this.add.existing(this.timerText);
            }
            this.fuelCellPool = new FuelCellPool_1.FuelCellPool(this.game);
            this.add.existing(this.fuelCellPool);
            if (MotionUtil_2.MotionUtil.isGyroAvailable) {
                this.createHeadsUpInstructions(); // create the tutorial instructions
            }
            else {
                this.createBShowInstructions();
            }
            this.dismissOverlay();
            this.startGame();
        }
        update() {
            super.update();
            if (this.timerText) {
                this.timerText.setText(Math.floor(this.itemSpawner.getElapsed() / 1000).toString());
            }
            // if the game has ended, finish it
            if (this.gameState === GameState.DEAD) {
                this.onGameOver();
                return;
            }
            if (this.gameState !== GameState.AWAITING_INPUT) {
                return;
            }
            // check if the ship is out of lives. change game state to OUTRO if we run out of players
            this.checkShipDead();
            this.checkCollision();
            this.itemSpawner.update();
        }
        /**
         * Check collision with the ship and destroys the objects
         * that collide with the ship.
         */
        checkCollision() {
            let toDestroy = [];
            // check collision with objects in the lane
            for (let i = 0; i < this.lanes.length; i++) {
                this.lanes[i].laneObjects.forEach((laneObject) => {
                    // get the ship's bounds
                    let shipBounds = this.ship.hitbox.getBounds();
                    let boundsA = new Phaser.Rectangle(shipBounds.x, shipBounds.y, shipBounds.width, shipBounds.height);
                    // get the lane object's hitbox bounds
                    let objectBounds = laneObject.hitbox.getBounds();
                    let boundsB = new Phaser.Rectangle(objectBounds.x, objectBounds.y, objectBounds.width, objectBounds.height);
                    // check if the ship and the lane object are colliding
                    if (Phaser.Rectangle.intersects(boundsA, boundsB)) {
                        const intersectionPoint = this.findIntersectionPoint(boundsA, boundsB);
                        this.movingDirection = MovingDirection.STRAIGHT; // reset the movement direction now that we've calculated collision point
                        laneObject.collision(this.ship, intersectionPoint);
                        toDestroy.push(laneObject);
                        // check if the ship's shield is still active and update the text if it's destroyed
                        if (!this.shieldAlreadyFlickered && !this.ship.isShieldActive) {
                            this.shieldStatusText.text = LocalizationManager_24.LocalizationManager.Instance.text("asteroids.shield_down").toUpperCase(); // update the shield status text
                            this.shieldStatusText.fill = Game_30.Game.FONT_COLOUR_RED;
                            this.flashShieldDownText();
                        }
                    }
                });
                // loop over the array of lane objects that need to be destroyed (i.e. things that were hit)
                toDestroy.forEach((laneObject) => {
                    this.lanes[i].destroyItem(laneObject);
                });
                toDestroy = []; // reset the toDestroy array for next lane iteration
            }
        }
        /**
         * Checks if the ship is dead.
         */
        checkShipDead() {
            if (!this.ship.isDead()) {
                return;
            }
            // the ship is dead
            // pass to next person if there is any remaining
            if (this.currentRound < (this.playersSession.length - 1)) { // notice -1 because it will increment
                this.currentRound++;
                this.changePlayer();
                this.ship.respawn(Game_30.Game.PLAYER_TINTS[this.playersSession[this.currentRound][0]]);
                this.shieldStatusText.text = LocalizationManager_24.LocalizationManager.Instance.text("asteroids.shield_active").toUpperCase(); // update the shield status text
                this.shieldStatusText.fill = Game_30.Game.DEFAULT_FONT_COLOUR;
                this.shieldAlreadyFlickered = false; // reset the shieldAlreadyFlickered text
                // clear the lanes for next player
                for (let lane of this.lanes) {
                    lane.clear();
                }
                // Else, if no more remaining rounds/players are left
            }
            else {
                // TODO notice even this gets set, the part outside this 2 ifs gets executed anyway
                this.gameState = GameState.DEAD;
                // TODO this.endGame() could be here
            }
        }
        changePlayer() {
            // stop the spawning while we change players and play the tutorial
            this.itemSpawner.stopSpawning();
            this.gameState = GameState.PLAYER_PASS;
            this.doTurn();
        }
        /**
         * Sets up the score, ship, input events,
         * and puts the game in the starting state.
         */
        startGame() {
            console.log(`[${this.stateName}] startGame()`);
            this.ship.x = this.game.width * 0.9;
            this.ship.y = this.game.world.centerY;
            this.ship.setCurrentLane(this.lanes[1]);
            // create item spawner and spawning loop
            this.itemSpawner = new ItemSpawner_1.ItemSpawner(this.game, this.lanes, this.difficultyLevel, this.fuelCellPool, this.shutdownSignal);
            if (MotionUtil_2.MotionUtil.isGyroAvailable) {
                // subscribe to the DEVICE_ON_MOTION_CHANGED event and call findLane whenever every time it changes
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.movementCallback);
            }
            else {
                this.game.input.onDown.add(this.onMouseClicked, this);
            }
            // listen to update score signal
            this.ship.scoreUpdateSignal.add(() => {
                this.refreshScoreText();
            }, this);
            this.doTurn();
        }
        doTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                this.ship.visible = false;
                this.scoreIcon.visible = false;
                this.scoreText.visible = false;
                this.grid.stopScroll();
                if (MotionUtil_2.MotionUtil.isGyroAvailable) {
                    yield this.displayHeadsUpInstructions();
                }
                else {
                    let names = this.getCurrentPlayersNames();
                    yield this.displayBShowInstructions(names[0], names[1]);
                }
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_15.SMAnalyticsManager.Instance.trackGamePlay(this.stateName);
                this.game.sound.play('asteroids-sfx-game-start');
                this.grid.autoScroll(this.game.width * AsteroidsGameState.GRID_SCROLL_SPEED, 0);
                // tween the ship to screen and then tween it back to "playing position"
                this.ship.left = this.game.width;
                this.ship.y = this.game.world.centerY; // reposition the ship back to middle lane
                this.ship.visible = true;
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.ship).to({ x: this.game.width * 0.5 }, 1000, Phaser.Easing.Quadratic.Out, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.ship).to({ x: this.game.width * 0.9 }, 1000, Phaser.Easing.Quadratic.InOut, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // if the tutorial flag is active, start the tutorial
                if (this.shouldShowTutorial) {
                    this.shouldShowTutorial = false; // SWB-1168 only show tutorial once
                    yield this.playTutorial();
                    if (this.isShutdown) {
                        return;
                    }
                }
                this.refreshScoreText();
                this.itemSpawner.resetDifficulty();
                this.itemSpawner.startSpawning();
                // set state to awaiting_input
                this.gameState = GameState.AWAITING_INPUT;
            });
        }
        flashShieldDownText() {
            return __awaiter(this, void 0, void 0, function* () {
                this.shieldAlreadyFlickered = true;
                let countdownTimer = this.game.time.create(true); // create a timer
                let loopEvent = this.game.time.events.loop(250, this.flickShieldStatusText, this); // start the flickering every 250ms
                countdownTimer.start(); // start the countdown timer
                yield PromiseUtil_36.PromiseUtil.time(this.game, 1600, this.shutdownSignal).catch(() => { }); // end the flickering after a period of time
                this.game.time.events.remove(loopEvent);
                if (this.isShutdown) {
                    return;
                }
                this.shieldStatusFlicker = true;
                this.shieldStatusText.visible = true;
            });
        }
        flickShieldStatusText() {
            if (this.shieldStatusFlicker) {
                this.shieldStatusText.visible = true;
                this.shieldStatusFlicker = false;
            }
            else {
                this.shieldStatusText.visible = false;
                this.shieldStatusFlicker = true;
            }
        }
        /**
         * Find the center point of the ship's and object's hitbox. Find the midpoint of the
         * line between those two points and return that as the collision point.
         * @param rectA - the ship's hitbox bounds
         * @param rectB - the object's hitbox bounds
         */
        findIntersectionPoint(rectA, rectB) {
            const middlePointA = new Phaser.Point(rectA.centerX, rectA.centerY);
            const middlePointB = new Phaser.Point(rectB.centerX, rectB.centerY);
            const lineBetweenTwoPoints = new Phaser.Line(middlePointA.x, middlePointA.y, middlePointB.x, middlePointB.y);
            return lineBetweenTwoPoints.midPoint();
        }
        getCurrentPlayerName() {
            return this.playersSession[this.currentRound].map(playerIndex => GameController_16.GameController.Instance.players[playerIndex].name)[0];
        }
        /**
         * Store the y position of the mouse click and check on which lane that landed on.
         * Calls the moveShip function which moves the ship to the lane that was clicked.
         */
        onMouseClicked(pointer) {
            const yClicked = pointer.y; // store the y value of the click (Note: use y because we've flipped background)
            // Very top reorients the device
            if (this.actionTutorial === TutorialAction.AWAITING_RESET && !this.isCodeScreenVisible) {
                this.reorientDeviceSignal.dispatch();
                return;
            }
            // get current lane
            let laneIndex;
            for (laneIndex = 0; laneIndex < this.lanes.length; laneIndex++) {
                if (this.lanes[laneIndex] === this.ship.getCurrentLane()) {
                    break;
                }
            }
            // check which lane was clicked
            if (yClicked < this.game.height * 0.3) {
                if (this.gameState === GameState.TUTORIAL) {
                    // if the user went right during tutorial phase 1, don't trigger any left events
                    if (this.isFuelCellStage && this.wentRight) {
                        return;
                    }
                    this.actionTutorial = TutorialAction.RIGHT;
                    this.tutorialActionSignal.dispatch();
                }
                else {
                    this.moveShip(laneIndex - 1); // move left
                }
                return;
            }
            else if (yClicked > this.game.height * 0.6) {
                if (this.gameState === GameState.TUTORIAL) {
                    if (this.isFuelCellStage && !this.wentRight) {
                        return;
                    }
                    this.actionTutorial = TutorialAction.LEFT;
                    this.tutorialActionSignal.dispatch();
                }
                else {
                    this.moveShip(laneIndex + 1); // move right
                }
                return;
            }
        }
        /**
         * Find which lane to call based on the phone's pitch and tell the ship to
         * move
         *
         * @param result - contains information about pitch
         */
        onGyro(result) {
            // If we need re-orienting and this motion event puts us approximately level, dispatch
            if (this.actionTutorial === TutorialAction.AWAITING_RESET && !this.isCodeScreenVisible &&
                Math.abs(result.attitude.roll - (Math.PI * 0.5)) < AsteroidsGameState.REORIENTATION_DEAD_ZONE_VALUE) {
                // This is a reset to (approximately) level
                this.reorientDeviceSignal.dispatch();
            }
            if (this.gameState === GameState.TUTORIAL) {
                // if we're not awaiting an input, don't do anything
                if (this.actionTutorial !== TutorialAction.AWAITING_ACTION) {
                    return;
                }
                // if we're on the fuel cell stage, only respond to the input that is opposite to wentRight
                if (this.isFuelCellStage) {
                    if (this.wentRight) {
                        if (result.attitude.pitch > AsteroidsGameState.MOVEMENT_PITCH) {
                            this.actionTutorial = TutorialAction.LEFT;
                        }
                    }
                    else {
                        if (result.attitude.pitch < -AsteroidsGameState.MOVEMENT_PITCH) {
                            this.actionTutorial = TutorialAction.RIGHT;
                        }
                    }
                    // if we updated the action (i.e. an action was made) dispatch the signal
                    if (this.actionTutorial !== TutorialAction.AWAITING_ACTION) {
                        this.tutorialActionSignal.dispatch();
                    }
                    return;
                }
                if (result.attitude.pitch < -AsteroidsGameState.MOVEMENT_PITCH) {
                    this.actionTutorial = TutorialAction.RIGHT;
                }
                else if (result.attitude.pitch > AsteroidsGameState.MOVEMENT_PITCH) {
                    this.actionTutorial = TutorialAction.LEFT;
                }
                // if we updated the action (i.e. an action was made) dispatch the signal
                if (this.actionTutorial !== TutorialAction.AWAITING_ACTION) {
                    this.tutorialActionSignal.dispatch();
                }
                return;
            }
            if (result.attitude.pitch < -AsteroidsGameState.MOVEMENT_PITCH) {
                this.moveShip(0);
            }
            else if (result.attitude.pitch > AsteroidsGameState.MOVEMENT_PITCH) {
                this.moveShip(2);
            }
            else {
                this.moveShip(1);
            }
        }
        /**
         * This function moves the ship to a given lane.
         *
         * @param lane - the lane that we are moving the ship to
         */
        moveShip(laneIndex) {
            return __awaiter(this, void 0, void 0, function* () {
                if (laneIndex < 0 || laneIndex >= this.lanes.length) {
                    return;
                }
                let lane = this.lanes[laneIndex];
                // if the ship is not moving to a new lane, don't display any turning animations
                if (this.ship.getCurrentLane() === lane) {
                    return;
                }
                if (this.shipMovementTween && this.shipAngleMovementTween) {
                    this.game.tweens.removeFrom(this.shipMovementTween);
                    this.game.tweens.removeFrom(this.shipAngleMovementTween);
                }
                if (this.ship.y > lane.startingY) {
                    this.movingDirection = MovingDirection.RIGHT;
                    this.shipAngleMovementTween = this.game.add.tween(this.ship.getShip()).to({ angle: 45 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                }
                else if (this.ship.y < lane.startingY) {
                    this.movingDirection = MovingDirection.LEFT;
                    this.shipAngleMovementTween = this.game.add.tween(this.ship.getShip()).to({ angle: -45 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                }
                this.shipMovementTween = this.game.add.tween(this.ship).to({ y: lane.startingY + ((this.game.height / 3) / 2) }, 250, Phaser.Easing.Sinusoidal.Out, true);
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.shipMovementTween, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.game.add.tween(this.ship.getShip()).to({ angle: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                this.movingDirection = MovingDirection.STRAIGHT; // set moving direction to straight now that we've stopped moving
                this.ship.setCurrentLane(lane);
            });
        }
        /**
         * Update the score text to the ship's current score
         */
        refreshScoreText() {
            if (this.ship.score > 0) {
                this.scoreIcon.visible = true;
                this.scoreText.visible = true;
            }
            this.scoreText.setText('x' + this.ship.score.toString());
        }
        /**
         * Create a dialog notifying the user how many starts they collected
         * and transition them to hub
         */
        onGameOver() {
            this.endGame();
            const starsCollectedString = LocalizationManager_24.LocalizationManager.Instance.textWithCount("asteroids.num_stars_collected", this.ship.score);
            const dialog = new BasicSpaceLandscapeDialog_1.BasicSpaceLandscapeDialog(this.game, LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.result_display", { star_count: starsCollectedString }), LocalizationManager_24.LocalizationManager.Instance.text("asteroids.button_great"), 60, this.shutdownSignal);
            dialog.onButtonClickedSignal.addOnce(() => {
                this.transitionToHub();
            });
            this.add.existing(dialog);
            dialog.show();
        }
        onEndGame() {
            this.gameState = GameState.OUTRO;
            // unsubscribe from the DEVICE_ON_MOTION_CHANGED event so the findLane method is not called when game ends
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.movementCallback);
        }
        // Posts the score for the players
        postScore(isInterrupted) {
            GameController_16.GameController.Instance.postMinigameScore(StateNames_9.StateNames.GAME_ASTEROIDS, this.ship.score);
            GameController_16.GameController.Instance.achievementManager.onAsteroidsEndGame(this.ship.score / this.playersSession.length);
        }
        /**
         * Clears all the game objects, events, and timers.
         */
        clearObjects() {
            // destroy the ship
            this.ship.destroy();
            // remove all events from the timer (there could be warning icon's still active - this will get rid of them)
            this.game.time.events.removeAll();
            // loop over all lanes destroying everything currently in them
            for (let i = 0; i < this.lanes.length; i++) {
                this.lanes[i].laneObjects.forEach((laneObject) => {
                    this.lanes[i].destroyItem(laneObject);
                });
            }
            // destroy the TimerEvents inside itemSpawner
            this.itemSpawner.shutdown();
        }
        /**
         * Transition from asteroid game to the hub
         */
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToHub()`);
                yield this.showOverlay();
                GameController_16.GameController.Instance.finishGame();
                this.game.state.start(StateNames_9.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        /********************************
         * ********* Tutorial ***********
         * ******************************
         */
        /**
         * Launches the tutorial. An asteroid is created and tweened
         * down half way, before the game is paused and awaits
         * an action from the player. While waiting for an action, the
         * player is asked to dodge the asteroid by tilting. Once the choice
         * is made, a fuel cell will spawn in a different lane and once again
         * the game await an action to collect the fuel cell. Finally,
         * "Get Ready" text shows up and the player is given 3 seconds
         * before the actual game starts.
         */
        playTutorial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.gameState = GameState.TUTORIAL; // set the gamestate to tutorial so other game events don't get triggered
                this.scoreIcon.visible = false;
                this.scoreText.visible = false;
                this.isFuelCellStage = false;
                const isHeadsUpTutorial = MotionUtil_2.MotionUtil.isGyroAvailable;
                let prevShipX = this.ship.x;
                if (isHeadsUpTutorial) {
                    this.tutorialBottomBar.y = this.game.height;
                    this.tutorialBottomBar.x = this.game.width;
                    this.tutorialPilotText.x = this.game.width + (this.tutorialBottomBar.height * 0.5);
                    this.tutorialPilotText.y = (this.game.height) * 0.5;
                    this.tutorialPilotText.text = LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.tutorial_hold", { player_name: this.getCurrentPlayerName() }).toUpperCase();
                    this.tutorialBottomBar.visible = isHeadsUpTutorial;
                    this.tutorialPilotText.visible = isHeadsUpTutorial;
                    const shipOffset = (isHeadsUpTutorial ? this.tutorialBottomBar.height : 0);
                    this.game.add.tween(this.ship)
                        .to({ x: prevShipX - shipOffset }, 600, Phaser.Easing.Sinusoidal.In, true);
                    this.game.add.tween(this.tutorialPilotText)
                        .to({ x: this.game.width - (shipOffset * 0.5) }, 600, Phaser.Easing.Sinusoidal.In, true);
                    yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomBar)
                        .to({ x: this.game.width - shipOffset }, 600, Phaser.Easing.Sinusoidal.In, true), this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                }
                // display the warning and then tween the asteroid down the lane
                yield this.itemSpawner.lanes[1].showWarning(ItemSpawner_1.ItemSpawner.MAX_WARNING_TIME_MS);
                if (this.isShutdown) {
                    return;
                }
                // Creates a new temporary asteroid
                let startY = this.lanes[1].startingY;
                let tempAsteroid = new Asteroid_2.Asteroid(this.game, this.shutdownSignal);
                tempAsteroid.x = 0;
                tempAsteroid.centerY = startY + ((this.game.height / 3) / 2);
                tempAsteroid.visible = true;
                this.itemLayer.add(tempAsteroid);
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(tempAsteroid).to({ x: this.game.width * 0.35 }, 1400, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // stop the animations on the asteroid while we wait for the player's action
                tempAsteroid.removeAnimations();
                this.grid.stopScroll();
                // display beam
                yield this.displayTutorialBeam(tempAsteroid.centerY);
                if (this.isShutdown) {
                    return;
                }
                // display the controls
                this.displayDodgeText();
                let fingerFlashTimeout = -1;
                if (isHeadsUpTutorial) {
                    this.displayPhoneControlsDodge();
                }
                else {
                    this.fingerprintTop.visible = false;
                    this.fingerprintBottom.visible = true;
                    fingerFlashTimeout = setInterval(() => {
                        this.fingerprintTop.visible = !this.fingerprintTop.visible;
                        this.fingerprintBottom.visible = !this.fingerprintBottom.visible;
                    }, 500);
                }
                // wait for the guest to repond with the required action
                yield PromiseUtil_36.PromiseUtil.signal(this.tutorialActionSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                if (isHeadsUpTutorial) {
                    this.phoneIcon.animations.getAnimation('phone-tilting').stop();
                }
                else {
                    clearTimeout(fingerFlashTimeout);
                    fingerFlashTimeout = -1;
                    this.fingerprintTop.visible = false;
                    this.fingerprintBottom.visible = false;
                }
                // as soon as the player has made a move (left or right lane), move the ship there
                if (this.actionTutorial === TutorialAction.LEFT) {
                    this.wentRight = false;
                    yield this.moveShip(2);
                }
                else {
                    this.wentRight = true;
                    yield this.moveShip(0);
                }
                if (this.isShutdown) {
                    return;
                }
                // hide the controls
                if (isHeadsUpTutorial) {
                    this.game.add.tween(this.arrowLeft).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.arrowRight).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.phoneIcon).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                }
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialText).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.tutorialText.visible = false;
                if (isHeadsUpTutorial) {
                    this.arrowLeft.visible = false;
                    this.arrowRight.visible = false;
                    this.phoneIcon.visible = false;
                }
                // play the animations while the asteroid passes
                this.grid.autoScroll(this.game.width * AsteroidsGameState.GRID_SCROLL_SPEED, 0);
                tempAsteroid.unpauseAnimations();
                // let the asteroid pass
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(tempAsteroid).to({ x: this.game.width + tempAsteroid.width }, 1400, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // tween a fuel cell down
                let tempFuelCell = new FuelCell_3.FuelCell(this.game, this.fuelCellPool, this.shutdownSignal);
                // determine which lane to spawn the fuel cell (depends on which way they dodged the asteroid above)
                if (this.actionTutorial === TutorialAction.LEFT) {
                    startY = this.lanes[0].startingY;
                }
                else {
                    startY = this.lanes[2].startingY;
                }
                tempFuelCell.x = 0;
                tempFuelCell.centerY = startY + ((this.game.height / 3) / 2);
                // tween the fuel cell down into the lane
                tempFuelCell.visible = true;
                this.itemLayer.add(tempFuelCell);
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(tempFuelCell).to({ x: this.game.width * 0.6 }, 1400, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // freeze the grid scrolling and fuelcell animation
                this.grid.stopScroll();
                tempFuelCell.removeAnimations();
                // display beam
                yield this.displayTutorialBeam(tempFuelCell.centerY);
                if (this.isShutdown) {
                    return;
                }
                // controls for telling the player to collect it
                this.displayCollectText();
                if (isHeadsUpTutorial) {
                    this.displayPhoneControlsCollect();
                    if (this.isShutdown) {
                        return;
                    }
                }
                else {
                    this.fingerprintTop.visible = !this.wentRight;
                    this.fingerprintBottom.visible = this.wentRight;
                    fingerFlashTimeout = setInterval(() => {
                        this.fingerprintTop.visible = !this.wentRight && !this.fingerprintTop.visible;
                        this.fingerprintBottom.visible = this.wentRight && !this.fingerprintBottom.visible;
                    }, 350);
                }
                yield PromiseUtil_36.PromiseUtil.signal(this.tutorialActionSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                if (fingerFlashTimeout !== -1) {
                    clearTimeout(fingerFlashTimeout);
                    this.fingerprintTop.visible = false;
                    this.fingerprintBottom.visible = false;
                }
                // as soon as the player has made a move (left or right lane), move the ship there
                if (this.actionTutorial === TutorialAction.LEFT) {
                    yield this.moveShip(2);
                }
                else {
                    yield this.moveShip(0);
                }
                if (this.isShutdown) {
                    return;
                }
                // fade out the fuel cell to make it seem like the player has picked it up
                tempFuelCell.collision(this.ship, tempFuelCell.position).then(() => {
                    this.refreshScoreText();
                });
                tempFuelCell.destroy();
                // hide the controls
                if (isHeadsUpTutorial) {
                    this.game.add.tween(this.arrowLeft).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.arrowRight).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                    this.game.add.tween(this.phoneIcon).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                }
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialText).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.tutorialText.visible = false;
                this.tutorialText.alpha = 1;
                // reset the alpha for the next time these are used
                if (isHeadsUpTutorial) {
                    this.arrowLeft.visible = false;
                    this.arrowRight.visible = false;
                    this.phoneIcon.visible = false;
                    this.arrowLeft.alpha = 1;
                    this.arrowRight.alpha = 1;
                    this.phoneIcon.alpha = 1;
                }
                this.grid.autoScroll(this.game.width * AsteroidsGameState.GRID_SCROLL_SPEED, 0);
                /// hide hud overlay
                if (isHeadsUpTutorial) {
                    this.game.add.tween(this.ship)
                        .to({ x: prevShipX }, 600, Phaser.Easing.Sinusoidal.In, true);
                    this.game.add.tween(this.tutorialPilotText)
                        .to({ x: this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                    this.game.add.tween(this.tutorialDontTouchText)
                        .to({ x: -this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                    yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomBar)
                        .to({ x: this.game.width + this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true), this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    this.tutorialBottomBar.visible = false;
                    this.tutorialPilotText.visible = false;
                    this.tutorialDontTouchText.visible = false;
                }
            });
        }
        /**
         * Creates the "Get Ready" text and countdown timer
         * (3, 2, 1) used at the end of the tutorial.
         */
        displayGetReadyTimer() {
            return __awaiter(this, void 0, void 0, function* () {
                const countDownText = new Phaser.Text(this.game, 0, 0, "3", {
                    font: Game_30.Game.DEFAULT_FONT,
                    fill: Game_30.Game.FONT_COLOUR_BLUE,
                    fontSize: Game_30.Game.FONT_SIZE_EXTRA_LARGE,
                    stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: this.game.width * 0.01,
                    align: 'center',
                });
                countDownText.anchor.set(0.5);
                countDownText.angle = -90;
                countDownText.alpha = 0;
                countDownText.x = this.game.width * 0.5;
                countDownText.y = this.game.height * 0.25;
                this.add.existing(countDownText);
                const durationMs = 500;
                // Show the countdown text (& move the infographic across)
                countDownText.text = LocalizationManager_24.LocalizationManager.Instance.text("asteroids.tutorial_get_ready").toUpperCase();
                countDownText.fontSize = 150;
                this.game.add.tween(countDownText).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.tutorialPersonIcon).to({ y: this.game.height * 0.85 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true);
                this.game.add.tween(this.tutorialGroupIcon).to({ y: this.game.height * 0.6 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true);
                // Start the countdown!
                const countdownStrings = ['3', '2', '1', LocalizationManager_24.LocalizationManager.Instance.text("asteroids.tutorial_go").toUpperCase()];
                for (let i = 0; i < countdownStrings.length; i++) {
                    yield PromiseUtil_36.PromiseUtil.time(this.game, 1000, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    countDownText.fontSize = 250;
                    countDownText.text = countdownStrings[i];
                    this.game.add.tween(countDownText.scale).to({ x: 1.5, y: 1.5 }, durationMs * 0.5, Phaser.Easing.Cubic.InOut, true, 0, 0, true);
                }
                this.game.add.tween(countDownText).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    countDownText.destroy();
                });
            });
        }
        displayCollectText() {
            this.tutorialText.text = LocalizationManager_24.LocalizationManager.Instance.text("asteroids.tutorial_collect");
            this.tutorialText.alpha = 1;
            this.tutorialText.visible = true;
        }
        /**
         * Display icons and text to tell the user
         * to collect the fuel cell.
         * This function is used in the second phase of the
         * tutorial.
         */
        displayPhoneControlsCollect() {
            return __awaiter(this, void 0, void 0, function* () {
                this.phoneIcon.loadTexture('asteroids-tutorial-phone-icon', 0);
                this.phoneIcon.alpha = 1;
                this.phoneIcon.visible = true;
                this.game.add.tween(this.phoneIcon).to({ alpha: 0.1 }, 800, Phaser.Easing.Linear.None, true, 0, Infinity, true);
                // if the player went left to dodge the asteroid, display the phone tilting to the left and the left arrow
                // else display the phone tilting to the right with the right arrow
                this.isFuelCellStage = true;
                if (this.actionTutorial === TutorialAction.LEFT) {
                    this.arrowRight.visible = true;
                    this.arrowRight.alpha = 1;
                    this.actionTutorial = TutorialAction.AWAITING_ACTION;
                    // keep modifying the texture of the phone to teach players about tilting
                    while (this.actionTutorial === TutorialAction.AWAITING_ACTION) {
                        this.phoneIcon.loadTexture('asteroids-tutorial-phone-icon', 1);
                        yield PromiseUtil_36.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { });
                        if (this.isShutdown) {
                            return;
                        }
                        this.phoneIcon.loadTexture('asteroids-tutorial-phone-icon', 2);
                        yield PromiseUtil_36.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { });
                        if (this.isShutdown) {
                            return;
                        }
                    }
                }
                else {
                    this.arrowLeft.visible = true;
                    this.arrowLeft.alpha = 1;
                    this.actionTutorial = TutorialAction.AWAITING_ACTION;
                    // keep modifying the texture of the phone to teach players about tilting
                    while (this.actionTutorial === TutorialAction.AWAITING_ACTION) {
                        this.phoneIcon.loadTexture('asteroids-tutorial-phone-icon', 2);
                        yield PromiseUtil_36.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { });
                        if (this.isShutdown) {
                            return;
                        }
                        this.phoneIcon.loadTexture('asteroids-tutorial-phone-icon', 1);
                        yield PromiseUtil_36.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { });
                        if (this.isShutdown) {
                            return;
                        }
                    }
                }
            });
        }
        /**
         * Display icons and text to tell the user
         * to dodge the asteroid.
         * This function is used in the first phase of the
         * tutorial.
         */
        displayPhoneControlsDodge() {
            this.arrowRight.visible = true;
            this.arrowLeft.visible = true;
            this.phoneIcon.visible = true;
            this.phoneIcon.alpha = 1;
            this.game.add.tween(this.phoneIcon).to({ alpha: 0.1 }, 800, Phaser.Easing.Linear.None, true, 0, Infinity, true);
            // play the tilting animation to teach the player about tilting
            if (!this.phoneIcon.animations.getAnimation('phone-tilting').isPlaying) {
                this.phoneIcon.play('phone-tilting', 2, true);
            }
        }
        displayDodgeText() {
            this.actionTutorial = TutorialAction.AWAITING_ACTION;
            this.tutorialText.text = LocalizationManager_24.LocalizationManager.Instance.text("asteroids.tutorial_dodge");
            this.tutorialText.visible = true;
        }
        /**
         * Creates a beam and spawns it in the center of the lane by
         * fading the alpha in and out. The beam is destroyed and removed from the group
         * at the end of the function.
         *
         * @param y - the y location for the beam (this should be in the center of the lane)
         */
        displayTutorialBeam(y) {
            return __awaiter(this, void 0, void 0, function* () {
                const beam = new Phaser.Image(this.game, 0, y, 'asteroids-tutorial-pause-glow');
                beam.alpha = 0;
                beam.anchor.setTo(0.5);
                beam.angle = -90;
                beam.x = this.game.width * 0.5;
                beam.height = this.game.width;
                this.tutorialLayer.add(beam);
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(beam).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 0, 0, true), this.shutdownSignal).catch(() => { });
                beam.destroy();
            });
        }
        /**
         * Shows the instructions for each player
         * with a picture of a person holding the phone on forehead
         */
        displayHeadsUpInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                // update the player names in the text fields
                let playerName = this.getCurrentPlayerName();
                this.tutorialNavigatorText.text = LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.tutorial_navigator", { player_name: playerName }).toUpperCase();
                this.tutorialPilotText.text = LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.tutorial_pilot", { player_name: playerName }).toUpperCase();
                // set the position of the tutorial UI elements
                this.tutorialTopBar.y = this.game.height;
                this.tutorialTopBar.x = 0;
                this.tutorialBottomBar.y = this.game.height;
                this.tutorialBottomBar.x = this.game.width - this.tutorialBottomBar.height;
                this.tutorialNavigatorText.x = this.tutorialTopBar.height * 0.5;
                this.tutorialNavigatorText.y = this.game.height * 0.5;
                this.tutorialPilotText.x = this.game.width - (this.tutorialBottomBar.height * 0.5);
                this.tutorialPilotText.y = this.game.height * 0.5;
                this.tutorialDontTouchText.x = this.game.width * 0.2;
                this.tutorialDontTouchText.y = this.game.height * 0.5;
                ScaleUtil_22.scaleToBounds(this.tutorialPersonIcon, this.game.height * 0.3, this.game.width * 0.63);
                this.tutorialPersonIcon.centerX = this.game.width * 0.8;
                this.tutorialPersonIcon.centerY = this.game.height * 0.53;
                ScaleUtil_22.scaleToBounds(this.tutorialGroupIcon, this.game.height * 0.6, this.game.width * 0.63);
                this.tutorialGroupIcon.centerX = this.game.width * 0.85;
                this.tutorialGroupIcon.centerY = this.game.height * 0.175;
                // set the visibility of the instructions and menu bars to true
                this.tutorialTopBar.visible = true;
                this.tutorialBottomBar.visible = true;
                this.tutorialPersonIcon.visible = true;
                this.tutorialGroupIcon.visible = true;
                this.tutorialNavigatorText.visible = true;
                this.tutorialPilotText.visible = true;
                this.tutorialDontTouchText.visible = true;
                SMAnalyticsManager_15.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                SMAnalyticsManager_15.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                this.codeButton.visible = true;
                this.game.add.tween(this.codeButton).from({ top: this.game.height }, 300, Phaser.Easing.Linear.None, true);
                this.codeButton.toggleCodeScreen.add(this.onCodeScreenToggled, this);
                yield PromiseUtil_36.PromiseUtil.time(this.game, 3000, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // Wait for orientation
                this.actionTutorial = TutorialAction.AWAITING_RESET;
                yield PromiseUtil_36.PromiseUtil.signal(this.reorientDeviceSignal, this.shutdownSignal).catch(() => { });
                this.codeButton.visible = false;
                this.codeButton.toggleCodeScreen.remove(this.onCodeScreenToggled, this);
                this.actionTutorial = null;
                if (this.isShutdown) {
                    return;
                }
                yield this.displayGetReadyTimer();
                if (this.isShutdown) {
                    return;
                }
                // tween all the stuff off screen
                this.game.add.tween(this.tutorialTopBar)
                    .to({ x: -this.tutorialTopBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.tutorialNavigatorText)
                    .to({ x: -this.tutorialTopBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.tutorialPersonIcon)
                    .to({ x: this.game.width + this.tutorialPersonIcon.width }, 500, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.tutorialGroupIcon)
                    .to({ x: this.game.width + this.tutorialGroupIcon.width }, 500, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.tutorialPilotText)
                    .to({ x: this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.tutorialDontTouchText)
                    .to({ x: -this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true);
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomBar)
                    .to({ x: this.game.width + this.tutorialBottomBar.width }, 600, Phaser.Easing.Sinusoidal.In, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // hide all the instructions and menu bars again
                this.tutorialTopBar.visible = false;
                this.tutorialBottomBar.visible = false;
                this.tutorialPersonIcon.visible = false;
                this.tutorialGroupIcon.visible = false;
                this.tutorialNavigatorText.visible = false;
                this.tutorialPilotText.visible = false;
                this.tutorialDontTouchText.visible = false;
            });
        }
        getCurrentPlayersNames() {
            if (this.playersSession[this.currentRound]) {
                return this.playersSession[this.currentRound].map(playerIndex => GameController_16.GameController.Instance.players[playerIndex].name);
            }
            else {
                return null;
            }
        }
        displayBShowInstructions(player1, player2) {
            return __awaiter(this, void 0, void 0, function* () {
                this.topDialog.playerName = player1;
                this.bottomDialog.playerName = player2;
                SMAnalyticsManager_15.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                yield Promise.all([
                    this.topDialog.show(),
                    this.bottomDialog.show()
                ]);
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_15.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                yield Promise.all([
                    this.topDialog.hide(),
                    this.bottomDialog.hide()
                ]);
            });
        }
        onCodeScreenToggled(isCodeScreenVisible) {
            this.isCodeScreenVisible = isCodeScreenVisible;
        }
        createSharedInstructions() {
            this.tutorialText = new Phaser.Text(this.game, 0, 0, LocalizationManager_24.LocalizationManager.Instance.text("asteroids.tutorial_dodge"), {
                font: Game_30.Game.DEFAULT_FONT,
                fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_30.Game.FONT_SIZE_LARGE,
                stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
            });
            this.tutorialText.anchor.set(0.5);
            this.tutorialText.angle = -90;
            this.tutorialText.y = this.game.height * 0.5;
            this.tutorialText.right = this.game.width * 0.8;
            this.tutorialText.visible = false;
            this.add.existing(this.tutorialText);
        }
        /**
         * Initializes the UI elements needed for the tutorial.
         */
        createHeadsUpInstructions() {
            this.tutorialTopBar = new Phaser.TileSprite(this.game, 0, 0, this.game.height, this.game.width * 0.15, 'asteroids-tutorial-topbar');
            const tileScale = (this.game.width * 0.15) / this.tutorialTopBar.texture.height;
            this.tutorialTopBar.tileScale.set(tileScale, tileScale);
            this.tutorialTopBar.angle = -90;
            this.tutorialTopBar.visible = false;
            this.add.existing(this.tutorialTopBar);
            this.tutorialBottomBar = new Phaser.TileSprite(this.game, 0, 0, this.game.height, this.game.width * 0.15, 'asteroids-tutorial-bottombar');
            this.tutorialBottomBar.tileScale.set(tileScale, tileScale);
            this.tutorialBottomBar.angle = -90;
            this.tutorialBottomBar.visible = false;
            this.add.existing(this.tutorialBottomBar);
            this.tutorialPersonIcon = new Phaser.Image(this.game, 0, 0, 'asteroids-tutorial-player');
            this.tutorialPersonIcon.animations.add('loop', [0, 0, 0, 1, 1, 1, 0, 0, 0, 2, 2, 2], 3, true).play();
            this.tutorialPersonIcon.anchor.set(0.5, 1);
            this.tutorialPersonIcon.angle = -90;
            this.tutorialPersonIcon.visible = false;
            this.add.existing(this.tutorialPersonIcon);
            this.tutorialGroupIcon = new Phaser.Image(this.game, 0, 0, 'asteroids-tutorial-group');
            this.tutorialGroupIcon.animations.add('loop', [0, 0, 1, 1, 1, 0, 0, 0, 2, 2, 2, 0], 3, true).play();
            this.tutorialGroupIcon.anchor.set(0.5, 1);
            this.tutorialGroupIcon.angle = -90;
            this.tutorialGroupIcon.visible = false;
            this.add.existing(this.tutorialGroupIcon);
            let name = this.getCurrentPlayerName();
            this.tutorialNavigatorText = new Phaser.Text(this.game, 0, 0, LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.tutorial_navigator", { player_name: name }).toUpperCase(), {
                font: Game_30.Game.DEFAULT_FONT,
                fill: Game_30.Game.DEFAULT_FONT_STROKE,
                fontSize: 70,
                stroke: Game_30.Game.DEFAULT_FONT_COLOUR,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
            });
            this.tutorialNavigatorText.anchor.set(0.5);
            this.tutorialNavigatorText.angle = -90;
            this.tutorialNavigatorText.visible = false;
            this.add.existing(this.tutorialNavigatorText);
            TextUtil_12.TextUtil.scaleDownSingleLineTextToWidth(this.tutorialNavigatorText, this.game.height);
            this.tutorialPilotText = new Phaser.Text(this.game, 0, 0, LocalizationManager_24.LocalizationManager.Instance.textWithReplacements("asteroids.tutorial_pilot", { player_name: name }).toUpperCase(), {
                font: Game_30.Game.DEFAULT_FONT,
                fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                fontSize: 70,
                stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
            });
            this.tutorialPilotText.anchor.set(0.5);
            this.tutorialPilotText.angle = -90;
            this.tutorialPilotText.visible = false;
            this.add.existing(this.tutorialPilotText);
            TextUtil_12.TextUtil.scaleDownSingleLineTextToWidth(this.tutorialPilotText, this.game.height);
            this.codeButton = new CodeButton_2.CodeButton(this.game, this.shutdownSignal, "code_btn_floating");
            this.add.existing(this.codeButton);
            this.codeButton.visible = false;
            this.codeButton.angle = -90;
            this.codeButton.top = this.game.height * 0.96;
            this.codeButton.left = this.game.width * 0.08;
            this.tutorialDontTouchText = new Phaser.Text(this.game, 0, 0, LocalizationManager_24.LocalizationManager.Instance.text("asteroids.dont_touch_pilot").toUpperCase(), {
                font: Game_30.Game.DEFAULT_FONT,
                fill: Game_30.Game.DEFAULT_FONT_COLOUR,
                fontSize: 70,
                stroke: Game_30.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01,
                align: 'center',
            });
            this.tutorialDontTouchText.anchor.set(0.5);
            this.tutorialDontTouchText.angle = -90;
            this.tutorialDontTouchText.visible = false;
            this.add.existing(this.tutorialDontTouchText);
            TextUtil_12.TextUtil.scaleDownSingleLineTextToWidth(this.tutorialDontTouchText, this.game.height);
            this.phoneIcon = new Phaser.Image(this.game, 0, 0, 'asteroids-tutorial-phone-icon', 0);
            this.phoneIcon.anchor.setTo(0.5);
            this.phoneIcon.angle = -90;
            this.phoneIcon.x = this.game.width * 0.4;
            this.phoneIcon.y = this.game.height * 0.5;
            this.phoneIcon.visible = false;
            ScaleUtil_22.scaleToBounds(this.phoneIcon, this.game.width * 0.7, this.game.height * 0.7);
            this.add.existing(this.phoneIcon);
            this.phoneIcon.animations.add('phone-tilting', [0, 1, 0, 2]);
            this.arrowRight = new Phaser.Image(this.game, 0, 0, 'asteroids-tutorial-arrow');
            this.arrowRight.anchor.setTo(0.5);
            this.arrowRight.angle = -90;
            this.arrowRight.x = this.game.width * 0.4;
            this.arrowRight.y = this.game.height * 0.15;
            this.arrowRight.visible = false;
            ScaleUtil_22.scaleToBounds(this.arrowRight, this.game.width * 0.45, this.game.height * 0.45);
            this.add.existing(this.arrowRight);
            this.arrowLeft = new Phaser.Image(this.game, 0, 0, 'asteroids-tutorial-arrow');
            this.arrowLeft.anchor.setTo(0.5);
            this.arrowLeft.angle = -90;
            this.arrowLeft.x = this.game.width * 0.4;
            this.arrowLeft.y = this.game.height * 0.85;
            this.arrowLeft.visible = false;
            ScaleUtil_22.scaleToBounds(this.arrowLeft, this.game.width * 0.45, this.game.height * 0.45);
            this.arrowLeft.scale.x *= -1;
            this.add.existing(this.arrowLeft);
            this.createSharedInstructions();
        }
        createBShowInstructions() {
            this.fingerprintTop = new Phaser.Image(this.game, 0, 0, 'fingerprint', 0);
            this.fingerprintTop.anchor.setTo(0.5, 1.0);
            this.fingerprintTop.x = this.game.width * 0.5;
            this.fingerprintTop.y = this.game.height * 0.05;
            this.fingerprintTop.visible = false;
            ScaleUtil_22.scaleToBounds(this.fingerprintTop, this.game.width * 0.2, this.game.height * 0.2);
            this.add.existing(this.fingerprintTop);
            this.fingerprintTop.scale.y *= -1;
            this.fingerprintBottom = new Phaser.Image(this.game, 0, 0, 'fingerprint', 0);
            this.fingerprintBottom.anchor.setTo(0.5, 1.0);
            this.fingerprintBottom.x = this.game.width * 0.5;
            this.fingerprintBottom.y = this.game.height * 0.95;
            this.fingerprintBottom.visible = false;
            ScaleUtil_22.scaleToBounds(this.fingerprintBottom, this.game.width * 0.2, this.game.height * 0.2);
            this.add.existing(this.fingerprintBottom);
            this.createSharedInstructions();
            this.topDialog = new AsteroidsHalfDialog_1.AsteroidsHalfDialog(this.game, true, this.shutdownSignal);
            this.topDialog.angle = 180;
            this.topDialog.left = 0;
            this.topDialog.top = 0;
            this.add.existing(this.topDialog);
            this.bottomDialog = new AsteroidsHalfDialog_1.AsteroidsHalfDialog(this.game, false, this.shutdownSignal);
            this.bottomDialog.right = this.game.width;
            this.bottomDialog.bottom = this.game.height;
            this.add.existing(this.bottomDialog);
        }
        shutdown() {
            this.tutorialActionSignal.dispose();
            this.tutorialActionSignal = null;
            this.reorientDeviceSignal.dispose();
            this.reorientDeviceSignal = null;
            if (MotionUtil_2.MotionUtil.isGyroAvailable) {
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.movementCallback);
                this.movementCallback = null;
            }
            else {
                this.game.input.onDown.remove(this.onMouseClicked, this);
            }
            this.clearObjects();
            this.codeButton = null;
            this.fuelCellPool = null;
            this.movingDirection = null;
            this.gameState = null;
            this.lanes = [];
            this.ship = null;
            this.scoreText = null;
            this.scoreIcon = null;
            this.grid = null;
            this.itemSpawner = null;
            this.shipMovementTween = null;
            this.shipAngleMovementTween = null;
            this.shieldStatusText = null;
            this.topUI = null;
            this.itemLayer = null;
            this.tutorialLayer = null;
            this.tutorialPilotText = null;
            this.tutorialNavigatorText = null;
            this.tutorialDontTouchText = null;
            this.tutorialTopBar = null;
            this.tutorialBottomBar = null;
            this.tutorialPersonIcon = null;
            this.tutorialGroupIcon = null;
            this.tutorialText = null;
            this.phoneIcon = null;
            this.arrowRight = null;
            this.arrowLeft = null;
            this.actionTutorial = null;
            this.timerText = null;
            let soundNames = ['collect-fuel', 'game-start', 'hit-asteroid-0', 'hit-asteroid-1', 'hit-asteroid-2',
                'shield-down', 'shield-up'];
            for (let sound of soundNames) {
                this.game.cache.removeSound("asteroids-sfx-" + sound);
            }
            // load background assets
            this.game.cache.removeImage("asteroids-background");
            this.game.cache.removeImage("asteroids-grid");
            this.game.cache.removeImage("asteroids-overlay");
            this.game.cache.removeImage("asteroids-topui");
            this.game.cache.removeImage("asteroid-incoming-alert");
            this.game.cache.removeImage("asteroids-pickup");
            this.game.cache.removeImage("ship-shield");
            this.game.cache.removeImage('probe');
            this.game.cache.removeImage("progress-bar-border");
            this.game.cache.removeImage("progress-bar-inner");
            // load the lanes line
            this.game.cache.removeImage("asteroids-lane");
            // load the streak for the prove/ship and asteroid
            this.game.cache.removeImage("asteroids-streak");
            // load asteroid assets
            this.game.cache.removeImage("asteroids-ast1");
            this.game.cache.removeImage("asteroids-ast2");
            this.game.cache.removeImage("asteroids-ast3");
            this.game.cache.removeImage("asteroids-ast4");
            // load fuel cell assets
            this.game.cache.removeImage("asteroids-fuelcell-scanner");
            // load the trail spritesheet
            this.game.cache.removeImage('asteroids-trail');
            this.game.cache.removeJSON('asteroids-trail');
            // load the collision asset
            this.game.cache.removeImage("asteroid-collision");
            this.game.cache.removeImage("fuelcell-collision");
            // load the probes (with outlines)
            this.game.cache.removeImage('asteroids-probes-active');
            // removes assets for the tutorial
            this.game.cache.removeImage("fingerprint");
            this.game.cache.removeImage('eb-instruction-panel');
            this.game.cache.removeImage("asteroids-tutorial-topbar");
            this.game.cache.removeImage("asteroids-tutorial-person-holding-phone");
            this.game.cache.removeImage("asteroids-tutorial-bottombar");
            this.game.cache.removeImage("asteroids-tutorial-pause-glow");
            this.game.cache.removeImage("asteroids-tutorial-arrow");
            this.game.cache.removeImage('asteroids-tutorial-phone-icon');
            this.game.cache.removeImage("asteroids-tutorial-beam");
            this.game.cache.removeImage('asteroids-tutorial-group');
            this.game.cache.removeImage('asteroids-tutorial-player');
            super.shutdown();
        }
    }
    AsteroidsGameState.SCORE_X_POS = 0.1;
    AsteroidsGameState.SCORE_Y_POS = 0.93;
    AsteroidsGameState.GRID_SCROLL_SPEED = 0.4;
    AsteroidsGameState.MOVEMENT_PITCH = 0.4; // the pitch required for a registered move
    AsteroidsGameState.REORIENTATION_DEAD_ZONE_VALUE = 0.5;
    exports.AsteroidsGameState = AsteroidsGameState;
});
define("minigame/memory/Player", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Player {
        constructor(playerName) {
            this.correctActions = 0; // How many correct answers the player has
            this.totalActions = 0;
            this.playerName = playerName;
            this.operationsAssigned = [];
        }
        assignOperation(operationNumber) {
            this.operationsAssigned.push(operationNumber);
        }
    }
    exports.Player = Player;
});
define("minigame/memory/Instruction", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Instruction {
        constructor(playerName, roundNumber, player, answer) {
            this.playerName = playerName;
            this.roundNumber = roundNumber;
            this.player = player;
            this.answer = answer;
        }
    }
    exports.Instruction = Instruction;
});
define("minigame/memory/MemoryGameLogic", ["require", "exports", "logic/GameController", "StateNames", "minigame/memory/Player", "minigame/memory/Instruction", "switchback-common/localization/LocalizationManager", "switchback-common/util/MathUtil", "switchback-common/util/CollectionUtil"], function (require, exports, GameController_17, StateNames_10, Player_1, Instruction_1, LocalizationManager_25, MathUtil_2, CollectionUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var MemoryInputButton;
    (function (MemoryInputButton) {
        MemoryInputButton[MemoryInputButton["THRUSTERS_1"] = 1] = "THRUSTERS_1";
        MemoryInputButton[MemoryInputButton["THRUSTERS_2"] = 2] = "THRUSTERS_2";
        MemoryInputButton[MemoryInputButton["THRUSTERS_3"] = 3] = "THRUSTERS_3";
        MemoryInputButton[MemoryInputButton["THRUSTERS_4"] = 4] = "THRUSTERS_4";
        MemoryInputButton[MemoryInputButton["AIRLOCK"] = 5] = "AIRLOCK";
        MemoryInputButton[MemoryInputButton["VENT_ARM"] = 6] = "VENT_ARM";
        MemoryInputButton[MemoryInputButton["INTERNAL_POWER_INCREASE"] = 7] = "INTERNAL_POWER_INCREASE";
        MemoryInputButton[MemoryInputButton["INTERNAL_POWER_DECREASE"] = 8] = "INTERNAL_POWER_DECREASE";
        MemoryInputButton[MemoryInputButton["THROTTLE_INCREASE"] = 9] = "THROTTLE_INCREASE";
        MemoryInputButton[MemoryInputButton["THROTTLE_DECREASE"] = 10] = "THROTTLE_DECREASE";
    })(MemoryInputButton = exports.MemoryInputButton || (exports.MemoryInputButton = {}));
    class MemoryGameLogic {
        constructor() {
            this.playerActions = [];
            // Setup players
            // For the sake of the scoring simplicity, this has the same indexes as the
            // players in GameController.players
            this.playersArray = GameController_17.GameController.Instance.players.map((player) => new Player_1.Player(player.name));
            const numPlays = GameController_17.GameController.Instance.state.getMinigamePlays(StateNames_10.StateNames.GAME_MEMORY);
            this.difficulty = Math.min(numPlays, 2);
            // Shuffle players
            this.playersInGame = this.playersArray.slice();
            Phaser.ArrayUtils.shuffle(this.playersInGame);
            // Notice that playersInGame is already shuffled, that's why it's safe to take always the first player
            this.controllerPlayer = this.playersInGame[0];
            // if there are more than 2 players, one is a dedicated controller (otherwise is a soft controller)
            this.dedicatedController = this.playersInGame.length > 2;
            if (this.dedicatedController) {
                // If 3 or more players in the team, exclude controller from game
                this.playersInGame.splice(0, 1);
            }
            this.timeRemainingS = MemoryGameLogic.TIME_ALLOCATED_S;
            // Round is incremented in assignPlayerOperations()
            this.round = 0;
            this.gameOver = false;
            this.score = 0;
            this.operations = new Map();
            for (let item in MemoryInputButton) {
                // Iterating over enums is hard
                // This check ensures we are only iterating over the enum *values* (e.g. "1", "2"), not the names as well
                if (Number(item)) {
                    this.operations.set(Number(item), LocalizationManager_25.LocalizationManager.Instance.text("memory.operations." + item.valueOf()));
                }
            }
            this.taskNumberMap = new Map();
            for (let i = 1; i < 10; i++) {
                this.taskNumberMap.set(i, LocalizationManager_25.LocalizationManager.Instance.text("memory.task_numbers." + i));
            }
            this.operationsInGame = [];
            this.operations.forEach((value, key, map) => {
                this.operationsInGame.push(key);
            });
            Phaser.ArrayUtils.shuffle(this.operationsInGame);
            this.nextOperationIndex = 0;
            this.nextPlayerIndex = 0;
        }
        addPlayerAction(id) {
            if (this.playerActions) {
                this.playerActions.push(id);
            }
        }
        checkAction() {
            // if the player action is not the same as the given operation, action is wrong
            const currentTurn = this.playerActions.length - 1;
            const currentInstruction = this.instructionsInPlay[currentTurn];
            let thisPlayer = currentInstruction.player;
            thisPlayer.totalActions += 1; // count one action
            if (currentInstruction.answer === this.playerActions[currentTurn] && !this.gameOver) {
                this.score += 1;
                thisPlayer.correctActions += 1; // count a hit
                return true;
            }
            else {
                this.timeRemainingS = Math.max(0, this.timeRemainingS - MemoryGameLogic.WRONG_ANSWER_TIME_PENALTY[this.difficulty]);
                this.playerActions.pop();
                return false;
            }
        }
        assignPlayerOperations() {
            this.round++;
            // assigns each player an operation and displays it
            for (let i = 0; i < this.playersArray.length; i++) {
                // assigns a random operation to each player
                let operationNumber = this.getNextOperation();
                const tempPlayer = this.playersArray[i];
                /*
                * If the player already has the same operation from the previous assignment,
                * Change the operation.
                */
                const hasPreviousAssignment = tempPlayer.operationsAssigned.length >= 1;
                if (hasPreviousAssignment) {
                    const playerPreviousAssignment = tempPlayer.operationsAssigned[tempPlayer.operationsAssigned.length - 1];
                    if (operationNumber === playerPreviousAssignment) {
                        operationNumber = this.getNextOperation();
                    }
                }
                tempPlayer.assignOperation(operationNumber);
            }
            // initializeRound
            // delete all previous actions
            this.playerActions = [];
            let winActions = [];
            this.instructionsInPlay = [];
            // increase the number of actions required by players to take
            const numberOfActions = Math.min(this.round, MemoryGameLogic.MAX_ACTIONS_PER_GUEST[this.difficulty]) * this.playersInGame.length;
            for (let i = 0; i < numberOfActions; i++) {
                // randomly pick a player and action/round number
                const rndPlayer = this.getNextPlayer();
                let rndActionNum = MathUtil_2.getRandomInt(0, this.round - 1);
                if (rndPlayer.lastActionIndexQueried !== undefined && rndPlayer.lastActionIndexQueried === rndActionNum) {
                    rndActionNum = (rndActionNum + 1) % this.round;
                }
                // add the current round action number to the player's action index queried array so that it will not repeat next round
                rndPlayer.lastActionIndexQueried = rndActionNum;
                // used for game logic
                // using the above 2 randomized variables, find the operation number
                // push the operation number found into winActions array
                const nextOperationNum = rndPlayer.operationsAssigned[rndActionNum];
                winActions.push(nextOperationNum);
                // push the randomized player and randomized round action into instructions, and the expected answer
                this.instructionsInPlay.push(new Instruction_1.Instruction(rndPlayer.playerName, rndActionNum, rndPlayer, nextOperationNum));
            }
        }
        getNextPlayer() {
            console.log("[Memory] Getting next player " + this.nextPlayerIndex);
            const currentPlayer = this.playersInGame[this.nextPlayerIndex];
            this.nextPlayerIndex++;
            if (this.nextPlayerIndex === this.playersInGame.length) {
                // shuffle the array
                this.playersInGame = CollectionUtil_2.shuffleNonRepeatingArray(this.playersInGame);
                this.nextPlayerIndex = 0;
            }
            return currentPlayer;
        }
        getNextOperation() {
            const currentOperation = this.operationsInGame[this.nextOperationIndex];
            this.nextOperationIndex++;
            if (this.nextOperationIndex === this.operationsInGame.length) {
                // shuffle the array
                this.operationsInGame = CollectionUtil_2.shuffleNonRepeatingArray(this.operationsInGame);
                this.nextOperationIndex = 0;
            }
            return currentOperation;
        }
        // Post the score, the general and the player scores
        // force, skips the check for a score (used by the interrupted game)
        postScores(force = false) {
            // Ensures the game doesn't get mark completed if players 
            // just started the game before getting interrupted
            if (!(force || this.score)) {
                return;
            }
            // Minigame score
            // Mark a minigame as completed and scores the game
            GameController_17.GameController.Instance.postMinigameScore(StateNames_10.StateNames.GAME_MEMORY, this.score);
        }
    }
    MemoryGameLogic.TIME_ALLOCATED_S = 90;
    // caps the max number of actions per guest each round to 3
    // each parameter has multiple values based on game difficulty
    MemoryGameLogic.MAX_ACTIONS_PER_GUEST = [3, 3, 2];
    MemoryGameLogic.WRONG_ANSWER_TIME_PENALTY = [6, 8, 10];
    exports.MemoryGameLogic = MemoryGameLogic;
});
define("minigame/memory/MemoryInputs", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil"], function (require, exports, Game_31, ScaleUtil_23, PromiseUtil_37) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MemoryInput extends Phaser.Image {
        constructor(game, x, y, tag, inputId) {
            super(game, x, y, tag);
            this.inputId = inputId;
        }
    }
    exports.MemoryInput = MemoryInput;
    class SlidingToggle extends Phaser.Group {
        constructor(game, toggledLeftText, toggledRightText, inputId) {
            super(game);
            this.inputEnableChildren = true;
            this.isToggledLeft = true;
            this.background = new MemoryInput(this.game, 0, 0, "button-toggle", inputId);
            this.background.anchor.set(0.5);
            ScaleUtil_23.scaleToBounds(this.background, this.game.width * 0.45, this.game.height * 0.15);
            this.toggleCircle = new Phaser.Image(this.game, 0, 0, "button-toggle-circle");
            this.toggleCircleOffset = this.background.height * 0.2;
            ScaleUtil_23.scaleToBounds(this.toggleCircle, this.background.height * 0.65, this.background.height * 0.65);
            this.toggleCircle.left = this.background.left + this.toggleCircleOffset;
            this.toggleCircle.centerY = this.background.centerY;
            this.toggledLeftText = new Phaser.Text(this.game, 0, 0, toggledLeftText, {
                font: Game_31.Game.DEFAULT_FONT,
                fill: Game_31.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_31.Game.FONT_SIZE_SMALL
            });
            this.toggledLeftText.setScaleMinMax(1);
            this.toggledRightText = new Phaser.Text(this.game, 0, 0, toggledRightText, {
                font: Game_31.Game.DEFAULT_FONT,
                fill: Game_31.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_31.Game.FONT_SIZE_SMALL
            });
            this.toggledRightText.setScaleMinMax(1);
            this.toggledRightText.alpha = 0;
            const textOffset = (this.background.width - this.toggleCircle.width) / 2;
            this.toggledLeftText.centerX = this.background.width * -0.5 + this.toggleCircle.width + textOffset;
            this.toggledLeftText.centerY = this.background.centerY;
            this.toggledRightText.centerX = this.background.width * -0.5 + textOffset;
            this.toggledRightText.centerY = this.background.centerY;
            this.add(this.background);
            this.add(this.toggleCircle);
            this.add(this.toggledLeftText);
            this.add(this.toggledRightText);
            this.onChildInputUp.add(() => {
                this.onInputClickedSignal.dispatch(this.background);
                this.toggleState();
            });
        }
        toggleState() {
            // Hide/show left/right text
            this.game.add.tween(this.toggledLeftText).to({ alpha: !this.isToggledLeft }, 500, Phaser.Easing.Linear.None, true);
            this.game.add.tween(this.toggledRightText).to({ alpha: this.isToggledLeft }, 500, Phaser.Easing.Linear.None, true);
            // Slide circle
            if (this.isToggledLeft) {
                this.game.add.tween(this.toggleCircle).to({ right: this.background.right - this.toggleCircleOffset }, 500, Phaser.Easing.Linear.None, true);
            }
            else {
                this.game.add.tween(this.toggleCircle).to({ left: this.background.left + this.toggleCircleOffset }, 500, Phaser.Easing.Linear.None, true);
            }
            this.isToggledLeft = !this.isToggledLeft;
        }
        destroy() {
            this.onChildInputUp.removeAll();
            this.onInputClickedSignal.dispose();
            this.onInputClickedSignal = null;
            super.destroy();
        }
    }
    exports.SlidingToggle = SlidingToggle;
    class RoundToggle extends Phaser.Group {
        constructor(game, buttonContent, inputId, spinDelay) {
            super(game);
            this.isToggledOn = false;
            this.inputEnableChildren = true;
            // Need to spin
            this.lightStateBackground = new MemoryInput(this.game, 0, 0, "button-thrusters", inputId);
            this.lightStateBackground.frame = 1;
            ScaleUtil_23.scaleToBounds(this.lightStateBackground, this.game.width * 0.2, this.game.width * 0.2);
            this.add(this.lightStateBackground);
            this.lightStateBackground.anchor.set(0.5);
            this.game.add.tween(this.lightStateBackground).to({ angle: 360 }, 4000, Phaser.Easing.Linear.None, true, spinDelay, Infinity);
            this.darkStateBackground = new Phaser.Image(this.game, 0, 0, "button-thrusters", 0);
            ScaleUtil_23.scaleToBounds(this.darkStateBackground, this.game.width * 0.2, this.game.width * 0.2);
            this.darkStateBackground.visible = false;
            this.add(this.darkStateBackground);
            this.darkStateBackground.anchor.set(0.5);
            this.game.add.tween(this.darkStateBackground).to({ angle: 360 }, 4000, Phaser.Easing.Linear.None, true, spinDelay, Infinity);
            this.buttonLabel = new Phaser.Text(this.game, 0, 0, buttonContent, {
                font: Game_31.Game.DEFAULT_FONT,
                fill: 'black',
                fontSize: Game_31.Game.FONT_SIZE_MEDIUM
            });
            this.buttonLabel.setScaleMinMax(1);
            this.buttonLabel.centerX = this.lightStateBackground.centerX;
            this.buttonLabel.centerY = this.lightStateBackground.centerY;
            this.add(this.buttonLabel);
            this.onChildInputUp.add(() => {
                this.onInputClickedSignal.dispatch(this.lightStateBackground);
                this.toggleState();
            });
        }
        toggleState() {
            if (this.isToggledOn) {
                this.isToggledOn = false;
                this.lightStateBackground.visible = true;
                this.darkStateBackground.visible = false;
                this.buttonLabel.fill = 'black';
            }
            else {
                this.isToggledOn = true;
                this.darkStateBackground.visible = true;
                this.lightStateBackground.visible = false;
                this.buttonLabel.fill = 'white';
            }
        }
        destroy() {
            this.onChildInputUp.removeAll();
            this.onInputClickedSignal.dispose();
            this.onInputClickedSignal = null;
            super.destroy();
        }
    }
    exports.RoundToggle = RoundToggle;
    class PowerButtonPanel extends Phaser.Group {
        constructor(game, leftButtonText, rightButtonText, leftInputId, rightInputId) {
            super(game);
            const background = new Phaser.Image(this.game, 0, 0, "powerbutton-outline");
            ScaleUtil_23.scaleToBounds(background, this.game.width * 0.9, this.game.height * 0.1);
            this.add(background);
            const buttonLeft = new MemoryInput(this.game, 0, 0, "button-power", leftInputId);
            buttonLeft.inputEnabled = true;
            ScaleUtil_23.scaleToBounds(buttonLeft, this.game.width * 0.34, this.game.height * 0.09);
            buttonLeft.anchor.set(0.5);
            buttonLeft.angle = 180;
            buttonLeft.left = background.left + this.game.width * 0.015;
            buttonLeft.centerY = background.centerY;
            buttonLeft.events.onInputUp.add(() => {
                this.onInputClickedSignal.dispatch(buttonLeft);
                this.game.add.tween(buttonLeft.scale).to({ x: 0.8, y: 0.8 }, 100, Phaser.Easing.Linear.None, true, 0, 0, true);
                this.playActiveAnimation();
            });
            this.add(buttonLeft);
            const buttonRight = new MemoryInput(this.game, 0, 0, "button-power", rightInputId);
            buttonRight.inputEnabled = true;
            ScaleUtil_23.scaleToBounds(buttonRight, this.game.width * 0.34, this.game.height * 0.09);
            buttonRight.anchor.set(0.5);
            buttonRight.right = background.right - this.game.width * 0.015;
            buttonRight.centerY = background.centerY;
            buttonRight.events.onInputUp.add(() => {
                this.onInputClickedSignal.dispatch(buttonRight);
                this.game.add.tween(buttonRight.scale).to({ x: 0.8, y: 0.8 }, 100, Phaser.Easing.Linear.None, true, 0, 0, true);
                this.playActiveAnimation();
            });
            this.add(buttonRight);
            const leftText = new Phaser.Text(this.game, 0, 0, leftButtonText, {
                font: Game_31.Game.DEFAULT_FONT,
                fill: 'black',
                fontSize: Game_31.Game.FONT_SIZE_SMALL
            });
            leftText.setScaleMinMax(1);
            leftText.centerX = buttonLeft.centerX;
            leftText.centerY = background.centerY;
            this.add(leftText);
            const rightText = new Phaser.Text(this.game, 0, 0, rightButtonText, {
                font: Game_31.Game.DEFAULT_FONT,
                fill: 'black',
                fontSize: Game_31.Game.FONT_SIZE_SMALL
            });
            rightText.setScaleMinMax(1);
            rightText.centerX = buttonRight.centerX;
            rightText.centerY = background.centerY;
            this.add(rightText);
            // Add animated elements
            this.powerBarsIdle = new Phaser.Image(this.game, 0, 0, "powerbars-idle", "00");
            ScaleUtil_23.scaleToBounds(this.powerBarsIdle, this.game.width * 0.15, this.game.height * 0.1);
            this.powerBarsIdle.centerX = background.centerX;
            this.powerBarsIdle.centerY = background.centerY;
            this.add(this.powerBarsIdle);
            let idleAnimFrames = [];
            const frameCount = this.game.cache.getFrameCount("powerbars-idle");
            for (let i = 0; i <= frameCount; i++) {
                const frameName = i < 10 ? "0" + i : i.toString();
                idleAnimFrames.push(frameName);
            }
            this.powerBarsIdle.animations.add("idle", idleAnimFrames, 20, true, false);
            this.powerBarsIdle.animations.play("idle");
            this.powerBarsActive = new Phaser.Image(this.game, 0, 0, "powerbars-active", "00");
            ScaleUtil_23.scaleToBounds(this.powerBarsActive, this.game.width * 0.15, this.game.height * 0.1);
            this.powerBarsActive.centerX = background.centerX;
            this.powerBarsActive.centerY = background.centerY;
            this.powerBarsActive.visible = false;
            this.add(this.powerBarsActive);
            let activeAnimFrames = [];
            for (let i = 0; i <= 12; i++) {
                const frameName = i < 10 ? "0" + i : i.toString();
                activeAnimFrames.push(frameName);
            }
            this.powerBarsActive.animations.add("active", activeAnimFrames, 18, true, false);
        }
        playActiveAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                this.powerBarsIdle.visible = false;
                this.powerBarsActive.visible = true;
                this.powerBarsActive.animations.play("active");
                yield PromiseUtil_37.PromiseUtil.time(this.game, 2500);
                this.powerBarsIdle.visible = true;
                this.powerBarsActive.visible = false;
                this.powerBarsIdle.animations.play("idle");
            });
        }
        destroy() {
            this.onInputClickedSignal.dispose();
            this.onInputClickedSignal = null;
            super.destroy();
        }
    }
    exports.PowerButtonPanel = PowerButtonPanel;
});
define("minigame/memory/SlidingInstructionPanelComponents", ["require", "exports", "Game", "minigame/memory/MemoryGameState", "minigame/memory/SlidingInstructionPanel", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil"], function (require, exports, Game_32, MemoryGameState_1, SlidingInstructionPanel_1, ScaleUtil_24, LocalizationManager_26, PromiseUtil_38) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    // This class is the animated graph images shown as feedback after an action
    class FeedbackAnimationGroup extends Phaser.Group {
        constructor(game, isCorrect) {
            super(game);
            const textKey = isCorrect ? "memory.correct" : "memory.incorrect";
            const text = new Phaser.Text(this.game, 0, 0, LocalizationManager_26.LocalizationManager.Instance.text(textKey), {
                font: Game_32.Game.DEFAULT_FONT,
                fill: isCorrect ? MemoryGameState_1.MemoryGameState.GREEN : MemoryGameState_1.MemoryGameState.RED,
                fontSize: 90
            });
            text.setScaleMinMax(1);
            text.centerX = this.game.width * 0.5;
            this.add(text);
            const imageKey = isCorrect ? "correct-bars" : "incorrect-bars";
            this.animation = new Phaser.Image(this.game, 0, 0, imageKey, "00");
            ScaleUtil_24.scaleToBounds(this.animation, this.game.width * 0.55, this.game.height * 0.2);
            this.animation.centerX = this.game.width * 0.5;
            let animFrames = [];
            const frameCount = this.game.cache.getFrameCount(imageKey);
            for (let i = 0; i <= frameCount; i++) {
                const frameName = i < 10 ? "0" + i : i.toString();
                animFrames.push(frameName);
            }
            this.animation.animations.add("feedback", animFrames, 20, true, false);
            this.animation.centerX = this.game.width * 0.5;
            this.animation.bottom = text.top;
            this.add(this.animation);
        }
        playAnimation(shutdownSignal) {
            return __awaiter(this, void 0, void 0, function* () {
                let isShutdown = false;
                yield PromiseUtil_38.PromiseUtil.animation(this.animation.animations.play("feedback", 18, false), shutdownSignal).catch(() => { isShutdown = true; });
                return !isShutdown;
            });
        }
    }
    exports.FeedbackAnimationGroup = FeedbackAnimationGroup;
    // This group is the set of text instructing the controller
    // e.g. "Steven, Perform Nick's first task"
    class InstructionGroup extends Phaser.Group {
        constructor(game) {
            super(game);
            this.controllerNameText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, SlidingInstructionPanel_1.SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fill: MemoryGameState_1.MemoryGameState.YELLOW, fontSize: Game_32.Game.FONT_SIZE_MEDIUM }));
            this.controllerNameText.setScaleMinMax(1);
            this.instructionText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, SlidingInstructionPanel_1.SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fontSize: Game_32.Game.FONT_SIZE_MEDIUM, align: 'center', wordWrap: true, wordWrapWidth: this.game.width * 0.9 }));
            this.instructionText.setScaleMinMax(1);
            this.instructionText.centerX = this.game.width * 0.5;
            this.instructionText.top = this.controllerNameText.bottom;
            this.controllerNameText.centerX = this.instructionText.centerX;
            this.add(this.controllerNameText);
            this.add(this.instructionText);
        }
        show(instructions, name) {
            this.controllerNameText.text = name;
            this.instructionText.text = instructions;
            this.instructionText.centerX = this.game.width * 0.5;
            this.controllerNameText.centerX = this.instructionText.centerX;
        }
    }
    exports.InstructionGroup = InstructionGroup;
    // This is the group of text that initially notifies each player of their action
    // e.g. "Cinderella, Remember, Your nth task is:"
    class DialogInstructionTextGroup extends Phaser.Group {
        constructor(game, playerNameText, instructionText) {
            super(game);
            this.width = this.game.width;
            this.playerName = new Phaser.Text(this.game, 0, 0, playerNameText, Object.assign({}, SlidingInstructionPanel_1.SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fill: MemoryGameState_1.MemoryGameState.YELLOW, fontSize: Game_32.Game.FONT_SIZE_LARGE }));
            this.playerName.setScaleMinMax(1);
            this.playerName.anchor.set(0.5, 0);
            this.add(this.playerName);
            this.reminder = new Phaser.Text(this.game, 0, 0, LocalizationManager_26.LocalizationManager.Instance.text("memory.task_notification"), Object.assign({}, SlidingInstructionPanel_1.SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { wordWrap: true, wordWrapWidth: this.game.width * 0.9, align: 'center', fontSize: Game_32.Game.FONT_SIZE_LARGE }));
            this.reminder.setScaleMinMax(1);
            this.reminder.anchor.set(0.5, 0);
            this.reminder.top = this.playerName.bottom;
            this.add(this.reminder);
            this.instruction = new Phaser.Text(this.game, 0, 0, instructionText, Object.assign({}, SlidingInstructionPanel_1.SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fill: MemoryGameState_1.MemoryGameState.YELLOW, wordWrap: true, wordWrapWidth: this.game.width * 0.9, align: 'center', fontSize: Game_32.Game.FONT_SIZE_LARGE }));
            this.instruction.setScaleMinMax(1);
            this.instruction.top = this.reminder.bottom;
            this.instruction.anchor.set(0.5, 0);
            this.add(this.instruction);
            this.playerName.centerX = this.width * 0.5;
            this.reminder.centerX = this.playerName.centerX;
            this.instruction.centerX = this.playerName.centerX;
        }
        setDialogInstructions(playerName, taskNumber, task) {
            this.playerName.text = playerName;
            this.reminder.text = LocalizationManager_26.LocalizationManager.Instance.textWithReplacements("memory.task_notification", { nth: taskNumber });
            this.instruction.text = task;
        }
    }
    exports.DialogInstructionTextGroup = DialogInstructionTextGroup;
});
define("minigame/memory/SlidingInstructionPanel", ["require", "exports", "Game", "minigame/memory/MemoryGameState", "minigame/memory/SlidingInstructionPanelComponents", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil"], function (require, exports, Game_33, MemoryGameState_2, SlidingInstructionPanelComponents_1, ScaleUtil_25, LocalizationManager_27, PromiseUtil_39) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /*
        This class handles all behaviour needed by the instruction panel on the top
        of the screen, that animates to the middle of the screen between rounds.
    */
    class SlidingInstructionPanel extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            SlidingInstructionPanel.DEFAULT_TEXT_STYLE = {
                font: Game_33.Game.DEFAULT_FONT,
                fill: Game_33.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_33.Game.FONT_SIZE_MEDIUM,
                stroke: Game_33.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_33.Game.FONT_STROKE_SMALL
            };
            this.retractedPanelPosition = this.game.height * 0.18;
            this.isDialogShown = false;
            this.backgroundImage = new Phaser.Image(this.game, 0, 0, "instruction-panel");
            this.backgroundImage.width = this.game.width;
            this.backgroundImage.height = this.game.height * 0.4;
            this.backgroundImage.bottom = this.retractedPanelPosition;
            this.backgroundImage.centerX = this.game.width * 0.5;
            this.add(this.backgroundImage);
            // Perform instruction text
            this.instructionDisplayGroup = new SlidingInstructionPanelComponents_1.InstructionGroup(this.game);
            this.instructionDisplayGroup.bottom = this.backgroundImage.bottom;
            this.instructionDisplayGroup.visible = false;
            this.add(this.instructionDisplayGroup);
            // Correct/Incorrect result animations
            this.correctDisplayGroup = new SlidingInstructionPanelComponents_1.FeedbackAnimationGroup(this.game, true);
            this.correctDisplayGroup.bottom = this.bottom;
            this.correctDisplayGroup.visible = false;
            this.add(this.correctDisplayGroup);
            this.incorrectDisplayGroup = new SlidingInstructionPanelComponents_1.FeedbackAnimationGroup(this.game, false);
            this.incorrectDisplayGroup.bottom = this.bottom;
            this.incorrectDisplayGroup.visible = false;
            this.add(this.incorrectDisplayGroup);
            this.taskReminderGroup = new Phaser.Group(this.game);
            const taskReminderTitle = new Phaser.Text(this.game, 0, 0, LocalizationManager_27.LocalizationManager.Instance.text("memory.the_task_was"), SlidingInstructionPanel.DEFAULT_TEXT_STYLE);
            taskReminderTitle.setScaleMinMax(1);
            taskReminderTitle.centerX = this.game.width * 0.5;
            this.taskReminderGroup.add(taskReminderTitle);
            this.taskReminderText = new Phaser.Text(this.game, 0, 0, LocalizationManager_27.LocalizationManager.Instance.text("memory.operations.1"), Object.assign({}, SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fill: MemoryGameState_2.MemoryGameState.YELLOW, wordWrap: true, wordWrapWidth: this.game.width * 0.9, align: 'center' }));
            this.taskReminderText.setScaleMinMax(1);
            this.taskReminderText.centerX = this.game.width * 0.5;
            this.taskReminderText.top = taskReminderTitle.bottom;
            this.taskReminderGroup.add(this.taskReminderText);
            this.taskReminderGroup.centerY = this.game.height * 0.09;
            this.taskReminderGroup.visible = false;
            this.add(this.taskReminderGroup);
            this.dialogContents = new SlidingInstructionPanelComponents_1.DialogInstructionTextGroup(this.game, "", "");
            this.dialogContents.centerX = this.game.width * 0.5;
            this.dialogContents.centerY = this.backgroundImage.centerY;
            this.add(this.dialogContents);
            this.nextRoundText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, SlidingInstructionPanel.DEFAULT_TEXT_STYLE, { fontSize: Game_33.Game.FONT_SIZE_LARGE, fill: MemoryGameState_2.MemoryGameState.YELLOW, wordWrap: true, wordWrapWidth: this.game.width * 0.9, align: 'center' }));
            this.nextRoundText.setScaleMinMax(1);
            this.nextRoundText.anchor.set(0.5);
            this.nextRoundText.x = this.game.width * 0.5;
            this.nextRoundText.y = this.backgroundImage.centerY;
            this.nextRoundText.visible = false;
            this.add(this.nextRoundText);
            this.okButton = new Phaser.Image(this.game, 0, 0, "button-ok");
            this.okButton.inputEnabled = true;
            this.okButton.top = this.backgroundImage.bottom;
            this.okButton.visible = false;
            this.add(this.okButton);
            this.okButton.events.onInputUp.add(() => {
                this.onButtonClickedSignal.dispatch();
            });
            const okButtonText = new Phaser.Text(this.game, 0, 0, LocalizationManager_27.LocalizationManager.Instance.text("buttons.ok"), {
                font: Game_33.Game.DEFAULT_FONT,
                fill: 'black',
                fontSize: Game_33.Game.FONT_SIZE_MEDIUM,
                align: 'center'
            });
            okButtonText.setScaleMinMax(1);
            this.okButton.addChild(okButtonText);
            okButtonText.anchor.set(0.5);
            okButtonText.x = this.okButton.width * 0.5;
            okButtonText.y = this.okButton.height * 0.5;
            ScaleUtil_25.scaleToBounds(this.okButton, this.game.width * 0.4, this.game.height * 0.1);
            this.okButton.centerX = this.game.width * 0.5;
        }
        // Shows Correct/Incorrect animation based on result
        showFeedback(isCorrect, correctInstructionText) {
            return __awaiter(this, void 0, void 0, function* () {
                this.instructionDisplayGroup.visible = false;
                if (isCorrect) {
                    // Play correct animation
                    this.correctDisplayGroup.visible = true;
                    this.isShutdown = !(yield this.correctDisplayGroup.playAnimation(this.shutdownSignal));
                    if (this.isShutdown) {
                        return;
                    }
                    yield PromiseUtil_39.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    this.correctDisplayGroup.visible = false;
                }
                else {
                    // Play incorrect animation
                    this.incorrectDisplayGroup.visible = true;
                    this.isShutdown = !(yield this.incorrectDisplayGroup.playAnimation(this.shutdownSignal));
                    if (this.isShutdown) {
                        return;
                    }
                    this.incorrectDisplayGroup.visible = false;
                    this.game.sound.play("flash-correct-answer");
                    // Flash instruction reminder text
                    let i = 5;
                    while (--i) {
                        // Set reminder text to correct instruction
                        this.taskReminderText.text = correctInstructionText;
                        // Recenter text
                        this.taskReminderText.centerX = this.game.width * 0.5;
                        this.taskReminderGroup.centerY = this.game.height * 0.09;
                        this.taskReminderGroup.visible = true;
                        yield PromiseUtil_39.PromiseUtil.time(this.game, 200, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                        if (this.isShutdown) {
                            return;
                        }
                        this.taskReminderGroup.visible = false;
                        yield PromiseUtil_39.PromiseUtil.time(this.game, 200, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                        if (this.isShutdown) {
                            return;
                        }
                    }
                    this.taskReminderGroup.visible = true;
                    yield PromiseUtil_39.PromiseUtil.time(this.game, 1000, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    this.taskReminderGroup.visible = false;
                }
                this.instructionDisplayGroup.visible = true;
            });
        }
        showInstructionsDialog(playerName, taskNumber, task) {
            this.dialogContents.setDialogInstructions(playerName, taskNumber, task);
            this.dialogContents.visible = true;
            this.dialogContents.centerY = this.backgroundImage.y + this.backgroundImage.height * 0.5;
            if (!this.isDialogShown) {
                this.game.add.tween(this)
                    .to({ centerY: this.game.height * 0.4 }, 500, Phaser.Easing.Sinusoidal.InOut, true)
                    .onComplete.addOnce(() => { this.okButton.visible = true; });
                this.isDialogShown = true;
            }
        }
        showInstructions(instructions, name) {
            this.instructionDisplayGroup.show(instructions, name);
            this.instructionDisplayGroup.visible = true;
            this.instructionDisplayGroup.centerY = this.game.height * 0.09;
        }
        showNextRoundStarting(nthRound) {
            this.hideAllContent();
            this.nextRoundText.text = LocalizationManager_27.LocalizationManager.Instance.textWithReplacements("memory.next_round_starting", { nth: nthRound });
            this.nextRoundText.visible = true;
            if (!this.isDialogShown) {
                this.game.add.tween(this)
                    .to({ centerY: this.game.height * 0.4 }, 500, Phaser.Easing.Linear.None, true)
                    .onComplete.addOnce(() => { this.okButton.visible = true; });
                this.isDialogShown = true;
            }
        }
        hideDialog() {
            this.isDialogShown = false;
            this.hideAllContent();
            this.game.add.tween(this)
                .to({ bottom: this.retractedPanelPosition + this.okButton.height }, 500, Phaser.Easing.Sinusoidal.InOut, true)
                .onComplete.addOnce(() => { this.okButton.visible = false; });
        }
        hideAllContent() {
            const contentItems = [
                this.instructionDisplayGroup,
                this.dialogContents,
                this.correctDisplayGroup,
                this.incorrectDisplayGroup,
                this.taskReminderGroup,
                this.nextRoundText
            ];
            for (let i = 0; i < contentItems.length; i++) {
                contentItems[i].visible = false;
            }
        }
        destroy() {
            this.shutdownSignal = null;
            this.okButton.events.onInputUp.removeAll();
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.SlidingInstructionPanel = SlidingInstructionPanel;
});
define("minigame/memory/Intro", ["require", "exports", "component/dialog/BasicSpaceButtonDialog", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "switchback-common/util/PromiseUtil", "component/dialog/TwoTextDialog", "logic/SMAnalyticsManager", "StateNames", "logic/GameController"], function (require, exports, BasicSpaceButtonDialog_2, LocalizationManager_28, TextUtil_13, PromiseUtil_40, TwoTextDialog_2, SMAnalyticsManager_16, StateNames_11, GameController_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to group the dialogs showing the instructions previous to the game
     * @author Jeronimo@CerebralFix.com
     */
    class Intro extends Phaser.Group {
        constructor(game, commander, commanderPlays, shutdownSignal) {
            super(game);
            this.isShutdown = false;
            this.shutdownSignal = shutdownSignal;
            const textReplacement = { name: commander };
            this.dialogA = new TwoTextDialog_2.TwoTextDialog(this.game, LocalizationManager_28.LocalizationManager.Instance.text("memory.intro.page1_1"), LocalizationManager_28.LocalizationManager.Instance.textWithReplacements("memory.intro.page1_2", textReplacement), "buttons.next", this.shutdownSignal, TextUtil_13.TextUtil.getFontSize(this.game.width, 30));
            this.dialogB = new BasicSpaceButtonDialog_2.BasicSpaceButtonDialog(this.game, LocalizationManager_28.LocalizationManager.Instance.textWithReplacements("memory.intro.page2", textReplacement), LocalizationManager_28.LocalizationManager.Instance.text("buttons.next"), this.shutdownSignal, TextUtil_13.TextUtil.getFontSize(this.game.width, 30));
            let dialogCInstructions = LocalizationManager_28.LocalizationManager.Instance.textWithReplacements("memory.intro.page3_1", textReplacement);
            if (commanderPlays) {
                dialogCInstructions += "\n" + LocalizationManager_28.LocalizationManager.Instance.textWithReplacements("memory.intro.page3_2", textReplacement);
            }
            this.dialogC = new BasicSpaceButtonDialog_2.BasicSpaceButtonDialog(this.game, dialogCInstructions, LocalizationManager_28.LocalizationManager.Instance.text("buttons.next"), this.shutdownSignal, TextUtil_13.TextUtil.getFontSize(this.game.width, 30));
            this.add(this.dialogA);
            this.add(this.dialogB);
            this.add(this.dialogC);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.dialogA.show();
                if (this.dialogA.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                this.trackState(1);
                yield PromiseUtil_40.PromiseUtil.signal(this.dialogA.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.dialogA.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                yield this.dialogA.hide();
                if (this.dialogA.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                yield this.dialogB.show();
                if (this.dialogB.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                this.trackState(2);
                yield PromiseUtil_40.PromiseUtil.signal(this.dialogB.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.dialogB.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                yield this.dialogB.hide();
                if (this.dialogB.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                yield this.dialogC.show();
                if (this.dialogC.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                this.trackState(3);
                yield PromiseUtil_40.PromiseUtil.signal(this.dialogC.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.dialogC.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
                yield this.dialogC.hide();
                if (this.dialogC.isShutdown) {
                    this.isShutdown = true;
                    return;
                }
            });
        }
        trackState(pageNum) {
            const metricsGameName = SMAnalyticsManager_16.SMAnalyticsManager.gameNameMap.get(StateNames_11.StateNames.GAME_MEMORY);
            const numGamePlays = GameController_18.GameController.Instance.state.getMinigamePlays(StateNames_11.StateNames.GAME_MEMORY) + 1;
            SMAnalyticsManager_16.SMAnalyticsManager.Instance.trackState(`content/game/inqueue/minigame/instructions/${pageNum}_3`, `CommanderInstructions${pageNum}_3`, {
                [SMAnalyticsManager_16.SMAnalyticsManager.GAME_NAME_MINI]: metricsGameName,
                [SMAnalyticsManager_16.SMAnalyticsManager.GAME_PLAYS]: numGamePlays
            });
        }
        destroy() {
            this.shutdownSignal = null;
            this.dialogA.destroy();
            this.dialogB.destroy();
            this.dialogC.destroy();
            super.destroy();
        }
    }
    exports.Intro = Intro;
});
define("minigame/memory/MemoryGameState", ["require", "exports", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "Game", "minigame/memory/MemoryGameLogic", "component/dialog/BasicSpaceButtonDialog", "logic/GameController", "StateNames", "minigame/memory/MemoryInputs", "minigame/memory/SlidingInstructionPanel", "minigame/memory/Intro", "logic/SMAnalyticsManager", "state/SpaceGameBaseState"], function (require, exports, LocalizationManager_29, PromiseUtil_41, TextUtil_14, ScaleUtil_26, Game_34, MemoryGameLogic_1, BasicSpaceButtonDialog_3, GameController_19, StateNames_12, MemoryInputs_1, SlidingInstructionPanel_2, Intro_1, SMAnalyticsManager_17, SpaceGameBaseState_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     * @author nick@cerebralfix.com
     * @author joel@cerebralfix.com
     */
    class MemoryGameState extends SpaceGameBaseState_2.SpaceGameBaseState {
        preload() {
            super.preload();
            // Backgrounds and panels
            this.game.load.image("bg-tile", 'assets/minigame/memory/bg_tile.png');
            this.game.load.image("1x1", 'assets/minigame/memory/1x1.png');
            this.game.load.image("top-panel", 'assets/minigame/memory/top_panel.png');
            this.game.load.image("bottom-panel", 'assets/minigame/memory/bottom_panel.png');
            this.game.load.image("instruction-panel", 'assets/minigame/memory/instruction_panel.png');
            // Buttons and toggles
            this.game.load.image("button-ok", 'assets/minigame/memory/button_ok.png');
            this.game.load.image("button-power", 'assets/minigame/memory/button_power.png');
            this.game.load.image("powerbutton-outline", 'assets/minigame/memory/powerbutton_outline.png');
            this.game.load.image("button-toggle", 'assets/minigame/memory/button_toggle.png');
            this.game.load.image("button-toggle-circle", 'assets/minigame/memory/button_toggle_circle.png');
            this.load.spritesheet("button-thrusters", 'assets/minigame/memory/button_thrusters.png', 199, 199, 2);
            // Anim and misc
            this.game.load.image("radar-gradient", 'assets/minigame/memory/radar_gradient.png');
            this.game.load.image("target", 'assets/minigame/memory/target.png');
            this.game.load.image("green-circle", 'assets/minigame/memory/greencircle.png');
            this.game.load.atlasJSONArray("correct-bars", "assets/minigame/memory/anim_correctbars.png", "assets/minigame/memory/anim_correctbars.json");
            this.game.load.atlasJSONArray("incorrect-bars", "assets/minigame/memory/anim_incorrectbars.png", "assets/minigame/memory/anim_incorrectbars.json");
            this.game.load.atlasJSONArray("powerbars-active", "assets/minigame/memory/anim_powerbars_active.png", "assets/minigame/memory/anim_powerbars_active.json");
            this.game.load.atlasJSONArray("powerbars-idle", "assets/minigame/memory/anim_powerbars_idle.png", "assets/minigame/memory/anim_powerbars_idle.json");
            // Audio
            this.game.load.audio("memory-airlock-on", ["assets/sound/memory/airlock_on.m4a", "assets/sound/memory/airlock_on.ogg"]);
            this.game.load.audio("memory-complete", ["assets/sound/memory/complete.m4a", "assets/sound/memory/complete.ogg"]);
            this.game.load.audio("memory-correct", ["assets/sound/memory/correct.m4a", "assets/sound/memory/correct.ogg"]);
            this.game.load.audio("memory-decrease-power", ["assets/sound/memory/decrease_power.m4a", "assets/sound/memory/decrease_power.ogg"]);
            this.game.load.audio("memory-decrease-throttle", ["assets/sound/memory/decrease_throttle.m4a", "assets/sound/memory/decrease_throttle.ogg"]);
            this.game.load.audio("memory-incorrect", ["assets/sound/memory/incorrect.m4a", "assets/sound/memory/incorrect.ogg"]);
            this.game.load.audio("memory-increase-power", ["assets/sound/memory/increase_power.m4a", "assets/sound/memory/increase_power.ogg"]);
            this.game.load.audio("memory-increase-throttle", ["assets/sound/memory/increase_throttle.m4a", "assets/sound/memory/increase_throttle.ogg"]);
            this.game.load.audio("memory-task-given", ["assets/sound/memory/task_given.m4a", "assets/sound/memory/task_given.ogg"]);
            this.game.load.audio("memory-thruster-off", ["assets/sound/memory/thruster_off.m4a", "assets/sound/memory/thruster_off.ogg"]);
            this.game.load.audio("memory-thruster-on", ["assets/sound/memory/thruster_on.m4a", "assets/sound/memory/thruster_on.ogg"]);
            this.game.load.audio("memory-ventarm-on", ["assets/sound/memory/ventarm_on.m4a", "assets/sound/memory/ventarm_on.ogg"]);
            this.game.load.audio("flash-correct-answer", ["assets/sound/memory/flash_correct_answer.m4a", "assets/sound/memory/flash_correct_answer.ogg"]);
        }
        init() {
            this.stateName = StateNames_12.StateNames.GAME_MEMORY;
            super.init();
            this.inputSignal = new Phaser.Signal();
            this.logic = new MemoryGameLogic_1.MemoryGameLogic();
            // Inits sound keys that correspond to inputs
            this.soundKeys = new Map();
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THRUSTERS_1, 'memory-thruster-on');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THRUSTERS_2, 'memory-thruster-off');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THRUSTERS_3, 'memory-thruster-on');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THRUSTERS_4, 'memory-thruster-off');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.AIRLOCK, 'memory-airlock-on');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.VENT_ARM, 'memory-ventarm-on');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.INTERNAL_POWER_INCREASE, 'memory-increase-power');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.INTERNAL_POWER_DECREASE, 'memory-decrease-power');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THROTTLE_INCREASE, 'memory-increase-throttle');
            this.soundKeys.set(MemoryGameLogic_1.MemoryInputButton.THROTTLE_DECREASE, 'memory-decrease-throttle');
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.padding = this.game.width * 0.025;
                this.gameplayGroup = new Phaser.Group(this.game);
                this.maskGroup = new Phaser.Group(this.game);
                this.slidingDialogGroup = new Phaser.Group(this.game);
                // fullscreen mask to block clicks
                this.mask = new Phaser.Image(this.game, 0, 0, '1x1');
                this.mask.width = this.game.width;
                this.mask.height = this.game.height;
                this.mask.tint = 0x000000;
                this.mask.alpha = 0;
                this.maskGroup.add(this.mask);
                this.mask.inputEnabled = true;
                this.correctScreenSwipe = new Phaser.Image(this.game, 0, 0, "radar-gradient");
                this.correctScreenSwipe.width = this.game.width;
                this.correctScreenSwipe.height = this.game.height * 0.4;
                this.correctScreenSwipe.top = this.game.height;
                this.correctScreenSwipe.left = 0;
                this.maskGroup.add(this.correctScreenSwipe);
                const background = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, "bg-tile");
                background.width = this.game.width;
                background.height = this.game.height;
                this.gameplayGroup.add(background);
                // Target image that displays when selecting button
                this.inputTargetImage = new Phaser.Image(this.game, 0, 0, "target");
                // Saving off this value so that we can reset the size after animations
                this.targetSize = this.game.width * 0.3;
                this.inputTargetImage.width = this.targetSize;
                this.inputTargetImage.height = this.targetSize;
                this.inputTargetImage.visible = false;
                this.inputTargetImage.anchor.set(0.5);
                this.maskGroup.add(this.inputTargetImage);
                // Circular graphic that radiates out when selecting button
                this.inputRadiusGraphics = new Phaser.Graphics(this.game, 0, 0);
                this.inputRadiusGraphics.beginFill(Phaser.Color.hexToRGB(MemoryGameState.GREEN));
                this.inputRadiusGraphics.drawCircle(0, 0, this.inputTargetImage.width / 3);
                this.inputRadiusGraphics.endFill();
                this.inputRadiusGraphics.visible = false;
                this.maskGroup.add(this.inputRadiusGraphics);
                const launchPanelText = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.launch_panel"), {
                    font: Game_34.Game.DEFAULT_FONT,
                    fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                    fontSize: Game_34.Game.FONT_SIZE_MEDIUM
                });
                launchPanelText.setScaleMinMax(1);
                launchPanelText.top = this.padding;
                launchPanelText.centerX = this.game.width * 0.5;
                this.gameplayGroup.add(launchPanelText);
                this.createTopPanel();
                this.createThrustersToggles();
                this.createBottomPanel();
                this.instructionPanel = new SlidingInstructionPanel_2.SlidingInstructionPanel(this.game, this.shutdownSignal);
                this.instructionPanel.bottom = 0;
                this.slidingDialogGroup.add(this.instructionPanel);
                this.add.existing(this.gameplayGroup);
                this.add.existing(this.maskGroup);
                this.add.existing(this.slidingDialogGroup);
                this.dismissOverlay();
                // Initialises round
                this.logic.assignPlayerOperations();
                // Show intro dialog
                yield this.showIntroDialog();
                if (this.isShutdown) {
                    return;
                }
                this.makeTimer();
                SMAnalyticsManager_17.SMAnalyticsManager.Instance.trackGamePlay(this.stateName);
                // Displays each player's instruction
                this.displayRoundInstructions();
            });
        }
        showIntroDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                // show stuff
                this.disableAllGameInput();
                const commanderPlay = this.logic.playersInGame.length > 2;
                const intro = new Intro_1.Intro(this.game, this.logic.controllerPlayer.playerName, commanderPlay, this.shutdownSignal);
                this.maskGroup.add(intro);
                this.game.add.tween(this.mask).to({ alpha: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                yield intro.show();
                if (intro.isShutdown) {
                    return;
                }
                intro.destroy();
                // hide stuff and continue
                this.game.add.tween(this.mask).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                this.enableAllGameInput();
            });
        }
        displayRoundInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                // Pause timer while instructions
                this.timer.pause();
                if (this.logic.round > 1) {
                    yield this.showNextRoundStarting();
                    if (this.isShutdown) {
                        return;
                    }
                }
                for (let i = 0; i < this.logic.playersInGame.length; i++) {
                    const player = this.logic.playersInGame[i];
                    const taskId = player.operationsAssigned[player.operationsAssigned.length - 1];
                    const taskNumber = this.logic.taskNumberMap.get(this.logic.round);
                    const operation = this.logic.operations.get(taskId);
                    const hideOnClick = (i === this.logic.playersInGame.length - 1) ? true : false;
                    yield this.showInstructionDialog(player.playerName, taskNumber, operation, hideOnClick);
                    if (this.isShutdown) {
                        return;
                    }
                }
                // Reenable timer
                this.timer.resume();
                this.displayNextTurnInstruction();
            });
        }
        showFeedback(isCorrect, correctInstructionText, inputButton) {
            return __awaiter(this, void 0, void 0, function* () {
                // Show target animation over selected button
                this.inputTargetImage.centerX = inputButton.worldPosition.x;
                this.inputTargetImage.centerY = inputButton.worldPosition.y;
                this.inputTargetImage.visible = true;
                this.game.add.tween(this.inputTargetImage).to({ angle: 360 }, 1200, Phaser.Easing.Linear.None, true, 0, 2);
                this.game.add.tween(this.inputTargetImage.scale).to({ x: 1.2, y: 1.2 }, 300, Phaser.Easing.Linear.None, true, 0, 4, true)
                    .onComplete.addOnce(() => {
                    this.inputTargetImage.width = this.targetSize;
                    this.inputTargetImage.height = this.targetSize;
                });
                // Play sound associated with input
                this.game.sound.play(this.soundKeys.get(inputButton.inputId));
                if (isCorrect) {
                    this.inputTargetImage.tint = Phaser.Color.hexToRGB(MemoryGameState.GREEN);
                    this.inputRadiusGraphics.visible = true;
                    // Move circular graphic to active button
                    this.inputRadiusGraphics.x = inputButton.worldPosition.x;
                    this.inputRadiusGraphics.y = inputButton.worldPosition.y;
                    this.inputRadiusGraphics.alpha = 1;
                    this.game.sound.play('memory-correct');
                    // Scale up graphic and fade out
                    this.game.add.tween(this.inputRadiusGraphics.scale).to({ x: 30, y: 30 }, 1000, Phaser.Easing.Linear.None, true)
                        .onComplete.add(() => {
                        this.inputRadiusGraphics.scale.x = 1;
                        this.inputRadiusGraphics.scale.y = 1;
                    });
                    this.game.add.tween(this.inputRadiusGraphics).to({ alpha: 0 }, 1300, Phaser.Easing.Quartic.Out, true);
                    // Show screen swipe animation, move sprite back to bottom of the screen when done
                    this.game.add.tween(this.correctScreenSwipe).to({ bottom: 0 }, 1500, Phaser.Easing.Linear.None, true)
                        .onComplete.add(() => { this.correctScreenSwipe.top = this.game.height; });
                }
                else {
                    this.inputTargetImage.tint = Phaser.Color.hexToRGB(MemoryGameState.RED);
                    this.game.sound.play('memory-incorrect');
                    // Flash red background mask
                    this.mask.tint = Phaser.Color.hexToRGB(MemoryGameState.RED);
                    this.game.add.tween(this.mask).to({ alpha: 0.8 }, 200, Phaser.Easing.Linear.None, true, 0, 3, true)
                        .onComplete.add(() => { this.mask.tint = 0x000000; });
                    // Flash timer
                    this.flashCountdownTimer();
                }
                yield this.instructionPanel.showFeedback(isCorrect, correctInstructionText);
                if (this.isShutdown) {
                    return;
                }
                this.inputTargetImage.visible = false;
                this.inputTargetImage.angle = 0;
            });
        }
        flashCountdownTimer() {
            return __awaiter(this, void 0, void 0, function* () {
                // Flash countdown timer, make text red
                for (let i = 0; i < 3; i++) {
                    // change font color to red
                    this.timeLeftCountdown.fill = MemoryGameState.RED;
                    yield PromiseUtil_41.PromiseUtil.time(this.game, 200, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    // change font color back to yellow
                    this.timeLeftCountdown.fill = MemoryGameState.YELLOW;
                    yield PromiseUtil_41.PromiseUtil.time(this.game, 200, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                }
                // Ensures the timer color will always return back to its original color
                this.timeLeftCountdown.fill = MemoryGameState.YELLOW;
            });
        }
        showNextRoundStarting() {
            return __awaiter(this, void 0, void 0, function* () {
                this.instructionPanel.showNextRoundStarting(this.logic.round);
                this.game.add.tween(this.mask).to({ alpha: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                // Prevent user from interacting with game buttons while dialog is shown
                this.disableAllGameInput();
                yield PromiseUtil_41.PromiseUtil.signal(this.instructionPanel.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.instructionPanel.hideAllContent();
                this.enableAllGameInput();
            });
        }
        disableAllGameInput() {
            // Enable input on the full screen mask so that it intercepts any non-dialog input
            this.mask.inputEnabled = true;
        }
        enableAllGameInput() {
            this.mask.inputEnabled = false;
        }
        showInstructionDialog(playerName, taskNumber, task, hideOnClick) {
            return __awaiter(this, void 0, void 0, function* () {
                // Prevent user from interacting with game buttons while dialog is shown
                this.disableAllGameInput();
                this.instructionPanel.showInstructionsDialog(playerName, taskNumber, task);
                this.game.add.tween(this.mask).to({ alpha: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_41.PromiseUtil.signal(this.instructionPanel.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                if (hideOnClick) {
                    this.instructionPanel.hideDialog();
                    this.game.add.tween(this.mask).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                }
                this.enableAllGameInput();
            });
        }
        createTopPanel() {
            const topPanel = new Phaser.Image(this.game, 0, 0, "top-panel");
            ScaleUtil_26.scaleToBounds(topPanel, this.game.width - this.padding * 2, this.game.height * 0.32);
            topPanel.centerX = this.game.width * 0.5;
            topPanel.top = this.game.height * 0.18 + this.padding;
            this.gameplayGroup.add(topPanel);
            const timeDisplayGroup = new Phaser.Group(this.game);
            const timeLeftText = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.time_left"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: MemoryGameState.YELLOW,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            this.timeLeftCountdown = new Phaser.Text(this.game, 0, 0, "00:00", {
                font: Game_34.Game.DEFAULT_FONT,
                fill: MemoryGameState.YELLOW,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            this.timeLeftCountdown.left = timeLeftText.right + this.padding;
            timeDisplayGroup.add(timeLeftText);
            timeDisplayGroup.add(this.timeLeftCountdown);
            timeDisplayGroup.centerX = this.game.width * 0.5;
            timeDisplayGroup.top = topPanel.top + this.game.width * 0.04;
            this.gameplayGroup.add(timeDisplayGroup);
            const airlockToggle = new MemoryInputs_1.SlidingToggle(this.game, LocalizationManager_29.LocalizationManager.Instance.text("memory.airlock_open"), LocalizationManager_29.LocalizationManager.Instance.text("memory.airlock_close"), MemoryGameLogic_1.MemoryInputButton.AIRLOCK);
            airlockToggle.onInputClickedSignal = this.inputSignal;
            airlockToggle.top = topPanel.top + this.game.height * 0.085;
            airlockToggle.right = topPanel.right - this.padding;
            this.gameplayGroup.add(airlockToggle);
            const ventArmToggle = new MemoryInputs_1.SlidingToggle(this.game, LocalizationManager_29.LocalizationManager.Instance.text("memory.vent_arm_retract"), LocalizationManager_29.LocalizationManager.Instance.text("memory.vent_arm_extend"), MemoryGameLogic_1.MemoryInputButton.VENT_ARM);
            ventArmToggle.onInputClickedSignal = this.inputSignal;
            ventArmToggle.top = airlockToggle.bottom + this.padding;
            ventArmToggle.right = topPanel.right - this.padding;
            this.gameplayGroup.add(ventArmToggle);
            const textOffset = (topPanel.width - airlockToggle.width - this.padding) / 2;
            const airlockLabel = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.airlock"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            airlockLabel.setScaleMinMax(1);
            airlockLabel.centerX = this.padding + textOffset;
            airlockLabel.centerY = airlockToggle.centerY;
            this.gameplayGroup.add(airlockLabel);
            const ventArmLabel = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.vent_arm"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            ventArmLabel.setScaleMinMax(1);
            ventArmLabel.centerX = this.padding + textOffset;
            ventArmLabel.centerY = ventArmToggle.centerY;
            this.gameplayGroup.add(ventArmLabel);
        }
        createThrustersToggles() {
            const thrustersTitle = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.thrusters"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_34.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_34.Game.FONT_STROKE_SMALL,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            thrustersTitle.setScaleMinMax(1);
            thrustersTitle.centerX = this.game.width * 0.5;
            thrustersTitle.centerY = this.game.height * 0.5;
            this.gameplayGroup.add(thrustersTitle);
            const thrustersGroup = new Phaser.Group(this.game);
            let leftPos = 0;
            // this makes sure the buttons don't rotate in unison
            let spinDelay = 0;
            const thrustersValues = [MemoryGameLogic_1.MemoryInputButton.THRUSTERS_1,
                MemoryGameLogic_1.MemoryInputButton.THRUSTERS_2,
                MemoryGameLogic_1.MemoryInputButton.THRUSTERS_3,
                MemoryGameLogic_1.MemoryInputButton.THRUSTERS_4];
            for (let i in thrustersValues) {
                const value = thrustersValues[i];
                // This relies on the id of each toggle also being the text value we want to show on the button
                const thrustersToggle = new MemoryInputs_1.RoundToggle(this.game, value.toString(), value, spinDelay);
                thrustersToggle.onInputClickedSignal = this.inputSignal;
                thrustersToggle.left = leftPos;
                leftPos = thrustersToggle.right + this.padding;
                spinDelay += 450;
                thrustersGroup.add(thrustersToggle);
            }
            thrustersGroup.centerX = this.game.width * 0.5;
            thrustersGroup.top = thrustersTitle.bottom + this.padding / 2;
            this.gameplayGroup.add(thrustersGroup);
        }
        createBottomPanel() {
            const bottomPanel = new Phaser.Image(this.game, 0, 0, "bottom-panel");
            bottomPanel.width = this.game.width - this.padding * 2;
            bottomPanel.height = this.game.height * 0.37;
            bottomPanel.centerX = this.game.width * 0.5;
            bottomPanel.bottom = this.game.height - this.padding;
            this.gameplayGroup.add(bottomPanel);
            const intPowerLabel = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.internal_power"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            intPowerLabel.setScaleMinMax(1);
            intPowerLabel.centerX = this.game.width * 0.5;
            intPowerLabel.top = bottomPanel.top + this.game.height * 0.06;
            this.gameplayGroup.add(intPowerLabel);
            const intPowerButton = new MemoryInputs_1.PowerButtonPanel(this.game, LocalizationManager_29.LocalizationManager.Instance.text("memory.toggle_decrease"), LocalizationManager_29.LocalizationManager.Instance.text("memory.toggle_increase"), MemoryGameLogic_1.MemoryInputButton.INTERNAL_POWER_DECREASE, MemoryGameLogic_1.MemoryInputButton.INTERNAL_POWER_INCREASE);
            intPowerButton.onInputClickedSignal = this.inputSignal;
            intPowerButton.centerX = this.game.width * 0.5;
            intPowerButton.top = intPowerLabel.bottom + this.padding / 2;
            this.gameplayGroup.add(intPowerButton);
            const throttleLabel = new Phaser.Text(this.game, 0, 0, LocalizationManager_29.LocalizationManager.Instance.text("memory.throttle"), {
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_34.Game.FONT_SIZE_MEDIUM
            });
            throttleLabel.setScaleMinMax(1);
            throttleLabel.centerX = this.game.width * 0.5;
            throttleLabel.top = intPowerButton.bottom + this.padding / 2;
            this.gameplayGroup.add(throttleLabel);
            const throttleButton = new MemoryInputs_1.PowerButtonPanel(this.game, LocalizationManager_29.LocalizationManager.Instance.text("memory.toggle_decrease"), LocalizationManager_29.LocalizationManager.Instance.text("memory.toggle_increase"), MemoryGameLogic_1.MemoryInputButton.THROTTLE_DECREASE, MemoryGameLogic_1.MemoryInputButton.THROTTLE_INCREASE);
            throttleButton.onInputClickedSignal = this.inputSignal;
            throttleButton.centerX = this.game.width * 0.5;
            throttleButton.top = throttleLabel.bottom + this.padding / 2;
            this.gameplayGroup.add(throttleButton);
        }
        makeTimer() {
            this.timer = this.game.time.create(false);
            this.timer.loop(1000, () => {
                if (!this.logic.gameOver) { // to catch bugs that might occur with awaits
                    this.logic.timeRemainingS--;
                    if (this.logic.timeRemainingS < 0) {
                        this.onGameOver();
                    }
                    else {
                        this.updateTimerDisplay();
                    }
                }
            });
            this.timer.start();
        }
        onGameOver() {
            this.disableAllGameInput();
            this.endGame();
            this.game.sound.play('memory-complete');
            this.updateTimerDisplay();
            this.showGameOverDialog();
        }
        onEndGame() {
            this.logic.timeRemainingS = 0;
            this.logic.gameOver = true;
            this.timer.stop(true);
        }
        postScore(isInterrupted) {
            this.logic.postScores(true);
            GameController_19.GameController.Instance.achievementManager.onMemoryEndGame(this.logic.round);
        }
        updateTimerDisplay() {
            this.timeLeftCountdown.text = TextUtil_14.TextUtil.formattedTime(this.logic.timeRemainingS);
        }
        displayNextTurnInstruction() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.logic.gameOver) { // catch bugs that might occur with the await delays
                    return;
                }
                const currentInstruction = this.logic.instructionsInPlay[this.logic.playerActions.length];
                const nthRound = this.logic.taskNumberMap.get(currentInstruction.roundNumber + 1);
                const instructions = LocalizationManager_29.LocalizationManager.Instance.textWithReplacements("memory.perform_task", { name: currentInstruction.playerName, nth: nthRound });
                this.instructionPanel.showInstructions(instructions, this.logic.controllerPlayer.playerName);
                this.game.sound.play('memory-task-given');
                this.enableAllGameInput();
                // Wait for button input
                const inputButton = yield PromiseUtil_41.PromiseUtil.signal(this.inputSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.gameplayGroup.ignoreChildInput = true;
                this.logic.addPlayerAction(inputButton.inputId);
                // Disable buttons while we're displaying feedback
                this.disableAllGameInput();
                // Check for win
                const success = this.logic.checkAction();
                const currentInstructionString = LocalizationManager_29.LocalizationManager.Instance.text("memory.operations." + currentInstruction.answer);
                yield this.showFeedback(success, currentInstructionString, inputButton);
                if (this.isShutdown) {
                    return;
                }
                if (this.logic.instructionsInPlay.length === this.logic.playerActions.length) {
                    // end of round, show next allocations
                    this.logic.assignPlayerOperations();
                    // Displays each player's instruction
                    yield this.displayRoundInstructions();
                }
                else {
                    this.displayNextTurnInstruction();
                }
            });
        }
        showGameOverDialog() {
            this.disableAllGameInput();
            this.game.add.tween(this.slidingDialogGroup).to({ bottom: 0 }, 200, Phaser.Easing.Linear.None, true);
            this.game.add.tween(this.mask).to({ alpha: 0.8 }, 200, Phaser.Easing.Linear.None, true);
            const dialog = new BasicSpaceButtonDialog_3.BasicSpaceButtonDialog(this.game, LocalizationManager_29.LocalizationManager.Instance.textWithReplacements("memory.great_job", { score: this.logic.score }), LocalizationManager_29.LocalizationManager.Instance.text("buttons.ok"), this.shutdownSignal);
            dialog.onButtonClickedSignal.addOnce(() => {
                dialog.hide();
                this.transitionToOutro();
            });
            this.add.existing(dialog);
            dialog.show();
        }
        transitionToOutro() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToOutro()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                GameController_19.GameController.Instance.finishGame();
                this.game.state.start(StateNames_12.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        shutdown() {
            if (this.inputSignal) {
                this.inputSignal.dispose();
                this.inputSignal = null;
            }
            if (this.timer) {
                this.timer.stop(true);
                this.timer = null;
            }
            this.gameplayGroup = null;
            this.maskGroup = null;
            this.slidingDialogGroup = null;
            this.instructionPanel = null;
            this.mask = null;
            this.correctScreenSwipe = null;
            this.inputTargetImage = null;
            this.inputRadiusGraphics = null;
            this.timeLeftCountdown = null;
            this.logic = null;
            // Backgrounds and panels
            this.game.cache.removeImage("bg-tile");
            this.game.cache.removeImage("1x1");
            this.game.cache.removeImage("top-panel");
            this.game.cache.removeImage("bottom-panel");
            this.game.cache.removeImage("instruction-panel");
            // Buttons and toggles
            this.game.cache.removeImage("button-ok");
            this.game.cache.removeImage("button-power");
            this.game.cache.removeImage("powerbutton-outline");
            this.game.cache.removeImage("button-toggle");
            this.game.cache.removeImage("button-toggle-circle");
            this.load.cache.removeImage("button-thrusters");
            // Anim and misc
            this.game.cache.removeImage("radar-gradient");
            this.game.cache.removeImage("target");
            this.game.cache.removeImage("green-circle");
            this.game.cache.removeImage("correct-bars");
            this.game.cache.removeImage("incorrect-bars");
            this.game.cache.removeImage("powerbars-active");
            this.game.cache.removeImage("powerbars-idle");
            // Audio
            this.game.cache.removeSound("memory-airlock-on");
            this.game.cache.removeSound("memory-complete");
            this.game.cache.removeSound("memory-correct");
            this.game.cache.removeSound("memory-decrease-power");
            this.game.cache.removeSound("memory-decrease-throttle");
            this.game.cache.removeSound("memory-incorrect");
            this.game.cache.removeSound("memory-increase-power");
            this.game.cache.removeSound("memory-increase-throttle");
            this.game.cache.removeSound("memory-task-given");
            this.game.cache.removeSound("memory-thruster-off");
            this.game.cache.removeSound("memory-thruster-on");
            this.game.cache.removeSound("memory-ventarm-on");
            this.game.cache.removeSound("flash-correct-answer");
            super.shutdown();
        }
    }
    MemoryGameState.GREEN = "#59ea81";
    MemoryGameState.RED = "#ed2323";
    MemoryGameState.YELLOW = "#ffed2e";
    exports.MemoryGameState = MemoryGameState;
});
define("minigame/relay/RelayGameLogic", ["require", "exports", "logic/GameController", "StateNames"], function (require, exports, GameController_20, StateNames_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     * This class holds the logic for the whole RelayGame
     * That includes the intro, outro and the subgames. It's shared across all of them. And needs to be like that.
     */
    class RelayGameLogic {
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init() {
            this.numTasksComplete = 0;
            this.numTasksFailed = 0;
            this.numGamesPlayed = 0;
            this.possibleGames = [
                StateNames_13.StateNames.GAME_RELAY_LOCKS,
                StateNames_13.StateNames.GAME_RELAY_SWITCHES,
                StateNames_13.StateNames.GAME_RELAY_SCANNER,
                StateNames_13.StateNames.GAME_RELAY_WRENCH
            ];
            this.players = [];
            // Populate game names
            const gameController = GameController_20.GameController.Instance;
            for (let i in gameController.players) {
                this.players.push(Number(i));
            }
            // Shuffle players
            Phaser.ArrayUtils.shuffle(this.players);
            // Different group sizes get different times
            switch (gameController.players.length) {
                default:
                case 2: {
                    this.globalTimeRemaining = 60000;
                    break;
                }
                case 3:
                case 4: {
                    this.globalTimeRemaining = 75000;
                    break;
                }
                case 5: {
                    this.globalTimeRemaining = 90000;
                    break;
                }
            }
        }
        // Returns the index of the current player relative to GameController.players
        getCurrentPlayerIndex() {
            return this.numGamesPlayed % this.players.length;
        }
        getPlayerName(index) {
            let playerIndex = this.players[index % this.players.length];
            return GameController_20.GameController.Instance.players[playerIndex].name;
        }
        // Returns the current player name
        get currentPlayer() {
            return this.getPlayerName(this.getCurrentPlayerIndex());
        }
        // If there is a next player, this returns a string formatted name of them.
        // If there isn't a next player, this function returns an empty string.
        get nextPlayer() {
            return this.isLastGame ? "" : this.getPlayerName(this.getCurrentPlayerIndex() + 1);
        }
        get isLastGame() {
            return this.globalTimeRemaining <= 0;
        }
        gameFailed() {
            this.numTasksFailed++;
            this.numGamesPlayed++;
        }
        gameCompleted() {
            this.numTasksComplete++;
            this.numGamesPlayed++;
        }
        /**
         * If the players still have time left, this function will find a game
         * that the next player hasn't played.
         *
         * If there's no time left, this will instruct the caller to go to
         * the outro.
         */
        getNextState() {
            if (this.globalTimeRemaining <= 0) {
                return StateNames_13.StateNames.GAME_RELAY_OUTRO;
            }
            let player = GameController_20.GameController.Instance.players[this.getCurrentPlayerIndex()];
            if (!player.relayPossibleGames || player.relayPossibleGames.length === 0) {
                player.relayPossibleGames = this.possibleGames.slice();
                Phaser.ArrayUtils.shuffle(player.relayPossibleGames);
            }
            return player.relayPossibleGames.shift();
        }
    }
    RelayGameLogic.TIME_ON_DIALOGS_MS = 5000;
    exports.RelayGameLogic = RelayGameLogic;
});
define("minigame/relay/RelayGameIntroState", ["require", "exports", "state/SpaceBaseState", "switchback-common/localization/LocalizationManager", "minigame/relay/RelayGameLogic", "component/dialog/TwoTextDialog", "StateNames", "logic/SMAnalyticsManager", "logic/GameController"], function (require, exports, SpaceBaseState_5, LocalizationManager_30, RelayGameLogic_1, TwoTextDialog_3, StateNames_14, SMAnalyticsManager_18, GameController_21) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class RelayGameIntroState extends SpaceBaseState_5.SpaceBaseState {
        init() {
            this.stateName = StateNames_14.StateNames.GAME_RELAY_INTRO;
            super.init();
        }
        preload() {
            super.preload();
            this.game.load.audio("bar_charge", ["assets/sound/relay/bar_charge.ogg", "assets/sound/relay/bar_charge.m4a"]);
            this.game.load.audio("complete_microgame", ["assets/sound/relay/complete_microgame.ogg", "assets/sound/relay/complete_microgame.m4a"]);
            this.game.load.audio("lockgame_correct", ["assets/sound/relay/lockgame_correct.ogg", "assets/sound/relay/lockgame_correct.m4a"]);
            this.game.load.audio("lockgame_incorrect", ["assets/sound/relay/lockgame_incorrect.ogg", "assets/sound/relay/lockgame_incorrect.m4a"]);
            this.game.load.audio("new_target", ["assets/sound/relay/new_target.ogg", "assets/sound/relay/new_target.m4a"]);
            this.game.load.audio("target_1", ["assets/sound/relay/target_1.ogg", "assets/sound/relay/target_1.m4a"]);
            this.game.load.audio("target_2", ["assets/sound/relay/target_2.ogg", "assets/sound/relay/target_2.m4a"]);
            this.game.load.audio("wrenchgame_tap_corrrect", ["assets/sound/relay/wrenchgame_tap_corrrect.ogg", "assets/sound/relay/wrenchgame_tap_corrrect.m4a"]);
            this.game.load.audio("wrenchgame_tap_wrong", ["assets/sound/relay/wrenchgame_tap_wrong.ogg", "assets/sound/relay/wrenchgame_tap_wrong.m4a"]);
            for (let i = 1; i <= 4; i++) {
                this.game.load.audio(`click_switch${i}`, [`assets/sound/relay/click_switch${i}.ogg`, `assets/sound/relay/click_switch${i}.m4a`]);
            }
        }
        create() {
            super.create();
            // Initialise a new round of relays
            RelayGameLogic_1.RelayGameLogic.Instance.init();
            const background = new Phaser.Image(this.game, 0, 0, "background");
            // TODO: Scale nicely
            background.width = this.game.width;
            background.height = this.game.height;
            let instructionDialog = new TwoTextDialog_3.TwoTextDialog(this.game, LocalizationManager_30.LocalizationManager.Instance.text("relay.instructions_main"), LocalizationManager_30.LocalizationManager.Instance.text("relay.instructions_sub"), "buttons.start", this.shutdownSignal);
            instructionDialog.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play("cta-button");
                yield instructionDialog.hide();
                instructionDialog.destroy();
                if (this.isShutdown) {
                    return;
                }
                this.transitionToGame();
            }), this);
            this.add.existing(background);
            this.add.existing(instructionDialog);
            this.dismissOverlay();
            SMAnalyticsManager_18.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/instructions', 'MaintenanceInstructions', {
                [SMAnalyticsManager_18.SMAnalyticsManager.GAME_NAME_MINI]: SMAnalyticsManager_18.SMAnalyticsManager.gameNameMap.get(this.stateName),
                [SMAnalyticsManager_18.SMAnalyticsManager.GAME_PLAYS]: GameController_21.GameController.Instance.state.getMinigamePlays(this.stateName) + 1
            });
            instructionDialog.show();
        }
        transitionToGame() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToGame()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(RelayGameLogic_1.RelayGameLogic.Instance.getNextState());
            });
        }
    }
    exports.RelayGameIntroState = RelayGameIntroState;
});
define("component/dialog/TimedSpaceDialog", ["require", "exports", "switchback-common/util/TextUtil", "component/dialog/BasicSpaceButtonDialog", "Game", "component/coderedeem/CodeButton"], function (require, exports, TextUtil_15, BasicSpaceButtonDialog_4, Game_35, CodeButton_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TimedSpaceDialog extends BasicSpaceButtonDialog_4.BasicSpaceButtonDialog {
        constructor(game, descriptionText, buttonText, timeMS, shutdownSignal, skippable = true) {
            super(game, descriptionText, buttonText, shutdownSignal);
            this.isClosing = false;
            this.isCodeScreenVisible = false;
            this.timeRemaining = timeMS;
            this.countdownText = new Phaser.Text(game, this.left + this.width * 0.3, this.dialogText.bottom, "0:00", {
                font: Game_35.Game.DEFAULT_FONT,
                fill: Game_35.Game.FONT_COLOUR_BLUE,
                fontSize: Game_35.Game.FONT_SIZE_MEDIUM,
                stroke: Game_35.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_35.Game.DEFAULT_STROKE_THICKNESS
            });
            this.countdownText.setScaleMinMax(1);
            this.countdownText.anchor.set(0.5, 0);
            this.countdownText.centerX = this.button.centerX;
            this.button.top = this.countdownText.bottom;
            this.content.add(this.countdownText);
            this.countdownRefreshRef = setInterval(this.refreshCountdownText.bind(this), 1000);
            this.refreshCountdownText();
            if (!skippable) {
                this.button.destroy();
            }
            this.resizeBackground();
            this.codeButton = new CodeButton_3.CodeButton(this.game, this.shutdownSignal);
            this.slidingGroup.add(this.codeButton);
            this.codeButton.left = this.backplate.left + this.backplate.width * 0.028;
            this.codeButton.top = this.backplate.top + this.backplate.width * 0.032;
            this.codeButton.toggleCodeScreen.add(this.onCodeScreenToggled, this);
        }
        onCodeScreenToggled(isCodeScreenVisible) {
            this.isCodeScreenVisible = isCodeScreenVisible;
        }
        refreshCountdownText() {
            // Calculate the time difference in minutes and seconds, ensuring they don't go below 0
            let secsRemaining = Math.max(0, Math.round(this.timeRemaining / 1000));
            let minsRemaining = Math.max(0, Math.floor(secsRemaining / 60));
            secsRemaining -= (minsRemaining * 60);
            // Pad the text with 0's
            this.countdownText.setText(`${minsRemaining}:${TextUtil_15.TextUtil.leftPad(secsRemaining.toString(), "0", 2)}`);
        }
        update() {
            super.update();
            if (!this.isCodeScreenVisible) {
                this.timeRemaining -= this.game.time.elapsedMS;
                if (this.timeRemaining <= 0 && !this.isClosing) {
                    this.codeButton.toggleCodeScreen.remove(this.onCodeScreenToggled, this);
                    this.dismissDialog();
                }
            }
        }
        dismissDialog() {
            this.isClosing = true;
            this.onButtonClickedSignal.dispatch();
        }
        destroy() {
            clearInterval(this.countdownRefreshRef);
            super.destroy();
        }
    }
    exports.TimedSpaceDialog = TimedSpaceDialog;
});
define("minigame/relay/RelayGameState", ["require", "exports", "StateNames", "logic/GameController", "Game", "minigame/relay/RelayGameLogic", "component/dialog/TimedSpaceDialog", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "logic/SMAnalyticsManager", "state/SpaceGameBaseState", "component/coderedeem/RedeemCodeDialog"], function (require, exports, StateNames_15, GameController_22, Game_36, RelayGameLogic_2, TimedSpaceDialog_1, LocalizationManager_31, TextUtil_16, ScaleUtil_27, SMAnalyticsManager_19, SpaceGameBaseState_3, RedeemCodeDialog_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class RelayGameState extends SpaceGameBaseState_3.SpaceGameBaseState {
        constructor() {
            super(...arguments);
            this.isGamePlaying = false;
        }
        init() {
            super.init();
            const numPlays = GameController_22.GameController.Instance.state.getMinigamePlays(StateNames_15.StateNames.GAME_ASTEROIDS);
            this.difficultyLevel = Math.min(numPlays, 2); // difficulty is based on number of plays and is caped at 3
            this.timeRemaining = RelayGameState.TIME_GIVEN_MS[this.difficultyLevel];
            this.isGamePlaying = true;
        }
        preload() {
            super.preload();
            RedeemCodeDialog_5.RedeemCodeDialog.preload(this.game);
            this.game.load.image('frame-header', 'assets/common/frame-header.png');
        }
        create() {
            super.create();
            this.metricMiniGameName = SMAnalyticsManager_19.SMAnalyticsManager.gameNameMap.get(StateNames_15.StateNames.GAME_RELAY_INTRO);
            this.metricMicroGameName = SMAnalyticsManager_19.SMAnalyticsManager.gameNameMap.get(this.stateName);
            const frameHeader = new Phaser.Image(this.game, this.game.width * 0.5, 0, 'frame-header');
            frameHeader.anchor.set(0.5, 0);
            frameHeader.width = this.game.width * 0.8;
            frameHeader.scale.y = frameHeader.scale.x;
            this.add.existing(frameHeader);
            this.countdownText = new Phaser.Text(this.game, frameHeader.x, frameHeader.top + frameHeader.height * 0.4, '0:15', {
                font: Game_36.Game.DEFAULT_FONT,
                fontSize: Game_36.Game.FONT_SIZE_LARGE,
                fill: Game_36.Game.FONT_COLOUR_GREEN,
                stroke: Game_36.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_36.Game.FONT_STROKE_SMALL,
                align: 'center'
            });
            this.countdownText.setScaleMinMax(1);
            this.countdownText.anchor.set(0.5);
            ScaleUtil_27.scaleToBounds(this.countdownText, this.game.width * 0.1, this.game.height * 0.03);
            this.add.existing(this.countdownText);
            this.countdownRefreshRef = setInterval(this.refreshCountdownText.bind(this), 1000);
            this.refreshCountdownText();
            let introDialog = new TimedSpaceDialog_1.TimedSpaceDialog(this.game, this.introText, LocalizationManager_31.LocalizationManager.Instance.text("relay.button_ready"), RelayGameLogic_2.RelayGameLogic.TIME_ON_DIALOGS_MS, this.shutdownSignal);
            introDialog.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                yield introDialog.hide();
                introDialog.destroy();
                if (this.isShutdown) {
                    return;
                }
                this.beginGame();
                this.isPassAndPlay = false;
            }));
            this.add.existing(introDialog);
            introDialog.show();
            this.isPassAndPlay = true;
            SMAnalyticsManager_19.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/microgame/passandplay', `Maintenance${this.metricMicroGameName}PassAndPlay`, {
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MINI]: this.metricMiniGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MICRO]: this.metricMicroGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_PLAYS]: GameController_22.GameController.Instance.state.getMinigamePlays(this.stateName) + 1
            });
            this.dismissOverlay();
        }
        beginGame() {
            console.log(`[${this.stateName}] beginGame()`);
            SMAnalyticsManager_19.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/microgame', `Maintenance${this.metricMicroGameName}Game`, {
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MINI]: this.metricMiniGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MICRO]: this.metricMicroGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_PLAYS]: GameController_22.GameController.Instance.state.getMinigamePlays(this.stateName) + 1
            });
            SMAnalyticsManager_19.SMAnalyticsManager.Instance.startTiming(`RocketRace:Micro:${this.metricMiniGameName}${this.metricMicroGameName}`, {
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MINI]: SMAnalyticsManager_19.SMAnalyticsManager.gameNameMap.get(StateNames_15.StateNames.GAME_RELAY_INTRO),
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MICRO]: this.metricMicroGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_PLAYS]: GameController_22.GameController.Instance.state.getMinigamePlays(this.stateName) + 1
            });
        }
        refreshCountdownText() {
            // Calculate the time difference in minutes and seconds, ensuring they don't go below 0
            let secondsUntilLaunch = Math.max(0, Math.round(this.timeRemaining / 1000));
            let minsUntilLaunch = Math.max(0, Math.floor(secondsUntilLaunch / 60));
            secondsUntilLaunch -= (minsUntilLaunch * 60);
            // Pad the text with 0's
            this.countdownText.setText(`${TextUtil_16.TextUtil.leftPad(minsUntilLaunch.toString(), "0", 2)}:${TextUtil_16.TextUtil.leftPad(secondsUntilLaunch.toString(), "0", 2)}`);
            if (this.timeRemaining > RelayGameState.TIME_GIVEN_MS[this.difficultyLevel] * 0.66) {
                this.countdownText.fill = Game_36.Game.FONT_COLOUR_GREEN;
            }
            else if (this.timeRemaining > RelayGameState.TIME_GIVEN_MS[this.difficultyLevel] * 0.33) {
                this.countdownText.fill = Game_36.Game.FONT_COLOUR_YELLOW;
            }
            else {
                this.countdownText.fill = Game_36.Game.FONT_COLOUR_RED;
            }
        }
        update() {
            super.update();
            if (!this.isGamePlaying || this.isPassAndPlay) {
                return;
            }
            this.timeRemaining -= this.game.time.elapsedMS;
            RelayGameLogic_2.RelayGameLogic.Instance.globalTimeRemaining -= this.game.time.elapsedMS;
            // Out of time. Stop the game.
            if (this.timeRemaining <= 0) {
                this.onGameOver(false);
            }
        }
        onGameOver(won) {
            console.log(`[${this.stateName}] onGameComplete(${won})`);
            this.isGamePlaying = false;
            if (won) {
                this.game.sound.play("complete_microgame");
            }
            // Prefix for the dialog text
            const prefixText = won
                ? LocalizationManager_31.LocalizationManager.Instance.text("relay.result_success")
                : LocalizationManager_31.LocalizationManager.Instance.text("relay.result_failure");
            // Figure out the text to use
            let outroDialogText;
            let outroDialogButtonText;
            if (RelayGameLogic_2.RelayGameLogic.Instance.isLastGame) {
                outroDialogText = `${prefixText} ${RelayGameLogic_2.RelayGameLogic.Instance.currentPlayer}!`;
                outroDialogButtonText = LocalizationManager_31.LocalizationManager.Instance.text("relay.button_ok");
            }
            else {
                const outroText = LocalizationManager_31.LocalizationManager.Instance.textWithReplacements("relay.pass_the_device", {
                    current_player_name: RelayGameLogic_2.RelayGameLogic.Instance.currentPlayer,
                    next_player_name: RelayGameLogic_2.RelayGameLogic.Instance.nextPlayer
                });
                outroDialogText = `${prefixText} ${outroText}`;
                outroDialogButtonText = LocalizationManager_31.LocalizationManager.Instance.text("relay.button_done");
            }
            // Set up the outro dialog
            let outroDialog = new TimedSpaceDialog_1.TimedSpaceDialog(this.game, outroDialogText, outroDialogButtonText, RelayGameLogic_2.RelayGameLogic.TIME_ON_DIALOGS_MS, this.shutdownSignal);
            outroDialog.onButtonClickedSignal.addOnce(() => {
                this.transitionToNextGame();
            });
            this.add.existing(outroDialog);
            outroDialog.show();
            this.isPassAndPlay = true;
            won ? RelayGameLogic_2.RelayGameLogic.Instance.gameCompleted() : RelayGameLogic_2.RelayGameLogic.Instance.gameFailed();
            this.endGame(false);
        }
        postScore(isInterrupted) {
            const numGamePlays = GameController_22.GameController.Instance.state.incrementMinigamePlays(this.stateName);
            // Their 'score' (for metric's sake) is their proportional time remaining
            const score = (this.timeRemaining / RelayGameState.TIME_GIVEN_MS[this.difficultyLevel]);
            SMAnalyticsManager_19.SMAnalyticsManager.Instance.endTiming(`RocketRace:Micro:${this.metricMiniGameName}${this.metricMicroGameName}`, `${this.metricMiniGameName}${this.metricMicroGameName}`, 'Game', '', {
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MINI]: SMAnalyticsManager_19.SMAnalyticsManager.gameNameMap.get(StateNames_15.StateNames.GAME_RELAY_INTRO),
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_NAME_MICRO]: this.metricMicroGameName,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_PLAYS]: numGamePlays,
                [SMAnalyticsManager_19.SMAnalyticsManager.GAME_SCORE]: score
            });
            let relayLogic = RelayGameLogic_2.RelayGameLogic.Instance;
            if (isInterrupted || relayLogic.isLastGame) {
                // Mark a minigame as completed
                GameController_22.GameController.Instance.postMinigameScore(StateNames_15.StateNames.GAME_RELAY_INTRO, relayLogic.numTasksComplete);
                GameController_22.GameController.Instance.achievementManager.onRelayEndGame(relayLogic.numTasksComplete / relayLogic.players.length);
            }
        }
        transitionToNextGame() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToNextGame()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(RelayGameLogic_2.RelayGameLogic.Instance.getNextState());
            });
        }
        shutdown() {
            // Stop refreshing the countdown
            clearInterval(this.countdownRefreshRef);
            this.countdownText = null;
            super.shutdown();
        }
    }
    RelayGameState.TIME_GIVEN_MS = [20000, 16000, 12000];
    exports.RelayGameState = RelayGameState;
});
define("minigame/relay/locks/Lock", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/CameraUtil"], function (require, exports, PromiseUtil_42, CameraUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var LockDifficulty;
    (function (LockDifficulty) {
        LockDifficulty[LockDifficulty["EASY"] = 0] = "EASY";
        LockDifficulty[LockDifficulty["MEDIUM"] = 1] = "MEDIUM";
        LockDifficulty[LockDifficulty["HARD"] = 2] = "HARD";
    })(LockDifficulty = exports.LockDifficulty || (exports.LockDifficulty = {}));
    class LockArm extends Phaser.Group {
        constructor(game) {
            super(game);
            this.arm = new Phaser.Image(this.game, 0, 0, 'lock-arm');
            this.add(this.arm);
            this.redLight = new Phaser.Image(this.game, this.arm.width * 0.24, this.arm.height * 0.52, 'lock-light');
            this.redLight.anchor.set(0.5);
            this.add(this.redLight);
        }
        // locked true = green
        // locked false = red;
        set locked(locked) {
            this.redLight.visible = !locked;
        }
    }
    class Lock extends Phaser.Group {
        constructor(game, difficulty, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            // Background
            this.background = new Phaser.Image(game, 0, 0, "lock-bar-background");
            this.background.width = this.game.width;
            this.add(this.background);
            // Target Area
            this.target = new Phaser.Image(game, 0, 0, "lock-bar-target");
            this.target.height = this.background.height;
            // The x padding to leave between the bar and the sides of the background
            const xPadding = this.background.width * 0.075;
            switch (difficulty) {
                case LockDifficulty.EASY: {
                    this.target.width = game.rnd.integerInRange(this.background.width * 0.4, this.background.width * 0.5);
                    break;
                }
                case LockDifficulty.MEDIUM: {
                    this.target.width = game.rnd.integerInRange(this.background.width * 0.3, this.background.width * 0.4);
                    break;
                }
                case LockDifficulty.HARD: {
                    this.target.width = game.rnd.integerInRange(this.background.width * 0.2, this.background.width * 0.3);
                    break;
                }
            }
            this.target.x = game.rnd.integerInRange(xPadding, this.background.width - xPadding - this.target.width);
            this.targetOrigXPos = this.target.x;
            this.targetOrigWidth = this.target.width;
            this.add(this.target);
            // Locking Arm
            this.lockArm = new LockArm(this.game);
            this.lockArm.width = this.background.width * 0.65;
            this.lockArm.scale.y = this.lockArm.scale.x;
            this.lockArm.left = this.background.right;
            this.lockArm.centerY = this.background.top + this.background.height * 0.475;
            this.add(this.lockArm);
            // Bar Frame
            const frame = new Phaser.Image(game, 0, 0, "lock-bar-frame");
            frame.width = this.game.width;
            frame.scale.y = frame.scale.x;
            frame.centerX = this.background.centerX;
            frame.centerY = this.background.centerY;
            this.add(frame);
            // Moving bar
            this.bar = new Phaser.Image(game, 0, this.background.centerY, "lock-bar");
            this.bar.height = this.background.height;
            this.bar.scale.x = this.bar.scale.y;
            this.bar.visible = false;
            this.bar.anchor.set(0.5);
            this.barOrigScale = this.bar.scale.x;
            this.add(this.bar);
            this.bar.x = xPadding;
            this.barMovementTween = this.game.add.tween(this.bar)
                .to({ x: this.background.width - xPadding }, Lock.BAR_SPEED, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            // Randomise the progress through the tween (so all bars don't start at the same position)
            this.barMovementTween.updateTweenData('dt', this.game.rnd.frac() * Lock.BAR_SPEED);
        }
        unlock() {
            return __awaiter(this, void 0, void 0, function* () {
                // Fade the bar in
                this.bar.visible = true;
                this.bar.alpha = 0;
                this.lockArm.locked = false;
                // Ensure the target green area is in the right place/size and the locking arm is out of the way
                // Scale the target bar in from a large size
                this.bar.scale.set(this.barOrigScale * 5);
                yield Promise.all([
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.target)
                        .to({ width: this.targetOrigWidth, x: this.targetOrigXPos }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.lockArm)
                        .to({ left: this.game.width }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.bar)
                        .to({ alpha: 1 }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.bar.scale)
                        .to({ x: this.barOrigScale, y: this.barOrigScale }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // ...and start the target bar moving again
                if (this.barMovementTween && this.barMovementTween.isPaused) {
                    this.barMovementTween.resume();
                }
            });
        }
        // This lock has been completed!
        lock() {
            return __awaiter(this, void 0, void 0, function* () {
                this.barMovementTween.pause();
                // Increase the size of the green area and bring the locking arm in
                // Fade out and scale up the white bar
                this.bar.scale.set(this.barOrigScale);
                this.game.sound.play("lockgame_correct");
                yield Promise.all([
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.bar)
                        .to({ alpha: 0 }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.bar.scale)
                        .to({ x: this.barOrigScale * 5, y: this.barOrigScale * 5 }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.target)
                        .to({ width: this.background.width, x: this.background.x }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.lockArm)
                        .to({ right: this.game.width }, Lock.ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true), this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Shake the camera to seal the lock in place!
                CameraUtil_2.cameraShake(this.game, 0.005, Lock.ANIMATION_DURATION_MS);
                // And change the color of the lock arm's light
                this.lockArm.locked = true;
            });
        }
        // Used to reset the lock back to an un-interactable state.
        clear() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bar.visible = false;
            });
        }
        // Tries to lock the bar into place.
        // Returns whether the locking was a success
        tryLock() {
            // Sprite.getBounds() returns a PIXI.Rectangle which is not usable as a Phaser.Rectangle. So we have to construct our own here
            const targetRect = new Phaser.Rectangle(this.target.left, this.target.top, this.target.width, this.target.height);
            const barRect = new Phaser.Rectangle(this.bar.left, this.bar.top, this.bar.width, this.bar.height);
            // Check to see if the bar is within the target zone
            return Phaser.Rectangle.intersects(targetRect, barRect);
        }
    }
    Lock.BAR_SPEED = 1000;
    Lock.ANIMATION_DURATION_MS = 350;
    exports.Lock = Lock;
});
define("minigame/relay/locks/LocksGameState", ["require", "exports", "minigame/relay/RelayGameLogic", "minigame/relay/RelayGameState", "minigame/relay/locks/Lock", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "StateNames"], function (require, exports, RelayGameLogic_3, RelayGameState_1, Lock_1, LocalizationManager_32, PromiseUtil_43, StateNames_16) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class LocksGameState extends RelayGameState_1.RelayGameState {
        init() {
            this.stateName = StateNames_16.StateNames.GAME_RELAY_LOCKS;
            super.init();
            this.locks = [];
        }
        preload() {
            super.preload();
            this.game.load.image("lock-background", "assets/minigame/relay/locks/locks-background.png");
            this.game.load.image("bg-frame", "assets/common/bg-frame.png");
            this.game.load.image("lock-bar-target", "assets/minigame/relay/locks/bar_target.png");
            this.game.load.image("lock-bar-background", "assets/minigame/relay/locks/bar_bg.png");
            this.game.load.image("lock-bar", "assets/minigame/relay/locks/bar.png");
            this.game.load.image("lock-bar-frame", "assets/minigame/relay/locks/bar-frame.png");
            this.game.load.image("lock-arm", "assets/minigame/relay/locks/lock-arm.png");
            this.game.load.image("lock-light", "assets/minigame/relay/locks/lock-light.png");
        }
        create() {
            this.background = new Phaser.Image(this.game, 0, 0, "lock-background");
            this.background.width = this.game.width;
            this.background.height = this.game.height;
            this.background.inputEnabled = true;
            this.add.existing(this.background);
            const yInc = this.game.height * 0.175;
            let yPos = yInc;
            const lockDifficulties = [Lock_1.LockDifficulty.EASY, Lock_1.LockDifficulty.EASY, Lock_1.LockDifficulty.MEDIUM, Lock_1.LockDifficulty.MEDIUM, Lock_1.LockDifficulty.HARD];
            for (let i = 0; i < LocksGameState.NUM_LOCKS; i++) {
                const lock = new Lock_1.Lock(this.game, lockDifficulties[i], this.shutdownSignal);
                lock.centerY = yPos;
                this.add.existing(lock);
                this.locks.push(lock);
                yPos += yInc;
            }
            this.introText = LocalizationManager_32.LocalizationManager.Instance.textWithReplacements("relay.locks_intro", { player_name: RelayGameLogic_3.RelayGameLogic.Instance.currentPlayer });
            const panel = new Phaser.Image(this.game, 0, 0, "bg-frame");
            panel.width = this.game.width;
            panel.height = this.game.height;
            this.add.existing(panel);
            // Flip every other lock horizontally
            this.locks.forEach((lock, index) => {
                if (index % 2 === 0) {
                    lock.scale.x *= -1;
                    lock.right = this.game.width;
                }
            });
            super.create();
        }
        beginGame() {
            this.locks[0].unlock();
            this.play();
            super.beginGame();
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                let lockIndex = 0;
                while (lockIndex < LocksGameState.NUM_LOCKS) {
                    const lockIndexAtLoopStart = lockIndex;
                    yield PromiseUtil_43.PromiseUtil.signal(this.background.events.onInputDown, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    const success = this.locks[lockIndex].tryLock();
                    if (success) {
                        yield this.locks[lockIndex].lock();
                        if (this.isShutdown) {
                            return;
                        }
                        lockIndex++;
                    }
                    else {
                        if (lockIndex > 0) {
                            this.locks[lockIndex].clear();
                            lockIndex--;
                            this.game.sound.play("lockgame_incorrect");
                        }
                    }
                    // If (we've changed index since the start of the loop), 
                    // && (there are still locks left)
                    // Unlock the next one
                    if (lockIndex !== lockIndexAtLoopStart && lockIndex < LocksGameState.NUM_LOCKS) {
                        yield this.locks[lockIndex].unlock();
                        if (this.isShutdown) {
                            return;
                        }
                    }
                }
                this.onGameOver(true);
            });
        }
        shutdown() {
            this.background = null;
            this.locks = [];
            super.shutdown();
        }
        static unloadAssets(game) {
            game.cache.removeImage("lock-background");
            game.cache.removeImage("bg-frame");
            game.cache.removeImage("lock-bar-target");
            game.cache.removeImage("lock-bar-background");
            game.cache.removeImage("lock-bar");
            game.cache.removeImage("lock-bar-frame");
            game.cache.removeImage("lock-arm");
            game.cache.removeImage("lock-light");
        }
    }
    LocksGameState.NUM_LOCKS = 5;
    exports.LocksGameState = LocksGameState;
});
define("component/map/PlanetView", ["require", "exports", "model/PlanetStruct"], function (require, exports, PlanetStruct_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlanetView extends Phaser.Group {
        constructor(game, data) {
            super(game);
            this.onClicked = new Phaser.Signal();
            this.base = new Phaser.Sprite(game, 0, 0, "planet-base");
            this.base.anchor.set(0.5);
            this.add(this.base);
            // Click listener
            this.base.inputEnabled = true;
            this.base.input.pixelPerfectClick = true;
            this.base.events.onInputDown.add(() => {
                this.onClicked.dispatch(data.id);
            }, this);
            if (data.specialId === PlanetStruct_2.SpecialPlanetEnum.NONE) {
                // Tinting
                this.base.tint = PlanetView.PLANET_TINTS[data.tint];
                // Texturing
                this.texture = new Phaser.Image(game, 0, 0, "planet-textures", data.texture);
                this.texture.anchor.set(0.5);
                this.texture.width = this.texture.height = this.base.width;
                this.add(this.texture);
                // Optional feature
                if (data.feature >= 0) {
                    this.feature = new Phaser.Image(game, 0, 0, `planet-feature-${data.feature}`, 0);
                    this.feature.anchor.set(0.5);
                    this.feature.width = this.feature.height = this.base.width;
                    this.add(this.feature);
                }
            }
        }
        /**
         * Returns the base sprite of the planet.
         * Used for camera.focusOn(sprite: Phaser.Sprite)
         */
        get sprite() {
            return this.base;
        }
        setHighlighted(highlighted, color) {
            // If we're turning the highlight on...
            if (highlighted) {
                // ...but we haven't created the highlight image yet - create it!
                if (!this.highlightImage) {
                    this.highlightImage = new Phaser.Image(this.game, 0, 0, 'map-hex');
                    this.highlightImage.anchor.set(0.5);
                    this.addChild(this.highlightImage);
                }
                // Tint it to the player's color
                this.highlightImage.tint = color;
                // Scale it up
                const sizeDest = this.width / this.scale.x;
                this.game.add.tween(this.highlightImage).to({ width: sizeDest, height: sizeDest }, 250, Phaser.Easing.Sinusoidal.InOut, true);
            }
            // If we're turning the highlight off (and we 
            // actually have a highlight to actually turn off)
            else if (!highlighted && this.highlightImage) {
                // Tween it down to nothingness
                this.game.add.tween(this.highlightImage.scale).to({ x: 0, y: 0 }, 250, Phaser.Easing.Sinusoidal.InOut, true);
            }
        }
        /**
         * Hides or changes frame of components of the
         * view based on hidden status.
         */
        set hidden(hidden) {
            this.base.frame = hidden ? 1 : 0;
            if (this.texture) {
                this.texture.visible = !hidden;
            }
            if (this.feature) {
                this.feature.frame = hidden ? 1 : 0;
            }
        }
        set collected(collected) {
            if (collected) {
                if (!this.collectedImage) {
                    this.collectedImage = new Phaser.Image(this.game, 0, 0, 'collected-planet-icon');
                    this.collectedImage.anchor.set(0.5);
                    this.collectedImage.width = this.width * 0.35;
                    this.collectedImage.scale.y = this.collectedImage.scale.x;
                    this.add(this.collectedImage);
                }
            }
            else {
                if (this.collectedImage) {
                    this.collectedImage.destroy();
                }
            }
        }
        destroy() {
            this.onClicked.dispose();
            super.destroy();
        }
    }
    PlanetView.PLANET_TINTS = [
        0xFFFFFF,
        0x718985, 0x7a5d58, 0xf3a619, 0xd55e37, 0xbc233d,
        0xb055ccf, 0x583df1, 0x5c83cf, 0xccde0a, 0x56df2c
    ];
    exports.PlanetView = PlanetView;
});
define("minigame/relay/scanner/ScannerLine", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ScannerLine extends Phaser.Group {
        constructor(game) {
            super(game);
            this.line = new Phaser.Image(this.game, 0, 0, 'scanner-line-seeking');
            this.line.anchor.set(0.5);
            this.add(this.line);
            this.burstFX1 = new Phaser.Image(this.game, 0, 0, 'scanner-line-burst-fx');
            this.burstFX1.anchor.set(0.5);
            this.burstFX1.alpha = 0;
            this.add(this.burstFX1);
            this.burstFX2 = new Phaser.Image(this.game, 0, 0, 'scanner-line-burst-fx');
            this.burstFX2.anchor.set(0.5);
            this.burstFX2.alpha = 0;
            this.add(this.burstFX2);
        }
        setPlaced(placed) {
            this.line.loadTexture(placed ? 'scanner-line-placed' : 'scanner-line-seeking');
            if (placed) {
                // Animate the burst fx
                this.burstFX1.scale.x = 0;
                this.burstFX2.scale.x = 0;
                this.game.add.tween(this.burstFX1.scale).to({ x: 2 }, 500, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.burstFX2.scale).to({ x: 2 }, 500, Phaser.Easing.Linear.None, true, 200);
                this.game.add.tween(this.burstFX1).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 0, 0, true);
                this.game.add.tween(this.burstFX2).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 200, 0, true);
            }
        }
    }
    exports.ScannerLine = ScannerLine;
});
define("component/ProgressBar", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ProgressBar extends Phaser.Group {
        constructor(game, progress = 0, progressBarFrameKey = 'progress-bar-border', backFillColor = 0x000000, backFillAlpha = 1) {
            super(game);
            this._progress = progress;
            this.border = new Phaser.Image(game, 0, 0, progressBarFrameKey);
            this.fullBarWidth = this.border.width * 0.95;
            this.bar = new Phaser.Image(game, 0, 0, 'progress-bar-inner');
            this.bar.anchor.set(0, 0.5);
            this.bar.width = this.fullBarWidth * this._progress;
            this.bar.centerX = this.border.centerX - (this.fullBarWidth * 0.5);
            this.backFill = new Phaser.Graphics(this.game, this.bar.left, this.bar.top);
            this.backFill.beginFill(backFillColor, backFillAlpha);
            this.backFill.drawRect(0, 0, this.fullBarWidth, this.bar.height);
            this.backFill.endFill();
            this.add(this.backFill);
            this.add(this.bar);
            this.add(this.border);
        }
        set progress(progress) {
            const newProgress = Phaser.Math.clamp(progress, 0, 1);
            this.bar.width = this.fullBarWidth * this._progress;
            this._progress = newProgress;
        }
        get progress() {
            return this._progress;
        }
    }
    exports.ProgressBar = ProgressBar;
    class ProgressBarWithTextFrame extends ProgressBar {
        constructor(game, progress = 0, backFillColor = 0x000000, backFillAlpha = 1) {
            super(game, progress, 'progress-bar-border', backFillColor, backFillAlpha);
            this.bar.height = this.border.height * 0.365;
            this.bar.centerY = this.border.height * 0.705;
            this.backFill.height = this.bar.height;
            this.backFill.top = this.bar.top;
        }
    }
    exports.ProgressBarWithTextFrame = ProgressBarWithTextFrame;
    class SimpleProgressBar extends ProgressBar {
        constructor(game, progress = 0, backFillColor = 0x000000, backFillAlpha = 1) {
            super(game, progress, 'progress-bar-border-simple', backFillColor, backFillAlpha);
            this.bar.height = this.border.height * 0.7;
            this.bar.centerY = this.border.centerY;
            this.backFill.height = this.bar.height;
            this.backFill.top = this.bar.top;
        }
    }
    exports.SimpleProgressBar = SimpleProgressBar;
});
define("minigame/relay/scanner/ScannerGameState", ["require", "exports", "minigame/relay/RelayGameState", "switchback-common/localization/LocalizationManager", "minigame/relay/RelayGameLogic", "StateNames", "switchback-common/util/PromiseUtil", "component/map/PlanetView", "model/PlanetStruct", "minigame/relay/scanner/ScannerLine", "component/ProgressBar", "Game"], function (require, exports, RelayGameState_2, LocalizationManager_33, RelayGameLogic_4, StateNames_17, PromiseUtil_44, PlanetView_1, PlanetStruct_3, ScannerLine_1, ProgressBar_1, Game_37) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class ScannerGameState extends RelayGameState_2.RelayGameState {
        init() {
            this.stateName = StateNames_17.StateNames.GAME_RELAY_SCANNER;
            super.init();
            this.planets = [];
            this.calibrationScore = 0;
            this.currentTargetPlanet = null;
            this.currentTargetPlanetIndex = 0;
        }
        preload() {
            super.preload();
            this.game.load.spritesheet("planet-base", "assets/map/planet-base.png", 400, 400);
            this.game.load.spritesheet("planet-textures", "assets/map/planet-textures.png", 200, 200);
            for (let i = 0; i < 10; i++) {
                this.game.load.spritesheet(`planet-feature-${i}`, `assets/map/features/feature-${i}.png`, 200, 200);
            }
            this.game.load.image("scanner-background", "assets/cutscene/galaxy_bg.png");
            this.game.load.image("scanner-markers", "assets/minigame/relay/scanner/scanner-markers.png");
            this.game.load.image("scanner-radial-marker", "assets/minigame/relay/scanner/scanner-radial-marker.png");
            this.game.load.image("scanner-target-marker", "assets/minigame/relay/scanner/scanner-target-marker.png");
            this.game.load.image("scanner-overlay", "assets/minigame/relay/scanner/scanner-overlay.png");
            this.game.load.image("scanner-line-seeking", "assets/minigame/relay/scanner/scanner-line-seeking.png");
            this.game.load.image("scanner-line-placed", "assets/minigame/relay/scanner/scanner-line-placed.png");
            this.game.load.image("scanner-line-burst-fx", "assets/minigame/relay/scanner/scanner-line-burst-fx.png");
            this.game.load.image("bg-frame-clear", "assets/common/bg-frame-clear.png");
            this.game.load.image("frame-header", "assets/common/frame-header.png");
            this.game.load.image("progress-bar-border-simple", "assets/common/progress-bar-border-simple.png");
            this.game.load.image("progress-bar-inner", "assets/common/progress-bar-inner.png");
        }
        create() {
            this.background = new Phaser.Image(this.game, 0, 0, "scanner-background");
            this.background.width = this.game.width;
            this.background.height = this.game.height;
            this.background.inputEnabled = true;
            this.add.existing(this.background);
            this.radialMarker = new Phaser.Image(this.game, 0, 0, "scanner-radial-marker");
            this.radialMarker.width = this.game.width * 1.25;
            this.radialMarker.scale.y = this.radialMarker.scale.x;
            this.radialMarkerOrigScale = this.radialMarker.scale.x;
            this.radialMarker.alpha = 0;
            this.radialMarker.anchor.set(0.5);
            this.game.add.tween(this.radialMarker).to({ angle: 360 }, 7500, Phaser.Easing.Linear.None, true, 0, Infinity);
            this.add.existing(this.radialMarker);
            this.createPlanet(8, 2, 8, 0, 0.3, 0.2);
            this.createPlanet(2, 1, 9, 2, 0.7, 0.25);
            this.createPlanet(4, 7, 4, 1, 0.65, 0.45);
            this.createPlanet(9, 4, 7, 1, 0.3, 0.6);
            this.createPlanet(7, 9, 6, 0, 0.3, 0.85);
            this.createPlanet(6, 8, 5, 2, 0.66, 0.7);
            Phaser.ArrayUtils.shuffle(this.planets);
            const overlay = new Phaser.Image(this.game, 0, 0, "scanner-overlay");
            overlay.width = this.game.width;
            overlay.height = this.game.height;
            this.add.existing(overlay);
            const markers = new Phaser.Image(this.game, 0, 0, "scanner-markers");
            markers.width = this.game.width;
            markers.height = this.game.height;
            this.add.existing(markers);
            this.scannerLineVertical = new ScannerLine_1.ScannerLine(this.game);
            this.scannerLineVertical.height = this.game.height;
            this.scannerLineVertical.width = this.game.width * ScannerGameState.LINE_THICKNESS_GAME_WIDTH;
            this.scannerLineVertical.x = this.game.width * ScannerGameState.X_BOUND_MIN;
            this.scannerLineVertical.y = this.game.height * 0.5;
            this.scannerLineVertical.alpha = 0;
            this.add.existing(this.scannerLineVertical);
            this.scannerLineHorizontal = new ScannerLine_1.ScannerLine(this.game);
            this.scannerLineHorizontal.angle = 90;
            this.scannerLineHorizontal.height = this.game.width;
            this.scannerLineHorizontal.width = this.game.width * ScannerGameState.LINE_THICKNESS_GAME_WIDTH;
            this.scannerLineHorizontal.x = this.game.width * 0.5;
            this.scannerLineHorizontal.y = this.game.height * ScannerGameState.Y_BOUND_MIN;
            this.scannerLineHorizontal.alpha = 0;
            this.add.existing(this.scannerLineHorizontal);
            this.target = new Phaser.Image(this.game, 0, 0, 'scanner-target-marker');
            this.target.width = this.game.width * 0.35;
            this.target.scale.y = this.target.scale.x;
            this.targetOrigScale = this.target.scale.y;
            this.target.anchor.set(0.5);
            this.target.alpha = 0;
            this.game.add.existing(this.target);
            this.targetText = new Phaser.Text(this.game, 0, 0, LocalizationManager_33.LocalizationManager.Instance.text('relay.scanner_target'), {
                font: Game_37.Game.DEFAULT_FONT,
                fontSize: Game_37.Game.FONT_SIZE_MEDIUM,
                fill: Game_37.Game.FONT_COLOUR_YELLOW,
                stroke: Game_37.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_37.Game.FONT_STROKE_SMALL
            });
            this.targetText.setScaleMinMax(1);
            this.targetText.anchor.set(0.5);
            this.targetText.alpha = 0;
            this.game.add.existing(this.targetText);
            const frame = new Phaser.Image(this.game, 0, 0, "bg-frame-clear");
            frame.width = this.game.width;
            frame.height = this.game.height;
            this.add.existing(frame);
            this.bottomPanel = new Phaser.Image(this.game, 0, 0, 'frame-header');
            this.bottomPanel.anchor.set(0.5, 0);
            this.bottomPanel.width = this.game.width * 0.85;
            this.bottomPanel.scale.y = this.bottomPanel.scale.x;
            this.bottomPanel.angle = 180;
            this.bottomPanel.position.set(this.game.width * 0.5, this.game.height);
            this.add.existing(this.bottomPanel);
            this.progressBar = new ProgressBar_1.SimpleProgressBar(this.game, 0, 0x000000, 1);
            this.progressBar.centerX = this.game.width * 0.5;
            this.progressBar.bottom = this.game.height;
            this.add.existing(this.progressBar);
            this.calibrationScoreText = new Phaser.Text(this.game, this.game.width * 0.5, 0, '100%', {
                font: Game_37.Game.DEFAULT_FONT,
                fontSize: Game_37.Game.FONT_SIZE_LARGE,
                fill: Game_37.Game.FONT_COLOUR_BLUE,
                stroke: Game_37.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_37.Game.FONT_STROKE_SMALL
            });
            this.calibrationScoreText.anchor.set(0.5);
            this.calibrationScoreText.alpha = 0;
            this.calibrationScoreTextOrigScale = this.calibrationScoreText.scale.x;
            this.game.add.existing(this.calibrationScoreText);
            this.introText = LocalizationManager_33.LocalizationManager.Instance.textWithReplacements("relay.scanner_intro", { player_name: RelayGameLogic_4.RelayGameLogic.Instance.currentPlayer });
            super.create();
        }
        createPlanet(tintId, textureId, featureId, sizeId, widthProportion, heightProportion) {
            const planet = new PlanetView_1.PlanetView(this.game, {
                tint: tintId,
                texture: textureId,
                feature: featureId,
                specialId: PlanetStruct_3.SpecialPlanetEnum.NONE
            });
            planet.position = new Phaser.Point(this.game.width * widthProportion, this.game.height * heightProportion);
            planet.hidden = false;
            planet.width = this.game.width * ScannerGameState.PLANET_SCALES[sizeId];
            planet.scale.y = planet.scale.x;
            planet.ignoreChildInput = true;
            this.planets.push(planet);
            this.add.existing(planet);
        }
        beginGame() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("beginGame").call(this);
                while (this.calibrationScore < 1) {
                    // Set a new target and play the animation to show it
                    this.currentTargetPlanet = this.planets[++this.currentTargetPlanetIndex % this.planets.length];
                    yield this.playNewTargetAnimation();
                    if (this.isShutdown) {
                        return;
                    }
                    // Get input
                    const x = yield this.getHorizontalCalibration();
                    if (this.isShutdown) {
                        return;
                    }
                    this.game.sound.play("target_1");
                    const y = yield this.getVerticalCalibration();
                    if (this.isShutdown) {
                        return;
                    }
                    this.game.sound.play("target_2");
                    // Calculate how far off they were
                    const distance = Phaser.Point.distance(new Phaser.Point(x, y), this.currentTargetPlanet.position);
                    const score = this.getCalibrationScore(distance);
                    // Add to the main score, scaled by the global modifier
                    this.calibrationScore += score * ScannerGameState.GLOBAL_PROGRESS_MODIFIER_PER_CALIBRATION;
                    if (this.calibrationScore > 1.0) {
                        this.isGamePlaying = false;
                    }
                    // Show the score they got for this calibration
                    yield this.showScore(score * 100);
                    if (this.isShutdown) {
                        return;
                    }
                    // Animate that on the bar
                    this.game.add.tween(this.progressBar).to({ progress: this.calibrationScore }, 200, Phaser.Easing.Linear.None, true);
                    this.game.sound.play("bar_charge");
                    // Play the completion animation
                    yield this.playTargetCalibratedAnimation();
                    if (this.isShutdown) {
                        return;
                    }
                }
                // They've broken out of the while loop, so they've won!
                this.onGameOver(true);
            });
        }
        getCalibrationScore(distance) {
            const maxDistance = ScannerGameState.DISTANCE_MAX_GAME_WIDTH * this.game.width;
            const minDistance = ScannerGameState.DISTANCE_MIN_GAME_WIDTH * this.game.width;
            const percentageOffPerfectScore = distance / (maxDistance - minDistance);
            return Math.max(ScannerGameState.MIN_SCORE_PER_CALIBRATION, 1 - percentageOffPerfectScore);
        }
        showScore(percent) {
            return __awaiter(this, void 0, void 0, function* () {
                this.calibrationScoreText.text = `${percent.toFixed(0)}%`;
                // Move the score to where the target planet is
                this.calibrationScoreText.centerX = this.currentTargetPlanet.centerX;
                this.calibrationScoreText.centerY = this.currentTargetPlanet.centerY;
                // Fade out the target text
                this.game.add.tween(this.targetText)
                    .to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                // Show the score (fade and pulse)
                yield Promise.all([
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.calibrationScoreText)
                        .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.calibrationScoreText.scale)
                        .to({ x: this.calibrationScoreTextOrigScale * 2, y: this.calibrationScoreTextOrigScale * 2 }, 350, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true), this.shutdownSignal)
                ]).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // Fade out the score text (with a small delay)
                this.game.add.tween(this.calibrationScoreText)
                    .to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 300);
                // and tween it to the progress bar
                yield PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.calibrationScoreText)
                    .to({ x: this.progressBar.centerX, y: this.progressBar.centerY }, 350, Phaser.Easing.Sinusoidal.InOut, true, 0), this.shutdownSignal).catch(() => { });
            });
        }
        playNewTargetAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                this.radialMarker.position = this.currentTargetPlanet.position.clone();
                this.target.position = this.currentTargetPlanet.position.clone();
                this.targetText.x = this.target.x;
                this.targetText.y = this.target.y + (this.game.height * 0.1);
                this.radialMarker.scale.set(this.radialMarkerOrigScale);
                this.target.scale.set(this.targetOrigScale * 3);
                this.game.sound.play("new_target");
                // Tween the target in from a large size
                this.game.add.tween(this.target.scale)
                    .to({ x: this.targetOrigScale, y: this.targetOrigScale }, 350, Phaser.Easing.Cubic.Out, true)
                    .onComplete.addOnce(() => {
                    // Start an infinite pulse tween when complete
                    this.game.add.tween(this.target.scale)
                        .to({ x: this.targetOrigScale * 1.5, y: this.targetOrigScale * 1.5 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                });
                yield Promise.all([
                    // Tween the radial marker's size up
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.radialMarker.scale)
                        .from({ x: this.radialMarkerOrigScale * 0.6, y: this.radialMarkerOrigScale * 0.6 }, 350, Phaser.Easing.Cubic.Out, true), this.shutdownSignal),
                    // Tween the alpha of everything in
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.radialMarker)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.target)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.targetText)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal)
                ]).catch(() => { });
            });
        }
        playTargetCalibratedAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                // Remove the infinite pulsing tween from the target
                this.game.tweens.removeFrom(this.target.scale);
                yield Promise.all([
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.target.scale)
                        .to({ x: this.targetOrigScale * 3, y: this.targetOrigScale * 3 }, 350, Phaser.Easing.Cubic.In, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.target)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.radialMarker)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.scannerLineHorizontal)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.scannerLineVertical)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal)
                ]).catch(() => { });
            });
        }
        getHorizontalCalibration() {
            return __awaiter(this, void 0, void 0, function* () {
                // Reset the line
                this.scannerLineVertical.setPlaced(false);
                this.scannerLineVertical.x = this.game.width * ScannerGameState.X_BOUND_MIN;
                // Animate it in and across
                this.game.add.tween(this.scannerLineVertical)
                    .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                const horizontalMovementTween = this.game.add.tween(this.scannerLineVertical)
                    .to({ x: this.game.width * ScannerGameState.X_BOUND_MAX }, 1000, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                // Wait for a tap!
                yield PromiseUtil_44.PromiseUtil.signal(this.background.events.onInputDown, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // Stop the tween (and set the full alpha if necessary)
                horizontalMovementTween.stop();
                this.scannerLineVertical.alpha = 1;
                this.scannerLineVertical.setPlaced(true);
                return this.scannerLineVertical.x;
            });
        }
        getVerticalCalibration() {
            return __awaiter(this, void 0, void 0, function* () {
                // Reset the line
                this.scannerLineHorizontal.setPlaced(false);
                this.scannerLineHorizontal.y = this.game.height * ScannerGameState.Y_BOUND_MIN;
                // Animate it in and across
                this.game.add.tween(this.scannerLineHorizontal)
                    .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                const verticalMovementTween = this.game.add.tween(this.scannerLineHorizontal)
                    .to({ y: this.game.height * ScannerGameState.Y_BOUND_MAX }, 1500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                // Wait for a tap!
                yield PromiseUtil_44.PromiseUtil.signal(this.background.events.onInputDown, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // Stop the tween (and set the full alpha if necessary)
                verticalMovementTween.stop();
                this.scannerLineHorizontal.alpha = 1;
                this.scannerLineHorizontal.setPlaced(true);
                return this.scannerLineHorizontal.y;
            });
        }
        shutdown() {
            this.planets = null;
            this.background = null;
            this.target = null;
            this.targetText = null;
            this.radialMarker = null;
            this.currentTargetPlanet = null;
            this.scannerLineVertical = null;
            this.scannerLineHorizontal = null;
            this.bottomPanel = null;
            this.progressBar = null;
            this.calibrationScoreText = null;
            super.shutdown();
        }
        static unloadAssets(game) {
            game.cache.removeImage("planet-base");
            game.cache.removeImage("planet-textures");
            for (let i = 0; i < 10; i++) {
                game.cache.removeImage(`planet-feature-${i}`);
            }
            game.cache.removeImage("scanner-background");
            game.cache.removeImage("scanner-markers");
            game.cache.removeImage("scanner-radial-marker");
            game.cache.removeImage("scanner-target-marker");
            game.cache.removeImage("scanner-overlay");
            game.cache.removeImage("scanner-line-seeking");
            game.cache.removeImage("scanner-line-placed");
            game.cache.removeImage("scanner-line-burst-fx");
            game.cache.removeImage("bg-frame-clear");
            game.cache.removeImage("frame-header");
            game.cache.removeImage("progress-bar-border-simple");
            game.cache.removeImage("progress-bar-inner");
        }
    }
    ScannerGameState.PLANET_SCALES = [0.3, 0.4, 0.5];
    ScannerGameState.X_BOUND_MIN = 0.1;
    ScannerGameState.X_BOUND_MAX = 0.9;
    ScannerGameState.Y_BOUND_MIN = 0.1;
    ScannerGameState.Y_BOUND_MAX = 0.9;
    ScannerGameState.LINE_THICKNESS_GAME_WIDTH = 0.35;
    // The smallest distance. This, or less, is equivalent to a perfect score.
    ScannerGameState.DISTANCE_MIN_GAME_WIDTH = 0.02;
    // The maximum distance to care about. Anything larger than this is a failure,
    // and only receives ScannerGameState.MIN_SCORE_PER_CALIBRATION out of pity
    ScannerGameState.DISTANCE_MAX_GAME_WIDTH = 0.75;
    // A 'fail' still shows as this (0.1 = 10%)
    ScannerGameState.MIN_SCORE_PER_CALIBRATION = 0.1;
    // The amount to progress the game's main progress bar for a perfect calibration
    // At 0.5, this means a perfect score awards 50% completion
    ScannerGameState.GLOBAL_PROGRESS_MODIFIER_PER_CALIBRATION = 0.5;
    exports.ScannerGameState = ScannerGameState;
});
define("minigame/relay/switches/SwitchesGameState", ["require", "exports", "minigame/relay/RelayGameLogic", "minigame/relay/RelayGameState", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "StateNames"], function (require, exports, RelayGameLogic_5, RelayGameState_3, LocalizationManager_34, ScaleUtil_28, StateNames_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    // Helper class to handle the animation for the switch
    class Switch extends Phaser.Sprite {
        constructor(game, x, y) {
            super(game, x, y, 'switches-buttons', 0);
            this.putDown();
            this.animations.add(Switch.A_TO_ON, [0, 1, 2, 3, 4], Switch.FPS);
            this.animations.add(Switch.A_TO_OFF, [4, 3, 2, 1, 0], Switch.FPS);
        }
        // Instantly resets the button to the off state
        putDown() {
            this.state = false;
            this.frame = 0;
        }
        isDown() { return this.state; }
        setIsDown(down) {
            if (this.state === down) {
                return;
            }
            this.state = down;
            this.play(this.state ? Switch.A_TO_ON : Switch.A_TO_OFF);
        }
        toggle() {
            this.setIsDown(!this.state);
        }
    }
    Switch.A_TO_ON = "to-on";
    Switch.A_TO_OFF = "to-off";
    Switch.FPS = 20; // 1/0.05
    exports.Switch = Switch;
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     * @author qi@cerebralfix.com
     */
    class SwitchesGameState extends RelayGameState_3.RelayGameState {
        init() {
            this.stateName = StateNames_18.StateNames.GAME_RELAY_SWITCHES;
            super.init();
            this.switchButtons = [];
        }
        preload() {
            super.preload();
            this.game.load.image("switches-background", "assets/minigame/relay/switches/background.jpg");
            this.game.load.image("switches-fx", "assets/minigame/relay/switches/switch_fx.png");
            this.game.load.spritesheet("switches-buttons", "assets/minigame/relay/switches/switches.png", 169, 185);
        }
        create() {
            const background = new Phaser.Image(this.game, 0, 0, "switches-background");
            // TODO: Scale nicely
            background.width = this.game.width;
            background.height = this.game.height;
            this.add.existing(background);
            this.createSwitches();
            this.tapFXPool = new Phaser.Group(this.game);
            this.add.existing(this.tapFXPool);
            this.introText = LocalizationManager_34.LocalizationManager.Instance.textWithReplacements("relay.switches_intro", { player_name: RelayGameLogic_5.RelayGameLogic.Instance.currentPlayer });
            super.create(); // Important in order for the timer to show at the top
            this.input.maxPointers = 3;
        }
        // Returns a new Pulse FX for the tap, recycle the old ones
        // TODO use component/fx/Taps
        getTapFX() {
            // Tries to get a dead sprite, or creates a new one
            let fx = this.tapFXPool.getFirstDead();
            if (fx) {
                fx.revive();
            }
            else {
                fx = new Phaser.Sprite(this.game, 0, 0, "switches-fx");
                fx.anchor.set(0.5);
                this.tapFXPool.add(fx);
            }
            const duration = 500;
            // FX Scale
            const scale = this.game.width * 0.002;
            fx.scale.y = fx.scale.x = scale * 0.1;
            this.game.add.tween(fx.scale).to({ x: scale, y: scale }, duration, Phaser.Easing.Linear.None, true);
            // FX Alpha
            fx.alpha = 1;
            let tween = this.game.add.tween(fx);
            tween.to({ alpha: 0 }, duration / 2.0, Phaser.Easing.Linear.None, true, duration / 2.0, 0, false);
            tween.onComplete.addOnce(fx.kill, fx); // Kill it when done
            return fx;
        }
        createSwitches() {
            // Bunch of positioning variables to make the layout logic easier
            const xStartPos = this.game.width * 0.225;
            const yStartPos = this.game.height * 0.225;
            const xInc = (this.game.width - (xStartPos * 2)) / (SwitchesGameState.NUM_COLUMNS - 1);
            const yInc = this.game.height * 0.16;
            let xPos = xStartPos;
            let yPos = yStartPos;
            for (let row = 0; row < SwitchesGameState.NUM_ROWS; row++) {
                for (let col = 0; col < SwitchesGameState.NUM_COLUMNS; col++) {
                    const switchButt = new Switch(this.game, xPos, yPos);
                    switchButt.anchor.set(0.5);
                    ScaleUtil_28.scaleToBounds(switchButt, this.game.width / (SwitchesGameState.NUM_COLUMNS + 1), this.game.height / (SwitchesGameState.NUM_ROWS + 1));
                    switchButt.events.onInputDown.add(() => { this.onSwitchFlipped(switchButt); });
                    this.switchButtons.push(switchButt);
                    this.add.existing(switchButt);
                    // Input disabling workaround :sadface:
                    this.game.time.events.add(1, () => {
                        switchButt.inputEnabled = false;
                    }, this);
                    xPos += xInc;
                }
                xPos = xStartPos;
                yPos += yInc;
            }
            // Pre-flip some switches
            Phaser.ArrayUtils.shuffle(this.switchButtons);
            for (let i = 0; i < SwitchesGameState.MAX_NUM_PRE_FLIPPED; i++) {
                this.switchButtons[i].setIsDown(true);
            }
        }
        beginGame() {
            // Enable input on all buttons
            this.switchButtons.forEach((butt) => {
                butt.inputEnabled = true;
            }, this);
            super.beginGame();
        }
        onSwitchFlipped(switchFlipped) {
            // If our input disabling fails for whatever reason, this check should stop 
            // any switches from being flipped after the game finishes.
            if (!this.isGamePlaying) {
                return;
            }
            this.game.sound.play('click_switch' + this.game.rnd.integerInRange(1, 4));
            switchFlipped.toggle();
            if (switchFlipped.isDown()) {
                const fx = this.getTapFX();
                fx.centerX = switchFlipped.centerX;
                fx.centerY = switchFlipped.centerY - (switchFlipped.height * 0.1);
            }
            // Calculate how many switches are left.
            // Maintaining a single number and incrementing/decrementing it had bugs when
            // multiple switches were flipped simultaneously.
            // This fix ensures the number is always correct.
            let numSwitchesLeft = this.switchButtons.filter((switchButt) => !switchButt.isDown()).length;
            // Finally, if all switches are now pressed, the player has won
            if (numSwitchesLeft <= 0) {
                this.onGameOver(true);
            }
            else {
                // We have a small chance to re-flip a switch
                this.rollForReFlip(switchFlipped);
            }
        }
        rollForReFlip(switchJustFlipped) {
            if (Math.random() < SwitchesGameState.CHANCE_FOR_RE_FLIP) {
                // Get a list of all switches that are flipped on, and aren't the one the player just flipped.
                let flippedSwitches = this.switchButtons.filter((s) => {
                    return s.isDown() && s !== switchJustFlipped;
                }, this);
                // Shuffle it
                Phaser.ArrayUtils.shuffle(flippedSwitches);
                // Generate a number of switches to flip back
                const numSwitchesToFlipBack = this.game.rnd.integerInRange(1, 2);
                for (let i = 0; i < numSwitchesToFlipBack && i < flippedSwitches.length; i++) {
                    flippedSwitches[i].setIsDown(false);
                }
            }
        }
        onGameOver(won) {
            // Stop input on all buttons
            this.switchButtons.forEach((butt) => {
                butt.inputEnabled = false;
            }, this);
            super.onGameOver(won);
        }
        shutdown() {
            this.switchButtons = [];
            this.tapFXPool = null;
            super.shutdown();
        }
        static unloadAssets(game) {
            game.cache.removeImage("switches-background");
            game.cache.removeImage("switches-fx");
            game.cache.removeImage("switches-buttons");
        }
    }
    SwitchesGameState.NUM_ROWS = 5;
    SwitchesGameState.NUM_COLUMNS = 4;
    SwitchesGameState.MAX_NUM_PRE_FLIPPED = 5;
    SwitchesGameState.CHANCE_FOR_RE_FLIP = 0.25;
    exports.SwitchesGameState = SwitchesGameState;
});
define("minigame/relay/wrench/WrenchGameState", ["require", "exports", "minigame/relay/RelayGameState", "switchback-common/localization/LocalizationManager", "minigame/relay/RelayGameLogic", "StateNames", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "Game"], function (require, exports, RelayGameState_4, LocalizationManager_35, RelayGameLogic_6, StateNames_19, PromiseUtil_45, ScaleUtil_29, Game_38) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class WrenchGameState extends RelayGameState_4.RelayGameState {
        init() {
            this.stateName = StateNames_19.StateNames.GAME_RELAY_WRENCH;
            super.init();
            this.itemsToMove = [];
        }
        preload() {
            super.preload();
            this.game.load.image("wrench-background", "assets/minigame/relay/wrench/wrench_bg.jpg");
            this.game.load.image("toolbox", "assets/minigame/relay/wrench/Toolbox.png");
            this.game.load.image("bg-frame", "assets/common/bg-frame.png");
            this.game.load.image("clue-panel", "assets/minigame/relay/wrench/Clue_Panel.png");
            this.game.load.image("item1", "assets/minigame/relay/wrench/Item_01.png");
            this.game.load.image("clue1", "assets/minigame/relay/wrench/Item_01_Find.png");
            this.game.load.image("item2", "assets/minigame/relay/wrench/Item_02.png");
            this.game.load.image("clue2", "assets/minigame/relay/wrench/Item_02_Find.png");
            this.game.load.image("item3", "assets/minigame/relay/wrench/Item_03.png");
            this.game.load.image("clue3", "assets/minigame/relay/wrench/Item_03_Find.png");
            this.game.load.image("item4", "assets/minigame/relay/wrench/Item_04.png");
            this.game.load.image("clue4", "assets/minigame/relay/wrench/Item_04_Find.png");
            this.game.load.image("item5", "assets/minigame/relay/wrench/Item_05.png");
            this.game.load.image("clue5", "assets/minigame/relay/wrench/Item_05_Find.png");
            this.game.load.image("junk1", "assets/minigame/relay/wrench/DummyItem_01.png");
            this.game.load.image("junk2", "assets/minigame/relay/wrench/DummyItem_02.png");
            this.game.load.image("junk3", "assets/minigame/relay/wrench/DummyItem_03.png");
            this.game.load.image("junk4", "assets/minigame/relay/wrench/DummyItem_04.png");
            this.game.load.image("junk5", "assets/minigame/relay/wrench/DummyItem_05.png");
        }
        create() {
            const background = new Phaser.Image(this.game, 0, 0, "wrench-background");
            background.anchor.set(0.5);
            ScaleUtil_29.scaleToCover(background, this.game.width, this.game.height);
            background.x = this.game.width * 0.5;
            background.y = this.game.height * 0.5;
            this.add.existing(background);
            this.game.add.tween(background).to({ angle: background.angle + this.game.rnd.pick([15, -15]) }, 15000, Phaser.Easing.Sinusoidal.InOut).yoyo(true).loop(true).start();
            this.toolBox = new Phaser.Image(this.game, 0, 0, "toolbox");
            this.toolBox.anchor.set(0.5);
            this.toolBox.width = this.game.width * 0.8;
            this.toolBox.scale.y = this.toolBox.scale.x;
            this.toolBox.x = this.game.width * 0.5;
            this.toolBox.bottom = this.game.height;
            this.add.existing(this.toolBox);
            this.game.add.tween(this.toolBox).to({ angle: this.toolBox.angle + this.game.rnd.pick([15, -15]) }, 12000, Phaser.Easing.Sinusoidal.InOut).yoyo(true).loop(true).start();
            this.introText = LocalizationManager_35.LocalizationManager.Instance.textWithReplacements("relay.wrench_intro", { player_name: RelayGameLogic_6.RelayGameLogic.Instance.currentPlayer });
            const itemLayer = new Phaser.Group(this.game);
            this.add.existing(itemLayer);
            const frame = new Phaser.Image(this.game, 0, 0, 'bg-frame');
            frame.width = this.game.width;
            frame.height = this.game.height;
            this.add.existing(frame);
            // Create a plain group around the panel so it makes it easier to scale in and out
            this.cluePanel = new Phaser.Group(this.game);
            this.add.existing(this.cluePanel);
            const panel = new Phaser.Image(this.game, 0, 0, 'clue-panel');
            panel.width = this.game.width * 0.3;
            panel.scale.y = this.cluePanel.scale.x;
            this.cluePanel.add(panel);
            this.cluePanel.pivot.set(this.cluePanel.width * 0.5, this.cluePanel.height * 0.5);
            this.cluePanel.position.set(this.game.width * 0.5, this.game.height * 0.85);
            this.findText = new Phaser.Text(this.game, 0, 0, LocalizationManager_35.LocalizationManager.Instance.text('relay.wrench_find'), Object.assign({}, Game_38.Game.DEFAULT_FONT_STYLE, { fill: Game_38.Game.FONT_COLOUR_GREEN, fontSize: Game_38.Game.FONT_SIZE_EXTRA_LARGE, stroke: 'black' }));
            this.findText.centerX = this.cluePanel.centerX;
            this.findText.centerY = this.cluePanel.bottom;
            this.findText.alpha = 0;
            this.add.existing(this.findText);
            this.clue = new Phaser.Image(this.game, 0, 0, "clue1");
            this.clue.anchor.set(0.5);
            this.clue.position.set(this.cluePanel.x, this.cluePanel.y);
            this.clue.width = this.cluePanel.width;
            this.clue.scale.y = this.clue.scale.x;
            this.clue.visible = false;
            this.add.existing(this.clue);
            // Make this invisible after positioning other assets around it
            this.cluePanel.scale.set(0);
            const bottomBound = this.cluePanel.centerY;
            // Create target items
            this.targetItems = [
                { itemTag: "item1", clueTag: "clue1" },
                { itemTag: "item2", clueTag: "clue2" },
                { itemTag: "item3", clueTag: "clue3" },
                { itemTag: "item4", clueTag: "clue4" },
                { itemTag: "item5", clueTag: "clue5" }
            ].map((data) => {
                const item = new Phaser.Image(this.game, 0, 0, data.itemTag);
                item.anchor.set(0.5);
                item.scale.set(this.game.width / 980);
                item.x = this.game.rnd.frac() * this.game.width;
                item.y = this.game.rnd.frac() * bottomBound;
                item.inputEnabled = false;
                return {
                    item,
                    clueTag: data.clueTag
                };
            });
            // Create junk items
            const numCopies = 3;
            const junkItems = [
                "junk1",
                "junk2",
                "junk3",
                "junk4",
                "junk5",
            ].reduce((sum, itemKey) => {
                const copies = [];
                for (let i = 0; i < numCopies; i++) {
                    const item = new Phaser.Image(this.game, 0, 0, itemKey);
                    item.anchor.set(0.5);
                    item.scale.set(this.game.width / 980);
                    item.x = this.game.rnd.frac() * this.game.width;
                    item.y = this.game.rnd.frac() * bottomBound;
                    item.inputEnabled = false;
                    copies.push(item);
                }
                return sum.concat(copies);
            }, []);
            // Create floating items
            const allItems = Phaser.ArrayUtils.shuffle(this.targetItems.map(target => target.item).concat(junkItems));
            allItems.forEach(item => {
                // Start it spinning
                this.game.add.tween(item).to({ angle: this.game.rnd.pick([360, -360]) }, (this.game.rnd.frac() * 6000) + 4000).loop(true).start();
                // Make sure all the tools float on approximate diagonals to make it more interesting
                const maxDeviationFromDiagonal = 22.5; // degrees
                const angleToRotateBy = (this.game.rnd.frac() * maxDeviationFromDiagonal * 2) + maxDeviationFromDiagonal + this.game.rnd.pick([0, 90, 180, 270]);
                this.itemsToMove.push({
                    item,
                    velocity: new Phaser.Point(0, 1).rotate(0, 0, angleToRotateBy, true)
                });
                itemLayer.add(item);
            });
            super.create();
        }
        beginGame() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("beginGame").call(this);
                PromiseUtil_45.PromiseUtil.time(this.game, RelayGameState_4.RelayGameState.TIME_GIVEN_MS[this.difficultyLevel]).then(() => {
                    if (this.isGamePlaying) {
                        this.hideTarget();
                        this.onGameOver(false);
                    }
                });
                Phaser.ArrayUtils.shuffle(this.targetItems);
                while (this.targetItems.length > 0) {
                    const target = this.targetItems[0];
                    yield this.showTarget(target);
                    if (this.isShutdown) {
                        return;
                    }
                    target.item.inputEnabled = true;
                    yield PromiseUtil_45.PromiseUtil.signal(target.item.events.onInputDown, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    this.game.sound.play("wrenchgame_tap_corrrect");
                    this.game.tweens.removeFrom(target.item);
                    this.game.add.tween(target.item)
                        .to({
                        x: this.toolBox.centerX,
                        y: this.toolBox.top + (this.toolBox.height * 0.4),
                        alpha: 0
                    }, 300, Phaser.Easing.Sinusoidal.InOut, true)
                        .onComplete.addOnce(() => {
                        target.item.destroy();
                    });
                    this.targetItems.shift();
                    yield this.hideTarget();
                    if (this.isShutdown) {
                        return;
                    }
                }
                if (this.isGamePlaying) {
                    this.onGameOver(true);
                }
            });
        }
        update() {
            super.update();
            this.itemsToMove.forEach(itemData => {
                itemData.item.x += itemData.velocity.x * this.game.time.physicsElapsedMS * WrenchGameState.DRIFT_SPEED;
                if (itemData.item.x < 0 || itemData.item.x > this.game.width) {
                    itemData.velocity.x *= -1;
                }
                itemData.item.y += itemData.velocity.y * this.game.time.physicsElapsedMS * WrenchGameState.DRIFT_SPEED;
                const bottomBound = this.cluePanel.centerY;
                if (itemData.item.y < 0 || itemData.item.y > bottomBound) {
                    itemData.velocity.y *= -1;
                }
            });
        }
        showTarget(target) {
            return __awaiter(this, void 0, void 0, function* () {
                this.clue.loadTexture(target.clueTag);
                this.game.add.tween(this.findText).to({ alpha: 1 }, WrenchGameState.CLUE_FADE_TIME_MS, Phaser.Easing.Sinusoidal.In).start();
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.cluePanel.scale).to({ x: 1, y: 1 }, WrenchGameState.CLUE_FADE_TIME_MS, Phaser.Easing.Sinusoidal.In).start(), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.clue.visible = true;
                this.findText.visible = true;
                // Disappear for 100ms, then show for 800ms
                this.clueBlinkTimer = this.game.time.events.loop(800, () => __awaiter(this, void 0, void 0, function* () {
                    this.clue.visible = false;
                    yield PromiseUtil_45.PromiseUtil.time(this.game, 100, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    // Make sure the timer wasn't killed while we were waiting
                    if (this.clueBlinkTimer !== null) {
                        this.clue.visible = true;
                    }
                }));
            });
        }
        hideTarget() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.time.events.remove(this.clueBlinkTimer);
                this.clueBlinkTimer = null;
                this.clue.visible = false;
                this.findText.visible = false;
                this.game.add.tween(this.findText).to({ alpha: 0 }, WrenchGameState.CLUE_FADE_TIME_MS, Phaser.Easing.Sinusoidal.Out).start();
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.cluePanel.scale).to({ x: 0, y: 0 }, WrenchGameState.CLUE_FADE_TIME_MS, Phaser.Easing.Sinusoidal.Out).start(), this.shutdownSignal).catch(() => { });
            });
        }
        shutdown() {
            this.itemsToMove = [];
            this.targetItems = null;
            this.clue = null;
            this.cluePanel = null;
            this.findText = null;
            this.clueBlinkTimer = null;
            this.toolBox = null;
            super.shutdown();
        }
        static unloadAssets(game) {
            game.cache.removeImage("wrench-background");
            game.cache.removeImage("toolbox");
            game.cache.removeImage("bg-frame");
            game.cache.removeImage("clue-panel");
            game.cache.removeImage("item1");
            game.cache.removeImage("clue1");
            game.cache.removeImage("item2");
            game.cache.removeImage("clue2");
            game.cache.removeImage("item3");
            game.cache.removeImage("clue3");
            game.cache.removeImage("item4");
            game.cache.removeImage("clue4");
            game.cache.removeImage("item5");
            game.cache.removeImage("clue5");
            game.cache.removeImage("junk1");
            game.cache.removeImage("junk2");
            game.cache.removeImage("junk3");
            game.cache.removeImage("junk4");
            game.cache.removeImage("junk5");
        }
    }
    WrenchGameState.DRIFT_SPEED = 0.3;
    WrenchGameState.CLUE_FADE_TIME_MS = 150;
    exports.WrenchGameState = WrenchGameState;
});
define("minigame/relay/RelayGameOutroState", ["require", "exports", "minigame/relay/RelayGameLogic", "component/dialog/BasicSpaceButtonDialog", "logic/GameController", "StateNames", "state/SpaceBaseState", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "logic/SMAnalyticsManager", "minigame/relay/locks/LocksGameState", "minigame/relay/scanner/ScannerGameState", "minigame/relay/switches/SwitchesGameState", "minigame/relay/wrench/WrenchGameState"], function (require, exports, RelayGameLogic_7, BasicSpaceButtonDialog_5, GameController_23, StateNames_20, SpaceBaseState_6, LocalizationManager_36, TextUtil_17, SMAnalyticsManager_20, LocksGameState_1, ScannerGameState_1, SwitchesGameState_1, WrenchGameState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class RelayGameOutroState extends SpaceBaseState_6.SpaceBaseState {
        init() {
            this.stateName = StateNames_20.StateNames.GAME_RELAY_OUTRO;
            super.init();
        }
        create() {
            super.create();
            const background = new Phaser.Image(this.game, 0, 0, "background");
            // TODO: Scale nicely
            background.width = this.game.width;
            background.height = this.game.height;
            // Mark a minigame as completed
            let relayLogic = RelayGameLogic_7.RelayGameLogic.Instance;
            let outroDialog = new BasicSpaceButtonDialog_5.BasicSpaceButtonDialog(this.game, LocalizationManager_36.LocalizationManager.Instance.textWithReplacements("relay.outro", {
                tasks_completed: relayLogic.numTasksComplete,
                tasks_total: relayLogic.numGamesPlayed
            }), LocalizationManager_36.LocalizationManager.Instance.text("relay.button_ok"), this.shutdownSignal, TextUtil_17.TextUtil.getFontSize(this.game.width, 30));
            outroDialog.onButtonClickedSignal.addOnce(() => {
                outroDialog.hide();
                this.transitionToOutro();
            });
            this.add.existing(background);
            this.add.existing(outroDialog);
            this.dismissOverlay();
            outroDialog.show();
            SMAnalyticsManager_20.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/score', `MaintenanceScore`, {
                [SMAnalyticsManager_20.SMAnalyticsManager.GAME_NAME_MINI]: SMAnalyticsManager_20.SMAnalyticsManager.gameNameMap.get(this.stateName),
                [SMAnalyticsManager_20.SMAnalyticsManager.GAME_PLAYS]: GameController_23.GameController.Instance.state.getMinigamePlays(this.stateName),
                [SMAnalyticsManager_20.SMAnalyticsManager.GAME_SCORE]: relayLogic.numTasksComplete
            });
        }
        transitionToOutro() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToOutro()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                GameController_23.GameController.Instance.finishGame();
                this.game.state.start(StateNames_20.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        shutdown() {
            // Done playing the relay games, unload their assets 
            // Do this once at the end so we don't thrash the memory as relay games aren't pooled
            // TODO: pool relay game states
            LocksGameState_1.LocksGameState.unloadAssets(this.game);
            ScannerGameState_1.ScannerGameState.unloadAssets(this.game);
            SwitchesGameState_1.SwitchesGameState.unloadAssets(this.game);
            WrenchGameState_1.WrenchGameState.unloadAssets(this.game);
            super.shutdown();
        }
    }
    exports.RelayGameOutroState = RelayGameOutroState;
});
define("minigame/pipes/PipeHalfDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/button/CTAButton", "Game", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "component/coderedeem/CodeButton"], function (require, exports, PromiseUtil_46, CTAButton_6, Game_39, ScaleUtil_30, LocalizationManager_37, CodeButton_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PipeHalfDialog extends Phaser.Group {
        constructor(game, isTop, isTutorialMode, shutdownSignal) {
            super(game);
            this.tiles = [];
            this.isTop = isTop;
            this.isTutorialMode = isTutorialMode;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            const bg = new Phaser.Image(this.game, 0, 0, 'pipes-tutorial-panel');
            bg.anchor.set(0, 1);
            bg.width = this.game.width;
            bg.height = this.game.height * 0.5;
            this.add(bg);
            this.nameText = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.11, '', {
                fill: Game_39.Game.FONT_COLOUR_BLUE,
                font: Game_39.Game.DEFAULT_FONT,
                stroke: Game_39.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_39.Game.FONT_STROKE_LARGE,
                fontSize: Game_39.Game.FONT_SIZE_LARGE,
                align: 'center'
            });
            this.nameText.anchor.set(0.5);
            this.add(this.nameText);
            ////////////////////////////////
            // INTERACTIVE TILES
            ////////////////////////////////
            this.tileGroup = new Phaser.Group(this.game);
            this.add(this.tileGroup);
            const tileWidth = this.game.width / 6;
            this.tapTile = new Phaser.Sprite(this.game, tileWidth, bg.top + bg.height * 0.35, 'pipes-tutorial-taps');
            this.tapTile.anchor.set(0.5);
            this.tapTile.angle = 90;
            this.tiles.push(this.tapTile);
            for (let i = 1; i < 4; i++) {
                const tile = new Phaser.Image(this.game, tileWidth * (i + 1), bg.top + bg.height * 0.35, 'pipes-tutorial-tiles', 0);
                this.tiles.push(tile);
            }
            const sinkTile = new Phaser.Image(this.game, tileWidth * 5, bg.top + bg.height * 0.35, 'pipes-tutorial-sm');
            this.tiles.push(sinkTile);
            this.tiles.forEach((tile) => {
                tile.anchor.set(0.5);
                tile.width = tileWidth;
                tile.scale.y = tile.scale.x;
                this.tileGroup.add(tile);
            });
            ////////////////////////////////
            // PROBE
            ////////////////////////////////
            this.probe = new Phaser.Image(this.game, this.tiles[0].centerX, this.tiles[0].centerY, 'pipes-vessels');
            this.probe.anchor.set(0.5);
            this.probe.angle = 90;
            ScaleUtil_30.scaleToBounds(this.probe, this.tiles[0].width * 0.8, this.tiles[0].height * 0.8);
            this.add(this.probe);
            ////////////////////////////////
            // MARKER
            ////////////////////////////////
            this.marker = new Phaser.Image(this.game, this.tiles[1].centerX, this.tiles[1].top, 'pipes-tutorial-marker');
            this.marker.anchor.set(0.5, 1);
            ScaleUtil_30.scaleToBounds(this.marker, this.tiles[0].width * 0.8, this.tiles[0].height * 0.8);
            this.game.add.tween(this.marker).to({ y: this.marker.y - this.game.height * 0.02 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.add(this.marker);
            this.target = new Phaser.Image(this.game, 0, 0, 'pipes-tutorial-target');
            this.target.anchor.set(0.5);
            ScaleUtil_30.scaleToBounds(this.target, this.tiles[1].width * 0.5, this.tiles[1].height * 0.5);
            this.target.centerX = this.tiles[1].centerX;
            this.target.centerY = this.tiles[1].centerY;
            this.target.alpha = 1;
            this.add(this.target);
            // Add tweens for scaling and fading the target
            // Store references so we can pause/resume them
            this.targetTweens = [
                this.game.add.tween(this.target).to({ width: this.tiles[1].width * 1.2, height: this.tiles[1].height * 1.2 }, 500, Phaser.Easing.Sinusoidal.Out, true, 0, Infinity),
                this.game.add.tween(this.target).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true, 0, Infinity)
            ];
            this.handAnim = new Phaser.Sprite(this.game, 0, 0, 'pipes-hand-anim');
            ScaleUtil_30.scaleToBounds(this.handAnim, this.tiles[1].width * 0.7, this.tiles[1].height * 0.7);
            this.handAnim.top = this.tiles[1].centerY - this.handAnim.width * 0.25;
            this.handAnim.left = this.tiles[1].centerX - this.handAnim.width * 0.25;
            this.handAnim.alpha = 0;
            this.add(this.handAnim);
            this.handAnim.animations.add('tap', [1, 0, 0, 1, 1], 10, true).play();
            ////////////////////////////////
            // COMPLETION UI
            ////////////////////////////////
            this.successBox = new Phaser.Image(this.game, this.game.width * 0.5, bg.top + bg.height * 0.35, 'pipes-success-box');
            this.successBox.anchor.set(0.5);
            this.successBox.width = this.game.width * 0.8;
            this.successBox.scale.y = this.successBox.scale.x;
            this.add(this.successBox);
            this.successFlare = new Phaser.Image(this.game, this.successBox.centerX, this.successBox.centerY, 'pipe-box-flare');
            this.successFlare.anchor.set(0.5);
            this.successFlare.width = this.game.width * 0.9;
            this.successFlare.scale.y = this.successFlare.scale.x;
            this.add(this.successFlare);
            this.successText = new Phaser.Text(this.game, this.successBox.centerX, this.successBox.centerY, LocalizationManager_37.LocalizationManager.Instance.text("pipes.success"), {
                fill: Game_39.Game.FONT_COLOUR_GREEN,
                font: Game_39.Game.DEFAULT_FONT,
                stroke: Game_39.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_39.Game.FONT_STROKE_LARGE,
                fontSize: Game_39.Game.FONT_SIZE_LARGE,
            });
            this.successText.anchor.set(0.5);
            this.add(this.successText);
            if (!this.isTutorialMode) {
                this.successBox.visible = false;
                this.successText.visible = false;
                this.successFlare.visible = false;
                this.handAnim.visible = false;
                this.target.visible = false;
                this.marker.visible = false;
                this.probe.visible = false;
                this.tileGroup.visible = false;
                this.readyButton = new CTAButton_6.CTAButton(this.game, 'READY', Game_39.Game.FONT_SIZE_MEDIUM, true, this.shutdownSignal);
                this.readyButton.centerX = bg.centerX;
                this.readyButton.centerY = bg.top + bg.height * 0.35;
                this.add(this.readyButton);
            }
            ////////////////////////////////
            // INSTRUCTION TEXT
            ////////////////////////////////
            const instructionText = new Phaser.Text(this.game, bg.centerX, this.tiles[0].bottom + this.game.height * 0.02, LocalizationManager_37.LocalizationManager.Instance.text('pipes.tutorial'), {
                fill: Game_39.Game.DEFAULT_FONT_COLOUR,
                font: Game_39.Game.DEFAULT_FONT,
                stroke: Game_39.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_39.Game.FONT_STROKE_LARGE,
                fontSize: 50,
                align: 'left',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            instructionText.anchor.set(0.5, 0);
            this.add(instructionText);
            ////////////////////////////////
            // BOTTOM TEXT
            ////////////////////////////////
            this.bottomGroup = new Phaser.Group(this.game);
            this.add(this.bottomGroup);
            this.bottomTextFlare = new Phaser.Image(this.game, 0, 0, 'lightbeam');
            this.bottomTextFlare.anchor.set(0.5);
            this.bottomTextFlare.width = this.game.width * 0.8;
            this.bottomTextFlare.scale.y = this.bottomTextFlare.scale.x;
            this.bottomTextFlare.alpha = 0;
            this.bottomGroup.add(this.bottomTextFlare);
            this.bottomText = new Phaser.Text(this.game, 0, 0, LocalizationManager_37.LocalizationManager.Instance.text('pipes.tutorial_waiting'), {
                fill: Game_39.Game.FONT_COLOUR_BLUE,
                font: Game_39.Game.DEFAULT_FONT,
                stroke: Game_39.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_39.Game.FONT_STROKE_LARGE,
                fontSize: Game_39.Game.FONT_SIZE_LARGE,
                align: 'center',
                valign: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            this.bottomText.anchor.set(0.5);
            this.bottomText.alpha = 0;
            this.bottomGroup.add(this.bottomText);
            this.bottomGroup.x = this.game.width * 0.5;
            this.bottomGroup.y = -bg.height * 0.15;
            this.game.add.tween(this.bottomGroup.scale).to({ x: 1.05, y: 1.05 }, 650, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.codeButton = new CodeButton_4.CodeButton(this.game, this.shutdownSignal);
            this.codeButton.top = bg.top + this.game.height * 0.01;
            this.add(this.codeButton);
            this.codeButton.visible = false;
        }
        resetForInteraction() {
            this.tiles[1].angle = this.tiles[3].angle = 0;
            this.tiles[2].angle = 90;
            this.tiles[1].frame = this.tiles[2].frame = this.tiles[3].frame = 0;
            this.tiles[1].alpha = this.tiles[2].alpha = this.tiles[3].alpha = 1;
            this.probe.centerX = this.tiles[0].centerX;
            this.marker.centerX = this.tiles[1].centerX;
            this.marker.alpha = 1;
            this.target.centerX = this.tiles[1].centerX;
            this.target.centerY = this.tiles[1].centerY;
            for (let tween of this.targetTweens) {
                tween.resume();
            }
            this.target.alpha = 1;
            this.handAnim.top = this.tiles[1].centerY - this.handAnim.width * 0.25;
            this.handAnim.left = this.tiles[1].centerX - this.handAnim.width * 0.25;
            this.handAnim.alpha = 1;
            this.probe.alpha = 1;
            this.tileGroup.alpha = 1;
            this.successBox.alpha = 0;
            this.successFlare.alpha = 0;
            this.successText.alpha = 0;
            this.game.tweens.removeFrom(this.successFlare);
            this.bottomGroup.alpha = 1;
            this.bottomText.alpha = 0;
            this.bottomTextFlare.alpha = 0;
            ScaleUtil_30.scaleToBounds(this.probe, this.tiles[0].width * 0.8, this.tiles[0].height * 0.8);
        }
        playStageOneAnimations() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.bottomGroup).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                // Play tap sound
                this.game.sound.play("pipes-tap");
                // Fade out the marker and move it to the next tile
                PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.marker).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true)).then(() => {
                    this.marker.centerX = this.tiles[3].centerX;
                    this.game.add.tween(this.marker).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                });
                PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.target).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true)).then(() => {
                    this.target.centerX = this.tiles[3].centerX;
                    this.game.add.tween(this.target).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                });
                PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.handAnim).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true)).then(() => {
                    this.handAnim.left = this.tiles[3].centerX - this.handAnim.width * 0.25;
                    this.game.add.tween(this.handAnim).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                });
                const newFrame = Number(this.probe.frame) + 1;
                // Rotate, bounce and change the state of the first tile
                yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.tiles[1]).to({ angle: 90 }, 200, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.bounceTile(this.tiles[1]);
                this.tiles[1].frame = newFrame;
                yield PromiseUtil_46.PromiseUtil.time(this.game, 100, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Bounce and change the state of the second tile
                this.bounceTile(this.tiles[2]);
                this.tiles[2].frame = newFrame;
                this.tiles[3].inputEnabled = true;
            });
        }
        playStageTwoAnimations() {
            return __awaiter(this, void 0, void 0, function* () {
                // Play tap sound
                this.game.sound.play("pipes-tap");
                // Fade out the marker for good.
                this.game.add.tween(this.marker).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.handAnim).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.target).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true)
                    .onComplete.add(() => {
                    for (let tween of this.targetTweens) {
                        tween.pause();
                    }
                });
                // Rotate, bounce and change the state of the third tile
                yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.tiles[3]).to({ angle: 90 }, 200, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.bounceTile(this.tiles[3]);
                this.tiles[3].frame = Number(this.probe.frame) + 1;
            });
        }
        playCompletionAnimations() {
            return __awaiter(this, void 0, void 0, function* () {
                // Fade out the path tiles
                this.game.add.tween(this.tiles[1]).to({ alpha: 0.2 }, 400, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.tiles[2]).to({ alpha: 0.2 }, 400, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.tiles[3]).to({ alpha: 0.2 }, 400, Phaser.Easing.Linear.None, true);
                // Move, scale and fade out probe
                this.game.add.tween(this.probe.scale).to({ x: this.probe.scale.x * 1.2, y: this.probe.scale.y * 1.2 }, 800, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.probe).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 600);
                yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.probe).to({ x: this.tiles[4].centerX }, 800, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield Promise.all([
                    this.bounceTile(this.tiles[1]),
                    this.bounceTile(this.tiles[2]),
                    this.bounceTile(this.tiles[3]),
                    this.bounceTile(this.tiles[4])
                ]);
                if (this.isShutdown) {
                    return;
                }
                // Hide the tile group, we're done with it now.
                this.game.add.tween(this.tileGroup).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                // Fade in all the SUCCESS message components
                this.game.add.tween(this.successBox).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.successText).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.successFlare).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true);
                // Scale in the text and flare
                this.game.add.tween(this.successFlare.scale).from({ x: 0, y: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.successText.scale).from({ x: 0, y: 0 }, 500, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Passively animate the success flare once it's faded & scaled in
                this.game.add.tween(this.successFlare).to({ alpha: 0.7 }, 550, Phaser.Easing.Linear.None, true, 0, Infinity, true);
            });
        }
        bounceTile(tile, amount = 1.25) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(tile.scale).to({ x: tile.scale.x * amount, y: tile.scale.y * amount }, 100, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        set playerName(name) {
            this.nameText.text = name;
        }
        set playerIndex(index) {
            this.tapTile.frame = this.probe.frame = index % this.game.cache.getFrameCount("pipes-vessels");
        }
        showThenListen() {
            return __awaiter(this, void 0, void 0, function* () {
                this.resetForInteraction();
                this.visible = true;
                if (this.isTop) {
                    yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                if (this.isShutdown) {
                    return;
                }
                this.codeButton.visible = true;
                this.game.add.tween(this.codeButton).from({ right: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield this.play();
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isTutorialMode) {
                    this.tiles[1].inputEnabled = true;
                    yield PromiseUtil_46.PromiseUtil.signal(this.tiles[1].events.onInputDown, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.playStageOneAnimations();
                    if (this.isShutdown) {
                        return;
                    }
                    yield PromiseUtil_46.PromiseUtil.signal(this.tiles[3].events.onInputDown, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.playStageTwoAnimations();
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.playCompletionAnimations();
                    if (this.isShutdown) {
                        return;
                    }
                }
                else {
                    yield this.readyButton.show();
                    if (this.isShutdown) {
                        return;
                    }
                    yield PromiseUtil_46.PromiseUtil.signal(this.readyButton.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.readyButton.hide();
                    if (this.isShutdown) {
                        return;
                    }
                    this.bottomGroup.centerY = this.readyButton.centerY;
                }
                // Show text "waiting for other player"
                this.bottomText.alpha = 1;
                this.bottomTextFlare.alpha = 1;
                this.game.add.tween(this.bottomGroup).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isTop) {
                    yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                this.visible = false;
            });
        }
    }
    exports.PipeHalfDialog = PipeHalfDialog;
});
define("minigame/pipes/Tile", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Direction;
    (function (Direction) {
        Direction[Direction["UP"] = 0] = "UP";
        Direction[Direction["RIGHT"] = 90] = "RIGHT";
        Direction[Direction["DOWN"] = 180] = "DOWN";
        Direction[Direction["LEFT"] = 270] = "LEFT";
    })(Direction = exports.Direction || (exports.Direction = {}));
    class Tile {
        constructor() {
            this.isSinkTile = false;
            this.angle = 0;
            this.correctAngle = 0; // The solution angle for this tile
            this.canRotate = true;
            this.isOccupied = false;
        }
    }
    exports.Tile = Tile;
    class CornerTile extends Tile {
        constructor() {
            super();
            this.assetTag = "pipes-tile-corner";
        }
        atCorrectAngle() {
            // Corner has only one possible solution. Round to ensure correctness.
            return Math.round(this.angle) === Math.round(this.correctAngle);
        }
        get conduits() {
            switch (this.angle) {
                case 360:
                case 0: {
                    return [Direction.UP, Direction.RIGHT];
                }
                case 90: {
                    return [Direction.RIGHT, Direction.DOWN];
                }
                case 180: {
                    return [Direction.DOWN, Direction.LEFT];
                }
                case 270: {
                    return [Direction.LEFT, Direction.UP];
                }
                default: {
                    return [];
                }
            }
        }
    }
    exports.CornerTile = CornerTile;
    class NullTile extends Tile {
        constructor() {
            super();
            this.assetTag = "pipes-tile-null";
            this.canRotate = false;
        }
        atCorrectAngle() {
            // Nulls don't really rotate, always correct
            return true;
        }
        get conduits() {
            return [];
        }
    }
    exports.NullTile = NullTile;
    class StraightTile extends Tile {
        constructor() {
            super();
            this.assetTag = "pipes-tile-straight";
        }
        atCorrectAngle() {
            // Straight tiles can be solved in two ways
            let currentAngle = Math.round(this.angle % 180);
            return currentAngle === Math.round(this.correctAngle);
        }
        get conduits() {
            switch (this.angle) {
                case 360:
                case 180:
                case 0: {
                    return [Direction.UP, Direction.DOWN];
                }
                case 270:
                case 90: {
                    return [Direction.RIGHT, Direction.LEFT];
                }
                default: {
                    return [];
                }
            }
        }
    }
    exports.StraightTile = StraightTile;
    class TapTile extends Tile {
        constructor() {
            super();
            this.assetTag = "pipes-tile-tap";
            this.canRotate = false;
            this.isOccupied = true;
        }
        atCorrectAngle() {
            // Taps has only one possible solution
            return Math.round(this.angle) === Math.round(this.correctAngle);
        }
        get conduits() {
            switch (this.angle) {
                case 360:
                case 0: {
                    return [Direction.UP];
                }
                case 90: {
                    return [Direction.RIGHT];
                }
                case 180: {
                    return [Direction.DOWN];
                }
                case 270: {
                    return [Direction.LEFT];
                }
                default: {
                    return [];
                }
            }
        }
    }
    exports.TapTile = TapTile;
    class SinkTile extends Tile {
        constructor() {
            super();
            this.isSinkTile = true;
            this.assetTag = "pipes-tile-sink";
            this.canRotate = false;
            // Makes the code more robust
            this.angle = 0;
            this.correctAngle = 0;
        }
        atCorrectAngle() {
            // Always at a correct angle
            return true;
        }
        get conduits() {
            return [Direction.UP, Direction.DOWN, Direction.LEFT, Direction.RIGHT];
        }
    }
    exports.SinkTile = SinkTile;
});
define("minigame/pipes/PlayerBox", ["require", "exports", "switchback-common/util/TextUtil", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "component/map/PlanetView", "logic/MapController"], function (require, exports, TextUtil_18, Game_40, LocalizationManager_38, PromiseUtil_47, PlanetView_2, MapController_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayerBox extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.box = new Phaser.Image(this.game, this.game.width * 0.5, 0, "pipe-box");
            this.add(this.box);
            this.box.anchor.set(0.5, 0);
            this.box.width = this.game.width * 0.86;
            this.box.scale.y = this.box.scale.x;
            this.planet = null;
            this.planetLabel = new Phaser.Text(this.game, 0, 0, '', {
                font: Game_40.Game.DEFAULT_FONT,
                fontSize: Game_40.Game.FONT_SIZE_SMALL,
                fill: Game_40.Game.DEFAULT_FONT_COLOUR,
                align: "left"
            });
            this.planetLabel.anchor.set(0);
            this.planetLabel.y = this.box.height * 0.73;
            this.planetLabel.x = this.box.left + this.box.width * 0.15;
            this.add(this.planetLabel);
            let textsLeft = this.box.left + this.box.width * 0.36;
            this.playerNameText = new Phaser.Text(this.game, textsLeft, this.box.top + this.box.height * 0.25, '', {
                font: Game_40.Game.DEFAULT_FONT,
                fontSize: TextUtil_18.TextUtil.getFontSize(this.game.width, 23),
                fill: Game_40.Game.FONT_COLOUR_YELLOW,
                align: "left"
            });
            this.add(this.playerNameText);
            this.textField = new Phaser.Text(this.game, textsLeft, this.box.top + this.box.height * 0.47, LocalizationManager_38.LocalizationManager.Instance.text("pipes.description"), {
                font: Game_40.Game.DEFAULT_FONT,
                fontSize: TextUtil_18.TextUtil.getFontSize(this.game.width, 13),
                fill: Game_40.Game.DEFAULT_FONT_COLOUR,
                wordWrap: true,
                wordWrapWidth: this.box.width * 0.6,
                align: 'left'
            });
            this.add(this.textField);
            // create the group for the success popup
            this.successGroup = new Phaser.Group(this.game, this);
            this.successGroup.position.set(this.box.left + this.box.width * 0.6, this.box.height * 0.57);
            this.successGroup.alpha = 0;
            // here's the flare
            this.successFlare = new Phaser.Image(this.game, 0, 0, "pipe-box-flare");
            this.successFlare.anchor.set(0.5);
            this.successFlare.position.set(0.5);
            this.successGroup.add(this.successFlare);
            // here's the text
            this.successText = new Phaser.Text(this.game, 0, 0, LocalizationManager_38.LocalizationManager.Instance.text("pipes.success").toUpperCase(), {
                font: Game_40.Game.DEFAULT_FONT,
                fontSize: this.box.width * 0.08125,
                fill: Game_40.Game.FONT_COLOUR_GREEN,
                wordWrap: true,
                wordWrapWidth: this.box.width * 0.6,
                align: 'center',
                stroke: "black",
                strokeThickness: this.game.width * 0.012
            });
            this.successText.anchor.set(0.5);
            this.successText.position.set(0.5);
            let shadowSize = this.game.width * 0.01;
            this.successText.setShadow(shadowSize, shadowSize, 'rgba(0, 0, 0, 1)', 0, true, true);
            this.successGroup.add(this.successText);
        }
        setPlanet(id) {
            return __awaiter(this, void 0, void 0, function* () {
                const fadeDuration = 500;
                if (this.planet !== null) {
                    // Fade out if possible
                    yield Promise.all([
                        PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.planet).to({ alpha: 0 }, fadeDuration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                        PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.planetLabel).to({ alpha: 0 }, fadeDuration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal)
                    ]).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    // DESTROY!! (planet only)
                    this.planet.destroy();
                }
                let data = MapController_2.MapController.Instance.planets.get(id);
                this.planet = new PlanetView_2.PlanetView(this.game, data);
                this.planet.alpha = 0;
                this.box.addChild(this.planet);
                this.planet.scale.set(0.8);
                this.planet.x = this.box.width * -0.3;
                this.planet.y = this.box.height * 0.55;
                this.planetLabel.text = "Planet: " + data.name;
                // it is necesary to have this here
                this.planetLabel.anchor.set(0);
                this.planetLabel.x = this.box.left + this.box.width * 0.05;
                // Fade In
                yield Promise.all([
                    PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.planet).to({ alpha: 1 }, fadeDuration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal),
                    PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.planetLabel).to({ alpha: 1 }, fadeDuration, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
            });
        }
        set player(playerName) {
            this.playerNameText.text = playerName;
            this.game.add.tween(this.playerNameText).to({ alpha: 0.1 }, 500, Phaser.Easing.Linear.None, true, 0, 3, true);
        }
        setText(msg) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.textField).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.textField.text = msg;
                yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.textField).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        // Shows the success animation
        showSuccess() {
            return __awaiter(this, void 0, void 0, function* () {
                // remove previous tweens
                this.game.tweens.removeFrom(this.successGroup);
                this.game.tweens.removeFrom(this.successFlare);
                this.game.tweens.removeFrom(this.successFlare.scale);
                this.game.tweens.removeFrom(this.textField);
                // hides previous text
                yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.textField).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true, 0), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.textField.text = "";
                // set up
                const duration = 500;
                this.successGroup.alpha = 0;
                this.successGroup.scale.set(0.2);
                this.successFlare.scale.set(0);
                this.successFlare.alpha = 1;
                // Do animations 
                // fade in group
                let tween = this.game.add.tween(this.successGroup)
                    .to({ alpha: 1 }, duration, Phaser.Easing.Linear.None, true);
                // scale group
                let tweenScale = this.game.add.tween(this.successGroup.scale)
                    .to({ x: 1, y: 1 }, duration, Phaser.Easing.Back.Out, true);
                // scale flare
                let tweenFlare = this.game.add.tween(this.successFlare.scale)
                    .to({ x: 1.4, y: 1.4 }, duration * 0.75, null, true);
                // fade out flare
                let tweenFlareOut = this.game.add.tween(this.successFlare)
                    .to({ alpha: 0 }, duration * 0.25);
                tweenFlare.chain(tweenFlareOut);
                // await animations
                // Note that awaiting for TweenFlare only waits for the first tween, not the chained one. That's intentional.
                // Though it actually makes not much of a difference anyway (as after this there's a 2000ms wait)
                yield Promise.all([
                    PromiseUtil_47.PromiseUtil.tween(this.game, tween, this.shutdownSignal),
                    PromiseUtil_47.PromiseUtil.tween(this.game, tweenScale, this.shutdownSignal),
                    PromiseUtil_47.PromiseUtil.tween(this.game, tweenFlare, this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Wait a little...
                yield PromiseUtil_47.PromiseUtil.time(this.game, 350, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Now hide
                yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.successGroup)
                    .to({ alpha: 0 }, duration, null, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                // And blame other player
                // Use this when implement the LOCK_PATHS_ASAP
                // this.textField.text = LocalizationManager.Instance.text("pipes.waiting");
                // await PromiseUtil.tween(this.game, this.game.add.tween(this.textField).to({alpha: 1}, 300, Phaser.Easing.Linear.None, true, 0));
            });
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.PlayerBox = PlayerBox;
});
define("minigame/pipes/TileImage", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_48) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /*
     * @author Jeronimo@CerebralFix.com
     */
    // Represents one of the tiles
    // Please note that the onInputEvent will trigger for one of its child sprites, 
    // so you need to access to its parent to get to this class (specifically to change the angle)
    class TileImage extends Phaser.Group {
        constructor(game, key, shutdownSignal) {
            super(game);
            // Hint image
            this.hintImg = null;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.filled = false;
            this.highlightTweens = [];
            this.hintTweens = null;
            this.rotateTween = null;
            this.hintImg = null;
            this.img = new Phaser.Image(this.game, 0, 0, key);
            this.img.anchor.set(0.5);
            let lightAsset = key + "-lights";
            this.numLightFrames = this.game.cache.getFrameCount(lightAsset);
            this.fillImg = new Phaser.Image(this.game, 0, 0, lightAsset, 0);
            this.fillImg.width = this.img.width;
            this.fillImg.height = this.img.height;
            this.fillImg.anchor.set(0.5);
            this.fillImg.alpha = 0;
            // Note the order
            this.add(this.img);
            this.add(this.fillImg);
        }
        // Stops the hinting animation, and destroy the hint image
        stopHint() {
            let wasHinting;
            // Clears previous tweens if any
            if (this.hintTweens) {
                wasHinting = true;
                for (let tween of this.hintTweens) {
                    this.game.tweens.remove(tween);
                }
                this.hintTweens = null;
            }
            else {
                wasHinting = false;
            }
            // Very important, so it doesn't stays in an inconsistent state
            this.img.angle = 0;
            this.fillImg.angle = 0;
            // The best place to check for null is here
            if (this.hintImg) {
                this.hintImg.destroy();
                this.hintImg = null;
            }
            return wasHinting;
        }
        // creates a new hint image
        createHintImg() {
            // Ensure to destroy the previous one first
            this.stopHint();
            this.hintImg = new Phaser.Image(this.game, 0, 0, 'pipes-tile-hint');
            this.hintImg.width = this.img.width;
            this.hintImg.height = this.img.height;
            this.hintImg.anchor.set(0.5);
            this.hintImg.alpha = 0;
            this.hintImg.angle = 0;
            this.addAt(this.hintImg, 0);
        }
        // The hint animation. Calling many times will reset the previous
        // Once triggered it will keep triggering until calling to stopHint
        hint() {
            return __awaiter(this, void 0, void 0, function* () {
                this.createHintImg();
                // Remove the previous one and create a new one
                const durationMS = 500; // Duration per loop
                const loops = 3;
                const scaleOriginal = 0.5;
                const scaleTo = scaleOriginal * 2;
                // Initialize the hint image
                this.hintImg.scale.set(scaleOriginal);
                // Do the animation of the hint (alpha, angle, scale)
                // Durations and loops tweaked so the whole animation is repeatable
                // `hintTweens` holds all the tweens, so they can be awaited
                this.hintTweens = [
                    this.game.add.tween(this.hintImg).to({ alpha: 1 }, durationMS / 2.0, Phaser.Easing.Linear.None, false, 0, loops, true),
                    this.game.add.tween(this.hintImg).to({ angle: 360 }, durationMS, Phaser.Easing.Linear.None, false, 0, loops),
                    this.game.add.tween(this.hintImg.scale).to({ x: scaleTo, y: scaleTo }, durationMS / 2.0, Phaser.Easing.Linear.None, false, 0, loops, true)
                ];
                // Now the tweens for the tiles, we need to tween and synchronize both the img and fillimg
                // So we will animate both at the same time, in parallel.
                const imgs = [this.img, this.fillImg];
                // The animation is complex and needs to repeat. So we will store a chain for each img/fillImg
                // we will keep the first tween on the chain, so we can start them all at the same time.
                // when we're ready (so it doesn't de-synchronize)
                // So the tweens above are also heads
                let tweenHeads = this.hintTweens.slice();
                for (let nextImage in imgs) {
                    const img = imgs[nextImage];
                    // last tween in the chain (for this img)
                    let tweenTail = null;
                    // For each loop , we will repeat the animation
                    for (let i = 0; i < loops; i++) {
                        // And the tweens for the actual tile,
                        // we need to synchronize and animate BOTH the img and the filling image
                        // Expected: 0 > -15 > 0 > 15 > 0
                        let angleTA = this.game.add.tween(img).to({ angle: this.img.angle - 15 }, durationMS / 4.0, Phaser.Easing.Linear.None, false, 0, 0, true);
                        let angleTB = this.game.add.tween(img).to({ angle: this.img.angle + 15 }, durationMS / 4.0, Phaser.Easing.Linear.None, false, 0, 0, true);
                        angleTA.chain(angleTB); // chain the second to the first part
                        // if it's the first tween, use it as head, otherwise chain it
                        if (tweenTail == null) {
                            tweenTail = angleTA;
                            tweenHeads.push(angleTA);
                        }
                        else {
                            tweenTail.chain(angleTA); // chain to the previous one
                        }
                        tweenTail = angleTB; // and store the last as the new tail
                        // it is very important to put all the tweens in the hintTweens, because if any is missing, then it will play even when calling 
                        // destroyHint, for the simple reason that it wont be stopped
                        this.hintTweens.push(angleTA);
                        this.hintTweens.push(angleTB);
                    }
                }
                // Create the promises in order to await them
                let promises = [];
                for (let tween of this.hintTweens) {
                    promises.push(PromiseUtil_48.PromiseUtil.tween(this.game, tween, this.shutdownSignal));
                }
                // start the heads finally
                for (const tween of tweenHeads) {
                    tween.start();
                }
                // Does the animation and then resets the scale
                // Keeps animating forever until `stopHint` gets called
                yield Promise.all(promises).then(this.hint.bind(this)).catch(() => { this.isShutdown = true; });
            });
        }
        // Returns the current filling color, 0 if unfilled
        getColor() {
            return this.filled ? Number(this.fillImg.frame) : 0;
        }
        // Sets a tile as highlighted or not
        highlight(active, playerNum = null, durationMS = 100) {
            return __awaiter(this, void 0, void 0, function* () {
                let newFrame = Number(this.fillImg.frame);
                if (playerNum !== null) {
                    newFrame = playerNum % this.numLightFrames;
                }
                // Skip if this is already set. 
                // The game actually relies on the fact that there's no animation delay in this case
                if (active === this.filled && this.fillImg.frame === newFrame) {
                    return;
                }
                // Set the flag to avoid multiple calls
                this.filled = active;
                this.fillImg.frame = newFrame;
                // Clears previous tweens if any
                if (this.highlightTweens) {
                    for (let tween of this.highlightTweens) {
                        this.game.tweens.remove(tween);
                    }
                }
                // The scale is re-set before and after the animation, 
                // because a previous animation could have been stopped before it ended
                const scaleTo = 1.1;
                const scaleOriginal = 1.0;
                this.scale.set(scaleOriginal);
                // Do both tweens in parallel, scale duration /2, because scale is a Yoyo and will get doubled
                // Store the *tween*, so they can be removed
                // I thought a bounce will look better but Linear looks fluid and nice
                this.highlightTweens = [
                    this.game.add.tween(this.scale).to({ x: scaleTo, y: scaleTo }, durationMS / 2.0, Phaser.Easing.Linear.None, true, 0, 0, true),
                    this.game.add.tween(this.fillImg).to({ alpha: this.filled ? 1 : 0 }, durationMS, Phaser.Easing.Linear.None, true, 0)
                ];
                // Create the promises in order to await them
                let promises = [];
                for (let tween of this.highlightTweens) {
                    promises.push(PromiseUtil_48.PromiseUtil.tween(this.game, tween, this.shutdownSignal));
                }
                // Does the animation and then resets the scale
                yield Promise.all(promises).catch(() => { this.isShutdown = true; });
                this.scale.set(scaleOriginal);
            });
        }
        // overrides inputEnabled, the actual sprite that handles it is the img
        set inputEnabled(value) {
            this.img.inputEnabled = value;
        }
        // same as inputs, takes the events of the img sprite, not the group. BEWARE
        get events() {
            return this.img.events;
        }
        rotateBy(a) {
            return __awaiter(this, void 0, void 0, function* () {
                // Remove previous rotation tween if any
                if (this.rotateTween) {
                    this.game.tweens.remove(this.rotateTween);
                }
                // Kills the hint animation if any
                this.stopHint();
                this.angle = this._angle; // ensure we start from the proper angle
                // store new destination angle
                // notice the use of this.angle, because Phaser's angle is actually a setter
                // that will wrap (hence modify) the literal value of the angle
                this._angle = this.angle + a;
                // now rotation is acid even though we have tweens :)
                // Animate the rotation
                this.rotateTween = this.game.add.tween(this).to({ angle: this._angle }, 100, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_48.PromiseUtil.tween(this.game, this.rotateTween, this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        // Sets the angle of the whole group immediately
        setAngle(a) {
            this.angle = this._angle = a;
        }
        // Gets the assigned angle
        getAngle() {
            return this._angle;
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.TileImage = TileImage;
});
define("minigame/pipes/PipeLogic", ["require", "exports", "minigame/pipes/Tile", "switchback-common/util/CollectionUtil"], function (require, exports, Tile_1, CollectionUtil_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    /**
     * This class is used to construct Tiles with.
     */
    class TileData {
    }
    exports.TileData = TileData;
    class PipeLevelData {
    }
    exports.PipeLevelData = PipeLevelData;
    class Coord {
        constructor(r, c) {
            this.row = r;
            this.col = c;
        }
    }
    exports.Coord = Coord;
    /**
     * This class contains all of the logic for the pipe game.
     */
    class PipeLogic {
        /**
         * Parse the data and populate a 2D grid, creating tiles of specific types and at specific angles.
         * Also finds and saves references to the Taps.
         * @param data Information about the grid configuration
         */
        parse(data) {
            this.grid = [];
            this.taps = [];
            this.pathLocked = [];
            this.planets = data.planets || [];
            data.tiles.forEach((row, rowIndex) => {
                this.grid[rowIndex] = [];
                this.pathLocked[rowIndex] = false;
                row.forEach((tileData, colIndex) => {
                    let tile;
                    switch (tileData.type) {
                        case -1: {
                            tile = new Tile_1.NullTile();
                            break;
                        }
                        case 0: {
                            this.sink = tile = new Tile_1.SinkTile();
                            break;
                        }
                        case 1: {
                            tile = new Tile_1.TapTile();
                            // Taps are a bit of a special case - they're always a 'head' to start, 
                            // which means they need to be assigned a fillingDirection earlier than other tiles
                            this.taps.push(tile);
                            tile.fillingDirection = tileData.angle;
                            break;
                        }
                        case 2: {
                            tile = new Tile_1.StraightTile();
                            break;
                        }
                        case 3: {
                            tile = new Tile_1.CornerTile();
                            break;
                        }
                    }
                    // Set the correct angle of the tile, used to know if the tile is at the correct rotation
                    // Sinks should be left untouched
                    if (!tile.isSinkTile) {
                        // get it, or set a random one.
                        tile.correctAngle = (tileData.angle === undefined) ? this.getRandomAngle() : tileData.angle;
                        // if it's a rotating tile, set a random current one
                        tile.angle = tile.canRotate ? this.getRandomAngle() : tile.correctAngle;
                    }
                    this.grid[rowIndex][colIndex] = tile;
                }, this);
            }, this);
        }
        // Returns the next tile that should be changed for a tapIndex. Or null if there is no such tile.
        getHint(tapIndex) {
            // get the current path
            const path = this.getPath(tapIndex);
            // better safe than sorry
            if (path.length === 0) {
                return null;
            }
            // Try to get one tile ahead because the last one might be correct, but the next one no
            // For that we check if the last tile is the sink (so we don't cross over the sink)
            const lastTile = path[path.length - 1];
            if (!lastTile.isSinkTile) {
                // try to get the next one just in case
                // it doesn't matter if its the wrong one
                // Because if the previous one was pointing the wrong direction, it will return at that point
                const nextTile = this.getConnectingTile(path[path.length - 1], false);
                if (nextTile) {
                    path.push(nextTile);
                }
            }
            else {
                path.pop(); // remove the sink from the path, so we don't check it. just to be sure
            }
            // Now traverse all the tiles, and test if they're in the correct position, 
            // If not, it will return
            for (let tile of path) {
                if (!tile.atCorrectAngle()) {
                    return tile;
                }
            }
            // If we've reached here, evey tile is in the correct position
            return null;
        }
        getRandomAngle() {
            const rnd = Math.random();
            if (rnd < 0.25) {
                return 0;
            }
            else if (rnd < 0.5) {
                return 90;
            }
            else if (rnd < 0.75) {
                return 180;
            }
            else {
                return 270;
            }
        }
        // Resets the "isOccupied" flag on all tiles to allow to walk them again.
        resetTiles() {
            for (let row of this.grid) {
                for (let tile of row) {
                    // I wanted to prescind of this flag but its actually used to calculate correctly 
                    // Which is the _next_ tile to connect to, because the prior will get this flag set
                    // it's used on many functions, mainly getConnectingTile, hence its dependants
                    tile.isOccupied = false;
                }
            }
        }
        /* Note: Due to the complexity and the fact that there's a ticket where probes handles individually
        there are 2 groups of functions,
            1) The individual functions for each probe/tap
            2) the groupal ones for the whole game.
        */
        /// 1) Individual Functions
        // Utility function that tells when a path is a winning path
        isWinningPath(path) {
            let lastTile = path[path.length - 1];
            return (lastTile && lastTile.isSinkTile);
        }
        // Returns the path for the current tap to the last tile it reaches (included (if possible)) the sink
        getPath(tapIndex) {
            // Reset the occupied flag, so the getConnectionTile works fine
            this.resetTiles();
            let path = [];
            // Simple queue walking
            let head = this.taps[tapIndex];
            while (head) {
                path.push(head);
                // If we've reached the sink, we're done
                if (head.isSinkTile) {
                    break;
                }
                // mark occupied only here (after the break)
                // so the sink doesn't get marked (this way other taps can reach into it too)
                head.isOccupied = true;
                // Try to get the next tile, and if it's correct, queue it for testing
                const connectingTile = this.getConnectingTile(head);
                if (connectingTile !== null && !connectingTile.isOccupied) {
                    head = connectingTile;
                }
                else {
                    head = null;
                }
            }
            return path;
        }
        // Returns an array of paths (array of tiles from the tap till the sink (if it reaches ))
        // ie [ [TapA, TileA1, TileA2, ...., SinkA], [TapB, TileB1, TileB2, ...., SinkB] ]
        getPaths() {
            let paths = [];
            for (let tapIndex in this.taps) {
                paths.push(this.getPath(Number(tapIndex)));
            }
            return paths;
        }
        /**
         * Finds and rotates a tile by 90 degrees CW
         * assuming the tile is a tile type that can be turned
         * and isn't occupied by anything.
         */
        rotateTile(row, col) {
            const tile = this.grid[row][col];
            if (tile && tile.canRotate) {
                tile.angle = (tile.angle += 90) % 360;
                return true;
            }
            return false;
        }
        /**
         * Ensure these two tiles do not connect. Rotate away the connecting tile in the direction.
         */
        rotateTileAway(row, col, direction) {
            const tile = this.grid[row][col];
            if (tile) {
                let turnCount = 0; // it takes a maximum of two turns to ensure that a tile doesn't match another
                let connectingTile;
                while (turnCount++ < 2 && (connectingTile = this.getConnectingTileTowards(new Coord(row, col), direction))) {
                    connectingTile.angle = (connectingTile.angle += 90) % 360;
                }
            }
        }
        /**
         * Finds adjacent connecting tiles if there are any. Will only look in the
         * directions this tile has a conduit in.
         * @param tile The tile to search around for adjacent connections
         * @param onlyMutual Only returns a tile if the given tile and the adyacent have both aligned conduits,
         * 		otherwise it will return a tile adyacent to the free conduit of the given tile
         */
        getConnectingTile(tile, onlyMutual = true) {
            // Get the coords of the current tile
            const coords = CollectionUtil_3.getIndexIn2DArray(this.grid, tile);
            // For each of THIS tile's conduits, try to move to the adjacent tile along that direction.
            // If we find any suitable connections, return that tile.
            for (let i = 0; i < tile.conduits.length; i++) {
                const connectedTile = this.getConnectingTileTowards(coords, tile.conduits[i], onlyMutual);
                if (connectedTile) {
                    return connectedTile;
                }
            }
            return null;
        }
        /**
         * It's not sufficient to get an array of adjacent tiles surrounding a tile and check for
         * connections between them. A tile with a LEFT conduit can connect to a tile with a RIGHT conduit -
         * but ONLY if that second tile is to the LEFT of the original tile. e.g
         * --> <-- joins up, <-- --> doesn't.
         *
         * This function allow us to quickly say "can I go left/right/up/down", checking for edges of the grid at the same time.
         * @param coords The coordinate of the given tile
         * @param direction The direction we intend to move to
         * @param onlyIfMutual Will return the tile only if both conduits are aligned, otherwise, if one of the free conduits of the given tile points to
         * 			another tile
         */
        getConnectingTileTowards(coords, direction, onlyIfMutual = true) {
            // The direction of the opposite tile that should connect from
            let fromDirection;
            // Coords to which we want to move
            let toCoords = new Coord(coords.row, coords.col);
            // initialize correctly
            switch (direction) {
                case Tile_1.Direction.DOWN:
                    fromDirection = Tile_1.Direction.UP;
                    toCoords.row++;
                    break;
                case Tile_1.Direction.UP:
                    fromDirection = Tile_1.Direction.DOWN;
                    toCoords.row--;
                    break;
                case Tile_1.Direction.LEFT:
                    fromDirection = Tile_1.Direction.RIGHT;
                    toCoords.col--;
                    break;
                case Tile_1.Direction.RIGHT:
                    fromDirection = Tile_1.Direction.LEFT;
                    toCoords.col++;
                    break;
            }
            // Try to get a tile for the given coords
            const tileToCheck = this.grid[toCoords.row] ? this.grid[toCoords.row][toCoords.col] : null;
            if (tileToCheck) {
                // Can we move into that tile?
                const canGoTo = (!tileToCheck.isOccupied || tileToCheck.isSinkTile);
                // Is that tile connected to this one?
                const canComeFrom = (!onlyIfMutual || tileToCheck.conduits.indexOf(fromDirection) !== -1);
                if (canGoTo && canComeFrom) {
                    tileToCheck.fillingDirection = direction;
                    return tileToCheck;
                }
            }
            return null;
        }
    }
    exports.PipeLogic = PipeLogic;
});
define("minigame/pipes/PipeState", ["require", "exports", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "switchback-common/util/MathUtil", "switchback-common/util/ScaleUtil", "switchback-common/util/CollectionUtil", "minigame/pipes/PipeHalfDialog", "config/SpaceMountainConfig", "StateNames", "minigame/pipes/Tile", "minigame/pipes/PlayerBox", "minigame/pipes/TileImage", "minigame/pipes/PipeLogic", "logic/GameController", "logic/SMAnalyticsManager", "state/SpaceGameBaseState", "component/dialog/BasicSpaceButtonDialog", "component/coderedeem/RedeemCodeDialog"], function (require, exports, LocalizationManager_39, PromiseUtil_49, MathUtil_3, ScaleUtil_31, CollectionUtil_4, PipeHalfDialog_1, SpaceMountainConfig_11, StateNames_21, Tile_2, PlayerBox_1, TileImage_1, PipeLogic_1, GameController_24, SMAnalyticsManager_21, SpaceGameBaseState_4, BasicSpaceButtonDialog_6, RedeemCodeDialog_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class PipeState extends SpaceGameBaseState_4.SpaceGameBaseState {
        init() {
            this.stateName = StateNames_21.StateNames.GAME_PIPES;
            super.init();
            this.roundTimeMs = 0;
            this.MAX_PLAYTIME_BEFORE_STUCK = 120000;
            this.logic = new PipeLogic_1.PipeLogic();
            this.levelsCompletedSuccessfully = 0;
            this.levelsToPlay = 0;
            this.levelsPlayed = 0;
            this.levelData = [];
            this.ships = [];
            this.walkPromise = null;
            // Means you can't play yet, will be reset upon calling this.startLevel
            this.isLevelComplete = true;
            // init player score vars
            let rounds = Math.min(4, GameController_24.GameController.Instance.players.length);
            this.playersSession = GameController_24.GameController.Instance.getPlayerSession(2, rounds, true);
            this.levelsPerTeam = GameController_24.GameController.Instance.players.length > 2 ? 1 : 2;
            this.currentRound = 0;
            // Initialize scores
            this.lastRoundTimeMS = 0;
            this.roundTimes = [];
            this.correctLevelsPerRound = [];
            for (let player in this.playersSession) {
                // Initialized only once per full game
                if (player) {
                    this.roundTimes.push(0);
                    this.correctLevelsPerRound.push(0);
                }
            }
            // not using .map trick for readability
            // This prevents the tile fill from rounding up
            // a pixel and peaking out from behind the pipe tiles
            this.game.renderer.renderSession.roundPixels = true;
        }
        preload() {
            super.preload();
            RedeemCodeDialog_6.RedeemCodeDialog.preload(this.game);
            this.game.load.json("pipe-levels", "assets/minigame/pipe/levels.json");
            this.game.load.image("pipes-background", "assets/minigame/pipe/background.png");
            // Tiles
            // nulls are needed to get the correct size in the grid
            this.game.load.image("pipes-tile-null", "assets/minigame/pipe/null-tile.png");
            this.game.load.image("pipes-tile-straight", "assets/minigame/pipe/tile-straight.png");
            this.game.load.image("pipes-tile-sink", "assets/minigame/pipe/tile-sink.png");
            this.game.load.image("pipes-tile-hint", "assets/minigame/pipe/tile-hint.png");
            this.game.load.image("pipes-tile-corner", "assets/minigame/pipe/tile-corner.png");
            this.game.load.spritesheet("pipes-tile-corner-lights", "assets/minigame/pipe/tile-corner-lights.png", 200, 200);
            this.game.load.spritesheet("pipes-tile-straight-lights", "assets/minigame/pipe/tile-straight-lights.png", 200, 200);
            this.game.load.spritesheet("pipes-tile-tap-lights", "assets/minigame/pipe/tile-tap-lights.png", 200, 200);
            this.game.load.image("pipes-tile-null-lights", "assets/minigame/pipe/null-tile.png");
            this.game.load.image("pipes-tile-sink-lights", "assets/minigame/pipe/null-tile.png");
            this.game.load.image("pipes-tile-tap", "assets/minigame/pipe/null-tile.png");
            // Tutorial
            this.game.load.image("pipes-tutorial-panel", "assets/minigame/pipe/pipes-tutorial-panel.png");
            this.game.load.image("pipes-tutorial-sm", "assets/minigame/pipe/tutorial-sm.png");
            this.game.load.image("pipes-tutorial-marker", "assets/minigame/pipe/tutorial-marker.png");
            this.game.load.image("pipes-success-box", "assets/minigame/pipe/success_border.png");
            this.game.load.image("lightbeam", "assets/common/lightbeam.png");
            this.game.load.spritesheet("pipes-tutorial-tiles", "assets/minigame/pipe/tutorial_tiles.png", 200, 200);
            this.game.load.spritesheet("pipes-tutorial-taps", "assets/minigame/pipe/tutorial_taps.png", 200, 200);
            this.game.load.atlas("pipes-hand-anim", "assets/common/anim-hand.png", "assets/common/anim-hand.json");
            this.game.load.image("pipes-tutorial-target", "assets/minigame/memory/target.png");
            this.game.load.spritesheet("pipes-vessels", "assets/minigame/pipe/probes.png", 108, 108);
            // PlayerBox
            this.game.load.image("pipe-box", "assets/minigame/pipe/box.png");
            this.game.load.image("pipe-box-flare", "assets/minigame/pipe/success_flare.png");
            // PlayerBox - Planets
            this.game.load.spritesheet("planet-base", "assets/map/planet-base.png", 400, 400);
            this.game.load.spritesheet("planet-textures", "assets/map/planet-textures.png", 200, 200);
            for (let i = 0; i < 10; i++) {
                this.game.load.spritesheet(`planet-feature-${i}`, `assets/map/features/feature-${i}.png`, 200, 200);
            }
            // Audio
            this.game.load.audio("pipes-tap", ["assets/sound/pipes/tap_pipe.m4a", "assets/sound/pipes/tap_pipe.ogg"]);
            this.game.load.audio("pipes-probe-launch-1", ["assets/sound/pipes/probe_launch_1.m4a", "assets/sound/pipes/probe_launch_1.ogg"]);
            this.game.load.audio("pipes-probe-launch-2", ["assets/sound/pipes/probe_launch_2.m4a", "assets/sound/pipes/probe_launch_2.ogg"]);
            this.game.load.audio("pipes-probe-launch-3", ["assets/sound/pipes/probe_launch_3.m4a", "assets/sound/pipes/probe_launch_3.ogg"]);
            this.game.load.audio("pipes-complete", ["assets/sound/memory/correct.m4a", "assets/sound/memory/correct.ogg"]);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.input.maxPointers = 2;
                const numPlays = GameController_24.GameController.Instance.state.getMinigamePlays(StateNames_21.StateNames.GAME_PIPES);
                // vary the hint delay with difficulty level
                if (numPlays === 0) {
                    this.hintDelayFirst = 500;
                    this.hintDelayGeneral = 2000;
                }
                else if (numPlays === 1) {
                    this.hintDelayFirst = 3000;
                    this.hintDelayGeneral = 5000;
                }
                else {
                    this.hintDelayFirst = 5000;
                    this.hintDelayGeneral = 10000;
                }
                // Shuffle the levels and select some for this session
                const allLevelData = this.game.cache.getJSON("pipe-levels").levels;
                let availableLevels;
                if (!SpaceMountainConfig_11.SpaceMountainConfig.Instance.forceAllPipesLevels) {
                    if (numPlays === 0 || numPlays === 1) {
                        // play 1+2 only use level 1 difficulty levels
                        availableLevels = allLevelData.filter(level => level.difficulty === 1);
                    }
                    else if (numPlays === 2 || numPlays === 3) {
                        // play 3+4 only use level 2 difficulty levels
                        availableLevels = allLevelData.filter(level => level.difficulty === 2);
                    }
                    else {
                        // level 5+ difficulty use all levels
                        availableLevels = allLevelData.slice(); // clone array as we don't want raw data shuffled
                    }
                    Phaser.ArrayUtils.shuffle(availableLevels);
                    this.levelsToPlay = this.playersSession.length * this.levelsPerTeam;
                }
                else {
                    this.levelsToPlay = allLevelData.length;
                    availableLevels = allLevelData;
                }
                // clone number of levels we need to actually use
                this.levelData = availableLevels.slice(0, this.levelsToPlay);
                this.mainGroup = new Phaser.Group(this.game);
                this.add.existing(this.mainGroup);
                /////////////////////
                // Background
                /////////////////////
                this.stage.backgroundColor = 0x000000;
                const background = new Phaser.Image(this.game, 0, 0, "pipes-background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.mainGroup.add(background);
                /////////////////////
                // Player Info Boxes
                /////////////////////
                this.topBox = new PlayerBox_1.PlayerBox(this.game, this.shutdownSignal);
                this.topBox.angle = 180;
                this.topBox.centerX = this.game.world.centerX;
                this.mainGroup.add(this.topBox);
                this.bottomBox = new PlayerBox_1.PlayerBox(this.game, this.shutdownSignal);
                this.bottomBox.centerX = this.game.world.centerX;
                this.mainGroup.add(this.bottomBox);
                // Create the ships sprites
                this.createShips();
                this.createLevel(this.levelsCompletedSuccessfully);
                this.topBox.bottom = this.tileGroup.top + this.tileImages[0][0].height * 0.9;
                this.bottomBox.top = this.tileGroup.bottom - this.tileImages[0][0].height * 0.9;
                const isTutorial = !GameController_24.GameController.Instance.hasCompletedMinigame(this.stateName);
                //////////////////////////
                // INTRO DIALOG
                //////////////////////////
                this.topDialog = new PipeHalfDialog_1.PipeHalfDialog(this.game, true, isTutorial, this.shutdownSignal);
                this.topDialog.angle = 180;
                this.topDialog.left = 0;
                this.topDialog.top = 0;
                this.add.existing(this.topDialog);
                this.topDialog.resetForInteraction();
                this.bottomDialog = new PipeHalfDialog_1.PipeHalfDialog(this.game, false, isTutorial, this.shutdownSignal);
                this.bottomDialog.left = 0;
                this.bottomDialog.bottom = this.game.height;
                this.add.existing(this.bottomDialog);
                this.bottomDialog.resetForInteraction();
                // Fix Z ordering (must be created in other order than display order)
                this.mainGroup.sendToBack(this.bottomBox);
                this.mainGroup.sendToBack(this.topBox);
                this.mainGroup.sendToBack(background);
                ///////////////////////////////
                // Done creating, overlay coming
                ///////////////////////////////
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.onLevelStart();
            });
        }
        // Stops and destroys the hint Timer
        destroyHintTimer() {
            if (this.hintTimerEvent) {
                this.hintTimerEvent = null;
            }
            if (this.hintTimer) {
                this.hintTimer.stop();
                this.hintTimer.destroy();
                this.hintTimer = null;
            }
        }
        // Re-start the hint timer, stopping the previous one
        restartHintTimer() {
            // Notice that the timer only gets reset after a tile is turned. This is important because hints now
            // stay on indefinitely, otherwise animations will get interrupted and look ugly.
            this.destroyHintTimer();
            this.hintTimer = this.game.time.create(true);
            // Do a simple add because showHints will re call this. Needed because AC says we need to change times after the 1st time
            this.hintTimerEvent = this.hintTimer.add(this.hintTimerDelay, this.showHints, this);
            this.hintTimer.start();
        }
        // Stops all hints, including the valid ones
        stopAllHints() {
            // In theory (and i've tested this) by the way i coded this, this is not 100% necessary.
            // because when a tile is turned, the hints are killed, and in theory it will only show the hint on a tile that
            // needs to be turned and until it needs to be turned. But it's better be safe and sorry (even though A-Ha disagrees)
            for (let row of this.logic.grid) {
                for (let tile of row) {
                    // If a tile is not in any path, it needs to be drained
                    const tileImg = this.tileImageForTile(tile);
                    if (tileImg) {
                        tileImg.stopHint();
                    }
                }
            }
        }
        // stops only invalid hints
        // used under a special case on onTileClicked
        // this is slower and more complicated, so use it only if necessary
        stopInvalidHints() {
            let hints = [];
            for (let tap = 0; tap < this.logic.taps.length; tap++) {
                // try to get a hint
                const tile = this.logic.getHint(tap);
                if (tile) {
                    hints.push(tile);
                }
            }
            for (let row of this.logic.grid) {
                for (let tile of row) {
                    if (hints.indexOf(tile) < 0) {
                        // If a tile is not in any path, it needs to be drained
                        const tileImg = this.tileImageForTile(tile);
                        if (tileImg.stopHint()) {
                            this.restartHintTimer();
                        }
                    }
                }
            }
        }
        // Try to show the hints, called by the hint timer
        showHints() {
            // clears all the invalid hints, including valid, which will be re-triggered
            this.stopAllHints();
            for (let tap = 0; tap < this.logic.taps.length; tap++) {
                // try to get a hint
                const tile = this.logic.getHint(tap);
                if (!tile) {
                    continue;
                }
                // animate it if so
                const tileImage = this.tileImageForTile(tile);
                tileImage.hint();
            }
            // after the 1st hint, change the time of the timer
            this.hintTimerDelay = this.hintDelayGeneral;
            // Keeps the timer running
            this.restartHintTimer();
        }
        // Creates the sprites for the ship. (They can be probes or the actual ship depending on the phase)
        createShips() {
            // Destroy previous if any
            if (this.ships) {
                for (let ship of this.ships) {
                    ship.destroy();
                }
            }
            this.ships = [];
            // How many probes are in the spritesheet
            let probeFrames = this.game.cache.getFrameCount("pipes-vessels");
            let players = this.playersSession[this.currentRound];
            for (let i in players) {
                let ship = new Phaser.Sprite(this.game, 0, 0, "pipes-vessels", Number(i) % probeFrames);
                this.ships.push(ship);
            }
            return this.ships;
        }
        update() {
            super.update();
            this.roundTimeMs += this.game.time.elapsedMS;
            if (this.roundTimeMs > this.MAX_PLAYTIME_BEFORE_STUCK && !this.isLevelComplete) {
                this.isLevelComplete = true;
                this.roundTimes[this.currentRound] = this.roundTimeMs;
                this.onGameTimeout();
            }
        }
        onGameTimeout() {
            return __awaiter(this, void 0, void 0, function* () {
                this.setInputEnabled(false);
                const timeoutDialog = new BasicSpaceButtonDialog_6.BasicSpaceButtonDialog(this.game, LocalizationManager_39.LocalizationManager.Instance.text('activity_timout.description'), LocalizationManager_39.LocalizationManager.Instance.text('activity_timout.button'), this.shutdownSignal);
                timeoutDialog.show();
                yield PromiseUtil_49.PromiseUtil.signal(timeoutDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                yield timeoutDialog.hide();
                if (timeoutDialog.isShutdown) {
                    return;
                }
                timeoutDialog.destroy();
                this.onGameOver();
            });
        }
        /**
         * Constructs a grid of tile images from level data
         */
        createLevel(levelIndex) {
            this.logic.parse(this.levelData[levelIndex]);
            // ensure that none of the taps and sinks have connected tiles so the game doesn't
            // start and end with connections
            // This assumes that the sinks and taps are always in the same place in the json files 
            // (which they currently are by design)
            this.logic.rotateTileAway(0, 4, Tile_2.Direction.DOWN);
            this.logic.rotateTileAway(3, 2, Tile_2.Direction.UP);
            this.logic.rotateTileAway(3, 2, Tile_2.Direction.RIGHT);
            this.logic.rotateTileAway(3, 2, Tile_2.Direction.DOWN);
            this.logic.rotateTileAway(3, 2, Tile_2.Direction.LEFT);
            this.logic.rotateTileAway(6, 0, Tile_2.Direction.UP);
            // Clean up old level
            if (this.tileGroup) {
                // Avoid destroying the ships, as they'll be reused
                for (let ship of this.ships) {
                    if (ship && ship.parent) {
                        ship.parent.removeChild(ship);
                    }
                }
                this.tileGroup.destroy();
            }
            this.tileGroup = new Phaser.Group(this.game);
            this.tileGroup.alpha = 0;
            this.tileImages = [];
            let xPos = 0;
            let yPos = 0;
            for (let row = 0; row < this.logic.grid.length; row++) {
                let rowHeight = 0;
                this.tileImages.push([]);
                for (let col = 0; col < this.logic.grid[row].length; col++) {
                    // Create the tileImg from the tile data
                    const tile = this.logic.grid[row][col];
                    const tileImg = new TileImage_1.TileImage(this.game, tile.assetTag, this.shutdownSignal);
                    tileImg.position.set(xPos, yPos);
                    tileImg.setAngle(tile.angle);
                    // Add click listeners if appropriate
                    if (tile.canRotate) {
                        tileImg.inputEnabled = true;
                        tileImg.events.onInputDown.add((tileImg) => {
                            this.onTileClicked(tileImg.parent, row, col);
                        }, this);
                    }
                    // Add the tile to our easy-access array and visible group
                    this.tileImages[row][col] = tileImg;
                    this.tileGroup.add(tileImg);
                    // Sink size is non-normal. This allow to have a different sized sink.
                    if (!tile.isSinkTile) {
                        // Move the position along for the next tile
                        xPos += tileImg.width;
                        rowHeight = Math.max(rowHeight, tileImg.height);
                    }
                    else {
                        xPos += rowHeight;
                    }
                }
                xPos = 0;
                yPos += rowHeight;
            }
            // Scale and position the populated tile group
            ScaleUtil_31.scaleToBounds(this.tileGroup, this.game.width * 0.9, this.game.height * 0.8);
            this.tileGroup.centerX = this.game.world.centerX;
            this.tileGroup.centerY = this.game.world.centerY;
            this.mainGroup.add(this.tileGroup);
        }
        // Does initialization when a round begins
        onLevelStart() {
            return __awaiter(this, void 0, void 0, function* () {
                this.isLevelComplete = false;
                this.roundTimeMs = 0;
                this.setMessages(LocalizationManager_39.LocalizationManager.Instance.text("pipes.description"));
                this.resetProbes();
                SMAnalyticsManager_21.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                SMAnalyticsManager_21.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                if (!SpaceMountainConfig_11.SpaceMountainConfig.Instance.skipTutorials) {
                    yield Promise.all([
                        this.topDialog.showThenListen(),
                        this.bottomDialog.showThenListen()
                    ]);
                    if (this.isShutdown) {
                        return;
                    }
                }
                yield Promise.all([
                    this.topDialog.hide(),
                    this.bottomDialog.hide()
                ]);
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_21.SMAnalyticsManager.Instance.trackGamePlay(this.stateName);
                // Does the first walk to light the taps and any connected tile
                this.refreshBoard();
                yield this.fadeTiles(true);
                if (this.isShutdown) {
                    return;
                }
                this.setInputEnabled(true);
                // Start round timer after tutorial dialog completed
                this.lastRoundTimeMS = this.game.time.now;
                // When a level starts, start the hint timer
                // First hint has another delay
                this.hintTimerDelay = this.hintDelayFirst;
                this.restartHintTimer();
            });
        }
        /**
         * When the level is over, move on to the next one if possible.
         * If we're out of levels, they've won!
         * TODO: This needs to change to more like "select x number of levels, play them all, then game won"
         */
        onLevelComplete(won) {
            return __awaiter(this, void 0, void 0, function* () {
                // Disables the input to avoid timing problems
                this.setInputEnabled(false);
                if (this.isLevelComplete) {
                    return;
                }
                this.isLevelComplete = true;
                // addPlayerTimes depends on time, as well as the "finished" level, so do it before anything else
                this.addPlayerTimes(won);
                this.destroyHintTimer();
                // Actually there's no way to lose. This could probably be removed in the future
                if (won) {
                    // Increment the docks made for the global screen at the end
                    this.levelsCompletedSuccessfully++;
                }
                else {
                    this.setMessages(LocalizationManager_39.LocalizationManager.Instance.text("pipes.level_failed"));
                }
                // go to next round
                this.currentRound++;
                if (this.currentRound >= this.playersSession.length) {
                    // start all over again
                    this.currentRound = 0;
                    // Note nothing needs to be done with scores
                    // As we loop the rounds, the times for several dockings will get added 
                }
                // Load next level
                this.levelsPlayed++;
                if (this.levelData[this.levelsPlayed]) {
                    this.setMessages(LocalizationManager_39.LocalizationManager.Instance.text("pipes.next_level"));
                    yield this.fadeTiles(false);
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.createLevel(this.levelsPlayed);
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.onLevelStart();
                }
                else {
                    this.onGameOver();
                }
            });
        }
        // Does the calculations for the player scores and times
        // @param won : Defaults to true. If the player has won the round.
        addPlayerTimes(won = true) {
            // Gets the elapsed time
            let elapsedMS = this.game.time.now - this.lastRoundTimeMS;
            // this.lastRoundTimeMS gets updated on onLevelStart due to pauses
            // Add it to the current players
            this.roundTimes[this.currentRound] += elapsedMS;
            // only count the correct rounds, (read comment in postScore to know why)
            if (won) {
                this.correctLevelsPerRound[this.currentRound] += 1;
            }
        }
        onGameOver() {
            return __awaiter(this, void 0, void 0, function* () {
                this.endGame();
                this.transitionToOutro();
            });
        }
        // Posts the score for the minigame and the players
        postScore(isInterrupted) {
            // Notice this is the same code as dockings almost, that's on purpose
            // Note : done this way on purpose to maintain similarity with asteroids, so we can merge that later
            // Scores for different levels are added together in the same index
            let totalTime = 0;
            // so we make a parTime depending on how many levels needs to be done
            // that way is an average
            for (let roundIndex in this.playersSession) {
                let roundSeconds = (this.roundTimes[roundIndex] * 0.001) || 0.00001;
                // 0.00001 to avoid divide by zero errors if something weird happens
                // * 0.001 converts to seconds
                totalTime += roundSeconds;
            }
            let rounds = this.playersSession.length;
            GameController_24.GameController.Instance.postMinigameScore(StateNames_21.StateNames.GAME_PIPES, totalTime);
            GameController_24.GameController.Instance.achievementManager.onEnergyBallsEndGame(totalTime / rounds);
        }
        /**
         * Enables/disables the input for the tiles.
         */
        setInputEnabled(enable) {
            // There can be problems with asyncs, walking the board and breaking the board
            // So the best places to use this function is on onLevelStart, onLevelComplete, onGameOver, etc (as it is now)
            // Sets the correct input taking into account if it can rotate or not
            // That way we avoid the hijack of the click for the Sink
            // which has a non-normal size. Also avoiding problems and reducing callbacks
            for (let row = 0; row < this.logic.grid.length; row++) {
                for (let col = 0; col < this.logic.grid[row].length; col++) {
                    const tile = this.logic.grid[row][col];
                    const tileImg = this.tileImages[row][col];
                    tileImg.inputEnabled = tile.canRotate && enable;
                }
            }
        }
        /**
         * Displays messages to players on both ends of the device
         * @param msg The message to display
         */
        setMessages(msg) {
            this.bottomBox.setText(msg);
            this.topBox.setText(msg);
        }
        // Utility to return a TileImage corresponding to a tile
        tileImageForTile(tile) {
            const coord = CollectionUtil_4.getIndexIn2DArray(this.logic.grid, tile);
            return this.tileImages[coord.row][coord.col];
        }
        // Flies one of the probes along a path (ie this.logic.getPaths()[0])
        flyProbe(path, probeIndex) {
            return __awaiter(this, void 0, void 0, function* () {
                // Fade out and pulsate the highlighted path
                let tweens = [];
                for (let tile of path) {
                    if (tile.isSinkTile) {
                        continue;
                    }
                    // Main animation for fade-out. All the tiles fades in parallel
                    const tileImage = this.tileImageForTile(tile);
                    let tween = this.game.add.tween(tileImage);
                    tileImage.alpha = 1;
                    tween.to({ alpha: 0.20 }, 300, Phaser.Easing.Linear.None, true);
                    tweens.push(PromiseUtil_49.PromiseUtil.tween(this.game, tween, this.shutdownSignal));
                    // Then they keep pulsating (in parallel)
                    let pulse = this.game.add.tween(tileImage);
                    tileImage.alpha = 0.2;
                    pulse.to({ alpha: 0.35 }, 900, Phaser.Easing.Linear.None, false, 0, -1, true);
                    tween.chain(pulse);
                    tween.start();
                }
                yield Promise.all(tweens).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.game.sound.play(`pipes-probe-launch-${this.game.rnd.integerInRange(1, 3)}`);
                //// Fly the actual probe
                // Take and remove the tap from the list
                let tap = path.shift();
                let lastDirection = tap.angle;
                // store the current position to be able to do correct animations
                let lastCoord = CollectionUtil_4.getIndexIn2DArray(this.logic.grid, tap);
                let ship = this.ships[probeIndex];
                // Loop the tiles on the path and make the probe follows
                for (let tile of path) {
                    // Gets the next tile to go to and its coords
                    let nextCoord = CollectionUtil_4.getIndexIn2DArray(this.logic.grid, tile);
                    let nextTile = this.tileImages[nextCoord.row][nextCoord.col];
                    let nextPos = { centerX: nextTile.centerX, centerY: nextTile.centerY };
                    // Direction it should point to
                    let nextDirection = null;
                    if (lastCoord.col < nextCoord.col) {
                        nextDirection = Tile_2.Direction.RIGHT;
                    }
                    else if (lastCoord.col > nextCoord.col) {
                        nextDirection = Tile_2.Direction.LEFT;
                    }
                    if (lastCoord.row < nextCoord.row) {
                        nextDirection = Tile_2.Direction.DOWN;
                    }
                    else if (lastCoord.row > nextCoord.row) {
                        nextDirection = Tile_2.Direction.UP;
                    }
                    // tween the rotation (if any)
                    if (lastDirection !== nextDirection) {
                        // Using lastDirection and nextDirection for getShorterAngleDif helps.
                        // But for the tween is necessary to use the ship.angle value (which can be in one of those weird phaser formats)
                        let angleDif = MathUtil_3.getShorterAngleDif(lastDirection, nextDirection);
                        this.game.add.tween(ship).to({ angle: ship.angle + angleDif }, 75, Phaser.Easing.Linear.None, true);
                    }
                    // Fade out if it reached the sink
                    if (tile.isSinkTile) {
                        this.game.add.tween(ship).to({ alpha: 0.0 }, 200, Phaser.Easing.Linear.None, true, 200);
                    }
                    // Tween the position
                    let tween = this.game.add.tween(ship).to(nextPos, 300, Phaser.Easing.Linear.None, true);
                    yield PromiseUtil_49.PromiseUtil.tween(this.game, tween, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    // Loop update
                    lastCoord = nextCoord;
                    lastDirection = nextDirection;
                }
                // Play completion sound
                this.game.sound.play('pipes-complete');
                let playerBox = [this.topBox, this.bottomBox][probeIndex];
                yield playerBox.showSuccess();
            });
        }
        // Animates the tiles, in or out
        fadeTiles(fadeIn = true) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_49.PromiseUtil.tween(this.game, this.game.add.tween(this.tileGroup).to({ alpha: fadeIn ? 1 : 0 }, 1000, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
            });
        }
        // Sets the probes back at the starting point, also setting the correct color (frame) for each player
        resetProbes() {
            return __awaiter(this, void 0, void 0, function* () {
                // Set the players names
                let boxes = [this.topBox, this.bottomBox];
                let dialogs = [this.topDialog, this.bottomDialog];
                let players = this.playersSession[this.currentRound];
                // Sets: player names, dialog players, planets
                let planetsPromises = [];
                for (let boxIndex in boxes) {
                    let playerIndex = players[boxIndex];
                    dialogs[boxIndex].playerIndex = playerIndex;
                    boxes[boxIndex].player = dialogs[boxIndex].playerName = GameController_24.GameController.Instance.players[playerIndex].name;
                    planetsPromises.push(boxes[boxIndex].setPlanet(this.logic.planets[boxIndex]));
                }
                yield Promise.all(planetsPromises);
                if (this.isShutdown) {
                    return;
                }
                // Fade out the probes first
                for (let ship of this.ships) {
                    ship.alpha = 0;
                }
                let probeFrames = this.game.cache.getFrameCount("pipes-vessels");
                // Now reset the position and angle, change color and fade in
                let tweens = [];
                for (let i in players) {
                    let tap = this.logic.taps[i];
                    let upwards = tap.angle === Tile_2.Direction.UP;
                    const tileImg = this.tileImageForTile(tap);
                    let ship = this.ships[i];
                    // set color for player, if its a probe
                    ship.frame = players[i] % probeFrames;
                    // scaleToBounds(ship, tileImg.width, tileImg.height);
                    ship.anchor.set(0.5);
                    ship.angle = tap.angle;
                    ship.centerX = tileImg.centerX;
                    // Tap's center is not exactly where the ship should go, so correct it
                    ship.centerY = tileImg.centerY + (tileImg.height * (upwards ? -0.105 : 0.09));
                    this.tileGroup.add(ship);
                    // Fade in probe
                    ship.alpha = 0;
                    let tween = this.game.add.tween(ship).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                    // Set planet
                    tweens.push(boxes[i].setPlanet(this.logic.planets[i]));
                    // Set player name
                    tweens.push(PromiseUtil_49.PromiseUtil.tween(this.game, tween, this.shutdownSignal));
                }
                yield Promise.all(tweens).catch(() => { });
            });
        }
        // Important entry point:
        // Tries to refresh the board, upon any change
        // if it was already refreshing, it will kill and sync to the previous one
        refreshBoard(fillSpeedMS = 200) {
            return __awaiter(this, void 0, void 0, function* () {
                // if there where a previous one, we want to kill it
                // Thanks for Joel's great idea. It's a while not an if, because this function 
                // can be triggered multiple times (specially when using multitouch)
                // that means that there can be 2, 3 or even more functions waiting
                // over the SAME walkPromise. Hence we must wait until EACH one ends
                while (this.walkPromise !== null) {
                    // flag to tell the previous to quit
                    this.boardChanged = true;
                    // wait for it to stop
                    yield this.walkPromise;
                    if (this.isShutdown) {
                        return;
                    }
                }
                // try again, and store it as the current one
                this.walkPromise = this.doRefreshBoard(fillSpeedMS);
                yield this.walkPromise;
                this.walkPromise = null;
            });
        }
        // This fixes the case when there are taps that gets connected to another tap
        fixMixedPaths(paths) {
            // Test the path against the other
            for (let i = 0; i < paths.length; i++) {
                let path = paths[i];
                // if it's an empty path continue
                if (path.length === 0) {
                    continue;
                }
                for (let j = 0; j < paths.length; j++) {
                    // Skip current path
                    if (i === j) {
                        continue;
                    }
                    let otherPath = paths[j];
                    // If they are the same length, continue
                    if (otherPath.length !== path.length) {
                        continue;
                    }
                    // If the ending and beginning taps are different, continue
                    if (path[0] !== otherPath[otherPath.length - 1]) {
                        continue;
                    }
                    // Where to divide the paths, by default the middle of the path
                    let breakPoint = path.length / 2.0;
                    // Now we traverse the path, and when the color of the path changes, then we split both
                    // Use the tap color as starting color
                    let color = this.tileImageForTile(path[0]).getColor();
                    for (let k in path) {
                        const tileImg = this.tileImageForTile(path[k]);
                        if (color !== tileImg.getColor()) {
                            breakPoint = Number(k);
                            break;
                        }
                    }
                    // Split the paths at the breaking point
                    path.splice(breakPoint);
                    otherPath.splice(otherPath.length - breakPoint);
                }
            }
        }
        // Important entry point. Actually refreshes the board.
        // Each time the board changes, it re-scans and evaluates its state
        // Triggers all the animations and flow changes
        // Will watch for the this.boardChanged flag to break
        doRefreshBoard(fillSpeedMS = 200) {
            return __awaiter(this, void 0, void 0, function* () {
                // Important to set this flag here. There's a REALLY TINY timing chance
                // that this flag will be set to false just after another call to refresh board sets it to true
                // hence skipping the flag. But we can't avoid that without OS support, and chances are it ain't gonna happen.
                // same for this.boardChanged
                this.boardChanged = false;
                // The path for each probe, can be complete, or not
                let paths = this.logic.getPaths();
                // Fix the edge case when 2 paths can be overlapped, connecting both taps together
                this.fixMixedPaths(paths);
                // There's a case when both paths can connect to each other
                // Tests for completion and locks each path
                let allComplete = true;
                // SWB-910 Lock paths asap.
                for (let pathIndex in paths) {
                    let path = paths[pathIndex];
                    if (this.logic.isWinningPath(path)) {
                        for (let tile of path) {
                            // Disable the rotation on a locked path
                            // relies on the fact that when a board gets created, it actually instantiate new tiles
                            tile.canRotate = false;
                            // do not lock here, this function _must_ be able to be interrupted
                        }
                    }
                    else {
                        // notice no break, all the paths should be scanned
                        allComplete = false;
                    }
                }
                // Checks for completion asap, to disable the inputs and avoid timing problems
                // (aka the player disconnects a path while the game is ending)
                // if finished, disable the input
                if (allComplete) {
                    this.setInputEnabled(false);
                }
                // Drain all the unconnected tiles, stores the animations in parallel
                // Traverses all the tiles. If a tile is not on the connected array, then it's drained
                let drainTweens = [];
                for (let row of this.logic.grid) {
                    for (let tile of row) {
                        let missing = true;
                        // There are one path per player, so we need to check in all of them
                        for (let chain of paths) {
                            if (chain.indexOf(tile) > -1) {
                                missing = false;
                            }
                        }
                        if (missing) {
                            // If a tile is not in any path, it needs to be drained
                            const tileImg = this.tileImageForTile(tile);
                            drainTweens.push(tileImg.highlight(false, null, fillSpeedMS));
                        }
                    }
                }
                // Check for flag. Note: Done before the await Promise.all on purpose
                if (this.boardChanged) {
                    return;
                }
                // Does the drain animations
                yield Promise.all(drainTweens);
                // Now fill/highlight the connected ones
                // tiles are awaited one by one, so it can be interrupted here
                const players = this.playersSession[this.currentRound];
                let pos = 0;
                let hasNext = true;
                while (hasNext) {
                    hasNext = false;
                    for (let chainIndex in paths) {
                        const chain = paths[chainIndex];
                        if (this.boardChanged) {
                            return;
                        }
                        if (pos < chain.length) {
                            hasNext = true;
                            let tileImage = this.tileImageForTile(chain[pos]);
                            yield tileImage.highlight(true, players[chainIndex], fillSpeedMS);
                            if (this.isShutdown) {
                                return;
                            }
                        }
                    }
                    pos++;
                }
                // animations are stored in a tween so they can be run in parallel
                let tweens = [];
                for (let pathIndex in paths) {
                    let path = paths[pathIndex];
                    // if a path is already locked, don't start the animation again
                    if (!this.logic.pathLocked[pathIndex] && this.logic.isWinningPath(path)) {
                        this.logic.pathLocked[pathIndex] = true;
                        tweens.push(this.flyProbe(path, Number(pathIndex)));
                    }
                }
                // Don't await here or the board will get frozen while one of the path is animating
                // but schedule closing the level once it finished
                Promise.all(tweens).then(() => {
                    // but if the board was modifying while i was flying, go back, let the next function
                    // do it
                    if (this.boardChanged) {
                        return;
                    }
                    if (allComplete) {
                        this.onLevelComplete(true);
                    }
                });
            });
        }
        /**
         * Tries to rotate a tile. If the rotation is successful
         * (determined by the logic) the view of the tile is updated.
         * @param tileImg The TileImage clicked
         * @param row row of the tile clicked
         * @param col column of the tile clicked
         */
        onTileClicked(tileImg, row, col) {
            return __awaiter(this, void 0, void 0, function* () {
                // Destroys the current timer and queue a new one
                // ensuring the X seconds of _inactivity_ before showing a hint
                // The order is important, as refreshBoard could trigger the whole level-end and return after showing the next level
                this.game.sound.play("pipes-tap");
                if (this.logic.rotateTile(row, col)) {
                    yield tileImg.rotateBy(90);
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.stopInvalidHints();
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.refreshBoard(); // after this point we might as well be in a whole new level (Wooooooooooooooa!)
                }
            });
        }
        transitionToOutro() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                GameController_24.GameController.Instance.finishGame();
                this.game.state.start(StateNames_21.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        shutdown() {
            this.logic = null;
            this.tileImages = null;
            this.tileGroup = null;
            this.ships = null;
            this.mainGroup = null;
            this.bottomBox = null;
            this.topBox = null;
            this.levelData = null;
            this.walkPromise = null;
            this.topDialog = null;
            this.bottomDialog = null;
            this.hintTimer = null;
            this.hintTimerEvent = null;
            this.game.cache.removeJSON("pipe-levels");
            this.game.cache.removeImage("pipes-background");
            // Tiles
            // nulls are needed to get the correct size in the grid
            this.game.cache.removeImage("pipes-tile-null");
            this.game.cache.removeImage("pipes-tile-straight");
            this.game.cache.removeImage("pipes-tile-sink");
            this.game.cache.removeImage("pipes-tile-hint");
            this.game.cache.removeImage("pipes-tile-corner");
            this.game.cache.removeImage("pipes-tile-corner-lights");
            this.game.cache.removeImage("pipes-tile-straight-lights");
            this.game.cache.removeImage("pipes-tile-tap-lights");
            this.game.cache.removeImage("pipes-tile-null-lights");
            this.game.cache.removeImage("pipes-tile-sink-lights");
            this.game.cache.removeImage("pipes-tile-tap");
            // Tutorial
            this.game.cache.removeImage("pipes-tutorial-panel");
            this.game.cache.removeImage("pipes-tutorial-sm");
            this.game.cache.removeImage("pipes-tutorial-marker");
            this.game.cache.removeImage("pipes-success-box");
            this.game.cache.removeImage("lightbeam");
            this.game.cache.removeImage("pipes-tutorial-tiles");
            this.game.cache.removeImage("pipes-tutorial-taps");
            this.game.cache.removeImage("pipes-hand-anim");
            this.game.cache.removeImage("pipes-tutorial-target");
            this.game.cache.removeImage("pipes-vessels");
            // PlayerBox
            this.game.cache.removeImage("pipe-box");
            this.game.cache.removeImage("pipe-box-flare");
            // PlayerBox - Planets
            this.game.cache.removeImage("planet-base");
            this.game.cache.removeImage("planet-textures");
            for (let i = 0; i < 10; i++) {
                this.game.cache.removeImage(`planet-feature-${i}`);
            }
            // Audio
            this.game.cache.removeSound("pipes-tap");
            this.game.cache.removeSound("pipes-probe-launch-1");
            this.game.cache.removeSound("pipes-probe-launch-2");
            this.game.cache.removeSound("pipes-probe-launch-3");
            this.game.cache.removeSound("pipes-complete");
            super.shutdown();
        }
    }
    exports.PipeState = PipeState;
});
define("minigame/docking/DockingHalfDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/button/CTAButton", "Game", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "component/coderedeem/CodeButton", "switchback-common/util/MotionUtil"], function (require, exports, PromiseUtil_50, CTAButton_7, Game_41, ScaleUtil_32, LocalizationManager_40, CodeButton_5, MotionUtil_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DockingTutorialAnimation extends Phaser.Group {
        constructor(game, scaleToHeight, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.phone = new Phaser.Image(this.game, 0, 0, "tutorial-phone", 0);
            ScaleUtil_32.scaleToBounds(this.phone, this.game.width, scaleToHeight);
            this.add(this.phone);
            if (MotionUtil_3.MotionUtil.isGyroAvailable) {
                this.animatingDockTarget = new Phaser.Point(this.phone.width * 0.423, this.phone.height * 0.647);
                this.animatingDockOrigin = new Phaser.Point(this.phone.width * 0.5, this.phone.height * 0.5);
                this.animatingDock = new Phaser.Image(this.game, 0, 0, "tutorial-triangle");
                this.animatingDock.anchor.set(0.5);
                this.animatingDock.scale.x = this.phone.scale.x;
                this.animatingDock.scale.y = this.phone.scale.y;
                this.animatingDock.centerX = this.animatingDockOrigin.x;
                this.animatingDock.centerY = this.animatingDockOrigin.y;
                this.add(this.animatingDock);
                this.animateInfographic();
            }
            else {
                this.phone.frame = 1;
            }
        }
        animateInfographic() {
            return __awaiter(this, void 0, void 0, function* () {
                let isShutdown = false;
                while (true) {
                    this.phone.frame = 0;
                    const triangleTween = this.game.add.tween(this.animatingDock)
                        .to({ centerX: this.animatingDockTarget.x, centerY: this.animatingDockTarget.y }, 350, Phaser.Easing.Sinusoidal.InOut, true, 250);
                    triangleTween.onComplete.add(() => {
                        this.game.add.tween(this.animatingDock.scale).to({ x: 0, y: 0 }, 200, Phaser.Easing.Sinusoidal.In, true, 200);
                    });
                    yield PromiseUtil_50.PromiseUtil.tween(this.game, triangleTween, this.shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        return;
                    }
                    yield PromiseUtil_50.PromiseUtil.time(this.game, 500, this.shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        return;
                    }
                    this.phone.frame = 1;
                    this.animatingDock.centerX = this.animatingDockOrigin.x;
                    this.animatingDock.centerY = this.animatingDockOrigin.y;
                    this.animatingDock.scale.x = this.phone.scale.x;
                    this.animatingDock.scale.y = this.phone.scale.y;
                    yield PromiseUtil_50.PromiseUtil.time(this.game, 450, this.shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        return;
                    }
                }
            });
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.DockingTutorialAnimation = DockingTutorialAnimation;
    class DockingHalfDialog extends Phaser.Group {
        constructor(game, isTop, shutdownSignal) {
            super(game);
            this.isTop = isTop;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            const bg = new Phaser.Image(this.game, 0, 0, 'eb-instruction-panel');
            bg.inputEnabled = true;
            bg.anchor.set(0, 1);
            bg.width = this.game.width;
            bg.height = this.game.height * 0.5;
            this.add(bg);
            this.codeButton = new CodeButton_5.CodeButton(this.game, this.shutdownSignal);
            this.codeButton.top = bg.top + this.game.height * 0.005;
            this.add(this.codeButton);
            this.codeButton.visible = false;
            this.readyButton = new CTAButton_7.CTAButton(this.game, LocalizationManager_40.LocalizationManager.Instance.text('buttons.ready'), Game_41.Game.FONT_SIZE_MEDIUM, true, this.shutdownSignal);
            this.readyButton.centerX = bg.centerX;
            this.readyButton.bottom = bg.bottom - this.game.height * 0.02;
            this.add(this.readyButton);
            this.nameText = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.15, '', {
                fill: Game_41.Game.FONT_COLOUR_BLUE,
                font: Game_41.Game.DEFAULT_FONT,
                stroke: Game_41.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_41.Game.FONT_STROKE_LARGE,
                fontSize: Game_41.Game.FONT_SIZE_MEDIUM,
                align: 'center'
            });
            this.nameText.anchor.set(0.5);
            this.add(this.nameText);
            const instructionText = new Phaser.Text(this.game, 0, 0, LocalizationManager_40.LocalizationManager.Instance.text("docking.intro_top_and_tail"), {
                fill: Game_41.Game.DEFAULT_FONT_COLOUR,
                font: Game_41.Game.DEFAULT_FONT,
                stroke: Game_41.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_41.Game.FONT_STROKE_LARGE,
                fontSize: Game_41.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            instructionText.centerX = bg.centerX;
            instructionText.top = this.nameText.bottom;
            this.add(instructionText);
            const animHeight = this.readyButton.top - instructionText.bottom;
            this.infographic = new DockingTutorialAnimation(this.game, animHeight, this.shutdownSignal);
            this.infographic.centerX = this.game.width * 0.5;
            this.infographic.top = instructionText.bottom;
            this.add(this.infographic);
        }
        set playerName(name) {
            this.nameText.text = name;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                if (this.isTop) {
                    yield PromiseUtil_50.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_50.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                if (this.isShutdown) {
                    return;
                }
                this.codeButton.visible = true;
                this.game.add.tween(this.codeButton).from({ right: 0 }, 300, Phaser.Easing.Linear.None, true);
                yield this.readyButton.show();
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_50.PromiseUtil.signal(this.readyButton.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield this.readyButton.hide();
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isTop) {
                    yield PromiseUtil_50.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_50.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                this.visible = false;
            });
        }
        destroy() {
            this.shutdownSignal = null;
            this.readyButton = null;
            this.nameText = null;
            this.infographic = null;
            super.destroy();
        }
    }
    exports.DockingHalfDialog = DockingHalfDialog;
});
define("minigame/docking/Dock", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Dock extends Phaser.Image {
        constructor(game, dockType, hitboxSize) {
            super(game, 0, 0, 'docking-shapes', `${dockType}-off`);
            this.dockType = dockType;
            this.anchor.set(0.5);
            this.hitboxSize = hitboxSize;
            this.halfHitboxSize = this.hitboxSize * 0.5;
            this.scannerFlare = new Phaser.Image(this.game, 0, 0, 'docking-scanner-flare');
            this.scannerFlare.anchor.set(0.5);
            this.scannerFlare.width = this.game.width * 0.13;
            this.scannerFlare.scale.y = this.scannerFlare.scale.x;
            this.initScannerFlareScale = this.scannerFlare.scale.x;
            this.addChild(this.scannerFlare);
        }
        get hitbox() {
            return new Phaser.Rectangle(this.worldPosition.x - this.halfHitboxSize, this.worldPosition.y - this.halfHitboxSize, this.hitboxSize, this.hitboxSize);
        }
        set active(active) {
            const suffix = active ? 'on' : 'off';
            this.frameName = `${this.dockType}-${suffix}`;
            this.game.tweens.removeFrom(this.scannerFlare);
            if (active) {
                this.game.add.tween(this.scannerFlare.scale).to({ x: this.initScannerFlareScale * 1.5, y: this.initScannerFlareScale * 1.5 }, 150, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            }
        }
        destroy() {
            this.scannerFlare = null;
            super.destroy();
        }
    }
    exports.Dock = Dock;
});
define("minigame/docking/DockingShip", ["require", "exports", "minigame/docking/Dock", "minigame/docking/DockingGameState"], function (require, exports, Dock_1, DockingGameState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DockingShip extends Phaser.Group {
        constructor(game) {
            super(game);
            this.base = new Phaser.Image(this.game, 0, 0, 'docking-docks');
            this.add(this.base);
            // Populate the dock areas based on the dock image
            const locations = [
                new Phaser.Point(this.width * 0.655, this.height * 0.66),
                new Phaser.Point(this.width * 0.655, this.height * 0.36),
                new Phaser.Point(this.width * 0.35, this.height * 0.78),
                new Phaser.Point(this.width * 0.355, this.height * 0.51),
                new Phaser.Point(this.width * 0.355, this.height * 0.23)
            ];
            const shapes = [
                DockingGameState_1.DockType.STAR,
                DockingGameState_1.DockType.DIAMOND,
                DockingGameState_1.DockType.TRIANGLE,
                DockingGameState_1.DockType.PENTAGON,
                DockingGameState_1.DockType.HEXAGON
            ];
            Phaser.ArrayUtils.shuffle(shapes);
            this.docks = new Map();
            locations.forEach((location, index) => {
                const shape = shapes[index];
                const dockShape = new Dock_1.Dock(this.game, shape, this.width * 0.1);
                dockShape.x = location.x;
                dockShape.y = location.y;
                dockShape.width = this.width * 0.25;
                dockShape.scale.y = dockShape.scale.x;
                dockShape.angle = -90;
                this.add(dockShape);
                this.docks.set(shape, dockShape);
            });
        }
        /**
         * One of the more important functions. Given a
         * particular dock type (e.g. the target dock type),
         * this function returns the rect bounds of that dock.
         * This can then be checked against to see if the reticle
         * is overlapping the target dock.
         */
        getTargetDockArea(dockTargetType) {
            return this.docks.get(dockTargetType).hitbox;
        }
        getTargetLocation(dockTargetType) {
            return this.docks.get(dockTargetType).position;
        }
        // Set all targets as active
        setTargetsActive() {
            for (let dock of this.docks.values()) {
                dock.active = true;
            }
        }
        setActiveTarget(dockTargetType) {
            this.docks.get(dockTargetType).active = true;
        }
        destroy() {
            this.base = null;
            this.docks = null;
            super.destroy();
        }
    }
    exports.DockingShip = DockingShip;
});
define("minigame/docking/Thruster", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_51) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * The fancy Thruster animation group that docks onto the spaceship
     * @author joel@cerebralfix.com
     */
    class Thruster extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.isActive = true;
            this.progress = 0;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.boosterFlare = new Phaser.Image(this.game, 0, 0, 'docking-booster-flare');
            this.boosterFlare.anchor.set(0.5);
            this.boosterFlare.width = this.game.width * 0.2;
            this.boosterFlare.scale.y = this.boosterFlare.scale.x;
            this.boosterFlare.alpha = 0;
            this.thruster = new Phaser.Image(this.game, 0, 0, 'docking-thruster', 8);
            this.thruster.anchor.set(0.5, 0.66);
            this.thruster.width = this.game.width * 0.2;
            this.thruster.scale.y = this.thruster.scale.x;
            this.thruster.angle = Thruster.ANGLE;
            this.thrusterOrigScale = this.thruster.scale.clone();
            this.tractorCone = new Phaser.Image(this.game, 0, 0, 'docking-tractor-cone');
            this.tractorCone.height = this.game.height * 0.7;
            this.tractorCone.width = this.thruster.width * 1.33;
            this.tractorConeOrigScale = this.tractorCone.scale.clone();
            this.tractorCone.anchor.set(0.5, 0.98);
            this.tractorCone.angle = Thruster.ANGLE;
            this.add(this.tractorCone);
            this.add(this.thruster);
            this.add(this.boosterFlare);
        }
        /**
         * Performs the final, locking-into-place animation
         */
        finalise(animationDurationMs) {
            return __awaiter(this, void 0, void 0, function* () {
                // This thruster should no longer be animating 
                // by itself in the update loop - deactivate it
                this.isActive = false;
                // Make sure it's in the final frame
                this.thruster.frame = Thruster.THRUSTER_FRAMES_FULL;
                yield PromiseUtil_51.PromiseUtil.tween(this.game, this.game.add.tween(this.thruster.scale).to({ x: this.thruster.scale.x * 1.4, y: this.thruster.scale.y * 1.4 }, animationDurationMs * 0.15, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.boosterFlareFlash(animationDurationMs * 0.2);
                // Initial lock on shake
                yield this.cameraShake(0.005, animationDurationMs * 0.2);
                if (this.isShutdown) {
                    return;
                }
                // Spin into place
                yield PromiseUtil_51.PromiseUtil.tween(this.game, this.game.add.tween(this.thruster).to({ angle: this.thruster.angle + 360 }, animationDurationMs * 0.3, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.boosterFlareFlash(animationDurationMs * 0.2);
                // Final confirmation shake
                yield this.cameraShake(0.005, animationDurationMs * 0.2);
                this.cleanup();
            });
        }
        boosterFlareFlash(durationMs) {
            this.game.add.tween(this.boosterFlare).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true, 0, 0, true);
            this.game.add.tween(this.boosterFlare.scale).to({ x: this.boosterFlare.scale.x * 1.3, y: this.boosterFlare.scale.y * 1.3 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
        }
        // Gets rid of the things we don't need anymore
        // (leaving the things we still need!)
        cleanup() {
            this.tractorCone.destroy();
            this.boosterFlare.destroy();
        }
        cameraShake(intensity, durationMS) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.camera.shake(intensity, durationMS);
                yield PromiseUtil_51.PromiseUtil.time(this.game, durationMS, this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        update() {
            super.update();
            if (!this.isActive) {
                return;
            }
            this.tractorCone.alpha = this.progress * (1 / Thruster.TRACTOR_BEAM_ALPHA_MAX);
            this.thruster.alpha = this.progress * (1 / Thruster.THRUSTER_ALPHA_MAX);
            this.tractorCone.scale.set(this.tractorConeOrigScale.x * (Thruster.TRACTOR_BEAM_SCALE_X_EMPTY + (Thruster.TRACTOR_BEAM_SCALE_X_DIFF * this.progress)), this.tractorConeOrigScale.y * (Thruster.TRACTOR_BEAM_SCALE_Y_EMPTY + (Thruster.TRACTOR_BEAM_SCALE_Y_DIFF * this.progress)));
            this.thruster.scale.set(this.thrusterOrigScale.x * (Thruster.THRUSTER_SCALE_EMPTY + (Thruster.THRUSTER_SCALE_DIFF * this.progress)));
            this.thruster.x = this.game.width * (Thruster.THRUSTER_POSITION_OFFSET_X_EMPTY + (Thruster.THRUSTER_POSITION_OFFSET_X_DIFF * this.progress));
            this.thruster.y = this.game.height * (Thruster.THRUSTER_POSITION_OFFSET_Y_EMPTY + (Thruster.THRUSTER_POSITION_OFFSET_Y_DIFF * this.progress));
            const frameProgressValue = Math.max((this.progress - Thruster.THRUSTER_FRAMES_START), 0);
            const actualFrameProgressValue = frameProgressValue / Thruster.THRUSTER_FRAMES_DURATION;
            this.thruster.frame = Math.floor(actualFrameProgressValue * Thruster.THRUSTER_FRAMES_FULL);
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    Thruster.THRUSTER_ALPHA_MAX = 0.3; // the progress value when the thruster is at full alpha
    Thruster.TRACTOR_BEAM_ALPHA_MAX = 0.25; // the progress value when the tractor cone is at full alpha
    Thruster.TRACTOR_BEAM_SCALE_X_FULL = 1;
    Thruster.TRACTOR_BEAM_SCALE_X_EMPTY = 2.5;
    Thruster.TRACTOR_BEAM_SCALE_X_DIFF = Thruster.TRACTOR_BEAM_SCALE_X_FULL - Thruster.TRACTOR_BEAM_SCALE_X_EMPTY;
    Thruster.TRACTOR_BEAM_SCALE_Y_FULL = 0;
    Thruster.TRACTOR_BEAM_SCALE_Y_EMPTY = 1;
    Thruster.TRACTOR_BEAM_SCALE_Y_DIFF = Thruster.TRACTOR_BEAM_SCALE_Y_FULL - Thruster.TRACTOR_BEAM_SCALE_Y_EMPTY;
    Thruster.THRUSTER_POSITION_OFFSET_Y_FULL = 0;
    Thruster.THRUSTER_POSITION_OFFSET_Y_EMPTY = -0.3; // in game.heights
    Thruster.THRUSTER_POSITION_OFFSET_Y_DIFF = Thruster.THRUSTER_POSITION_OFFSET_Y_FULL - Thruster.THRUSTER_POSITION_OFFSET_Y_EMPTY;
    Thruster.THRUSTER_POSITION_OFFSET_X_FULL = 0;
    Thruster.THRUSTER_POSITION_OFFSET_X_EMPTY = 0.3; // in game.widths
    Thruster.THRUSTER_POSITION_OFFSET_X_DIFF = Thruster.THRUSTER_POSITION_OFFSET_X_FULL - Thruster.THRUSTER_POSITION_OFFSET_X_EMPTY;
    Thruster.THRUSTER_SCALE_FULL = 1;
    Thruster.THRUSTER_SCALE_EMPTY = 2;
    Thruster.THRUSTER_SCALE_DIFF = Thruster.THRUSTER_SCALE_FULL - Thruster.THRUSTER_SCALE_EMPTY;
    Thruster.THRUSTER_FRAMES_FULL = 8; // how many frames to animate through
    Thruster.THRUSTER_FRAMES_START = 0.75; // when (in progress) to start animating through the frames
    Thruster.THRUSTER_FRAMES_DURATION = (1 - Thruster.THRUSTER_FRAMES_START);
    Thruster.ANGLE = 30;
    exports.Thruster = Thruster;
});
define("minigame/docking/HoldBar", ["require", "exports", "switchback-common/util/ScaleUtil", "Game", "component/ProgressBar"], function (require, exports, ScaleUtil_33, Game_42, ProgressBar_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HoldBar extends ProgressBar_2.ProgressBarWithTextFrame {
        constructor(game, holdString) {
            super(game);
            this.holdText = new Phaser.Text(game, 0, 0, holdString, {
                font: Game_42.Game.DEFAULT_FONT,
                fill: Game_42.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_42.Game.FONT_SIZE_LARGE,
                stroke: Game_42.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_42.Game.DEFAULT_STROKE_THICKNESS,
                align: 'center'
            });
            ScaleUtil_33.scaleToBounds(this.holdText, this.width * 0.55, this.height * 0.65);
            this.holdText.anchor.set(0.5);
            this.holdText.y = this.top + this.height * 0.175;
            this.holdText.centerX = this.centerX;
            this.add(this.holdText);
        }
    }
    exports.HoldBar = HoldBar;
});
define("minigame/docking/DockingButton", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DockingButton extends Phaser.Sprite {
        constructor(game) {
            super(game, 0, 0, 'docking-button');
            this.pressedSignal = new Phaser.Signal();
            this._isDown = false;
            this.inputEnabled = true;
            this.events.onInputDown.add(this.onButtonPressed, this);
            this.events.onInputUp.add(this.onButtonReleased, this);
        }
        get isDown() {
            return this._isDown;
        }
        onButtonPressed() {
            return __awaiter(this, void 0, void 0, function* () {
                // this.game.sound.play('magnet-click', 0.5);
                this.pressedSignal.dispatch(true);
                this._isDown = true;
            });
        }
        onButtonReleased() {
            return __awaiter(this, void 0, void 0, function* () {
                this._isDown = false;
                this.pressedSignal.dispatch(false);
            });
        }
        disable() {
            this.inputEnabled = false;
            this._isDown = false;
        }
        enable() {
            this.inputEnabled = true;
        }
        destroy() {
            this.pressedSignal.dispose();
            this.events.onInputDown.remove(this.onButtonPressed, this);
            this.events.onInputUp.remove(this.onButtonReleased, this);
            super.destroy();
        }
    }
    exports.DockingButton = DockingButton;
});
define("minigame/docking/DockingGameState", ["require", "exports", "logic/GameController", "component/dialog/BasicSpaceLandscapeDialog", "StateNames", "Game", "minigame/docking/DockingHalfDialog", "state/SpaceGameBaseState", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "minigame/docking/DockingShip", "minigame/docking/Thruster", "minigame/docking/HoldBar", "logic/SMAnalyticsManager", "component/coderedeem/RedeemCodeDialog", "switchback-common/util/MotionUtil", "minigame/docking/DockingButton"], function (require, exports, GameController_25, BasicSpaceLandscapeDialog_2, StateNames_22, Game_43, DockingHalfDialog_1, SpaceGameBaseState_5, LocalizationManager_41, PromiseUtil_52, DockingShip_1, Thruster_1, HoldBar_1, SMAnalyticsManager_22, RedeemCodeDialog_7, MotionUtil_4, DockingButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DockType {
    }
    DockType.STAR = 'star';
    DockType.DIAMOND = 'diamond';
    DockType.HEXAGON = 'hexagon';
    DockType.PENTAGON = 'pentagon';
    DockType.TRIANGLE = 'triangle';
    exports.DockType = DockType;
    /**
     * @author joel@cerebralfix.com
     */
    class DockingGameState extends SpaceGameBaseState_5.SpaceGameBaseState {
        constructor() {
            super(...arguments);
            /// Used for score and player management
            this.roundTimes = []; // Todo this could be simply an accumulator of "totalTime"
            this.isGamePlaying = false; // flag to set true when the game starts
        }
        init() {
            this.stateName = StateNames_22.StateNames.GAME_DOCKING;
            super.init();
            this.MAX_PLAYTIME_BEFORE_STUCK = 180000;
            this.dockProgress = 0;
            this.isSeekingTarget = false;
            this.showingProgress = false;
            this.canPlayDockFailSound = false;
            this.canPlayDockStartSound = true;
            this.timeTakenMS = 0;
            this.currentAttitude = new Phaser.Point(0, 0);
            const numPlays = GameController_25.GameController.Instance.state.getMinigamePlays(StateNames_22.StateNames.GAME_DOCKING);
            this.difficultyLevel = Math.min(numPlays, 2); // difficulty is based on number of plays and is caped at 3
            this.numSuccessfulDocksMade = 0;
            let numRounds = GameController_25.GameController.Instance.players.length;
            switch (GameController_25.GameController.Instance.players.length) {
                case 2: {
                    this.docksPerTeam = 5;
                    // If we don't specify that we only want one round, 
                    // getPlayerSession() will return two sets of pairs
                    numRounds = 1;
                    break;
                }
                case 3:
                case 4: {
                    this.docksPerTeam = 3;
                    break;
                }
                case 5:
                default: {
                    this.docksPerTeam = 2;
                    break;
                }
            }
            this.playersSession = GameController_25.GameController.Instance.getPlayerSession(2, numRounds, true);
            this.currentRound = 0;
            // Initialize scores
            this.roundTimes = [];
            for (let player in this.playersSession) {
                // Initialized only once per full game
                if (player) {
                    this.roundTimes.push(0);
                }
            }
            // not using .map trick for readability
        }
        preload() {
            super.preload();
            RedeemCodeDialog_7.RedeemCodeDialog.preload(this.game);
            this.game.load.image("docking-bg", "assets/minigame/docking/background_docking.jpg");
            this.game.load.image("docking-frame", "assets/minigame/docking/frame.png");
            this.game.load.image("docking-docks", "assets/minigame/docking/docks.png");
            this.game.load.image("docking-tractor-cone", "assets/minigame/docking/tractor_cone.png");
            this.game.load.image("docking-scanner-flare", "assets/minigame/docking/scanner-flare.png");
            this.game.load.image("docking-booster-flare", "assets/minigame/docking/booster-flare.png");
            this.game.load.image("docking-success-flare", "assets/minigame/docking/success-flare.png");
            this.game.load.atlas("docking-cursors", "assets/minigame/docking/cursors.png", "assets/minigame/docking/cursors.json");
            this.game.load.atlas("docking-shapes", "assets/minigame/docking/shapes.png", "assets/minigame/docking/shapes.json");
            this.game.load.atlas("docking-thruster", "assets/minigame/docking/thruster.png", "assets/minigame/docking/thruster.json");
            this.game.load.image("docking-button", "assets/minigame/docking/direction-button.png");
            // assets used for the CTA
            this.game.load.image("progress-bar-border", "assets/common/progress-bar-border.png");
            this.game.load.image("progress-bar-inner", "assets/common/progress-bar-inner.png");
            // load the sounds
            this.game.load.audio('docking-hold-start', ['assets/sound/docking/hold_start.m4a', 'assets/sound/docking/hold_start.ogg']);
            this.game.load.audio('docking-hold-fail', ['assets/sound/docking/hold_fail.m4a', 'assets/sound/docking/hold_fail.ogg']);
            this.game.load.audio('docking-hold-progress', ['assets/sound/docking/hold_progress.m4a', 'assets/sound/docking/hold_progress.ogg']);
            this.game.load.audio('docking-hold-success', ['assets/sound/docking/hold_success.m4a', 'assets/sound/docking/hold_success.ogg']);
            this.game.load.audio('docking-movement-loop', ['assets/sound/docking/movement_loop.m4a', 'assets/sound/docking/movement_loop.ogg']);
            // Instructions
            this.game.load.image('eb-instruction-panel', 'assets/minigame/energy-ball/eb-instruction-panel.jpg');
            this.game.load.image("tutorial-triangle", "assets/minigame/docking/tutorial_triangle.png");
            this.game.load.spritesheet("tutorial-phone", "assets/minigame/docking/tutorial_phone.png", 843, 363);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.currentTargetIndex = 0;
                this.dampModifier = 1; // damp value initialized to 1 because we don't want to apply anything yet
                // Shuffle all available targets
                this.targets = [DockType.STAR, DockType.DIAMOND, DockType.HEXAGON, DockType.PENTAGON, DockType.TRIANGLE];
                Phaser.ArrayUtils.shuffle(this.targets);
                ////////////////////////////////
                // Space Background
                ////////////////////////////////
                this.background = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, "docking-bg");
                this.background.width = this.game.width * 2;
                this.background.scale.y = this.background.scale.x;
                this.background.anchor.set(0.5);
                this.background.angle = -90;
                this.add.existing(this.background);
                ////////////////////////////////
                // Space Port Dock Group
                ////////////////////////////////
                this.ship = new DockingShip_1.DockingShip(this.game);
                this.ship.height = this.game.height * 1.6;
                this.ship.scale.x = this.ship.scale.y;
                this.ship.centerX = this.game.width * 0.5;
                this.ship.centerY = this.game.height * 0.5;
                this.add.existing(this.ship);
                ////////////////////////////////
                // Completed Thrusters - on top of the ship, behind the cursor
                ////////////////////////////////
                this.completedThrusters = new Phaser.Group(this.game);
                this.completedThrusters.scale = this.ship.scale;
                this.add.existing(this.completedThrusters);
                ////////////////////////////////
                // Visible Screen Cursor
                ////////////////////////////////
                this.dockReticle = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, "docking-cursors");
                this.dockReticle.height = this.game.height * 0.2;
                this.dockReticle.scale.x = this.dockReticle.scale.y;
                this.dockReticle.anchor.set(0.5);
                this.dockReticle.alpha = 0;
                this.dockReticle.angle = -90;
                this.origReticleScale = this.dockReticle.scale.x;
                this.add.existing(this.dockReticle);
                ////////////////////////////////
                // Completed Thrusters - on top of the cursor
                ////////////////////////////////
                this.activeThrusters = new Phaser.Group(this.game);
                this.activeThrusters.scale = this.ship.scale;
                this.add.existing(this.activeThrusters);
                // Set all targets to their active state
                this.ship.setTargetsActive();
                if (!MotionUtil_4.MotionUtil.isGyroAvailable) {
                    this.btnTopLeft = new DockingButton_1.DockingButton(this.game);
                    this.btnTopLeft.width = this.game.width * 0.3;
                    this.btnTopLeft.scale.y = this.btnTopLeft.scale.x;
                    this.add.existing(this.btnTopLeft);
                    this.btnTopRight = new DockingButton_1.DockingButton(this.game);
                    this.btnTopRight.width = this.game.width * 0.3;
                    this.btnTopRight.scale.y = this.btnTopRight.scale.x;
                    this.btnTopRight.scale.x = -this.btnTopRight.scale.x;
                    this.btnTopRight.left = this.game.width;
                    this.add.existing(this.btnTopRight);
                    this.btnBottomLeft = new DockingButton_1.DockingButton(this.game);
                    this.btnBottomLeft.width = this.game.width * 0.3;
                    this.btnBottomLeft.scale.y = -this.btnBottomLeft.scale.x;
                    this.btnBottomLeft.top = this.game.height;
                    this.add.existing(this.btnBottomLeft);
                    this.btnBottomRight = new DockingButton_1.DockingButton(this.game);
                    this.btnBottomRight.width = this.game.width * 0.3;
                    this.btnBottomRight.scale.x = -this.btnBottomRight.scale.x;
                    this.btnBottomRight.scale.y = this.btnBottomRight.scale.x;
                    this.btnBottomRight.left = this.game.width;
                    this.btnBottomRight.top = this.game.height;
                    this.add.existing(this.btnBottomRight);
                }
                ////////////////////////////////
                // Success Text + Flare
                ////////////////////////////////
                this.successFlare = new Phaser.Image(this.game, 0, 0, 'docking-success-flare');
                this.successFlare.anchor.set(0.5);
                this.successFlare.width = this.game.height;
                this.successFlare.scale.y = this.successFlare.scale.x;
                this.successFlare.angle = -90;
                this.successFlare.visible = false;
                this.add.existing(this.successFlare);
                this.successText = new Phaser.Text(this.game, 0, 0, LocalizationManager_41.LocalizationManager.Instance.text('docking.success'), Object.assign({}, Game_43.Game.DEFAULT_FONT_STYLE, { fontSize: Game_43.Game.FONT_SIZE_LARGE, fill: Game_43.Game.FONT_COLOUR_GREEN }));
                this.successText.anchor.set(0.5);
                this.successText.angle = -90;
                this.successText.visible = false;
                this.add.existing(this.successText);
                ////////////////////////////////
                // Invisible Cursor Rect (used for collisions)
                ////////////////////////////////
                const cursorHitboxSize = this.ship.width * 0.05;
                this.reticleBody = new Phaser.Rectangle(this.game.width * 0.5 - (cursorHitboxSize * 0.5), this.game.height * 0.5 - (cursorHitboxSize * 0.5), cursorHitboxSize, cursorHitboxSize);
                ////////////////////////////////
                // Whole Screen Frame
                ////////////////////////////////
                const frame = new Phaser.Image(this.game, 0, 0, 'docking-frame');
                frame.width = this.game.width;
                frame.height = this.game.height;
                frame.fixedToCamera = true;
                this.add.existing(frame);
                // Group that contains the progress through the current dock
                this.holdBar = new HoldBar_1.HoldBar(this.game, LocalizationManager_41.LocalizationManager.Instance.text('docking.hold'));
                this.holdBar.width = this.game.height * 0.4;
                this.holdBar.scale.y = this.holdBar.scale.x;
                this.holdBar.angle = -90;
                this.holdBar.right = this.game.width;
                this.holdBar.centerY = this.game.height * 0.5;
                this.holdBar.alpha = 0;
                this.add.existing(this.holdBar);
                ////////////////////////////////
                // Create the looping sounds
                ////////////////////////////////
                this.dockingMovementLoopSound = this.game.sound.add('docking-movement-loop', 1, true);
                this.dockingProgressLoopSound = this.game.sound.add('docking-hold-progress', 1, true);
                ////////////////////////////////
                // Create Intro Dialog
                ////////////////////////////////
                this.topDialog = new DockingHalfDialog_1.DockingHalfDialog(this.game, true, this.shutdownSignal);
                this.topDialog.angle = 180;
                this.topDialog.left = 0;
                this.topDialog.top = 0;
                this.add.existing(this.topDialog);
                this.bottomDialog = new DockingHalfDialog_1.DockingHalfDialog(this.game, false, this.shutdownSignal);
                this.bottomDialog.right = this.game.width;
                this.bottomDialog.bottom = this.game.height;
                this.add.existing(this.bottomDialog);
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                let names = this.getCurrentPlayersNames();
                this.showInstructions(names[0], names[1]);
            });
        }
        showInstructions(player1, player2) {
            return __awaiter(this, void 0, void 0, function* () {
                this.topDialog.playerName = player1;
                this.bottomDialog.playerName = player2;
                SMAnalyticsManager_22.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                yield Promise.all([
                    this.topDialog.show(),
                    this.bottomDialog.show()
                ]);
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_22.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                // Remove thrusters from previous round
                this.completedThrusters.removeAll();
                yield Promise.all([
                    this.topDialog.hide(),
                    this.bottomDialog.hide()
                ]);
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_22.SMAnalyticsManager.Instance.trackGamePlay(this.stateName);
                this.startGame();
            });
        }
        startGame() {
            this.isGamePlaying = true;
            this.canPlayDockFailSound = false;
            if (MotionUtil_4.MotionUtil.isGyroAvailable) {
                this.motionCallback = (result) => {
                    this.onMotionChanged(result);
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
            }
            this.dockProgress = 0;
            this.currentTargetIndex = (this.currentTargetIndex + 1) % this.targets.length;
            this.isSeekingTarget = true;
            this.currentDockingThruster = new Thruster_1.Thruster(this.game, this.shutdownSignal);
            this.currentDockingThruster.x = this.ship.getTargetLocation(this.target).x;
            this.currentDockingThruster.y = this.ship.getTargetLocation(this.target).y;
            this.activeThrusters.add(this.currentDockingThruster);
            this.updateReticle();
        }
        getCurrentPlayersNames() {
            if (this.playersSession[this.currentRound]) {
                return this.playersSession[this.currentRound].map(playerIndex => GameController_25.GameController.Instance.players[playerIndex].name);
            }
            else {
                return null;
            }
        }
        get target() {
            return this.targets[this.currentTargetIndex];
        }
        /**
         * Animates the reticle to switch to the new target
         */
        updateReticle() {
            this.dockReticle.frameName = this.target;
            // Fade cursor in and begin ping pong pulsing
            this.dockReticle.scale.set(5);
            PromiseUtil_52.PromiseUtil.tween(this.game, this.game.add.tween(this.dockReticle.scale).to({ x: this.origReticleScale, y: this.origReticleScale }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0)).then(() => {
                this.game.add.tween(this.dockReticle.scale).to({ x: this.origReticleScale * 1.1, y: this.origReticleScale * 1.1 }, 1000, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            });
            this.game.add.tween(this.dockReticle).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true, 0);
        }
        /**
         * Once a dock has finished, this progresses the game to the next dock
         * OR, if this player has completed all their docks, the next player
         * OR, if the game has run through all players, ends the game.
         */
        onDockComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                this.isGamePlaying = false;
                // play the success sound when docking completes
                this.game.sound.play('docking-hold-success');
                this.dockingMovementLoopSound.stop();
                // Stop looking for targets for a bit
                this.isSeekingTarget = false;
                // Reset the docking progress
                this.dockProgress = 0;
                // Increment the docks made
                this.numSuccessfulDocksMade++;
                // Show the animation
                yield this.showDockCompleteAnimation();
                if (this.isShutdown) {
                    return;
                }
                this.completedThrusters.add(this.currentDockingThruster);
                // If this pair has completed the number of docks per team
                if (this.numSuccessfulDocksMade >= this.docksPerTeam) {
                    // We're done with this round
                    this.currentRound++;
                    this.numSuccessfulDocksMade = 0;
                    // We need to switch pairs
                    let names = this.getCurrentPlayersNames();
                    if (names) {
                        yield this.showInstructions(names[0], names[1]);
                    }
                    else {
                        // Everyone has played, finish the game
                        this.onGameOver();
                        return;
                    }
                }
                else {
                    this.startGame();
                }
            });
        }
        showDockCompleteAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this.dockReticle);
                const completionAnimationDurationMs = 1500;
                this.game.add.tween(this.dockReticle.scale).to({ x: 10, y: 10 }, 750, Phaser.Easing.Linear.None, true, completionAnimationDurationMs * 0.3);
                this.game.add.tween(this.dockReticle).to({ alpha: 0 }, 750, Phaser.Easing.Linear.None, true, completionAnimationDurationMs * 0.3);
                // Move & make visible the success text + flare
                this.successText.centerX = this.currentDockingThruster.worldPosition.x;
                this.successText.centerY = this.currentDockingThruster.worldPosition.y;
                this.successFlare.centerX = this.currentDockingThruster.worldPosition.x;
                this.successFlare.centerY = this.currentDockingThruster.worldPosition.y;
                this.successFlare.visible = this.successText.visible = true;
                this.game.add.tween(this.successFlare).from({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true, completionAnimationDurationMs * 0.8);
                this.game.add.tween(this.successText).from({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true, completionAnimationDurationMs * 0.8);
                this.game.add.tween(this.successFlare.scale).from({ x: 0 }, 300, Phaser.Easing.Sinusoidal.In, true, completionAnimationDurationMs * 0.8);
                this.game.add.tween(this.successText.scale).from({ x: 0, y: 0 }, 400, Phaser.Easing.Back.Out, true, completionAnimationDurationMs * 0.8);
                yield this.currentDockingThruster.finalise(completionAnimationDurationMs);
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_52.PromiseUtil.time(this.game, 1000, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.successFlare.visible = this.successText.visible = false;
            });
        }
        update() {
            super.update();
            if (!this.isGamePlaying) {
                return;
            }
            if (!MotionUtil_4.MotionUtil.isGyroAvailable) {
                let isButtonPressed = false;
                const newAttitude = new Phaser.Point();
                if (this.btnTopLeft.isDown) {
                    isButtonPressed = true;
                    newAttitude.add(DockingGameState.BUTTON_MOTION_COEFFICIENT, 1 * DockingGameState.BUTTON_MOTION_COEFFICIENT);
                }
                if (this.btnTopRight.isDown) {
                    isButtonPressed = true;
                    newAttitude.add(-1 * DockingGameState.BUTTON_MOTION_COEFFICIENT, 1 * DockingGameState.BUTTON_MOTION_COEFFICIENT);
                }
                if (this.btnBottomLeft.isDown) {
                    isButtonPressed = true;
                    newAttitude.add(1 * DockingGameState.BUTTON_MOTION_COEFFICIENT, -1 * DockingGameState.BUTTON_MOTION_COEFFICIENT);
                }
                if (this.btnBottomRight.isDown) {
                    isButtonPressed = true;
                    newAttitude.add(-1 * DockingGameState.BUTTON_MOTION_COEFFICIENT, -1 * DockingGameState.BUTTON_MOTION_COEFFICIENT);
                }
                if (!isButtonPressed) {
                    this.currentAttitude.multiply(DockingGameState.BUTTON_MOVEMENT_DAMPENING, DockingGameState.BUTTON_MOVEMENT_DAMPENING);
                }
                else {
                    this.currentAttitude = newAttitude;
                }
            }
            if (this.isSeekingTarget) {
                // Only increase time if the player is actively playing
                this.timeTakenMS += this.game.time.elapsedMS;
                // Update each players time
                this.roundTimes[this.currentRound] += this.game.time.elapsedMS;
                // How much should the dockProgress change (either way)? This must be done here to be framerate independent.
                const progressChange = (1 / DockingGameState.DOCK_TIME_TO_COMPLETE_MS) * (this.game.time.elapsedMS);
                // If the reticle is in the target area, dockProgress is going to go up - make sure we're showing it
                if (Phaser.Rectangle.intersects(this.reticleBody, this.ship.getTargetDockArea(this.target))) {
                    // if the dock start sound flag has been set to true, play the sound, else don't play it to avoid it being constantly spammed while reticle is intersecting
                    if (this.canPlayDockStartSound) {
                        this.game.sound.play('docking-hold-start');
                        this.dockingProgressLoopSound.play(); // play the docking progression loop sound
                        this.dockingMovementLoopSound.stop();
                        this.canPlayDockStartSound = false;
                        this.canPlayDockFailSound = true; // set dock fail sound flag to true meaning next time it exits the bounds, it will play the fail sound
                    }
                    this.dampModifier = DockingGameState.LOCKED_ON_DAMP_MODIFIER[this.difficultyLevel]; // set a damp value to reduce tilting sensitivty
                    this.dockProgress = Math.min(this.dockProgress + progressChange, 1);
                    this.showProgress();
                }
                else {
                    // if the dock fail sound flag has been set to true, play the sound, else don't play it to avoid it being constantly spammed while reticle is not intersecting
                    if (this.canPlayDockFailSound) {
                        this.game.sound.play('docking-hold-fail');
                        this.dockingProgressLoopSound.stop(); // stop the docking progression loop sound
                        this.canPlayDockFailSound = false;
                        this.dockingMovementLoopSound.play();
                    }
                    // otherwise we're out of the target area, dock progress will be going down
                    this.dockProgress = Math.max(this.dockProgress - progressChange, 0);
                    this.dampModifier = 1; // reset damp value
                    this.canPlayDockStartSound = true; // set dock start sound flag to true meaning next time it enters the bounds, it will play the start sound
                }
                // If we hit 1, we completed a dock
                if (this.dockProgress >= 1) {
                    this.dockingProgressLoopSound.stop(); // stop the docking progression loop sound once we're done docking
                    this.onDockComplete();
                }
                // Update the progress bar scale regardless of any outcomes
                this.holdBar.progress = this.dockProgress;
                this.currentDockingThruster.progress = this.dockProgress;
            }
            if (this.dockProgress <= 0) {
                // If we hit 0, we have no progress to show - hide it
                this.hideProgress();
            }
            /**
             * Continue moving the scene around based on current attitude.
             * This ensures movement every frame, which looks smoother.
             */
            this.move(this.currentAttitude.x, this.currentAttitude.y);
            if (this.roundTimes[this.currentRound] > this.MAX_PLAYTIME_BEFORE_STUCK && this.isSeekingTarget) {
                this.isGamePlaying = false;
                this.onGameTimeout();
            }
        }
        onGameTimeout() {
            return __awaiter(this, void 0, void 0, function* () {
                const timeoutDialog = new BasicSpaceLandscapeDialog_2.BasicSpaceLandscapeDialog(this.game, LocalizationManager_41.LocalizationManager.Instance.text('activity_timout.description'), LocalizationManager_41.LocalizationManager.Instance.text('activity_timout.button'), Game_43.Game.FONT_SIZE_MEDIUM, this.shutdownSignal);
                timeoutDialog.show();
                yield PromiseUtil_52.PromiseUtil.signal(timeoutDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                yield timeoutDialog.hide();
                if (timeoutDialog.isShutdown) {
                    return;
                }
                timeoutDialog.destroy();
                this.endGame();
                this.transitionToHub();
            });
        }
        // If the progress group isn't already showing, fade it in.
        showProgress() {
            if (!this.showingProgress) {
                this.showingProgress = true;
                this.game.tweens.removeFrom(this.holdBar);
                this.game.add.tween(this.holdBar).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true, 0);
            }
        }
        // If the progress group is showing, fade it out.
        hideProgress() {
            if (this.showingProgress) {
                this.showingProgress = false;
                this.game.tweens.removeFrom(this.holdBar);
                this.game.add.tween(this.holdBar).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true, 0);
            }
        }
        /**
         * Receives motion events from the PlayAPI. Processes these to:
         * 1. Adjust for forehead orientation
         * 2. Add a dead zone based on GYRO_MINIMUM_THRESHOLD
         * 3. Multiply by DEVICE_MOTION_COEFFICIENT to make numbers more usable
         * 4. Invert to make it better feeling
         */
        onMotionChanged(result) {
            // Pull the values out, adjusting the roll for forehead orientation
            let roll = result.attitude.roll;
            let pitch = result.attitude.pitch;
            // Ignore values smaller than GYRO_MINIMUM_THRESHOLD
            if (Math.abs(roll) < DockingGameState.GYRO_MINIMUM_THRESHOLD) {
                roll = 0;
            }
            if (Math.abs(pitch) < DockingGameState.GYRO_MINIMUM_THRESHOLD) {
                pitch = 0;
            }
            // Amplify the values
            roll *= DockingGameState.GYRO_MOTION_COEFFICIENT[this.difficultyLevel];
            pitch *= DockingGameState.GYRO_MOTION_COEFFICIENT[this.difficultyLevel];
            // This is now our current attitude
            this.currentAttitude.set(-roll, -pitch);
        }
        /**
         * Called every frame. This moves the dock and BG around
         * with a lerped, clamped action.
         */
        move(x, y) {
            if (!this.isGamePlaying) {
                return;
            }
            // if the docking movement loop sound is not playing, start it
            if (!this.dockingMovementLoopSound.isPlaying) {
                this.dockingMovementLoopSound.play();
            }
            let shipXDest = this.ship.centerX + (x * DockingGameState.DOCK_MOVEMENT_COEFFICIENT * this.dampModifier);
            let shipYDest = this.ship.centerY + (y * DockingGameState.DOCK_MOVEMENT_COEFFICIENT * this.dampModifier);
            if (shipXDest - (this.ship.width * 0.5) > this.game.width * 0.5) {
                shipXDest = (this.game.width + this.ship.width) * 0.5;
            }
            else if (shipXDest + (this.ship.width * 0.5) < this.game.width * 0.5) {
                shipXDest = (this.game.width - this.ship.width) * 0.5;
            }
            if (shipYDest - (this.ship.height * 0.5) > this.game.height * 0.5) {
                shipYDest = (this.game.height + this.ship.height) * 0.5;
            }
            else if (shipYDest + (this.ship.height * 0.5) < this.game.height * 0.5) {
                shipYDest = (this.game.height - this.ship.height) * 0.5;
            }
            // Smooth the values because we don't get frequent enough motion events via the PlayAPI
            this.ship.centerX = Phaser.Math.linearInterpolation([this.ship.centerX, shipXDest], 0.05);
            this.ship.centerY = Phaser.Math.linearInterpolation([this.ship.centerY, shipYDest], 0.05);
            // TODO: Just make this follow the ship's position and stop calculating so many things
            const bgXDest = Phaser.Math.clamp(this.background.x + (x * DockingGameState.BACKGROUND_MOVEMENT_COEFFICIENT * this.dampModifier), 0, this.game.width);
            const bgYDest = Phaser.Math.clamp(this.background.y + (y * DockingGameState.BACKGROUND_MOVEMENT_COEFFICIENT * this.dampModifier), 0, this.game.height);
            this.background.x = Phaser.Math.linearInterpolation([this.background.x, bgXDest], 0.05);
            this.background.y = Phaser.Math.linearInterpolation([this.background.y, bgYDest], 0.05);
            // Ensure our thruster layers follow the ship
            this.completedThrusters.position = this.ship.position;
            this.activeThrusters.position = this.ship.position;
        }
        /**
         * 1. Sends off score
         * 2. Shows game over dialog
         * 3. Transitions to hub
         */
        onGameOver() {
            this.endGame();
            // Show the game over dialog
            const timeInSeconds = this.timeTakenMS * 0.001;
            const gameOverDialog = new BasicSpaceLandscapeDialog_2.BasicSpaceLandscapeDialog(this.game, LocalizationManager_41.LocalizationManager.Instance.textWithReplacements("docking.outro", { time_taken: timeInSeconds.toFixed(2) }), LocalizationManager_41.LocalizationManager.Instance.text("buttons.ok"), 60, this.shutdownSignal);
            this.add.existing(gameOverDialog);
            gameOverDialog.show();
            gameOverDialog.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                // Transition back to the hub when they've finished with the dialog
                gameOverDialog.hide();
                this.transitionToHub();
            }), this);
        }
        /**
         * Calculate their proportional score according to par time
         */
        postScore(isInterrupted) {
            // Note :  done this way on purpose to maintain similarity with asteroids, so we can merge that later
            let totalTime = 0;
            // Scores for different dockings are added together in the same index
            // so we make a parTime depending on how many dockings needs to be done
            // that way is an average
            for (let roundIndex in this.playersSession) {
                let roundSeconds = (this.roundTimes[roundIndex] * 0.001) || 0.00001;
                // 0.00001 to avoid divide by zero errors if something weird happens
                // * 0.001 converts to seconds
                totalTime += roundSeconds;
            }
            let rounds = this.playersSession.length;
            GameController_25.GameController.Instance.postMinigameScore(StateNames_22.StateNames.GAME_DOCKING, totalTime);
            GameController_25.GameController.Instance.achievementManager.onDockingEndGame(totalTime / rounds);
        }
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                console.log(`[${this.stateName}] transitionToHub()`);
                yield this.showOverlay();
                GameController_25.GameController.Instance.finishGame();
                this.game.state.start(StateNames_22.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        shutdown() {
            if (this.motionCallback) {
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                this.motionCallback = null;
            }
            if (this.btnTopLeft) {
                this.btnTopLeft.destroy();
                this.btnTopLeft = null;
                this.btnTopRight.destroy();
                this.btnTopRight = null;
                this.btnBottomLeft.destroy();
                this.btnBottomLeft = null;
                this.btnBottomRight.destroy();
                this.btnBottomRight = null;
            }
            this.background = null;
            this.ship = null;
            this.completedThrusters = null;
            this.activeThrusters = null;
            this.currentDockingThruster = null;
            this.successFlare = null;
            this.successText = null;
            this.dockReticle = null;
            this.reticleBody = null;
            this.topDialog = null;
            this.bottomDialog = null;
            this.holdBar = null;
            this.currentAttitude = null;
            this.dockingProgressLoopSound = null;
            this.dockingMovementLoopSound = null;
            this.game.cache.removeImage("docking-bg");
            this.game.cache.removeImage("docking-frame");
            this.game.cache.removeImage("docking-docks");
            this.game.cache.removeImage("docking-tractor-cone");
            this.game.cache.removeImage("docking-scanner-flare");
            this.game.cache.removeImage("docking-booster-flare");
            this.game.cache.removeImage("docking-success-flare");
            this.game.cache.removeImage("docking-cursors");
            this.game.cache.removeImage("docking-shapes");
            this.game.cache.removeImage("docking-thruster");
            // assets used for the CTA
            this.game.cache.removeImage("progress-bar-border");
            this.game.cache.removeImage("progress-bar-inner");
            // load the sounds
            this.game.cache.removeSound('docking-hold-start');
            this.game.cache.removeSound('docking-hold-fail');
            this.game.cache.removeSound('docking-hold-progress');
            this.game.cache.removeSound('docking-hold-success');
            this.game.cache.removeSound('docking-movement-loop');
            // Instructions
            this.game.cache.removeImage('eb-instruction-panel');
            this.game.cache.removeImage("tutorial-triangle");
            this.game.cache.removeImage("tutorial-phone");
            super.shutdown();
        }
    }
    /*
     * This is the amount that the sensitivity is decreased to while a shape is being locked on.
     * Between 0 and 1; 0 means no movement, 1 means full movement.
     * Has different values for different difficulties
     */
    DockingGameState.LOCKED_ON_DAMP_MODIFIER = [0.7, 0.8, 0.9];
    // How fast the dock and BG move
    DockingGameState.DOCK_MOVEMENT_COEFFICIENT = 1;
    DockingGameState.BACKGROUND_MOVEMENT_COEFFICIENT = 0.5;
    // Amount to multiple the motion events by to get it feelin' good
    DockingGameState.GYRO_MOTION_COEFFICIENT = [250, 400, 600];
    DockingGameState.BUTTON_MOTION_COEFFICIENT = 150;
    DockingGameState.BUTTON_MOVEMENT_DAMPENING = 0.9;
    // 'dead zone' - gyro readings under this value are ignored
    DockingGameState.GYRO_MINIMUM_THRESHOLD = 0.00;
    // How long each dock should be held for
    DockingGameState.DOCK_TIME_TO_COMPLETE_MS = 3500;
    exports.DockingGameState = DockingGameState;
});
define("minigame/energy-balls/EnergyBall", ["require", "exports", "minigame/energy-balls/EnergyBallState"], function (require, exports, EnergyBallState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * An energy ball is a sprite that moves up and down
     * and has its
     */
    class EnergyBall extends Phaser.Group {
        constructor(game, numTarget, difficulty) {
            super(game);
            this.ballCompleteSignal = new Phaser.Signal();
            this.currentSpeed = 0;
            this.count = 0;
            this.timeOfLastUpdate = 0;
            this.targetCount = numTarget;
            this.difficulty = difficulty;
            this.canisterImage = new Phaser.Image(game, 0, 0, `eb-canister-${numTarget}`, 0);
            this.canisterImage.anchor.set(0.5);
            this.canisterImage.width = this.game.width * 0.13;
            this.canisterImage.scale.y = this.canisterImage.scale.x;
            this.glowImage = new Phaser.Image(game, 0, 0, `eb-canister-glow`);
            this.glowImage.anchor.set(0.5);
            this.glowImage.width = this.canisterImage.width * 2;
            this.glowImage.scale.y = this.glowImage.scale.x;
            this.origGlowImageScale = this.glowImage.scale.clone();
            this.glowImage.scale.set(0);
            this.game.add.tween(this.glowImage).to({ angle: 360 }, 3000, Phaser.Easing.Linear.None, true, 0, Infinity);
            this.ventSprite = new Phaser.Sprite(this.game, 0, 0, 'eb-anim-gas');
            this.ventSprite.animations.add('vent', [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11], 24, false);
            this.ventSprite.anchor.set(0.5, 1);
            this.ventSprite.y = this.canisterImage.y;
            this.ventSprite.width = this.canisterImage.width * 2;
            this.ventSprite.scale.y = this.ventSprite.scale.x;
            this.ventSprite.visible = false;
            this.add(this.ventSprite);
            this.add(this.glowImage);
            this.add(this.canisterImage);
        }
        update() {
            super.update();
            this.y += ((this.currentSpeed * this.currentDirection) * (this.game.time.elapsedMS / 1000));
        }
        stop() {
            this.currentSpeed = 0;
        }
        start() {
            this.updateSpeed();
        }
        set direction(direction) {
            this.currentDirection = direction;
        }
        decrementCount() {
            // Figure out the new and old counts
            const oldCount = this.count;
            this.count = Phaser.Math.clamp(--this.count, 0, this.targetCount);
            this.canisterImage.frame = this.count;
            this.updateGlow();
            this.game.sound.play('eb-remove-charge');
            // If we had some charge that we're just about to 
            // lose, we should play the venting animation (after positioning it)
            if (oldCount > 0) {
                this.ventSprite.angle = this.currentDirection === EnergyBallState_1.Direction.UP ? 0 : 180;
                this.ventSprite.visible = true;
                this.ventSprite.animations.play('vent', 18, false).onComplete.addOnce(() => {
                    this.ventSprite.visible = false;
                }, this);
            }
            const durationMS = 600;
            // Rock to the right, then back and forth, then back to the middle
            this.game.add.tween(this.canisterImage).to({ angle: 20 }, durationMS * 0.1, Phaser.Easing.Sinusoidal.Out, true).onComplete.addOnce(() => {
                this.game.add.tween(this.canisterImage).to({ angle: -20 }, durationMS * 0.2, Phaser.Easing.Sinusoidal.InOut, true, 0, 1, true).onComplete.addOnce(() => {
                    this.game.add.tween(this.canisterImage).to({ angle: 0 }, durationMS * 0.1, Phaser.Easing.Sinusoidal.In, true);
                });
            });
            this.updateSpeed();
        }
        incrementCount() {
            if (!this.canCharge) {
                return;
            }
            this.timeOfLastUpdate = this.game.time.now;
            // Figure out the new count
            this.count = Phaser.Math.clamp(++this.count, 0, this.targetCount);
            this.canisterImage.frame = this.count;
            this.updateGlow();
            this.game.sound.play('eb-add-charge');
            if (this.count >= this.targetCount) {
                this.ballCompleteSignal.dispatch();
            }
            else {
                this.game.add.tween(this.canisterImage.scale).from({ x: this.scale.x * 1.2, y: this.scale.y * 1.5 }, 350, Phaser.Easing.Linear.None, true);
            }
            this.updateSpeed();
        }
        updateGlow() {
            const scaleDest = this.origGlowImageScale.x * (this.count / (this.targetCount - 1));
            this.glowImage.scale.set(scaleDest);
        }
        get canCharge() {
            return this.game.time.now > this.timeOfLastUpdate + EnergyBall.TIME_BETWEEN_UPDATES;
        }
        updateSpeed() {
            this.currentSpeed = this.game.height * EnergyBall.SPEED_SCREEN_HEIGHT_PROPORTION;
            // the canisters accellerate on difficulty 2 & 3
            if (this.difficulty >= 1) {
                this.currentSpeed += this.currentSpeed * (this.count / 12);
            }
        }
        destroy() {
            this.ballCompleteSignal.dispose();
            super.destroy();
        }
    }
    EnergyBall.SPEED_SCREEN_HEIGHT_PROPORTION = 0.5;
    EnergyBall.TIME_BETWEEN_UPDATES = 500;
    exports.EnergyBall = EnergyBall;
});
define("minigame/energy-balls/EnergyBallChannel", ["require", "exports", "minigame/energy-balls/EnergyBall", "minigame/energy-balls/EnergyBallState", "switchback-common/util/PromiseUtil"], function (require, exports, EnergyBall_1, EnergyBallState_2, PromiseUtil_53) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class EnergyBallChannel extends Phaser.Group {
        constructor(game, isTutorial, difficulty, shutdownSignal) {
            super(game);
            this.completed = false;
            this.channelCompleteSignal = new Phaser.Signal();
            this.showTutorialSignal = new Phaser.Signal();
            this.isTutorial = isTutorial;
            this.shutdownSignal = shutdownSignal;
            this.backplateEndTutorialBuffer = EnergyBallChannel.BACKPLATE_END_TUTORIAL_BUFFER_PROPORTION * this.game.height;
            //////////////////////////
            // Backplate
            //////////////////////////
            this.backplate = new Phaser.Image(game, 0, this.game.height * 0.5, "eb-column");
            this.backplate.anchor.set(0.5);
            this.backplate.height = this.game.height * 0.9;
            this.backplate.scale.x = this.backplate.scale.y;
            this.backplateBottomEnd = this.backplate.bottom - this.backplate.height * EnergyBallChannel.BACKPLATE_END_PROPORTION;
            this.backplateTopEnd = this.backplate.top + this.backplate.height * EnergyBallChannel.BACKPLATE_END_PROPORTION;
            //////////////////////////
            // Ball
            //////////////////////////
            this.ball = new EnergyBall_1.EnergyBall(game, (this.isTutorial ? EnergyBallChannel.TUTORIAL_CANISTER_COUNT : EnergyBallChannel.TARGET_CANISTER_COUNT[difficulty]), difficulty);
            this.ball.x = this.backplate.centerX;
            this.ball.y = this.game.height * 0.5;
            this.ball.centerY = this.game.rnd.integerInRange(this.game.height * 0.25, this.game.height * 0.75);
            this.ball.ballCompleteSignal.addOnce(this.complete, this);
            // 50% chance to start going upwards
            this.ball.direction = this.game.rnd.frac() > 0.5 ? EnergyBallState_2.Direction.UP : EnergyBallState_2.Direction.DOWN;
            //////////////////////////
            // Channel Caps
            //////////////////////////
            this.topCapImage = new Phaser.Image(this.game, this.backplate.centerX, this.backplateTopEnd + (this.game.height * 0.025), 'eb-channel-lock');
            this.topCapImage.anchor.set(0.5);
            this.topCapImage.width = this.backplate.width * 1.05;
            this.topCapImage.scale.y = this.topCapImage.scale.x;
            this.topCapImage.angle = 180;
            this.topCapImage.visible = false;
            this.bottomCapImage = new Phaser.Image(this.game, this.backplate.centerX, this.backplateBottomEnd - (this.game.height * 0.025), 'eb-channel-lock');
            this.bottomCapImage.anchor.set(0.5);
            this.bottomCapImage.width = this.topCapImage.width;
            this.bottomCapImage.scale.y = this.bottomCapImage.scale.x;
            this.bottomCapImage.visible = false;
            //////////////////////////
            // Channel Beam
            //////////////////////////
            this.channelBeam = new Phaser.Image(this.game, this.backplate.centerX, this.backplate.centerY, 'eb-channel-beam');
            this.channelBeam.anchor.set(0.5);
            this.channelBeam.width = this.backplate.width * 0.8;
            this.channelBeam.height = this.bottomCapImage.centerY - this.topCapImage.centerY;
            this.channelBeam.visible = false;
            this.add(this.backplate);
            this.add(this.channelBeam);
            this.add(this.ball);
            this.add(this.topCapImage);
            this.add(this.bottomCapImage);
        }
        update() {
            super.update();
            if (!this.completed) {
                if (this.ball.y >= this.backplateBottomEnd) {
                    this.ball.direction = EnergyBallState_2.Direction.UP;
                    this.ball.y = this.backplateBottomEnd;
                    this.ball.decrementCount();
                }
                else if (this.ball.y <= this.backplateTopEnd) {
                    this.ball.direction = EnergyBallState_2.Direction.DOWN;
                    this.ball.y = this.backplateTopEnd;
                    this.ball.decrementCount();
                }
            }
            // Stop the balls when they approach the end in tutorial mode
            if (this.isTutorial && !this.completed) {
                if (this.ball.y < (this.backplateTopEnd + this.backplateEndTutorialBuffer)) {
                    this.showTutorialSignal.dispatch(EnergyBallState_2.Direction.UP);
                    this.ball.stop();
                }
                else if (this.ball.y > (this.backplateBottomEnd - this.backplateEndTutorialBuffer)) {
                    this.showTutorialSignal.dispatch(EnergyBallState_2.Direction.DOWN);
                    this.ball.stop();
                }
            }
        }
        complete() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.completed) {
                    return;
                }
                this.completed = true;
                this.game.sound.play('eb-charge-bar');
                this.ball.stop();
                // Move, then remove ball
                const ballDestY = (this.ball.y > this.game.height * 0.5) ? this.bottomCapImage.centerY : this.topCapImage.centerY;
                yield PromiseUtil_53.PromiseUtil.tween(this.game, this.game.add.tween(this.ball).to({ y: ballDestY }, 100, Phaser.Easing.Linear.None, true, 0));
                this.game.add.tween(this.ball.scale).to({ x: 0, y: 0 }, 200, Phaser.Easing.Linear.None, true, 0);
                // Show & animate channel caps
                this.topCapImage.visible = true;
                this.bottomCapImage.visible = true;
                let isShutdown = false;
                yield Promise.all([
                    PromiseUtil_53.PromiseUtil.tween(this.game, this.game.add.tween(this.topCapImage.scale).from({ x: 0, y: 0 }, 350, Phaser.Easing.Back.Out, true, 0), this.shutdownSignal),
                    PromiseUtil_53.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomCapImage.scale).from({ x: 0, y: 0 }, 350, Phaser.Easing.Back.Out, true, 0), this.shutdownSignal)
                ]).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                // Show locked in channel beam
                this.channelBeam.visible = true;
                this.game.add.tween(this.channelBeam.scale).from({ x: 0 }, 300, Phaser.Easing.Back.Out, true, 0);
                this.channelCompleteSignal.dispatch();
            });
        }
        start() {
            this.ball.start();
        }
        destroy() {
            this.shutdownSignal = null;
            this.channelCompleteSignal.dispose();
            this.showTutorialSignal.dispose();
            super.destroy();
        }
    }
    EnergyBallChannel.BACKPLATE_END_PROPORTION = 0.025;
    EnergyBallChannel.BACKPLATE_END_TUTORIAL_BUFFER_PROPORTION = 0.04;
    // lanes and canister sizes are varied based on difficulty
    EnergyBallChannel.TUTORIAL_CANISTER_COUNT = 4;
    EnergyBallChannel.TARGET_CANISTER_COUNT = [4, 6, 6];
    exports.EnergyBallChannel = EnergyBallChannel;
});
define("minigame/energy-balls/TapZone", ["require", "exports", "switchback-common/util/PromiseUtil", "minigame/energy-balls/EnergyBallState", "switchback-common/extension/Easing"], function (require, exports, PromiseUtil_54, EnergyBallState_3, Easing_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TapZone extends Phaser.Group {
        constructor(game, tapZoneFX, angle = 0) {
            super(game);
            this.onTappedSignal = new Phaser.Signal();
            this.lastLockTime = 0;
            this.tapZoneFX = tapZoneFX;
            this.backplate = new Phaser.Image(game, 0, 0, 'eb-tap-bar');
            this.backplate.width = this.game.width;
            this.backplate.scale.y = this.backplate.scale.x;
            this.backplate.inputEnabled = true;
            this.backplate.events.onInputDown.add(() => {
                if (this.game.time.now > this.lastLockTime + TapZone.LOCKOUT_TIME_MS) {
                    this.onTappedSignal.dispatch();
                }
            }, this);
            this.backplate.anchor.set(0.5);
            this.backplate.angle = angle;
            this.backplate.left = 0;
            this.backplate.top = 0;
            this.add(this.backplate);
        }
        get tapArea() {
            return new Phaser.Rectangle(0, this.backplate.worldPosition.y - this.backplate.height * 0.5, this.game.width, this.backplate.height);
        }
        lock() {
            this.lastLockTime = this.game.time.now;
            this.tapZoneFX.playLockAnimation(TapZone.LOCKOUT_TIME_MS);
            this.game.sound.play('eb-tap-quickly');
        }
        flash() {
            this.tapZoneFX.playFlashAnimation();
        }
        update() {
            super.update();
        }
        destroy() {
            this.onTappedSignal.dispose();
            super.destroy();
        }
    }
    TapZone.LOCKOUT_TIME_MS = 500;
    exports.TapZone = TapZone;
    class TapZoneFX extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            //////////////////////////
            // Tap Rectangle Indicators
            //////////////////////////
            const movementRange = this.game.height * 0.015;
            this.tapRectBottom = new Phaser.Image(this.game, this.game.width * 0.5, -this.game.height * 0.05, 'eb-charge-zone');
            this.tapRectBottom.anchor.set(0.5);
            this.tapRectBottom.angle = 180;
            this.tapRectBottom.width = this.game.width * 0.825;
            this.tapRectBottom.scale.y = this.tapRectBottom.scale.x;
            this.game.add.tween(this.tapRectBottom).to({ y: this.tapRectBottom.y - movementRange }, 750, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.tapRectTop = new Phaser.Image(this.game, this.game.width * 0.5, -this.game.height * 0.15, 'eb-charge-zone');
            this.tapRectTop.anchor.set(0.5);
            this.tapRectTop.width = this.game.width * 0.825;
            this.tapRectTop.scale.y = this.tapRectTop.scale.x;
            this.game.add.tween(this.tapRectTop).to({ y: this.tapRectTop.y + movementRange }, 750, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            //////////////////////////
            // Tap flash gradient
            //////////////////////////
            this.gradientOverlay = new Phaser.Image(this.game, 0, 0, 'eb-flash-overlay');
            this.gradientOverlay.width = this.game.width;
            this.gradientOverlay.height = this.game.height * 0.35;
            this.gradientOverlay.anchor.set(0, 1);
            this.gradientOverlay.alpha = 0;
            //////////////////////////
            // Failed tap lockout bars
            //////////////////////////
            this.failedTapNotches = [];
            const numNotches = 7;
            const yPos = -this.game.height * 0.1;
            const xStart = this.game.width * 0.175;
            const xPad = (this.game.width - xStart * 2) / (numNotches - 1);
            for (let i = 0; i < numNotches; i++) {
                const failedNotch = new Phaser.Image(this.game, 0, 0, 'eb-red-bars');
                failedNotch.x = xStart + (xPad * i);
                failedNotch.y = yPos;
                failedNotch.width = this.game.width * 0.1;
                failedNotch.scale.y = failedNotch.scale.x;
                failedNotch.anchor.set(0.5);
                failedNotch.visible = false;
                this.failedTapNotches.push(failedNotch);
            }
            //////////////////////////
            // Side 'Things'
            //////////////////////////
            const leftSideThing = new Phaser.Image(this.game, 0, 0, 'eb-side-thing');
            leftSideThing.anchor.set(0, 1);
            const rightSideThing = new Phaser.Image(this.game, 0, 0, 'eb-side-thing');
            rightSideThing.anchor.set(0, 1);
            rightSideThing.width *= -1;
            rightSideThing.left = this.game.width;
            //////////////////////////
            // Side flames
            //////////////////////////
            const leftFlame = new Phaser.Image(this.game, 0, 0, 'eb-blue-flame');
            leftFlame.anchor.set(0, 0.5);
            leftFlame.y = leftSideThing.top + leftSideThing.height * 0.595;
            leftFlame.width = this.game.width * 0.15;
            leftFlame.height = this.game.height * 0.07;
            this.game.add.tween(leftFlame.scale).to({ x: leftFlame.scale.x * 0.0075, y: leftFlame.scale.y * 0.01 }, this.game.rnd.integerInRange(100, 200), Easing_4.shake(0.1, 0.1), true, game.rnd.integerInRange(0, 75), Infinity);
            const rightFlame = new Phaser.Image(this.game, 0, 0, 'eb-blue-flame');
            rightFlame.anchor.set(0, 0.5);
            rightFlame.left = this.game.width;
            rightFlame.y = rightSideThing.top + rightSideThing.height * 0.595;
            rightFlame.width = -this.game.width * 0.15;
            rightFlame.height = this.game.height * 0.07;
            this.game.add.tween(rightFlame.scale).to({ x: rightFlame.scale.x * 0.0075, y: rightFlame.scale.y * 0.01 }, this.game.rnd.integerInRange(150, 200), Easing_4.shake(0.2, 0.1), true, game.rnd.integerInRange(0, 100), Infinity);
            //////////////////////////
            // Lightning!
            //////////////////////////
            this.lightning = new Phaser.Sprite(this.game, 0, 0, 'eb-lightning', 0);
            this.lightning.animations.add('flash', [0, 1, 2, 3, 4, 5, 6], 24);
            this.lightning.width = this.game.width;
            this.lightning.scale.y = this.lightning.scale.x;
            this.lightning.centerY = leftSideThing.top + leftSideThing.height * 0.6;
            this.lightning.visible = false;
            this.add(this.lightning);
            this.add(leftFlame);
            this.add(rightFlame);
            this.add(leftSideThing);
            this.add(rightSideThing);
            this.add(this.gradientOverlay);
            this.add(this.tapRectTop);
            this.add(this.tapRectBottom);
            this.failedTapNotches.forEach(notch => this.add(notch));
        }
        playFlashAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                this.gradientOverlay.alpha = 0;
                this.gradientOverlay.tint = EnergyBallState_3.EnergyBallState.BLUE_COLOR;
                this.game.add.tween(this.gradientOverlay).to({ alpha: TapZoneFX.GRADIENT_OVERLAY_ALPHA }, 100, Phaser.Easing.Linear.None, true, 0, 0, true);
                this.lightning.visible = true;
                yield PromiseUtil_54.PromiseUtil.animation(this.lightning.animations.play('flash'), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                this.lightning.visible = false;
            });
        }
        playLockAnimation(durationMS) {
            return __awaiter(this, void 0, void 0, function* () {
                // Hide the tap rects
                this.tapRectBottom.visible = false;
                this.tapRectTop.visible = false;
                // Tint the overlay
                this.gradientOverlay.tint = EnergyBallState_3.EnergyBallState.RED_COLOR;
                // Flash the gradient overlay in quickly
                yield PromiseUtil_54.PromiseUtil.tween(this.game, this.game.add.tween(this.gradientOverlay).to({ alpha: TapZoneFX.GRADIENT_OVERLAY_ALPHA }, durationMS * 0.2, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                // Then fade it out
                this.game.add.tween(this.gradientOverlay).to({ alpha: 0 }, durationMS * 0.8, Phaser.Easing.Sinusoidal.In, true);
                // Track how much time we have left for use in the while() loop below
                let isLockedOut = true;
                PromiseUtil_54.PromiseUtil.time(this.game, durationMS * 0.8).then(() => {
                    isLockedOut = false;
                });
                // Hide all notches
                this.failedTapNotches.forEach(notch => notch.visible = false);
                // While we're still locked out, cycle the bars
                let centerNotchIndex = Math.floor(this.failedTapNotches.length / 2);
                let currentIndexOffset = 0;
                while (isLockedOut && centerNotchIndex + currentIndexOffset < this.failedTapNotches.length) {
                    // Turn on new notches, starting at the center
                    this.failedTapNotches[centerNotchIndex + currentIndexOffset].visible = true;
                    this.failedTapNotches[centerNotchIndex - currentIndexOffset].visible = true;
                    yield PromiseUtil_54.PromiseUtil.time(this.game, durationMS * 0.1, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    // Turn off all notches that aren't the current ones (after a slight delay)
                    this.failedTapNotches.forEach((notch, index) => {
                        notch.visible = index === (centerNotchIndex + currentIndexOffset) || index === (centerNotchIndex - currentIndexOffset);
                    });
                    yield PromiseUtil_54.PromiseUtil.time(this.game, durationMS * 0.1, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    currentIndexOffset++;
                }
                // Hide all notches again
                this.failedTapNotches.forEach(notch => notch.visible = false);
                // Show the tap rects again
                this.tapRectBottom.visible = true;
                this.tapRectTop.visible = true;
            });
        }
    }
    TapZoneFX.GRADIENT_OVERLAY_ALPHA = 0.45;
    exports.TapZoneFX = TapZoneFX;
});
define("minigame/energy-balls/Round", ["require", "exports", "minigame/energy-balls/EnergyBallChannel"], function (require, exports, EnergyBallChannel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A round represents a portion of the gameplay in energy balls.
     *
     * It's basically just a collection of EnergyBallChannels and dispatches
     * a signal to the game when all the channels have been completed.
     */
    class Round extends Phaser.Group {
        constructor(game, numBalls, isTutorial, difficulty, shutdownSignal) {
            super(game);
            this.channels = [];
            this.roundCompleteSignal = new Phaser.Signal();
            this.channelsToComplete = 0;
            this.channelsCompleted = 0;
            let xPos = game.width * 0.035;
            const xInc = (game.width - (xPos * 2)) / numBalls;
            for (let i = 0; i < numBalls; i++) {
                const channel = new EnergyBallChannel_1.EnergyBallChannel(this.game, isTutorial, difficulty, shutdownSignal);
                channel.centerX = xPos;
                channel.width = Math.min(xInc, channel.width);
                this.channels.push(channel);
                this.add(channel);
                xPos += xInc;
                this.channelsToComplete++;
                channel.channelCompleteSignal.addOnce(this.onChannelComplete, this);
            }
        }
        start() {
            this.channels.forEach((channel) => {
                channel.start();
            }, this);
        }
        onChannelComplete() {
            this.channelsCompleted++;
            if (this.channelsCompleted >= this.channelsToComplete) {
                this.game.sound.play('eb-all-charged');
                this.roundCompleteSignal.dispatch();
            }
        }
        destroy() {
            this.roundCompleteSignal.dispose();
            super.destroy();
        }
    }
    exports.Round = Round;
});
define("minigame/energy-balls/EnergyBallsHalfDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "component/button/CTAButton", "Game", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "component/coderedeem/CodeButton"], function (require, exports, PromiseUtil_55, CTAButton_8, Game_44, ScaleUtil_34, LocalizationManager_42, CodeButton_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class EnergyBallsHalfDialog extends Phaser.Group {
        constructor(game, isTop, numNotches, shutdownSignal) {
            super(game);
            this.filledNotches = [];
            this._progress = 0;
            this.isTop = isTop;
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            const bg = new Phaser.Image(this.game, 0, 0, 'eb-instruction-panel');
            bg.inputEnabled = true;
            bg.anchor.set(0, 1);
            bg.width = this.game.width;
            bg.height = this.game.height * 0.5;
            this.add(bg);
            this.readyButton = new CTAButton_8.CTAButton(this.game, LocalizationManager_42.LocalizationManager.Instance.text('buttons.ready'), Game_44.Game.FONT_SIZE_MEDIUM, true, this.shutdownSignal);
            this.readyButton.centerX = bg.centerX;
            this.readyButton.bottom = bg.bottom - this.game.height * 0.025;
            this.add(this.readyButton);
            this.nameText = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.15, '', {
                fill: Game_44.Game.FONT_COLOUR_BLUE,
                font: Game_44.Game.DEFAULT_FONT,
                stroke: Game_44.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_44.Game.FONT_STROKE_LARGE,
                fontSize: Game_44.Game.FONT_SIZE_MEDIUM,
                align: 'center'
            });
            this.nameText.anchor.set(0.5);
            this.add(this.nameText);
            this.dummyCanister = new Phaser.Image(this.game, 0, 0, 'eb-tutorial-canister');
            this.dummyCanister.centerX = this.width * 0.25;
            this.dummyCanister.centerY = -this.height * 0.7;
            this.dummyCanister.visible = false;
            this.add(this.dummyCanister);
            //////////////////////////
            // Tap Rectangle Indicators
            //////////////////////////
            this.tapRectBottom = new Phaser.Image(this.game, this.game.width * 0.5, -this.height * 0.29, 'eb-charge-zone');
            this.tapRectBottom.anchor.set(0.5);
            this.tapRectBottom.angle = 180;
            this.tapRectBottom.width = this.game.width * 0.825;
            this.tapRectBottom.scale.y = this.tapRectBottom.scale.x;
            this.add(this.tapRectBottom);
            this.tapRectTop = new Phaser.Image(this.game, this.game.width * 0.5, this.tapRectBottom.y - (this.height * 0.14), 'eb-charge-zone');
            this.tapRectTop.anchor.set(0.5);
            this.tapRectTop.width = this.game.width * 0.825;
            this.tapRectTop.scale.y = this.tapRectTop.scale.x;
            this.add(this.tapRectTop);
            this.tapRectGraphic = new Phaser.Graphics(this.game);
            this.tapRectGraphic.beginFill(EnergyBallsHalfDialog.FLASH_COLOR, EnergyBallsHalfDialog.FLASH_ALPHA);
            this.tapRectGraphic.drawRect(this.tapRectTop.left + this.tapRectTop.width * 0.025, this.tapRectTop.top + this.tapRectTop.height * 0.275, this.tapRectTop.width * 0.95, this.tapRectBottom.y - this.tapRectTop.y + (this.tapRectBottom.height * 0.4));
            this.tapRectGraphic.endFill();
            this.tapRectGraphic.visible = false;
            this.add(this.tapRectGraphic);
            this.handImage = new Phaser.Image(this.game, 0, 0, 'eb-anim-hand');
            this.handImage.centerX = this.centerX;
            this.handImage.centerY = -this.height * 0.33;
            ScaleUtil_34.scaleToBounds(this.handImage, this.width * 0.2, this.height * 0.25);
            this.add(this.handImage);
            const instructionText = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.3, LocalizationManager_42.LocalizationManager.Instance.text('energy_balls.intro'), {
                fill: Game_44.Game.DEFAULT_FONT_COLOUR,
                font: Game_44.Game.DEFAULT_FONT,
                stroke: Game_44.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_44.Game.FONT_STROKE_LARGE,
                fontSize: Game_44.Game.FONT_SIZE_SMALL,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8
            });
            instructionText.anchor.set(0.5);
            this.add(instructionText);
            this.createScoreNotches(numNotches);
            this.codeButton = new CodeButton_6.CodeButton(this.game, this.shutdownSignal);
            this.codeButton.top = bg.top + this.game.height * 0.005;
            this.add(this.codeButton);
            this.codeButton.visible = false;
            this.doAnimations();
        }
        createScoreNotches(numNotches) {
            this.notchGroup = new Phaser.Group(this.game);
            const xInc = this.game.width / (numNotches + 1);
            for (let i = 0; i < numNotches; i++) {
                const bgNotch = new Phaser.Image(this.game, xInc * (i + 1), -this.height, 'eb-notch-bg');
                ScaleUtil_34.scaleToBounds(bgNotch, this.game.width * 0.15, this.game.width * 0.15);
                bgNotch.anchor.set(0.5, 0);
                this.add(bgNotch);
                // The top half doesn't have the filled notches
                if (!this.isTop) {
                    const filledNotch = new Phaser.Image(this.game, bgNotch.centerX, bgNotch.top, 'eb-filled-notch');
                    filledNotch.width = bgNotch.width * 1.2;
                    filledNotch.scale.y = filledNotch.scale.x;
                    filledNotch.anchor.set(0.5);
                    filledNotch.visible = false;
                    this.notchGroup.add(filledNotch);
                    this.filledNotches.push(filledNotch);
                }
            }
            this.add(this.notchGroup);
        }
        set progress(val) {
            this._progress++;
            for (let i = 0; i < this.filledNotches.length; i++) {
                this.filledNotches[i].visible = i < this._progress;
            }
        }
        set playerName(name) {
            this.nameText.text = name;
        }
        showThenHide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                if (this.isTop) {
                    yield PromiseUtil_55.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                else {
                    yield PromiseUtil_55.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                }
                if (this.isShutdown) {
                    return;
                }
                this.codeButton.visible = true;
                this.game.add.tween(this.codeButton).from({ right: 0 }, 300, Phaser.Easing.Linear.None, true);
                this.isShutdown = !(yield this.readyButton.show());
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_55.PromiseUtil.signal(this.readyButton.onButtonClickedSignal, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                this.game.sound.play('eb-add-charge');
                this.isShutdown = !(yield this.readyButton.hide());
                if (this.isShutdown) {
                    return;
                }
                if (this.isTop) {
                    yield PromiseUtil_55.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { });
                }
                else {
                    yield PromiseUtil_55.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ top: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true), this.shutdownSignal).catch(() => { });
                }
                this.visible = false;
            });
        }
        doAnimations() {
            return __awaiter(this, void 0, void 0, function* () {
                const loopDurationMS = 1200;
                const tapRectTopY = this.tapRectTop.y;
                const tapRectBottomY = this.tapRectBottom.y;
                const tapRectMovementRange = this.game.height * 0.015;
                let dummyCanisterNum = 0;
                // Create our animation loop function in here, which we'll start using shortly
                const animationLoop = () => __awaiter(this, void 0, void 0, function* () {
                    // Reset the canister, then start it going down
                    this.dummyCanister.visible = true;
                    dummyCanisterNum = (dummyCanisterNum + 1) % 3;
                    this.dummyCanister.centerX = (dummyCanisterNum + 1) * (this.width * 0.25);
                    this.dummyCanister.centerY = -this.game.height * 0.35;
                    this.game.tweens.removeFrom(this.dummyCanister);
                    this.game.add.tween(this.dummyCanister).to({ centerY: -this.game.height * 0.225 }, loopDurationMS * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true);
                    // Reset the tap bars, then start them squishing
                    this.tapRectTop.y = tapRectTopY - tapRectMovementRange;
                    this.tapRectBottom.y = tapRectBottomY + tapRectMovementRange;
                    this.game.tweens.removeFrom(this.tapRectTop);
                    this.game.tweens.removeFrom(this.tapRectBottom);
                    this.game.add.tween(this.tapRectTop).to({ y: this.tapRectTop.y + tapRectMovementRange }, loopDurationMS * 0.5, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
                    this.game.add.tween(this.tapRectBottom).to({ y: this.tapRectBottom.y - tapRectMovementRange }, loopDurationMS * 0.5, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
                    // Reset the finger & flash graphic
                    this.handImage.frame = 0;
                    this.tapRectGraphic.visible = false;
                    this.game.tweens.removeFrom(this.handImage);
                    this.game.add.tween(this.handImage).to({}, loopDurationMS * 0.4, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                        this.handImage.frame = 1;
                        this.tapRectGraphic.visible = true;
                        yield PromiseUtil_55.PromiseUtil.time(this.game, loopDurationMS * 0.2, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                        this.tapRectGraphic.visible = false;
                        this.handImage.frame = 0;
                    }));
                });
                // Create an empty tween, which calls the animationLoop()
                // function when it starts and when it repeats
                this.animationLoopTween = this.game.add.tween(this).to({}, loopDurationMS, Phaser.Easing.Linear.None, true, 0, Infinity);
                this.animationLoopTween.onStart.add(animationLoop, this);
                this.animationLoopTween.onRepeat.add(animationLoop, this);
            });
        }
        destroy() {
            if (this.animationLoopTween) {
                this.animationLoopTween.stop();
                this.animationLoopTween = null;
            }
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    EnergyBallsHalfDialog.FLASH_COLOR = 0xFFED2E;
    EnergyBallsHalfDialog.FLASH_ALPHA = 0.3;
    exports.EnergyBallsHalfDialog = EnergyBallsHalfDialog;
});
define("minigame/energy-balls/EnergyBallState", ["require", "exports", "logic/GameController", "StateNames", "minigame/energy-balls/TapZone", "Game", "minigame/energy-balls/Round", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "minigame/energy-balls/EnergyBallsHalfDialog", "logic/SMAnalyticsManager", "state/SpaceGameBaseState", "component/dialog/BasicSpaceButtonDialog", "component/coderedeem/RedeemCodeDialog"], function (require, exports, GameController_26, StateNames_23, TapZone_1, Game_45, Round_1, LocalizationManager_43, PromiseUtil_56, EnergyBallsHalfDialog_1, SMAnalyticsManager_23, SpaceGameBaseState_6, BasicSpaceButtonDialog_7, RedeemCodeDialog_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var Direction;
    (function (Direction) {
        Direction[Direction["UP"] = -1] = "UP";
        Direction[Direction["DOWN"] = 1] = "DOWN";
    })(Direction = exports.Direction || (exports.Direction = {}));
    /**
     * @author joel@cerebralfix.com
     */
    class EnergyBallState extends SpaceGameBaseState_6.SpaceGameBaseState {
        init() {
            this.stateName = StateNames_23.StateNames.GAME_ENERGY_BALLS;
            super.init();
            const numPlays = GameController_26.GameController.Instance.state.getMinigamePlays(StateNames_23.StateNames.GAME_ENERGY_BALLS);
            this.difficulty = Math.min(numPlays, 2);
            this.MAX_PLAYTIME_BEFORE_STUCK = 120000;
            this.currentRound = null;
            this.roundsCompleted = 0;
            this.isPlaying = false;
            // TODO: Get tutorial status from save state
            this.isTutorial = !GameController_26.GameController.Instance.hasCompletedMinigame(this.stateName);
            this.isTutorialShowing = false;
        }
        preload() {
            super.preload();
            RedeemCodeDialog_8.RedeemCodeDialog.preload(this.game);
            this.game.load.image('bg-frame', 'assets/common/bg-frame.png');
            this.game.load.image('eb-blue-flame', 'assets/minigame/energy-ball/eb-blue-flame.png');
            this.game.load.image('eb-red-bars', 'assets/minigame/energy-ball/eb-red-bars.png');
            this.game.load.image('eb-column', 'assets/minigame/energy-ball/eb-column.png');
            this.game.load.image('eb-charge-zone', 'assets/minigame/energy-ball/eb-charge-zone.png');
            this.game.load.image('eb-flash-overlay', 'assets/minigame/energy-ball/eb-flash-overlay.png');
            this.game.load.image('eb-tap-bar', 'assets/minigame/energy-ball/eb-tap-bar.png');
            this.game.load.image('eb-channel-lock', 'assets/minigame/energy-ball/eb-channel-lock.png');
            this.game.load.image('eb-canister-glow', 'assets/minigame/energy-ball/eb-canister-glow.png');
            this.game.load.image('eb-side-thing', 'assets/minigame/energy-ball/eb-side-thing.png');
            this.game.load.image('eb-channel-beam', 'assets/minigame/energy-ball/eb-channel-beam.png');
            this.game.load.image('eb-instruction-panel', 'assets/minigame/energy-ball/eb-instruction-panel.jpg');
            this.game.load.image('eb-tutorial-canister', 'assets/minigame/energy-ball/eb-tutorial-canister.png');
            this.game.load.image('eb-filled-notch', 'assets/minigame/energy-ball/eb-filled-notch.png');
            this.game.load.image('eb-notch-bg', 'assets/minigame/energy-ball/eb-notch-bg.png');
            this.game.load.image('eb-tutorial-circle', 'assets/minigame/energy-ball/eb-tutorial-circle.png');
            this.game.load.atlas('eb-lightning', 'assets/minigame/energy-ball/eb-lightning.png', 'assets/minigame/energy-ball/eb-lightning.json');
            this.game.load.atlas('eb-anim-gas', 'assets/minigame/energy-ball/eb-anim-gas.png', 'assets/minigame/energy-ball/eb-anim-gas.json');
            this.game.load.atlas('eb-anim-hand', "assets/common/anim-hand.png", "assets/common/anim-hand.json");
            this.game.load.atlas('eb-canister-4', 'assets/minigame/energy-ball/eb-canister-4.png', 'assets/minigame/energy-ball/eb-canister-4.json');
            this.game.load.atlas('eb-canister-6', 'assets/minigame/energy-ball/eb-canister-6.png', 'assets/minigame/energy-ball/eb-canister-6.json');
            this.game.load.audio("eb-add-charge", ["assets/sound/energy-balls/add_charge.m4a", "assets/sound/energy-balls/add_charge.ogg"]);
            this.game.load.audio("eb-remove-charge", ["assets/sound/energy-balls/remove_charge.m4a", "assets/sound/energy-balls/remove_charge.ogg"]);
            this.game.load.audio("eb-all-charged", ["assets/sound/energy-balls/all_charged.m4a", "assets/sound/energy-balls/all_charged.ogg"]);
            this.game.load.audio("eb-charge-bar", ["assets/sound/energy-balls/charge_bar.m4a", "assets/sound/energy-balls/charge_bar.ogg"]);
            this.game.load.audio("eb-tap-normal", ["assets/sound/energy-balls/tap_bar_normal.m4a", "assets/sound/energy-balls/tap_bar_normal.ogg"]);
            this.game.load.audio("eb-tap-quickly", ["assets/sound/energy-balls/tap_too_quickly.m4a", "assets/sound/energy-balls/tap_too_quickly.ogg"]);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.input.maxPointers = 4;
                // Set up our session and scores
                this.playersSession = GameController_26.GameController.Instance.getPlayerSession(2, null, true);
                this.roundTimes = [];
                for (let player in this.playersSession) {
                    if (player) {
                        this.roundTimes.push(0);
                    }
                }
                // The first pair go twice in the tutorial
                if (this.isTutorial) {
                    this.playersSession.unshift(this.playersSession[0]);
                }
                //////////////////////////
                // CORE LAYERS
                //////////////////////////
                this.tapZoneLayer = new Phaser.Group(this.game);
                this.gameLayer = new Phaser.Group(this.game);
                this.tapZoneFXLayer = new Phaser.Group(this.game);
                this.add.existing(this.tapZoneLayer);
                this.add.existing(this.gameLayer);
                this.add.existing(this.tapZoneFXLayer);
                //////////////////////////
                // TAP ZONES + ZONE FX
                //////////////////////////
                const topZoneFX = new TapZone_1.TapZoneFX(this.game, this.shutdownSignal);
                const bottomZoneFX = new TapZone_1.TapZoneFX(this.game, this.shutdownSignal);
                const topZone = new TapZone_1.TapZone(this.game, topZoneFX, 180);
                const bottomZone = new TapZone_1.TapZone(this.game, bottomZoneFX);
                // Rotate the top FX zone
                topZoneFX.angle = 180;
                topZoneFX.left = 0;
                topZoneFX.top = 0;
                // Position them all
                topZone.top = 0;
                topZoneFX.top = topZone.top;
                bottomZone.bottom = this.game.height;
                bottomZoneFX.bottom = bottomZone.bottom;
                this.tapZoneLayer.add(topZone);
                this.tapZoneLayer.add(bottomZone);
                this.tapZoneFXLayer.add(topZoneFX);
                this.tapZoneFXLayer.add(bottomZoneFX);
                this.tapZoneLayer.forEach((zone) => {
                    zone.onTappedSignal.add(() => {
                        this.onTapZoneClicked(zone);
                    }, this);
                }, this);
                const frame = new Phaser.Image(this.game, 0, 0, 'bg-frame');
                frame.width = this.game.width;
                frame.height = this.game.height;
                this.add.existing(frame);
                //////////////////////////
                // INTRO DIALOG
                //////////////////////////
                this.topDialog = new EnergyBallsHalfDialog_1.EnergyBallsHalfDialog(this.game, true, this.playersSession.length, this.shutdownSignal);
                this.topDialog.angle = 180;
                this.topDialog.left = 0;
                this.topDialog.top = 0;
                this.topDialog.playerName = GameController_26.GameController.Instance.players[this.playersSession[this.roundsCompleted][1]].name;
                this.add.existing(this.topDialog);
                this.bottomDialog = new EnergyBallsHalfDialog_1.EnergyBallsHalfDialog(this.game, false, this.playersSession.length, this.shutdownSignal);
                this.bottomDialog.right = this.game.width;
                this.bottomDialog.bottom = this.game.height;
                this.bottomDialog.playerName = GameController_26.GameController.Instance.players[this.playersSession[this.roundsCompleted][0]].name;
                this.add.existing(this.bottomDialog);
                this.tutorialHand = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, 'eb-anim-hand');
                this.tutorialHand.animations.add('loop', null, 2, true).play();
                this.tutorialHand.anchor.set(0.5);
                this.tutorialHand.visible = false;
                this.add.existing(this.tutorialHand);
                this.tutorialText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.5, LocalizationManager_43.LocalizationManager.Instance.text('energy_balls.tap'), {
                    font: Game_45.Game.DEFAULT_FONT,
                    stroke: Game_45.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_45.Game.DEFAULT_STROKE_THICKNESS,
                    fontSize: Game_45.Game.FONT_SIZE_LARGE,
                    fill: Game_45.Game.DEFAULT_FONT_COLOUR
                });
                this.tutorialText.anchor.set(0.5);
                this.tutorialText.visible = false;
                this.game.add.tween(this.tutorialText.scale).to({ x: 1.25, y: 1.25 }, 500, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                this.add.existing(this.tutorialText);
                this.tutorialCircle = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, 'eb-tutorial-circle');
                this.tutorialCircle.anchor.set(0.5);
                this.tutorialCircle.visible = false;
                this.tutorialCircle.width = this.game.width * 0.25;
                this.tutorialCircle.scale.y = this.tutorialCircle.scale.x;
                this.game.add.tween(this.tutorialCircle).to({ angle: 360 }, 3000, Phaser.Easing.Linear.None, true, 0, Infinity);
                this.add.existing(this.tutorialCircle);
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_23.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                SMAnalyticsManager_23.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                yield Promise.all([
                    this.topDialog.showThenHide(),
                    this.bottomDialog.showThenHide()
                ]);
                if (this.isShutdown) {
                    return;
                }
                this.nextRound();
            });
        }
        /**
         * Sets up a new round by creating, showing, and then
         * setting up a listener for round complete.
         */
        nextRound() {
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_23.SMAnalyticsManager.Instance.trackGamePlay(this.stateName);
                // Create round (1 lane only for tutorial)
                this.currentRound = new Round_1.Round(this.game, this.isTutorial ? EnergyBallState.TUTORIAL_LANE_COUNT : EnergyBallState.TARGET_LANE_COUNT[this.difficulty], this.isTutorial, this.difficulty, this.shutdownSignal);
                this.currentRound.right = 0;
                this.currentRound.centerY = this.game.height * 0.5;
                this.gameLayer.add(this.currentRound);
                // Set up listener for roundCompleteSignal
                this.currentRound.roundCompleteSignal.addOnce(this.onRoundComplete, this);
                // Slide it in
                yield PromiseUtil_56.PromiseUtil.tween(this.game, this.game.add.tween(this.currentRound)
                    .to({ centerX: this.game.width * 0.5 }, 750, Phaser.Easing.Sinusoidal.InOut, true, 0), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.currentRound.channels[0].showTutorialSignal.add(this.showTutorial, this);
                this.currentRound.start();
                this.isPlaying = true;
            });
        }
        showTutorial(direction) {
            if (!this.isTutorialShowing) {
                this.isTutorialShowing = true;
                if (direction === Direction.DOWN) {
                    this.tutorialHand.angle = 0;
                    this.tutorialText.angle = 0;
                    this.tutorialCircle.position.set(this.game.width * 0.5, this.game.height * 0.89);
                    this.tutorialHand.position.set(this.game.width * 0.75, this.game.height * 0.93);
                    this.tutorialText.position.set(this.game.width * 0.75, this.game.height * 0.8);
                }
                else {
                    this.tutorialHand.angle = 180;
                    this.tutorialText.angle = 180;
                    this.tutorialCircle.position.set(this.game.width * 0.5, this.game.height * 0.105);
                    this.tutorialHand.position.set(this.game.width * 0.25, this.game.height * 0.07);
                    this.tutorialText.position.set(this.game.width * 0.25, this.game.height * 0.2);
                }
                this.tutorialHand.visible = true;
                this.tutorialText.visible = true;
                this.tutorialCircle.visible = true;
            }
        }
        hideTutorial() {
            if (this.isTutorialShowing) {
                this.isTutorialShowing = false;
                this.tutorialHand.visible = false;
                this.tutorialText.visible = false;
                this.tutorialCircle.visible = false;
            }
        }
        update() {
            super.update();
            if (!this.isPlaying) {
                return;
            }
            this.roundTimes[this.roundsCompleted] += this.game.time.elapsedMS;
            if (this.roundTimes[this.roundsCompleted] > this.MAX_PLAYTIME_BEFORE_STUCK) {
                this.isPlaying = false;
                this.onGameTimeout();
            }
        }
        onGameTimeout() {
            return __awaiter(this, void 0, void 0, function* () {
                const timeoutDialog = new BasicSpaceButtonDialog_7.BasicSpaceButtonDialog(this.game, LocalizationManager_43.LocalizationManager.Instance.text('activity_timout.description'), LocalizationManager_43.LocalizationManager.Instance.text('activity_timout.button'), this.shutdownSignal);
                timeoutDialog.show();
                yield PromiseUtil_56.PromiseUtil.signal(timeoutDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                yield timeoutDialog.hide();
                if (timeoutDialog.isShutdown) {
                    return;
                }
                timeoutDialog.destroy();
                this.onGameOver();
            });
        }
        /**
         * When the round is complete, we have to do a few things:
         * 1. end the round - marking the time
         * 2. slide the old round off screen
         * 3. show the progress indicator
         * 4. show player name text at each end
         * 5a. if there are more rounds to play, make a new round
         * 5b. if there aren't, game over!
         */
        onRoundComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                this.isPlaying = false;
                yield PromiseUtil_56.PromiseUtil.tween(this.game, this.game.add.tween(this.currentRound).to({ left: this.game.width }, 750, Phaser.Easing.Sinusoidal.InOut, true, 1000), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.currentRound.destroy();
                this.roundsCompleted++;
                this.bottomDialog.progress = this.roundsCompleted;
                if (this.roundsCompleted < this.playersSession.length) {
                    if (this.isTutorial) {
                        // We can end the tutorial now
                        this.isTutorial = false;
                    }
                    this.topDialog.playerName = GameController_26.GameController.Instance.players[this.playersSession[this.roundsCompleted][1]].name.toUpperCase();
                    this.bottomDialog.playerName = GameController_26.GameController.Instance.players[this.playersSession[this.roundsCompleted][0]].name.toUpperCase();
                    SMAnalyticsManager_23.SMAnalyticsManager.Instance.trackPassAndPlay(this.stateName);
                    SMAnalyticsManager_23.SMAnalyticsManager.Instance.trackInstructions(this.stateName);
                    yield Promise.all([
                        this.topDialog.showThenHide(),
                        this.bottomDialog.showThenHide()
                    ]);
                    if (this.isShutdown) {
                        return;
                    }
                    this.nextRound();
                }
                else {
                    this.onGameOver();
                }
            });
        }
        /**
         * Post off the score and show the game over dialog
         */
        onGameOver() {
            this.endGame();
            this.transitionToOutro();
        }
        postScore(isInterrupted) {
            let totalTime = 0;
            for (let roundIndex in this.playersSession) {
                let roundSeconds = (this.roundTimes[roundIndex] * 0.001) || 0.00001;
                totalTime += roundSeconds;
            }
            let rounds = this.playersSession.length;
            GameController_26.GameController.Instance.postMinigameScore(StateNames_23.StateNames.GAME_ENERGY_BALLS, totalTime);
            GameController_26.GameController.Instance.achievementManager.onEnergyBallsEndGame(totalTime / rounds);
        }
        /**
         * This is called when a tap zone is clicked by the player.
         *
         * It checks to see if any balls in any channels collide
         * with that zone at the time. If any do, it tells the ball to
         * rebound and change direction, incrementing the score of the channel.
         *
         * If no balls collide, the player missed - the channel locks briefly.
         * @param zone
         */
        onTapZoneClicked(zone) {
            this.game.sound.play('eb-tap-normal');
            let didHitBall = false;
            if (this.currentRound) {
                this.currentRound.channels.forEach((chan) => {
                    if (chan.completed || !chan.ball.canCharge) {
                        return;
                    }
                    // If the channel's ball can be charged and is over the discharge zone
                    if (Phaser.Rectangle.intersects(new Phaser.Rectangle(chan.ball.left, chan.ball.top, chan.ball.width, chan.ball.height), zone.tapArea)) {
                        didHitBall = true;
                        chan.ball.direction = chan.ball.y > this.game.height * 0.5 ? Direction.UP : Direction.DOWN;
                        chan.ball.incrementCount();
                        zone.flash();
                        chan.ball.start();
                        this.hideTutorial();
                    }
                }, this);
            }
            if (!didHitBall) {
                zone.lock();
            }
        }
        transitionToOutro() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                GameController_26.GameController.Instance.finishGame();
                this.game.state.start(StateNames_23.StateNames.NARRATIVE_DEBRIEFING);
            });
        }
        shutdown() {
            this.tapZoneLayer = null;
            this.gameLayer = null;
            this.tapZoneFXLayer = null;
            this.currentRound = null;
            this.topDialog = null;
            this.bottomDialog = null;
            this.tutorialHand = null;
            this.tutorialText = null;
            this.tutorialCircle = null;
            this.game.cache.removeImage('bg-frame');
            this.game.cache.removeImage('eb-blue-flame');
            this.game.cache.removeImage('eb-red-bars');
            this.game.cache.removeImage('eb-column');
            this.game.cache.removeImage('eb-charge-zone');
            this.game.cache.removeImage('eb-flash-overlay');
            this.game.cache.removeImage('eb-tap-bar');
            this.game.cache.removeImage('eb-channel-lock');
            this.game.cache.removeImage('eb-canister-glow');
            this.game.cache.removeImage('eb-side-thing');
            this.game.cache.removeImage('eb-channel-beam');
            this.game.cache.removeImage('eb-instruction-panel');
            this.game.cache.removeImage('eb-tutorial-canister');
            this.game.cache.removeImage('eb-filled-notch');
            this.game.cache.removeImage('eb-notch-bg');
            this.game.cache.removeImage('eb-tutorial-circle');
            this.game.cache.removeImage('eb-lightning');
            this.game.cache.removeImage('eb-anim-gas');
            this.game.cache.removeImage('eb-anim-hand');
            this.game.cache.removeImage('eb-canister-4');
            this.game.cache.removeImage('eb-canister-6');
            this.game.cache.removeSound("eb-add-charge");
            this.game.cache.removeSound("eb-remove-charge");
            this.game.cache.removeSound("eb-all-charged");
            this.game.cache.removeSound("eb-charge-bar");
            this.game.cache.removeSound("eb-tap-normal");
            this.game.cache.removeSound("eb-tap-quickly");
            super.shutdown();
        }
    }
    EnergyBallState.BLUE_COLOR = 0x00CBFE;
    EnergyBallState.RED_COLOR = 0xEF2731;
    EnergyBallState.TUTORIAL_LANE_COUNT = 1;
    EnergyBallState.TARGET_LANE_COUNT = [3, 3, 4];
    exports.EnergyBallState = EnergyBallState;
});
define("switchback-common/util/DeviceUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to support device-specific functions.
     * @author nick@cerebralfix.com
     */
    class DeviceUtil {
        /*
        * This function applies the MaliGPU fix if this device is running a Mali-400 GPU.
        * This is required to apply a phaser render setting that fixes a
        * canvas strobing issue present in these GPUs.
        */
        static applyMaliGPUFix(game) {
            if (DeviceUtil.isMaliGPU()) {
                // Fix for strobing canvas when running Phaser on Mali-400 GPUs.
                game.preserveDrawingBuffer = true;
            }
        }
        /*
        * This function detects whether the device is running a Mali-400 GPU.
        * This is required to apply a phaser render setting that fixes a
        * canvas strobing issue present in these GPUs.
        */
        static isMaliGPU() {
            const canv = document.createElement("canvas");
            canv.setAttribute("id", "MaliTestCanvas");
            canv.setAttribute("width", "1");
            canv.setAttribute("height", "1");
            document.body.appendChild(canv);
            const canvas = document.getElementById("MaliTestCanvas");
            const gl = canvas.getContext("webgl", { stencil: true });
            canvas.parentNode.removeChild(canvas);
            if (!gl) {
                return false;
            }
            const dbgRenderInfo = gl.getExtension("WEBGL_debug_renderer_info");
            let renderer;
            if (dbgRenderInfo !== null) {
                renderer = gl.getParameter(dbgRenderInfo.UNMASKED_RENDERER_WEBGL);
            }
            else {
                return false;
            }
            return renderer.search("Mali-400") !== -1;
        }
        /**
         * Checks the userAgent to try to figure out
         * which platform we're running on.
         */
        static isIos() {
            return /iPad|iPhone|iPod/.test(navigator.userAgent);
        }
    }
    exports.DeviceUtil = DeviceUtil;
});
define("component/missions/MissionPanel", ["require", "exports", "Game", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil"], function (require, exports, Game_46, TextUtil_19, LocalizationManager_44, PromiseUtil_57) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionPanel extends Phaser.Group {
        constructor(game, bgPanelKey, shutdownSignal = null) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.bgPanel = new Phaser.Image(this.game, 0, 0, bgPanelKey);
            this.bgPanel.width = this.game.width * 0.8;
            this.bgPanel.scale.y = this.bgPanel.scale.x;
            this.bgPanel.anchor.set(0.5, 0);
            this.missionNumberText = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.13, "", {
                font: Game_46.Game.DEFAULT_FONT,
                fontSize: Game_46.Game.FONT_SIZE_SMALL,
                fill: "#335f90"
            });
            this.missionNumberText.anchor.set(0.5);
            this.titleText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, Game_46.Game.DEFAULT_FONT_STYLE, { fontSize: Game_46.Game.FONT_SIZE_LARGE, fill: Game_46.Game.FONT_COLOUR_YELLOW, align: 'left' }));
            this.titleText.anchor.set(0.5);
            this.titleText.y = this.bgPanel.height * 0.55;
            this.emptyMissionText = new Phaser.Text(this.game, 0, 0, "AWAITING MISSION", {
                font: Game_46.Game.DEFAULT_FONT,
                fontSize: Game_46.Game.FONT_SIZE_LARGE,
                fill: "#335f90",
                wordWrap: true,
                wordWrapWidth: this.bgPanel.width * 0.9,
                align: 'center'
            });
            this.emptyMissionText.anchor.set(0.5);
            this.add(this.bgPanel);
            this.add(this.missionNumberText);
            this.add(this.titleText);
            this.add(this.emptyMissionText);
        }
        getMission() {
            return this._mission;
        }
        setMission(mission) {
            this._mission = mission;
            this.populatePanelWithMission(this._mission);
        }
        populatePanelWithMission(mission) {
            if (mission) {
                this.missionNumberText.text = LocalizationManager_44.LocalizationManager.Instance.textWithReplacements("mission.number", {
                    mission_number: mission.id + 1
                });
                this.missionNumberText.fontSize = Game_46.Game.FONT_SIZE_SMALL;
                TextUtil_19.TextUtil.scaleDownSingleLineTextToWidth(this.missionNumberText, this.bgPanel.width * 0.9);
                this.missionNumberText.visible = true;
                this.titleText.text = mission.title;
                this.titleText.fontSize = Game_46.Game.FONT_SIZE_LARGE;
                TextUtil_19.TextUtil.scaleDownSingleLineTextToWidth(this.titleText, this.bgPanel.width * 0.9);
                this.titleText.visible = true;
                this.emptyMissionText.visible = false;
            }
            else {
                this.missionNumberText.visible = false;
                this.titleText.visible = false;
                this.emptyMissionText.centerY = this.bgPanel.top + this.bgPanel.height * 0.55;
                this.emptyMissionText.visible = true;
            }
        }
        replaceMission(mission) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.slideOut();
                if (this.isShutdown) {
                    return;
                }
                this.setMission(mission);
                yield this.slideIn();
            });
        }
        slideOut() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: this.game.width + this.x }, 350, Phaser.Easing.Sinusoidal.In, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                this.visible = false;
            });
        }
        slideIn() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.x = -(this.width * 1.5);
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ centerX: 0 }, 350, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
            });
        }
        destroy() {
            this.shutdownSignal = null;
            super.destroy();
        }
    }
    exports.MissionPanel = MissionPanel;
});
define("component/missions/MissionProgressBar", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_58) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionProgressBar extends Phaser.Group {
        constructor(game) {
            super(game);
            const frame = new Phaser.Image(this.game, 0, 0, 'progress-bar-frame');
            this.back = new Phaser.Image(this.game, frame.width * 0.0225, frame.centerY, 'progress-bar-back');
            this.back.width = frame.width * 0.955;
            this.back.height = frame.height * 0.75;
            this.back.anchor.set(0, 0.5);
            this.fill = new Phaser.Image(this.game, this.back.left, frame.centerY, 'progress-bar-fill');
            this.fill.height = this.back.height;
            this.fill.width = 0;
            this.fill.anchor.set(0, 0.5);
            this.fillChange = new Phaser.Image(this.game, this.back.left, this.fill.centerY, 'progress-bar-fill-change');
            this.fillChange.height = this.fill.height;
            this.fillChange.anchor.set(0, 0.5);
            this.fillChange.width = 0;
            this.add(this.back);
            this.add(this.fillChange);
            this.add(this.fill);
            this.add(frame);
        }
        setProgress(progressProportion) {
            progressProportion = Math.max(0, Math.min(1, progressProportion));
            this.fill.width = this.back.width * progressProportion;
            this.fillChange.width = this.back.width * progressProportion;
        }
        animateProgress(totalProgressProportion, shutdownSignal) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play("missions_prebar_fill");
                let isShutdown = false;
                PromiseUtil_58.PromiseUtil.time(this.game, 500, shutdownSignal).then(() => {
                    this.game.sound.play("missions_bar_fill");
                }).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return !isShutdown;
                }
                yield Promise.all([
                    PromiseUtil_58.PromiseUtil.tween(this.game, this.game.add.tween(this.fillChange)
                        .to({ width: this.back.width * totalProgressProportion }, 400, Phaser.Easing.Sinusoidal.InOut, true), shutdownSignal),
                    PromiseUtil_58.PromiseUtil.tween(this.game, this.game.add.tween(this.fill)
                        .to({ width: this.back.width * totalProgressProportion }, 400, Phaser.Easing.Sinusoidal.InOut, true, 500), shutdownSignal)
                ]).catch(() => { isShutdown = true; });
                return !isShutdown;
            });
        }
    }
    exports.MissionProgressBar = MissionProgressBar;
});
define("component/missions/MissionPanelDetail", ["require", "exports", "component/missions/MissionPanel", "Game", "component/missions/MissionProgressBar", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil"], function (require, exports, MissionPanel_1, Game_47, MissionProgressBar_1, ScaleUtil_35, TextUtil_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionPanelDetail extends MissionPanel_1.MissionPanel {
        constructor(game, shutdownSignal) {
            super(game, "mission-window-tall", shutdownSignal);
            this.missionNumberText.y = this.bgPanel.height * 0.075;
            this.titleText.y = this.bgPanel.height * 0.3;
            this.descriptionText = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.5, "", Object.assign({}, Game_47.Game.DEFAULT_FONT_STYLE, { fontSize: Game_47.Game.FONT_SIZE_SMALL, fill: Game_47.Game.DEFAULT_FONT_COLOUR, align: 'left' }));
            TextUtil_20.TextUtil.scaleDownSingleLineTextToWidth(this.descriptionText, this.bgPanel.width * 0.9);
            this.descriptionText.anchor.set(0, 0.5);
            this.bar = new MissionProgressBar_1.MissionProgressBar(this.game);
            ScaleUtil_35.scaleToBounds(this.bar, this.bgPanel.width * 0.9, this.bgPanel.height * 0.3);
            this.bar.centerX = this.bgPanel.centerX;
            this.bar.centerY = this.bgPanel.height * 0.75;
            this.add(this.descriptionText);
            this.add(this.bar);
        }
        populatePanelWithMission(mission) {
            super.populatePanelWithMission(mission);
            if (mission) {
                this.descriptionText.text = mission.description;
                this.descriptionText.left = this.titleText.left;
                this.descriptionText.visible = true;
                this.bar.setProgress(mission.progressProportion);
                this.bar.visible = true;
            }
            else {
                this.descriptionText.visible = false;
                this.bar.visible = false;
            }
        }
        updatePendingProgress() {
            return __awaiter(this, void 0, void 0, function* () {
                return yield this.bar.animateProgress(Math.min(1, this.getMission().pendingProgressProportion + this.getMission().progressProportion), this.shutdownSignal);
            });
        }
    }
    exports.MissionPanelDetail = MissionPanelDetail;
});
define("component/missions/MissionRewardPanel", ["require", "exports", "Game", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "logic/mission/MissionRewardType", "switchback-common/util/ScaleUtil"], function (require, exports, Game_48, TextUtil_21, LocalizationManager_45, MissionRewardType_2, ScaleUtil_36) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionRewardPanel extends Phaser.Group {
        constructor(game, mission) {
            super(game);
            this.bgPanel = new Phaser.Image(this.game, 0, 0, "mission-reward-window");
            this.bgPanel.width = this.game.width * 0.8;
            this.bgPanel.scale.y = this.bgPanel.scale.x;
            this.bgPanel.anchor.set(0.5, 0);
            this.add(this.bgPanel);
            const rewardHeader = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.075, LocalizationManager_45.LocalizationManager.Instance.text("mission.reward"), {
                font: Game_48.Game.DEFAULT_FONT,
                fontSize: Game_48.Game.FONT_SIZE_SMALL,
                fill: "#1992C9"
            });
            rewardHeader.anchor.set(0.5);
            this.add(rewardHeader);
            TextUtil_21.TextUtil.scaleDownSingleLineTextToWidth(rewardHeader, this.bgPanel.width * 0.9);
            this.rewardText = new Phaser.Text(this.game, -this.bgPanel.width * 0.2, this.bgPanel.height * 0.475, mission.rewardDescription, Object.assign({}, Game_48.Game.DEFAULT_FONT_STYLE, { fontSize: Game_48.Game.FONT_SIZE_MEDIUM, wordWrap: true, wordWrapWidth: this.bgPanel.width * 0.45, align: 'center' }));
            this.rewardText.lineSpacing = -this.game.height * 0.02;
            this.rewardText.anchor.set(0.5);
            TextUtil_21.TextUtil.scaleDownTextToHeight(this.rewardText, this.bgPanel.height * 0.8);
            this.add(this.rewardText);
            const rewardIconTag = mission.rewardType === MissionRewardType_2.MissionRewardType.PARTS ? 'ship-part-icon' : 'race-icon';
            this.rewardIcon = new Phaser.Image(this.game, this.bgPanel.width * 0.25, this.bgPanel.height * 0.525, rewardIconTag);
            this.rewardIcon.anchor.set(0.5);
            ScaleUtil_36.scaleToBounds(this.rewardIcon, this.bgPanel.width * 0.5, this.bgPanel.height * 0.7);
            this.add(this.rewardIcon);
        }
    }
    exports.MissionRewardPanel = MissionRewardPanel;
});
define("component/button/TextButton", ["require", "exports", "switchback-common/util/TextUtil", "Game", "switchback-common/util/PromiseUtil"], function (require, exports, TextUtil_22, Game_49, PromiseUtil_59) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextButton extends Phaser.Group {
        constructor(game, buttonText, textSize, shutdownSignal = null) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal(); // the signal for this button
            this.shutdownSignal = shutdownSignal;
            this.inputEnableChildren = true;
            this.onChildInputDown.add(() => {
                if (!this.ignoreChildInput) {
                    this.game.sound.play("cta-button");
                }
                this.onButtonClickedSignal.dispatch();
            }, this);
            const background = new Phaser.Image(game, 0, 0, "button-continue");
            background.anchor.set(0.5);
            this.add(background);
            const text = new Phaser.Text(this.game, 0, 0, buttonText, {
                font: Game_49.Game.DEFAULT_FONT,
                fill: Game_49.Game.DEFAULT_FONT_COLOUR,
                fontSize: textSize || Game_49.Game.FONT_SIZE_LARGE,
                stroke: Game_49.Game.DEFAULT_FONT_STROKE,
                strokeThickness: this.game.width * 0.01
            });
            TextUtil_22.TextUtil.scaleDownSingleLineTextToWidth(text, background.width * 0.9);
            text.anchor.set(0.5);
            this.add(text);
            this.alpha = 0;
            this.ignoreChildInput = true;
        }
        reset() {
            this.alpha = 0;
            if (!this.origScale) {
                this.origScale = this.scale.x;
            }
            this.scale.set(0);
        }
        show(durationMS = 350) {
            return __awaiter(this, void 0, void 0, function* () {
                this.reset();
                let isShutdown = false;
                yield Promise.all([
                    PromiseUtil_59.PromiseUtil.tween(this.game, this.game.add.tween(this)
                        .to({ alpha: 1 }, durationMS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_59.PromiseUtil.tween(this.game, this.game.add.tween(this.scale)
                        .to({ x: this.origScale, y: this.origScale }, durationMS, Phaser.Easing.Back.Out, true), this.shutdownSignal)
                ]).catch(() => { isShutdown = true; });
                this.ignoreChildInput = false;
                return !isShutdown;
            });
        }
        hide(durationMS = 150) {
            return __awaiter(this, void 0, void 0, function* () {
                this.ignoreChildInput = true;
                let isShutdown = false;
                yield Promise.all([
                    PromiseUtil_59.PromiseUtil.tween(this.game, this.game.add.tween(this)
                        .to({ alpha: 0 }, durationMS, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_59.PromiseUtil.tween(this.game, this.game.add.tween(this.scale)
                        .to({ x: 0, y: 0 }, durationMS, Phaser.Easing.Back.In, true), this.shutdownSignal)
                ]).catch(() => { isShutdown = true; });
                this.ignoreChildInput = true;
                return !isShutdown;
            });
        }
        destroy() {
            this.shutdownSignal = null;
            this.onChildInputDown.removeAll();
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.TextButton = TextButton;
});
define("component/dialog/MissionCompleteDialog", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "component/missions/MissionRewardPanel", "component/dialog/BasicSpaceDialog", "component/missions/MissionPanel", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "component/button/TextButton", "logic/SMAnalyticsManager", "logic/GameController"], function (require, exports, LocalizationManager_46, Game_50, MissionRewardPanel_1, BasicSpaceDialog_6, MissionPanel_2, ScaleUtil_37, PromiseUtil_60, TextButton_1, SMAnalyticsManager_24, GameController_27) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionCompleteDialog extends BasicSpaceDialog_6.BasicSpaceDialog {
        constructor(game, mission, shutdownSignal) {
            super(game, shutdownSignal, "");
            const yPadding = (this.game.height * 0.015);
            this.iconCircle = new Phaser.Image(this.game, 0, 0, 'mission-wheel');
            ScaleUtil_37.scaleToBounds(this.iconCircle, this.game.width * 0.5, this.game.height * 0.25);
            this.iconCircle.anchor.set(0.82, 0);
            this.content.add(this.iconCircle);
            this.animatingIcon = new Phaser.Image(this.game, this.iconCircle.centerX, this.iconCircle.centerY, 'boost-symbol');
            ScaleUtil_37.scaleToBounds(this.animatingIcon, this.iconCircle.width * 0.95, this.iconCircle.height * 0.95);
            this.animatingIcon.anchor.set(0.475, 0.5);
            this.animatingIcon.animations.add('enter', null, 24);
            this.content.add(this.animatingIcon);
            this.titleLines = new Phaser.Image(this.game, 0, this.iconCircle.centerY, 'text-lines');
            this.titleLines.anchor.set(0.5);
            this.titleLines.width = this.game.width * 0.8;
            this.titleLines.scale.y = this.titleLines.scale.x;
            this.content.add(this.titleLines);
            this.titleText1 = new Phaser.Text(this.game, this.titleLines.left + this.titleLines.width * 0.1, this.titleLines.centerY, LocalizationManager_46.LocalizationManager.Instance.text("mission.mission_complete_1"), Object.assign({}, Game_50.Game.DEFAULT_FONT_STYLE, { fontSize: Game_50.Game.FONT_SIZE_LARGE }));
            this.titleText1.anchor.set(0, 0.45);
            this.content.add(this.titleText1);
            this.titleText2 = new Phaser.Text(this.game, this.titleLines.right, this.titleText1.bottom, LocalizationManager_46.LocalizationManager.Instance.text("mission.mission_complete_2"), Object.assign({}, Game_50.Game.DEFAULT_FONT_STYLE, { fontSize: Game_50.Game.FONT_SIZE_LARGE, fill: Game_50.Game.FONT_COLOUR_BLUE }));
            this.titleText2.anchor.set(1, 0.2);
            this.content.add(this.titleText2);
            this.missionPanel = new MissionPanel_2.MissionPanel(this.game, "mission-window", this.shutdownSignal);
            this.missionPanel.setMission(mission);
            this.missionPanel.centerX = 0;
            this.missionPanel.top = this.titleText2.bottom + yPadding;
            this.content.add(this.missionPanel);
            this.rewardPanel = new MissionRewardPanel_1.MissionRewardPanel(this.game, mission);
            this.rewardPanel.centerX = 0;
            this.rewardPanel.top = this.missionPanel.bottom + yPadding;
            this.content.add(this.rewardPanel);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.button = new TextButton_1.TextButton(this.game, LocalizationManager_46.LocalizationManager.Instance.text("buttons.continue"));
            this.button.width = this.game.width * 0.8;
            this.button.scale.y = this.button.scale.x;
            this.button.x = 0;
            this.button.top = this.rewardPanel.bottom + yPadding;
            this.content.add(this.button);
            // Resize dialog bg with 2.75% screen height as vertical padding...
            this.resizeBackground(true, 0.0275);
        }
        reset() {
            this.titleLines.alpha = 0;
            this.titleText1.alpha = 0;
            this.titleText2.alpha = 0;
            this.iconCircle.alpha = 0;
            this.animatingIcon.alpha = 0;
            this.animatingIconOrigScale = this.animatingIcon.scale.x;
            this.missionPanelOrigScale = this.missionPanel.scale.x;
            this.missionPanel.scale.set(0);
            this.rewardPanelOrigScale = this.rewardPanel.scale.x;
            this.rewardPanel.scale.set(0);
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MissionCompleteDialog] show()`);
                SMAnalyticsManager_24.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub/missions/complete', 'MissionComplete', {
                    [SMAnalyticsManager_24.SMAnalyticsManager.GAME_SCORE]: GameController_27.GameController.Instance.missionManager.getNumMissionsCompleted()
                });
                this.reset();
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                /**
                 * Perform some more reset-like operations here - has to
                 * happen after show, as show() tries to position with
                 * centerX and these operations throw out the balance of the dialog
                 */
                this.titleText1.x -= this.game.width * 0.15;
                this.titleText2.x += this.game.width * 0.15;
                this.animatingIcon.scale.set(5);
                this.game.sound.play("missions_complete_anim");
                // Animate everything in!
                yield Promise.all([
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.iconCircle)
                        .to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.titleLines)
                        .to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true), this.shutdownSignal),
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.titleText1)
                        .to({ alpha: 1, x: this.titleText1.x + this.game.width * 0.15 }, 250, Phaser.Easing.Back.Out, true), this.shutdownSignal),
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.titleText2)
                        .to({ alpha: 1, x: this.titleText2.x - this.game.width * 0.15 }, 250, Phaser.Easing.Back.Out, true), this.shutdownSignal),
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.animatingIcon)
                        .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true, 250), this.shutdownSignal),
                    PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.animatingIcon.scale)
                        .to({ x: this.animatingIconOrigScale, y: this.animatingIconOrigScale }, 350, Phaser.Easing.Back.Out, true, 250), this.shutdownSignal).then(() => {
                        this.animatingIcon.animations.play('enter');
                    })
                ]).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_60.PromiseUtil.time(this.game, 450, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.missionPanel.scale)
                    .to({ x: this.missionPanelOrigScale, y: this.missionPanelOrigScale }, 400, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this.rewardPanel.scale)
                    .to({ x: this.rewardPanelOrigScale, y: this.rewardPanelOrigScale }, 400, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield this.button.show();
                if (this.isShutdown) {
                    return;
                }
                this.button.onButtonClickedSignal.addOnce(() => {
                    this.onButtonClickedSignal.dispatch();
                });
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_24.SMAnalyticsManager.Instance.trackAction('Continue', 'MissionComplete', 'Continue', '', {
                    [SMAnalyticsManager_24.SMAnalyticsManager.GAME_SCORE]: GameController_27.GameController.Instance.missionManager.getNumMissionsCompleted()
                });
                this.button.hide();
                this.button.onButtonClickedSignal.removeAll();
                yield _super("hide").call(this);
            });
        }
        destroy() {
            this.button.onButtonClickedSignal.removeAll();
            if (this.onButtonClickedSignal) {
                this.onButtonClickedSignal.dispose();
            }
            super.destroy();
        }
    }
    exports.MissionCompleteDialog = MissionCompleteDialog;
});
define("component/dialog/MissionDialog", ["require", "exports", "switchback-common/localization/LocalizationManager", "component/missions/MissionPanelDetail", "Game", "component/dialog/BasicSpaceDialog", "logic/GameController", "switchback-common/util/PromiseUtil", "logic/mission/MissionManager", "component/dialog/MissionCompleteDialog", "component/button/TextButton", "logic/mission/MissionRewardType", "logic/SMAnalyticsManager"], function (require, exports, LocalizationManager_47, MissionPanelDetail_1, Game_51, BasicSpaceDialog_7, GameController_28, PromiseUtil_61, MissionManager_2, MissionCompleteDialog_1, TextButton_2, MissionRewardType_3, SMAnalyticsManager_25) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MissionDialog extends BasicSpaceDialog_7.BasicSpaceDialog {
        constructor(game, missions, shutdownSignal) {
            super(game, shutdownSignal, "");
            this.missionPanels = [];
            const yPadding = this.game.height * 0.015;
            const titleText = new Phaser.Text(this.game, 0, 0, LocalizationManager_47.LocalizationManager.Instance.text("mission.title"), Object.assign({}, Game_51.Game.DEFAULT_FONT_STYLE, { fontSize: Game_51.Game.FONT_SIZE_LARGE, align: 'center' }));
            titleText.anchor.set(0.5, 0);
            const lightbeam = new Phaser.Image(this.game, 0, titleText.centerY, 'lightbeam');
            lightbeam.anchor.set(0.5);
            lightbeam.width = this.game.width * 0.75;
            lightbeam.scale.y = lightbeam.scale.x;
            this.content.add(lightbeam);
            this.content.add(titleText);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.button = new TextButton_2.TextButton(this.game, LocalizationManager_47.LocalizationManager.Instance.text("buttons.home"));
            this.button.width = this.game.width * 0.8;
            this.button.scale.y = this.button.scale.x;
            this.content.add(this.button);
            this.button.centerX = 0;
            let yPos = titleText.bottom;
            for (let i = 0; i < MissionManager_2.MissionManager.MAX_NUM_SIMULTANEOUS_MISSIONS; i++) {
                const panel = new MissionPanelDetail_1.MissionPanelDetail(this.game, this.shutdownSignal);
                panel.setMission(missions[i]);
                panel.centerX = 0;
                panel.y = yPos;
                yPos += panel.height + yPadding;
                this.content.add(panel);
                this.missionPanels.push(panel);
            }
            this.button.top = this.missionPanels[this.missionPanels.length - 1].bottom + yPadding;
            this.resizeBackground(true, 0.03);
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MissionDialog] show()`);
                SMAnalyticsManager_25.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub/missions', 'Missions', {
                    [SMAnalyticsManager_25.SMAnalyticsManager.GAME_SCORE]: GameController_28.GameController.Instance.missionManager.getNumMissionsCompleted()
                });
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.updateAllMissions();
                if (this.isShutdown) {
                    return;
                }
                yield this.replaceCompletedMissions();
                if (this.isShutdown) {
                    return;
                }
                yield this.button.show();
                if (this.isShutdown) {
                    return;
                }
                this.button.onButtonClickedSignal.addOnce(() => {
                    this.onButtonClickedSignal.dispatch();
                });
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_25.SMAnalyticsManager.Instance.trackAction('Back', 'Missions', 'Back', '', {
                    [SMAnalyticsManager_25.SMAnalyticsManager.GAME_SCORE]: GameController_28.GameController.Instance.missionManager.getNumMissionsCompleted()
                });
                this.button.hide();
                this.button.onButtonClickedSignal.removeAll();
                yield _super("hide").call(this);
            });
        }
        updateAllMissions() {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < this.missionPanels.length; i++) {
                    yield this.updateMission(this.missionPanels[i]);
                    if (this.isShutdown) {
                        return;
                    }
                }
            });
        }
        replaceCompletedMissions() {
            return __awaiter(this, void 0, void 0, function* () {
                const panelsToReplace = this.getReplaceableMissionPanels();
                const newMissions = GameController_28.GameController.Instance.missionManager.getNewMissions(panelsToReplace.length);
                for (let i = 0; i < panelsToReplace.length; i++) {
                    yield panelsToReplace[i].replaceMission(newMissions[i]);
                    if (this.isShutdown) {
                        return;
                    }
                }
            });
        }
        getReplaceableMissionPanels() {
            return this.missionPanels.filter((panel) => {
                return panel.getMission() ? !panel.getMission().isActive : false;
            });
        }
        /**
         * If the panel is showing a mission, and that
         * mission has some pending progress to award,
         * then update it!
         */
        updateMission(missionPanel) {
            return __awaiter(this, void 0, void 0, function* () {
                let didCompleteMission = false;
                if (missionPanel.getMission() && missionPanel.getMission().pendingProgress > 0) {
                    let isShutdown = !(yield missionPanel.updatePendingProgress());
                    if (isShutdown) {
                        return;
                    }
                    missionPanel.getMission().awardPendingProgress();
                    if (missionPanel.getMission().hasBeenCompleted) {
                        didCompleteMission = true;
                        // Show the mission complete dialog and wait for them to dismiss it
                        const missionCompleteDialog = new MissionCompleteDialog_1.MissionCompleteDialog(this.game, missionPanel.getMission(), this.shutdownSignal);
                        this.game.add.existing(missionCompleteDialog);
                        yield missionCompleteDialog.show();
                        if (missionCompleteDialog.isShutdown) {
                            this.isShutdown = true;
                            return;
                        }
                        yield PromiseUtil_61.PromiseUtil.signal(missionCompleteDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { isShutdown = true; });
                        if (isShutdown) {
                            return;
                        }
                        // As soon as they've dismissed the mission complete dialog, award the rewards and mark it as done!
                        missionPanel.getMission().isActive = false;
                        if (missionPanel.getMission().rewardType === MissionRewardType_3.MissionRewardType.PARTS) {
                            GameController_28.GameController.Instance.addPendingShipParts(missionPanel.getMission().rewardValue);
                        }
                        else if (missionPanel.getMission().rewardType === MissionRewardType_3.MissionRewardType.RACE) {
                            GameController_28.GameController.Instance.state.isSpaceRaceUnlocked = true;
                            SMAnalyticsManager_25.SMAnalyticsManager.Instance.trackAction('Installation_Unlocked', 'Installation', 'Unlocked', 'SpaceRace', {
                                [SMAnalyticsManager_25.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace"
                            });
                        }
                        // Clean up the dialog
                        yield missionCompleteDialog.hide();
                        if (missionCompleteDialog.isShutdown) {
                            this.isShutdown = true;
                            return;
                        }
                        missionCompleteDialog.destroy();
                    }
                }
                return didCompleteMission;
            });
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.MissionDialog = MissionDialog;
});
define("component/dialog/CodeReminderDialog", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "component/dialog/BasicSpaceDialog", "component/button/TextButton", "switchback-common/util/TextUtil"], function (require, exports, LocalizationManager_48, Game_52, BasicSpaceDialog_8, TextButton_3, TextUtil_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeReminderDialog extends BasicSpaceDialog_8.BasicSpaceDialog {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal, "");
            const yPadding = this.game.height * 0.015;
            const titleText = new Phaser.Text(this.game, 0, 0, LocalizationManager_48.LocalizationManager.Instance.text("code_redeem.reminder_title"), Object.assign({}, Game_52.Game.DEFAULT_FONT_STYLE, { fontSize: Game_52.Game.FONT_SIZE_LARGE, align: 'center' }));
            TextUtil_23.TextUtil.scaleDownSingleLineTextToWidth(titleText, this.backplate.width * 0.90);
            titleText.centerX = this.game.width * 0.5;
            this.content.add(titleText);
            const codePhoto = new Phaser.Image(this.game, 0, 0, "code-example");
            codePhoto.width = this.game.width * 0.7;
            codePhoto.scale.y = codePhoto.scale.x;
            codePhoto.centerX = this.game.width * 0.5;
            codePhoto.top = titleText.bottom = yPadding;
            this.content.add(codePhoto);
            const text = new Phaser.Text(this.game, 0, 0, LocalizationManager_48.LocalizationManager.Instance.text("code_redeem.reminder"), Object.assign({}, Game_52.Game.DEFAULT_FONT_STYLE, { fontSize: Game_52.Game.FONT_SIZE_MEDIUM, align: 'center', strokeThickness: Game_52.Game.DEFAULT_STROKE_THICKNESS, wordWrap: true, wordWrapWidth: this.game.width * 0.7 }));
            text.centerX = this.game.width * 0.5;
            text.top = codePhoto.bottom + yPadding;
            this.content.add(text);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.button = new TextButton_3.TextButton(this.game, LocalizationManager_48.LocalizationManager.Instance.text("buttons.ok"), null, this.shutdownSignal);
            this.button.width = this.game.width * 0.8;
            this.button.scale.y = this.button.scale.x;
            this.content.add(this.button);
            this.button.centerX = this.game.width * 0.5;
            this.button.top = text.bottom + yPadding;
            this.resizeBackground(true, 0.03);
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[CodeReminderDialog] show()`);
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.button.show();
                if (this.isShutdown) {
                    return;
                }
                this.button.onButtonClickedSignal.addOnce(() => {
                    this.onButtonClickedSignal.dispatch();
                });
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.button.hide();
                this.button.onButtonClickedSignal.removeAll();
                yield _super("hide").call(this);
            });
        }
        destroy() {
            if (this.onButtonClickedSignal) {
                this.onButtonClickedSignal.dispose();
            }
            super.destroy();
        }
    }
    exports.CodeReminderDialog = CodeReminderDialog;
});
define("component/CountdownPanel", ["require", "exports", "switchback-common/util/TextUtil", "Game", "switchback-common/localization/LocalizationManager"], function (require, exports, TextUtil_24, Game_53, LocalizationManager_49) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CountdownPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            const label = new Phaser.Text(this.game, 0, 0, LocalizationManager_49.LocalizationManager.Instance.text("race.race_timer_prefix"), Object.assign({}, Game_53.Game.DEFAULT_FONT_STYLE, { fontSize: Game_53.Game.FONT_SIZE_MEDIUM }));
            this.add(label);
            this.countdownText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, Game_53.Game.DEFAULT_FONT_STYLE, { fontSize: Game_53.Game.FONT_SIZE_LARGE }));
            this.countdownText.left = label.right;
            this.countdownText.centerY = label.centerY;
            this.add(this.countdownText);
            this.alpha = 0;
            // Set a fake time so we can get the sizing right
            this.setTimeMs(1);
        }
        setTimeMs(timeMs) {
            const time = Math.round(timeMs * 0.001);
            if (time > 0) {
                this.countdownText.fill = Game_53.Game.DEFAULT_FONT_COLOUR;
                this.countdownText.text = TextUtil_24.TextUtil.formattedTime(time);
            }
            else {
                this.countdownText.fill = Game_53.Game.FONT_COLOUR_BLUE;
                this.countdownText.text = LocalizationManager_49.LocalizationManager.Instance.text("race.ready");
            }
        }
        show() {
            this.game.add.tween(this).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
        }
        hide() {
            this.game.add.tween(this).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
        }
    }
    exports.CountdownPanel = CountdownPanel;
});
define("component/button/HubButton", ["require", "exports", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "logic/GameController"], function (require, exports, Game_54, ScaleUtil_38, PromiseUtil_62, GameController_29) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HubButton extends Phaser.Group {
        constructor(game, buttonFrame, primaryText, subText, primaryTextSize, textColor, shutdownSignal) {
            super(game);
            this.shutdownSignal = shutdownSignal;
            this.isShutdown = false;
            this.onButtonClickedSignal = new Phaser.Signal();
            this.ignoreChildInput = false;
            /**
             * Create button components
             */
            this.mainButton = new Phaser.Image(this.game, 0, 0, 'hub-buttons', buttonFrame);
            this.mainButton.inputEnabled = true;
            ScaleUtil_38.scaleToBounds(this.mainButton, this.game.width * 0.5, this.game.height * 0.25);
            this.mainButton.anchor.set(0.5);
            this.upgradeCounter = new Phaser.Image(this.game, 0, 0, "hub-button-upgrade-counter");
            this.upgradeCounter.anchor.set(1);
            this.upgradeCounter.width = this.mainButton.width * 0.2;
            this.upgradeCounter.scale.y = this.upgradeCounter.scale.x;
            this.upgradeCounter.centerY = this.mainButton.top + this.mainButton.height * 0.1;
            this.upgradeCounter.centerX = -this.mainButton.width * 0.2;
            this.upgradeCounter.visible = false;
            this.upgradeCounterText = new Phaser.Text(this.game, 0, 0, GameController_29.GameController.Instance.numShipPartsToAward.toString(), Object.assign({}, Game_54.Game.DEFAULT_FONT_STYLE, { fill: Game_54.Game.FONT_COLOUR_BLUE, strokeThickness: 0, fontSize: Game_54.Game.FONT_SIZE_MEDIUM }));
            this.upgradeCounterText.anchor.set(1);
            this.upgradeCounterText.centerX = this.upgradeCounter.centerX;
            this.upgradeCounterText.centerY = this.upgradeCounter.centerY;
            this.upgradeCounterText.visible = false;
            this.mainButtonBackplate = new Phaser.Image(this.game, this.mainButton.left + this.mainButton.width * 0.75, this.mainButton.centerY, 'hub-button-backplate');
            this.mainButtonBackplate.anchor.set(0, 0.45);
            this.mainButtonBackplate.visible = false;
            this.mainButtonBackplate.inputEnabled = true;
            ScaleUtil_38.scaleToBounds(this.mainButtonBackplate, this.game.width * 0.6, this.mainButton.height * 0.7);
            this.mainText = new Phaser.Text(this.game, this.mainButton.centerX, this.mainButtonBackplate.top + this.mainButtonBackplate.height * 0.35, primaryText, {
                font: Game_54.Game.DEFAULT_FONT,
                stroke: Game_54.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_54.Game.FONT_STROKE_LARGE,
                fill: textColor,
                fontSize: primaryTextSize,
                align: 'center'
            });
            this.mainText.alpha = 0;
            this.mainText.anchor.set(0.5);
            this.subText = new Phaser.Text(this.game, this.mainButton.centerX, this.mainButtonBackplate.top + this.mainButtonBackplate.height * 0.8, subText, {
                font: Game_54.Game.DEFAULT_FONT,
                stroke: Game_54.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_54.Game.DEFAULT_STROKE_THICKNESS,
                fill: textColor,
                fontSize: Game_54.Game.FONT_SIZE_SMALL,
                align: 'center'
            });
            this.subText.alpha = 0;
            this.subText.anchor.set(0.5);
            this.add(this.mainButtonBackplate);
            this.add(this.mainText);
            this.add(this.subText);
            this.add(this.mainButton);
            this.add(this.upgradeCounter);
            this.add(this.upgradeCounterText);
            this.onChildInputDown.add(this.onButtonClickedSignal.dispatch, this.onButtonClickedSignal);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                /**
                 * Start idle animations
                 */
                yield PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.mainButton.scale).from({ x: 0, y: 0 }, 250, Phaser.Easing.Cubic.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                if (GameController_29.GameController.Instance.hasShipPartsToAward) {
                    // Show and pulse upgrade counter
                    this.upgradeCounter.visible = true;
                    this.game.add.tween(this.upgradeCounter.scale)
                        .to({ x: this.upgradeCounter.scale.x * 0.9, y: this.upgradeCounter.scale.y * 0.9 }, 450, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                    this.upgradeCounterText.visible = true;
                    this.game.add.tween(this.upgradeCounterText)
                        .to({ x: this.upgradeCounterText.x * 0.95, y: this.upgradeCounterText.y * 0.95 }, 450, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                }
                this.game.add.tween(this.mainButton.scale)
                    .to({ x: this.mainButton.scale.x * 0.95, y: this.mainButton.scale.y * 0.95 }, 450, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                this.mainButtonBackplate.visible = true;
                yield PromiseUtil_62.PromiseUtil.tween2(this.game.add.tween(this.mainButtonBackplate.scale)
                    .from({ x: 0 }, 250, Phaser.Easing.Cubic.Out, true), this.shutdownSignal).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield Promise.all([
                    PromiseUtil_62.PromiseUtil.tween2(this.game.add.tween(this.mainText)
                        .to({ right: this.mainButtonBackplate.left + this.mainButtonBackplate.width * 0.775, alpha: 1 }, 250, Phaser.Easing.Cubic.Out, true), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween2(this.game.add.tween(this.subText)
                        .to({ x: this.mainButtonBackplate.left + this.mainButtonBackplate.width * 0.48, alpha: 1 }, 250, Phaser.Easing.Cubic.Out, true, 125), this.shutdownSignal)
                ]).catch(() => { });
            });
        }
        animateButtonClick(yoyo) {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_62.PromiseUtil.tween2(this.game.add.tween(this.mainButton.scale)
                        .to({ x: this.mainButton.scale.x * 0.9, y: this.mainButton.scale.y * 0.9 }, 100, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, yoyo), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween2(this.game.add.tween(this.upgradeCounter.scale)
                        .to({ x: this.upgradeCounter.scale.x * 0.9, y: this.upgradeCounter.scale.y }, 100, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, yoyo), this.shutdownSignal)
                ]).catch(() => { });
            });
        }
        updateUpgradeInfo() {
            if (GameController_29.GameController.Instance.hasShipPartsToAward) {
                this.upgradeCounter.visible = true;
                this.upgradeCounterText.visible = true;
                this.upgradeCounterText.text = GameController_29.GameController.Instance.numShipPartsToAward.toString();
            }
            else {
                this.upgradeCounter.visible = false;
                this.upgradeCounterText.visible = false;
            }
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this.mainButton.scale);
                this.game.tweens.removeFrom(this.upgradeCounter.scale);
                this.game.tweens.removeFrom(this.upgradeCounterText.scale);
                yield Promise.all([
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.mainText)
                        .to({ x: this.mainButton.centerX, alpha: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.subText)
                        .to({ x: this.mainButton.centerX * 0.48, alpha: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.mainButtonBackplate.scale)
                        .to({ x: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
                if (this.isShutdown) {
                    return;
                }
                yield Promise.all([
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.mainButton.scale).to({ x: 0, y: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.upgradeCounter.scale).to({ x: 0, y: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal),
                    PromiseUtil_62.PromiseUtil.tween(this.game, this.game.add.tween(this.upgradeCounterText.scale).to({ x: 0, y: 0 }, 150, Phaser.Easing.Cubic.In, true), this.shutdownSignal)
                ]).catch(() => { this.isShutdown = true; });
            });
        }
        destroy() {
            this.shutdownSignal = null;
            if (this.onButtonClickedSignal) {
                this.onButtonClickedSignal.dispose();
                this.onButtonClickedSignal = null;
            }
            this.upgradeCounter.destroy();
            this.upgradeCounterText.destroy();
            this.mainButton.destroy();
            this.subText.destroy();
            this.mainText.destroy();
            super.destroy();
        }
    }
    exports.HubButton = HubButton;
});
define("component/coderedeem/tutorial/BaseCodeTutorialStep", ["require", "exports", "component/dialog/BasicSpaceDialog"], function (require, exports, BasicSpaceDialog_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BaseCodeTutorialStep extends BasicSpaceDialog_9.BasicSpaceDialog {
        constructor(game, shutdownSignal, descriptionText, customScale) {
            super(game, shutdownSignal, descriptionText, customScale);
            this.inputEnableChildren = true;
            this.onButtonClickedSignal = new Phaser.Signal();
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.BaseCodeTutorialStep = BaseCodeTutorialStep;
});
define("component/coderedeem/tutorial/CodeTutorialStep1", ["require", "exports", "switchback-common/util/ScaleUtil", "component/button/ArrowButton", "Game", "switchback-common/localization/LocalizationManager", "component/coderedeem/tutorial/BaseCodeTutorialStep"], function (require, exports, ScaleUtil_39, ArrowButton_14, Game_55, LocalizationManager_50, BaseCodeTutorialStep_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeTutorialStep1 extends BaseCodeTutorialStep_1.BaseCodeTutorialStep {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal, "");
            const spacing = (this.game.height * 0.02);
            const title = new Phaser.Text(this.game, 0, 0, LocalizationManager_50.LocalizationManager.Instance.text("code_tutorial.1_title"), {
                font: Game_55.Game.DEFAULT_FONT,
                fill: Game_55.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_55.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            title.setScaleMinMax(1);
            const infoText = new Phaser.Text(this.game, 0, 0, LocalizationManager_50.LocalizationManager.Instance.text("code_tutorial.1_look_out"), {
                font: Game_55.Game.DEFAULT_FONT,
                fill: Game_55.Game.FONT_COLOUR_BLUE,
                fontSize: Game_55.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                stroke: 'black',
                strokeThickness: this.game.width * 0.015
            });
            infoText.setScaleMinMax(1);
            const tripleLightbeams = new Phaser.Image(this.game, 0, 0, "lightbeams");
            ScaleUtil_39.scaleToBounds(tripleLightbeams, this.game.width, this.game.height);
            tripleLightbeams.top = 0;
            tripleLightbeams.anchor.set(0.5, 0);
            this.content.add(tripleLightbeams);
            title.anchor.set(0.5, 0);
            title.centerY = tripleLightbeams.centerY;
            this.content.add(title);
            infoText.top = tripleLightbeams.bottom + spacing;
            infoText.anchor.set(0.5, 0);
            this.content.add(infoText);
            const launchImage = new Phaser.Image(this.game, 0, 0, "codes-tutorial-sample1");
            ScaleUtil_39.scaleToBounds(launchImage, this.game.width * 0.8, this.game.height * 0.5);
            launchImage.top = infoText.bottom + spacing;
            launchImage.anchor.set(0.5, 0);
            this.content.add(launchImage);
            this.nextButton = new ArrowButton_14.ArrowButton(this.game, "buttons.next", true, 70);
            ScaleUtil_39.scaleToBounds(this.nextButton, this.game.width * 0.5, this.game.height * 0.15);
            this.nextButton.top = launchImage.bottom + (spacing * 2);
            this.nextButton.centerX = 0;
            this.nextButton.inputEnableChildren = true;
            this.nextButton.onButtonClickedSignal.addOnce(this.onButtonClickedSignal.dispatch, this.onButtonClickedSignal);
            this.content.add(this.nextButton);
            this.resizeBackground();
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.nextButton.show();
            });
        }
    }
    exports.CodeTutorialStep1 = CodeTutorialStep1;
});
define("component/coderedeem/tutorial/CodeTutorialStep4", ["require", "exports", "switchback-common/util/ScaleUtil", "component/button/ArrowButton", "Game", "switchback-common/localization/LocalizationManager", "component/coderedeem/tutorial/BaseCodeTutorialStep"], function (require, exports, ScaleUtil_40, ArrowButton_15, Game_56, LocalizationManager_51, BaseCodeTutorialStep_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeTutorialStep4 extends BaseCodeTutorialStep_2.BaseCodeTutorialStep {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal, "");
            const spacing = (this.game.height * 0.02);
            const title = new Phaser.Text(this.game, 0, 0, LocalizationManager_51.LocalizationManager.Instance.text("code_tutorial.1_title"), {
                font: Game_56.Game.DEFAULT_FONT,
                fill: Game_56.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_56.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            title.setScaleMinMax(1);
            const infoText = new Phaser.Text(this.game, 0, 0, LocalizationManager_51.LocalizationManager.Instance.text("code_tutorial.4_upgrade"), {
                font: Game_56.Game.DEFAULT_FONT,
                fill: Game_56.Game.FONT_COLOUR_BLUE,
                fontSize: Game_56.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                stroke: 'black',
                strokeThickness: this.game.width * 0.015
            });
            infoText.setScaleMinMax(1);
            const tripleLightbeams = new Phaser.Image(this.game, 0, 0, "lightbeams");
            ScaleUtil_40.scaleToBounds(tripleLightbeams, this.game.width, this.game.height);
            tripleLightbeams.top = 0;
            tripleLightbeams.anchor.set(0.5, 0);
            this.content.add(tripleLightbeams);
            title.anchor.set(0.5, 0);
            title.centerY = tripleLightbeams.centerY;
            this.content.add(title);
            infoText.top = tripleLightbeams.bottom + spacing;
            infoText.anchor.set(0.5, 0);
            this.content.add(infoText);
            const launchImage = new Phaser.Image(this.game, 0, 0, "codes-tutorial-sample4");
            ScaleUtil_40.scaleToBounds(launchImage, this.game.width * 0.8, this.game.height * 0.5);
            launchImage.top = infoText.bottom + spacing;
            launchImage.anchor.set(0.5, 0);
            this.content.add(launchImage);
            this.nextButton = new ArrowButton_15.ArrowButton(this.game, "buttons.next", true, 70);
            ScaleUtil_40.scaleToBounds(this.nextButton, this.game.width * 0.5, this.game.height * 0.15);
            this.nextButton.top = launchImage.bottom + (spacing * 2);
            this.nextButton.centerX = 0;
            this.nextButton.inputEnableChildren = true;
            this.nextButton.onButtonClickedSignal.addOnce(this.onButtonClickedSignal.dispatch, this.onButtonClickedSignal);
            this.content.add(this.nextButton);
            this.resizeBackground();
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.nextButton.show();
            });
        }
    }
    exports.CodeTutorialStep4 = CodeTutorialStep4;
});
define("component/fx/Taps", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Taps extends Phaser.Group {
        // Returns a new Pulse FX for the tap, recycle the old ones
        getTap(maxScale = 0.002) {
            // Tries to get a dead sprite, or creates a new one
            let fx = this.getFirstDead();
            if (fx) {
                fx.revive();
            }
            else {
                fx = new Phaser.Sprite(this.game, 0, 0, "switches-fx");
                fx.anchor.set(0.5);
                this.add(fx);
            }
            const duration = 500;
            // FX Scale
            const scale = this.game.width * maxScale;
            fx.scale.y = fx.scale.x = scale * 0.1;
            this.game.add.tween(fx.scale).to({ x: scale, y: scale }, duration, Phaser.Easing.Linear.None, true);
            // FX Alpha
            fx.alpha = 1;
            let tween = this.game.add.tween(fx);
            tween.to({ alpha: 0 }, duration / 2.0, Phaser.Easing.Linear.None, true, duration / 2.0, 0, false);
            tween.onComplete.addOnce(fx.kill, fx); // Kill it when done
            return fx;
        }
    }
    exports.Taps = Taps;
});
define("component/coderedeem/tutorial/CodeTutorialStep3", ["require", "exports", "switchback-common/util/ScaleUtil", "component/button/ArrowButton", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "component/fx/Taps", "component/coderedeem/tutorial/BaseCodeTutorialStep"], function (require, exports, ScaleUtil_41, ArrowButton_16, Game_57, LocalizationManager_52, PromiseUtil_63, Taps_1, BaseCodeTutorialStep_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeTutorialStep3 extends BaseCodeTutorialStep_3.BaseCodeTutorialStep {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal, "");
            const spacing = (this.game.height * 0.02);
            const title = new Phaser.Text(this.game, 0, 0, LocalizationManager_52.LocalizationManager.Instance.text("code_tutorial.1_title"), {
                font: Game_57.Game.DEFAULT_FONT,
                fill: Game_57.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_57.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            title.setScaleMinMax(1);
            const infoText = new Phaser.Text(this.game, 0, 0, LocalizationManager_52.LocalizationManager.Instance.text("code_tutorial.3_enter"), {
                font: Game_57.Game.DEFAULT_FONT,
                fill: Game_57.Game.FONT_COLOUR_BLUE,
                fontSize: Game_57.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                stroke: 'black',
                strokeThickness: this.game.width * 0.015
            });
            infoText.setScaleMinMax(1);
            const tripleLightbeams = new Phaser.Image(this.game, 0, 0, "lightbeams");
            ScaleUtil_41.scaleToBounds(tripleLightbeams, this.game.width, this.game.height);
            tripleLightbeams.top = 0;
            tripleLightbeams.anchor.set(0.5, 0);
            this.content.add(tripleLightbeams);
            title.anchor.set(0.5, 0);
            title.centerY = tripleLightbeams.centerY;
            this.content.add(title);
            infoText.top = tripleLightbeams.bottom + spacing;
            infoText.anchor.set(0.5, 0);
            this.content.add(infoText);
            const launchImage = new Phaser.Image(this.game, 0, 0, "codes-tutorial-sample3");
            ScaleUtil_41.scaleToBounds(launchImage, this.game.width * 0.8, this.game.height * 0.5);
            launchImage.top = infoText.bottom + spacing;
            launchImage.anchor.set(0.5, 0);
            this.content.add(launchImage);
            this.image = launchImage;
            this.nextButton = new ArrowButton_16.ArrowButton(this.game, "buttons.next", true, 70);
            ScaleUtil_41.scaleToBounds(this.nextButton, this.game.width * 0.5, this.game.height * 0.15);
            this.nextButton.top = launchImage.bottom + (spacing * 2);
            this.nextButton.centerX = 0;
            this.nextButton.inputEnableChildren = true;
            this.nextButton.onButtonClickedSignal.addOnce(this.onButtonClickedSignal.dispatch, this.onButtonClickedSignal);
            this.content.add(this.nextButton);
            this.resizeBackground();
        }
        codeAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                const stepMS = 1000;
                this.taps = new Taps_1.Taps(this.game);
                const tapPos = [
                    [0, 0],
                    [0.445, 0.475],
                    [0.553, 0.626],
                    // [0.571, 0.628],
                    [0.445, 0.772],
                    [0.667, 0.777]
                ];
                const codes = new Phaser.Sprite(this.game, 0, 0, "codes-tutorial-3anim");
                ScaleUtil_41.scaleToBounds(codes, this.image.width * 0.45, this.image.height * 0.45);
                codes.centerX = this.image.centerX;
                codes.top = this.image.top + (this.image.height * 0.15);
                this.content.add(codes);
                const frames = this.game.cache.getFrameCount("codes-tutorial-3anim");
                let frame = 0;
                while (this.alive) {
                    // this way we play the tap before setting the frame
                    yield PromiseUtil_63.PromiseUtil.time(this.game, stepMS / 2, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                    if (this.isShutdown) {
                        return;
                    }
                    frame = (frame + 1) % frames;
                    if (frame > 0 && this.alive) {
                        const tap = this.taps.getTap(0.001);
                        tap.centerX = this.image.left + (this.image.width * tapPos[frame][0]);
                        tap.centerY = this.image.top + (this.image.height * tapPos[frame][1]);
                        this.content.add(tap);
                        yield PromiseUtil_63.PromiseUtil.time(this.game, stepMS / 2, this.shutdownSignal).catch(() => { this.isShutdown = true; });
                        if (this.isShutdown) {
                            return;
                        }
                    }
                    codes.frame = frame;
                }
            });
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.codeAnimation();
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.nextButton.show();
            });
        }
        destroy() {
            this.alive = false;
            super.destroy();
        }
    }
    exports.CodeTutorialStep3 = CodeTutorialStep3;
});
define("component/coderedeem/tutorial/CodeTutorialStep2", ["require", "exports", "switchback-common/util/ScaleUtil", "component/button/ArrowButton", "Game", "switchback-common/localization/LocalizationManager", "component/coderedeem/tutorial/BaseCodeTutorialStep"], function (require, exports, ScaleUtil_42, ArrowButton_17, Game_58, LocalizationManager_53, BaseCodeTutorialStep_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeTutorialStep2 extends BaseCodeTutorialStep_4.BaseCodeTutorialStep {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal, "");
            const spacing = (this.game.height * 0.02);
            const title = new Phaser.Text(this.game, 0, 0, LocalizationManager_53.LocalizationManager.Instance.text("code_tutorial.1_title"), {
                font: Game_58.Game.DEFAULT_FONT,
                fill: Game_58.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_58.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                stroke: 'black',
                strokeThickness: this.game.width * 0.02
            });
            title.setScaleMinMax(1);
            const infoText = new Phaser.Text(this.game, 0, 0, LocalizationManager_53.LocalizationManager.Instance.text("code_tutorial.2_press_code"), {
                font: Game_58.Game.DEFAULT_FONT,
                fill: Game_58.Game.FONT_COLOUR_BLUE,
                fontSize: Game_58.Game.FONT_SIZE_MEDIUM,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.8,
                stroke: 'black',
                strokeThickness: this.game.width * 0.015
            });
            infoText.setScaleMinMax(1);
            const tripleLightbeams = new Phaser.Image(this.game, 0, 0, "lightbeams");
            ScaleUtil_42.scaleToBounds(tripleLightbeams, this.game.width, this.game.height);
            tripleLightbeams.top = 0;
            tripleLightbeams.anchor.set(0.5, 0);
            this.content.add(tripleLightbeams);
            title.anchor.set(0.5, 0);
            title.centerY = tripleLightbeams.centerY;
            this.content.add(title);
            infoText.top = tripleLightbeams.bottom + spacing;
            infoText.anchor.set(0.5, 0);
            this.content.add(infoText);
            const launchImage = new Phaser.Image(this.game, 0, 0, "codes-tutorial-sample2");
            ScaleUtil_42.scaleToBounds(launchImage, this.game.width * 0.8, this.game.height * 0.5);
            launchImage.top = infoText.bottom + spacing;
            launchImage.anchor.set(0.5, 0);
            this.content.add(launchImage);
            this.nextButton = new ArrowButton_17.ArrowButton(this.game, "buttons.next", true, 70);
            ScaleUtil_42.scaleToBounds(this.nextButton, this.game.width * 0.5, this.game.height * 0.15);
            this.nextButton.top = launchImage.bottom + (spacing * 2);
            this.nextButton.centerX = 0;
            this.nextButton.inputEnableChildren = true;
            this.nextButton.onButtonClickedSignal.addOnce(this.onButtonClickedSignal.dispatch, this.onButtonClickedSignal);
            this.content.add(this.nextButton);
            this.resizeBackground();
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                if (this.isShutdown) {
                    return;
                }
                yield this.nextButton.show();
            });
        }
    }
    exports.CodeTutorialStep2 = CodeTutorialStep2;
});
define("component/coderedeem/tutorial/CodeRedeemTutorial", ["require", "exports", "component/coderedeem/tutorial/CodeTutorialStep1", "switchback-common/util/PromiseUtil", "logic/GameController", "component/coderedeem/tutorial/CodeTutorialStep4", "component/coderedeem/tutorial/CodeTutorialStep3", "component/coderedeem/tutorial/CodeTutorialStep2"], function (require, exports, CodeTutorialStep1_1, PromiseUtil_64, GameController_30, CodeTutorialStep4_1, CodeTutorialStep3_1, CodeTutorialStep2_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeRedeemTutorial extends Phaser.Group {
        constructor(game, shutdownSignal) {
            super(game);
            this.screens = [];
            this.screens.push(new CodeTutorialStep1_1.CodeTutorialStep1(this.game, shutdownSignal));
            this.screens.push(new CodeTutorialStep2_1.CodeTutorialStep2(this.game, shutdownSignal));
            this.screens.push(new CodeTutorialStep3_1.CodeTutorialStep3(this.game, shutdownSignal));
            this.screens.push(new CodeTutorialStep4_1.CodeTutorialStep4(this.game, shutdownSignal));
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('intro-start');
                for (let screen of this.screens) {
                    this.add(screen);
                    yield screen.show();
                    yield PromiseUtil_64.PromiseUtil.signal(screen.onButtonClickedSignal);
                    yield screen.hide();
                    screen.destroy();
                }
                GameController_30.GameController.Instance.state.ftueCodeTutorial = true;
            });
        }
    }
    exports.CodeRedeemTutorial = CodeRedeemTutorial;
});
define("component/dialog/ExitConfirmDialog", ["require", "exports", "switchback-common/util/ScaleUtil", "component/dialog/ConfirmDialog", "logic/GameController", "config/SpaceMountainConfig", "switchback-common/enum/Park", "model/SpaceSaveState"], function (require, exports, ScaleUtil_43, ConfirmDialog_3, GameController_31, SpaceMountainConfig_12, Park_10, SpaceSaveState_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ExitConfirmDialog extends ConfirmDialog_3.ConfirmDialog {
        constructor(game, descriptionText, confirmButtonText, declineButtonText, shutdownSignal, extra = null) {
            super(game, descriptionText, confirmButtonText, declineButtonText, shutdownSignal, extra);
            let queueIndex = (SpaceMountainConfig_12.SpaceMountainConfig.Instance.park === Park_10.Park.DLR ||
                GameController_31.GameController.Instance.state.queueType === SpaceSaveState_5.Queue.PRIMARY ? 1 : 2);
            let queueImage = new Phaser.Image(this.game, 0, 0, "merge-" + queueIndex);
            queueImage.inputEnabled = true;
            ScaleUtil_43.scaleToBounds(queueImage, this.game.width * 0.88, this.game.height * 0.75);
            this.content.add(queueImage);
            queueImage.centerX = this.dialogText.centerX;
            queueImage.top = this.dialogText.bottom + this.game.height * 0.02;
            this.buttonGroup.top = queueImage.bottom + this.game.height * 0.02;
            this.resizeBackground();
        }
    }
    exports.ExitConfirmDialog = ExitConfirmDialog;
});
define("switchback-common/util/TimeUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class TimeUtil {
        /**
         * Outputs time as a string in minutes:seconds format.
         * Seconds are padded, minutes are not
         *
         * @param timeMS In milliseconds
         */
        static toDigitalTime(timeMS) {
            timeMS = Math.round(timeMS * 0.001);
            const minutes = Math.floor(timeMS / 60);
            const seconds = timeMS % 60;
            return minutes + ":" + (seconds < 10 ? "0" + seconds : seconds).toString();
        }
    }
    exports.TimeUtil = TimeUtil;
});
define("component/ship/ShipUpgradeAnimation", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "Game", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "logic/GameController", "switchback-common/util/TimeUtil"], function (require, exports, ScaleUtil_44, LocalizationManager_54, Game_59, PromiseUtil_65, TextUtil_25, GameController_32, TimeUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ShipUpgradeAnimation extends Phaser.Group {
        constructor(game) {
            super(game);
            // shade
            this.shade = new Phaser.Graphics(this.game, 0, 0);
            this.shade.beginFill(0x000000, 0.75);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.alpha = 0;
            this.add(this.shade);
            // next race assets
            this.raceTimerPanel = new Phaser.Group(this.game);
            this.add(this.raceTimerPanel);
            this.bgPanel = new Phaser.Image(this.game, 0, 0, "mission-reward-window");
            this.bgPanel.width = this.game.width * 0.8;
            this.bgPanel.scale.y = this.bgPanel.scale.x;
            this.raceTimerPanel.add(this.bgPanel);
            this.raceTimeTitle = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.004, LocalizationManager_54.LocalizationManager.Instance.text("hub.race_title"), {
                font: Game_59.Game.DEFAULT_FONT,
                fontSize: Game_59.Game.FONT_SIZE_SMALL,
                fill: "#1992C9"
            });
            TextUtil_25.TextUtil.scaleDownSingleLineTextToWidth(this.raceTimeTitle, this.bgPanel.width * 0.9);
            this.raceTimeTitle.centerX = this.bgPanel.width * 0.5;
            this.raceTimerPanel.add(this.raceTimeTitle);
            this.raceTimeCaption = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, Game_59.Game.DEFAULT_FONT_STYLE, { fontSize: Game_59.Game.FONT_SIZE_MEDIUM, align: 'center' }));
            // Ship improved animation assets
            this.shipAnimationPanel = new Phaser.Group(this.game);
            this.add(this.shipAnimationPanel);
            this.iconCircle = new Phaser.Image(this.game, 0, 0, 'mission-wheel');
            ScaleUtil_44.scaleToBounds(this.iconCircle, this.game.width * 0.65, this.game.height * 0.33);
            this.iconCircle.anchor.set(0.75, 0);
            this.shipAnimationPanel.add(this.iconCircle);
            this.animatingIcon = new Phaser.Image(this.game, this.iconCircle.centerX, this.iconCircle.centerY, 'boost-symbol');
            ScaleUtil_44.scaleToBounds(this.animatingIcon, this.iconCircle.width * 0.95, this.iconCircle.height * 0.95);
            this.animatingIcon.anchor.set(0.475, 0.5);
            this.animatingIcon.animations.add('enter', null, 24);
            this.shipAnimationPanel.add(this.animatingIcon);
            this.titleLines = new Phaser.Image(this.game, 0, this.iconCircle.centerY, 'text-lines');
            this.titleLines.anchor.set(0.5);
            this.titleLines.width = this.game.width * 0.8;
            this.titleLines.scale.y = this.titleLines.scale.x;
            this.shipAnimationPanel.add(this.titleLines);
            this.titleText1 = new Phaser.Text(this.game, 0, this.titleLines.centerY, LocalizationManager_54.LocalizationManager.Instance.text("hub.ship_upgrade_performance"), Object.assign({}, Game_59.Game.DEFAULT_FONT_STYLE, { fontSize: Game_59.Game.FONT_SIZE_LARGE }));
            this.titleText1.centerX = this.titleLines.centerX;
            this.titleText1.anchor.set(0, 0.45);
            this.shipAnimationPanel.add(this.titleText1);
            this.titleText2 = new Phaser.Text(this.game, this.titleLines.right, this.titleText1.bottom, LocalizationManager_54.LocalizationManager.Instance.text("hub.ship_upgrade_improved"), Object.assign({}, Game_59.Game.DEFAULT_FONT_STYLE, { fontSize: Game_59.Game.FONT_SIZE_LARGE, fill: Game_59.Game.FONT_COLOUR_BLUE }));
            this.titleText2.anchor.set(1, 0.2);
            this.shipAnimationPanel.add(this.titleText2);
            // align assets
            this.shipAnimationPanel.centerX = this.game.width * 0.5;
            this.shipAnimationPanel.top = this.game.height * 0.15;
            this.raceTimerPanel.centerX = this.game.width * 0.5;
            this.raceTimerPanel.bottom = this.game.height * ShipUpgradeAnimation.RACE_PANEL_Y_OFFSET;
            this.visible = false;
        }
        reset() {
            this.titleLines.alpha = 0;
            this.titleText1.alpha = 0;
            this.titleText2.alpha = 0;
            this.iconCircle.alpha = 0;
            this.animatingIcon.alpha = 0;
            this.animatingIconOrigScale = this.animatingIcon.scale.x;
            this.raceTimerPanelOrigScale = this.raceTimerPanel.scale.x;
            this.raceTimerPanel.scale.set(0);
            // calculate race time
            let canRace = GameController_32.GameController.Instance.raceController.canShipParticipateInRace();
            let timeUntilRace = GameController_32.GameController.Instance.raceController.getTimeUntilScheduledRaceMS();
            if (!canRace) {
                // race hasn't been unlocked yet
                this.raceTimeCaption.text = LocalizationManager_54.LocalizationManager.Instance.text("hub.race_unlock");
                this.raceTimeCaption.wordWrap = true;
                this.raceTimeCaption.wordWrapWidth = this.bgPanel.width * 0.95;
                TextUtil_25.TextUtil.scaleDownTextToHeight(this.raceTimeCaption, this.bgPanel.height * 0.75);
                this.raceTimeCaption.centerX = this.bgPanel.width * 0.5;
                this.raceTimeCaption.top = this.bgPanel.height * 0.20;
                this.raceTimerPanel.add(this.raceTimeCaption);
            }
            else {
                // race has been unlocked, show time to next race
                this.raceTimeCaption.text = LocalizationManager_54.LocalizationManager.Instance.text("hub.next_race_in");
                TextUtil_25.TextUtil.scaleDownSingleLineTextToWidth(this.raceTimeCaption, this.bgPanel.width * 0.95);
                this.raceTimeCaption.centerX = this.bgPanel.width * 0.5;
                this.raceTimeCaption.top = this.bgPanel.height * 0.16;
                this.raceTimerPanel.add(this.raceTimeCaption);
                let timerText;
                if (timeUntilRace <= 0) {
                    timerText = LocalizationManager_54.LocalizationManager.Instance.text("hub.race_timer_now");
                }
                else {
                    timerText = LocalizationManager_54.LocalizationManager.Instance.text("hub.race_timer") + TimeUtil_1.TimeUtil.toDigitalTime(timeUntilRace);
                }
                this.raceTime = new Phaser.Text(this.game, 0, 0, timerText, Object.assign({}, Game_59.Game.DEFAULT_FONT_STYLE, { fontSize: Game_59.Game.FONT_SIZE_EXTRA_LARGE, align: 'center' }));
                TextUtil_25.TextUtil.scaleDownSingleLineTextToWidth(this.raceTime, this.bgPanel.width * 0.95);
                this.raceTime.centerX = this.bgPanel.width * 0.5;
                this.raceTime.top = this.raceTimeCaption.bottom;
                this.raceTimerPanel.add(this.raceTime);
            }
        }
        show(shutdownSignal) {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                // absorbs all clicks on screen so that nothing else can be clicked in game
                this.shade.inputEnabled = true;
                this.game.add.tween(this.shade).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true);
                this.reset();
                this.game.sound.play("race_speed_increased");
                let isShutdown = false;
                yield PromiseUtil_65.PromiseUtil.time(this.game, 500, shutdownSignal).catch(() => { isShutdown = true; }); // initial pause after parts are applied
                if (isShutdown) {
                    return;
                }
                /**
                 * Perform some more reset-like operations here - has to
                 * happen after show, as show() tries to position with
                 * centerX and these operations throw out the balance of the dialog
                 */
                this.titleText1.x -= this.game.width * 0.15;
                this.titleText2.x += this.game.width * 0.15;
                this.animatingIcon.scale.set(5);
                // this.game.sound.play("missions_complete_anim");
                // Animate everything in!
                this.game.add.tween(this.iconCircle)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.titleLines)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.titleText1)
                    .to({ alpha: 1, centerX: this.titleLines.centerX }, 350, Phaser.Easing.Back.Out, true);
                this.game.add.tween(this.titleText2)
                    .to({ alpha: 1, x: this.titleLines.right }, 250, Phaser.Easing.Back.Out, true);
                this.game.add.tween(this.animatingIcon)
                    .to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true, 350);
                this.game.add.tween(this.animatingIcon.scale)
                    .to({ x: this.animatingIconOrigScale, y: this.animatingIconOrigScale }, 450, Phaser.Easing.Back.Out, true, 250).onComplete.addOnce(() => {
                    this.animatingIcon.animations.play('enter');
                });
                yield PromiseUtil_65.PromiseUtil.time(this.game, 950, shutdownSignal).catch(() => { isShutdown = true; }); // wait for the above tweens plus an extra 500 delay
                if (isShutdown) {
                    return;
                }
                yield PromiseUtil_65.PromiseUtil.tween(this.game, this.game.add.tween(this.raceTimerPanel.scale)
                    .to({ x: this.raceTimerPanelOrigScale, y: this.raceTimerPanelOrigScale }, 400, Phaser.Easing.Back.Out, true).onUpdateCallback(() => {
                    // can't scale a group around an anchor so use this
                    this.raceTimerPanel.centerX = this.game.width * 0.5;
                    this.raceTimerPanel.bottom = this.game.height * ShipUpgradeAnimation.RACE_PANEL_Y_OFFSET;
                }), shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                yield PromiseUtil_65.PromiseUtil.time(this.game, 1500, shutdownSignal).catch(() => { isShutdown = true; }); // hold to take in the information
                if (isShutdown) {
                    return;
                }
                // hide everything again
                this.game.add.tween(this.titleText1)
                    .to({ alpha: 0, centerX: this.titleText1.x + this.game.width * 0.15 }, 350, Phaser.Easing.Back.In, true);
                this.game.add.tween(this.titleText2)
                    .to({ alpha: 0, x: this.titleText2.x - this.game.width * 0.15 }, 250, Phaser.Easing.Back.In, true);
                this.game.add.tween(this.iconCircle)
                    .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true, 200);
                this.game.add.tween(this.titleLines)
                    .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true, 200);
                this.game.add.tween(this.animatingIcon)
                    .to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true, 200);
                this.game.add.tween(this.raceTimerPanel.scale)
                    .to({ x: 0, y: 0 }, 400, Phaser.Easing.Back.In, true, 500).onUpdateCallback(() => {
                    // can't scale a group around an anchor so use this
                    this.raceTimerPanel.centerX = this.game.width * 0.5;
                    this.raceTimerPanel.bottom = this.game.height * ShipUpgradeAnimation.RACE_PANEL_Y_OFFSET;
                });
                this.game.add.tween(this.shade).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_65.PromiseUtil.time(this.game, 500, shutdownSignal).catch(() => { }); // wait for the above tweens to complete
                this.visible = false;
            });
        }
    }
    ShipUpgradeAnimation.RACE_PANEL_Y_OFFSET = 0.756;
    exports.ShipUpgradeAnimation = ShipUpgradeAnimation;
});
define("state/HubState", ["require", "exports", "state/SpaceBaseState", "StateNames", "logic/GameController", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "Game", "switchback-common/localization/LocalizationManager", "component/ship/ShipSprite", "config/SpaceMountainConfig", "component/dialog/ConfirmDialog", "switchback-common/enum/Park", "component/dialog/MissionDialog", "component/dialog/CodeReminderDialog", "component/CountdownPanel", "logic/RaceController", "logic/SMAnalyticsManager", "component/button/HubButton", "component/coderedeem/tutorial/CodeRedeemTutorial", "component/dialog/ExitConfirmDialog", "component/dialog/TwoTextDialog", "model/SpaceSaveState", "component/ship/ShipUpgradeAnimation"], function (require, exports, SpaceBaseState_7, StateNames_24, GameController_33, PromiseUtil_66, ScaleUtil_45, Game_60, LocalizationManager_55, ShipSprite_4, SpaceMountainConfig_13, ConfirmDialog_4, Park_11, MissionDialog_1, CodeReminderDialog_1, CountdownPanel_1, RaceController_3, SMAnalyticsManager_26, HubButton_1, CodeRedeemTutorial_1, ExitConfirmDialog_1, TwoTextDialog_4, SpaceSaveState_6, ShipUpgradeAnimation_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HubState extends SpaceBaseState_7.SpaceBaseState {
        constructor() {
            super(...arguments);
            this.buttonFrames = new Map([
                [StateNames_24.StateNames.GAME_MEMORY, 'commander'],
                [StateNames_24.StateNames.GAME_RELAY_INTRO, 'relay'],
                [StateNames_24.StateNames.GAME_PIPES, 'navigating'],
                [StateNames_24.StateNames.GAME_DOCKING, 'docking'],
                [StateNames_24.StateNames.GAME_ASTEROIDS, 'asteroids'],
                [StateNames_24.StateNames.GAME_ENERGY_BALLS, 'engineering']
            ]);
        }
        init() {
            this.stateName = StateNames_24.StateNames.HUB;
            super.init();
            this.isCountdownPanelShowing = false;
            this.isShowingRaceButton = false;
        }
        preload() {
            super.preload();
            this.game.load.audio("race_speed_increased", ["assets/sound/hub/race_speed_increased.ogg", "assets/sound/hub/race_speed_increased.m4a"]);
            this.game.load.audio("intro-start", ["assets/sound/ui/intro-start.m4a", "assets/sound/ui/intro-start.ogg"]);
            this.game.load.audio("intro-end", ["assets/sound/ui/intro-end.m4a", "assets/sound/ui/intro-end.ogg"]);
            this.game.load.audio("missions_bar_fill", ["assets/sound/hub/missions_bar_fill.m4a", "assets/sound/hub/missions_bar_fill.ogg"]);
            this.game.load.audio("missions_complete_anim", ["assets/sound/hub/missions_complete_anim.m4a", "assets/sound/hub/missions_complete_anim.ogg"]);
            this.game.load.audio("missions_prebar_fill", ["assets/sound/hub/missions_prebar_fill.m4a", "assets/sound/hub/missions_prebar_fill.ogg"]);
            this.game.load.audio("button-appear", ["assets/sound/hub/button-appear.m4a", "assets/sound/hub/button-appear.ogg"]);
            this.game.load.audio("ship-fly-in", ["assets/sound/hub/ship-fly-in.m4a", "assets/sound/hub/ship-fly-in.ogg"]);
            this.game.load.audio("ship-take-off", ["assets/sound/hub/ship-take-off.m4a", "assets/sound/hub/ship-take-off.ogg"]);
            this.game.load.audio("click-minigame", ["assets/sound/ui/click-minigame.m4a", "assets/sound/ui/click-minigame.ogg"]);
            this.game.load.audio("click-launch", ["assets/sound/ui/click-launch.m4a", "assets/sound/ui/click-launch.ogg"]);
            for (let i = 1; i <= 3; i++) {
                this.game.load.audio(`upgrade_applied${i}`, [`assets/sound/hub/upgrade_applied${i}.m4a`, `assets/sound/hub/upgrade_applied${i}.ogg`]);
            }
            if (SpaceMountainConfig_13.SpaceMountainConfig.Instance.park === Park_11.Park.DLR) {
                this.load.image('ship-base', 'assets/ship/ship-base-dlr.png');
            }
            else if (SpaceMountainConfig_13.SpaceMountainConfig.Instance.park === Park_11.Park.WDW) {
                this.load.image('ship-base', 'assets/ship/ship-base-wdw.png');
            }
            this.load.atlas('slipstream-effect', 'assets/hub/slipstream-effect.png', 'assets/hub/slipstream-effect.json');
            this.load.atlas('hub-buttons', 'assets/button/hub-buttons.png', 'assets/button/hub-buttons.json');
            this.load.image('hub-button-backplate', 'assets/hub/hub-button-backplate.png');
            this.load.image('hub-button-upgrade-counter', 'assets/hub/upgrade_counter.png');
            this.load.image('hub-frame', 'assets/hub/hub-frame.png');
            this.load.image('bg-space-clean', 'assets/common/bg-space-clean.jpg');
            this.load.image('hub-grid', 'assets/hub/grid.png');
            this.load.atlas("ship-upgrades", "assets/ship/ship-upgrades.png", "assets/ship/ship-upgrades.json");
            this.load.atlas("bonus-upgrades", "assets/ship/bonus-upgrades.png", "assets/ship/bonus-upgrades.json");
            this.load.image('thruster-passive', 'assets/ship/thruster-passive.png');
            this.load.image('thruster-launch', 'assets/ship/thruster-launch.png');
            this.load.image("code-button", "assets/button/code_btn.png");
            this.load.image("code-button-glow", "assets/button/code_btn_glow.png");
            this.load.image("code-example", "assets/code-redeem/tutorial/sample_1.png");
            this.load.image("quit-button", "assets/button/quit_btn.png");
            this.load.image("back-button-small", "assets/button/back_button_small.png");
            this.load.image("confirm-button-small", "assets/button/confirm_button_small.png");
            this.load.image("missions-button", "assets/button/missions_button.png");
            this.load.spritesheet("boost-symbol", "assets/hub/boost-symbol.png", 256, 256);
            this.load.image("mission-reward-window", "assets/hub/mission-reward-window.png");
            this.load.image("mission-wheel", "assets/hub/mission-wheel.png");
            this.load.image("mission-window-tall", "assets/hub/mission-window-tall.png");
            this.load.image("mission-window", "assets/hub/mission-window.png");
            this.load.image("progress-bar-back", "assets/hub/progress-bar-back.png");
            this.load.image("progress-bar-fill", "assets/hub/progress-bar-fill.png");
            this.load.image("progress-bar-fill-change", "assets/hub/progress-bar-fill-change.png");
            this.load.image("progress-bar-frame", "assets/hub/progress-bar-frame.png");
            this.load.image("ship-part-icon", "assets/hub/ship-part-icon.png");
            this.load.image("race-icon", "assets/hub/race-icon.png");
            this.load.image("text-lines", "assets/hub/text-lines.png");
            // exit confirmation dialog
            if (SpaceMountainConfig_13.SpaceMountainConfig.Instance.park === Park_11.Park.DLR) {
                this.load.image("merge-1", "assets/queues/dlr-merge.png");
            }
            else {
                this.load.image("merge-1", "assets/queues/wdw-merge-1.png");
                this.load.image("merge-2", "assets/queues/wdw-merge-2.png");
            }
            // Code Redeem Tutorial
            // Back button assets
            this.game.load.image("next-circle", "assets/common/next_circle.png");
            this.game.load.image("next-arrow", "assets/common/next_arrow.png");
            this.game.load.image("next-line", "assets/common/next_line.png");
            // Success dialog assets
            this.game.load.image("lightbeams", "assets/common/lightbeams.png");
            // Tutorials
            this.game.load.image("codes-tutorial-sample1", "assets/code-redeem/tutorial/sample_1.png");
            this.game.load.image("codes-tutorial-sample2", "assets/code-redeem/tutorial/sample_2.png");
            this.game.load.image("codes-tutorial-sample3", "assets/code-redeem/tutorial/sample_3.png");
            this.game.load.image("codes-tutorial-sample4", "assets/code-redeem/tutorial/sample_4.png");
            this.game.load.image("switches-fx", "assets/minigame/relay/switches/switch_fx.png");
            this.game.load.spritesheet('codes-tutorial-3anim', 'assets/code-redeem/tutorial/sample_3_anim.png', 388, 109);
        }
        create() {
            super.create();
            // SWB-1361 send player to a race if they boot up the game post-merge
            if (GameController_33.GameController.Instance.state.hasMerged) {
                GameController_33.GameController.Instance.raceController.scheduleOnDeviceRaceFromLast(0);
                super.transitionToPreRaceState();
                return;
            }
            //////////////////////////////////////
            // BACKGROUND
            //////////////////////////////////////
            const bg = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, 'bg-space-clean');
            const tileScale = this.game.width / bg.texture.width;
            bg.tileScale.set(tileScale, tileScale);
            bg.autoScroll(0, this.game.height * 0.5);
            //////////////////////////////////////
            // GRID BACKGROUND OVERLAY
            //////////////////////////////////////
            this.gridBg = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, 'hub-grid');
            this.gridBg.alpha = 0;
            //////////////////////////////////////
            // UI FRAME
            //////////////////////////////////////
            const frame = new Phaser.Image(this.game, 0, 0, 'hub-frame');
            frame.width = this.game.width;
            frame.height = this.game.height;
            const levelTitle = new Phaser.Text(this.game, 0, 0, LocalizationManager_55.LocalizationManager.Instance.text("hub.ship_level"), Object.assign({}, Game_60.Game.DEFAULT_FONT_STYLE, { fill: Game_60.Game.FONT_COLOUR_BLUE, fontSize: Game_60.Game.FONT_SIZE_SMALL }));
            levelTitle.centerX = this.game.width * 0.5;
            levelTitle.centerY = this.game.height * 0.685;
            this.levelDisplay = new Phaser.Text(this.game, 0, 0, GameController_33.GameController.Instance.getShipLevelText(), Object.assign({}, Game_60.Game.DEFAULT_FONT_STYLE, { fontSize: Game_60.Game.FONT_SIZE_LARGE }));
            this.levelDisplay.anchor.set(0.5);
            this.levelDisplay.centerX = this.game.width * 0.5;
            this.levelDisplay.centerY = this.game.height * 0.74;
            //////////////////////////////////////
            // SHIP
            //////////////////////////////////////
            this.shipGroup = new Phaser.Group(this.game);
            this.ship = new ShipSprite_4.ShipSprite(this.game, this.shutdownSignal);
            this.ship.width = this.game.width * 0.9;
            this.ship.scale.y = this.ship.scale.x;
            this.shipGroup.add(this.ship);
            this.shipGroup.x = this.game.width * 0.5;
            this.shipGroup.top = this.game.height;
            this.shipUpgradeAnimation = new ShipUpgradeAnimation_1.ShipUpgradeAnimation(this.game);
            //////////////////////////////////////
            // CODE BUTTON
            //////////////////////////////////////
            this.uiLayer = new Phaser.Group(this.game);
            this.uiLayer.width = this.game.width;
            this.uiLayer.height = this.game.height;
            const codeText = LocalizationManager_55.LocalizationManager.Instance.text("hub.code");
            const codeButtonText = new Phaser.Text(this.game, 0, 0, codeText, {
                font: Game_60.Game.DEFAULT_FONT,
                fill: Game_60.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_60.Game.FONT_SIZE_SMALL,
                stroke: Game_60.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_60.Game.FONT_STROKE_SMALL
            });
            codeButtonText.setScaleMinMax(1);
            this.codeButton = new Phaser.Image(this.game, 0, 0, "code-button");
            this.codeButton.width = this.game.width * 0.25;
            this.codeButton.scale.y = this.codeButton.scale.x;
            this.codeButton.inputEnabled = true;
            this.codeButton.left = this.game.width * 0.025;
            this.codeButton.top = this.game.height * 0.015;
            codeButtonText.centerX = this.codeButton.centerX;
            codeButtonText.centerY = this.codeButton.centerY;
            this.codeButtonGlow = new Phaser.Image(this.game, 0, 0, "code-button-glow");
            this.codeButtonGlow.scale = this.codeButton.scale;
            this.codeButtonGlow.centerX = this.codeButton.centerX;
            this.codeButtonGlow.centerY = this.codeButton.centerY;
            this.codeButtonGlow.alpha = 0;
            this.codeReminderDialog = new CodeReminderDialog_1.CodeReminderDialog(this.game, this.shutdownSignal);
            if (!GameController_33.GameController.Instance.codeReminderTimer) {
                GameController_33.GameController.Instance.codeReminderTimer = this.game.time.now;
            }
            //////////////////////////////////////
            // QUIT BUTTON
            //////////////////////////////////////
            const quitText = LocalizationManager_55.LocalizationManager.Instance.text("hub.quit");
            const quitButtonText = new Phaser.Text(this.game, 0, 0, quitText, {
                font: Game_60.Game.DEFAULT_FONT,
                fill: Game_60.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_60.Game.FONT_SIZE_SMALL,
                stroke: Game_60.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_60.Game.FONT_STROKE_SMALL
            });
            quitButtonText.setScaleMinMax(1);
            this.quitButton = new Phaser.Image(this.game, 0, 0, "quit-button");
            this.quitButton.scale.set(this.codeButton.scale.x);
            this.quitButton.right = this.game.width * 0.975;
            this.quitButton.top = this.codeButton.top;
            this.quitButton.inputEnabled = true;
            quitButtonText.centerX = this.quitButton.centerX;
            quitButtonText.centerY = this.quitButton.centerY;
            //////////////////////////////////////
            // MISSIONS
            //////////////////////////////////////
            this.missionDialog = new MissionDialog_1.MissionDialog(this.game, GameController_33.GameController.Instance.missionManager.getActiveMissions(), this.shutdownSignal);
            const missionText = LocalizationManager_55.LocalizationManager.Instance.text("hub.missions");
            const missionButtonText = new Phaser.Text(this.game, 0, 0, missionText, {
                font: Game_60.Game.DEFAULT_FONT,
                fill: Game_60.Game.DEFAULT_FONT_COLOUR,
                fontSize: Game_60.Game.FONT_SIZE_MEDIUM,
                stroke: Game_60.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_60.Game.FONT_STROKE_SMALL
            });
            missionButtonText.setScaleMinMax(1);
            this.missionButton = new Phaser.Image(this.game, 0, 0, "missions-button");
            this.missionButton.width = this.game.width * 0.5;
            this.missionButton.scale.y = this.missionButton.scale.x;
            this.missionButton.inputEnabled = true;
            this.missionButton.centerX = this.game.width * 0.5;
            this.missionButton.top = this.game.height * 0.06;
            missionButtonText.centerX = this.missionButton.centerX;
            missionButtonText.centerY = this.missionButton.centerY;
            //////////////////////////////////////
            // RACE COUNTDOWN
            //////////////////////////////////////
            this.countdownPanel = new CountdownPanel_1.CountdownPanel(this.game);
            ScaleUtil_45.scaleToBounds(this.countdownPanel, this.game.width * 0.4, this.game.height * 0.1);
            this.countdownPanel.centerX = this.game.width * 0.5;
            this.countdownPanel.top = this.game.height * 0.005;
            //////////////////////////////////////
            // SCENE ORDERING
            //////////////////////////////////////
            this.add.existing(bg);
            this.add.existing(this.gridBg);
            this.add.existing(this.shipGroup);
            this.add.existing(frame);
            this.uiLayer.add(this.levelDisplay);
            this.uiLayer.add(levelTitle);
            if (SpaceMountainConfig_13.SpaceMountainConfig.Instance.showCodeRedeem) {
                this.uiLayer.add(this.codeButton);
                this.uiLayer.add(this.codeButtonGlow);
                this.uiLayer.add(codeButtonText);
            }
            this.uiLayer.add(this.missionButton);
            this.uiLayer.add(missionButtonText);
            this.uiLayer.add(this.quitButton);
            this.uiLayer.add(quitButtonText);
            this.uiLayer.add(this.countdownPanel);
            this.add.existing(this.uiLayer);
            this.add.existing(this.shipUpgradeAnimation);
            this.add.existing(this.missionDialog);
            this.add.existing(this.codeReminderDialog);
            this.showStairsWarning = SpaceMountainConfig_13.SpaceMountainConfig.Instance.park === Park_11.Park.WDW &&
                GameController_33.GameController.Instance.state.queueType === SpaceSaveState_6.Queue.PRIMARY;
            this.start();
        }
        onCodeButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play("cta-button");
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('Redeem', 'Hub', 'Redeem', '', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                });
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(StateNames_24.StateNames.REDEEM_CODE);
            });
        }
        // After they've seen the forced mission screen, allow them to pop up missions at any time
        onMissionButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play("cta-button");
                yield this.missionDialog.show();
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('Missions', 'Hub', 'Missions', '', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                });
            });
        }
        onQuitButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play("cta-button");
                const confirmDialog = new ConfirmDialog_4.ConfirmDialog(this.game, LocalizationManager_55.LocalizationManager.Instance.text("exit_game_confirmation.exit_question"), LocalizationManager_55.LocalizationManager.Instance.text("buttons.quit"), LocalizationManager_55.LocalizationManager.Instance.text("buttons.back"), this.shutdownSignal);
                this.add.existing(confirmDialog);
                confirmDialog.show();
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('Exit', 'Hub', 'Exit', '', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                });
                const confirmed = yield PromiseUtil_66.PromiseUtil.signal(confirmDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.game.sound.play("cta-button");
                yield confirmDialog.hide();
                confirmDialog.destroy();
                const adobeAction = confirmed ? "Exit_Confirm" : "Exit_Cancel";
                const gaAction = confirmed ? "ExitConfirm" : "ExitCancel";
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction(adobeAction, 'Hub', gaAction);
                if (confirmed) {
                    yield SMAnalyticsManager_26.SMAnalyticsManager.Instance.endTiming("RocketRace", "RocketRace", 'Game', '');
                    PlayAPI.closeGame();
                }
            });
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                const gameController = GameController_33.GameController.Instance;
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                yield this.doMissions();
                if (this.isShutdown) {
                    return;
                }
                yield this.showShip();
                if (this.isShutdown) {
                    return;
                }
                if (gameController.hasShipPartsToAward) {
                    yield this.performShipUpgrades();
                    if (this.isShutdown) {
                        return;
                    }
                }
                gameController.exitBeaconTriggeredSignal.addOnce(this.showOutroConfirmation, this);
                /// This part handles going to outro
                // Depends of :  * ExitBeacon * Whether the user have already hit the merge point
                if (gameController.state.hasMerged) {
                    // if has merged, go to outro
                    this.game.state.start(StateNames_24.StateNames.OUTRO);
                    return;
                }
                else if (gameController.shouldShowExitReminder()) {
                    // if haven't merged, but an exit beacon gets triggered, show the confirmation
                    // notice that the beacon is ignored otherwise (which is good)
                    yield this.showOutroConfirmation();
                    if (this.isShutdown) {
                        return;
                    }
                }
                else if (!(gameController.state.ftueCodeTutorial || SpaceMountainConfig_13.SpaceMountainConfig.Instance.skipTutorials)) {
                    yield this.showCodeTutorial();
                    if (this.isShutdown) {
                        return;
                    }
                }
                else if ((gameController.state.shipBonusParts <= 0 && gameController.state.numGamePlays > gameController.gamePlayCountAtLastCodeReminder) ||
                    this.game.time.now > gameController.codeReminderTimer + HubState.CODE_REMINDER_TIMER) {
                    // show code reminder after every game until they find and enter a code, then revert to 5 minute timer
                    // the gamePlayCountAtLastCodeReminder ensures that a new game has been played (and they haven't just 
                    // been in and out of the Redeem Code state)
                    yield this.showCodeReminder();
                    if (this.isShutdown) {
                        return;
                    }
                    gameController.gamePlayCountAtLastCodeReminder = gameController.state.numGamePlays;
                    gameController.codeReminderTimer = this.game.time.now;
                }
                this.codeButton.events.onInputUp.add(this.onCodeButtonClicked, this);
                this.missionButton.events.onInputUp.add(this.onMissionButtonClicked, this);
                this.quitButton.events.onInputUp.add(this.onQuitButtonClicked, this);
                this.timerRefreshRef = setInterval(this.raceTimerRefresh.bind(this), 500);
                // If we can participate in a race, show the race button. And don't show the game button!
                if (gameController.raceController.canShipParticipateInRace() &&
                    gameController.raceController.getTimeUntilScheduledRaceMS() <= 0) {
                    this.isShowingRaceButton = true;
                    this.showRaceButton();
                }
                else {
                    this.showGameButton();
                }
            });
        }
        showCodeReminder() {
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/redeem/reminder', 'RedeemReminder');
                this.codeReminderDialog.show();
                yield PromiseUtil_66.PromiseUtil.signal(this.codeReminderDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.codeReminderDialog.hide();
                // Pulse the code button until they leave the hub
                this.game.add.tween(this.codeButtonGlow).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true, 0, Infinity, true);
            });
        }
        showCodeTutorial() {
            return __awaiter(this, void 0, void 0, function* () {
                const tutorial = new CodeRedeemTutorial_1.CodeRedeemTutorial(this.game, this.shutdownSignal);
                this.add.existing(tutorial);
                yield tutorial.show();
                if (this.isShutdown) {
                    return;
                }
                tutorial.destroy();
            });
        }
        doMissions() {
            return __awaiter(this, void 0, void 0, function* () {
                // Force show the missions screen if we have anything to award
                if (GameController_33.GameController.Instance.missionManager.hasPendingMissionProgressToAward()) {
                    yield this.missionDialog.show();
                    if (this.isShutdown) {
                        return;
                    }
                    yield PromiseUtil_66.PromiseUtil.signal(this.missionDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    this.missionDialog.hide();
                    // Makes sense to save that they've seen these animations
                    GameController_33.GameController.Instance.state.save();
                }
                // And dismiss it at any time too!
                this.missionDialog.onButtonClickedSignal.add(() => {
                    this.missionDialog.hide();
                }, this);
            });
        }
        showShip() {
            return __awaiter(this, void 0, void 0, function* () {
                this.ship.startWobble();
                yield this.ship.animateLaunch(10, 0);
                if (this.isShutdown) {
                    return;
                }
                this.ship.stopAnimateLaunch(350, 500);
                this.game.sound.play("ship-fly-in");
                yield PromiseUtil_66.PromiseUtil.tween2(this.game.add.tween(this.shipGroup)
                    .to({ y: this.game.height * 0.35 }, 750, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { });
            });
        }
        showOutroConfirmation() {
            return __awaiter(this, void 0, void 0, function* () {
                // Asks whether to exit or continue
                let dialog = new ExitConfirmDialog_1.ExitConfirmDialog(this.game, LocalizationManager_55.LocalizationManager.Instance.text("outro_confirmation.confirmation1"), LocalizationManager_55.LocalizationManager.Instance.text("buttons.yes"), LocalizationManager_55.LocalizationManager.Instance.text("buttons.no"), this.shutdownSignal);
                this.add.existing(dialog);
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'OutroAreYouHere');
                yield dialog.show();
                if (this.isShutdown) {
                    return;
                }
                yield PromiseUtil_66.PromiseUtil.signal(dialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction(dialog.isConfirmed ? "Declare_Yes" : "Declare_No", 'Location', dialog.isConfirmed ? "Yes" : "No", "Outro");
                yield dialog.hide();
                if (this.isShutdown) {
                    return;
                }
                dialog.destroy();
                // secondary confirmation
                if (dialog.isConfirmed) {
                    const confirmDialog = new ConfirmDialog_4.ConfirmDialog(this.game, LocalizationManager_55.LocalizationManager.Instance.text('outro_confirmation.confirmation2'), LocalizationManager_55.LocalizationManager.Instance.text('buttons.yes'), LocalizationManager_55.LocalizationManager.Instance.text('buttons.no'), this.shutdownSignal);
                    this.add.existing(confirmDialog);
                    SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/declare', 'OutroAreYouHere2');
                    yield confirmDialog.show();
                    if (this.isShutdown) {
                        return;
                    }
                    yield PromiseUtil_66.PromiseUtil.signal(confirmDialog.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction(confirmDialog.isConfirmed ? "Declare_Yes" : "Declare_No", 'Location', confirmDialog.isConfirmed ? "Yes" : "No", "Outro");
                    yield confirmDialog.hide();
                    if (this.isShutdown) {
                        return;
                    }
                    if (confirmDialog.isConfirmed) {
                        yield this.showOverlay();
                        if (this.isShutdown) {
                            return;
                        }
                        this.game.state.start(StateNames_24.StateNames.OUTRO);
                    }
                    confirmDialog.destroy();
                }
            });
        }
        showGameButton() {
            return __awaiter(this, void 0, void 0, function* () {
                const nextGame = GameController_33.GameController.Instance.getNextGame();
                this.metricGameName = SMAnalyticsManager_26.SMAnalyticsManager.gameNameMap.get(nextGame);
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub', 'Hub', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                });
                this.showButton(this.buttonFrames.get(nextGame), LocalizationManager_55.LocalizationManager.Instance.text('hub.play'), LocalizationManager_55.LocalizationManager.Instance.text("game_names." + nextGame), Game_60.Game.FONT_SIZE_LARGE, Game_60.Game.FONT_COLOUR_GREEN);
                this.mainHubButton.onButtonClickedSignal.removeAll();
                this.mainHubButton.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.onGameButtonClicked();
                    if (this.isShutdown) {
                        return;
                    }
                    SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('StartGame', 'Hub', 'Play', this.metricGameName, {
                        [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                    });
                    this.onPlayGameClicked(nextGame);
                }), this);
            });
        }
        showRaceButton() {
            return __awaiter(this, void 0, void 0, function* () {
                this.metricGameName = "SpaceRace";
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub', 'Hub', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.GAME_NAME_MINI]: this.metricGameName
                });
                this.showButton('race', LocalizationManager_55.LocalizationManager.Instance.text('hub.race'), '', Game_60.Game.FONT_SIZE_LARGE, Game_60.Game.FONT_COLOUR_BLUE);
                this.mainHubButton.onButtonClickedSignal.removeAll();
                this.mainHubButton.onButtonClickedSignal.addOnce(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.onGameButtonClicked();
                    if (this.isShutdown) {
                        return;
                    }
                    SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('Installation_Trigger', 'Installation', 'Trigger', this.metricGameName, {
                        [SMAnalyticsManager_26.SMAnalyticsManager.INSTALLATION_NAME]: this.metricGameName
                    });
                    this.transitionToPreRaceState();
                }), this);
            });
        }
        onGameButtonClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('click-minigame');
                // Activate the ship's booster!
                this.ship.animateLaunch(200, 200);
                // Pulse the button a lil'
                this.mainHubButton.animateButtonClick(false);
                // Animate the ship off-screen and hide the button
                this.game.sound.play("ship-take-off");
                yield Promise.all([
                    PromiseUtil_66.PromiseUtil.tween2(this.game.add.tween(this.shipGroup)
                        .to({ bottom: 0 }, 750, Phaser.Easing.Back.In, true), this.shutdownSignal),
                    this.mainHubButton.hide()
                ]).catch(() => { });
            });
        }
        performShipUpgrades() {
            return __awaiter(this, void 0, void 0, function* () {
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub/shipupgrade', 'ShipUpgrade');
                const fadeGridInTween = this.game.add.tween(this.gridBg).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                fadeGridInTween.onComplete.addOnce(() => {
                    this.game.add.tween(this.gridBg).to({ alpha: 0.65 }, 500, Phaser.Easing.Linear.None, true, 0, Infinity, true);
                });
                PromiseUtil_66.PromiseUtil.tween2(fadeGridInTween, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.showButton('upgrade', LocalizationManager_55.LocalizationManager.Instance.text('hub.upgrade'), '', Game_60.Game.FONT_SIZE_MEDIUM, Game_60.Game.FONT_COLOUR_BLUE);
                yield PromiseUtil_66.PromiseUtil.signal(this.mainHubButton.onButtonClickedSignal, this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackAction('Upgrade', 'Hub', 'Upgrade');
                this.game.sound.play('click-launch');
                yield this.mainHubButton.animateButtonClick(true);
                if (this.isShutdown) {
                    return;
                }
                this.mainHubButton.hide();
                SMAnalyticsManager_26.SMAnalyticsManager.Instance.trackState('content/game/inqueue/hub/shipupgrade/animation', 'ShipUpgradeAnimation', {
                    [SMAnalyticsManager_26.SMAnalyticsManager.CUTSCENE]: "UpgradeShip"
                });
                const levelPulse = this.game.add.tween(this.levelDisplay.scale).to({ x: this.levelDisplay.scale.x * 1.5, y: this.levelDisplay.scale.y * 1.5, }, 150, Phaser.Easing.Sinusoidal.InOut, false, 0, 0, true);
                while (GameController_33.GameController.Instance.hasShipPartsToAward) {
                    yield PromiseUtil_66.PromiseUtil.time(this.game, 400, this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                    yield this.ship.performUpgrade();
                    if (this.isShutdown) {
                        return;
                    }
                    yield levelPulse.start();
                    if (this.isShutdown) {
                        return;
                    }
                    this.levelDisplay.text = GameController_33.GameController.Instance.getShipLevelText();
                }
                this.mainHubButton.updateUpgradeInfo();
                // Save that they've applied these upgrades
                GameController_33.GameController.Instance.state.save();
                yield this.shipUpgradeAnimation.show(this.shutdownSignal);
                if (this.isShutdown) {
                    return;
                }
                this.game.tweens.removeFrom(this.gridBg);
                yield PromiseUtil_66.PromiseUtil.tween2(this.game.add.tween(this.gridBg).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true), this.shutdownSignal).catch(() => { });
            });
        }
        showButton(buttonFrame, primaryText, subText, primaryTextSize, textColor) {
            if (this.mainHubButton) {
                this.mainHubButton.destroy();
                this.mainHubButton = null;
            }
            this.mainHubButton = new HubButton_1.HubButton(this.game, buttonFrame, primaryText, subText, primaryTextSize, textColor, this.shutdownSignal);
            this.mainHubButton.x = (this.game.width - this.mainHubButton.width) * 0.5;
            this.mainHubButton.bottom = this.game.height;
            this.uiLayer.add(this.mainHubButton);
            this.game.sound.play("button-appear");
            this.mainHubButton.show();
        }
        /**
         * If there's a race that we can participate in, update the timer text.
         * If the timer text isn't showing, better show it!
         *
         * Otherwise, hide the timer if it's not hidden already.
         */
        raceTimerRefresh() {
            return __awaiter(this, void 0, void 0, function* () {
                const raceState = GameController_33.GameController.Instance.raceController.getRaceState();
                let startable = (raceState === RaceController_3.RaceState.QUEUED_NOT_STARTED) || (raceState === RaceController_3.RaceState.STARTABLE_NOT_STARTED);
                startable = startable && GameController_33.GameController.Instance.raceController.canShipParticipateInRace();
                let timeUntilRace = GameController_33.GameController.Instance.raceController.getTimeUntilScheduledRaceMS();
                if (startable) {
                    if (timeUntilRace < 0 && !this.isShowingRaceButton) {
                        this.isShowingRaceButton = true;
                        yield this.mainHubButton.hide();
                        if (this.isShutdown) {
                            return;
                        }
                        this.showRaceButton();
                    }
                    if (!this.isCountdownPanelShowing) {
                        this.isCountdownPanelShowing = true;
                        this.countdownPanel.show();
                    }
                    this.countdownPanel.setTimeMs(Math.max(0, timeUntilRace));
                }
                else {
                    if (this.isCountdownPanelShowing) {
                        yield this.countdownPanel.hide();
                        if (this.isShutdown) {
                            return;
                        }
                        this.isCountdownPanelShowing = false;
                    }
                }
            });
        }
        onPlayGameClicked(gameName) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.showStairsWarning && (gameName === StateNames_24.StateNames.GAME_ENERGY_BALLS ||
                    gameName === StateNames_24.StateNames.GAME_ASTEROIDS || gameName === StateNames_24.StateNames.GAME_DOCKING ||
                    gameName === StateNames_24.StateNames.GAME_PIPES)) {
                    const confirmDialog = new TwoTextDialog_4.TwoTextDialog(this.game, LocalizationManager_55.LocalizationManager.Instance.text("watch_the_stairs.title"), LocalizationManager_55.LocalizationManager.Instance.text("watch_the_stairs.body"), "buttons.continue", this.shutdownSignal);
                    confirmDialog.onButtonClickedSignal.add(() => __awaiter(this, void 0, void 0, function* () {
                        this.game.sound.play("cta-button");
                        yield confirmDialog.hide();
                        if (this.isShutdown) {
                            return;
                        }
                        this.startGame(gameName);
                        confirmDialog.destroy();
                    }), this);
                    this.add.existing(confirmDialog);
                    confirmDialog.show();
                }
                else {
                    yield this.startGame(gameName);
                }
            });
        }
        startGame(gameName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToIntro(${gameName})`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                GameController_33.GameController.Instance.startGame(gameName);
                this.game.state.start(StateNames_24.StateNames.NARRATIVE_BRIEFING);
            });
        }
        shutdown() {
            GameController_33.GameController.Instance.exitBeaconTriggeredSignal.remove(this.showOutroConfirmation, this);
            clearInterval(this.timerRefreshRef);
            if (this.codeButton) {
                this.codeButton.events.onInputUp.remove(this.onCodeButtonClicked, this);
                this.codeButton = null;
            }
            if (this.missionButton) {
                this.missionButton.events.onInputUp.remove(this.onMissionButtonClicked, this);
                this.missionButton = null;
            }
            if (this.quitButton) {
                this.quitButton.events.onInputUp.remove(this.onQuitButtonClicked, this);
                this.quitButton = null;
            }
            this.shipGroup = null;
            this.ship = null;
            this.gridBg = null;
            this.levelDisplay = null;
            this.countdownPanel = null;
            this.uiLayer = null;
            this.mainHubButton = null;
            this.missionDialog = null;
            this.missionButton = null;
            this.codeReminderDialog = null;
            this.codeButtonGlow = null;
            super.shutdown();
        }
    }
    HubState.CODE_REMINDER_TIMER = 60000 * 5; // 5 mins
    exports.HubState = HubState;
});
define("component/ship/RaceShip", ["require", "exports", "switchback-common/extension/Easing", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "logic/RaceController", "logic/GameController"], function (require, exports, Easing_5, PromiseUtil_67, ScaleUtil_46, RaceController_4, GameController_34) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RaceShip extends Phaser.Group {
        constructor(game, raceMovements, numParts, numBonusParts, bodyTint, trailTint, isPlayerShip, isPlayable) {
            super(game);
            this.isPlayerShip = isPlayerShip;
            this.numBonusParts = numBonusParts;
            this.totalParts = numParts + numBonusParts;
            this.WOBBLE_OFFSET_Y = this.game.height * 0.02;
            this.isRaceComplete = false;
            // tint the players ship if they are playing on the installation
            const isShipTinted = !this.isPlayerShip || GameController_34.GameController.Instance.isInRangeOfInstallationPlayerConfirmed;
            // This provides the y difference that each placement position is worth
            // The maximum possible spread around the center point, divided by the number of ships.
            this.yPlacingMagnitude = (this.game.height * 0.4) / RaceController_4.RaceController.NUM_SHIPS_PER_RACE;
            this.shipGroup = new Phaser.Group(this.game);
            this.add(this.shipGroup);
            let partIndex = Math.max(numParts - 1, 0);
            this.base = new Phaser.Sprite(this.game, 0, 0, 'ship-base');
            this.base.anchor.set(0.5);
            this.shipGroup.add(this.base);
            // thuster offset for larger engines on level 19 & 20 bonus part ships
            const thrusterOffset = (numBonusParts > 18 ? this.base.height * 0.15 : 0);
            const thrusterPassive = new Phaser.Image(this.game, 0, this.base.bottom - (this.base.height * 0.1) + thrusterOffset, 'thruster-passive');
            thrusterPassive.width = this.base.width * 0.4;
            thrusterPassive.scale.y = thrusterPassive.scale.x;
            thrusterPassive.anchor.set(0.5, 0);
            thrusterPassive.tint = trailTint;
            this.shipGroup.add(thrusterPassive);
            this.game.add.tween(thrusterPassive.scale).to({ x: thrusterPassive.scale.x * 1.15 }, 200, Easing_5.shake(50, 50), true, 0, Infinity, true);
            const upgradeLayerLeft = new Phaser.Image(this.game, 0, 0, 'ship-upgrades', partIndex);
            upgradeLayerLeft.anchor.set(1, 0.5);
            upgradeLayerLeft.visible = partIndex > 0;
            this.shipGroup.add(upgradeLayerLeft);
            const upgradeLayerRight = new Phaser.Image(this.game, 0, 0, 'ship-upgrades', partIndex);
            upgradeLayerRight.anchor.set(1, 0.5);
            upgradeLayerRight.scale.x *= -1;
            upgradeLayerRight.visible = partIndex > 0;
            this.shipGroup.add(upgradeLayerRight);
            let bodyWidth = this.shipGroup.width;
            let bodyHeight = this.shipGroup.height;
            if (this.isPlayerShip) {
                // If we've redeemed bonus ship upgrades
                if (numBonusParts > 0) {
                    const bonusUpgradeLayerLeft = new Phaser.Image(this.game, 0, 0, 'bonus-upgrades', numBonusParts - 1);
                    bonusUpgradeLayerLeft.anchor.set(1, 0.5);
                    bonusUpgradeLayerLeft.top = upgradeLayerLeft.top;
                    this.shipGroup.add(bonusUpgradeLayerLeft);
                    const bonusUpgradeLayerRight = new Phaser.Image(this.game, 0, 0, 'bonus-upgrades', numBonusParts - 1);
                    bonusUpgradeLayerRight.anchor.set(1, 0.5);
                    bonusUpgradeLayerRight.scale.x *= -1;
                    bonusUpgradeLayerRight.top = upgradeLayerRight.top;
                    this.shipGroup.add(bonusUpgradeLayerRight);
                    if (isShipTinted) {
                        bonusUpgradeLayerLeft.tint = trailTint;
                        bonusUpgradeLayerRight.tint = trailTint;
                    }
                    bodyWidth = this.shipGroup.width;
                    bodyHeight = this.shipGroup.height;
                }
                if (isPlayable) {
                    const thrusterLaunch = new Phaser.Image(this.game, 0, this.base.bottom - (this.base.height * 0.1) + thrusterOffset, 'thruster-launch');
                    thrusterLaunch.width = this.base.width * 0.6;
                    thrusterLaunch.scale.y = thrusterLaunch.scale.x;
                    thrusterLaunch.anchor.set(0.5, 0);
                    thrusterLaunch.tint = trailTint;
                    thrusterLaunch.alpha = 0;
                    this.shipGroup.add(thrusterLaunch);
                    const slipstreamEffect = new Phaser.Image(this.game, 0, this.base.height * 0.15, 'slipstream-effect');
                    slipstreamEffect.anchor.set(0.5, 0.25);
                    slipstreamEffect.alpha = 0;
                    ScaleUtil_46.scaleToBounds(slipstreamEffect, this.base.width * 3, this.base.height * 2);
                    slipstreamEffect.animations.add('loop', null, 24, true).play();
                    this.shipGroup.add(slipstreamEffect);
                }
            }
            if (numBonusParts > 0) {
                ScaleUtil_46.scaleToBounds(this.shipGroup, this.game.width * 0.25, this.game.height * 0.25);
            }
            else {
                ScaleUtil_46.scaleToBounds(this.shipGroup, this.game.width * 0.15, this.game.height * 0.2);
            }
            // This ensures the physics body gets resized along with the ship base.
            if (isPlayable && this.isPlayerShip) {
                bodyWidth *= this.shipGroup.scale.x;
                bodyHeight *= this.shipGroup.scale.y;
                this.game.physics.arcade.enable(this.base);
                this.base.body.setSize(bodyWidth, bodyHeight, bodyWidth * 0.5, bodyHeight * 0.5);
            }
            this.origScale = this.shipGroup.scale.x;
            // the players ship isn't tinted as they know what it looks like from the hub
            // the installation and device race aren't sync'ed so don't want to associate the colours
            if (isShipTinted) {
                this.base.tint = bodyTint;
                upgradeLayerLeft.tint = bodyTint;
                upgradeLayerRight.tint = bodyTint;
            }
            // Schedule all the movements for this ship
            if (isPlayable) {
                this._currentLane = raceMovements[0].lane;
                raceMovements.forEach(this.scheduleMove, this);
                this.finishingPlace = raceMovements[raceMovements.length - 1].place;
                this.startWobble();
            }
        }
        get thrusterOffset() {
            let offset = this.game.height * 0.05;
            if (this.numBonusParts > 18) {
                offset += this.game.height * 0.03;
            }
            return offset;
        }
        startWobble() {
            if (!this.game) {
                return;
            }
            const randY = this.game.rnd.integerInRange(this.shipGroup.y - this.WOBBLE_OFFSET_Y, this.shipGroup.y + this.WOBBLE_OFFSET_Y);
            const duration = this.game.rnd.integerInRange(500, 1000);
            this.wobbleTween = this.game.add.tween(this.shipGroup)
                .to({ y: randY }, duration, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
            this.wobbleTween.onComplete.addOnce(this.startWobble, this);
        }
        get shipPosition() {
            return this.shipGroup.worldPosition;
        }
        get shipBody() {
            return this.base;
        }
        syncFinalPosition(duration) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_67.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ y: this.getYDestForPlace(this.finishingPlace) }, duration, Phaser.Easing.Linear.None, true));
            });
        }
        getYDestForPlace(place) {
            return this.yPlacingMagnitude * (place - (RaceController_4.RaceController.NUM_SHIPS_PER_RACE / 2));
        }
        scheduleMove(movement) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_67.PromiseUtil.realTime(RaceController_4.RaceController.RACE_DURATION_MS * movement.startTime);
                // catch exceptions caused by race getting out of sync
                if (this.isRaceComplete) {
                    return;
                }
                ////////////////////////////////////
                // MOVEMENT DURATION
                ////////////////////////////////////
                const durationMs = RaceController_4.RaceController.RACE_DURATION_MS * movement.duration;
                ////////////////////////////////////
                // X
                ////////////////////////////////////
                let xDest = (this.game.width / (RaceController_4.RaceController.NUM_SHIPS_PER_RACE + 1)) * movement.lane;
                // Add random movement if we're changing lane to ensure that
                // lanes don't look so 'fixed'
                xDest += (this.game.rnd.integerInRange(-this.game.width * 0.02, this.game.width * 0.02));
                ////////////////////////////////////
                // Y
                ////////////////////////////////////
                const yDest = this.getYDestForPlace(movement.place);
                ////////////////////////////////////
                // SCALE
                ////////////////////////////////////
                const scaleDest = (movement.lane % 2 === 0) ? this.origScale : this.origScale * 0.85;
                ////////////////////////////////////
                // ANGLE
                ////////////////////////////////////
                let angleDest = 0;
                // Don't give the ships an angle dip if they aren't really changing lanes.
                if (this._currentLane !== movement.lane) {
                    angleDest = xDest > this.x ? 10 : -10;
                }
                this.moveTo(durationMs, xDest, yDest, scaleDest, angleDest);
                this._currentLane = movement.lane;
            });
        }
        moveTo(duration, x, y, scale, angle) {
            // Ensure no tweens are still acting on these.
            // This could happen if someone goofed in the json and 
            // scheduled a tween while a previous one was still acting.
            this.game.tweens.removeFrom(this);
            if (duration === 0) {
                this.position.set(x, y);
                this.shipGroup.scale.set(scale);
            }
            else {
                if (this.isPlayerShip) {
                    // Don't move the player's X - let them do that.
                    this.game.add.tween(this)
                        .to({ y: y }, duration, Phaser.Easing.Sinusoidal.InOut, true);
                }
                else {
                    this.game.add.tween(this)
                        .to({ x: x, y: y }, duration, Phaser.Easing.Sinusoidal.InOut, true);
                    this.game.add.tween(this)
                        .to({ angle: angle }, duration * 0.5, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
                    this.game.add.tween(this.shipGroup.scale)
                        .to({ x: scale, y: scale }, duration, Phaser.Easing.Sinusoidal.InOut, true);
                }
            }
        }
        destroy() {
            this.game.tweens.removeFrom(this);
            if (this.isPlayerShip && this.base.body) {
                this.base.body.destroy();
            }
            this.shipGroup = null;
            this.wobbleTween = null;
            this.base = null;
            super.destroy();
        }
    }
    exports.RaceShip = RaceShip;
});
define("component/race/PlacementOverlayLine", ["require", "exports", "switchback-common/util/TextUtil", "Game"], function (require, exports, TextUtil_26, Game_61) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlacementOverlayLine extends Phaser.Group {
        constructor(game, placement) {
            super(game);
            this.placement = placement;
            this.placementText = new Phaser.Text(this.game, this.game.width * 0.025, 0, TextUtil_26.TextUtil.getOrdinalFormattedString(placement), {
                font: Game_61.Game.DEFAULT_FONT,
                fontSize: Game_61.Game.FONT_SIZE_MEDIUM,
                fill: Game_61.Game.DEFAULT_FONT_COLOUR,
                stroke: Game_61.Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game_61.Game.FONT_STROKE_SMALL
            });
            this.placementText.anchor.set(0, 0.6);
            TextUtil_26.TextUtil.scaleDownSingleLineTextToWidth(this.placementText, this.game.width * 0.08);
            this.add(this.placementText);
            this.line = new Phaser.Graphics(this.game);
            this.line.lineStyle(this.game.height * 0.005, 0xaaaaaa, 0.5);
            this.line.moveTo(this.placementText.right, 0);
            this.line.lineTo(this.game.width * 0.975, 0);
            this.add(this.line);
        }
        setTint(tint) {
            if (this._tint !== tint) {
                this.line.tint = tint;
                this.placementText.tint = tint;
                this._tint = tint;
            }
        }
    }
    exports.PlacementOverlayLine = PlacementOverlayLine;
});
define("component/race/RaceProgressBar", ["require", "exports", "switchback-common/util/ScaleUtil", "logic/RaceController", "switchback-common/localization/LocalizationManager", "Game"], function (require, exports, ScaleUtil_47, RaceController_5, LocalizationManager_56, Game_62) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RaceProgressBar extends Phaser.Group {
        constructor(game) {
            super(game);
            this.progressBase = new Phaser.Image(this.game, 0, 0, 'racebar-base');
            ScaleUtil_47.scaleToBounds(this.progressBase, this.game.width * 0.9, this.game.height * 0.1);
            this.progressBase.anchor.set(0.5, 0);
            this.add(this.progressBase);
            this.progressHead = new Phaser.Image(this.game, this.progressBase.left + this.progressBase.width * 0.1, this.progressBase.top + this.progressBase.height * 0.52, 'racebar-head');
            this.progressHead.scale.set(this.progressBase.scale.x);
            this.progressHead.anchor.set(0, 0.5);
            this.add(this.progressHead);
            this.progressFill = new Phaser.Image(this.game, this.progressHead.left, this.progressHead.top, 'racebar-fill');
            this.progressFill.width = 0;
            this.progressFill.height = this.progressHead.height;
            this.add(this.progressFill);
            const raceProgressFrame = new Phaser.Image(this.game, this.progressBase.centerX, this.progressBase.centerY, 'racebar-frame');
            ScaleUtil_47.scaleToBounds(raceProgressFrame, this.game.width * 0.9, this.game.height * 0.1);
            raceProgressFrame.anchor.set(0.5);
            this.add(raceProgressFrame);
            const raceActiveText = new Phaser.Text(this.game, 0, 0, LocalizationManager_56.LocalizationManager.Instance.text("race.race_in_progress"), Object.assign({}, Game_62.Game.DEFAULT_FONT_STYLE, { fontSize: Game_62.Game.FONT_SIZE_SMALL, align: 'center' }));
            raceActiveText.centerX = this.progressBase.centerX;
            raceActiveText.centerY = this.progressBase.centerY;
            this.add(raceActiveText);
        }
        start() {
            this.game.add.tween(this.progressFill)
                .to({ width: this.progressBase.width * 0.785 }, RaceController_5.RaceController.RACE_DURATION_MS, Phaser.Easing.Linear.None, true);
        }
        update() {
            super.update();
            // Ensure the progress bar's head UI follows the main fill
            this.progressHead.left = this.progressFill.right;
        }
    }
    exports.RaceProgressBar = RaceProgressBar;
});
define("component/race/Gate", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Gate extends Phaser.Sprite {
        constructor(game) {
            super(game, 0, 0);
            this.gateBase = new Phaser.Sprite(this.game, 0, 0, 'gate');
            this.gateBase.animations.add("idle", [5, 2, 4, 0, 3, 1], 6, true); // slow animation
            this.gateBase.animations.add("hit", [5, 2, 4, 0, 3, 1], 24, true); // fast when hit
            this.gateBase.play("idle");
            this.gateBase.width = this.game.width * 0.2;
            this.gateBase.scale.y = this.gateBase.scale.x;
            this.addChild(this.gateBase);
            this.game.physics.arcade.enable(this);
            this.body.setSize(this.gateBase.width * 0.2, this.gateBase.height * 0.6);
        }
        hitGate() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[Gate] hitGate()");
                this.gateBase.play("hit");
                this.body.destroy();
            });
        }
    }
    exports.Gate = Gate;
});
define("state/DeviceRaceState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "Game", "logic/GameController", "StateNames", "state/SpaceBaseState", "logic/RaceController", "component/ship/RaceShip", "component/race/PlacementOverlayLine", "component/race/RaceProgressBar", "logic/SMAnalyticsManager", "component/race/Gate"], function (require, exports, PromiseUtil_68, LocalizationManager_57, ScaleUtil_48, Game_63, GameController_35, StateNames_25, SpaceBaseState_8, RaceController_6, RaceShip_1, PlacementOverlayLine_1, RaceProgressBar_1, SMAnalyticsManager_27, Gate_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class DeviceRaceState extends SpaceBaseState_8.SpaceBaseState {
        constructor() {
            super(...arguments);
            this.hasSyncedPlayerPosition = false;
            this.lastGateSpawnTime = 0;
        }
        init() {
            this.stateName = StateNames_25.StateNames.DEVICE_RACE;
            super.init(true, false);
            this.ships = [];
            this.particleEmitters = [];
            this.lines = [];
            this.spinSpeed = 6;
            this.isRaceRunning = false;
            this.raceTimeRemaining = RaceController_6.RaceController.RACE_DURATION_MS;
            this.raceCompleteSignal = new Phaser.Signal();
            this.hasSyncedPlayerPosition = false;
            this.numGatesHit = 0;
            this.lastGateSpawnTime = this.game.time.now;
            DeviceRaceState.GATE_SPAWN_DEAD_ZONE_MS = RaceController_6.RaceController.RACE_DURATION_MS * DeviceRaceState.GATE_SPAWN_DEAD_ZONE_PROPORTION;
            DeviceRaceState.FINAL_SYNC_TIME_MS = DeviceRaceState.GATE_SPAWN_DEAD_ZONE_MS - DeviceRaceState.GATE_TRAVEL_DURATION_MS;
            DeviceRaceState.GATE_SPAWN_GAP_MS = (RaceController_6.RaceController.RACE_DURATION_MS - DeviceRaceState.GATE_SPAWN_DEAD_ZONE_MS) /
                DeviceRaceState.NUM_GATES_TO_SPAWN;
        }
        preload() {
            super.preload();
            this.game.load.image('bg-space-clean', 'assets/common/bg-space-clean.jpg');
            this.game.load.atlas('boost-button', 'assets/race/boost-button.png', 'assets/race/boost-button.json');
            this.game.load.image('racebar-head', 'assets/race/racebar_head.png');
            this.game.load.image('racebar-base', 'assets/race/racebar_base.png');
            this.game.load.image('racebar-frame', 'assets/race/racebar_frame.png');
            this.game.load.image('racebar-fill', 'assets/race/racebar_fill.png');
            this.game.load.image('ship-highlight-circle', 'assets/race/circle.png');
            this.game.load.image('race-frame', 'assets/race/race-frame.png');
            this.game.load.image('frame-header', 'assets/common/frame-header.png');
            this.game.load.image("lightbeam", "assets/common/lightbeam.png");
            this.game.load.image("circle-design", "assets/hub/circle_design.png");
            this.game.load.image("hyperion-glow", "assets/outro/hyperion_glow.png");
            this.game.load.spritesheet("gate", "assets/race/boostgate.png", 158, 180);
            this.game.load.image("outer_green_glow", "assets/race/outer_green_glow.png");
            for (let i = 0; i < RaceController_6.RaceController.NUM_SHIPS_PER_RACE; i++) {
                this.game.load.image(`race-trail-${i}`, `assets/race/ships/trail_${i}.png`);
            }
            // Reusing these SFX from pipes
            for (let i = 1; i <= 3; i++) {
                this.game.load.audio(`probe_launch_${i}`, [`assets/sound/pipes/probe_launch_${i}.m4a`, `assets/sound/pipes/probe_launch_${i}.ogg`]);
            }
            this.game.load.audio("race_background_loop", ["assets/sound/race/race_background_loop.ogg", "assets/sound/race/race_background_loop.m4a"]);
            this.game.load.audio("race_complete", ["assets/sound/race/race_complete.ogg", "assets/sound/race/race_complete.m4a"]);
            this.game.load.audio("hit_boost", ["assets/sound/race/hit_boost.ogg", "assets/sound/race/hit_boost.m4a"]);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.input.maxPointers = 2;
                this.game.physics.startSystem(Phaser.Physics.ARCADE);
                const raceController = GameController_35.GameController.Instance.raceController;
                //////////////////////////////
                // DATA
                //////////////////////////////
                const currentRaceId = raceController.getIdOfScheduledRace();
                this.playerShipId = raceController.getShipIdForRace(currentRaceId, GameController_35.GameController.Instance.getTotalShipParts());
                const shipVisuals = raceController.getShipVisualsForRace(currentRaceId, GameController_35.GameController.Instance.getTotalShipParts());
                //////////////////////////////
                // BACKGROUNDS
                //////////////////////////////
                this.bg = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, 'bg-space-clean');
                const bg1Scale = this.game.width / this.bg.texture.width;
                this.bg.tileScale.set(bg1Scale, bg1Scale);
                this.add.existing(this.bg);
                ///////////////////////////////
                // PLACEMENT LINES
                ///////////////////////////////
                for (let i = 1; i <= 3; i++) {
                    const line = new PlacementOverlayLine_1.PlacementOverlayLine(this.game, i);
                    this.add.existing(line);
                    this.lines.push(line);
                }
                this.particleGroup = new Phaser.Group(this.game);
                this.add.existing(this.particleGroup);
                this.gateGroup = new Phaser.Group(this.game);
                this.add.existing(this.gateGroup);
                this.shipGroup = new Phaser.Group(this.game);
                this.add.existing(this.shipGroup);
                this.shipGroup.y = this.game.height * 1.2;
                ///////////////////////////////
                // THE SHIPS!
                ///////////////////////////////
                const colors = raceController.getColors();
                const shipList = raceController.getShipList();
                for (let i = 0; i < shipList.length; i++) {
                    const bodyTint = parseInt("0x" + colors[shipList[i].bodyColor]);
                    const trailTint = parseInt("0x" + colors[shipList[i].trailColor]);
                    this.createShipTrailEmitter(i);
                    let numParts = 10;
                    let numBonusParts = 0;
                    // if this is our ship, use the parts we have.
                    if (i === this.playerShipId) {
                        const shipState = GameController_35.GameController.Instance.getShipState();
                        numParts = shipState.baseParts;
                        numBonusParts = shipState.bonusParts;
                    }
                    else {
                        // Otherwise use the number of parts required (for getting the place that this ship is going to get)
                        numParts = raceController.getRandomPartsForPlacing(raceController.getShipPlacingForId(i, currentRaceId));
                    }
                    let ship = new RaceShip_1.RaceShip(this.game, raceController.getRaceMovementsByShipAndRaceId(i, currentRaceId), numParts, numBonusParts, bodyTint, trailTint, i === this.playerShipId, true);
                    this.shipGroup.add(ship);
                    this.ships[i] = ship;
                }
                // Take a copy for sorting later
                this.shipsSortingCopy = this.ships.slice();
                ///////////////////////////////
                // PLAYER SHIP INDICATOR
                ///////////////////////////////
                const bodyColor = "0x" + colors[shipVisuals.bodyColor];
                this.playerShipCircle = new Phaser.Image(this.game, 0, 0, 'ship-highlight-circle');
                this.playerShipCircle.anchor.set(0.5);
                this.playerShipCircle.tint = parseInt(bodyColor);
                ScaleUtil_48.scaleToBounds(this.playerShipCircle, this.ships[0].width * 1.5, this.ships[0].height * 1.5);
                this.playerCircleSpinTween = this.game.add.tween(this.playerShipCircle).to({ angle: 360 }, 5000, Phaser.Easing.Linear.None, true, 0, Infinity);
                this.add.existing(this.playerShipCircle);
                this.playerShipCircleOffset = this.playerShipCircle.height * 0.1;
                ///////////////////////////////
                // GREEN OVERLAY FLASH
                ///////////////////////////////
                this.greenOverlay = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.01, 'outer_green_glow');
                this.greenOverlay.width = this.game.width * 0.98;
                this.greenOverlay.height = this.game.height * 0.815;
                this.greenOverlay.anchor.set(0.5, 0);
                this.greenOverlay.alpha = 0;
                this.add.existing(this.greenOverlay);
                ///////////////////////////////
                // RACE PROGRESS BAR
                ///////////////////////////////
                const frame = new Phaser.Image(this.game, 0, 0, 'race-frame');
                frame.width = this.game.width;
                frame.height = this.game.height;
                this.add.existing(frame);
                this.raceProgressBar = new RaceProgressBar_1.RaceProgressBar(this.game);
                this.raceProgressBar.centerX = this.game.width * 0.5;
                this.raceProgressBar.top = this.game.height * 0.025;
                this.add.existing(this.raceProgressBar);
                this.movementButtonL = new Phaser.Button(this.game, this.game.width * 0.05, this.game.height * 0.975, "boost-button", () => { }, this, 0, 0, 1, 0);
                this.movementButtonL.anchor.set(0, 1);
                ScaleUtil_48.scaleToBounds(this.movementButtonL, this.game.width * 0.425, this.game.height * 0.225);
                this.movementButtonL.onInputDown.add(() => {
                    this.game.sound.play(`probe_launch_${this.game.rnd.integerInRange(1, 3)}`);
                });
                this.add.existing(this.movementButtonL);
                this.movementButtonR = new Phaser.Button(this.game, this.game.width * 0.95, this.game.height * 0.975, "boost-button", () => { }, this, 0, 0, 1, 0);
                this.movementButtonR.anchor.set(0, 1);
                ScaleUtil_48.scaleToBounds(this.movementButtonR, this.game.width * 0.425, this.game.height * 0.225);
                this.movementButtonR.scale.x *= -1;
                this.movementButtonR.onInputDown.add(() => {
                    this.game.sound.play(`probe_launch_${this.game.rnd.integerInRange(1, 3)}`);
                });
                this.add.existing(this.movementButtonR);
                ///////////////////////////////
                // RACE FINISH OVERLAY
                ///////////////////////////////
                this.finishOverlay = new Phaser.Graphics(this.game, 0, 0);
                this.finishOverlay.beginFill(0x000000, 0.7);
                this.finishOverlay.drawRect(0, 0, this.game.width, this.game.height);
                this.finishOverlay.alpha = 0;
                this.finishOverlay.visible = false;
                this.finishOverlay.inputEnabled = true;
                this.add.existing(this.finishOverlay);
                ///////////////////////////////
                // RACE FINISH TEXT
                ///////////////////////////////
                this.spinCircle = new Phaser.Image(this.game, 0, 0, "circle-design");
                ScaleUtil_48.scaleToBounds(this.spinCircle, this.game.width * 0.65, this.game.width * 0.65);
                this.spinCircle.anchor.set(0.5);
                this.spinCircle.centerX = this.game.width * 0.5;
                this.spinCircle.centerY = this.game.height * 0.5;
                this.spinCircle.alpha = 0;
                this.add.existing(this.spinCircle);
                this.lightBeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
                ScaleUtil_48.scaleToBounds(this.lightBeam, this.game.width, this.game.height * 0.05);
                this.lightBeam.centerX = this.game.width * 0.5;
                this.lightBeam.centerY = this.spinCircle.centerY;
                this.lightBeam.alpha = 0;
                this.add.existing(this.lightBeam);
                this.finishText1 = new Phaser.Text(this.game, 0, 0, LocalizationManager_57.LocalizationManager.Instance.text("race.race_complete_1"), {
                    font: Game_63.Game.DEFAULT_FONT,
                    fill: Game_63.Game.DEFAULT_FONT_COLOUR,
                    fontSize: Game_63.Game.FONT_SIZE_LARGE,
                    stroke: Game_63.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_63.Game.FONT_STROKE_SMALL
                });
                this.finishText1.left = this.game.width;
                this.finishText1.bottom = this.lightBeam.top + this.game.height * 0.02;
                this.add.existing(this.finishText1);
                this.finishText2 = new Phaser.Text(this.game, 0, 0, LocalizationManager_57.LocalizationManager.Instance.text('race.race_complete_2'), {
                    font: Game_63.Game.DEFAULT_FONT,
                    fill: Game_63.Game.FONT_COLOUR_BLUE,
                    fontSize: Game_63.Game.FONT_SIZE_LARGE,
                    stroke: Game_63.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_63.Game.FONT_STROKE_SMALL
                });
                this.finishText2.right = 0;
                this.finishText2.top = this.lightBeam.bottom - this.game.height * 0.02;
                this.add.existing(this.finishText2);
                ///////////////////////////////
                // BEGIN
                ///////////////////////////////
                yield PromiseUtil_68.PromiseUtil.time(this.game, 350);
                this.openDoors();
                this.play();
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                const raceBgSFX = this.game.sound.play("race_background_loop", 1, true);
                // Scroll the background
                const bgMoveTween = this.game.add.tween(this.bg.tilePosition)
                    .to({ y: this.bg.texture.height }, 2500, Phaser.Easing.Linear.None, true, 0, Infinity);
                // And tween /those/ tween's timescales to get a gradual increase in speed
                this.game.add.tween(bgMoveTween)
                    .to({ timeScale: 3 }, RaceController_6.RaceController.RACE_DURATION_MS, Phaser.Easing.Linear.None, true);
                // Very quickly bring the race ships into the middle of the screen.
                this.game.add.tween(this.shipGroup).to({ y: this.game.height * 0.5 }, RaceController_6.RaceController.RACE_DURATION_MS * 0.05, Phaser.Easing.Circular.Out, true).onComplete.addOnce(() => {
                    // After that, gradually bring the racing ships group up to the top(ish) of the screen
                    this.game.add.tween(this.shipGroup).to({ y: this.game.height * 0.35 }, RaceController_6.RaceController.RACE_DURATION_MS * 0.95, Phaser.Easing.Linear.None, true);
                });
                SMAnalyticsManager_27.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame', 'SpaceRaceGame', {
                    [SMAnalyticsManager_27.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_27.SMAnalyticsManager.GAME_PLAYS]: GameController_35.GameController.Instance.state.getMinigamePlays(StateNames_25.StateNames.DEVICE_RACE) + 1,
                    [SMAnalyticsManager_27.SMAnalyticsManager.GAME_SCORE]: GameController_35.GameController.Instance.raceController.shipPlacement + 1
                });
                this.raceProgressBar.start();
                this.game.add.tween(this).to({ raceProgressBoostMultiplier: 2 }, RaceController_6.RaceController.RACE_DURATION_MS, Phaser.Easing.Linear.None, true);
                this.isRaceRunning = true;
                yield PromiseUtil_68.PromiseUtil.signal(this.raceCompleteSignal);
                GameController_35.GameController.Instance.raceController.raceCompleted();
                this.movementButtonR.frame = 1;
                this.movementButtonR.inputEnabled = false;
                this.movementButtonL.frame = 1;
                this.movementButtonL.inputEnabled = false;
                yield Promise.all([
                    this.showFinish(),
                    this.moveShipsOffScreen(),
                    PromiseUtil_68.PromiseUtil.tween(this.game, this.game.add.tween(raceBgSFX).to({ volume: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                raceBgSFX.destroy();
                this.transitionToPostRace();
            });
        }
        showFinish() {
            return __awaiter(this, void 0, void 0, function* () {
                this.finishOverlay.visible = true;
                yield PromiseUtil_68.PromiseUtil.tween(this.game, this.game.add.tween(this.finishOverlay)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                this.game.sound.play("race_complete");
                // Alpha
                this.game.add.tween(this.spinCircle).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.lightBeam).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true);
                // Scale
                this.game.add.tween(this.spinCircle.scale).from({ x: 0.8, y: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.lightBeam).from({ x: 0.8, y: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                // Spin 
                // This is tweening the value of spinSpeed, which is being used to set the rotation in the update loop
                this.game.add.tween(this).to({ spinSpeed: 0.5 }, 1000, Phaser.Easing.Sinusoidal.Out, true);
                // Text tweens
                this.game.add.tween(this.finishText1).to({ left: this.spinCircle.left }, 500, Phaser.Easing.Exponential.Out, true).chain(this.game.add.tween(this.finishText2).to({ right: this.spinCircle.right }, 500, Phaser.Easing.Exponential.Out, true));
            });
        }
        moveShipsOffScreen() {
            return __awaiter(this, void 0, void 0, function* () {
                // Make all the ships depart the screen in the order they placed 
                const shipDepartDelay = 500;
                this.ships.forEach((ship, index) => {
                    this.game.add.tween(ship)
                        .to({ y: ship.y - this.game.height * 0.5 }, 4000, Phaser.Easing.Sinusoidal.In, true, ship.finishingPlace * shipDepartDelay);
                });
                // Wait for 2 seconds after the last ship has started departing
                yield PromiseUtil_68.PromiseUtil.time(this.game, (shipDepartDelay * 8) + 2000);
            });
        }
        spawnGate() {
            return __awaiter(this, void 0, void 0, function* () {
                const gate = new Gate_1.Gate(this.game);
                gate.x = this.game.width * this.game.rnd.realInRange(0.025, 0.675);
                gate.bottom = 0;
                this.gateGroup.add(gate);
                yield PromiseUtil_68.PromiseUtil.tween(this.game, this.game.add.tween(gate)
                    .to({ top: this.game.height }, DeviceRaceState.GATE_TRAVEL_DURATION_MS, Phaser.Easing.Linear.None, true));
                gate.destroy();
            });
        }
        addPilotRating() {
            return __awaiter(this, void 0, void 0, function* () {
                this.numGatesHit++;
            });
        }
        /**
         * Creates the particles for a ship.
         * The ship's ID is used to fetch a specific
         * colored asset to comprise the trail.
         * @param shipId
         */
        createShipTrailEmitter(shipId) {
            const particleEmitter = new Phaser.Particles.Arcade.Emitter(this.game, 0, 0, 75);
            particleEmitter.gravity = 0;
            particleEmitter.minParticleSpeed.set(0, this.game.height * 0.1);
            particleEmitter.maxParticleSpeed.set(0, this.game.height * 0.1);
            particleEmitter.minRotation = 0;
            particleEmitter.maxRotation = 0;
            particleEmitter.setScale(1, 0.25, 1, 0.25, 750);
            particleEmitter.setAlpha(0.5, 0, 750);
            particleEmitter.lifespan = 750;
            particleEmitter.makeParticles(`race-trail-${shipId}`);
            this.particleGroup.add(particleEmitter);
            this.particleEmitters.push(particleEmitter);
        }
        update() {
            super.update();
            if (this.isRaceRunning) {
                this.raceTimeRemaining -= this.game.time.elapsed;
                if (this.raceTimeRemaining <= 0) {
                    this.ships.forEach((ship) => {
                        ship.isRaceComplete = true;
                    });
                    this.isRaceRunning = false;
                    this.raceCompleteSignal.dispatch();
                }
            }
            if (this.game.time.now > this.lastGateSpawnTime + DeviceRaceState.GATE_SPAWN_GAP_MS &&
                this.isRaceRunning &&
                this.raceTimeRemaining > DeviceRaceState.GATE_SPAWN_DEAD_ZONE_MS) {
                this.spawnGate();
                this.lastGateSpawnTime = this.game.time.now;
            }
            if (this.raceTimeRemaining < DeviceRaceState.FINAL_SYNC_TIME_MS &&
                !this.hasSyncedPlayerPosition &&
                this.isRaceRunning) {
                this.hasSyncedPlayerPosition = true;
                let finishingPlace = this.ships[this.playerShipId].finishingPlace;
                if (this.numGatesHit >= (DeviceRaceState.NUM_GATES_TO_SPAWN * 0.8)) {
                    // If we weren't already in 1st place, bump the player's ship up a place for their performance.
                    if (finishingPlace > 1) {
                        finishingPlace--;
                        this.ships[this.playerShipId].finishingPlace--;
                        // ShipPlacement is 0-indexed while finishingPlace is 1-indexed, so subtract another off.
                        GameController_35.GameController.Instance.raceController.shipPlacement = finishingPlace - 1;
                        // Move the ship that /was/ in that position down one!
                        const otherShip = this.ships.find((ship) => {
                            return ship.finishingPlace === finishingPlace;
                        });
                        otherShip.finishingPlace++;
                        otherShip.syncFinalPosition(DeviceRaceState.FINAL_SYNC_TIME_MS);
                        this.ships[this.playerShipId].syncFinalPosition(DeviceRaceState.FINAL_SYNC_TIME_MS);
                    }
                }
                // HACK: this should be refactored, it assumes that the order of the ships here 
                // will match the order of the ships on the results screen
                const racePositions = [];
                this.ships.forEach(ship => {
                    const raceResult = new RaceController_6.RaceResult();
                    raceResult.placement = ship.finishingPlace;
                    raceResult.numShipParts = ship.totalParts;
                    racePositions.push(raceResult);
                });
                GameController_35.GameController.Instance.raceController.setRaceResult(racePositions);
            }
            if (this.movementButtonL.input.checkPointerDown(this.game.input.pointer1) ||
                this.movementButtonL.input.checkPointerDown(this.game.input.pointer2)) {
                this.ships[this.playerShipId].x -= this.game.width * DeviceRaceState.PLAYER_SHIP_HORIZONTAL_MOVE_SPEED;
            }
            if (this.movementButtonR.input.checkPointerDown(this.game.input.pointer1) ||
                this.movementButtonR.input.checkPointerDown(this.game.input.pointer2)) {
                this.ships[this.playerShipId].x += this.game.width * DeviceRaceState.PLAYER_SHIP_HORIZONTAL_MOVE_SPEED;
            }
            this.ships[this.playerShipId].x = Phaser.Math.clamp(this.ships[this.playerShipId].x, this.game.width * 0.05, this.game.width * 0.95);
            this.ships.forEach((ship, index) => {
                const pos = ship.shipPosition;
                this.particleEmitters[index].emitParticle(pos.x, pos.y + ship.thrusterOffset);
            });
            /**
             * Find all the ships in vertically descending order,
             * then position the line placement overlays to correspond
             * to their placements.
             */
            const shipsInVerticalOrder = this.shipsSortingCopy.sort((a, b) => {
                return (a.shipPosition.y - b.shipPosition.y);
            });
            this.lines.forEach((line, index) => {
                line.bottom = shipsInVerticalOrder[index].shipPosition.y;
            }, this);
            /**
             * Positions the player-ship-specific elements to follow
             * the current player's ship at all times.
             */
            const playerShipLoc = this.ships[this.playerShipId].shipPosition;
            this.playerShipCircle.position.set(playerShipLoc.x, playerShipLoc.y + this.playerShipCircleOffset);
            this.spinCircle.angle += this.spinSpeed;
            this.game.physics.arcade.overlap(this.ships[this.playerShipId].shipBody, this.gateGroup, (shipBody, gate) => {
                gate.hitGate();
                this.game.sound.play("hit_boost");
                this.addPilotRating();
                this.game.tweens.removeFrom(this.greenOverlay);
                this.greenOverlay.alpha = 0;
                this.game.add.tween(this.greenOverlay).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true, 0, 0, true);
            });
        }
        transitionToPostRace() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_35.GameController.Instance.postMinigameScore(StateNames_25.StateNames.DEVICE_RACE, this.numGatesHit);
                yield this.closeDoors();
                this.game.state.start(StateNames_25.StateNames.POST_RACE_PLACEMENT);
            });
        }
        shutdown() {
            this.game.physics.destroy();
            this.raceCompleteSignal.dispose();
            this.raceCompleteSignal = null;
            if (this.playerCircleSpinTween) {
                this.playerCircleSpinTween.stop();
                this.playerCircleSpinTween = null;
            }
            this.bg = null;
            this.gateGroup = null;
            this.shipGroup = null;
            this.particleGroup = null;
            this.greenOverlay = null;
            this.ships = null;
            this.shipsSortingCopy = null;
            this.particleEmitters = null;
            this.lines = null;
            this.movementButtonL = null;
            this.movementButtonR = null;
            this.playerShipCircle = null;
            this.playerShipId = null;
            this.finishOverlay = null;
            this.spinCircle = null;
            this.lightBeam = null;
            this.finishText1 = null;
            this.finishText2 = null;
            this.raceProgressBar = null;
            this.game.cache.removeImage('bg-space-clean');
            this.game.cache.removeImage('boost-button');
            this.game.cache.removeImage('racebar-head');
            this.game.cache.removeImage('racebar-base');
            this.game.cache.removeImage('racebar-frame');
            this.game.cache.removeImage('racebar-fill');
            this.game.cache.removeImage('ship-highlight-circle');
            this.game.cache.removeImage('race-frame');
            this.game.cache.removeImage('frame-header');
            this.game.cache.removeImage("circle-design");
            this.game.cache.removeImage("hyperion-glow");
            this.game.cache.removeImage("gate");
            this.game.cache.removeImage("outer_green_glow");
            for (let i = 0; i < RaceController_6.RaceController.NUM_SHIPS_PER_RACE; i++) {
                this.game.cache.removeImage(`race-trail-${i}`);
            }
            this.game.cache.removeSound("race_background_loop");
            this.game.cache.removeSound("race_complete");
            this.game.cache.removeSound("hit_boost");
            for (let i = 1; i <= 3; i++) {
                this.game.cache.removeSound(`probe_launch_${i}`);
            }
            super.shutdown();
        }
    }
    DeviceRaceState.PLAYER_SHIP_HORIZONTAL_MOVE_SPEED = 0.01;
    DeviceRaceState.NUM_GATES_TO_SPAWN = 10;
    DeviceRaceState.GATE_SPAWN_DEAD_ZONE_PROPORTION = 0.15;
    DeviceRaceState.GATE_TRAVEL_DURATION_MS = 2000;
    exports.DeviceRaceState = DeviceRaceState;
});
define("state/InstallationRaceInProgressState", ["require", "exports", "Game", "logic/GameController", "StateNames", "switchback-common/localization/LocalizationManager", "state/SpaceBaseState", "switchback-common/util/ScaleUtil", "logic/RaceController", "component/race/RaceProgressBar", "logic/SMAnalyticsManager"], function (require, exports, Game_64, GameController_36, StateNames_26, LocalizationManager_58, SpaceBaseState_9, ScaleUtil_49, RaceController_7, RaceProgressBar_2, SMAnalyticsManager_28) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class InstallationRaceInProgressState extends SpaceBaseState_9.SpaceBaseState {
        init() {
            this.stateName = StateNames_26.StateNames.INSTALLATION_RACE_IN_PROGRESS;
            super.init(true, false);
        }
        preload() {
            super.preload();
            this.game.load.image('banner-background', 'assets/race/banner_yellow.png');
            this.game.load.image('arrow', 'assets/race/arrow_floating.png');
            this.game.load.image('racebar-head', 'assets/race/racebar_head.png');
            this.game.load.image('racebar-base', 'assets/race/racebar_base.png');
            this.game.load.image('racebar-frame', 'assets/race/racebar_frame.png');
            this.game.load.image('racebar-fill', 'assets/race/racebar_fill.png');
            this.shipList = GameController_36.GameController.Instance.raceController.getShipList();
            for (let i = 0; i < this.shipList.length; i++) {
                this.game.load.image("race-thumb-" + i, "assets/race/ships/thumb_" + i + ".png");
            }
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const raceController = GameController_36.GameController.Instance.raceController;
                const activeRaceId = raceController.getIdOfScheduledRace();
                const numParts = GameController_36.GameController.Instance.getTotalShipParts();
                const colors = raceController.getColors();
                const shipID = raceController.getShipIdForRace(activeRaceId, numParts);
                const shipVisuals = raceController.getShipVisualsForRace(activeRaceId, numParts);
                const padY = this.game.height * 0.01;
                this.timeUntilTransitionToPostRaceMs = RaceController_7.RaceController.RACE_DURATION_MS;
                // get the positions of all of the ships in this race and notify raceController.
                // This is only required as the device race can have different race results.
                // HACK: this should be refactored, it assumes that the order of the ships here 
                // will match the order of the ships on the results screen
                const racePositions = [];
                const shipList = raceController.getShipList();
                for (let i = 0; i < shipList.length; ++i) {
                    const shipMovements = raceController.getRaceMovementsByShipAndRaceId(i, activeRaceId);
                    const raceResult = new RaceController_7.RaceResult();
                    raceResult.placement = shipMovements[shipMovements.length - 1].place;
                    raceResult.numShipParts = raceController.getRandomPartsForPlacing(raceResult.placement - 1);
                    racePositions.push(raceResult);
                }
                GameController_36.GameController.Instance.raceController.setRaceResult(racePositions);
                const background = new Phaser.Image(this.game, 0, 0, "background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                this.raceProgressBar = new RaceProgressBar_2.RaceProgressBar(this.game);
                this.raceProgressBar.centerX = this.game.width * 0.5;
                this.raceProgressBar.top = this.game.height * 0.03;
                this.add.existing(this.raceProgressBar);
                // Ship info text
                const shipInfoGroup = new Phaser.Group(this.game);
                this.add.existing(shipInfoGroup);
                const yourShipIs = new Phaser.Text(this.game, this.game.width * 0.5, 0, LocalizationManager_58.LocalizationManager.Instance.text(`race.racing_as`), Object.assign({}, Game_64.Game.DEFAULT_FONT_STYLE, { fontSize: Game_64.Game.FONT_SIZE_MEDIUM, align: 'center', wordWrap: true, wordWrapWidth: this.game.width * 0.9 }));
                yourShipIs.anchor.set(0.5, 1);
                shipInfoGroup.add(yourShipIs);
                // ship thumbnail
                const shipKey = "race-thumb-" + (shipID % raceController.getShipList().length);
                const thumb = new Phaser.Sprite(this.game, 0, 0, shipKey);
                ScaleUtil_49.scaleToBounds(thumb, this.game.width * 0.9, this.game.height * 0.6);
                thumb.centerX = yourShipIs.centerX;
                thumb.top = yourShipIs.bottom + padY;
                shipInfoGroup.add(thumb);
                const colorTop = thumb.bottom + padY;
                const shipColorName = LocalizationManager_58.LocalizationManager.Instance.text(`color.${shipVisuals.bodyColor}`);
                const shipText = LocalizationManager_58.LocalizationManager.Instance.textWithReplacements("race.ship_color", { color: shipColorName }).toUpperCase();
                const colourLabel = new Phaser.Text(this.game, this.game.width * 0.5, colorTop, shipText, Object.assign({}, Game_64.Game.DEFAULT_FONT_STYLE, { fontSize: Game_64.Game.FONT_SIZE_LARGE, align: 'center' }));
                colourLabel.addColor("#" + colors[shipVisuals.bodyColor], 0);
                colourLabel.anchor.set(0.5, 0);
                shipInfoGroup.add(colourLabel);
                // trail
                const trailColorName = LocalizationManager_58.LocalizationManager.Instance.text(`color.${shipVisuals.trailColor}`);
                const trailText = LocalizationManager_58.LocalizationManager.Instance.textWithReplacements("race.trail_color", { color: trailColorName }).toUpperCase();
                const trailLabel = new Phaser.Text(this.game, this.game.width * 0.5, colourLabel.bottom, trailText, Object.assign({}, Game_64.Game.DEFAULT_FONT_STYLE, { fontSize: Game_64.Game.FONT_SIZE_LARGE, align: 'center' }));
                trailLabel.addColor("#" + colors[shipVisuals.trailColor], 0);
                trailLabel.anchor.set(0.5, 0);
                shipInfoGroup.add(trailLabel);
                const shipLevelLabel = new Phaser.Text(this.game, this.game.width * 0.5, trailLabel.bottom, LocalizationManager_58.LocalizationManager.Instance.textWithReplacements("race.level_x", { level: (GameController_36.GameController.Instance.getTotalShipParts() + 1).toString() }), Object.assign({}, Game_64.Game.DEFAULT_FONT_STYLE, { fontSize: Game_64.Game.FONT_SIZE_MEDIUM, strokeThickness: this.game.width * 0.01, align: 'center' }));
                shipLevelLabel.addColor("#" + colors[shipVisuals.bodyColor], 0);
                shipLevelLabel.anchor.set(0.5, 0);
                shipInfoGroup.add(shipLevelLabel);
                shipInfoGroup.centerY = this.game.height * 0.5;
                this.refreshCountdownTimer();
                this.intervalRef = setInterval(this.refreshCountdownTimer.bind(this), 250);
                yield this.openDoors();
                this.raceProgressBar.start();
                SMAnalyticsManager_28.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/watch', 'SpaceRaceWatch', {
                    [SMAnalyticsManager_28.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_28.SMAnalyticsManager.GAME_PLAYS]: GameController_36.GameController.Instance.state.getMinigamePlays(StateNames_26.StateNames.DEVICE_RACE) + 1,
                    [SMAnalyticsManager_28.SMAnalyticsManager.GAME_SCORE]: GameController_36.GameController.Instance.raceController.shipPlacement + 1
                });
            });
        }
        refreshCountdownTimer() {
            const timeUntilTransitionToPostRaceS = Math.ceil(this.timeUntilTransitionToPostRaceMs * 0.001);
            if (timeUntilTransitionToPostRaceS < 1) {
                this.transitionToPostRace();
            }
        }
        update() {
            super.update();
            this.timeUntilTransitionToPostRaceMs -= this.game.time.elapsedMS;
        }
        transitionToPostRace() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_36.GameController.Instance.raceController.raceCompleted();
                clearInterval(this.intervalRef);
                clearInterval(this.animateRoofIntervalRef);
                yield this.closeDoors();
                this.game.state.start(StateNames_26.StateNames.POST_RACE_PLACEMENT);
            });
        }
        shutdown() {
            clearInterval(this.intervalRef);
            this.raceProgressBar = null;
            this.game.cache.removeImage('banner-background');
            this.game.cache.removeImage('arrow');
            this.game.cache.removeImage('racebar-head');
            this.game.cache.removeImage('racebar-base');
            this.game.cache.removeImage('racebar-frame');
            this.game.cache.removeImage('racebar-fill');
            for (let i = 0; i < this.shipList.length; i++) {
                this.game.cache.removeImage("race-thumb-" + i);
            }
            this.shipList = null;
            super.shutdown();
        }
    }
    exports.InstallationRaceInProgressState = InstallationRaceInProgressState;
});
define("component/RewardPanel", ["require", "exports", "Game", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil"], function (require, exports, Game_65, TextUtil_27, LocalizationManager_59, ScaleUtil_50) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RewardPanel extends Phaser.Group {
        constructor(game, upgrades) {
            super(game);
            this.bgPanel = new Phaser.Image(this.game, 0, 0, "mission-reward-window");
            ScaleUtil_50.scaleToBounds(this.bgPanel, this.game.width * 0.8, this.game.height * 0.15);
            this.bgPanel.anchor.set(0.5, 0);
            this.add(this.bgPanel);
            const rewardHeader = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.075, LocalizationManager_59.LocalizationManager.Instance.text("mission.reward"), {
                font: Game_65.Game.DEFAULT_FONT,
                fontSize: Game_65.Game.FONT_SIZE_SMALL,
                fill: "#1992C9"
            });
            rewardHeader.anchor.set(0.5);
            this.add(rewardHeader);
            TextUtil_27.TextUtil.scaleDownSingleLineTextToWidth(rewardHeader, this.bgPanel.width * 0.9);
            this.rewardText = new Phaser.Text(this.game, -this.bgPanel.width * 0.2, this.bgPanel.height * 0.475, upgrades.toString() + LocalizationManager_59.LocalizationManager.Instance.text("race.ship_upgrade"), Object.assign({}, Game_65.Game.DEFAULT_FONT_STYLE, { fontSize: Game_65.Game.FONT_SIZE_MEDIUM, wordWrap: true, wordWrapWidth: this.bgPanel.width * 0.45, align: 'center' }));
            this.rewardText.lineSpacing = -this.game.height * 0.02;
            this.rewardText.anchor.set(0.5);
            TextUtil_27.TextUtil.scaleDownTextToHeight(this.rewardText, this.bgPanel.height * 0.8);
            this.add(this.rewardText);
            const rewardIconTag = 'ship-part-icon';
            this.rewardIcon = new Phaser.Image(this.game, this.bgPanel.width * 0.25, this.bgPanel.height * 0.525, rewardIconTag);
            this.rewardIcon.anchor.set(0.5);
            ScaleUtil_50.scaleToBounds(this.rewardIcon, this.bgPanel.width * 0.5, this.bgPanel.height * 0.7);
            this.add(this.rewardIcon);
        }
    }
    exports.RewardPanel = RewardPanel;
});
define("component/dialog/RaceInterruptDialog", ["require", "exports", "component/button/TextButton", "component/dialog/BasicSpaceDialog", "Game", "switchback-common/localization/LocalizationManager", "component/RewardPanel", "logic/GameController"], function (require, exports, TextButton_4, BasicSpaceDialog_10, Game_66, LocalizationManager_60, RewardPanel_1, GameController_37) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RaceInterruptDialog extends BasicSpaceDialog_10.BasicSpaceDialog {
        constructor(game, shutdownSignal) {
            super(game, shutdownSignal);
            const backgroundCircle = new Phaser.Image(this.game, this.width * 0.5, this.height * 0.2, "prerace-circle-design-red");
            backgroundCircle.anchor.set(0.5);
            backgroundCircle.centerX = this.game.width * 0.5;
            backgroundCircle.width = this.game.width * 0.6;
            backgroundCircle.scale.y = backgroundCircle.scale.x;
            this.content.add(backgroundCircle);
            const lightBeam = new Phaser.Image(this.game, this.game.width * 0.5, 0, "prerace-lightbeam");
            lightBeam.anchor.set(0.5);
            lightBeam.centerY = backgroundCircle.centerY;
            lightBeam.width = this.game.width * 0.9;
            lightBeam.height = this.game.height * 0.1;
            this.content.add(lightBeam);
            lightBeam.tint = RaceInterruptDialog.RED_TINT;
            const padding = this.game.width * 0.05;
            const topText = new Phaser.Text(this.game, 0, 0, LocalizationManager_60.LocalizationManager.Instance.text("race.training"), Object.assign({}, Game_66.Game.DEFAULT_FONT_STYLE, { fontSize: Game_66.Game.FONT_SIZE_EXTRA_LARGE }));
            topText.bottom = lightBeam.top + 2 * padding;
            topText.left = lightBeam.left + padding;
            this.content.add(topText);
            const bottomText = new Phaser.Text(this.game, 0, 0, LocalizationManager_60.LocalizationManager.Instance.text("race.paused"), Object.assign({}, Game_66.Game.DEFAULT_FONT_STYLE, { fill: Game_66.Game.FONT_COLOUR_RED, fontSize: Game_66.Game.FONT_SIZE_EXTRA_LARGE }));
            bottomText.top = lightBeam.bottom - 2 * padding;
            bottomText.right = lightBeam.right - padding;
            this.content.add(bottomText);
            const mainText = new Phaser.Text(this.game, 0, 0, LocalizationManager_60.LocalizationManager.Instance.text("race.interrupt_text"), Object.assign({}, Game_66.Game.DEFAULT_FONT_STYLE, { fontSize: Game_66.Game.FONT_SIZE_SMALL, wordWrap: true, wordWrapWidth: this.game.width * 0.8, align: 'center' }));
            mainText.centerX = this.game.width * 0.5;
            mainText.top = bottomText.bottom + padding;
            this.content.add(mainText);
            const rewardPanel = new RewardPanel_1.RewardPanel(this.game, GameController_37.GameController.Instance.getNumberPartsEarnedForLastPlayOf(GameController_37.GameController.Instance.activeGameState));
            rewardPanel.centerX = this.game.width * 0.5;
            rewardPanel.top = mainText.bottom + padding;
            this.content.add(rewardPanel);
            this.button = new TextButton_4.TextButton(this.game, LocalizationManager_60.LocalizationManager.Instance.text("buttons.continue"));
            this.button.width = this.game.width * 0.8;
            this.button.scale.y = this.button.scale.x;
            this.button.centerX = this.game.width * 0.5;
            this.button.top = rewardPanel.bottom + padding;
            this.content.add(this.button);
            this.onButtonClickedSignal = this.button.onButtonClickedSignal;
            this.resizeBackground(true);
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                // Hide the background mask, so that the pulsing alert frame can be seen
                this.shade.alpha = 0;
                this.button.show();
            });
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    RaceInterruptDialog.RED_TINT = 0xD80005;
    exports.RaceInterruptDialog = RaceInterruptDialog;
});
define("state/PreRaceState", ["require", "exports", "Game", "logic/GameController", "StateNames", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "state/SpaceBaseState", "logic/RaceController", "switchback-common/util/ScaleUtil", "component/dialog/BasicSpaceButtonDialog", "component/dialog/RaceInterruptDialog", "logic/SMAnalyticsManager", "component/dialog/ConfirmDialog"], function (require, exports, Game_67, GameController_38, StateNames_27, PromiseUtil_69, LocalizationManager_61, SpaceBaseState_10, RaceController_8, ScaleUtil_51, BasicSpaceButtonDialog_8, RaceInterruptDialog_1, SMAnalyticsManager_29, ConfirmDialog_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class PreRaceState extends SpaceBaseState_10.SpaceBaseState {
        init() {
            this.stateName = StateNames_27.StateNames.PRE_RACE;
            super.init(true, false);
            if (GameController_38.GameController.Instance.raceController.inInRangeOfInstallationBeacon()) {
                this.timeUntilRaceMs = (GameController_38.GameController.Instance.raceController.getTimeUntilScheduledRaceMS());
            }
            else {
                this.timeUntilRaceMs = 10000;
            }
        }
        preload() {
            super.preload();
            this.game.load.image("ship-thumb", "assets/ship/thumb.png");
            this.game.load.audio("countdownbeep1", ["assets/sound/race/countdownbeep1.ogg", "assets/sound/race/countdownbeep1.m4a"]);
            this.game.load.audio("countdownbeep2", ["assets/sound/race/countdownbeep2.ogg", "assets/sound/race/countdownbeep2.m4a"]);
            this.game.load.image("prerace-lightbeam", "assets/common/lightbeam.png");
            this.game.load.image("prerace-circle-design", "assets/hub/circle_design.png");
            this.game.load.image("prerace-circle-design-red", "assets/race/circle_design_red.png");
            this.game.load.image("prerace-hyperion-glow", "assets/outro/hyperion_glow.png");
            this.game.load.image("race-roof", "assets/race/dialogs/wdw_roof.png");
            this.game.load.image("border-red", "assets/race/borderflash_red.png");
            this.game.load.image("prerace-room-distant", "assets/race/dialogs/wdw_room_distant.png");
            this.game.load.image("prerace-room-ceiling", "assets/race/dialogs/wdw_room_ceiling.png");
            this.shipList = GameController_38.GameController.Instance.raceController.getShipList();
            for (let i = 0; i < this.shipList.length; i++) {
                this.game.load.image("prerace-thumb-" + i, "assets/race/ships/thumb_" + i + ".png");
            }
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const raceController = GameController_38.GameController.Instance.raceController;
                // If we somehow got here and there's no active race, jump back to the hub
                if (raceController.getRaceState() !== RaceController_8.RaceState.RACE_FLOW_STARTED) {
                    this.game.state.start(StateNames_27.StateNames.HUB);
                    return;
                }
                const padY = this.game.height * 0.03;
                /// visual objects
                const background = new Phaser.Image(this.game, 0, 0, "background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                this.raceStartingText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.05, LocalizationManager_61.LocalizationManager.Instance.text("race.race_starting"), Object.assign({}, Game_67.Game.DEFAULT_FONT_STYLE, { 
                    // fontSize: 75,
                    fontSize: Game_67.Game.FONT_SIZE_LARGE, align: 'center', fill: Game_67.Game.FONT_COLOUR_BLUE, wordWrap: true, wordWrapWidth: this.game.width * 0.8 }));
                this.raceStartingText.lineSpacing = -this.game.height * 0.015;
                this.raceStartingText.anchor.set(0.5, 0);
                this.add.existing(this.raceStartingText);
                const backgroundCircle = new Phaser.Image(this.game, this.game.width * 0.5, 0, "prerace-circle-design");
                backgroundCircle.anchor.set(0.5);
                backgroundCircle.width = this.game.width * 0.8;
                backgroundCircle.scale.y = backgroundCircle.scale.x;
                backgroundCircle.top = this.raceStartingText.bottom + (padY);
                this.add.existing(backgroundCircle);
                const backgroundStarburst = new Phaser.Image(this.game, backgroundCircle.centerX, backgroundCircle.centerY, "prerace-hyperion-glow");
                backgroundStarburst.anchor.set(0.5);
                backgroundStarburst.width = backgroundCircle.width * 0.65;
                backgroundStarburst.scale.y = backgroundStarburst.scale.x;
                this.add.existing(backgroundStarburst);
                const lightBeam = new Phaser.Image(this.game, this.game.width * 0.5, 0, "prerace-lightbeam");
                lightBeam.anchor.set(0.5);
                lightBeam.centerY = backgroundCircle.centerY;
                lightBeam.width = backgroundCircle.width * 1.2;
                this.add.existing(lightBeam);
                this.beam = backgroundStarburst;
                this.beamScale = backgroundStarburst.scale.x;
                this.timerText = new Phaser.Text(this.game, this.game.width * 0.5, backgroundCircle.centerY, '', Object.assign({}, Game_67.Game.DEFAULT_FONT_STYLE, { fontSize: 200, align: 'center' }));
                this.timerText.anchor.set(0.5);
                this.add.existing(this.timerText);
                this.game.add.tween(backgroundCircle).to({ angle: backgroundCircle.angle + 359 }, 45000, Phaser.Easing.Linear.None, true, 0, Infinity);
                this.game.add.tween(this.beam.scale).to({ x: this.beamScale * 1.2, y: this.beamScale * 1.2 }, 500, Phaser.Easing.Quadratic.InOut, true, 500, Infinity, true);
                this.game.add.tween(lightBeam.scale).to({ x: lightBeam.scale.x * 1.4, y: lightBeam.scale.y * 1.4 }, 500, Phaser.Easing.Quadratic.InOut, true, 500, Infinity, true);
                this.timerCaptionText = new Phaser.Text(this.game, this.game.width * 0.5, backgroundCircle.bottom + this.game.height * 0.04, LocalizationManager_61.LocalizationManager.Instance.text("race.race_starting_installation"), Object.assign({}, Game_67.Game.DEFAULT_FONT_STYLE, { fontSize: Game_67.Game.FONT_SIZE_MEDIUM, align: 'center', fill: Game_67.Game.DEFAULT_FONT_COLOUR, wordWrap: true, wordWrapWidth: this.game.width * 0.8 }));
                this.timerCaptionText.anchor.set(0.5, 0);
                this.add.existing(this.timerCaptionText);
                this.timerCaptionText.visible = false;
                SMAnalyticsManager_29.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/racestarting', 'SpaceRaceStarting', {
                    [SMAnalyticsManager_29.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_29.SMAnalyticsManager.GAME_PLAYS]: GameController_38.GameController.Instance.state.getMinigamePlays(StateNames_27.StateNames.DEVICE_RACE) + 1
                });
                this.doUserFlow();
            });
        }
        // Shows all the dialogs
        doUserFlow() {
            return __awaiter(this, void 0, void 0, function* () {
                // make timer wait for certain moment
                this.readyToRace = false;
                let wasInterrupted = false;
                if (GameController_38.GameController.Instance.isInGame) {
                    GameController_38.GameController.Instance.finishGame();
                    wasInterrupted = true;
                }
                this.refreshCountdownTimer();
                this.intervalRef = setInterval(this.refreshCountdownTimer.bind(this), PreRaceState.COUNTDOWN_REFRESH_RATE_MS);
                yield PromiseUtil_69.PromiseUtil.time(this.game, 350);
                this.openDoors();
                if (wasInterrupted) {
                    yield this.showGameInterruptDialog();
                }
                if (!GameController_38.GameController.Instance.raceController.inInRangeOfInstallationBeacon()) {
                    yield this.showRaceGuestsDialog();
                    this.readyToRace = true;
                    return;
                    // if its not on the right place, or haven't received a beacon don't ask the confuse the user with wayfinding
                }
                yield this.showRoofWayFind();
                // AC: if he answers NO to the wayfind its the same as hitting "i can't see the ceiling"
                if (!GameController_38.GameController.Instance.isInRangeOfInstallationPlayerConfirmed) {
                    yield this.showRaceGuestsDialog();
                    this.transitionToRaceState();
                    return;
                    // notice return before readyToRace
                }
                // at this point dialogs can be interrupted by refreshCountdownTimer
                this.readyToRace = true;
                yield this.showPreInstallationDialogs();
            });
        }
        refreshCountdownTimer() {
            this.timeUntilRaceMs -= PreRaceState.COUNTDOWN_REFRESH_RATE_MS;
            const timeUntilScheduledRaceS = Math.ceil(this.timeUntilRaceMs * 0.001);
            this.timerText.text = LocalizationManager_61.LocalizationManager.Instance.textWithReplacements(`race.launch_countdown`, {
                time: Math.max(0, timeUntilScheduledRaceS)
            }).toUpperCase();
            // waits for the wayfind dialog, or timeout
            // timeUntilScheduledRaceS can go negative,
            // this will interrupt other dialogs
            const timeOut = timeUntilScheduledRaceS <= (PreRaceState.MIN_TIME_BEFORE_RACE_MS * 0.001);
            if (timeOut && this.readyToRace) {
                this.game.sound.play('countdownbeep2');
                this.transitionToRaceState();
            }
            else {
                this.game.sound.play('countdownbeep1');
            }
        }
        showGameInterruptDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                let dialog;
                let redBorder;
                redBorder = new Phaser.Image(this.game, 0, 0, "border-red");
                redBorder.width = this.game.width;
                redBorder.height = this.game.height;
                this.add.existing(redBorder);
                this.game.add.tween(redBorder).to({ alpha: 0 }, 250, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                dialog = new RaceInterruptDialog_1.RaceInterruptDialog(this.game, this.shutdownSignal);
                this.add.existing(dialog);
                dialog.show();
                SMAnalyticsManager_29.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/interruption', 'SpaceRaceInterruption', {
                    [SMAnalyticsManager_29.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_29.SMAnalyticsManager.GAME_PLAYS]: GameController_38.GameController.Instance.state.getMinigamePlays(StateNames_27.StateNames.DEVICE_RACE) + 1
                });
                yield PromiseUtil_69.PromiseUtil.signal(dialog.onButtonClickedSignal);
                yield dialog.hide();
                dialog.destroy();
                this.game.tweens.removeFrom(redBorder);
                redBorder.destroy();
            });
        }
        /**
         * There are two wayfinding dialogs. It requires both of them to be confirmed to
         * show the installation.
         */
        showRoofWayFind() {
            return __awaiter(this, void 0, void 0, function* () {
                // wrong location, or no beacon or already confirmed
                if (!GameController_38.GameController.Instance.raceController.inInRangeOfInstallationBeacon() ||
                    GameController_38.GameController.Instance.isInRangeOfInstallationPlayerConfirmed) {
                    return;
                }
                let wayfindPhoto;
                let wayfindRoomDialog;
                // show first wayfinding dialog of the room
                if (!GameController_38.GameController.Instance.isInstallationFirstWayfindDialogConfirmed) {
                    wayfindPhoto = new Phaser.Sprite(this.game, 0, 0, "prerace-room-distant");
                    ScaleUtil_51.scaleToBounds(wayfindPhoto, this.game.width * 0.88, this.game.height * 0.8);
                    wayfindRoomDialog = new ConfirmDialog_5.ConfirmDialog(this.game, LocalizationManager_61.LocalizationManager.Instance.text("race.wayfind_text_distant"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.yes"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.no"), this.shutdownSignal, wayfindPhoto);
                    this.add.existing(wayfindRoomDialog);
                    wayfindRoomDialog.show();
                    GameController_38.GameController.Instance.isInstallationFirstWayfindDialogConfirmed = !!(yield PromiseUtil_69.PromiseUtil.signal(wayfindRoomDialog.onButtonClickedSignal));
                    yield wayfindRoomDialog.hide();
                    wayfindRoomDialog.destroy();
                    wayfindPhoto.destroy();
                }
                // if they confirm show the second wayfinding dialog of the ceiling
                if (GameController_38.GameController.Instance.isInstallationFirstWayfindDialogConfirmed) {
                    wayfindPhoto = new Phaser.Sprite(this.game, 0, 0, "prerace-room-ceiling");
                    ScaleUtil_51.scaleToBounds(wayfindPhoto, this.game.width * 0.88, this.game.height * 0.8);
                    wayfindRoomDialog = new ConfirmDialog_5.ConfirmDialog(this.game, LocalizationManager_61.LocalizationManager.Instance.text("race.wayfind_text_ceiling"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.yes"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.no"), this.shutdownSignal, wayfindPhoto);
                    this.add.existing(wayfindRoomDialog);
                    wayfindRoomDialog.show();
                    // if the race has been confirmed by the guest as being visible, it is now playable
                    // if guests quit the app and reenter they are forced to reconfirm as we can't guarantee 
                    // they are still in this room or in range of the ceiling
                    GameController_38.GameController.Instance.isInRangeOfInstallationPlayerConfirmed = !!(yield PromiseUtil_69.PromiseUtil.signal(wayfindRoomDialog.onButtonClickedSignal));
                    yield wayfindRoomDialog.hide();
                    wayfindRoomDialog.destroy();
                    wayfindPhoto.destroy();
                }
            });
        }
        // These can be interrupted on timeout
        showPreInstallationDialogs() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.timeUntilRaceMs <= PreRaceState.MIN_TIME_BEFORE_RACE_MS) {
                    return;
                }
                // can't show this text before the installation wayfinding is successful
                this.timerCaptionText.visible = true;
                // show remaining prerace dialogs
                yield this.showRaceGuestsDialog();
                yield this.showRoofReminderDialog();
                yield this.showStartAfterCountdownDialog();
            });
        }
        showRoofReminderDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                if (GameController_38.GameController.Instance.raceController.isRoofRaceScheduled()) {
                    // Show third wayfinding dialog - look up at window
                    const roof = new Phaser.Sprite(this.game, 0, 0, "race-roof");
                    ScaleUtil_51.scaleToBounds(roof, this.game.width * 0.88, this.game.height * 0.8);
                    const lookUp = new BasicSpaceButtonDialog_8.BasicSpaceButtonDialog(this.game, LocalizationManager_61.LocalizationManager.Instance.text("race.look_up"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.continue"), this.shutdownSignal, null, null, roof);
                    this.add.existing(lookUp);
                    lookUp.show();
                    yield PromiseUtil_69.PromiseUtil.signal(lookUp.onButtonClickedSignal);
                    yield lookUp.hide();
                }
            });
        }
        showStartAfterCountdownDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                const startAfterCountdownDialog = new BasicSpaceButtonDialog_8.BasicSpaceButtonDialog(this.game, LocalizationManager_61.LocalizationManager.Instance.text("race.launch_after_countdown"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.continue"), this.shutdownSignal);
                this.add.existing(startAfterCountdownDialog);
                startAfterCountdownDialog.show();
                yield PromiseUtil_69.PromiseUtil.signal(startAfterCountdownDialog.onButtonClickedSignal);
                yield startAfterCountdownDialog.hide();
                startAfterCountdownDialog.destroy();
            });
        }
        showRaceGuestsDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                const raceGuestsDialog = new BasicSpaceButtonDialog_8.BasicSpaceButtonDialog(this.game, LocalizationManager_61.LocalizationManager.Instance.text("race.race_everyone"), LocalizationManager_61.LocalizationManager.Instance.text("buttons.continue"), this.shutdownSignal);
                this.add.existing(raceGuestsDialog);
                raceGuestsDialog.show();
                yield PromiseUtil_69.PromiseUtil.signal(raceGuestsDialog.onButtonClickedSignal);
                yield raceGuestsDialog.hide();
                raceGuestsDialog.destroy();
            });
        }
        transitionToRaceState() {
            return __awaiter(this, void 0, void 0, function* () {
                clearInterval(this.intervalRef);
                GameController_38.GameController.Instance.state.numRaces++;
                yield this.closeDoors();
                const timeUntilScheduledRaceS = Math.ceil(this.timeUntilRaceMs * 0.001);
                // if !state.atRaceRoom run device race
                // if timeUntil <0 go to DeviceRace
                const doInstallation = GameController_38.GameController.Instance.raceController.isRoofRaceScheduled() &&
                    GameController_38.GameController.Instance.isInRangeOfInstallationPlayerConfirmed &&
                    timeUntilScheduledRaceS >= 0;
                SMAnalyticsManager_29.SMAnalyticsManager.Instance.trackAction('Installation_Result', 'Installation', 'Result', 'SpaceRace', {
                    [SMAnalyticsManager_29.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_29.SMAnalyticsManager.GAME_PLAYS]: GameController_38.GameController.Instance.state.getMinigamePlays(StateNames_27.StateNames.DEVICE_RACE) + 1,
                    [SMAnalyticsManager_29.SMAnalyticsManager.GAME_SCORE]: GameController_38.GameController.Instance.raceController.shipPlacement + 1,
                    [SMAnalyticsManager_29.SMAnalyticsManager.INSTALLATION_TRIGGER]: doInstallation ? 1 : 0
                });
                if (doInstallation) {
                    this.game.state.start(StateNames_27.StateNames.INSTALLATION_RACE_IN_PROGRESS);
                }
                else {
                    this.game.state.start(StateNames_27.StateNames.DEVICE_RACE);
                }
            });
        }
        shutdown() {
            clearInterval(this.intervalRef);
            this.game.cache.removeSound("countdownbeep1");
            this.game.cache.removeSound("countdownbeep2");
            this.game.cache.removeImage("ship-thumb");
            this.game.cache.removeImage("prerace-lightbeam");
            this.game.cache.removeImage("prerace-circle-design");
            this.game.cache.removeImage("prerace-circle-design-red");
            this.game.cache.removeImage("prerace-hyperion-glow");
            this.game.cache.removeImage("race-roof");
            this.game.cache.removeImage("border-red");
            this.game.cache.removeImage("prerace-room-distant");
            this.game.cache.removeImage("prerace-room-ceiling");
            for (let i = 0; i < this.shipList.length; i++) {
                this.game.cache.removeImage("prerace-thumb-" + i);
            }
            this.shipList = null;
            this.timerText = null;
            this.raceStartingText = null;
            this.timerCaptionText = null;
            this.beam = null;
            super.shutdown();
        }
    }
    PreRaceState.COUNTDOWN_REFRESH_RATE_MS = 1000;
    PreRaceState.MIN_TIME_BEFORE_RACE_MS = 2000;
    exports.PreRaceState = PreRaceState;
});
define("state/NarrativeState", ["require", "exports", "Game", "StateNames", "component/button/CTAButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "logic/GameController", "state/SpaceBaseState"], function (require, exports, Game_68, StateNames_28, CTAButton_9, LocalizationManager_62, PromiseUtil_70, ScaleUtil_52, GameController_39, SpaceBaseState_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NarrativeState extends SpaceBaseState_11.SpaceBaseState {
        init() {
            super.init();
            this.spinSpeed = 6;
        }
        preload() {
            super.preload();
            this.game.load.image("lightbeam", "assets/common/lightbeam.png");
            this.game.load.image("circle-design", "assets/hub/circle_design.png");
            this.game.load.audio("mission-brief-anim", ["assets/sound/narrative/mission-brief-anim.ogg", "assets/sound/narrative/mission-brief-anim.m4a"]);
            this.game.load.audio("mission-brief-cta-appear", ["assets/sound/narrative/mission-brief-cta-appear.ogg", "assets/sound/narrative/mission-brief-cta-appear.m4a"]);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const background = new Phaser.Image(this.game, 0, 0, "background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                const buttonTextKey = this.getNextButtonTextTag();
                this.nextButton = new CTAButton_9.CTAButton(this.game, LocalizationManager_62.LocalizationManager.Instance.text(buttonTextKey));
                this.nextButton.x = this.game.width * 0.5;
                this.nextButton.y = this.game.height * 0.85;
                this.add.existing(this.nextButton);
                this.spinCircle = new Phaser.Image(this.game, 0, 0, "circle-design");
                ScaleUtil_52.scaleToBounds(this.spinCircle, this.game.width * 0.65, this.game.width * 0.65);
                this.spinCircle.anchor.set(0.5);
                this.spinCircle.centerX = this.game.width * 0.5;
                this.spinCircle.centerY = this.game.height * 0.25;
                this.spinCircle.alpha = 0;
                this.add.existing(this.spinCircle);
                const lightbeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
                ScaleUtil_52.scaleToBounds(lightbeam, this.game.width, this.game.height * 0.05);
                lightbeam.centerX = this.game.width * 0.5;
                lightbeam.centerY = this.spinCircle.centerY;
                lightbeam.alpha = 0;
                this.add.existing(lightbeam);
                const missionText = new Phaser.Text(this.game, 0, 0, this.getTitle(), {
                    font: Game_68.Game.DEFAULT_FONT,
                    fill: Game_68.Game.DEFAULT_FONT_COLOUR,
                    fontSize: 100,
                    stroke: Game_68.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_68.Game.FONT_STROKE_SMALL
                });
                missionText.left = this.game.width;
                missionText.bottom = lightbeam.top + this.game.height * 0.02;
                this.add.existing(missionText);
                const lowerTextKey = this.getSubTitleTextTag();
                const briefText = new Phaser.Text(this.game, 0, 0, LocalizationManager_62.LocalizationManager.Instance.text(lowerTextKey), {
                    font: Game_68.Game.DEFAULT_FONT,
                    fill: Game_68.Game.FONT_COLOUR_BLUE,
                    fontSize: 100,
                    stroke: Game_68.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_68.Game.FONT_STROKE_SMALL
                });
                briefText.right = 0;
                briefText.top = lightbeam.bottom - this.game.height * 0.02;
                this.add.existing(briefText);
                this.createContent();
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.sound.play("mission-brief-anim");
                // Circle/lightbeam tweens
                // Alpha
                this.game.add.tween(this.spinCircle).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true);
                this.game.add.tween(lightbeam).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true);
                // Scale
                this.game.add.tween(this.spinCircle.scale).from({ x: 0.8, y: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(lightbeam.scale).from({ x: 0.8, y: 0.8 }, 200, Phaser.Easing.Linear.None, true);
                // Spin 
                // This is tweening the value of spinSpeed, which is being used to set the rotation in the update loop
                yield PromiseUtil_70.PromiseUtil.tween2(this.game.add.tween(this).to({ spinSpeed: 0.5 }, 1000, Phaser.Easing.Sinusoidal.Out, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                // Text tweens
                yield PromiseUtil_70.PromiseUtil.tween2(this.game.add.tween(missionText).to({ centerX: this.game.width * 0.5 }, 500, Phaser.Easing.Exponential.Out, true).chain(this.game.add.tween(briefText).to({ centerX: this.game.width * 0.5 }, 500, Phaser.Easing.Exponential.Out, true)), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                yield this.playContent();
            });
        }
        isHighscoreAvailable() {
            const gameController = GameController_39.GameController.Instance;
            if (!gameController.hasCompletedMinigame(GameController_39.GameController.Instance.activeGameState)) {
                // When there's no highscore don't add anything to the content
                // this return is important also to avoid creating a highscore before playing
                return false;
            }
            const mgScore = gameController.state.getMinigameScore(GameController_39.GameController.Instance.activeGameState);
            // It is possible to have played a game, but posted a score of 0. 
            // We don't want to show that as a highscore.
            if (mgScore.highScore === 0) {
                return false;
            }
            return true;
        }
        update() {
            super.update();
            this.spinCircle.angle += this.spinSpeed;
        }
        getTitle() {
            switch (GameController_39.GameController.Instance.activeGameState) {
                case (StateNames_28.StateNames.GAME_ASTEROIDS): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_asteroids");
                }
                case (StateNames_28.StateNames.GAME_DOCKING): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_docking");
                }
                case (StateNames_28.StateNames.GAME_ENERGY_BALLS): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_energy_balls");
                }
                case (StateNames_28.StateNames.GAME_MEMORY): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_memory");
                }
                case (StateNames_28.StateNames.GAME_PIPES): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_pipes");
                }
                case (StateNames_28.StateNames.GAME_RELAY_INTRO): {
                    return LocalizationManager_62.LocalizationManager.Instance.text("game_names.game_relay");
                }
            }
        }
        shutdown() {
            this.nextButton = null;
            this.spinCircle = null;
            this.game.cache.removeImage("circle-design");
            this.game.cache.removeSound("mission-brief-anim");
            this.game.cache.removeSound("mission-brief-cta-appear");
            super.shutdown();
        }
    }
    exports.NarrativeState = NarrativeState;
});
define("component/HighscorePanel", ["require", "exports", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "logic/GameController", "switchback-common/util/ScaleUtil"], function (require, exports, Game_69, LocalizationManager_63, TextUtil_28, GameController_40, ScaleUtil_53) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HighscorePanel extends Phaser.Group {
        constructor(game) {
            super(game);
            this.createContent();
            this.populatePanelWithHighscore();
        }
        createContent() {
            this.bgPanel = new Phaser.Image(this.game, 0, 0, "mission-window-tall");
            ScaleUtil_53.scaleToBounds(this.bgPanel, this.game.width * 0.8, this.game.height * 0.15);
            this.bgPanel.anchor.set(0.5, 0);
            this.headerText = new Phaser.Text(this.game, 0, this.bgPanel.height * 0.075, LocalizationManager_63.LocalizationManager.Instance.text("scores.high_score"), {
                font: Game_69.Game.DEFAULT_FONT,
                fontSize: Game_69.Game.FONT_SIZE_SMALL,
                fill: "#335f90"
            });
            this.headerText.anchor.set(0.5);
            TextUtil_28.TextUtil.scaleDownSingleLineTextToWidth(this.headerText, this.bgPanel.width * 0.9);
            this.highscoreText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, Game_69.Game.DEFAULT_FONT_STYLE, { fontSize: Game_69.Game.FONT_SIZE_LARGE, fill: Game_69.Game.FONT_COLOUR_BLUE, align: 'center' }));
            this.highscoreText.anchor.set(0.5);
            this.highscoreText.y = this.bgPanel.height * 0.375;
            this.descriptionText = new Phaser.Text(this.game, 0, 0, LocalizationManager_63.LocalizationManager.Instance.text("scores.beat_prompt"), {
                font: Game_69.Game.DEFAULT_FONT,
                fontSize: Game_69.Game.FONT_SIZE_SMALL,
                fill: Game_69.Game.FONT_COLOUR_PURPLE,
                strokeThickness: Game_69.Game.FONT_STROKE_LARGE,
                align: 'center'
            });
            this.descriptionText.anchor.set(0.5);
            TextUtil_28.TextUtil.scaleDownSingleLineTextToWidth(this.descriptionText, this.bgPanel.width * 0.9);
            this.add(this.bgPanel);
            this.add(this.headerText);
            this.add(this.highscoreText);
            this.add(this.descriptionText);
        }
        populatePanelWithHighscore() {
            const gameController = GameController_40.GameController.Instance;
            const mgScore = gameController.state.getMinigameScore(gameController.activeGameState);
            const isTime = gameController.isTimeBasedGame(gameController.activeGameState);
            if (isTime) {
                const bestTime = TextUtil_28.TextUtil.formattedTime(Math.round(mgScore.highScore));
                this.highscoreText.text = LocalizationManager_63.LocalizationManager.Instance.textWithReplacements("scores.best_time", {
                    score_number: bestTime
                });
            }
            else {
                const bestScore = Math.round(mgScore.highScore).toString();
                this.highscoreText.text = LocalizationManager_63.LocalizationManager.Instance.textWithReplacements("scores.best_score", {
                    score_number: bestScore
                });
            }
            this.highscoreText.fontSize = Game_69.Game.FONT_SIZE_LARGE;
            TextUtil_28.TextUtil.scaleDownSingleLineTextToWidth(this.highscoreText, this.bgPanel.width * 0.9);
            this.descriptionText.top = this.highscoreText.bottom;
        }
    }
    exports.HighscorePanel = HighscorePanel;
});
define("switchback-common/component/text/TypewriterText", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_71) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TypewriterText extends Phaser.Text {
        constructor() {
            super(...arguments);
            this.isSkipping = false;
            this.softSkip = false;
        }
        typewriter(letterDelay, soundTag = null, softSkipParent = null, shutdownSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (softSkipParent && !this.isSkipping) {
                    softSkipParent.events.onInputDown.addOnce(this.onSoftSkip, this);
                }
                let sndTypewriterLoop;
                let soundVolumeTween;
                if (soundTag && !this.isSkipping) {
                    sndTypewriterLoop = this.game.sound.play(soundTag, 0.0, true);
                    sndTypewriterLoop.play();
                    sndTypewriterLoop.volume = 0;
                    soundVolumeTween = this.game.add.tween(sndTypewriterLoop).to({ volume: 0.5 }, 500, Phaser.Easing.Linear.None, true);
                }
                const numChars = this.text.length;
                this.softSkip = false;
                let isShutdown = false;
                for (let i = 0; !this.isSkipping && !this.softSkip && i < numChars; i++) {
                    this.clearColors();
                    this.addColor("rgba(0,0,0,0)", i);
                    yield PromiseUtil_71.PromiseUtil.realTime(letterDelay, shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        if (sndTypewriterLoop) {
                            soundVolumeTween.stop();
                            sndTypewriterLoop.stop();
                        }
                        return;
                    }
                }
                if (softSkipParent) {
                    softSkipParent.events.onInputDown.remove(this.onSoftSkip, this);
                }
                this.clearColors();
                // Adding colors offsets the text kerning differently, so this has to be here for consistency even though it doesn't color anything
                this.addColor("rgba(0,0,0,0)", numChars);
                this.softSkip = false;
                if (sndTypewriterLoop) {
                    soundVolumeTween.stop();
                    sndTypewriterLoop.stop();
                }
            });
        }
        clearText() {
            this.addColor("rgba(0,0,0,0)", 0);
        }
        onSoftSkip() {
            this.softSkip = true;
        }
    }
    exports.TypewriterText = TypewriterText;
});
define("state/NarrativeBriefingState", ["require", "exports", "state/NarrativeState", "StateNames", "logic/GameController", "switchback-common/localization/LocalizationManager", "config/SpaceMountainConfig", "logic/SMAnalyticsManager", "Game", "switchback-common/util/PromiseUtil", "component/HighscorePanel", "switchback-common/component/text/TypewriterText"], function (require, exports, NarrativeState_1, StateNames_29, GameController_41, LocalizationManager_64, SpaceMountainConfig_14, SMAnalyticsManager_30, Game_70, PromiseUtil_72, HighscorePanel_1, TypewriterText_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NarrativeBriefingState extends NarrativeState_1.NarrativeState {
        init() {
            this.stateName = StateNames_29.StateNames.NARRATIVE_BRIEFING;
            super.init();
        }
        preload() {
            super.preload();
            this.game.load.audio("mission-brief-text-loop", ["assets/sound/narrative/mission-brief-text-loop.ogg", "assets/sound/narrative/mission-brief-text-loop.m4a"]);
        }
        createContent() {
            const padding = this.game.width * 0.05;
            this.narrativeText = new TypewriterText_1.TypewriterText(this.game, 0, 0, this.getContentTextTag(), {
                font: Game_70.Game.DEFAULT_FONT,
                fill: Game_70.Game.FONT_COLOUR_BLUE,
                fontSize: Game_70.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9,
                align: 'center'
            });
            this.narrativeText.centerX = this.game.width * 0.5;
            this.narrativeText.visible = false;
            this.add.existing(this.narrativeText);
            // calculate the number of parts earned
            if (this.isHighscoreAvailable()) {
                this.highscorePanel = new HighscorePanel_1.HighscorePanel(this.game);
                this.highscorePanel.centerX = this.game.width * 0.5;
                this.highscorePanel.bottom = this.nextButton.top - padding;
                this.add.existing(this.highscorePanel);
                this.highscorePanelOrigScale = this.highscorePanel.scale.x;
                this.highscorePanel.scale.set(0);
                const textCentre = (this.highscorePanel.top - this.spinCircle.bottom) * 0.5 + this.spinCircle.bottom;
                this.narrativeText.centerY = textCentre;
            }
            else {
                const textCentre = (this.nextButton.top - this.spinCircle.bottom) * 0.5 + this.spinCircle.bottom;
                this.narrativeText.centerY = textCentre;
            }
        }
        playContent() {
            return __awaiter(this, void 0, void 0, function* () {
                this.narrativeText.visible = true;
                yield this.narrativeText.typewriter(15, 'mission-brief-text-loop', null, this.shutdownSignal);
                if (this.isShutdown) {
                    return;
                }
                if (this.highscorePanel) {
                    yield PromiseUtil_72.PromiseUtil.tween(this.game, this.game.add.tween(this.highscorePanel.scale)
                        .to({ x: this.highscorePanelOrigScale, y: this.highscorePanelOrigScale }, 400, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                }
                this.game.sound.play("mission-brief-cta-appear");
                yield this.nextButton.show();
                if (this.isShutdown) {
                    return;
                }
                //////////////////////////
                // METRICS
                //////////////////////////
                const metricState = 'content/game/inqueue/minigame/title';
                const gaStateSuffix = 'Title';
                const gaAction = 'Play';
                const adobeAction = 'PlayMiniGame';
                const metricGameName = SMAnalyticsManager_30.SMAnalyticsManager.gameNameMap.get(GameController_41.GameController.Instance.activeGameState);
                /**
                 * N.B. This metric should be inclusive of the game we're just about to play.
                 * The internal tracking hasn't counted this game if we're showing the intro,
                 * so increment it on the intro to fake it for these calls.
                 */
                let numGamePlays = GameController_41.GameController.Instance.state.getMinigamePlays(this.stateName);
                let data = {
                    [SMAnalyticsManager_30.SMAnalyticsManager.GAME_NAME_MINI]: metricGameName,
                    [SMAnalyticsManager_30.SMAnalyticsManager.GAME_PLAYS]: numGamePlays + 1
                };
                SMAnalyticsManager_30.SMAnalyticsManager.Instance.trackState(metricState, `${metricGameName}${gaStateSuffix}`, data);
                //////////////////////////
                // END METRICS
                //////////////////////////
                this.nextButton.onButtonClickedSignal.addOnce(() => {
                    SMAnalyticsManager_30.SMAnalyticsManager.Instance.trackAction(adobeAction, gaStateSuffix, gaAction, metricGameName, data);
                    SMAnalyticsManager_30.SMAnalyticsManager.Instance.startTiming(`RocketRace:Mini:${metricGameName}`, data);
                    this.transitionToGame();
                });
            });
        }
        getSubTitleTextTag() {
            return "narrative_intro.brief";
        }
        getNextButtonTextTag() {
            return "buttons.start";
        }
        getContentTextTag() {
            switch (GameController_41.GameController.Instance.activeGameState) {
                case (StateNames_29.StateNames.GAME_ASTEROIDS): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.asteroids");
                }
                case (StateNames_29.StateNames.GAME_DOCKING): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.docking");
                }
                case (StateNames_29.StateNames.GAME_ENERGY_BALLS): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.energy_balls");
                }
                case (StateNames_29.StateNames.GAME_MEMORY): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.memory");
                }
                case (StateNames_29.StateNames.GAME_PIPES): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.pipes");
                }
                case (StateNames_29.StateNames.GAME_RELAY_INTRO): {
                    return LocalizationManager_64.LocalizationManager.Instance.text("narrative_intro.relay");
                }
            }
        }
        transitionToGame() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToGame()`);
                yield this.showOverlay();
                if (SpaceMountainConfig_14.SpaceMountainConfig.Instance.skipGames) {
                    GameController_41.GameController.Instance.postMinigameScore(GameController_41.GameController.Instance.activeGameState, 0);
                    GameController_41.GameController.Instance.finishGame();
                    this.game.state.start(StateNames_29.StateNames.NARRATIVE_DEBRIEFING);
                }
                else {
                    this.game.state.start(GameController_41.GameController.Instance.activeGameState);
                }
            });
        }
        shutdown() {
            this.narrativeText = null;
            this.highscorePanel = null;
            this.game.cache.removeSound("mission-brief-text-loop");
            super.shutdown();
        }
    }
    exports.NarrativeBriefingState = NarrativeBriefingState;
});
define("component/HighscoreResultPanel", ["require", "exports", "Game", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "logic/GameController", "component/HighscorePanel"], function (require, exports, Game_71, LocalizationManager_65, TextUtil_29, GameController_42, HighscorePanel_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HighscoreResultPanel extends HighscorePanel_2.HighscorePanel {
        createContent() {
            super.createContent();
            this.currentScoreText = new Phaser.Text(this.game, 0, 0, "", Object.assign({}, Game_71.Game.DEFAULT_FONT_STYLE, { fontSize: Game_71.Game.FONT_SIZE_LARGE, fill: Game_71.Game.FONT_COLOUR_YELLOW, align: 'left' }));
            this.currentScoreText.anchor.set(0.5);
            this.currentScoreText.y = this.bgPanel.height * 0.3;
            this.add(this.currentScoreText);
        }
        populatePanelWithHighscore() {
            super.populatePanelWithHighscore();
            const gameController = GameController_42.GameController.Instance;
            const mgScore = gameController.state.getMinigameScore(gameController.activeGameState);
            let isTime = gameController.isTimeBasedGame(gameController.activeGameState);
            if (isTime) {
                let currentTime = TextUtil_29.TextUtil.formattedTime(Math.round(mgScore.currentScore));
                this.currentScoreText.text = LocalizationManager_65.LocalizationManager.Instance.textWithReplacements("scores.current_time", {
                    score_number: currentTime
                });
            }
            else {
                let currentScore = Math.round(mgScore.currentScore).toString();
                this.currentScoreText.text = LocalizationManager_65.LocalizationManager.Instance.textWithReplacements("scores.current_score", {
                    score_number: currentScore
                });
            }
            this.currentScoreText.fontSize = Game_71.Game.FONT_SIZE_LARGE;
            TextUtil_29.TextUtil.scaleDownSingleLineTextToWidth(this.currentScoreText, this.bgPanel.width * 0.9);
            if (mgScore.wasHighScoreInLastPlay) {
                this.descriptionText.text = LocalizationManager_65.LocalizationManager.Instance.text("scores.new_highscore");
            }
            this.highscoreText.top = this.currentScoreText.bottom - this.bgPanel.height * 0.065;
            this.descriptionText.top = this.highscoreText.bottom - this.bgPanel.height * 0.065;
        }
    }
    exports.HighscoreResultPanel = HighscoreResultPanel;
});
define("state/NarrativeDebriefingState", ["require", "exports", "state/NarrativeState", "StateNames", "logic/GameController", "switchback-common/localization/LocalizationManager", "logic/SMAnalyticsManager", "component/RewardPanel", "switchback-common/util/PromiseUtil", "component/HighscoreResultPanel"], function (require, exports, NarrativeState_2, StateNames_30, GameController_43, LocalizationManager_66, SMAnalyticsManager_31, RewardPanel_2, PromiseUtil_73, HighscoreResultPanel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NarrativeDebriefingState extends NarrativeState_2.NarrativeState {
        init() {
            this.stateName = StateNames_30.StateNames.NARRATIVE_DEBRIEFING;
            super.init();
        }
        preload() {
            super.preload();
            this.load.image("mission-reward-window", "assets/hub/mission-reward-window.png");
            this.load.image("mission-window-tall", "assets/hub/mission-window-tall.png");
            this.load.image("ship-part-icon", "assets/hub/ship-part-icon.png");
        }
        createContent() {
            const padding = this.game.width * 0.05;
            // calculate the number of parts earned
            this.rewardPanel = new RewardPanel_2.RewardPanel(this.game, GameController_43.GameController.Instance.getNumberPartsEarnedForLastPlayOf(GameController_43.GameController.Instance.activeGameState));
            this.rewardPanel.centerX = this.game.width * 0.5;
            this.rewardPanel.bottom = this.nextButton.top - padding;
            this.add.existing(this.rewardPanel);
            this.rewardPanelOrigScale = this.rewardPanel.scale.x;
            this.rewardPanel.scale.set(0);
            if (this.isHighscoreAvailable()) {
                this.highscoreResultPanel = new HighscoreResultPanel_1.HighscoreResultPanel(this.game);
                this.highscoreResultPanel.centerX = this.game.width * 0.5;
                this.highscoreResultPanel.bottom = this.rewardPanel.top - padding;
                this.add.existing(this.highscoreResultPanel);
                this.highscoreResultPanelOrigScale = this.highscoreResultPanel.scale.x;
                this.highscoreResultPanel.scale.set(0);
            }
        }
        playContent() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.highscoreResultPanel) {
                    yield PromiseUtil_73.PromiseUtil.tween2(this.game.add.tween(this.highscoreResultPanel.scale)
                        .to({ x: this.highscoreResultPanelOrigScale, y: this.highscoreResultPanelOrigScale }, 400, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { });
                    if (this.isShutdown) {
                        return;
                    }
                }
                yield PromiseUtil_73.PromiseUtil.tween2(this.game.add.tween(this.rewardPanel.scale)
                    .to({ x: this.rewardPanelOrigScale, y: this.rewardPanelOrigScale }, 400, Phaser.Easing.Back.Out, true), this.shutdownSignal).catch(() => { });
                if (this.isShutdown) {
                    return;
                }
                this.game.sound.play("mission-brief-cta-appear");
                yield this.nextButton.show();
                if (this.isShutdown) {
                    return;
                }
                //////////////////////////
                // METRICS
                //////////////////////////
                const metricState = 'content/game/inqueue/minigame/result';
                const gaStateSuffix = 'Result';
                const gaAction = 'Continue';
                const adobeAction = 'ContinueGame';
                const metricGameName = SMAnalyticsManager_31.SMAnalyticsManager.gameNameMap.get(GameController_43.GameController.Instance.activeGameState);
                let numGamePlays = GameController_43.GameController.Instance.state.getMinigamePlays(this.stateName);
                let data = {
                    [SMAnalyticsManager_31.SMAnalyticsManager.GAME_NAME_MINI]: metricGameName,
                    [SMAnalyticsManager_31.SMAnalyticsManager.GAME_PLAYS]: numGamePlays,
                    [SMAnalyticsManager_31.SMAnalyticsManager.GAME_SCORE]: GameController_43.GameController.Instance.state.getMinigameScore(GameController_43.GameController.Instance.activeGameState).currentScore
                };
                SMAnalyticsManager_31.SMAnalyticsManager.Instance.trackState(metricState, `${metricGameName}${gaStateSuffix}`, data);
                //////////////////////////
                // END METRICS
                //////////////////////////
                this.nextButton.onButtonClickedSignal.addOnce(() => {
                    SMAnalyticsManager_31.SMAnalyticsManager.Instance.trackAction(adobeAction, gaStateSuffix, gaAction, metricGameName, data);
                    this.transitionToHub();
                });
            });
        }
        getSubTitleTextTag() {
            return "narrative_outro.debrief";
        }
        getContentTextTag() {
            switch (GameController_43.GameController.Instance.activeGameState) {
                case (StateNames_30.StateNames.GAME_ASTEROIDS): {
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.asteroids");
                }
                case (StateNames_30.StateNames.GAME_DOCKING): {
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.docking");
                }
                case (StateNames_30.StateNames.GAME_ENERGY_BALLS): {
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.energy_balls");
                }
                case (StateNames_30.StateNames.GAME_MEMORY): {
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.memory");
                }
                case (StateNames_30.StateNames.GAME_PIPES): {
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.pipes");
                }
                case (StateNames_30.StateNames.GAME_RELAY_INTRO): { // N.B. This uses the intro's statename, even for the outro lookup.
                    return LocalizationManager_66.LocalizationManager.Instance.text("narrative_outro.relay");
                }
            }
        }
        getNextButtonTextTag() {
            return "buttons.continue";
        }
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] transitionToHub()`);
                yield this.showOverlay();
                if (this.isShutdown) {
                    return;
                }
                this.game.state.start(StateNames_30.StateNames.HUB);
            });
        }
        shutdown() {
            this.highscoreResultPanel = null;
            this.highscoreResultPanelOrigScale = null;
            this.rewardPanel = null;
            this.rewardPanelOrigScale = null;
            super.shutdown();
        }
    }
    exports.NarrativeDebriefingState = NarrativeDebriefingState;
});
define("state/PostRacePlacementState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "Game", "logic/GameController", "StateNames", "state/SpaceBaseState", "component/button/CTAButton", "logic/SMAnalyticsManager", "switchback-common/util/ScaleUtil", "logic/RaceController", "component/ship/RaceShip"], function (require, exports, PromiseUtil_74, LocalizationManager_67, TextUtil_30, Game_72, GameController_44, StateNames_31, SpaceBaseState_12, CTAButton_10, SMAnalyticsManager_32, ScaleUtil_54, RaceController_9, RaceShip_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author steven@cerebralfix.com
     */
    class PostRacePlacementState extends SpaceBaseState_12.SpaceBaseState {
        init(...params) {
            this.stateName = StateNames_31.StateNames.POST_RACE_PLACEMENT;
            super.init(true, false);
        }
        preload() {
            super.preload();
            this.game.load.image('race-scoreboard', 'assets/race/race_scoreboard.png');
            this.game.load.image('results-highlight', 'assets/race/results_highlight.png');
            this.game.load.audio("race_won", ["assets/sound/race/race_won.ogg", "assets/sound/race/race_won.m4a"]);
            this.game.load.audio("race_not_won", ["assets/sound/race/race_not_won.ogg", "assets/sound/race/race_not_won.m4a"]);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const background = new Phaser.Image(this.game, 0, 0, "background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                const lightbeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
                ScaleUtil_54.scaleToBounds(lightbeam, this.game.width, this.game.height * 0.05);
                lightbeam.centerX = this.game.width * 0.5;
                this.add.existing(lightbeam);
                const titleText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.025, LocalizationManager_67.LocalizationManager.Instance.text("race.race_results"), {
                    font: Game_72.Game.DEFAULT_FONT,
                    fill: Game_72.Game.DEFAULT_FONT_COLOUR,
                    fontSize: 90,
                    stroke: Game_72.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_72.Game.FONT_STROKE_SMALL
                });
                TextUtil_30.TextUtil.scaleDownSingleLineTextToWidth(titleText, this.game.width * 0.75);
                titleText.anchor.set(0.5, 0);
                this.add.existing(titleText);
                lightbeam.centerY = titleText.centerY;
                this.continueButton = new CTAButton_10.CTAButton(this.game, LocalizationManager_67.LocalizationManager.Instance.text("buttons.continue"));
                this.continueButton.centerX = this.game.width * 0.5;
                this.continueButton.centerY = this.game.height * 0.875;
                this.add.existing(this.continueButton);
                const positionAboveContinue = this.continueButton.top - this.game.height * 0.03;
                this.scoreboardBack = new Phaser.Image(this.game, 0, 0, "race-scoreboard");
                ScaleUtil_54.scaleToBounds(this.scoreboardBack, this.game.width * 0.94, positionAboveContinue - (this.game.height * 0.12));
                this.scoreboardBack.centerX = this.game.width * 0.5;
                this.scoreboardBack.bottom = positionAboveContinue;
                this.add.existing(this.scoreboardBack);
                const positionLabel = new Phaser.Text(this.game, this.scoreboardBack.x + this.scoreboardBack.width * 0.185, this.scoreboardBack.y + this.scoreboardBack.height * 0.013, LocalizationManager_67.LocalizationManager.Instance.text("race.race_label_position"), Object.assign({}, Game_72.Game.DEFAULT_FONT_STYLE, { fontSize: Game_72.Game.FONT_SIZE_SMALL, align: 'center', fill: Game_72.Game.FONT_COLOUR_YELLOW }));
                TextUtil_30.TextUtil.scaleDownSingleLineTextToWidth(positionLabel, this.scoreboardBack.width * 0.34);
                positionLabel.anchor.set(0.5, 0);
                this.add.existing(positionLabel);
                const speedLabel = new Phaser.Text(this.game, this.scoreboardBack.x + this.scoreboardBack.width * 0.815, this.scoreboardBack.y + this.scoreboardBack.height * 0.013, LocalizationManager_67.LocalizationManager.Instance.text("race.race_label_speed"), Object.assign({}, Game_72.Game.DEFAULT_FONT_STYLE, { fontSize: Game_72.Game.FONT_SIZE_SMALL, align: 'center', fill: Game_72.Game.FONT_COLOUR_YELLOW }));
                TextUtil_30.TextUtil.scaleDownSingleLineTextToWidth(speedLabel, this.scoreboardBack.width * 0.34);
                speedLabel.anchor.set(0.5, 0);
                this.add.existing(speedLabel);
                GameController_44.GameController.Instance.state.incrementMinigamePlays(StateNames_31.StateNames.DEVICE_RACE);
                SMAnalyticsManager_32.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/placing', 'SpaceRacePlacing', {
                    [SMAnalyticsManager_32.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_32.SMAnalyticsManager.GAME_PLAYS]: GameController_44.GameController.Instance.state.getMinigamePlays(StateNames_31.StateNames.DEVICE_RACE),
                    [SMAnalyticsManager_32.SMAnalyticsManager.GAME_SCORE]: GameController_44.GameController.Instance.state.lastRacePlacement + 1
                });
                this.effectLayer = new Phaser.Group(this.game);
                this.add.existing(this.effectLayer);
                this.populateShips();
                this.playContent();
            });
        }
        populateShips() {
            const raceController = GameController_44.GameController.Instance.raceController;
            this.ships = [];
            const raceResults = raceController.getRaceResults();
            // select the four ships we'll display from the results
            // try and ensure that the player ship is the 2nd placed one if possible
            // so they feel they are doing well, but still have more to achieve
            const playerRaceResult = raceController.shipPlacement;
            let firstPlacement = Math.min(RaceController_9.RaceController.NUM_SHIPS_PER_RACE - PostRacePlacementState.PLACEMENTS_TO_SHOW, Math.max(0, playerRaceResult - 1));
            firstPlacement += 1; // make placement 1 based as race results are 1 based
            const shipList = raceController.getShipList();
            for (let i = 0; i < shipList.length; ++i) {
                if (raceResults[i].placement >= firstPlacement && raceResults[i].placement < firstPlacement + PostRacePlacementState.PLACEMENTS_TO_SHOW) {
                    this.createShip(i, raceResults[i], shipList[i]);
                }
            }
            // sort the selected ships into their placement order
            this.ships = this.ships.sort((a, b) => {
                return (a.finishingPlace - b.finishingPlace);
            });
        }
        /**
         * Note, this function has been copied from DeviceRaceState.
         * It should be generalized to reduce code duplication.
         *
         */
        createShip(shipIndex, raceResult, shipInfo) {
            const raceController = GameController_44.GameController.Instance.raceController;
            const currentRaceId = raceController.getIdOfScheduledRace();
            const playerShipId = raceController.getShipIdForRace(currentRaceId, GameController_44.GameController.Instance.getTotalShipParts());
            const colors = raceController.getColors();
            const bodyTint = parseInt("0x" + colors[shipInfo.bodyColor]);
            const trailTint = parseInt("0x" + colors[shipInfo.trailColor]);
            let numParts;
            let numBonusParts = 0;
            // if this is our ship, use the parts we have.
            if (shipIndex === playerShipId) {
                const shipState = GameController_44.GameController.Instance.getShipState();
                numParts = shipState.baseParts;
                numBonusParts = shipState.bonusParts;
            }
            else {
                // Otherwise use the number of parts required (for getting the place that this ship is going to get)
                numParts = raceResult.numShipParts;
            }
            let ship = new RaceShip_2.RaceShip(this.game, null, numParts, numBonusParts, bodyTint, trailTint, shipIndex === playerShipId, false);
            this.effectLayer.add(ship);
            ship.visible = false;
            ship.finishingPlace = raceResult.placement;
            this.ships.push(ship);
        }
        playContent() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_74.PromiseUtil.time(this.game, 350);
                yield this.openDoors();
                yield PromiseUtil_74.PromiseUtil.time(this.game, 250);
                yield this.showShips();
                yield PromiseUtil_74.PromiseUtil.time(this.game, 4000);
                yield this.continueButton.show();
                this.continueButton.onChildInputDown.addOnce(() => {
                    this.continueButton.hide();
                    this.transitionToHub();
                });
            });
        }
        showShips() {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < this.ships.length; ++i) {
                    const scoreboardOffset = this.scoreboardBack.height * ((1 / (PostRacePlacementState.PLACEMENTS_TO_SHOW + 1)) * (i + 1));
                    this.ships[i].centerX = this.game.width * 0.5;
                    this.ships[i].centerY = this.scoreboardBack.top + scoreboardOffset + this.ships[i].thrusterOffset - this.scoreboardBack.height * 0.04;
                    this.ships[i].visible = true;
                    this.game.add.tween(this.ships[i])
                        .from({ y: this.game.height + this.ships[i].height }, 1000, Phaser.Easing.Quartic.Out, true, 750 * i).onComplete.addOnce(this.onShipArrived, this);
                }
            });
        }
        onShipArrived(ship) {
            return __awaiter(this, void 0, void 0, function* () {
                let fontColourPlace = Game_72.Game.FONT_COLOUR_PURPLE;
                let fontColourSpeed = Game_72.Game.FONT_COLOUR_BLUE;
                if (ship.isPlayerShip) {
                    fontColourPlace = Game_72.Game.DEFAULT_FONT_COLOUR;
                    fontColourSpeed = Game_72.Game.DEFAULT_FONT_COLOUR;
                    const playerHighlight = new Phaser.Image(this.game, 0, 0, "results-highlight");
                    ScaleUtil_54.scaleToBounds(playerHighlight, this.scoreboardBack.width * 0.90, this.scoreboardBack.height * 0.2);
                    playerHighlight.anchor.set(0.5);
                    playerHighlight.centerX = this.game.width * 0.5;
                    playerHighlight.centerY = ship.centerY;
                    this.effectLayer.addChildAt(playerHighlight, 0); // under the ship
                    this.game.sound.play(ship.finishingPlace === 1 ? "race_won" : "race_not_won");
                }
                const lightbeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
                ScaleUtil_54.scaleToBounds(lightbeam, this.scoreboardBack.width * 0.98, this.game.height * 0.05);
                lightbeam.anchor.set(0.5);
                lightbeam.centerX = this.game.width * 0.5;
                lightbeam.centerY = ship.centerY;
                this.effectLayer.addChildAt(lightbeam, (ship.isPlayerShip ? 1 : 0)); // under the ship, over the highlight
                yield PromiseUtil_74.PromiseUtil.tween(this.game, this.game.add.tween(lightbeam.scale).from({ x: 0, y: 0 }, 300, Phaser.Easing.Linear.None, true));
                const positionText = new Phaser.Text(this.game, this.scoreboardBack.x + this.scoreboardBack.width * 0.2, 0, TextUtil_30.TextUtil.getOrdinalFormattedString(ship.finishingPlace), {
                    font: Game_72.Game.DEFAULT_FONT,
                    fill: fontColourPlace,
                    fontSize: Game_72.Game.FONT_SIZE_LARGE,
                    stroke: Game_72.Game.DEFAULT_FONT_STROKE,
                    align: 'center',
                    strokeThickness: Game_72.Game.FONT_STROKE_SMALL
                });
                TextUtil_30.TextUtil.scaleDownSingleLineTextToWidth(positionText, this.scoreboardBack.width * 0.3);
                positionText.anchor.set(0.5, 0);
                positionText.centerY = ship.centerY;
                this.add.existing(positionText);
                let shipSpeed = ship.totalParts + 1; // add 1 due to ships starting at ship speed of 1 without parts
                // the players ship includes pending parts
                if (ship.isPlayerShip) {
                    shipSpeed = GameController_44.GameController.Instance.getTotalShipParts() + 1;
                }
                const speedText = new Phaser.Text(this.game, this.scoreboardBack.x + this.scoreboardBack.width * 0.8, 0, shipSpeed.toString(), {
                    font: Game_72.Game.DEFAULT_FONT,
                    fill: fontColourSpeed,
                    fontSize: Game_72.Game.FONT_SIZE_MEDIUM,
                    stroke: Game_72.Game.DEFAULT_FONT_STROKE,
                    align: 'center',
                    strokeThickness: Game_72.Game.FONT_STROKE_SMALL
                });
                speedText.anchor.set(0.5, 0);
                speedText.centerY = ship.centerY;
                this.add.existing(speedText);
            });
        }
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.closeDoors();
                this.game.state.start(StateNames_31.StateNames.POST_RACE_REVIEW);
            });
        }
        shutdown() {
            this.continueButton = null;
            this.game.cache.removeSound("race_won");
            this.game.cache.removeSound("race_not_won");
            super.shutdown();
        }
    }
    PostRacePlacementState.PLACEMENTS_TO_SHOW = 4;
    exports.PostRacePlacementState = PostRacePlacementState;
});
define("state/PostRaceReviewState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/TextUtil", "Game", "logic/GameController", "StateNames", "state/SpaceBaseState", "component/button/CTAButton", "logic/mission/MissionType", "logic/SMAnalyticsManager", "switchback-common/util/ScaleUtil", "component/RewardPanel"], function (require, exports, PromiseUtil_75, LocalizationManager_68, TextUtil_31, Game_73, GameController_45, StateNames_32, SpaceBaseState_13, CTAButton_11, MissionType_3, SMAnalyticsManager_33, ScaleUtil_55, RewardPanel_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class PostRaceReviewState extends SpaceBaseState_13.SpaceBaseState {
        init(...params) {
            this.stateName = StateNames_32.StateNames.POST_RACE_REVIEW;
            super.init(true, false);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const background = new Phaser.Image(this.game, 0, 0, "background");
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                const yPadding = this.game.height * 0.05;
                const lightbeam = new Phaser.Image(this.game, 0, 0, "lightbeam");
                ScaleUtil_55.scaleToBounds(lightbeam, this.game.width, this.game.height * 0.05);
                lightbeam.centerX = this.game.width * 0.5;
                this.add.existing(lightbeam);
                const titleText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.025, LocalizationManager_68.LocalizationManager.Instance.text("race.race_complete"), {
                    font: Game_73.Game.DEFAULT_FONT,
                    fill: Game_73.Game.DEFAULT_FONT_COLOUR,
                    fontSize: 90,
                    stroke: Game_73.Game.DEFAULT_FONT_STROKE,
                    strokeThickness: Game_73.Game.FONT_STROKE_SMALL
                });
                TextUtil_31.TextUtil.scaleDownSingleLineTextToWidth(titleText, this.game.width * 0.75);
                titleText.anchor.set(0.5, 0);
                this.add.existing(titleText);
                lightbeam.centerY = titleText.centerY;
                this.placementTextLabel = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.15, LocalizationManager_68.LocalizationManager.Instance.text("race.you_placed"), Object.assign({}, Game_73.Game.DEFAULT_FONT_STYLE, { fontSize: Game_73.Game.FONT_SIZE_MEDIUM, fill: Game_73.Game.FONT_COLOUR_BLUE, align: 'center' }));
                this.placementTextLabel.anchor.set(0.5);
                this.add.existing(this.placementTextLabel);
                // N.B We +1 to this because you can't come in 0th place!
                const placementNumber = GameController_45.GameController.Instance.raceController.shipPlacement + 1;
                // calculate and grant new ship parts
                let shipPartsEarned;
                let descriptionTag;
                if (placementNumber === 1) {
                    shipPartsEarned = 2;
                    descriptionTag = "race.fastest_ship";
                }
                else {
                    shipPartsEarned = 1;
                    descriptionTag = "race.upgrade_your_ship";
                }
                GameController_45.GameController.Instance.addPendingShipParts(shipPartsEarned);
                this.placementTextNumber = new Phaser.Text(this.game, this.game.width * 0.5, this.placementTextLabel.bottom + yPadding, TextUtil_31.TextUtil.getOrdinalFormattedString(placementNumber), Object.assign({}, Game_73.Game.DEFAULT_FONT_STYLE, { fontSize: 200, align: 'center', fill: Game_73.Game.DEFAULT_FONT_COLOUR }));
                this.placementTextNumber.lineSpacing = -this.game.height * 0.02;
                this.placementTextNumber.anchor.set(0.5);
                this.add.existing(this.placementTextNumber);
                this.continueImprovingText = new Phaser.Text(this.game, this.game.width * 0.5, this.placementTextNumber.bottom, LocalizationManager_68.LocalizationManager.Instance.text(descriptionTag), Object.assign({}, Game_73.Game.DEFAULT_FONT_STYLE, { fontSize: Game_73.Game.FONT_SIZE_MEDIUM, fill: Game_73.Game.FONT_COLOUR_BLUE, align: 'center', wordWrap: true, wordWrapWidth: this.game.width * 0.9 }));
                this.continueImprovingText.anchor.set(0.5, 0);
                TextUtil_31.TextUtil.scaleDownTextToHeight(this.continueImprovingText, this.game.height * 0.4);
                this.add.existing(this.continueImprovingText);
                this.continueButton = new CTAButton_11.CTAButton(this.game, LocalizationManager_68.LocalizationManager.Instance.text("buttons.claim"));
                this.continueButton.centerX = this.game.width * 0.5;
                this.continueButton.centerY = this.game.height * 0.875;
                this.add.existing(this.continueButton);
                // calculate the number of parts earned
                this.rewardPanel = new RewardPanel_3.RewardPanel(this.game, shipPartsEarned);
                this.rewardPanel.centerX = this.game.width * 0.5;
                this.rewardPanel.bottom = this.continueButton.top - this.game.height * 0.02;
                this.add.existing(this.rewardPanel);
                GameController_45.GameController.Instance.achievementManager.onShipLaunch();
                GameController_45.GameController.Instance.state.lastRacePlacement = GameController_45.GameController.Instance.raceController.shipPlacement;
                // Post their mission progress (participation, and if they won)
                GameController_45.GameController.Instance.missionManager.addMissionPendingProgress(MissionType_3.MissionType.RACE_PARTICIPATION, 1);
                if (placementNumber === 1) {
                    GameController_45.GameController.Instance.achievementManager.onSpaceRaceWin();
                    GameController_45.GameController.Instance.missionManager.addMissionPendingProgress(MissionType_3.MissionType.RACE_WIN, 1);
                }
                GameController_45.GameController.Instance.state.incrementMinigamePlays(StateNames_32.StateNames.DEVICE_RACE);
                SMAnalyticsManager_33.SMAnalyticsManager.Instance.trackState('content/game/inqueue/minigame/result', 'SpaceRaceResult', {
                    [SMAnalyticsManager_33.SMAnalyticsManager.INSTALLATION_NAME]: "SpaceRace",
                    [SMAnalyticsManager_33.SMAnalyticsManager.GAME_PLAYS]: GameController_45.GameController.Instance.state.getMinigamePlays(StateNames_32.StateNames.DEVICE_RACE),
                    [SMAnalyticsManager_33.SMAnalyticsManager.GAME_SCORE]: GameController_45.GameController.Instance.state.lastRacePlacement + 1
                });
                this.playContent();
            });
        }
        playContent() {
            return __awaiter(this, void 0, void 0, function* () {
                this.placementTextLabel.visible = false;
                this.placementTextNumber.visible = false;
                this.continueImprovingText.visible = false;
                this.rewardPanel.visible = false;
                yield PromiseUtil_75.PromiseUtil.time(this.game, 350);
                yield this.openDoors();
                this.placementTextLabel.visible = true;
                yield PromiseUtil_75.PromiseUtil.tween(this.game, this.game.add.tween(this.placementTextNumber)
                    .from({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_75.PromiseUtil.time(this.game, 250);
                this.placementTextNumber.visible = true;
                yield PromiseUtil_75.PromiseUtil.tween(this.game, this.game.add.tween(this.placementTextNumber.scale)
                    .from({ x: 0, y: 0 }, 300, Phaser.Easing.Bounce.Out, true));
                yield PromiseUtil_75.PromiseUtil.time(this.game, 500);
                this.continueImprovingText.visible = true;
                yield PromiseUtil_75.PromiseUtil.tween(this.game, this.game.add.tween(this.continueImprovingText)
                    .from({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_75.PromiseUtil.time(this.game, 250);
                this.rewardPanel.visible = true;
                yield PromiseUtil_75.PromiseUtil.tween(this.game, this.game.add.tween(this.rewardPanel.scale)
                    .from({ x: 0, y: 0 }, 400, Phaser.Easing.Back.Out, true));
                yield PromiseUtil_75.PromiseUtil.time(this.game, 250);
                yield this.continueButton.show();
                this.continueButton.onChildInputDown.addOnce(() => {
                    this.continueButton.hide();
                    this.transitionToHub();
                });
            });
        }
        transitionToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                GameController_45.GameController.Instance.raceController.postRaceComplete();
                // SWB-958 Goto outro after the first race after merging
                if (GameController_45.GameController.Instance.state.shouldForceOutroAfterRace) {
                    this.game.state.start(StateNames_32.StateNames.OUTRO);
                }
                else {
                    this.game.state.start(StateNames_32.StateNames.HUB);
                }
            });
        }
        shutdown() {
            this.continueButton = null;
            this.placementTextLabel = null;
            this.placementTextNumber = null;
            this.continueImprovingText = null;
            this.rewardPanel = null;
            super.shutdown();
        }
    }
    exports.PostRaceReviewState = PostRaceReviewState;
});
define("Game", ["require", "exports", "state/LoadingState", "state/IntroState", "state/RedeemCodeState", "state/OutroState", "minigame/asteroids/AsteroidsGameState", "minigame/memory/MemoryGameState", "minigame/relay/RelayGameIntroState", "minigame/relay/RelayGameOutroState", "minigame/relay/scanner/ScannerGameState", "minigame/relay/switches/SwitchesGameState", "minigame/relay/locks/LocksGameState", "minigame/relay/wrench/WrenchGameState", "minigame/pipes/PipeState", "minigame/docking/DockingGameState", "minigame/energy-balls/EnergyBallState", "StateNames", "switchback-common/util/DeviceUtil", "switchback-common/state/BaseState", "state/HubState", "state/DeviceRaceState", "state/InstallationRaceInProgressState", "state/PreRaceState", "state/NarrativeBriefingState", "state/NarrativeDebriefingState", "state/PostRacePlacementState", "state/PostRaceReviewState"], function (require, exports, LoadingState_1, IntroState_1, RedeemCodeState_1, OutroState_1, AsteroidsGameState_2, MemoryGameState_3, RelayGameIntroState_1, RelayGameOutroState_1, ScannerGameState_2, SwitchesGameState_2, LocksGameState_2, WrenchGameState_2, PipeState_1, DockingGameState_2, EnergyBallState_4, StateNames_33, DeviceUtil_1, BaseState_3, HubState_1, DeviceRaceState_1, InstallationRaceInProgressState_1, PreRaceState_1, NarrativeBriefingState_1, NarrativeDebriefingState_1, PostRacePlacementState_1, PostRaceReviewState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Game extends Phaser.Game {
        constructor() {
            super('100', '100', Phaser.CANVAS);
            this.config.enableDebug = Game.ENABLE_DEBUG;
            this.clearBeforeRender = Game.ENABLE_BUFFER_CLEAR;
            console.log(`[Game] constructor()`);
            Game.FONT_STROKE_SMALL = this.width * 0.015;
            Game.FONT_STROKE_LARGE = this.width * 0.02;
            Game.DEFAULT_FONT_STYLE = {
                font: Game.DEFAULT_FONT,
                fill: Game.DEFAULT_FONT_COLOUR,
                stroke: Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game.FONT_STROKE_LARGE
            };
            Game.FONT_STYLE_TITLE = {
                font: Game.DEFAULT_FONT,
                fill: Game.FONT_COLOUR_BLUE,
                stroke: Game.DEFAULT_FONT_STROKE,
                strokeThickness: Game.FONT_STROKE_SMALL,
                fontSize: Game.FONT_SIZE_LARGE,
            };
            DeviceUtil_1.DeviceUtil.applyMaliGPUFix(this);
            // Set up a reference to our loadCommonAssets function, which will be used by all classes that extend BaseState
            BaseState_3.BaseState.loadCommonAssets = this.loadCommonAssets.bind(this);
            this.state.add(StateNames_33.StateNames.LOAD, LoadingState_1.LoadingState, false);
            this.state.add(StateNames_33.StateNames.INTRO, IntroState_1.IntroState, false);
            this.state.add(StateNames_33.StateNames.NARRATIVE_BRIEFING, NarrativeBriefingState_1.NarrativeBriefingState, false);
            this.state.add(StateNames_33.StateNames.NARRATIVE_DEBRIEFING, NarrativeDebriefingState_1.NarrativeDebriefingState, false);
            this.state.add(StateNames_33.StateNames.HUB, HubState_1.HubState, false);
            this.state.add(StateNames_33.StateNames.REDEEM_CODE, RedeemCodeState_1.RedeemCodeState, false);
            this.state.add(StateNames_33.StateNames.OUTRO, OutroState_1.OutroState, false);
            this.state.add(StateNames_33.StateNames.PRE_RACE, PreRaceState_1.PreRaceState, false);
            this.state.add(StateNames_33.StateNames.POST_RACE_PLACEMENT, PostRacePlacementState_1.PostRacePlacementState, false);
            this.state.add(StateNames_33.StateNames.POST_RACE_REVIEW, PostRaceReviewState_1.PostRaceReviewState, false);
            this.state.add(StateNames_33.StateNames.INSTALLATION_RACE_IN_PROGRESS, InstallationRaceInProgressState_1.InstallationRaceInProgressState, false);
            this.state.add(StateNames_33.StateNames.DEVICE_RACE, DeviceRaceState_1.DeviceRaceState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_INTRO, RelayGameIntroState_1.RelayGameIntroState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_OUTRO, RelayGameOutroState_1.RelayGameOutroState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_SWITCHES, SwitchesGameState_2.SwitchesGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_SCANNER, ScannerGameState_2.ScannerGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_LOCKS, LocksGameState_2.LocksGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_RELAY_WRENCH, WrenchGameState_2.WrenchGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_ENERGY_BALLS, EnergyBallState_4.EnergyBallState, false);
            this.state.add(StateNames_33.StateNames.GAME_ASTEROIDS, AsteroidsGameState_2.AsteroidsGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_MEMORY, MemoryGameState_3.MemoryGameState, false);
            this.state.add(StateNames_33.StateNames.GAME_PIPES, PipeState_1.PipeState, false);
            this.state.add(StateNames_33.StateNames.GAME_DOCKING, DockingGameState_2.DockingGameState, false);
            this.state.start(StateNames_33.StateNames.LOAD);
        }
        /*
        * This function is called in the preload of every state that extends BaseState. It will load
        * all our common assets into memory (unless they've already been loaded)
        */
        loadCommonAssets() {
            // Images
            this.load.image("background", "assets/common/background.jpg");
            this.load.image("back_button_small", "assets/button/back_button_small.png");
            this.load.image("dialog-top", "assets/common/dl_dialog_top.png", false);
            this.load.image("dialog-bot", "assets/common/dl_dialog_bot.png", false);
            this.load.image("dialog-mid", "assets/common/dl_dialog_mid.png", false);
            this.load.image("next-arrow", "assets/common/next_arrow.png", false);
            this.load.image("next-circle", "assets/common/next_circle.png", false);
            this.load.image("next-line", "assets/common/next_line.png", false);
            this.load.image("start-button-background", "assets/button/start_button.png", false);
            this.load.image("lightbeam", "assets/common/lightbeam.png", false);
        }
    }
    Game.DEFAULT_FONT = "orbitron-bold";
    Game.DEFAULT_FONT_COLOUR = "white";
    Game.DEFAULT_FONT_STROKE = "black";
    Game.FONT_COLOUR_RED = "#f11217";
    Game.FONT_COLOUR_BLUE = "#52e8e5";
    Game.FONT_COLOUR_YELLOW = "#FFE327";
    Game.FONT_COLOUR_GREEN = "#65E288";
    Game.FONT_COLOUR_PURPLE = "#84a6d6";
    Game.FONT_SIZE_EXTRA_LARGE = 100;
    Game.FONT_SIZE_LARGE = 80;
    Game.FONT_SIZE_MEDIUM = 60;
    Game.FONT_SIZE_SMALL = 40;
    Game.DEFAULT_STROKE_THICKNESS = 10;
    Game.PLAYER_TINTS = [0xf4ef0d, 0x27c7f9, 0xf11217, 0x76da00, 0xcb5dfe];
    Game.ENABLE_DEBUG = false;
    Game.ENABLE_BUFFER_CLEAR = true;
    exports.Game = Game;
});
/// <reference path="../node_modules/phaser/typescript/phaser.comments.d.ts" />
/// <reference path="./switchback-common/localization/typing/polyglot.d.ts" />
define("index", ["require", "exports", "Game"], function (require, exports, Game_74) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    new Game_74.Game();
});
define("component/coderedeem/CodeRedeemBackButton", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game"], function (require, exports, LocalizationManager_69, Game_75) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CodeRedeemBackButton extends Phaser.Group {
        constructor(game) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.clickableArea = new Phaser.Image(game, 0, 0, '');
            this.clickableArea.inputEnabled = true;
            this.clickableArea.events.onInputDown.add(() => {
                this.onButtonClickedSignal.dispatch();
            }, this);
            const button = new Phaser.Image(game, 0, 0, "next-circle");
            button.height = this.game.height * 0.1;
            button.width = button.height;
            const arrowImage = new Phaser.Image(game, 0, 0, "next-arrow");
            arrowImage.anchor.set(0.5);
            arrowImage.angle = 180;
            arrowImage.width = button.width * 0.65;
            arrowImage.height = button.height * 0.65;
            arrowImage.centerX = button.centerX;
            arrowImage.centerY = button.centerY;
            const lineImage = new Phaser.Image(game, 0, 0, "next-line");
            lineImage.anchor.set(0.5);
            lineImage.angle = 180;
            lineImage.width = game.width * 0.5;
            lineImage.height = button.height * 0.2;
            lineImage.left = button.centerX;
            lineImage.centerY = button.centerY;
            const backText = new Phaser.Text(game, 0, 0, LocalizationManager_69.LocalizationManager.Instance.text("code_redeem.back"), {
                fill: Game_75.Game.DEFAULT_FONT_COLOUR,
                font: Game_75.Game.DEFAULT_FONT,
                fontSize: Game_75.Game.FONT_SIZE_LARGE,
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.9
            });
            backText.setScaleMinMax(1);
            backText.left = button.right + this.game.width * 0.02;
            backText.centerY = button.centerY;
            this.add(lineImage);
            this.add(button);
            this.add(arrowImage);
            this.add(backText);
            this.clickableArea.width = button.width + backText.width;
            this.clickableArea.height = button.height;
            this.add(this.clickableArea);
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.CodeRedeemBackButton = CodeRedeemBackButton;
});
define("logic/comms/IClient", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/RadialCountdownBar", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/component/RadialProgressBar"], function (require, exports, PromiseUtil_76, RadialProgressBar_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialCountdownBar extends RadialProgressBar_2.RadialProgressBar {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100) {
            super(game, x, y, color, lineThicknessPx, radiusPx);
            // Start full
            this.angleMax = 360;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_76.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: 0 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
    }
    exports.RadialCountdownBar = RadialCountdownBar;
});
define("switchback-common/component/button/Button", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * This class adds an optional text label to the standard Phaser.Button.
     * The Text and TextStyle can be set on construction, or later.
     */
    class Button extends Phaser.Button {
        constructor(game, x, y, key, callback, callbackContext, text, style) {
            super(game, x, y, key, callback, callbackContext, "over", "out", "down", "out");
            this.textStyle = {};
            this.textStyle = style || {};
            if (text !== null && text !== "") {
                this.text = text;
            }
        }
        get text() {
            return this.textField.text;
        }
        set text(text) {
            if (!this.textField) {
                this.textField = new Phaser.Text(this.game, (this.width * 0.5), (this.height * 0.5), text, this.textStyle);
                this.textField.anchor.set(0.5);
                this.addChild(this.textField);
            }
            else {
                this.textField.text = text;
            }
        }
        get style() {
            return this.textStyle;
        }
        set style(style) {
            this.textStyle = style;
            if (this.textField) {
                this.textField.setStyle(this.textStyle, true);
            }
        }
        // This ensures that the text remains centered within the button when the button's anchor changes
        setAnchor(x, y) {
            // This allows developers to only pass in one argument if they want them both to be the same (as Phaser does)
            y = y || x;
            // Set the button's anchor as normal
            this.anchor.set(x, y);
            // Reposition the textField to remain centered in the button
            if (this.textField) {
                this.textField.position.set((this.width * 0.5) - this.offsetX, (this.height * 0.5) - this.offsetY);
            }
        }
    }
    exports.Button = Button;
});
define("switchback-common/component/button/DisablingButton", ["require", "exports", "switchback-common/component/button/Button"], function (require, exports, Button_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A disableable Phaser.Button with default button frame names
     */
    class DisablingButton extends Button_1.Button {
        constructor(game, x, y, key, callback, callbackContext, overFrame, outFrame, downFrame, upFrame, disabledFrame, disabled) {
            super(game, x, y, key, callback, callbackContext);
            this.dBool = false;
            this.originalOverFrame = overFrame || "over";
            this.originalOutFrame = outFrame || "out";
            this.originalDownFrame = downFrame || "down";
            this.originalUpFrame = upFrame || "out";
            this.disabledFrame = disabledFrame || "disabled";
            this.dBool = disabled;
        }
        get disabled() {
            return this.dBool;
        }
        set disabled(val) {
            this.dBool = val;
            this.updateState();
        }
        changeFrames(overFrame, outFrame, downFrame, upFrame, disabledFrame) {
            this.originalOverFrame = overFrame;
            this.originalOutFrame = outFrame;
            this.originalDownFrame = downFrame;
            this.originalUpFrame = upFrame;
            this.disabledFrame = disabledFrame;
            this.updateState();
        }
        updateState() {
            // Hack to work around issue where phaser does not correctly update the input handlers in constructors
            this.game.time.events.add(1, () => {
                this.inputEnabled = !this.dBool;
            });
            if (this.dBool) {
                super.setFrames(this.disabledFrame, this.disabledFrame, this.disabledFrame, this.disabledFrame);
            }
            else {
                super.setFrames(this.originalOverFrame, this.originalOutFrame, this.originalDownFrame, this.originalUpFrame);
            }
        }
    }
    exports.DisablingButton = DisablingButton;
});
define("switchback-common/component/button/GenericButton", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil"], function (require, exports, PromiseUtil_77, TextUtil_32) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GenericButton extends Phaser.Group {
        constructor(game, config) {
            super(game);
            this.onClickedSignal = new Phaser.Signal();
            this.enabled = true;
            this._cooldownMS = 0;
            this._buttonEffectInProgress = false;
            if (!config.backgroundKey) {
                console.error("[Generic Button] 'backgroundKey' will soon be a required parameter.");
            }
            this._shutdownSignal = config.shutdownSignal;
            this._cooldownMS = config.waitTime ? config.waitTime : 0;
            this._upTint = config.upTint ? config.upTint : 0xffffff;
            this._downTint = config.downTint ? config.downTint : 0xcccccc;
            this.pivot.set(0.5, 0.5);
            this.image = new Phaser.Image(this.game, 0, 0, config.backgroundKey || "common-button");
            this.image.inputEnabled = true;
            this.add(this.image);
            const buttonText = new Phaser.Text(this.game, 0, 0, config.text, config.fontStyle || {
                fill: 0xff0000,
                fontSize: 72,
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.8,
            });
            buttonText.anchor.set(0.5);
            buttonText.scale.y = buttonText.scale.x;
            buttonText.position.set(this.image.width / 2, this.image.height / 2);
            buttonText.centerY -= this.height * 0.033;
            this.text = buttonText;
            this.add(buttonText);
            if (config.onClickSoundKey) {
                this._clickSound = this.game.add.sound(config.onClickSoundKey, 1, false);
            }
            this.image.events.onInputUp.add(this.onClick, this);
            this.recenterText();
        }
        static preload(loader, url) {
            console.warn("[Generic Button] static preload is now deprecated and will be removed in future");
            loader.image("common-button", url);
        }
        static unload(cache) {
            console.warn("[Generic Button] static unload is now deprecated and will be removed in future");
            cache.removeImage("common-button");
        }
        destroy() {
            this.image.events.onInputUp.removeAll();
            this.onClickedSignal.dispose();
            super.destroy();
            this._shutdownSignal = null;
        }
        delayedFadeIn(delay) {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 0;
                this.enabled = false; // makes sure that once the button is visible it still cant be clicked
                this.visible = true;
                yield PromiseUtil_77.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 1 }, 250, Phaser.Easing.Cubic.InOut, true, delay), this._shutdownSignal).catch(() => undefined);
                this.enabled = true;
            });
        }
        recenterText() {
            this.text.centerX = this.image.centerX;
            this.text.centerY = this.image.centerY - this.image.height * 0.033;
            TextUtil_32.TextUtil.scaleDownTextToHeight(this.text, this.image.height * 0.9);
        }
        fadeOut() {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 1;
                let isShutdown = false;
                yield PromiseUtil_77.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 0 }, 200, Phaser.Easing.Cubic.InOut, true), this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this.visible = false;
                this.enabled = false;
            });
        }
        onClick() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this._buttonEffectInProgress || !this.enabled || this.onClickedSignal.getNumListeners() <= 0) {
                    return;
                }
                if (this._clickSound) {
                    this._clickSound.play();
                }
                if (this.text) {
                    this.text.tint = this._downTint;
                }
                this.image.tint = this._downTint;
                this._buttonEffectInProgress = true;
                // made new function in case need for tweens
                this.onClickedSignal.dispatch();
                let isShutdown = false;
                yield PromiseUtil_77.PromiseUtil.time(this.game, this._cooldownMS, this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this._buttonEffectInProgress = false;
                this.image.tint = this._upTint;
                if (this.text) {
                    this.text.tint = this._upTint;
                }
            });
        }
    }
    exports.GenericButton = GenericButton;
});
define("switchback-common/component/button/ToggleButton", ["require", "exports", "switchback-common/component/button/DisablingButton"], function (require, exports, DisablingButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A Phaser.Button with 2 toggle states. Can be disabled.
     */
    class ToggleButton extends DisablingButton_1.DisablingButton {
        constructor(game, x, y, key, callback, callbackContext, upFrame, downFrame, overFrame, disabledFrame, toggledUpFrame, toggledDownFrame, toggledOverFrame, toggledDisabledFrame) {
            super(game, x, y, key, callback, callbackContext, overFrame, upFrame, downFrame, upFrame);
            this.overFrame = overFrame || "over";
            this.upFrame = upFrame || "out";
            this.downFrame = downFrame || "down";
            this.normalDisabledFrame = disabledFrame || "disabled";
            this.toggledUpFrame = toggledUpFrame || "toggleout";
            this.toggledDownFrame = toggledDownFrame || "toggledown";
            this.toggledOverFrame = toggledOverFrame || "toggleover";
            this.toggledDisabledFrame = toggledDisabledFrame || "toggledisabled";
            this.t = false;
            this.onInputDown.add(() => {
                this.toggled = !this.t;
            });
        }
        get toggled() {
            return this.t;
        }
        set toggled(value) {
            this.t = value;
            if (this.t) {
                this.changeFrames(this.toggledOverFrame, this.toggledUpFrame, this.toggledDownFrame, this.toggledUpFrame, this.toggledDisabledFrame);
            }
            else {
                this.changeFrames(this.overFrame, this.upFrame, this.downFrame, this.upFrame, this.normalDisabledFrame);
            }
        }
        set disabled(val) {
            this.toggled = this.t;
            super.disabled = val;
        }
    }
    exports.ToggleButton = ToggleButton;
});
define("switchback-common/play/PlayCameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_78) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCameraUtil {
        /**
         * The transparent class references the project style.css file, it MUST be set in there
         * for the default value to work. If not, the camera stream can be blacked out.
         * .transparent {
         *     background: rgba(0, 0, 0, 0);
         * }
         * This allows the bg and transparency to be specified in css and not in the code base.
         * The bg color can also be changed throughout the play session and closing of the camera
         * doesn't need to care about that information.
         */
        static startCamera(cameraType, transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PlayAPI.startCamera(cameraType).then((apiResult) => {
                        document.body.classList.add(transparentClass);
                        resolve(true);
                        console.log("[PlayCameraUtil] StartCamera accepted: " + JSON.stringify(apiResult));
                    }, (apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera rejected: " + JSON.stringify(apiResult));
                    }).catch((apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera exception: " + JSON.stringify(apiResult));
                    });
                }));
            });
        }
        static captureImage(game, timeout = 5000, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[PlayCameraUtil] captureImage()");
                yield Promise.race([
                    PlayAPI.captureImage(),
                    PromiseUtil_78.PromiseUtil.time(game, timeout),
                ]).then((result) => {
                    if (!result) {
                        return;
                    }
                    if (result.imageData) {
                        return PlayCameraUtil.loadCaptureImage(game, result.imageData, imageName);
                    }
                    else {
                        console.log("[PlayCameraUtil.captureCamera] Unable to get imageData from result");
                    }
                });
            });
        }
        static loadCaptureImage(game, imageData, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    const loader = new Phaser.Loader(game);
                    loader.image(imageName, imageData);
                    loader.onLoadComplete.addOnce(resolve, this);
                    loader.start();
                });
            });
        }
        static closeCamera(transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                document.body.classList.remove(transparentClass);
                return PlayAPI.endCamera();
            });
        }
        static checkCameraAccess(promptForCameraPermissionCallback) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[PlayCameraUtil] cameraAccessStep()`);
                let cameraStatus = CameraAccessStatus.NOT_DETERMINED;
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 1: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    return CameraAccessStatus.DENIED;
                });
                if (promptForCameraPermissionCallback) {
                    yield promptForCameraPermissionCallback(cameraStatus);
                }
                // Recheck camera permissions to see if they've accepted them now
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 2: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    cameraStatus = CameraAccessStatus.DENIED;
                });
                return cameraStatus;
            });
        }
    }
    exports.PlayCameraUtil = PlayCameraUtil;
});
define("switchback-common/component/camera/PlayCamera", ["require", "exports", "switchback-common/play/PlayCameraUtil", "switchback-common/util/DeviceUtil", "switchback-common/util/PromiseUtil"], function (require, exports, PlayCameraUtil_1, DeviceUtil_2, PromiseUtil_79) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCamera extends Phaser.Group {
        constructor(game, overlayAsset, parent = null) {
            super(game, parent, null, false);
            this._capturedImage = new Phaser.Image(this.game, 0, 0, "");
            this._capturedImage.visible = false;
            this.add(this._capturedImage);
            if (overlayAsset) {
                this._overlay = new Phaser.Image(this.game, 0, 0, overlayAsset);
                this._overlay.width = this.game.width;
                this._overlay.height = this.game.height;
                this.add(this._overlay);
            }
        }
        openCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!PlayCamera.cameraPermissionsChecked) {
                    yield PlayCameraUtil_1.PlayCameraUtil.checkCameraAccess(this.checkCameraCallBack.bind(this));
                    PlayCamera.cameraPermissionsChecked = true;
                }
                // We can't trust the CameraAccessStatus, so we'll attempt to start the camera regardless
                const isSuccess = PlayCameraUtil_1.PlayCameraUtil.startCamera(CameraType.BACK_CAMERA);
                if (isSuccess) {
                    // This is necessary so the ui actually process the changes
                    // otherwise the player can tap to take the image too quickly causing a black photo
                    yield PromiseUtil_79.PromiseUtil.time(this.game, 100);
                }
                return isSuccess;
            });
        }
        /**
         * PlayAPI.endCamera() was not returning on Android devices as at 03/09/2018
         * so could not be awaited on as was the original scanner flow. Safe bet is to
         * ignore the result and handle any situation where the camera may still be open.
         */
        closeCamera(hideCapturedImage = true) {
            this._capturedImage.visible = !hideCapturedImage;
            PlayCameraUtil_1.PlayCameraUtil.closeCamera();
            PlayAPI.endCamera();
        }
        capture(timeout = 5000) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PlayCameraUtil_1.PlayCameraUtil.captureImage(this.game, timeout, PlayCamera.CAPTURE_TEXTURE);
                this.loadTexture();
                // Give some time for the frame to render before performing any other actions.
                // Calling endCamera too soon for example can cause a black flash between camera feed and photo
                yield PromiseUtil_79.PromiseUtil.time(this.game, 100);
            });
        }
        hideCaptureImage() {
            this._capturedImage.visible = false;
        }
        destroy() {
            this.game.cache.removeImage(PlayCamera.CAPTURE_TEXTURE);
            super.destroy();
        }
        onShowCameraPermissionsWarning() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        onShowCameraEnablePermissionsDialog(isIos) {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        loadTexture() {
            if (this.game.cache.checkImageKey(PlayCamera.CAPTURE_TEXTURE)) {
                this._capturedImage.loadTexture(PlayCamera.CAPTURE_TEXTURE);
                // Adjust it because the data comes back rotated...
                this._capturedImage.height = this.game.width;
                this._capturedImage.scale.x = this._capturedImage.scale.y;
                this._capturedImage.angle = 90;
                this._capturedImage.anchor.set(0.5);
                this._capturedImage.position.set(0, 0);
                this._capturedImage.centerX = this.game.width * 0.5;
                this._capturedImage.centerY = this.game.height * 0.5;
                this._capturedImage.visible = true;
            }
        }
        checkCameraCallBack(status) {
            return __awaiter(this, void 0, void 0, function* () {
                // Show the appropriate dialog
                if (status === CameraAccessStatus.AUTHORIZED) {
                    // We don't need to do anything. They've accepted the permissions already.
                    return;
                }
                else if (status === CameraAccessStatus.NOT_DETERMINED) {
                    // This device hasn't been asked for camera perms before.
                    // We should show a dialog to let them know they should accept them.
                    yield this.onShowCameraPermissionsWarning();
                }
                else if (status === CameraAccessStatus.DENIED) {
                    // Fall back to assuming they've declined.
                    // This dialog has the most info about enabling permissions.
                    yield this.onShowCameraEnablePermissionsDialog(DeviceUtil_2.DeviceUtil.isIos());
                }
                else if (status === CameraAccessStatus.RESTRICTED) {
                    // If the camera is Restricted, the user can't change it. We must handle it as if there were no camera.
                    // We won't show anything, and B-Show needs to be shown
                }
            });
        }
    }
    PlayCamera.CAPTURE_TEXTURE = "camera-image-capture";
    PlayCamera.cameraPermissionsChecked = false;
    exports.PlayCamera = PlayCamera;
});
define("switchback-common/component/keyboard/SpecialChars", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var SpecialChar;
    (function (SpecialChar) {
        SpecialChar["DELETE"] = "";
        SpecialChar["SUBMIT"] = "OK";
        SpecialChar["CANCEL"] = "CANCEL";
    })(SpecialChar = exports.SpecialChar || (exports.SpecialChar = {}));
});
define("switchback-common/component/keyboard/KeyCode", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/keyboard/Key", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, TextUtil_33, SpecialChars_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Key extends Phaser.Group {
        constructor(game, keyCode, keyTexture, width, height, fontStyle) {
            super(game);
            this.pressedSignal = new Phaser.Signal();
            const keyBg = new Phaser.Button(this.game, 0, 0, keyTexture, () => { this.pressedSignal.dispatch(keyCode); }, this, 0, 0, 1, 0);
            keyBg.width = width * Key.keyWidthMultiplier(keyCode);
            keyBg.height = height;
            this.add(keyBg);
            this.keyChar = new Phaser.Text(this.game, keyBg.centerX, keyBg.centerY, keyCode, fontStyle);
            this.keyChar.anchor.set(0.5, 0.45);
            TextUtil_33.TextUtil.scaleDownSingleLineTextToWidth(this.keyChar, keyBg.width * 0.95);
            this.add(this.keyChar);
        }
        static keyWidthMultiplier(char) {
            switch (char) {
                case SpecialChars_1.SpecialChar.DELETE:
                    return 1.56;
                case SpecialChars_1.SpecialChar.CANCEL:
                    return 1.83;
                case SpecialChars_1.SpecialChar.SUBMIT:
                    return 3.125;
                default:
                    return 1;
            }
        }
        get keyFontSize() {
            return this.keyChar.fontSize;
        }
        set keyFontSize(size) {
            this.keyChar.fontSize = size;
        }
    }
    exports.Key = Key;
});
define("switchback-common/component/keyboard/KeyLayoutData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayoutData {
    }
    exports.KeyLayoutData = KeyLayoutData;
});
define("switchback-common/component/keyboard/KeyLayouts", ["require", "exports", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, SpecialChars_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayouts {
    }
    KeyLayouts.LETTERS = [
        ["Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", SpecialChars_2.SpecialChar.DELETE],
        ["A", "S", "D", "F", "G", "H", "J", "K", "L", SpecialChars_2.SpecialChar.CANCEL],
        ["Z", "X", "C", "V", "B", "N", "M", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    KeyLayouts.NUMBERS = [
        ["7", "8", "9"],
        ["4", "5", "6"],
        ["1", "2", "3"],
        [SpecialChars_2.SpecialChar.DELETE, "0", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    exports.KeyLayouts = KeyLayouts;
});
define("switchback-common/component/keyboard/KeyboardOptions", ["require", "exports", "switchback-common/component/keyboard/KeyLayouts"], function (require, exports, KeyLayouts_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardOptions {
        constructor(params) {
            params = params || {};
            this.bgTexture = params.bgTexture || "kb-bg";
            this.cancelTexture = params.cancelTexture || "kb-cancel";
            this.charLimit = params.charLimit || 13;
            this.deleteTexture = params.deleteTexture || "kb-delete";
            this.keys = params.keys || KeyLayouts_1.KeyLayouts.LETTERS;
            this.keysTexture = params.keysTexture || "kb-regular";
            this.labelText = params.labelText || "";
            this.output = (params.output === undefined) ? true : params.output;
            this.outputBgTexture = params.outputBgTexture || "kb-output";
            this.submitTexture = params.submitTexture || "kb-submit";
            this.xPaddingProportion = (params.xPaddingProportion === undefined) ? 0.15 : params.xPaddingProportion;
            this.keysStyle = params.keyStyle || { fill: "#FFF", fontSize: 44 };
            this.labelStyle = params.labelStyle || { fill: "#FFF", fontSize: 44 };
            this.outputStyle = params.outputStyle || { fill: "#FFF", fontSize: 44 };
        }
    }
    exports.KeyboardOptions = KeyboardOptions;
});
define("switchback-common/component/keyboard/KeyboardController", ["require", "exports", "switchback-common/component/Signal", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, Signal_2, SpecialChars_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardController {
        constructor(options) {
            this._currentString = "";
            this.stringChangedSignal = new Signal_2.Signal();
            this.submitSignal = new Signal_2.Signal();
            this.cancelSignal = new Signal_2.Signal();
            this._options = options;
        }
        get options() {
            return this._options;
        }
        getTextureForKey(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    return this.options.submitTexture;
                }
                case SpecialChars_3.SpecialChar.DELETE: {
                    return this.options.deleteTexture;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    return this.options.cancelTexture;
                }
                default: {
                    return this.options.keysTexture;
                }
            }
        }
        keyPressed(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.DELETE: {
                    if (this._currentString.length > 0) {
                        this._currentString = this._currentString.substr(0, this._currentString.length - 1);
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                    break;
                }
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    this.submitSignal.dispatch(this._currentString);
                    break;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    this.cancelSignal.dispatch();
                    break;
                }
                default: {
                    if (this._currentString.length < this._options.charLimit) {
                        this._currentString += keyCode;
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                }
            }
        }
        get currentString() {
            return this._currentString;
        }
        clearString() {
            this._currentString = "";
        }
        destroy() {
            this.stringChangedSignal.dispose();
            this.submitSignal.dispose();
            this.cancelSignal.dispose();
        }
    }
    exports.KeyboardController = KeyboardController;
});
define("switchback-common/component/keyboard/Keyboard", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/Key", "switchback-common/util/ScaleUtil"], function (require, exports, KeyboardController_1, Key_1, ScaleUtil_56) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Keyboard extends Phaser.Group {
        get onSubmit() { return this.controller.submitSignal; }
        get onCancel() { return this.controller.cancelSignal; }
        get onChange() { return this.controller.stringChangedSignal; }
        constructor(game, bounds, options) {
            super(game);
            this.keysStyle = options.keysStyle;
            this.bounds = bounds;
            this.controller = new KeyboardController_1.KeyboardController(options);
            const label = new Phaser.Text(this.game, this.bounds.centerX, this.bounds.height * -0.3, options.labelText, options.labelStyle);
            label.anchor.set(0.5);
            this.add(label);
            this.keys = new Phaser.Group(this.game);
            this.add(this.keys);
            let keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.1, this.bounds.width, this.bounds.height);
            if (options.output) {
                const outputBG = new Phaser.Image(game, 0, 0, options.outputBgTexture);
                ScaleUtil_56.scaleToSingleBound(outputBG, bounds.width * 0.67);
                outputBG.centerX = this.bounds.width * 0.5;
                outputBG.centerY = this.bounds.height * -0.075;
                this.add(outputBG);
                this.outputString = new Phaser.Text(this.game, outputBG.centerX, outputBG.top + outputBG.height * 0.55, "", options.outputStyle);
                this.outputString.anchor.set(0.5);
                this.add(this.outputString);
                this.controller.stringChangedSignal.add(this.onStringChanged, this);
                keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.2, this.bounds.width, this.bounds.height * 0.75);
            }
            this.showKeys(this.controller.options.keys, keyBounds);
            this.bg = new Phaser.Image(game, 0, 0, options.bgTexture);
            this.bg.width = bounds.width;
            this.bg.height = bounds.height * 1.2;
            if (options.output) {
                this.bg.height = bounds.height * 1.5;
            }
            this.addAt(this.bg, 0);
        }
        destroy() {
            this.controller.destroy();
            super.destroy();
        }
        get currentString() {
            return this.controller.currentString;
        }
        clearString() {
            this.controller.clearString();
            this.outputString.text = this.controller.currentString;
        }
        showKeys(keys, keyBounds) {
            const layout = this.calculateKeyLayout(keys, keyBounds);
            let xPos;
            let yPos = keyBounds.y;
            keys.forEach((row) => {
                let rowWidthMulti = 0;
                for (let i = 0; i < row.length; i++) {
                    rowWidthMulti += Key_1.Key.keyWidthMultiplier(row[i]);
                }
                const totalKeyWidth = rowWidthMulti * layout.width;
                const totalSpaceWidth = (row.length - 1) * layout.xSpacing;
                const xOffset = (keyBounds.width - totalKeyWidth - totalSpaceWidth) * 0.5;
                xPos = xOffset + keyBounds.x + (layout.xSpacing * 0.5);
                row.forEach((key) => {
                    const keySprite = new Key_1.Key(this.game, key, this.controller.getTextureForKey(key), layout.width, layout.height, this.keysStyle);
                    keySprite.position.set(xPos, yPos);
                    this.keys.add(keySprite);
                    keySprite.pressedSignal.add((k) => {
                        this.controller.keyPressed(k);
                    });
                    xPos += layout.width + layout.xSpacing;
                }, this);
                yPos += layout.height + layout.ySpacing;
            }, this);
            /**
             * Now that we've added all the keys, we need to scale their
             * text down uniformly so that they fit on the key.
             */
            let minSize = Infinity;
            this.keys.children.forEach((child) => {
                if (child.keyFontSize) {
                    minSize = Math.min(child.keyFontSize, minSize);
                }
            });
            this.keys.children.forEach((child) => {
                child.keyFontSize = minSize;
            });
        }
        calculateKeyLayout(keys, keyBounds) {
            let longestRow = 0;
            keys.forEach((row) => {
                longestRow = Math.max(row.length, longestRow);
            });
            longestRow++; // Give us some extra room for special characters
            const width = (keyBounds.width * (1 - this.controller.options.xPaddingProportion)) / longestRow;
            const xSpacing = (keyBounds.width * this.controller.options.xPaddingProportion) / longestRow;
            const height = (keyBounds.height * 0.9) / keys.length;
            const ySpacing = (keyBounds.height * 0.15) / keys.length;
            return {
                width: width,
                height: height,
                xSpacing: xSpacing,
                ySpacing: ySpacing,
            };
        }
        onStringChanged(newString) {
            this.outputString.text = newString;
        }
    }
    exports.Keyboard = Keyboard;
});
define("switchback-common/component/scrolling/IScrollableGroupSettings", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/scrolling/ScrollableGroup", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ScrollableGroup extends Phaser.Group {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, parent, null, false);
            this.scrollerTapSignal = new Phaser.Signal();
            this._settings = {
                horizontalScroll: false,
                paddingBottom: 0,
                paddingRight: 0,
                timeConstantScroll: 325,
                verticalScroll: true,
            };
            this._hasAddedListeners = false;
            this.settings = settings;
            this._previousPosition = new Phaser.Point();
            this._velocity = new Phaser.Point();
            this._autoScrollTarget = new Phaser.Point();
            this._autoScrollAmplitude = new Phaser.Point();
            this._contents = new Phaser.Group(this.game, null, null, false);
            this._displayMask = new Phaser.Graphics(this.game);
            this._displayMask.beginFill(0x000000);
            this._displayMask.drawRect(0, 0, viewableWidth, viewableHeight);
            this._displayMask.alpha = 0;
            this._displayMask.endFill();
            this._inputId = -1;
            this.add(this._displayMask);
            this.add(this._contents);
            // Hijack the group's add function to ensure all game objects are
            // added to the content layer that gets scrolled
            this.add = this.addToContentLayer.bind(this);
        }
        static addTapInputSignal(game, obj, clickSignal) {
            const tapDistance = game.width * 0.05;
            let startClickPosition = null;
            let endClickPosition = null;
            obj.inputEnabled = true;
            obj.events.onInputDown.add(() => {
                startClickPosition = new Phaser.Point(game.input.x, game.input.y);
            }, this);
            obj.events.onInputUp.add(() => {
                endClickPosition = new Phaser.Point(game.input.x, game.input.y);
                const distance = Phaser.Point.distance(startClickPosition, endClickPosition, true);
                if (distance < tapDistance) {
                    clickSignal.dispatch();
                }
            }, this);
        }
        set settings(settings) {
            for (const property in settings) {
                if (this._settings.hasOwnProperty(property)) {
                    this._settings[property] = settings[property];
                }
            }
        }
        onContentsUpdated() {
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
        }
        start() {
            ScrollableGroup.addTapInputSignal(this.game, this._displayMask, this.scrollerTapSignal);
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
            this._contents.position.set(0, 0);
        }
        updateListeners() {
            const isHorizontalScrollable = this._settings.horizontalScroll &&
                this._contentLocalBounds.x + this._contentLocalBounds.width > this._displayMask.width;
            const isVerticalScrollable = this._settings.verticalScroll &&
                this._contentLocalBounds.y + this._contentLocalBounds.height > this._displayMask.height;
            // If it should be scrollable (in either direction), but has no listeners - add 'em all
            if (isHorizontalScrollable || isVerticalScrollable && !this._hasAddedListeners) {
                this.addListeners();
            }
            else if (!isHorizontalScrollable && !isVerticalScrollable && this._hasAddedListeners) {
                // Otherwise if it shouldn't be scrollable (in both directions) but has listeners - remove them!
                this.removeListeners();
            }
        }
        // TODO: Find a way to smooth this. Tween doesn't work 
        // when content is undersized due to bounds check
        moveToBottom() {
            this._contents.y = -(this._contents.height - this._displayMask.height);
        }
        scrollToY(y) {
            this._contents.y = -y;
        }
        stop() {
            this.removeListeners();
            this.reset();
        }
        update() {
            super.update();
            if (this._inputId >= 0) {
                return;
            }
            const elapsed = Date.now() - this._timestamp;
            if (this._autoScrollAmplitude.x !== 0) {
                const delta = -this._autoScrollAmplitude.x * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.x = this._autoScrollTarget.x + delta;
                }
                else {
                    this._autoScrollAmplitude.x = 0;
                }
            }
            if (this._autoScrollAmplitude.y !== 0) {
                const delta = -this._autoScrollAmplitude.y * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.y = this._autoScrollTarget.y + delta;
                }
                else {
                    this._autoScrollAmplitude.y = 0;
                }
            }
            this.limitMovement();
        }
        destroy() {
            this.removeListeners();
            this.scrollerTapSignal.dispose();
            super.destroy();
        }
        reset() {
            this._inputId = -1;
            this._timestamp = Date.now();
            this._velocity.set(0, 0);
            this._autoScrollTarget.set(0, 0);
            this._autoScrollAmplitude.set(0, 0);
        }
        addListeners() {
            if (!this._hasAddedListeners) {
                this.game.input.onDown.add(this.beginMove, this);
                this.game.input.addMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.add(this.endMove, this);
                this._displayMask.inputEnabled = true;
                this._hasAddedListeners = true;
            }
        }
        removeListeners() {
            if (this._hasAddedListeners) {
                this.game.input.onDown.remove(this.beginMove, this);
                this.game.input.deleteMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.remove(this.endMove, this);
                this._displayMask.inputEnabled = false;
                this._hasAddedListeners = false;
            }
        }
        beginMove(pointer) {
            if (this._inputId < 0 && this._displayMask.getBounds().contains(pointer.x, pointer.y)) {
                this.reset();
                this._inputId = pointer.id;
                this._previousPosition.set(pointer.x, pointer.y);
            }
        }
        endMove(pointer) {
            if (this._inputId === pointer.id) {
                if (this.game.input.activePointer.withinGame) {
                    const elapsed = Date.now() - this._timestamp;
                    if (elapsed <= 500) {
                        if (this._velocity.x > 20 || this._velocity.x < -20) {
                            this._autoScrollAmplitude.x = this._velocity.x;
                            this._autoScrollTarget.x = Math.round(this._contents.x + this._autoScrollAmplitude.x);
                        }
                        if (this._velocity.y > 20 || this._velocity.y < -20) {
                            this._autoScrollAmplitude.y = this._velocity.y;
                            this._autoScrollTarget.y = Math.round(this._contents.y + this._autoScrollAmplitude.y);
                        }
                    }
                }
                this._inputId = -1;
            }
        }
        moveCanvas(pointer, x, y) {
            if (this._inputId !== pointer.id) {
                return;
            }
            const now = Date.now();
            const elapsed = now - this._timestamp;
            this._timestamp = now;
            // If horizontal scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.horizontalScroll) {
                const delta = x - this._previousPosition.x;
                this._previousPosition.x = x;
                this._contents.position.x += delta;
                this._velocity.x = 100 * (delta / elapsed);
            }
            // If vertical scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.verticalScroll) {
                const delta = y - this._previousPosition.y;
                this._previousPosition.y = y;
                this._contents.position.y += delta;
                this._velocity.y = 100 * (delta / elapsed);
            }
            this.limitMovement();
        }
        limitMovement() {
            if (this._settings.horizontalScroll) {
                this.limitHorizontal();
            }
            if (this._settings.verticalScroll) {
                this.limitVertical();
            }
        }
        limitHorizontal() {
            if (this._contents.position.x >= 0) {
                this._contents.position.x = 0;
            }
            else {
                const contentWidth = this._contentLocalBounds.width + this._contentLocalBounds.x;
                const limit = this._displayMask.width - contentWidth - this._settings.paddingRight;
                if (this._contents.position.x <= limit) {
                    this._contents.position.x = limit;
                }
            }
        }
        limitVertical() {
            if (this._contents.position.y >= 0) {
                this._contents.position.y = 0;
            }
            else {
                const contentHeight = this._contentLocalBounds.height + this._contentLocalBounds.y;
                const limit = this._displayMask.height - contentHeight - this._settings.paddingBottom;
                if (this._contents.position.y <= limit) {
                    this._contents.position.y = limit;
                }
            }
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToContentLayer(object) {
            this._contents.add(object);
        }
    }
    exports.ScrollableGroup = ScrollableGroup;
});
define("switchback-common/component/scrolling/ScrollableMaskGroup", ["require", "exports", "switchback-common/component/scrolling/ScrollableGroup"], function (require, exports, ScrollableGroup_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A scrollable group whose contents outside the display area are masked out.
     */
    class ScrollableMaskGroup extends ScrollableGroup_1.ScrollableGroup {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, viewableWidth, viewableHeight, settings, parent);
            const mask = this.game.add.graphics(0, 0);
            mask.beginFill(0x000000, 0);
            mask.drawRect(0, 0, this._displayMask.width, this._displayMask.height);
            mask.endFill();
            this._contents.mask = mask;
        }
        /**
         * As the mask is added globally, we have to ensure we're masking from the correct location.
         * This must be called after this object moves.
         */
        updateMaskPosition() {
            this._contents.mask.x = this.worldPosition.x;
            this._contents.mask.y = this.worldPosition.y;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        beginMove(pointer) {
            super.beginMove(pointer);
            this._contents.mask.dirty = true;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        onContentsUpdated() {
            super.onContentsUpdated();
            this._contents.mask.dirty = true;
        }
    }
    exports.ScrollableMaskGroup = ScrollableMaskGroup;
});
/**
 * I can't pretend I understand this...
 * Found @ http://tiny.cc/uppary
 *
 * Modified:
 * 1. to fix retrieving the image from Phaser cache
 * 2. added frame default as 0
 *
 * Note: Don't configure a NineSlice to be smaller than its borders
 * e.g. if it has 100px borders make sure it's at least 200x200
 */
define("switchback-common/extension/NineSlice", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NineSlice {
        static create(aGame, aWidth, aHeight, aKey, aFrame = 0, aTop = 0, aLeft = 0, aBottom = 0, aRight = 0, aRepeats = false) {
            // image
            NineSlice.image = new Phaser.Image(aGame, 0, 0, aKey);
            // get frame
            if (typeof aFrame === "string") {
                NineSlice.frame = aGame.cache.getFrameByName(aKey, aFrame);
            }
            else {
                NineSlice.frame = aGame.cache.getFrameByIndex(aKey, aFrame);
            }
            // calculate dims
            NineSlice.calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats);
            // create nine image bitmap data
            NineSlice.nineImage = new Phaser.BitmapData(aGame, "NineImage" + (NineSlice.textureKey++), NineSlice.width, NineSlice.height);
            // render into bitmap data
            NineSlice.renderNineImage();
            return NineSlice.nineImage;
        }
        // -------------------------------------------------------------------------
        static calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats) {
            const frame = NineSlice.frame;
            NineSlice.centralWidth = frame.width - aLeft - aRight;
            NineSlice.centralHeight = frame.height - aTop - aBottom;
            if (aRepeats) {
                NineSlice.horizontalRepeats = aWidth;
                NineSlice.verticalRepeats = aHeight;
                NineSlice.width = aLeft + aRight + NineSlice.centralWidth * aWidth;
                NineSlice.height = aTop + aBottom + NineSlice.centralHeight * aHeight;
                NineSlice.lastWidth = 0;
                NineSlice.lastHeight = 0;
            }
            else {
                const w = aWidth - aLeft - aRight;
                NineSlice.horizontalRepeats = Math.floor(w / NineSlice.centralWidth);
                NineSlice.lastWidth = w % NineSlice.centralWidth;
                const h = aHeight - aTop - aBottom;
                NineSlice.verticalRepeats = Math.floor(h / NineSlice.centralHeight);
                NineSlice.lastHeight = h % NineSlice.centralHeight;
                NineSlice.width = aWidth;
                NineSlice.height = aHeight;
            }
            NineSlice.leftWidth = aLeft;
            NineSlice.rightWidth = aRight;
            NineSlice.topHeight = aTop;
            NineSlice.bottomHeight = aBottom;
        }
        static renderNineImage() {
            let sourceY = NineSlice.frame.y;
            let destY = 0;
            // top row
            if (NineSlice.topHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.topHeight);
                sourceY += NineSlice.topHeight;
                destY += NineSlice.topHeight;
            }
            // centrals
            for (let i = 0; i < NineSlice.verticalRepeats; i++) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.centralHeight);
                destY += NineSlice.centralHeight;
            }
            // last height
            if (NineSlice.lastHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.lastHeight);
                destY += NineSlice.lastHeight;
            }
            sourceY += NineSlice.centralHeight;
            // bottom
            if (NineSlice.bottomHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.bottomHeight);
            }
        }
        static renderNineImageRow(aImage, aSourceY, aDestY, aHeight) {
            let sourceX = NineSlice.frame.x;
            let destX = 0;
            // left
            if (NineSlice.leftWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.leftWidth, aHeight, destX, aDestY);
                destX += NineSlice.leftWidth;
                sourceX += NineSlice.leftWidth;
            }
            // centrals
            for (let i = 0; i < NineSlice.horizontalRepeats; i++) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.centralWidth, aHeight, destX, aDestY);
                destX += NineSlice.centralWidth;
            }
            // last width
            if (NineSlice.lastWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.lastWidth, aHeight, destX, aDestY);
                destX += NineSlice.lastWidth;
            }
            sourceX += NineSlice.centralWidth;
            // right
            if (NineSlice.rightWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.rightWidth, aHeight, destX, aDestY);
            }
        }
    }
    NineSlice.textureKey = 0;
    exports.NineSlice = NineSlice;
});
const pDisplayObjCont = PIXI.DisplayObjectContainer;
const pDisplayObj = PIXI.DisplayObject;
pDisplayObjCont.prototype.displayObjectUpdateTransform = pDisplayObj.prototype.updateTransform = function (parent) {
    let a;
    let b;
    let c;
    let d;
    let p;
    let pivot;
    let position;
    let pt;
    let ref;
    let rotation;
    let scale;
    let skew;
    let skewX;
    let skewY;
    let tx;
    let ty;
    let wt;
    if (!parent && !this.parent && !this.game) {
        return this;
    }
    p = this.parent;
    if (parent) {
        p = parent;
    }
    else if (!this.parent) {
        p = this.game.world;
    }
    pt = p.worldTransform;
    wt = this.worldTransform;
    ref = this, pivot = ref.pivot, position = ref.position, rotation = ref.rotation, scale = ref.scale, skew = ref.skew;
    a = b = c = d = tx = ty = null;
    if ((rotation % PIXI.PI_2) || skew) {
        skewX = (skew && skew.x) || 0;
        skewY = (skew && skew.y) || 0;
        a = scale.x * Math.cos(rotation + skewY);
        b = scale.x * Math.sin(rotation + skewY);
        c = scale.y * Math.sin(-rotation - skewX);
        d = scale.y * Math.cos(rotation + skewX);
        tx = position.x;
        ty = position.y;
        if (pivot.x || pivot.y) {
            tx -= pivot.x * a + pivot.y * c;
            ty -= pivot.x * b + pivot.y * d;
        }
        wt.a = a * pt.a + b * pt.c;
        wt.b = a * pt.b + b * pt.d;
        wt.c = c * pt.a + d * pt.c;
        wt.d = c * pt.b + d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    else {
        a = scale.x;
        d = scale.y;
        tx = position.x - (pivot.x * a);
        ty = position.y - (pivot.y * d);
        wt.a = a * pt.a;
        wt.b = a * pt.b;
        wt.c = d * pt.c;
        wt.d = d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    this.worldAlpha = this.alpha * p.worldAlpha;
    this.worldPosition.x = wt.tx;
    this.worldPosition.y = wt.ty;
    this.worldScale.x = scale.x * Math.sqrt(wt.a * wt.a + wt.c * wt.c);
    this.worldScale.y = scale.y * Math.sqrt(wt.b * wt.b + wt.d * wt.d);
    this.worldRotation = Math.atan2(-wt.c, wt.d);
    this._currentBounds = null;
    if (this.transformCallback) {
        this.transformCallback.call(this.transformCallbackContext, wt, pt);
    }
    return this;
};
define("switchback-common/play/PlayController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class PlayController {
        constructor() {
            window.webkit = {};
            window.webkit.messageHandlers = {};
            window.webkit.messageHandlers.NativeInterop = {};
            window.webkit.messageHandlers.NativeInterop.postMessage = (obj) => { return; };
        }
        start() {
            console.log("[PlayController] start()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
        eject() {
            console.log("[PlayController] eject()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        movePlayer(location) {
            console.log(`[PlayController] movePlayer(${location})`);
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: location,
            }));
        }
    }
    exports.PlayController = PlayController;
});
define("switchback-common/play/DummyApiRunner", ["require", "exports", "switchback-common/play/PlayController", "switchback-common/util/PromiseUtil"], function (require, exports, PlayController_1, PromiseUtil_80) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class DummyApiRunner {
        constructor() {
            this.controller = new PlayController_1.PlayController();
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_80.PromiseUtil.realTime(3000);
                this.controller.start();
                this.controller.movePlayer({
                    attractionName: "Little Foot's Wild Ride",
                    landName: "Land Before Time",
                    latitude: 66.33,
                    locationID: "beacon_" + Math.floor(Math.random() * 20),
                    longitude: 100.22,
                    parkName: "Animal Kingdom",
                });
                yield PromiseUtil_80.PromiseUtil.realTime(1500);
                this.controller.eject();
            });
        }
    }
    exports.DummyApiRunner = DummyApiRunner;
});
define("switchback-common/play/MiniGamePlaySession", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGamePlaySession {
        constructor(playerList) {
            this._playGroups = [];
            this._playerList = playerList;
        }
        startMiniGame(playerCount) {
            switch (playerCount) {
                case MiniGamePlaySession.PlayerCount.ALL:
                    this._playGroups = MiniGamePlaySession.generateAllPlayerGroup(this._playerList);
                    break;
                case MiniGamePlaySession.PlayerCount.SINGLE:
                    this._playGroups = MiniGamePlaySession.generateSinglePlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
                case MiniGamePlaySession.PlayerCount.TWO:
                    this._playGroups = MiniGamePlaySession.generateTwoPlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
            }
            this._currentPlayGroupIndex = 0;
        }
        playGroupFinished() {
            this._currentPlayGroupIndex++;
        }
        get roundIndex() {
            return this._currentPlayGroupIndex;
        }
        get isMiniGameSessionFinished() {
            return this._currentPlayGroupIndex >= this._playGroups.length;
        }
        get currentPlayGroup() {
            return this._playGroups[this._currentPlayGroupIndex];
        }
        get previousMiniGameLastPlayers() {
            return this._playGroups.length === 0 ? [] : this._playGroups[this._playGroups.length - 1].players;
        }
        static generateAllPlayerGroup(playerList) {
            const playGroups = [];
            playGroups.push({ players: playerList });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will only contain a single player.
         * The first group will not include any player from the list of players who should not play first
         * - Exception to this rule is if we only have one player
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateSinglePlayerGroups(playerList, shouldNotPlayFirstList) {
            const playGroups = [];
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_5.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_5.getRandomItem(playersAvailableToBeFirst);
            if (randomFirstPlayer) {
                playGroups.push({ players: [randomFirstPlayer] });
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
            }
            CollectionUtil_5.shuffle(playerPool);
            playerPool.forEach((player) => {
                playGroups.push({ players: [player] });
            });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will contain two players
         * The first group will have at least one player who is not in the list of players who should not play first
         * - Exception to this rule if we only have 2 or 3 players
         * NOTE: If we have less than two players then an empty mini game session is returned
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateTwoPlayerGroups(playerList, shouldNotPlayFirstList) {
            if (playerList.length < 2) {
                console.error(`[MiniGamePlaySession] Unable to generate play groups for two players when we don't have that many players`);
                return [];
            }
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_5.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_5.getRandomItem(playersAvailableToBeFirst);
            CollectionUtil_5.shuffle(playerPool);
            if (randomFirstPlayer) {
                // Remove the randomly selected player from the player pool,
                // Then insert them second in the array so that they get the first two plays
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
                playerPool.splice(1, 0, randomFirstPlayer);
            }
            if (playerPool.length === 2) {
                return [{ players: playerPool }];
            }
            const playSession = [];
            for (let i = 0; i < playerPool.length; i++) {
                const playerOne = playerPool[i];
                const playerTwo = (i + 1 === playerPool.length) ? playerPool[0] : playerPool[i + 1];
                const group = [playerOne, playerTwo];
                if (i % 2 === 1) {
                    group.reverse();
                }
                playSession.push({ players: group });
            }
            return playSession;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the current play group index to be set
         * @param index
         */
        TEST_setCurrentPlayGroupIndex(index) {
            this._currentPlayGroupIndex = index;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the last know players to be set
         * @param lastKnowPlayers
         */
        TEST_setLastKnownPlayers(lastKnowPlayers) {
            this._playGroups = [{ players: lastKnowPlayers }];
        }
    }
    exports.MiniGamePlaySession = MiniGamePlaySession;
    (function (MiniGamePlaySession) {
        let PlayerCount;
        (function (PlayerCount) {
            PlayerCount[PlayerCount["ALL"] = 0] = "ALL";
            PlayerCount[PlayerCount["SINGLE"] = 1] = "SINGLE";
            PlayerCount[PlayerCount["TWO"] = 2] = "TWO";
        })(PlayerCount = MiniGamePlaySession.PlayerCount || (MiniGamePlaySession.PlayerCount = {}));
    })(MiniGamePlaySession = exports.MiniGamePlaySession || (exports.MiniGamePlaySession = {}));
});
define("switchback-common/play/PlayerManager", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayerManager {
        openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                let players;
                try {
                    const result = yield PlayAPI.openPlayerManagement();
                    // if result contains no players, populate the players array with dummy players
                    if (result && result.players && result.players.length > 0) {
                        players = result.players.map((player) => {
                            return Object.assign({}, player, { instructionsViewed: new Set() });
                        });
                    }
                    else {
                        players = [
                            { name: "Player 1", type: "", identifier: "" },
                            { name: "Player 2", type: "", identifier: "" },
                        ];
                    }
                }
                catch (error) {
                    console.log("openPlayerManagement Error " + JSON.stringify(error));
                    // Initialize with fake data just so the app doesn't break
                    players = [
                        { name: "Player 1", type: "", identifier: "" },
                        { name: "Player 2", type: "", identifier: "" },
                    ];
                }
                this._players = players;
                return players;
            });
        }
        get players() {
            return this._players;
        }
        set players(p) {
            this._players = p;
        }
        /**
         * Player grouping
         * Returns a session, which is a list of groups. A group is a list of players that plays in a round.
         * @param GroupSize number = 1 : how many players per group. =1 individual game, =2 pair game, =more for bigger groups
         * If groupSize is bigger than the number of available players this will be set to this.players.length
         * @param rounds number = null : hom many rounds are needed, =null as many as possible without repeating,
         * if there aren't enough players it will start repeating
         * @param fixedPairs boolean = false : If true, for pair games, avoids making the players switch sides
         * after sequential rounds.
         * @param isShuffle boolean = false : If true, the player pool array is shuffled
         * @return list of groups. group=list of players : [ [A, B], [B, C], [C, A]]
         */
        // currently not being used in TSM 
        getPlayerSession(groupSize = 1, rounds = null, fixedPairs = false, isShuffle = false) {
            const result = [];
            if (rounds == null) {
                rounds = this.players.length;
            }
            groupSize = Math.min(groupSize, this.players.length);
            if (isShuffle) {
                CollectionUtil_6.shuffleNonRepeatingArray(this.players);
            }
            // set the pool of players
            const pool = [];
            for (const player of this.players) {
                pool.push(player);
            }
            for (let i = 0; i < rounds; i++) {
                const group = pool.slice(0, groupSize);
                // In case the game is in pairs and we want static positions, revert every other group
                if (fixedPairs && (groupSize === 2) && (i % 2 === 0)) {
                    group.reverse();
                }
                pool.push(pool.shift());
                result.push(group);
            }
            return result;
        }
        set prevPlayerNames(prevPlayers) {
            // stores the previous player names for comparison
            if (prevPlayers) {
                this.previousSessionPlayerNames = prevPlayers.slice();
            }
        }
        get prevPlayerNames() {
            return this.previousSessionPlayerNames;
        }
    }
    exports.PlayerManager = PlayerManager;
});
define("switchback-common/play/ShowEffectController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ShowEffectResult;
    (function (ShowEffectResult) {
        ShowEffectResult[ShowEffectResult["FAILURE"] = 0] = "FAILURE";
        ShowEffectResult[ShowEffectResult["SUCCESS"] = 1] = "SUCCESS";
        ShowEffectResult[ShowEffectResult["IN_USE"] = 2] = "IN_USE";
    })(ShowEffectResult = exports.ShowEffectResult || (exports.ShowEffectResult = {}));
    class ShowEffectController {
        constructor() {
            this.showControlResult = new Phaser.Signal();
        }
        init() {
            console.log('[ShowEffectController] init()');
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.FAILURE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.SUCCESS, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.DBeacon.sessionInit().then((result) => {
                console.log("PlayAPI.DBeacon.sessionInit successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionInit error:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            // N.B. We have to subscribe to this in order for the success events to work!
            // Do not remove!!
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE, (response) => { return; });
            console.log('[ShowEffectController] init() complete');
        }
        shutdown() {
            PlayAPI.DBeacon.sessionEnd().then((result) => {
                console.log("PlayAPI.DBeacon.sessionEnd successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionEnd error:\n" + JSON.stringify(error));
            });
        }
        get defaultShowEffectName() {
            return this._defaultShowEffectName;
        }
        set defaultShowEffectName(effectName) {
            this._defaultShowEffectName = effectName;
        }
        triggerDefaultEffect(timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this._defaultShowEffectName) {
                    return ShowEffectResult.FAILURE;
                }
                return yield this.triggerEffect(this._defaultShowEffectName, timeoutS);
            });
        }
        /**
         * Triggers an effect via the Play API.
         * with forced local timeout
         */
        triggerEffect(effectName, timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ShowEffectController] triggerEffect() " + effectName);
                return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    if (this.isInitializeFailed) {
                        resolve(ShowEffectResult.FAILURE);
                        return;
                    }
                    let secondaryTimeoutId;
                    this.showControlResult.add((result, response) => {
                        console.log("[ShowEffectController] triggerEffect() this.showControlResult.add(): " + result + " " + response.experienceName);
                        // ensure we heard the show control message for the effect we are waiting for
                        if (response.experienceName === effectName) {
                            if (secondaryTimeoutId !== undefined) {
                                clearTimeout(secondaryTimeoutId);
                            }
                            this.showControlResult.removeAll();
                            resolve(result);
                        }
                    });
                    PlayAPI.DBeacon.triggerEffect(effectName, {}, timeoutS).then((result) => {
                        console.log("PlayAPI.DBeacon.triggerEffect successfully called:\n" + JSON.stringify(result));
                        // additional safety check incase any of the show controller events fail to respond
                        // this is double the time that the PlayAPI says that requires to complete its calls
                        // this is only required as in somecases the PlayAPI has not been returning anything
                        secondaryTimeoutId = setTimeout(() => {
                            console.warn("PlayAPI.ShowControl.triggerEffect forced local timeout");
                            this.showControlResult.removeAll();
                            resolve(ShowEffectResult.FAILURE);
                        }, timeoutS * 2000);
                    }, (error) => {
                        console.error("PlayAPI.ShowControl.triggerEffect error:\n" + error);
                        if (secondaryTimeoutId !== undefined) {
                            clearTimeout(secondaryTimeoutId);
                        }
                        this.showControlResult.removeAll();
                        resolve(ShowEffectResult.FAILURE);
                    });
                }));
            });
        }
    }
    exports.ShowEffectController = ShowEffectController;
});
/**
 * Wrapper for Play Disney Parks AR Kit
 * @author joel@cerebralfix.com
 */
define("switchback-common/play/augmented-reality/ThreeARScene", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThreeARScene {
        constructor(canvas) {
            this._sessionStarted = false;
            this.markerDetectedSignal = new Phaser.Signal();
            console.log("[ThreeARScene] new()");
            this._canvasTarget = canvas;
        }
        get renderer() {
            return this._renderer;
        }
        get scene() {
            return this._scene;
        }
        get canvas() {
            return this._canvasTarget;
        }
        get camera() {
            return this._camera;
        }
        registerImage(name, pathInBundle, realWorldWidth) {
            if (!this._sessionStarted) {
                console.error("[ThreeARScene] registerImage() cannot register images before AR session is started");
                return;
            }
            PlayAPI.AugmentedReality.registerImage(name, pathInBundle, realWorldWidth).then(function () {
                console.log(`[ThreeARScene] registerImage(): successfully registered ${name}`);
            }).catch((e) => {
                console.warn(`[ThreeARScene] registerImage(): failed to register ${name}`);
                console.error(JSON.stringify(e));
            });
        }
        sessionStart() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ThreeARScene] startSession()");
                PlayAPI.AugmentedReality.sessionStart(this._canvasTarget).then(() => {
                    console.log(`[ThreeARScene] startSession(): success`);
                    this._sessionStarted = true;
                }).catch((e) => {
                    console.warn(`[ThreeARScene] startSession(): error`);
                    console.error(JSON.stringify(e));
                    this._sessionStarted = false;
                });
                // TODO: Figure out why sessionStart isn't returning, then remove this hack
                this._sessionStarted = true;
                if (!this._sessionStarted) {
                    return false;
                }
                else {
                    document.body.classList.add("transparent");
                }
                // Subscribe to PlayAPI image recognize events
                PlayAPI.subscribeToEvent(PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE, (event) => {
                    console.log("[ThreeARScene] PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE");
                    // Let any listeners know the details of the marker we found.
                    this.markerDetectedSignal.dispatch(event);
                });
                this._renderer = PlayAPI.AugmentedReality.renderer;
                this._scene = PlayAPI.AugmentedReality.scene;
                this._camera = PlayAPI.AugmentedReality.camera;
                return this._sessionStarted;
            });
        }
        endSession() {
            console.log("[ThreeARScene] endSession()");
            let response;
            PlayAPI.AugmentedReality.sessionEnd().then(() => {
                console.log(`[ThreeARScene] endSession(): success`);
                response = true;
            }).catch((e) => {
                console.warn(`[ThreeARScene] endSession(): error`);
                console.error(JSON.stringify(e));
                response = false;
            });
            this._sessionStarted = this._sessionStarted && response;
            document.body.classList.remove("transparent");
            return response;
        }
        destroy() {
            this._renderer.clear();
            this.markerDetectedSignal.dispose();
        }
    }
    exports.ThreeARScene = ThreeARScene;
});
define("switchback-common/play/debug/PlayDebugPanel", ["require", "exports", "switchback-common/play/api/MockPlayAPI"], function (require, exports, MockPlayAPI_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebugPanel can trigger events that the PlayAPI will respond to.
     * @author joel@cerebralfix.com
     */
    class PlayDebugPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            const BUTTON_PADDING = 5;
            const BUTTON_WIDTH = 200;
            const BUTTON_HEIGHT = 100;
            // EJECT beacon button
            const beaconEject = new Phaser.Button(this.game, BUTTON_PADDING, BUTTON_PADDING, "button-basic", this.onBeaconEjectButtonClicked, this, "over", "out", "down", "out");
            beaconEject.width = BUTTON_WIDTH;
            beaconEject.height = BUTTON_HEIGHT;
            beaconEject.alpha = 0.5;
            // EJECT beacon button text
            const beaconEjectText = new Phaser.Text(game, beaconEject.centerX, beaconEject.centerY, "EJECT\nBEACON");
            beaconEjectText.anchor.set(0.5);
            // START beacon button
            const beaconStart = new Phaser.Button(this.game, beaconEject.x + beaconEject.width + BUTTON_PADDING, beaconEject.y, "button-basic", this.onBeaconStartButtonClicked, this, "over", "out", "down", "out");
            beaconStart.width = BUTTON_WIDTH;
            beaconStart.height = BUTTON_HEIGHT;
            beaconStart.alpha = 0.5;
            // START beacon button text
            const beaconStartText = new Phaser.Text(game, beaconStart.centerX, beaconStart.centerY, "START\nBEACON");
            beaconStartText.anchor.set(0.5);
            // LOCATION change button
            const locationChange = new Phaser.Button(this.game, beaconStart.x + beaconStart.width + BUTTON_PADDING, beaconStart.y, "button-basic", this.onLocationChangeButtonClicked, this, "over", "out", "down", "out");
            locationChange.width = BUTTON_WIDTH;
            locationChange.height = BUTTON_HEIGHT;
            locationChange.alpha = 0.5;
            // LOCATION change button text
            const locationChangeText = new Phaser.Text(game, locationChange.centerX, locationChange.centerY, "RANDOM\nLOCATION");
            locationChangeText.anchor.set(0.5);
            // Background panel graphic
            const background = new Phaser.Image(game, 0, 0, "button-basic");
            background.alpha = 0.2;
            this.add(background);
            this.add(beaconEject);
            this.add(beaconStart);
            this.add(locationChange);
            this.add(beaconEjectText);
            this.add(beaconStartText);
            this.add(locationChangeText);
            // Size the background to sit behind everything that has been added
            background.width = this.width + BUTTON_PADDING;
            background.height = this.height + BUTTON_PADDING;
        }
        onLocationChangeButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: MockPlayAPI_2.MockPlayAPI.getFakeLocation(),
            }));
        }
        onBeaconEjectButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        onBeaconStartButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
    }
    exports.PlayDebugPanel = PlayDebugPanel;
});
define("switchback-common/play/debug/PlayDebug", ["require", "exports", "switchback-common/play/debug/PlayDebugPanel"], function (require, exports, PlayDebugPanel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebug creates a PlayDebugPanel that can be used to trigger events via the PlayAPI.
     * @author joel@cerebralfix.com
     */
    class PlayDebug {
        constructor(game) {
            // Toggle button
            const toggleButton = new Phaser.Button(game, 0, 0, "button-basic", this.onDebugButtonClicked, this, "over", "out", "down", "out");
            toggleButton.bottom = game.height;
            toggleButton.alpha = 0.3;
            // Toggle button text
            const toggleButtonText = new Phaser.Text(game, toggleButton.centerX, toggleButton.centerY, "DEBUG");
            toggleButtonText.anchor.set(0.5);
            // Debug panel
            this.debugPanel = new PlayDebugPanel_1.PlayDebugPanel(game);
            this.debugPanel.centerX = game.world.centerX;
            this.debugPanel.bottom = game.height;
            this.debugPanel.visible = false;
            game.add.existing(toggleButton);
            game.add.existing(toggleButtonText);
        }
        onDebugButtonClicked() {
            this.debugPanel.visible = !this.debugPanel.visible;
        }
    }
    exports.PlayDebug = PlayDebug;
});
define("switchback-common/test/keyboardController.spec", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/KeyboardOptions", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, KeyboardController_2, KeyboardOptions_1, SpecialChars_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("keyboardController", () => {
        let keyboardController;
        beforeEach(() => {
            keyboardController = new KeyboardController_2.KeyboardController(new KeyboardOptions_1.KeyboardOptions());
        });
        it("can add letters to a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            expect(keyboardController.currentString).toBe("TEST");
        });
        it("can submit a string", (done) => {
            keyboardController.submitSignal.addOnce((response) => {
                expect(response).toBe("TEST");
                done();
            });
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.SUBMIT);
        });
        it("can delete letters from a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.DELETE);
            expect(keyboardController.currentString).toBe("TES");
        });
    });
});
define("switchback-common/test/playerManager.spec", ["require", "exports", "switchback-common/play/PlayerManager"], function (require, exports, PlayerManager_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("PlayerManager", () => {
        let playerManager;
        beforeEach(() => {
            playerManager = new PlayerManager_1.PlayerManager();
        });
        describe("Getters and Setters", () => {
            describe("handles null parameters", () => {
                [null, undefined].forEach((value) => {
                    it(`prevPlayerNames handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
                [null, undefined].forEach((value) => {
                    it(`players handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
            });
            it("overwrites existing previous player names", () => {
                playerManager.prevPlayerNames = ["n1", "n2"];
                playerManager.prevPlayerNames = ["n3", "n4"];
                expect(playerManager.prevPlayerNames).toEqual(["n3", "n4"]);
            });
        });
        describe("Player Management", () => {
            it("has players", () => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
                expect(playerManager.players).toBeTruthy();
            });
        });
        describe("Player Sessions", () => {
            beforeEach(() => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
            });
            [2, 3, 4, 5].forEach((numberOfRounds) => {
                it("generates the correct number of rounds", () => {
                    const session = playerManager.getPlayerSession(2, numberOfRounds);
                    expect(session.length).toBe(numberOfRounds);
                });
            });
            [1, 2].forEach((numberOfRounds) => {
                it("generates rounds of the correct size", () => {
                    const session = playerManager.getPlayerSession(numberOfRounds);
                    session.forEach((round) => {
                        expect(round.length).toBe(numberOfRounds);
                    });
                });
            });
            it("has different players in each round", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                session.forEach((round) => {
                    expect(round[0]).not.toEqual(round[1]);
                });
            });
            it("prevents players from swapping sides", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                expect(session[0][1]).toEqual(session[1][1]);
            });
        });
    });
});
define("switchback-common/test/signal.spec", ["require", "exports", "switchback-common/component/Signal"], function (require, exports, Signal_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("Signal", () => {
        it("a listener is called each time a signal is dispatched", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("addOnce causes a listener to be called only once", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("dispatch calls every attached listener", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("disposing a signal stops a listener being called", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispose();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("listeners are called in the order they are added", () => {
            const signal = new Signal_3.Signal();
            const results = [];
            signal.addOnce(() => {
                results.push(1);
            });
            signal.addOnce(() => {
                results.push(2);
            });
            signal.dispatch();
            expect(results[0]).toBe(1);
            expect(results[1]).toBe(2);
        });
    });
});
define("switchback-common/test/unit/CollectionUtil.spec", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`CollectionUtil`, () => {
        describe(`Exclusive Left Join`, () => {
            it(`Excluded array contains 1 item found in the array`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual(["a", "b", "c"]);
            });
            it(`Excluded array contains 2 items found in the array`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["a"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["b"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Excluded array is empty`, () => {
                const result = CollectionUtil_7.exclusiveLeftJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual(["a", "b", "c"]);
            });
        });
        describe(`Inner Join`, () => {
            it(`Second array contains 1 item found in the array`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual([]);
            });
            it(`Second array contains 2 items found in the array`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["a"]);
            });
            it(`Second array is empty`, () => {
                const result = CollectionUtil_7.innerJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual([]);
            });
        });
    });
});
define("switchback-common/test/unit/MiniGamePlaySession.spec", ["require", "exports", "switchback-common/play/MiniGamePlaySession"], function (require, exports, MiniGamePlaySession_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`MiniGamePlaySession`, () => {
        const player0 = { identifier: "0", name: "player 0", type: "" };
        const player1 = { identifier: "1", name: "player 1", type: "" };
        const player2 = { identifier: "2", name: "player 2", type: "" };
        const player3 = { identifier: "3", name: "player 3", type: "" };
        const player4 = { identifier: "4", name: "player 4", type: "" };
        describe(`Player group generation`, () => {
            describe(`Single Player Mini Games`, () => {
                it(`The correct number of play session for a single player are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`First player is not one of the last known players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const lastKnownPlayers = [player0, player1, player2, player3];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, lastKnownPlayers);
                    expect(result[0].players).toEqual([player4]);
                });
            });
            describe(`Two Player Mini Games`, () => {
                it(`The correct number of play sessions for two people are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`Player sequence is correct for 5 players. One player plays twice in a row and in the same position`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result[0].players[1]).toEqual(result[1].players[1]);
                    expect(result[1].players[0]).toEqual(result[2].players[0]);
                    expect(result[2].players[1]).toEqual(result[3].players[1]);
                    expect(result[3].players[0]).toEqual(result[4].players[0]);
                    expect(result[0].players[0]).toEqual(result[4].players[1]);
                });
                it(`Unable to generate player groups for two player game with only one player`, () => {
                    const players = [player0];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(0);
                });
                it(`Only generates one play group when there is only two players`, () => {
                    const players = [player0, player1];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(1);
                });
            });
        });
        describe(`Player Group Cycling`, () => {
            let playSession;
            beforeEach(() => {
                playSession = new MiniGamePlaySession_1.MiniGamePlaySession([player0, player1, player2, player3]);
            });
            it(`Starting a single player mini game returns a single player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players.length).toBe(1);
            });
            it(`Starting a two player mini game returns a two player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players.length).toBe(2);
            });
            it(`The last group to finish should trigger the mini game session to be finished`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                playSession.TEST_setCurrentPlayGroupIndex(3);
                playSession.playGroupFinished();
                expect(playSession.isMiniGameSessionFinished).toBe(true);
            });
            it(`Starting a single player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players).toEqual([player1]);
            });
            it(`Starting a two player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players[1]).toEqual(player1);
            });
        });
    });
});
define("switchback-common/util/Animation", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function popSpin(x, y, width, imageKey, trailKey, game, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = parameters || {};
            const parent = params.parent || game.add.group();
            const tweenMS = params.tweenMS || 600;
            const spinMS = params.spinMS || 750;
            const bezierCurve = [
                [[0, 0], [-0.287, -0.413], [-0.705, -0.508], [-1, -0.539]],
                [[0, 0], [-0.609, 0.025], [-0.882, 0.280], [-1, 0.593]],
                [[0, 0], [0.287, -0.413], [0.705, -0.508], [1, -0.539]],
            ];
            const startPositions = [
                [x - width * 0.2, y - width * 0.1],
                [x - width * 0.25, y + width * 0.05],
                [x + width * 0.2, y],
            ];
            for (let i = 0; i < bezierCurve.length; i++) {
                const pos = startPositions[i];
                const bezier = bezierCurve[i];
                const star = new Phaser.Image(game, pos[0], pos[1], imageKey);
                const curveScale = width * 0.4;
                star.anchor.set(0.5);
                star.pivot.set(0.5, 0.5);
                star.position.set(pos[0], pos[1]);
                const starRatio = star.width / star.height;
                star.width = params.iconWidth || star.width;
                star.height = star.width * starRatio;
                star.visible = true;
                game.add.tween(star).to({ angle: 360 }, spinMS, Phaser.Easing.Linear.None, true, 0 - 1).start();
                const tween = game.add.tween(star).to({
                    x: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    y: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                const emitter = game.add.emitter(star.x, star.y, 50);
                emitter.maxParticleScale = 0.5;
                parent.addChild(emitter);
                parent.addChild(star);
                emitter.makeParticles(trailKey);
                emitter.setRotation(0, 0);
                emitter.setXSpeed(0, 0);
                emitter.setYSpeed(0, 0);
                emitter.alpha = 0.8;
                emitter.start(false, 5000, 10);
                emitter.emitX = star.position.x;
                emitter.emitY = star.position.y;
                emitter.gravity = 0;
                const emitTween = game.add.tween(emitter).to({
                    emitX: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    emitY: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                tween.interpolation(Phaser.Math.bezierInterpolation);
                emitTween.interpolation(Phaser.Math.bezierInterpolation);
                tween.onComplete.add(() => {
                    const fade = game.add.tween(star).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        star.destroy();
                    });
                    fade.start();
                }, this);
                emitTween.onComplete.addOnce(() => {
                    const fade = game.add.tween(emitter).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        // if (emitter) {
                        // 	emitter.destroy();
                        // }
                    });
                    emitter.start();
                }, this);
                tween.start();
                emitTween.start();
            }
        });
    }
    exports.popSpin = popSpin;
    function wiggle(game, tweenObj, angle = 0, offsetAngle = 15, duration = 500, autoStart = true) {
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: angle + offsetAngle }, duration, Phaser.Easing.Cubic.apply, autoStart, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: angle - offsetAngle }, duration, Phaser.Easing.Cubic.apply, false, 0, 0, true);
        tween.chain(tween2);
        return tween2;
    }
    exports.wiggle = wiggle;
    function attentionWiggle(game, tweenObj, angle = 0, duration = 500) {
        const startAngle = tweenObj.angle;
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: startAngle - angle }, duration * 0.25, Phaser.Easing.Cubic.apply, true, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: startAngle + angle }, duration * 0.5, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        const tween3 = game.add.tween(tweenObj);
        tween3.to({ angle: startAngle }, duration * 0.25, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        tween.chain(tween2);
        tween2.chain(tween3);
        return tween;
    }
    exports.attentionWiggle = attentionWiggle;
    function tintTween(game, tweenObj, tint, duration = 500, ease = Phaser.Easing.Linear.None, autoStart = false, delay = 0, repeat = 0, yoyo = false) {
        const colorBlend = { step: 0 };
        const startColor = tweenObj.tint;
        const colorTween = game.add.tween(colorBlend).to({ step: 100 }, duration, ease, autoStart, delay, repeat, yoyo);
        colorTween.onUpdateCallback(function () {
            tweenObj.tint = Phaser.Color.interpolateColor(startColor, tint, 100, colorBlend.step);
        });
        return colorTween.start();
    }
    exports.tintTween = tintTween;
    function fadeIn(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 0;
        object.visible = true;
        return game.add.tween(object).to({
            alpha: 1,
        }, duration, easing, false);
    }
    exports.fadeIn = fadeIn;
    function fadeOut(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 1;
        return game.add.tween(object).to({
            alpha: 0,
        }, duration, easing, false);
    }
    exports.fadeOut = fadeOut;
    function bounce(game, object, repeat = 0, bounceHeight) {
        bounceHeight = bounceHeight || this.game.height * 0.1;
        const destination = object.position.y - bounceHeight;
        const origin = object.position.y;
        const tween = game.add.tween(object).to({
            y: destination,
        }, 200, Phaser.Easing.Linear.None, false, 0).to({
            y: origin,
        }, 200, Phaser.Easing.Linear.None, false, 0);
        tween.repeatAll(repeat);
        return tween;
    }
    exports.bounce = bounce;
    function slideTo(game, object, targetPosition, duration = 200, easing = Phaser.Easing.Linear.None) {
        const tween = game.add.tween(object).to({
            x: targetPosition.x,
            y: targetPosition.y,
        }, duration, easing, false, 0, 0);
        return tween;
    }
    exports.slideTo = slideTo;
    function pulse(game, object, targetScale, duration = 200, easing = Phaser.Easing.Linear.None) {
        const originalWidth = object.width;
        const originalHeight = object.height;
        const tweenIn = game.add.tween(object).to({
            width: originalWidth * targetScale,
            height: originalHeight * targetScale,
        }, duration, easing, false, 0, 0);
        const tweenOut = game.add.tween(object).to({
            width: originalWidth,
            height: originalHeight,
        }, duration, easing, false, 0, 0);
        tweenOut.onComplete.add(() => {
            tweenIn.start();
        });
        tweenIn.chain(tweenOut);
        return tweenIn;
    }
    exports.pulse = pulse;
    function shake(game, object, amount, severity) {
        const times = amount || game.rnd.integerInRange(5, 7);
        const maxOffset = game.width * (severity || 0.02);
        const orgPos = object.position.clone();
        let firstTween;
        let prevTween;
        for (let i = 0; i < times; ++i) {
            const tween = game.add.tween(object).to({
                x: orgPos.x + game.rnd.integerInRange(-maxOffset, maxOffset),
                y: orgPos.y + game.rnd.integerInRange(-maxOffset, maxOffset),
            }, 20, Phaser.Easing.Linear.None, false);
            if (prevTween) {
                prevTween.chain(tween);
            }
            else {
                firstTween = tween;
            }
            prevTween = tween;
        }
        const tween = game.add.tween(object).to({
            x: orgPos.x,
            y: orgPos.y,
        }, 20, Phaser.Easing.Linear.None, false);
        prevTween.chain(tween);
        return firstTween;
    }
    exports.shake = shake;
    function blink(game, object, duration, times, autostart = true) {
        const tween = game.add.tween(object);
        tween.to({}, duration, Phaser.Easing.Linear.None, autostart, 0, times);
        tween.onRepeat.add(() => {
            object.visible = !object.visible;
        });
        return tween;
    }
    exports.blink = blink;
    function pulseFade(game, object, finalScale, duration, times = 0, autoStart = true) {
        const tweens = [
            // scale
            game.add.tween(object.scale).to({ x: finalScale, y: finalScale }, duration, Phaser.Easing.Linear.None, autoStart, 0, times),
            // tween alphas
            game.add.tween(object).to({ alpha: 0 }, duration, Phaser.Easing.Quartic.In, autoStart, 0, times)
        ];
        return tweens;
    }
    exports.pulseFade = pulseFade;
});
define("switchback-common/util/LiquidScaler", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class LiquidScaler {
        static init(game, minWidth, minHeight, maxScale) {
            if (this.instance) {
                console.error("LiquidScaler already initialized.");
                return;
            }
            this.instance = new LiquidScaler(game, minWidth, minHeight, maxScale);
        }
        static getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            return this.Instance.getScaledSize(widthPercent, heightPercent, maintainAspectRatio, useMaxScale);
        }
        static get Instance() {
            if (!this.instance) {
                console.error("LiquidScaler not initialized.");
            }
            return this.instance;
        }
        constructor(game, minWidth, minHeight, maxScale) {
            this.game = game;
            this.minWidth = minWidth;
            this.minHeight = minHeight;
            this.maxScale = maxScale;
        }
        getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            let width = widthPercent * this.minWidth;
            let height = heightPercent * this.minHeight;
            let verticalScale = this.game.height / this.minHeight;
            let horizontalScale = this.game.width / this.minWidth;
            if (maintainAspectRatio) {
                const isLong = horizontalScale > verticalScale;
                if (isLong) {
                    if (useMaxScale) {
                        verticalScale = Math.min(verticalScale, this.maxScale);
                    }
                    width *= verticalScale;
                    height *= verticalScale;
                }
                else {
                    if (useMaxScale) {
                        horizontalScale = Math.min(horizontalScale, this.maxScale);
                    }
                    width *= horizontalScale;
                    height *= horizontalScale;
                }
            }
            else {
                width = widthPercent * this.game.width;
                height = widthPercent * this.game.height;
            }
            return new Phaser.Rectangle(0, 0, width, height);
        }
    }
    exports.LiquidScaler = LiquidScaler;
});
define("switchback-common/util/MemoryUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MemoryUtil {
        /*
            Checks that all the child objects contained as attributes have been destroyed properly.
            Does not descend into the tree (childs of childs).
            Works for Phaser objects and Signals.
            To be used for debugging, and profiling only.
            How to use:
                When trying to profile certain class, call this function between destroy and nulling. ie:
    
                ```
                super.destroy();
                MemoryUtil.checkAllDestroyed(this);
    
                this.prop1 = null;
                this.prop2 = null;
                ```
    
                If certain attribute has not been destroyed properly it will be reported on the console.
        */
        static checkAllDestroyed(obj) {
            if (obj.stateName) {
                console.warn("Checking state " + obj.stateName);
            }
            const toIgnore = ["game", "make", "add", "camera", "cache", "input", "sound",
                "scale", "stage", "state", "time", "tweens", "world", "particles", "physics", "load"];
            let ret = true;
            for (const name in obj) {
                if (!obj.hasOwnProperty(name)) {
                    continue;
                }
                if (toIgnore.indexOf(name) > -1) {
                    continue;
                }
                const child = obj[name];
                if (!child) {
                    continue;
                }
                // if the object contains one of this properties and is not null, then it hasn't been destroyed properly
                if (child.parent || child.game || child.exists) {
                    ret = false;
                    console.warn("Object not properly destroyed. '" + name + "'", child);
                }
                // signals
                if (child._bindings || child._prevParams) {
                    ret = false;
                    console.warn("Signal not properly disposed. '" + name + "'", child);
                }
                // sounds
                if (child._buffer || child.externalNode) {
                    ret = false;
                    console.warn("Sound not properly disposed. '" + name + "'", child);
                }
            }
            return ret;
        }
    }
    exports.MemoryUtil = MemoryUtil;
});
define("switchback-common/util/PhysicsUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PhysicsUtil {
        /**
         * Custom physics data doesn't cope with being scaled.
         *
         * This function found at https://stackoverflow.com/a/43302120/1343176 fixes this
         * by scaling all vertices on the shape and rewriting them back to the cache at runtime.
         * @param originalPhysicsKey the key in cache of your physics data
         * @param newPhysicsKey the key you want the new physics data to have
         * @param shapeKey the specific shape in the original data to scale
         * @param scale the scale value
         */
        static resizePolygonData(game, originalPhysicsKey, newPhysicsKey, shapeKey, scale) {
            const newData = [];
            const data = game.cache.getPhysicsData(originalPhysicsKey, shapeKey);
            for (let i = 0; i < data.length; i++) {
                const vertices = [];
                for (let j = 0; j < data[i].shape.length; j += 2) {
                    vertices[j] = data[i].shape[j] * scale;
                    vertices[j + 1] = data[i].shape[j + 1] * scale;
                }
                newData.push({ shape: vertices });
            }
            const item = {};
            item[shapeKey] = newData;
            game.load.physics(newPhysicsKey, "", item);
        }
    }
    exports.PhysicsUtil = PhysicsUtil;
});
define("switchback-common/util/SpriteUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SpriteUtil {
        static greyscale(game, frame) {
            const bmd = game.make.bitmapData();
            bmd.load(frame);
            bmd.processPixelRGB(SpriteUtil.forEachPixel, this);
            return bmd;
        }
        static forEachPixel(pixel) {
            const gray = (pixel.r + pixel.g + pixel.b) / 3;
            pixel.r = gray;
            pixel.g = gray;
            pixel.b = gray;
            return pixel;
        }
    }
    exports.SpriteUtil = SpriteUtil;
});
define("switchback-common/util/WebUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Helper functions for web-specific tasks
     * @author joel@cerebralfix.com
     * @author nick@cerebralfix.com
     */
    class WebUtil {
        /**
         * Grabs the URL parameters and returns them as a dictionary.
         */
        static getQueryParams() {
            if (WebUtil.queryDict) {
                return WebUtil.queryDict;
            }
            const params = {};
            location.search.substr(1).split("&").forEach((item) => {
                const s = item.split("=");
                const key = s[0];
                const value = s[1] && decodeURIComponent(s[1]);
                if (!(key in params)) {
                    params[key] = new Array();
                }
                params[key].push(value);
            });
            WebUtil.queryDict = params;
            return WebUtil.queryDict;
        }
        // Fast syntax lookup for a specific query parametre
        static getQueryFor(key, index = 0) {
            const dict = WebUtil.getQueryParams();
            if (dict[key]) {
                return dict[key][index];
            }
            return null;
        }
    }
    exports.WebUtil = WebUtil;
});
